Package: gtregression
Type: Package
Title: Tools for Creating Publication-Ready Regression Tables
Version: 1.0.0
Authors@R: c(
    person(given = "Rubeshkumar", family = "Polani",
    email = "rubesh@thinkdenominator.com", 
    role = c("aut", "cre"), comment = c(ORCID = "0000-0002-0418-7592")),
    person(given = "Salin K", family = "Eliyas", 
    email = "salins13@gmail.com", role = "aut", 
    comment = c(ORCID = "0000-0002-8020-5860")),
    person(given = "Manikandanesan", family = "Sakthivel", 
    email = "nesanmbbs@gmail.com", role = "aut", 
    comment = c(ORCID = "0000-0002-5438-3970")),
    person(given = "Yuvaraj", family = "Krishnamoorthy",
    email = "yuvaraj@propulevidence.org", role = "aut",
    comment = c(ORCID = "0000-0003-4688-510X")),
    person(given = "Marie Gilbert", family = "Majella", 
    email = "gilbert2691@gmail.com", role = "aut", 
    comment = c(ORCID = "0000-0003-4036-5162"))
  )
Description: 
  Simplifies regression modeling in R by integrating multiple modeling and summarization
  tools into a cohesive, user-friendly interface. Designed to be accessible for researchers,
  particularly those in Low- and Middle-Income Countries (LMIC). Built upon widely accepted 
  statistical methods, including logistic regression (Hosmer et al. 2013, ISBN:9781118548429), 
  log-binomial regression (Spiegelman and Hertzmark 2005 <doi:10.1093/aje/kwi188>), 
  Poisson and robust Poisson regression (Zou 2004 <doi:10.1093/aje/kwh090>), 
  negative binomial regression (Hilbe 2011, ISBN:9780521179515), and linear regression 
  (Kutner et al. 2005, ISBN:9780071122214). Leverages multiple dependencies to ensure 
  high-quality output and generate reproducible, publication-ready tables in alignment with
  best practices in epidemiology and applied statistics.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1.0)
Imports: dplyr, gtsummary, risks, purrr, MASS, rlang, stats, lmtest,
        patchwork, ggtext, ggplot2, tidyr, utils, sandwich, tibble,
        broom, broom.helpers, gt, officer, flextable
VignetteBuilder: knitr
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, mlbench, car, forcats,
        pkgdown
RoxygenNote: 7.3.2
Config/testthat/edition: 3
URL: https://thinkdenominator.github.io/gtregression/
BugReports: https://github.com/ThinkDenominator/gtregression/issues
NeedsCompilation: no
Packaged: 2025-07-30 18:38:55 UTC; salineliyas
Author: Rubeshkumar Polani [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0418-7592>),
  Salin K Eliyas [aut] (ORCID: <https://orcid.org/0000-0002-8020-5860>),
  Manikandanesan Sakthivel [aut] (ORCID:
    <https://orcid.org/0000-0002-5438-3970>),
  Yuvaraj Krishnamoorthy [aut] (ORCID:
    <https://orcid.org/0000-0003-4688-510X>),
  Marie Gilbert Majella [aut] (ORCID:
    <https://orcid.org/0000-0003-4036-5162>)
Maintainer: Rubeshkumar Polani <rubesh@thinkdenominator.com>
Repository: CRAN
Date/Publication: 2025-08-18 14:50:13 UTC
Built: R 4.6.0; ; 2026-04-25 13:58:08 UTC; unix

Package: gtregression
Type: Package
Title: Tools for Creating Publication-Ready Regression Tables
Version: 1.1.0
Authors@R: c(
    person(given = "Rubeshkumar", family = "Polani",
    email = "rubesh@thinkdenominator.com", 
    role = c("aut", "cre"), comment = c(ORCID = "0000-0002-0418-7592")),
    person(given = "Salin K", family = "Eliyas", 
    email = "salins13@gmail.com", role = "aut", 
    comment = c(ORCID = "0000-0002-8020-5860")),
    person(given = "Manikandanesan", family = "Sakthivel", 
    email = "nesanmbbs@gmail.com", role = "aut", 
    comment = c(ORCID = "0000-0002-5438-3970")),
    person(given = "Mogan", family = "Kaviprawin", 
    email = "moganka1994@gmail.com", role = "aut", 
    comment = c(ORCID = "0000-0002-1659-059X")),
    person(given = "Yuvaraj", family = "Krishnamoorthy",
    email = "yuvaraj@propulevidence.org", role = "aut",
    comment = c(ORCID = "0000-0003-4688-510X")),
    person(given = "Marie Gilbert", family = "Majella", 
    email = "gilbert2691@gmail.com", role = "aut", 
    comment = c(ORCID = "0000-0003-4036-5162"))
  )
Description: 
  Simplifies regression modeling in R by integrating multiple modeling and summarization
  tools into a cohesive, user-friendly interface. Designed to be accessible for researchers,
  particularly those in Low- and Middle-Income Countries (LMIC). Built upon widely accepted 
  statistical methods, including logistic regression (Hosmer et al. 2013, ISBN:9781118548429), 
  log-binomial regression (Spiegelman and Hertzmark 2005 <doi:10.1093/aje/kwi188>), 
  Firth penalized logistic regression (Firth 1993 <doi:10.1093/biomet/80.1.27>),
  Poisson and robust Poisson regression (Zou 2004 <doi:10.1093/aje/kwh090>), 
  negative binomial regression (Hilbe 2011, ISBN:9780521179515), Cox proportional
  hazards regression, parametric survival regression, causal mediation analysis,
  and linear regression
  (Kutner et al. 2005, ISBN:9780071122214). Leverages multiple dependencies to ensure 
  high-quality output and generate reproducible, publication-ready tables in alignment with
  best practices in epidemiology and applied statistics.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
LazyData: true
Depends: R (>= 4.1.0)
Imports: dplyr, risks, purrr, MASS, rlang, stats, lmtest, patchwork,
        ggplot2, utils, sandwich, tibble, broom, broom.helpers, gt,
        officer, flextable, scales (>= 1.3.0), forestploter, logistf,
        survival
VignetteBuilder: knitr
Suggests: haven, mlbench (>= 2.1-11), testthat (>= 3.1.5), knitr,
        rmarkdown, car, forcats, pkgdown, ggtext, shiny, rstudioapi,
        DT, readxl, rio
Config/testthat/edition: 3
Config/Needs/website: pkgdown
URL: https://gtregression.thinkdenominator.com/,
        https://github.com/ThinkDenominator/gtregression
BugReports: https://github.com/ThinkDenominator/gtregression/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-21 05:27:00 UTC; drrubesh
Author: Rubeshkumar Polani [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0418-7592>),
  Salin K Eliyas [aut] (ORCID: <https://orcid.org/0000-0002-8020-5860>),
  Manikandanesan Sakthivel [aut] (ORCID:
    <https://orcid.org/0000-0002-5438-3970>),
  Mogan Kaviprawin [aut] (ORCID: <https://orcid.org/0000-0002-1659-059X>),
  Yuvaraj Krishnamoorthy [aut] (ORCID:
    <https://orcid.org/0000-0003-4688-510X>),
  Marie Gilbert Majella [aut] (ORCID:
    <https://orcid.org/0000-0003-4036-5162>)
Maintainer: Rubeshkumar Polani <rubesh@thinkdenominator.com>
Repository: CRAN
Date/Publication: 2026-08-21 07:30:02 UTC
Built: R 4.5.2; ; 2026-08-21 09:17:14 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("$",gtregression)
S3method(print,gtregression)
S3method(print,gtregression_forest)
S3method(print,gtregression_reg_check)
S3method(print,identify_confounder_result)
S3method(print,merged_table)
export(check_collinearity)
export(check_convergence)
export(check_ph)
export(compare_models)
export(cox_reg)
export(descriptive_table)
export(dissect)
export(forest_df)
export(forest_reg)
export(gtregression_app)
export(identify_confounder)
export(interaction_models)
export(km_plot)
export(km_risk_table)
export(logrank_test)
export(mediation_analysis)
export(merge_tables)
export(modify_table)
export(multi_reg)
export(plot_mediation)
export(plot_model_fit)
export(plot_reg)
export(plot_reg_combine)
export(plot_surv_fit)
export(rmst_table)
export(save_docx)
export(save_forest)
export(save_plot)
export(save_table)
export(select_models)
export(stratified_multi_reg)
export(stratified_uni_reg)
export(surv_model_compare)
export(surv_predict)
export(surv_reg)
export(survival_prob)
export(survival_quantiles)
export(survival_summary)
export(uni_reg)
importFrom(MASS,glm.nb)
importFrom(broom,tidy)
importFrom(broom.helpers,tidy_add_reference_rows)
importFrom(broom.helpers,tidy_plus_plus)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(flextable,body_add_flextable)
importFrom(forestploter,forest)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_step)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_linetype_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_light)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,grid.draw)
importFrom(grid,grid.newpage)
importFrom(grid,popViewport)
importFrom(grid,pushViewport)
importFrom(grid,unit)
importFrom(grid,viewport)
importFrom(officer,body_add_gg)
importFrom(officer,body_add_par)
importFrom(officer,read_docx)
importFrom(patchwork,wrap_plots)
importFrom(purrr,map)
importFrom(rlang,.data)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cooks.distance)
importFrom(stats,cor)
importFrom(stats,delete.response)
importFrom(stats,deviance)
importFrom(stats,glm)
importFrom(stats,glm.control)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,reformulate)
importFrom(stats,residuals)
importFrom(stats,setNames)
importFrom(stats,shapiro.test)
importFrom(stats,terms)
importFrom(survival,Surv)
importFrom(survival,cox.zph)
importFrom(survival,coxph)
importFrom(survival,survdiff)
importFrom(survival,survfit)
importFrom(survival,survreg)
importFrom(tibble,tibble)
importFrom(utils,data)
importFrom(utils,globalVariables)
importFrom(utils,head)

<div class="gtreg-hero">

<div>

<h1>

gtregression
</h1>

<p>

Publication-ready regression and survival analysis tables, plots, and
forest plots for real-world health data. Fit models, compare estimates,
visualise results, and export manuscript-ready outputs without
hand-formatting every coefficient.
</p>

<div class="gtreg-actions">

<a href="https://gtregression.thinkdenominator.com/articles/gtregression-intro.html">Start the
workflow</a><a class="secondary" href="https://gtregression.thinkdenominator.com/reference/">Explore
functions</a>

</div>

</div>

<div>

<img src="man/figures/gtregression_hex.png" alt="gtregression logo" class="gtreg-home-logo"/>

</div>

</div>

<!-- badges: start -->

[![R-CMD-check](https://github.com/ThinkDenominator/gtregression/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ThinkDenominator/gtregression/actions/workflows/R-CMD-check.yaml)
[![pkgdown](https://github.com/ThinkDenominator/gtregression/actions/workflows/pkgdown.yaml/badge.svg)](https://gtregression.thinkdenominator.com/)
[![CRAN
status](https://www.r-pkg.org/badges/version/gtregression)](https://CRAN.R-project.org/package=gtregression)
[![CRAN
checks](https://badges.cranchecks.info/worst/gtregression.svg)](https://cran.r-project.org/web/checks/check_results_gtregression.html)
[![CRAN
downloads](https://cranlogs.r-pkg.org/badges/last-month/gtregression)](https://cranlogs.r-pkg.org/downloads/total/last-month/gtregression)
[![CRAN downloads
total](https://cranlogs.r-pkg.org/badges/grand-total/gtregression)](https://CRAN.R-project.org/package=gtregression)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![License:
MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://github.com/ThinkDenominator/gtregression/blob/main/LICENSE.md)
[![Codecov](https://codecov.io/gh/ThinkDenominator/gtregression/branch/main/graph/badge.svg)](https://app.codecov.io/gh/ThinkDenominator/gtregression)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.16905350.svg)](https://doi.org/10.5281/zenodo.16905350)

<!-- badges: end -->

## Publication-Ready Regression, Survival, and Mediation Outputs

`gtregression` helps you move from model to manuscript: fit regression
models, produce clean tables, visualise estimates, merge outputs, and
export results without hand-formatting every coefficient.

It supports logistic, log-binomial, Poisson, robust Poisson, negative
binomial, linear, Cox, parametric survival, and causal mediation
workflows, including adjusted and stratified models.

| Build | What you get |
|----|----|
| Descriptive tables | Grouped summaries with row or column percentages |
| Regression tables | Crude, adjusted, stratified, linear, Cox, and parametric survival outputs |
| Survival analysis | Kaplan-Meier curves, survival summaries, RMST, log-rank tests, Cox PH checks, and survival predictions |
| Mediation analysis | Direct, indirect, total, and proportion mediated effects with causal caveats |
| Visualisations | Regression plots, survival curves, fitted survival curves, and forest tables |
| Interpretation helpers | Confounding, interaction, mediation, convergence, collinearity, model selection, and survival diagnostics |
| Exports | HTML, PDF, PNG, and Word-ready outputs |

## From Data to Manuscript

<div class="gtreg-analysis-path">

<div class="gtreg-path-intro">

<div class="gtreg-kicker">

One connected workflow

</div>

### Start with the question. Finish with a result you can use.

Each step leaves an inspectable object behind, so beginners have a clear
path and experienced analysts retain full control.

</div>

<div class="gtreg-path-steps">

<div class="gtreg-path-step">

<span class="gtreg-path-number">01</span>

<div class="gtreg-path-copy">

**Prepare**

Check variables, labels, levels, and missing data.

</div>

`dissect(data)` <span class="gtreg-path-result">Analysis-ready
data</span>

</div>

<div class="gtreg-path-step">

<span class="gtreg-path-number">02</span>

<div class="gtreg-path-copy">

**Describe**

Build a clear baseline table before modelling.

</div>

`descriptive_table(...)` <span class="gtreg-path-result">Table 1</span>

</div>

<div class="gtreg-path-step">

<span class="gtreg-path-number">03</span>

<div class="gtreg-path-copy">

**Model**

Fit crude, adjusted, stratified, or survival models.

</div>

`uni_reg() + multi_reg()` <span class="gtreg-path-result">Effect
estimates</span>

</div>

<div class="gtreg-path-step">

<span class="gtreg-path-number">04</span>

<div class="gtreg-path-copy">

**Interpret**

Review assumptions, confounding, interaction, and fit.

</div>

`check_*() + compare_models()`
<span class="gtreg-path-result">Defensible model</span>

</div>

<div class="gtreg-path-step">

<span class="gtreg-path-number">05</span>

<div class="gtreg-path-copy">

**Publish**

Merge, visualise, and export polished outputs.

</div>

`forest_reg() + save_table()`
<span class="gtreg-path-result">Manuscript-ready output</span>

</div>

</div>

</div>

## Why It Exists

Many students, researchers, and public health analysts need regression
outputs that are readable, reproducible, and report-ready.
`gtregression` keeps the R syntax approachable while preserving
transparent model objects underneath.

## Built on Trusted R Packages

`gtregression` is intentionally a readable interface over established R
packages. The package uses widely trusted modelling, tidying, plotting,
and reporting tools so users can inspect fitted models and understand
the statistical engines behind each output.

| Area | Core packages used |
|----|----|
| Data handling and tidy workflows | `dplyr`, `purrr`, `tibble`, `rlang` |
| Model fitting | `stats`, `MASS`, `survival`, `risks`, `logistf` |
| Robust and diagnostic inference | `sandwich`, `lmtest`, `broom`, `broom.helpers` |
| Tables and Word-ready reporting | `flextable`, `officer`, `gt` |
| Figures and forest plots | `ggplot2`, `patchwork`, `forestploter`, `scales` |
| Optional development and checking tools | `testthat`, `knitr`, `rmarkdown`, `pkgdown`, `car`, `forcats`, `ggtext` |

The user-facing functions return objects with fitted models, table
bodies, and display metadata that advanced users can audit, modify, or
reuse.

## Install

``` r
# CRAN
install.packages("gtregression")

# Development version
remotes::install_github("ThinkDenominator/gtregression")
```

## Five-Minute Workflow

``` r
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
  )

exposures <- c("age", "lwt", "race", "smoke", "ht", "ui")

attr(birthwt_data$age, "label") <- "Maternal age"
attr(birthwt_data$lwt, "label") <- "Maternal weight"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"

desc <- descriptive_table(
  birthwt_data,
  exposures = exposures,
  by = "low",
  percent = "column",
  show_overall = "last"
)

uni <- uni_reg(
  birthwt_data,
  outcome = "low",
  exposures = exposures,
  approach = "logit"
)

multi <- multi_reg(
  birthwt_data,
  outcome = "low",
  exposures = c("smoke", "ht", "ui"),
  adjust_for = c("age", "lwt", "race"),
  approach = "logit"
)

plot_reg(multi, title = "Adjusted Regression for Low Birth Weight")

forest_reg = forest_reg(forest_df(uni, multi)) 

merge_tables(desc, uni, multi)
```

Variable labels set with `attr(x, "label")` or `labelled::var_label()`
are used automatically in display tables and plots, while original
column names remain available internally for merging, modification, and
testing.

Objects stay inspectable:

``` r
desc$table
uni$table
multi$table
multi$models
```

Optional model-fit statistics can be requested without changing the
publication table:

``` r
uni_stats <- uni_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = exposures,
  approach = "logit",
  model_stats = TRUE
)

uni_stats$model_stats
```

## Browse by Task

| Task | Start here |
|----|----|
| First workflow | [Start Here](https://gtregression.thinkdenominator.com/articles/gtregression-intro.html) |
| Descriptive summaries | [Descriptive Tables](https://gtregression.thinkdenominator.com/articles/descriptive-tables.html) |
| Regression tables | [Regression Tables](https://gtregression.thinkdenominator.com/articles/regression-tables.html) |
| Survival analysis | [Survival Analysis](https://gtregression.thinkdenominator.com/articles/survival-analysis.html) |
| Causal mediation | [Causal Mediation](https://gtregression.thinkdenominator.com/articles/causal-mediation.html) |
| Visualise estimates | [Visualise Results](https://gtregression.thinkdenominator.com/articles/visualise-results.html) |
| Stratified models | [Stratified Analysis](https://gtregression.thinkdenominator.com/articles/stratified-analysis.html) |
| Diagnostics and selection | [Diagnostics](https://gtregression.thinkdenominator.com/articles/diagnostics-selection.html) |
| Confounding and interaction | [Confounding & Interaction](https://gtregression.thinkdenominator.com/articles/confounding-interaction.html) |
| Merge and export | [Customize and Export](https://gtregression.thinkdenominator.com/articles/customize-export.html) |

## Function Map

| Workflow | Functions |
|----|----|
| Describe | `descriptive_table()`, `dissect()` |
| Model | `uni_reg()`, `multi_reg()`, `cox_reg()`, `surv_reg()` |
| Survival | `km_plot()`, `km_risk_table()`, `survival_summary()`, `survival_quantiles()`, `survival_prob()`, `rmst_table()`, `logrank_test()`, `check_ph()`, `surv_model_compare()`, `plot_surv_fit()`, `surv_predict()` |
| Stratify | `stratified_uni_reg()`, `stratified_multi_reg()` |
| Visualise | `plot_reg()`, `plot_reg_combine()`, `forest_df()`, `forest_reg()` |
| Diagnose | `check_convergence()`, `check_collinearity()`, `check_ph()`, `select_models()` |
| Interpret | `identify_confounder()`, `interaction_models()`, `mediation_analysis()`, `plot_mediation()` |
| Polish and export | `modify_table()`, `merge_tables()`, `save_table()`, `save_plot()`, `save_docx()` |

## Citation

If you use `gtregression` in your work, please cite it as:

Polani R, Eliyas SK, Sakthivel M, Kaviprawin M, Krishnamoorthy Y,
Majella MG. *gtregression: Tools for Creating Publication-Ready
Regression Tables.* Zenodo. <https://doi.org/10.5281/zenodo.16905350>

## Acknowledgements

`gtregression` builds on the R ecosystem, especially `stats`,
`survival`, `MASS`, `risks`, `logistf`, `broom`, `broom.helpers`,
`sandwich`, `lmtest`, `dplyr`, `purrr`, `tibble`, `rlang`, `flextable`,
`officer`, `gt`, `ggplot2`, `patchwork`, `forestploter`, and `scales`.

## ----desc-setup, message=FALSE, warning=FALSE---------------------------------
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes")),
    ftv_cat = factor(case_when(
      ftv == 0 ~ "None",
      ftv == 1 ~ "One",
      ftv >= 2 ~ "Two or more"
    ), levels = c("None", "One", "Two or more"))
  )

birthwt_exposures <- c(
  "age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat", "ftv_cat"
)

attr(birthwt_data$age, "label") <- "Maternal age"
attr(birthwt_data$lwt, "label") <- "Maternal weight"
attr(birthwt_data$race, "label") <- "Maternal race"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"
attr(birthwt_data$ui, "label") <- "Uterine irritability"
attr(birthwt_data$ptl_cat, "label") <- "Previous preterm labour"
attr(birthwt_data$ftv_cat, "label") <- "First trimester visits"

## ----desc-column, message=FALSE, warning=FALSE--------------------------------
desc_column <- descriptive_table(
  data = birthwt_data,
  exposures = birthwt_exposures,
  by = "low",
  percent = "column",
  show_overall = "last",
  theme = clinical
)

desc_column

## ----desc-row, message=FALSE, warning=FALSE-----------------------------------
descriptive_table(
  data = birthwt_data,
  exposures = birthwt_exposures,
  by = "low",
  percent = "row",
  show_overall = "first",
  show_missing = no,
  theme = striped
)$table

## ----desc-statistic, message=FALSE, warning=FALSE-----------------------------
descriptive_table(
  data = birthwt_data,
  exposures = c("age", "lwt", "ftv", "smoke"),
  by = low,
  statistic = c(
    age = mean,
    lwt = median,
    ftv = categorical
  ),
  percent = column,
  show_missing = no
)

## ----desc-statistic-quoted, eval=FALSE----------------------------------------
# descriptive_table(
#   data = birthwt_data,
#   exposures = c("age", "lwt", "ftv", "smoke"),
#   by = "low",
#   statistic = c(
#     age = "mean",
#     lwt = "median",
#     ftv = "categorical"
#   )
# )

## ----desc-flex, message=FALSE, warning=FALSE----------------------------------
desc_gt <- descriptive_table(
  data = birthwt_data,
  exposures = birthwt_exposures,
  by = "low",
  percent = "column",
  format = gt
)

desc_gt$table

## ----surv-setup, message=FALSE, warning=FALSE---------------------------------
library(gtregression)
library(dplyr)

data("data_lungcancer", package = "gtregression")

lung_data <- data_lungcancer |>
  mutate(
    trt = factor(trt, levels = c(1, 2),
                 labels = c("Standard treatment", "Test treatment")),
    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes")),
    celltype = factor(
      celltype,
      levels = c("squamous", "smallcell", "adeno", "large"),
      labels = c("Squamous", "Small cell", "Adenocarcinoma", "Large cell")
    )
  )

attr(lung_data$time, "label") <- "Survival time"
attr(lung_data$status, "label") <- "Death status"
attr(lung_data$trt, "label") <- "Treatment group"
attr(lung_data$celltype, "label") <- "Cancer cell type"
attr(lung_data$karno, "label") <- "Karnofsky performance score"
attr(lung_data$age, "label") <- "Age"
attr(lung_data$prior, "label") <- "Prior therapy"

surv_exposures <- c("trt", "celltype", "karno", "age", "prior")

## ----surv-describe, message=FALSE, warning=FALSE------------------------------
lung_summary <- descriptive_table(
  data = lung_data,
  exposures = c("time", "status", "celltype", "karno", "age", "prior"),
  by = trt,
  statistic = c(time = "median", karno = "mean", age = "mean"),
  percent = column,
  show_overall = last
)

lung_summary$table

## ----surv-km-plot, message=FALSE, warning=FALSE-------------------------------
km_curve <- km_plot(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  break_time_by = 200,
  ylim = c(50, 100),
  title = "Kaplan-Meier Survival by Treatment"
)

km_curve

## ----surv-km-panel, fig.width=7, fig.height=5, message=FALSE, warning=FALSE----
km_trt_panel <- km_plot(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  risk_table = FALSE,
  break_time_by = 200,
  ylim = c(50, 100),
  title = "A. Treatment group",
  title_size = 10,
  title_face = plain,
  legend_position = bottom,
  base_size = 10
)

km_prior_panel <- km_plot(
  data = lung_data,
  time = time,
  event = status,
  by = prior,
  risk_table = FALSE,
  break_time_by = 200,
  ylim = c(50, 100),
  title = "B. Prior therapy",
  title_size = 10,
  title_face = plain,
  legend_position = bottom,
  base_size = 10
)

patchwork::wrap_plots(km_trt_panel, km_prior_panel, ncol = 2) +
  patchwork::plot_layout(guides = "collect") &
  ggplot2::theme(legend.position = "bottom")

## ----surv-observed-tables, message=FALSE, warning=FALSE-----------------------
survival_summary(
  data = lung_data,
  time = time,
  event = status,
  by = trt
)$table

survival_prob(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  times = c(90, 180, 365)
)$table

## ----surv-rmst, message=FALSE, warning=FALSE----------------------------------
rmst_table(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  tau = 365
)$table

## ----surv-logrank, message=FALSE, warning=FALSE-------------------------------
logrank_test(
  data = lung_data,
  time = time,
  event = status,
  by = trt
)$table

## ----surv-cox, message=FALSE, warning=FALSE-----------------------------------
cox_crude <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = surv_exposures
)

cox_adjusted <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno)
)

cox_adjusted$table

## ----surv-cox-interaction, message=FALSE, warning=FALSE-----------------------
cox_interaction <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno),
  interaction = trt*prior
)

cox_interaction$table

## ----surv-check-ph, message=FALSE, warning=FALSE------------------------------
check_ph(cox_adjusted)$table

## ----surv-parametric-checks, message=FALSE, warning=FALSE---------------------
surv_model_compare(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno),
  distributions = c(weibull, exponential, "log-normal", "log-logistic")
)$table

plot_surv_fit(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  adjust_for = c(age, karno),
  distributions = c(weibull, "log-logistic"),
  break_time_by = 200
)

## ----surv-parametric, message=FALSE, warning=FALSE----------------------------
surv_crude <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = surv_exposures,
  distribution = loglogistic
)

surv_adjusted <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno),
  distribution = loglogistic,
  model_stats = TRUE
)

surv_adjusted$table
surv_adjusted$model_stats

## ----surv-parametric-interaction, message=FALSE, warning=FALSE----------------
surv_interaction <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno),
  interaction = trt*prior,
  distribution = loglogistic
)

surv_interaction$table

## ----surv-predict, message=FALSE, warning=FALSE-------------------------------
surv_predict(
  model = surv_adjusted$models$trt,
  newdata = data.frame(
    trt = factor("Test treatment", levels = levels(lung_data$trt)),
    age = 60,
    karno = 70
  ),
  times = c(90, 180, 365)
)$table

## ----surv-visualise-export, message=FALSE, warning=FALSE----------------------
plot_reg_combine(
  cox_crude,
  cox_adjusted,
  show_ref = FALSE,
  title_uni = "Crude HR",
  title_multi = "Adjusted HR"
)

surv_forest_data <- forest_df(cox_crude, cox_adjusted, desc = lung_summary)

forest_reg(
  surv_forest_data,
  xlim = list(c(0.25, 8), c(0.25, 8)),
  ticks_at = list(c(0.5, 1, 2, 4, 8), c(0.5, 1, 2, 4, 8)),
  quiet = TRUE
)

options(shiny.sanitize.errors = FALSE)

required_app_packages <- c("shiny", "gt", "flextable", "ggplot2")
missing_app_packages <- required_app_packages[
  !vapply(required_app_packages, requireNamespace, logical(1), quietly = TRUE)
]
if (length(missing_app_packages)) {
  stop(
    "The gtregression app requires these packages: ",
    paste(missing_app_packages, collapse = ", "),
    ". Install them and run gtregression_app() again.",
    call. = FALSE
  )
}

library(shiny)

`%||%` <- function(a, b) if (!is.null(a)) a else b

gtx_ns <- asNamespace("gtregression")

# The app is sourced by Shiny outside the package namespace. Bind the internal
# modules and helpers here so installed-package behaviour matches load_all().
mod_data_prep_ui <- get("mod_data_prep_ui", envir = gtx_ns)
mod_data_prep_server <- get("mod_data_prep_server", envir = gtx_ns)
gtx_reference_candidates <- get("gtx_reference_candidates", envir = gtx_ns)
gtx_relevel_predictors <- get("gtx_relevel_predictors", envir = gtx_ns)
gtx_reference_code <- get("gtx_reference_code", envir = gtx_ns)

gtx_exported <- function(name) {
  name %in% getNamespaceExports("gtregression")
}

gtx_has_formal <- function(fun, arg) {
  arg %in% names(formals(get(fun, envir = gtx_ns)))
}

gtx_call <- function(fun, ...) {
  args <- list(...)
  f <- get(fun, envir = gtx_ns)
  args <- args[intersect(names(args), names(formals(f)))]
  do.call(f, args)
}

gtx_vec_code <- function(x) {
  if (length(x) == 0) return("character(0)")
  paste0("c(", paste(sprintf('\"%s\"', x), collapse = ", "), ")")
}

gtx_has_text <- function(x) {
  !is.null(x) && length(x) > 0 && !is.na(x[1]) && nzchar(x[1])
}

gtx_null_code <- function(x) {
  if (!gtx_has_text(x)) "NULL" else sprintf('\"%s\"', x[1])
}

gtx_object_name <- function(x, fallback) {
  x <- trimws(x %||% "")
  if (!nzchar(x)) return(fallback)
  out <- make.names(x)
  if (!grepl("^[A-Za-z.]", out)) out <- paste0("model_", out)
  out
}

gtx_bool_code <- function(x) {
  if (isTRUE(x)) "TRUE" else "FALSE"
}

gtx_limits_code <- function(x) {
  if (is.null(x) || length(x) != 2L || anyNA(x)) return("NULL")
  paste0("c(", paste(x, collapse = ", "), ")")
}

gtx_code_call <- function(fun, args) {
  keep <- !vapply(args, is.null, logical(1))
  args <- args[keep]
  if (!length(args)) return(paste0(fun, "()"))
  lines <- sprintf("  %s = %s", names(args), unlist(args, use.names = FALSE))
  paste0(fun, "(\n", paste(lines, collapse = ",\n"), "\n)")
}

gtx_code_assign <- function(name, fun, args) {
  paste0(name, " <- ", gtx_code_call(fun, args))
}

gtx_nonempty_vec_code <- function(x) {
  if (length(x)) gtx_vec_code(x) else NULL
}

gtx_table <- function(x) {
  if (inherits(x, "gt_tbl")) return(x)
  if (!is.null(x$table) && inherits(x$table, "gt_tbl")) return(x$table)
  if (!is.null(x$table) && inherits(x$table, "flextable")) {
    return(x$table)
  }
  if (inherits(x, "flextable")) return(x)
  if (is.data.frame(x)) return(gt::gt(x))
  if (!is.null(x$table_display) && is.data.frame(x$table_display)) return(gt::gt(x$table_display))
  if (!is.null(x$table_body) && is.data.frame(x$table_body)) return(gt::gt(x$table_body))
  stop("Could not find a displayable table in this result.", call. = FALSE)
}

gtx_table_output <- function(id) {
  uiOutput(id)
}

gtx_data_output <- function(id) {
  if (requireNamespace("DT", quietly = TRUE)) {
    DT::dataTableOutput(id)
  } else {
    tableOutput(id)
  }
}

gtx_render_data <- function(expr, page_length = 10) {
  if (requireNamespace("DT", quietly = TRUE)) {
    DT::renderDataTable({
      DT::datatable(expr(), options = list(scrollX = TRUE, pageLength = page_length))
    })
  } else {
    renderTable(expr(), striped = TRUE, bordered = TRUE, spacing = "s")
  }
}

gtx_render_table <- function(expr) {
  renderUI({
    res <- expr()
    req(res)
    tryCatch({
      tb <- gtx_table(res)
      if (inherits(tb, "flextable")) {
        return(flextable::htmltools_value(tb))
      }
      if (inherits(tb, "gt_tbl")) {
        return(HTML(gt::as_raw_html(tb, inline_css = TRUE)))
      }
      tb
    }, error = function(e) {
      div(
        class = "alert alert-danger",
        strong("This result could not be displayed. "),
        conditionMessage(e)
      )
    })
  })
}

gtx_capture <- function(expr) {
  value <- NULL
  text <- utils::capture.output(
    withCallingHandlers(
      {
        value <- eval.parent(substitute(expr))
        if (!inherits(value, c("gtregression", "gt_tbl", "ggplot", "patchwork"))) print(value)
      },
      message = function(m) {
        cat(conditionMessage(m), "\n")
        invokeRestart("muffleMessage")
      },
      warning = function(w) {
        cat("Warning: ", conditionMessage(w), "\n", sep = "")
        invokeRestart("muffleWarning")
      }
    )
  )
  list(value = value, text = paste(text, collapse = "\n"))
}

gtx_dataset_names <- function() {
  objs <- try(utils::data(package = "gtregression")$results[, "Item"], silent = TRUE)
  if (inherits(objs, "try-error")) character(0) else sort(unique(objs))
}

gtx_dataset <- function(name) {
  env <- new.env(parent = emptyenv())
  utils::data(list = name, package = "gtregression", envir = env)
  get(name, envir = env)
}

gtx_approaches <- c(
  "Logistic" = "logit",
  "Firth logistic" = "firth",
  "Log-binomial" = "logbinomial",
  "Poisson" = "poisson",
  "Robust Poisson" = "robpoisson",
  "Negative binomial" = "negbin",
  "Linear" = "linear"
)

gtx_parse_named_stats <- function(x) {
  if (is.null(x) || !nzchar(trimws(x))) return(NULL)
  parts <- strsplit(x, ",", fixed = TRUE)[[1]]
  parts <- trimws(parts[nzchar(trimws(parts))])
  out <- character(0)
  for (part in parts) {
    kv <- strsplit(part, "=", fixed = TRUE)[[1]]
    if (length(kv) != 2L || !nzchar(trimws(kv[1])) || !nzchar(trimws(kv[2]))) {
      stop("Statistic overrides should look like: age = mean, lwt = median", call. = FALSE)
    }
    out[trimws(kv[1])] <- trimws(kv[2])
  }
  out
}

gtx_statistic_code <- function(x) {
  stats <- tryCatch(gtx_parse_named_stats(x), error = function(e) NULL)
  if (is.null(stats) || !length(stats)) return(NULL)
  paste0(
    "c(",
    paste(
      sprintf("%s = %s", names(stats), shQuote(unname(stats))),
      collapse = ", "
    ),
    ")"
  )
}

gtx_parse_numeric_vector <- function(x, expected = NULL, name = "value") {
  if (!gtx_has_text(x)) return(NULL)
  parts <- trimws(strsplit(x, ",", fixed = TRUE)[[1]])
  parts <- parts[nzchar(parts)]
  out <- suppressWarnings(as.numeric(parts))
  if (!length(out) || anyNA(out)) {
    stop("`", name, "` should contain numbers separated by commas.", call. = FALSE)
  }
  if (!is.null(expected) && length(out) != expected) {
    stop("`", name, "` should contain exactly ", expected, " numbers.", call. = FALSE)
  }
  out
}

gtx_numeric_vector_code <- function(x, expected = NULL) {
  vals <- tryCatch(
    gtx_parse_numeric_vector(x, expected = expected, name = "value"),
    error = function(e) NULL
  )
  if (is.null(vals)) return(NULL)
  paste0("c(", paste(vals, collapse = ", "), ")")
}

gtx_inline_help <- function(...) {
  div(class = "gtx-field-help", ...)
}

gtx_select_buttons <- function(id) {
  div(
    class = "gtx-select-buttons",
    actionButton(paste0(id, "_all"), "Select all", class = "btn-default btn-xs"),
    actionButton(paste0(id, "_clear"), "Clear all", class = "btn-default btn-xs")
  )
}

gtx_template_birthwt <- function() {
  df <- gtx_dataset("data_birthwt")
  df$race <- factor(df$race, levels = c(1, 2, 3), labels = c("White", "Black", "Other"))
  df$smoke <- factor(df$smoke, levels = c(0, 1), labels = c("No", "Yes"))
  df$ht <- factor(df$ht, levels = c(0, 1), labels = c("No", "Yes"))
  df$ui <- factor(df$ui, levels = c(0, 1), labels = c("No", "Yes"))
  df$low <- factor(df$low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
  df$ptl_cat <- factor(ifelse(df$ptl > 0, "Yes", "No"), levels = c("No", "Yes"))
  df$ftv_cat <- ifelse(df$ftv == 0, "None", ifelse(df$ftv == 1, "One", "Two or more"))
  df$ftv_cat <- factor(df$ftv_cat, levels = c("None", "One", "Two or more"))
  df
}

gtx_template_lung <- function() {
  df <- gtx_dataset("data_lungcancer")
  df$trt <- factor(df$trt, levels = c(1, 2), labels = c("Standard treatment", "Test treatment"))
  df$prior <- factor(df$prior, levels = c(0, 10), labels = c("No", "Yes"))
  df$celltype <- factor(
    df$celltype,
    levels = c("squamous", "smallcell", "adeno", "large"),
    labels = c("Squamous", "Small cell", "Adenocarcinoma", "Large cell")
  )
  df
}

gtx_template_code <- function(template) {
  switch(
    template,
    birthwt = paste(
      'data("data_birthwt", package = "gtregression")',
      "",
      "df <- data_birthwt |>",
      "  dplyr::mutate(",
      '    race = factor(race, levels = c(1, 2, 3), labels = c("White", "Black", "Other")),',
      '    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),',
      '    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),',
      '    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),',
      '    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),',
      '    ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes")),',
      '    ftv_cat = factor(dplyr::case_when(',
      '      ftv == 0 ~ "None",',
      '      ftv == 1 ~ "One",',
      '      ftv >= 2 ~ "Two or more"',
      '    ), levels = c("None", "One", "Two or more"))',
      "  )",
      sep = "\n"
    ),
    lung = paste(
      'data("data_lungcancer", package = "gtregression")',
      "",
      "df <- data_lungcancer |>",
      "  dplyr::mutate(",
      '    trt = factor(trt, levels = c(1, 2), labels = c("Standard treatment", "Test treatment")),',
      '    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes")),',
      '    celltype = factor(',
      "      celltype,",
      '      levels = c("squamous", "smallcell", "adeno", "large"),',
      '      labels = c("Squamous", "Small cell", "Adenocarcinoma", "Large cell")',
      "    )",
      "  )",
      sep = "\n"
    ),
    diabetes = paste(
      'data("data_diabetes_mediation", package = "gtregression")',
      "df <- data_diabetes_mediation",
      sep = "\n"
    ),
    endometrial = paste(
      'data("data_endometrial", package = "gtregression")',
      "df <- data_endometrial",
      sep = "\n"
    ),
    ""
  )
}

gtx_validate_roles <- function(
    outcome = NULL, time = NULL, event = NULL, exposures = character(0),
    adjust_for = character(0), stratifier = NULL, context = "analysis") {
  exposures <- exposures[nzchar(exposures)]
  adjust_for <- adjust_for[nzchar(adjust_for)]
  protected <- unique(c(outcome, time, event, stratifier))
  protected <- protected[!is.na(protected) & nzchar(protected)]

  role_overlap <- intersect(exposures, protected)
  if (length(role_overlap)) {
    stop(
      "Role conflict in ", context, ": ",
      paste(role_overlap, collapse = ", "),
      " cannot also be selected as an exposure.",
      call. = FALSE
    )
  }

  adjust_overlap <- intersect(adjust_for, protected)
  if (length(adjust_overlap)) {
    stop(
      "Role conflict in ", context, ": ",
      paste(adjust_overlap, collapse = ", "),
      " cannot also be selected as an adjustment variable.",
      call. = FALSE
    )
  }

  duplicated_model_terms <- intersect(exposures, adjust_for)
  if (length(duplicated_model_terms)) {
    stop(
      "Role conflict in ", context, ": ",
      paste(duplicated_model_terms, collapse = ", "),
      " cannot be both exposures and adjustment variables.",
      call. = FALSE
    )
  }

  invisible(TRUE)
}

gtx_km_break_value <- function(x) {
  if (is.null(x) || !length(x) || is.na(x[1])) return(NULL)
  x[1]
}

gtx_export_size <- function(preset = "standard", is_forest = FALSE) {
  preset <- preset %||% "standard"
  if (is_forest) {
    return(switch(
      preset,
      wide = list(width = 16, height = 9),
      many_rows = list(width = 16, height = 13),
      compact = list(width = 10, height = 7),
      list(width = 13, height = 9)
    ))
  }
  switch(
    preset,
    wide = list(width = 11, height = 7),
    many_rows = list(width = 12, height = 8),
    compact = list(width = 7, height = 5),
    list(width = 9, height = 6)
  )
}

gtx_table_downloads <- function(output, id, result_reactive, table_get = function(x) x) {
  output[[paste0(id, "_docx")]] <- downloadHandler(
    filename = function() paste0(id, ".docx"),
    content = function(file) gtregression::save_table(table_get(result_reactive()), filename = file, format = "docx")
  )
  output[[paste0(id, "_rtf")]] <- downloadHandler(
    filename = function() paste0(id, ".rtf"),
    content = function(file) gtregression::save_table(table_get(result_reactive()), filename = file, format = "rtf")
  )
  output[[paste0(id, "_html")]] <- downloadHandler(
    filename = function() paste0(id, ".html"),
    content = function(file) gtregression::save_table(table_get(result_reactive()), filename = file, format = "html")
  )
}

gtx_plot_downloads <- function(output, id, plot_reactive, is_forest = FALSE, size_get = function() "standard") {
  output[[paste0(id, "_png")]] <- downloadHandler(
    filename = function() paste0(id, ".png"),
    content = function(file) {
      p <- plot_reactive()
      dims <- gtx_export_size(size_get(), is_forest = is_forest)
      if (is_forest) {
        gtregression::save_forest(p, filename = file, format = "png", width = dims$width, height = dims$height)
      } else {
        gtregression::save_plot(p, filename = file, format = "png", width = dims$width, height = dims$height)
      }
    }
  )
  output[[paste0(id, "_pdf")]] <- downloadHandler(
    filename = function() paste0(id, ".pdf"),
    content = function(file) {
      p <- plot_reactive()
      dims <- gtx_export_size(size_get(), is_forest = is_forest)
      if (is_forest) {
        gtregression::save_forest(p, filename = file, format = "pdf", width = dims$width, height = dims$height)
      } else {
        gtregression::save_plot(p, filename = file, format = "pdf", width = dims$width, height = dims$height)
      }
    }
  )
}

tagListDownload <- function(id, table = TRUE, plot = FALSE) {
  tags$div(
    class = "download-strip",
    if (table) tagList(
      downloadButton(paste0(id, "_docx"), "DOCX"),
      downloadButton(paste0(id, "_rtf"), "RTF"),
      downloadButton(paste0(id, "_html"), "HTML")
    ),
    if (plot) tagList(
      downloadButton(paste0(id, "_png"), "PNG"),
      downloadButton(paste0(id, "_pdf"), "PDF")
    )
  )
}

gtx_code_panel <- function(title, id) {
  div(
    class = "gtx-card",
    div(
      class = "gtx-code-head",
      h4(title),
      tags$button(
        type = "button",
        class = "btn btn-default btn-sm gtx-copy",
        onclick = sprintf("gtxCopyCode('%s', this)", id),
        icon("copy"),
        " Copy"
      )
    ),
    verbatimTextOutput(id)
  )
}

css <- HTML("
:root {
  --gtx-charcoal: #262626;
  --gtx-ink: #171717;
  --gtx-soft: #F6F6F3;
  --gtx-muted: #666864;
  --gtx-line: #D9DAD6;
  --gtx-accent: #386A5A;
  --gtx-accent-soft: #E8F0EC;
  --gtx-text: #292A28;
}
body {
  color: var(--gtx-text);
  background: #FCFCFA;
  font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif;
  font-size: 15px;
  line-height: 1.5;
}
.navbar {
  background: var(--gtx-charcoal) !important;
  border: 0;
  border-radius: 0;
  box-shadow: 0 1px 0 rgba(0, 0, 0, 0.18);
}
.navbar-brand {
  color: white !important;
  font-weight: 700;
  letter-spacing: 0;
}
.navbar-nav > li > a {
  color: #E4E4E1 !important;
  font-weight: 600;
}
.navbar-nav > li > a:hover,
.navbar-nav > li > a:focus,
.navbar-nav > .active > a,
.navbar-nav > .active > a:hover,
.navbar-nav > .active > a:focus {
  background: #111 !important;
  color: #fff !important;
}
.container-fluid { max-width: 1480px; }
h1, h2, h3, h4, h5, h6 { color: var(--gtx-ink); font-weight: 700; }
.gtx-card {
  border: 1px solid var(--gtx-line);
  background: #fff;
  border-radius: 8px;
  padding: 16px;
  margin-bottom: 16px;
  box-shadow: 0 1px 3px rgba(20, 20, 20, 0.05);
}
.gtx-side { position: sticky; top: 12px; }
.gtx-tool-menu { padding: 12px 16px 8px; }
.gtx-tool-menu h3 { margin: 0 0 8px; font-size: 1.05rem; }
.gtx-tool-menu .form-group { margin-bottom: 0; }
.gtx-tool-menu .shiny-options-group {
  display: flex;
  flex-wrap: wrap;
  gap: 2px 22px;
}
.gtx-tool-menu .radio-inline {
  border-bottom: 3px solid transparent;
  margin: 0;
  padding: 8px 2px 10px 22px;
  font-weight: 700;
}
.gtx-tool-menu .radio-inline:has(input:checked) {
  border-bottom-color: var(--gtx-accent);
  color: var(--gtx-ink);
}
.gtx-help { color: var(--gtx-muted); font-size: 0.94rem; margin-bottom: 12px; }
.gtx-badge {
  display: inline-block;
  padding: 3px 8px;
  border-radius: 999px;
  background: var(--gtx-accent-soft);
  color: #285143;
  font-size: 0.78rem;
  font-weight: 700;
}
	.download-strip .btn { margin-right: 6px; margin-bottom: 6px; }
	pre {
	  background: #F5F5F2;
	  border: 1px solid var(--gtx-line);
	  border-radius: 6px;
	  color: #242424;
	}
	.nav-tabs { border-bottom-color: var(--gtx-line); }
	.nav-tabs > li > a { color: #555753; font-weight: 650; border-radius: 6px 6px 0 0; }
	.nav-tabs > li.active > a,
	.nav-tabs > li.active > a:hover,
	.nav-tabs > li.active > a:focus {
	  color: var(--gtx-ink);
	  border-color: var(--gtx-line) var(--gtx-line) #fff;
	  box-shadow: inset 0 3px 0 var(--gtx-accent);
	}
	.form-group label { font-weight: 700; }
	.form-control,
	.selectize-input {
	  border-color: #C9CAC6;
	  border-radius: 5px;
	  box-shadow: none;
	}
	.form-control:focus,
	.selectize-input.focus {
	  border-color: var(--gtx-accent);
	  box-shadow: 0 0 0 3px rgba(56, 106, 90, 0.13);
	}
	input[type='checkbox'], input[type='radio'] { accent-color: var(--gtx-accent); }
	.btn { border-radius: 5px; font-weight: 650; }
	.btn-primary,
	.btn-info {
	  background: var(--gtx-charcoal);
	  border-color: var(--gtx-charcoal);
	  color: #fff;
	}
	.btn-primary:hover, .btn-primary:focus, .btn-primary:active,
	.btn-info:hover, .btn-info:focus, .btn-info:active {
	  background: #111;
	  border-color: #111;
	}
	.btn-default { background: #fff; border-color: #BFC0BC; color: #30312F; }
	.btn-default:hover, .btn-default:focus { background: var(--gtx-soft); border-color: #8E908B; }
	.table > thead > tr > th { color: var(--gtx-ink); border-bottom-color: #777873; }
	.table > tbody > tr > td { border-top-color: #E4E4E0; }
	.gtx-code-head {
	  display: flex;
	  align-items: center;
	  justify-content: space-between;
	  gap: 12px;
	  margin-bottom: 8px;
	}
	.gtx-code-head h4 { margin: 0; }
	.gtx-copy { white-space: nowrap; }
	.gtx-close-app {
	  position: fixed;
	  right: 18px;
	  bottom: 18px;
	  z-index: 2000;
	  box-shadow: 0 4px 12px rgba(15, 23, 42, 0.18);
	}
	.gtx-step {
	  border-left: 4px solid var(--gtx-accent);
	  background: var(--gtx-soft);
	  padding: 10px 12px;
	  margin: 10px 0;
	  color: #3D403C;
	}
	.gtx-note {
	  background: transparent;
	  border: 0;
	  border-left: 2px solid #C9CAC6;
	  border-radius: 0;
	  padding: 4px 0 4px 10px;
	  color: var(--gtx-muted);
	  margin: 10px 0;
	}
	.gtx-workflow-note { margin-bottom: 14px; }
	.gtx-workflow-note h3 { margin-top: 0; }
	.gtx-step-row {
	  display: grid;
	  grid-template-columns: repeat(4, minmax(0, 1fr));
	  gap: 8px;
	  margin-top: 12px;
	}
	.gtx-step-row .gtx-step {
	  margin: 0;
	  min-height: 48px;
	}
	.gtx-section-label {
	  color: var(--gtx-charcoal);
	  font-size: 15px;
	  font-weight: 750;
	  margin: 2px 0 8px;
	}
	.gtx-model-card {
	  background: #FAFAF8;
	  border: 1px solid var(--gtx-line);
	  border-left: 4px solid var(--gtx-accent);
	  border-radius: 6px;
	  margin: 12px 0;
	  padding: 14px 16px 8px;
	}
	.gtx-model-card h4 {
	  color: var(--gtx-charcoal);
	  font-size: 1rem;
	  font-weight: 750;
	  margin: 0 0 12px;
	}
	.gtx-inline-actions {
	  display: flex;
	  gap: 8px;
	  margin: 8px 0 4px;
	}
	.gtx-warning {
	  background: #FFF4E5;
	  border-left: 4px solid #B45309;
	  color: #713F12;
	  padding: 10px 12px;
	  margin: 10px 0;
	}
	@media (max-width: 900px) {
	  .gtx-step-row { grid-template-columns: repeat(2, minmax(0, 1fr)); }
	}
	.gtx-workflow-bar {
	  background: #F3F3F0;
	  border-bottom: 1px solid var(--gtx-line);
	  padding: 10px 24px;
	}
	.gtx-workflow-wrap {
	  max-width: 1480px;
	  margin: 0 auto;
	  display: flex;
	  align-items: center;
	  gap: 8px;
	  flex-wrap: wrap;
	}
	.gtx-workflow-step-pill {
	  border: 1px solid var(--gtx-line);
	  border-radius: 999px;
	  padding: 5px 10px;
	  font-size: 0.86rem;
	  font-weight: 700;
	  background: white;
	  color: var(--gtx-muted);
	}
	.gtx-workflow-step-pill.done {
	  background: var(--gtx-accent-soft);
	  border-color: #B9CEC5;
	  color: #285143;
	}
	.gtx-workflow-step-pill.current {
	  background: var(--gtx-charcoal);
	  border-color: var(--gtx-charcoal);
	  color: white;
	}
	.gtx-workflow-next {
	  margin-left: auto;
	  font-size: 0.9rem;
	  color: #454743;
	}
	.gtx-analysis-strip {
	  background: #fff;
	  border-bottom: 1px solid var(--gtx-line);
	  padding: 7px 24px;
	}
	.gtx-analysis-strip-inner {
	  max-width: 1480px;
	  margin: 0 auto;
	  display: flex;
	  gap: 18px;
	  align-items: center;
	  flex-wrap: wrap;
	  color: #50524e;
	  font-size: 0.86rem;
	}
	.gtx-analysis-item strong { color: var(--gtx-ink); }
	.gtx-preflight {
	  background: var(--gtx-accent-soft);
	  border-left: 4px solid var(--gtx-accent);
	  border-radius: 0 6px 6px 0;
	  padding: 10px 12px;
	  margin: 12px 0;
	  color: #30483f;
	  font-size: 0.9rem;
	}
	.gtx-preflight strong { color: #203b32; }
	.gtx-run-group {
	  display: grid;
	  grid-template-columns: 1fr 1fr;
	  gap: 8px;
	  margin-top: 8px;
	}
	.gtx-run-group .btn { width: 100%; }
	.gtx-field-help {
	  color: var(--gtx-muted);
	  font-size: 0.88rem;
	  margin: -6px 0 10px 0;
	  line-height: 1.35;
	}
	.gtx-template-grid {
	  display: grid;
	  grid-template-columns: 1fr 1fr;
	  gap: 8px;
	  margin-bottom: 10px;
	}
	.gtx-template-grid .btn { white-space: normal; }
	.gtx-app-mode {
	  background: var(--gtx-soft);
	  border: 1px solid var(--gtx-line);
	  border-radius: 8px;
	  padding: 10px 12px;
	  margin-bottom: 14px;
	}
	.gtx-export-grid {
	  display: grid;
	  grid-template-columns: 1fr 1fr;
	  gap: 8px;
	  align-items: end;
	}
	.gtx-select-buttons {
	  display: flex;
	  gap: 6px;
	  margin: -8px 0 12px 0;
	}
	.gtx-select-buttons .btn {
	  padding: 2px 8px;
	  font-size: 0.78rem;
	}
	.gtx-button-stack {
	  display: flex;
	  flex-direction: column;
	  gap: 8px;
	  margin-top: 8px;
	}
	.gtx-advanced-tool-tabs .nav-pills {
	  margin-bottom: 12px;
	}
	.gtx-advanced-tool-tabs .tab-content {
	  padding-top: 4px;
	}
	.nav-pills > li > a { color: #50524E; border-radius: 5px; font-weight: 650; }
	.nav-pills > li.active > a,
	.nav-pills > li.active > a:hover,
	.nav-pills > li.active > a:focus {
	  background: var(--gtx-charcoal);
	  color: white;
	}
	.shiny-output-error-validation {
	  color: #7A2E24;
	  background: #FAF1EE;
	  border-left: 3px solid #A64B3C;
	  padding: 9px 11px;
	}
	@media (max-width: 767px) {
	  body { font-size: 14px; }
	  .gtx-side { position: static; }
	  .gtx-workflow-bar { padding: 8px 12px; }
	  .gtx-workflow-next { width: 100%; margin-left: 0; }
	  .gtx-analysis-strip { padding: 7px 12px; }
	  .gtx-analysis-strip-inner { gap: 8px 14px; }
	  .gtx-run-group { grid-template-columns: 1fr; }
	}
	")

copy_js <- HTML("
function gtxCopyCode(id, button) {
  var el = document.getElementById(id);
  var text = el ? el.innerText : '';
  if (!text.trim()) return;
  navigator.clipboard.writeText(text).then(function() {
    var original = button.innerHTML;
    button.innerHTML = 'Copied';
    setTimeout(function() { button.innerHTML = original; }, 1200);
  }).catch(function() {
    window.prompt('Copy this code:', text);
  });
}
")

ui <- navbarPage(
  title = tagList("gtregression", span(class = "gtx-badge", "App")),
  id = "main_nav",
  header = tagList(
    tags$head(tags$style(css), tags$script(copy_js)),
    div(class = "gtx-close-app", actionButton("close_app", "Close app", icon = icon("power-off"), class = "btn-danger")),
    uiOutput("workflow_guide"),
    uiOutput("analysis_status")
  ),

  tabPanel(
    "Data",
    fluidRow(
      column(
        4,
        div(
          class = "gtx-card gtx-side",
          h3("Load Data"),
          div(class = "gtx-help", "Start with a built-in teaching dataset or import CSV, Excel, RDS, Stata, SPSS, or SAS data."),
          div(
            class = "gtx-step",
            "Recommended flow: load data, build descriptive tables, run univariable models, reselect variables, then run multivariable or survival models."
          ),
          h4("Quick-start templates"),
          div(
            class = "gtx-template-grid",
            actionButton("template_birthwt", "Birthweight regression"),
            actionButton("template_lung", "Lung survival"),
            actionButton("template_diabetes", "Diabetes mediation"),
            actionButton("template_endometrial", "Firth logistic")
          ),
          gtx_inline_help("Templates load labelled teaching data and preselect common variables. They are safe to overwrite by choosing another dataset."),
          tags$hr(),
          selectInput("dataset_name", "Built-in dataset", choices = c("", gtx_dataset_names())),
          actionButton("load_dataset", "Load built-in data", class = "btn-primary"),
          tags$hr(),
          fileInput(
            "data_file",
            "Import a data file",
            accept = c(".csv", ".xlsx", ".xls", ".rds", ".dta", ".sav", ".zsav", ".por", ".sas7bdat", ".xpt")
          ),
          gtx_inline_help("Supported: CSV, Excel, RDS data frames, Stata (.dta), SPSS (.sav/.zsav/.por), and SAS (.sas7bdat/.xpt)."),
          checkboxInput("csv_header", "CSV has header", TRUE),
          selectInput("csv_sep", "CSV separator", choices = c("Comma" = ",", "Semicolon" = ";", "Tab" = "\t"), selected = ","),
          textInput("xlsx_sheet", "Excel sheet name/index", value = ""),
          gtx_inline_help("CSV settings apply only to CSV files. The sheet field applies only to Excel; leave it blank for the first sheet."),
          actionButton("load_upload", "Load uploaded file", class = "btn-primary")
        )
      ),
      column(
        8,
        div(class = "gtx-card", h3("Preview"), div(class = "gtx-help", "First 10 rows only. Use Data Summary to inspect the full structure before modelling."), gtx_data_output("data_preview")),
        div(class = "gtx-card", h4("Data Summary"), verbatimTextOutput("data_summary")),
        gtx_code_panel("Reusable Code", "data_code")
      )
    )
  ),

  tabPanel(
    "Data Prep",
    value = "data_prep",
    mod_data_prep_ui("data_prep")
  ),

  tabPanel(
    "Descriptive",
    fluidRow(
      column(
        4,
        div(
          class = "gtx-card gtx-side",
          h3("Descriptive Table"),
          div(class = "gtx-help", "Use this first to understand the data and to create baseline tables for merge_table(), forest_df(), and reports."),
          div(class = "gtx-note", "Start by choosing Group by. Leave it as None for one overall summary; otherwise the selected variable defines the comparison columns."),
          uiOutput("desc_inputs"),
          selectInput("desc_percent", "Percent", choices = c("column", "row"), selected = "column"),
          selectInput("desc_overall", "Overall column", choices = c("no", "first", "last"), selected = "no"),
          selectInput("desc_missing", "Missing values", choices = c("ifany", "no"), selected = "ifany"),
          selectInput("desc_dich", "Binary display", choices = c("all_levels", "single_row"), selected = "all_levels"),
          textInput("desc_statistic", "Numeric statistic override", placeholder = 'Optional: age = mean, lwt = median'),
          gtx_inline_help("Use named overrides when a variable needs a different summary from the default."),
          actionButton("run_desc", "Run descriptive table", class = "btn-primary")
        )
      ),
      column(
        8,
        div(class = "gtx-card", tagListDownload("desc", table = TRUE), gtx_table_output("desc_table")),
        gtx_code_panel("Reusable Code", "desc_code")
      )
    )
  ),

  tabPanel(
    "Regression",
    fluidRow(
      column(
        4,
        div(
          class = "gtx-card gtx-side",
          h3("Regression Tables"),
          div(
            class = "gtx-step",
            "Run univariable analysis first. Then keep the important variables selected and run the multivariable model. Use Show reference categories when you plan to merge tables or create forest plots."
          ),
          div(class = "gtx-note", "Choose an outcome first, then exposures. Adjustment variables belong only in the multivariable model and should be selected using the study question, a DAG, and clinical knowledge."),
          uiOutput("reg_inputs"),
          uiOutput("reg_reference_inputs"),
          selectInput("reg_approach", "Approach", choices = gtx_approaches, selected = "logit"),
          checkboxInput("reg_show_ref", "Show reference categories", value = TRUE),
          checkboxInput("reg_model_stats", "Store model statistics", value = TRUE),
          gtx_inline_help("Enabled by default so the Model Stats tab is ready after fitting. These statistics are kept out of the publication table."),
          uiOutput("reg_preflight"),
          div(
            class = "gtx-run-group",
            actionButton("run_uni", "Run univariate", class = "btn-primary"),
            actionButton("run_multi", "Run multivariable", class = "btn-primary")
          ),
          tags$hr(),
          uiOutput("reg_strata_inputs"),
          checkboxInput("reg_strata_multi", "Stratified multivariable", value = FALSE),
          actionButton("run_strat", "Run stratified regression", class = "btn-primary")
        )
      ),
      column(
        8,
        tabsetPanel(
          tabPanel("Univariate", div(class = "gtx-card", tagListDownload("uni", table = TRUE), gtx_table_output("uni_table")), gtx_code_panel("Code", "uni_code")),
          tabPanel("Multivariable", div(class = "gtx-card", tagListDownload("multi", table = TRUE), gtx_table_output("multi_table")), gtx_code_panel("Code", "multi_code")),
          tabPanel("Stratified", div(class = "gtx-card", tagListDownload("strat", table = TRUE), gtx_table_output("strat_table")), gtx_code_panel("Code", "strat_code")),
          tabPanel("Model Stats", div(class = "gtx-card", gtx_data_output("reg_stats")))
        )
      )
    )
  ),

  tabPanel(
    "Survival",
    div(
      class = "gtx-card gtx-tool-menu",
      h3("Choose a survival model"),
      radioButtons(
        "surv_mode",
        label = NULL,
        choices = c(
          "Cox regression" = "cox",
          "Parametric survival" = "survreg"
        ),
        selected = "cox",
        inline = TRUE
      )
    ),
    fluidRow(
      column(
        4,
        div(
          class = "gtx-card gtx-side",
          uiOutput("surv_mode_intro"),
          div(class = "gtx-note", "Choose Cox for hazard ratios without a distributional assumption. Choose parametric survival when a specified distribution and time-ratio interpretation are appropriate."),
          uiOutput("surv_inputs"),
          checkboxInput("surv_show_ref", "Show reference categories", value = TRUE),
          checkboxInput("surv_multivariable", "Single multivariable model", value = FALSE),
          uiOutput("surv_preflight"),
          conditionalPanel(
            "input.surv_mode == 'cox'",
            actionButton("run_cox", "Run Cox regression", class = "btn-primary")
          ),
          conditionalPanel(
            "input.surv_mode == 'survreg'",
            actionButton("run_survreg", "Run parametric survival regression", class = "btn-primary")
          ),
          tags$hr(),
          h4("Kaplan-Meier curve"),
          div(class = "gtx-help", "This observed survival view is available for either model workflow."),
          uiOutput("km_group_input"),
          selectInput("km_theme", "KM theme", choices = c("classic", "minimal", "bw", "light", "none"), selected = "classic"),
          checkboxInput("km_y_percent", "Display y-axis as percent", TRUE),
          checkboxInput("km_grid", "Show grid", FALSE),
          checkboxInput("km_conf", "Show confidence interval", TRUE),
          checkboxInput("km_p", "Show log-rank p-value", TRUE),
          checkboxInput("km_risk", "Show risk table", TRUE),
          textInput("km_ylim", "Y limits", placeholder = "Optional: 50, 100"),
          textInput("km_xlim", "X limits", placeholder = "Optional: 0, 800"),
          tags$details(
            tags$summary("Additional KM options"),
            numericInput("km_break", "Time break interval", value = NA, min = 0)
          ),
          actionButton("run_km", "Draw KM plot", class = "btn-primary")
        )
      ),
      column(
        8,
        tabsetPanel(
          id = "surv_output_tab",
          tabPanel(
            "Model Result",
            conditionalPanel(
              "input.surv_mode == 'cox'",
              tabsetPanel(
                id = "cox_result_tab",
                tabPanel(
                  "Exposure models",
                  div(class = "gtx-card", tagListDownload("cox_exposure", table = TRUE), gtx_table_output("cox_exposure_table"))
                ),
                tabPanel(
                  "Multivariable model",
                  div(class = "gtx-card", tagListDownload("cox_multi", table = TRUE), gtx_table_output("cox_multi_table"))
                )
              )
            ),
            conditionalPanel(
              "input.surv_mode == 'survreg'",
              tabsetPanel(
                id = "survreg_result_tab",
                tabPanel(
                  "Exposure models",
                  div(class = "gtx-card", tagListDownload("survreg_exposure", table = TRUE), gtx_table_output("survreg_exposure_table"))
                ),
                tabPanel(
                  "Multivariable model",
                  div(class = "gtx-card", tagListDownload("survreg_multi", table = TRUE), gtx_table_output("survreg_multi_table"))
                )
              )
            )
          ),
          tabPanel("KM Plot", div(class = "gtx-card", tagListDownload("km", table = FALSE, plot = TRUE), plotOutput("km_plot", height = "650px"))),
          tabPanel("Survival Tables", div(class = "gtx-card", gtx_table_output("logrank_table")), div(class = "gtx-card", gtx_table_output("surv_summary_table"))),
          tabPanel("Code", gtx_code_panel("Code", "surv_code"))
        )
      )
    )
  ),

  tabPanel(
    "Visualise & Export",
    div(
      class = "gtx-card gtx-tool-menu",
      h3("Choose a tool"),
      radioButtons(
        "visual_tool",
        label = NULL,
        choices = c(
          "Merge tables" = "merge",
          "Modify table" = "modify",
          "Regression plot" = "plot",
          "Forest plot" = "forest",
          "Model fit" = "fit"
        ),
        selected = "merge",
        inline = TRUE
      )
    ),
    fluidRow(
      column(
        4,
        div(
          class = "gtx-card gtx-side",
          conditionalPanel(
            "input.visual_tool == 'merge'",
            h3("Merge Tables"),
            div(class = "gtx-help", "Choose two or more completed tables. They are merged in the order shown below."),
            uiOutput("merge_table_choices"),
            gtx_inline_help("After merging, use DOCX or RTF for editable manuscripts and HTML for a browser-ready copy."),
            actionButton("run_merge", "Merge selected tables", class = "btn-primary")
          ),
          conditionalPanel(
            "input.visual_tool == 'modify'",
            h3("Modify Table"),
            div(class = "gtx-help", "Refine a completed table without changing the original result. Use one mapping per line for labels."),
            uiOutput("modify_source_controls"),
            textAreaInput("modify_variable_labels", "Variable labels", placeholder = "age = Maternal age\nsmoke = Smoking during pregnancy", rows = 3),
            gtx_inline_help("One `internal name = display label` per line."),
            div(class = "gtx-note", strong("Level-label format: "), "use the internal variable name and the exact current level shown in the table. For example: `smoke: Yes = Smoker` or `race: Other = Other ethnic group`. Do not use the display label on the left of `:`."),
            textAreaInput("modify_level_labels", "Level labels", placeholder = "smoke: Yes = Smoker", rows = 3),
            gtx_inline_help("One `variable: current level = new level` per line. The original level is case-sensitive."),
            textAreaInput("modify_header_labels", "Header labels", placeholder = "estimate = Adjusted OR\np.value = P value", rows = 3),
            gtx_inline_help("Use a visible header name or an alias such as `estimate`, `p.value`, or `N`."),
            textInput("modify_caption", "Caption", placeholder = "Optional table title"),
            checkboxInput("modify_bold_labels", "Bold characteristic labels", TRUE),
            checkboxInput("modify_bold_levels", "Bold category levels", FALSE),
            checkboxInput("modify_italic_labels", "Italicize characteristic labels", FALSE),
            checkboxInput("modify_italic_levels", "Italicize category levels", FALSE),
            checkboxInput("modify_remove_n", "Remove displayed N columns", FALSE),
            checkboxInput("modify_remove_n_obs", "Remove complete-case N note", FALSE),
            checkboxInput("modify_remove_abbreviations", "Remove abbreviations note", FALSE),
            checkboxInput("modify_remove_adjustment_note", "Remove automatic adjustment note", FALSE),
            textAreaInput("modify_caveat", "Additional note", placeholder = "Optional manuscript note", rows = 2),
            actionButton("run_modify_table", "Apply table changes", class = "btn-primary")
          ),
          conditionalPanel(
            "input.visual_tool == 'plot'",
            h3("Regression Plot"),
            uiOutput("plot_content_controls"),
            checkboxInput("plot_log_x", "Use a log x-axis", FALSE),
            gtx_inline_help("Linear scale is the default. Use log scale only when you need proportional spacing around the null value of 1."),
            actionButton("run_plot_reg", "Preview regression plot", class = "btn-primary")
          ),
          conditionalPanel(
            "input.visual_tool == 'forest'",
            h3("Forest Plot"),
            uiOutput("forest_content_controls"),
            selectInput("forest_side", "Forest plot side", choices = c("Text then plot" = "right", "Plot then text" = "left"), selected = "right"),
            gtx_inline_help("Choose which side contains the plotted confidence intervals; this does not change the estimates."),
            numericInput("forest_ci_width", "CI column width", value = 20, min = 4, step = 1),
            gtx_inline_help("Increase this when confidence intervals or tick labels overlap. Wider columns need a wider export preset."),
            textInput("forest_xlim", "Forest x limits", placeholder = "Optional: 0.25, 12"),
            gtx_inline_help("Enter two comma-separated limits. Leave blank to use forestploter's automatic range."),
            textInput("forest_ticks", "Forest tick marks", placeholder = "Optional: 0.5, 1, 2, 4, 8"),
            gtx_inline_help("If x-axis labels overlap, enter fewer ticks such as 0.5, 1, 2, 4. Every tick must lie within x limits; use x limits as well when the interval range is very wide."),
            div(
              class = "gtx-help",
              "Combine descriptive, crude, and adjusted results when available. For overlapping labels, use fewer ticks, wider x limits, or a wider CI column."
            ),
            actionButton("run_forest", "Preview forest plot", class = "btn-primary")
          ),
          conditionalPanel(
            "input.visual_tool == 'fit'",
            h3("Model Fit"),
            div(class = "gtx-help", "Choose Auto for the recommended diagnostic panel. Calibration is most useful for multivariable binary-outcome models; bins only affects calibration."),
            uiOutput("fit_source_controls"),
            selectInput("fit_type", "Type", choices = c("Auto (recommended)" = "auto", "All diagnostics" = "all", "Residuals vs fitted" = "residual", "Normal Q-Q" = "qq", "Scale-location" = "scale_location", "Cook's distance" = "cooks", "Observed vs predicted" = "observed_predicted", "Calibration" = "calibration"), selected = "auto"),
            textInput("fit_model_name", "Model name for univariate objects", placeholder = "Optional, e.g. smoke"),
            conditionalPanel("input.fit_type == 'calibration'", numericInput("fit_bins", "Calibration bins", value = 10, min = 2)),
            numericInput("fit_base_size", "Base font size", value = 13, min = 6, step = 1),
            actionButton("run_fit_plot", "Draw model fit", class = "btn-primary")
          ),
          tags$hr(),
          div(
            class = "gtx-note",
            "Browser outputs are previews. Downloads are generated in temporary app files, then your browser chooses where to save them; the app never writes into a user folder itself. Copy the generated plot code into RStudio for final publication sizing."
          ),
          selectInput(
            "export_preset",
            "Export size preset",
            choices = c("Standard" = "standard", "Wide" = "wide", "Many rows" = "many_rows", "Compact" = "compact"),
            selected = "standard"
          ),
          gtx_inline_help("Wide adds horizontal room; Many rows adds width and height. These settings affect downloaded PNG/PDF files, not model results."),
          uiOutput("export_size_hint")
        )
      ),
      column(
        8,
        tabsetPanel(
          id = "visual_output_tab",
          tabPanel(
            "Merged Table",
            div(
              class = "gtx-card",
              tagListDownload("merged", table = TRUE),
              gtx_inline_help("HTML preserves a browser-viewable table; DOCX and RTF are intended for editable reports."),
              gtx_table_output("merged_table")
            )
          ),
          tabPanel(
            "Modified Table",
            div(class = "gtx-card", tagListDownload("modified", table = TRUE), gtx_table_output("modified_table"))
          ),
          tabPanel("plot_reg", div(class = "gtx-card", tagListDownload("plotreg", table = FALSE, plot = TRUE), plotOutput("plotreg_plot", height = "620px"))),
          tabPanel("forest_reg", div(class = "gtx-card", tagListDownload("forest", table = FALSE, plot = TRUE), plotOutput("forest_plot", height = "760px"))),
          tabPanel("Model fit", div(class = "gtx-card", tagListDownload("fitplot", table = FALSE, plot = TRUE), plotOutput("fit_plot", height = "680px"))),
          tabPanel("Code", gtx_code_panel("Code", "visual_code"))
        )
      )
    )
  ),

  tabPanel(
    "Advanced",
    div(
      class = "gtx-card gtx-workflow-note",
      h3("Model building and diagnostic tools"),
      p("Start with model selection when you want the app to build a candidate path from an outcome and predictor set. Use model comparison after fitting two or more models in the Regression or Survival tabs."),
      div(
        class = "gtx-step-row",
        span(class = "gtx-step", strong("1  Select"), " candidate predictors"),
        span(class = "gtx-step", strong("2  Fit"), " candidate models"),
        span(class = "gtx-step", strong("3  Compare"), " saved models"),
        span(class = "gtx-step", strong("4  Check"), " assumptions")
      )
    ),
    div(
      class = "gtx-card gtx-tool-menu",
      h3("Choose an advanced tool"),
      radioButtons(
        "advanced_tool",
        label = NULL,
        choices = c(
          "Select models" = "selection",
          "Compare models" = "comparison",
          "Confounder" = "confounder",
          "Interaction" = "interaction",
          "Convergence" = "convergence",
          "Collinearity" = "collinearity"
        ),
        selected = "selection",
        inline = TRUE
      )
    ),
    fluidRow(
      column(
        4,
        div(
          class = "gtx-card gtx-side",
          uiOutput("advanced_config")
        )
      ),
      column(
        8,
        tabsetPanel(
          id = "advanced_output_tab",
          tabPanel(
            "Output",
            div(
              class = "gtx-card",
              uiOutput("advanced_result_heading"),
              uiOutput("advanced_downloads"),
              gtx_table_output("advanced_table")
            ),
            div(class = "gtx-card", uiOutput("advanced_guidance"), verbatimTextOutput("advanced_text"))
          ),
          tabPanel("Code", gtx_code_panel("Code", "advanced_code"))
        )
      )
    )
  ),

  tabPanel(
    "Mediation",
    fluidRow(
      column(
        4,
        div(
          class = "gtx-card gtx-side",
          h3("Causal Mediation"),
          div(class = "gtx-note", "Specify the causal question before running this analysis. The result is a screening aid and depends on no unmeasured exposure-mediator, mediator-outcome, or exposure-outcome confounding assumptions."),
          uiOutput("med_inputs"),
          selectInput("med_approach", "Outcome model", choices = c("logit", "linear"), selected = "logit"),
          numericInput("med_sims", "Bootstrap replicates", value = 300, min = 50, step = 50),
          numericInput("med_seed", "Seed", value = 123, min = 1),
          actionButton("run_mediation", "Run mediation analysis", class = "btn-primary")
        )
      ),
      column(
        8,
        tabsetPanel(
          tabPanel("Table", div(class = "gtx-card", tagListDownload("med", table = TRUE), gtx_table_output("med_table"))),
          tabPanel("Diagram", div(class = "gtx-card", tagListDownload("medplot", table = FALSE, plot = TRUE), plotOutput("med_plot", height = "620px"))),
          tabPanel("Code", gtx_code_panel("Code", "med_code"))
        )
      )
    )
  ),

  tabPanel(
    "Session Code",
    fluidRow(
      column(
        12,
        div(
          class = "gtx-card",
          h3("Session Code"),
          div(class = "gtx-help", "This script is updated automatically from the completed steps in this session. Copy one command from any Code panel, or copy/download the complete workflow here."),
          downloadButton("download_session_script", "Download complete R script", class = "btn-primary"),
          tags$hr(),
          gtx_code_panel("Complete workflow", "full_workflow_code")
        )
      )
    )
  ),

  tabPanel(
    "Help",
    fluidRow(
      column(
        12,
        div(
          class = "gtx-card",
          h2("gtregression App"),
          p("This app is a menu-driven companion to the R package. It is designed for beginners who want publication-ready tables and plots, while always showing reproducible R code."),
          tags$ol(
            tags$li("Load a dataset."),
            tags$li("Create descriptive tables."),
            tags$li("Run univariable analysis to screen variables."),
            tags$li("Reselect the variables you want, then run multivariable, Cox, or survival models."),
            tags$li("Use confounder, interaction, model comparison, diagnostics, or mediation tools when needed."),
            tags$li("Use the Code tabs to copy the exact R commands into RStudio for reproducible saving and editing.")
          ),
          p("Use the Close app button in the bottom-right corner to stop the Shiny session cleanly."),
          h3("Dependency Philosophy"),
          p("The package keeps core statistical dependencies in Imports. App packages are loaded only when this app is launched, which keeps command-line gtregression light and CRAN-friendly."),
          h3("Important Notes"),
          tags$ul(
            tags$li("Use reference categories when you want binary variables to align cleanly across descriptive, regression, and forest outputs."),
            tags$li("For crowded forest plots, copy the forest code into RStudio and increase save_forest() width/height, x limits, or CI column width."),
            tags$li("If a browser preview is cramped, the saved output may still be fine when exported on a wider canvas."),
            tags$li("Mediation output is a causal aid only when the assumptions are supported by design, temporal order, and subject-matter knowledge.")
          )
        )
      )
    )
  )
)

server <- function(input, output, session) {
  rv <- reactiveValues(
    data = NULL, data_name = NULL, data_code = "",
    desc = NULL, uni = NULL, multi = NULL, strat = NULL,
    cox = NULL, survreg = NULL,
    cox_exposure = NULL, cox_multi = NULL,
    survreg_exposure = NULL, survreg_multi = NULL,
    km = NULL, mediation = NULL,
    plotreg = NULL, forest = NULL, fitplot = NULL, merged = NULL, modified = NULL,
    advanced = NULL, advanced_text = "", advanced_code = "",
    advanced_title = "Choose a tool to begin",
    advanced_guidance = "The app will explain how to read the selected result here.",
    compare_count = 2L,
    last_message = "Load data or choose a quick-start template."
  )

  observeEvent(input$close_app, {
    stopApp()
  })

  data_prep <- mod_data_prep_server("data_prep", reactive(rv$data))

  analysis_data <- reactive({
    data_prep$result()
  })

  require_analysis_data <- function() {
    data <- analysis_data()
    if (is.null(data)) {
      showNotification(
        "Load a dataset first. Original data is active automatically; use Data Prep only when you want prepared data.",
        type = "warning",
        duration = 7
      )
      return(NULL)
    }
    data
  }

  run_safely <- function(action, expr, on_error = NULL) {
    tryCatch(
      eval.parent(substitute(expr)),
      shiny.silent.error = function(e) NULL,
      error = function(e) {
        if (is.function(on_error)) on_error()
        message <- conditionMessage(e)
        rv$last_message <- paste0(action, " could not be completed. Review the inputs and try again.")
        showNotification(
          paste0(action, " could not be completed: ", message),
          type = "error",
          duration = 10
        )
        NULL
      }
    )
  }

  clear_advanced_error <- function() {
    rv$advanced <- NULL
    rv$advanced_text <- ""
    rv$advanced_guidance <- "Correct the highlighted choices and run the tool again. The app is still ready to use."
  }

  select_candidates <- function(exclude = character(0)) {
    vars <- names(analysis_data() %||% data.frame())
    setdiff(vars, exclude[nzchar(exclude)])
  }

  wire_select_buttons <- function(id, selected) {
    observeEvent(input[[paste0(id, "_all")]], {
      updateCheckboxGroupInput(session, id, selected = selected())
    }, ignoreInit = TRUE)

    observeEvent(input[[paste0(id, "_clear")]], {
      updateCheckboxGroupInput(session, id, selected = character(0))
    }, ignoreInit = TRUE)
  }

  wire_select_buttons("desc_exposures", reactive({
    select_candidates(input$desc_by %||% character(0))
  }))
  wire_select_buttons("reg_exposures", reactive({
    select_candidates(c(input$reg_outcome, input$reg_adjust %||% character(0)))
  }))
  wire_select_buttons("reg_adjust", reactive({
    select_candidates(c(input$reg_outcome, input$reg_exposures %||% character(0)))
  }))
  wire_select_buttons("surv_exposures", reactive({
    select_candidates(c(
      input$surv_time,
      input$surv_event,
      input$surv_stratifier,
      input$surv_adjust %||% character(0)
    ))
  }))
  wire_select_buttons("surv_adjust", reactive({
    select_candidates(c(
      input$surv_time,
      input$surv_event,
      input$surv_stratifier,
      input$surv_exposures %||% character(0)
    ))
  }))
  wire_select_buttons("adv_exposures", reactive({
    select_candidates(c(
      input$adv_outcome,
      input$adv_time,
      input$adv_event,
      input$adv_covariates %||% character(0)
    ))
  }))
  wire_select_buttons("adv_covariates", reactive({
    select_candidates(c(
      input$adv_outcome,
      input$adv_time,
      input$adv_event,
      input$adv_exposures %||% character(0)
    ))
  }))
  wire_select_buttons("conf_exposures", reactive({
    select_candidates(c(
      input$adv_outcome,
      input$adv_time,
      input$adv_event,
      input$conf_candidate
    ))
  }))
  wire_select_buttons("med_covariates", reactive({
    select_candidates(c(input$med_exposure, input$med_mediator, input$med_outcome))
  }))

  output$workflow_guide <- renderUI({
    steps <- c(
      Data = !is.null(rv$data),
      `Data Prep` = !is.null(analysis_data()),
      Describe = !is.null(rv$desc),
      Model = !is.null(rv$uni) || !is.null(rv$multi) || !is.null(rv$strat) ||
        !is.null(rv$cox) || !is.null(rv$survreg) || !is.null(rv$mediation),
      Visualise = !is.null(rv$km) || !is.null(rv$plotreg) ||
        !is.null(rv$forest) || !is.null(rv$fitplot),
      Export = !is.null(rv$merged)
    )
    first_pending <- if (any(!steps)) names(steps)[which(!steps)[1]] else "Export"
    div(
      class = "gtx-workflow-bar",
      div(
        class = "gtx-workflow-wrap",
        lapply(names(steps), function(nm) {
          cls <- if (isTRUE(steps[[nm]])) "done" else if (identical(nm, first_pending)) "current" else ""
          span(class = paste("gtx-workflow-step-pill", cls), nm)
        }),
        span(class = "gtx-workflow-next", strong("Suggested action: "), rv$last_message)
      )
    )
  })

  output$analysis_status <- renderUI({
    data <- analysis_data()
    source <- if (is.null(data)) {
      "Not selected"
    } else if (isTRUE(data_prep$using_prepared())) {
      "Prepared data"
    } else {
      "Original data"
    }
    outcome <- input$reg_outcome %||% ""
    model <- input$reg_approach %||% ""
    div(
      class = "gtx-analysis-strip",
      div(
        class = "gtx-analysis-strip-inner",
        span(class = "gtx-analysis-item", strong("Dataset: "), rv$data_name %||% "None"),
        span(class = "gtx-analysis-item", strong("Analysis source: "), source),
        if (!is.null(data)) span(class = "gtx-analysis-item", strong("Size: "), paste0(nrow(data), " rows x ", ncol(data), " columns")),
        if (gtx_has_text(outcome)) span(class = "gtx-analysis-item", strong("Outcome: "), outcome),
        if (gtx_has_text(model)) span(class = "gtx-analysis-item", strong("Approach: "), model)
      )
    )
  })

  output$reg_preflight <- renderUI({
    data <- analysis_data()
    exposures <- input$reg_exposures %||% character(0)
    if (is.null(data) || !gtx_has_text(input$reg_outcome) || !length(exposures)) {
      return(div(class = "gtx-preflight", strong("Before running: "), "choose an outcome and at least one exposure."))
    }
    adjustment <- input$reg_adjust %||% character(0)
    references <- selected_references()
    reference_text <- if (length(references)) {
      paste(paste(names(references), unlist(references), sep = " = "), collapse = "; ")
    } else {
      "No categorical predictor baselines selected"
    }
    div(
      class = "gtx-preflight",
      strong("Ready to run: "),
      paste0(
        input$reg_approach, " model; outcome = ", input$reg_outcome,
        "; ", length(exposures), " exposure", if (length(exposures) == 1) "" else "s",
        "; ", length(adjustment), " adjustment variable", if (length(adjustment) == 1) "" else "s",
        "; N = ", nrow(data), "."
      ),
      tags$br(),
      span("Reference categories: ", reference_text, ".")
    )
  })

  output$surv_preflight <- renderUI({
    data <- analysis_data()
    exposures <- input$surv_exposures %||% character(0)
    ready <- !is.null(data) && gtx_has_text(input$surv_time) &&
      gtx_has_text(input$surv_event) && length(exposures)
    if (!ready) {
      return(div(class = "gtx-preflight", strong("Before running: "), "choose time, event, and at least one exposure."))
    }
    adjustment <- input$surv_adjust %||% character(0)
    mode <- if (isTRUE(input$surv_multivariable)) "single multivariable model" else "separate exposure models"
    model_label <- if (identical(input$surv_mode, "survreg")) {
      paste0("parametric survival (", input$surv_dist %||% "weibull", ")")
    } else {
      "Cox regression"
    }
    div(
      class = "gtx-preflight",
      strong("Ready to run: "),
      paste0(
        model_label, "; time = ", input$surv_time, "; event = ", input$surv_event,
        "; ", length(exposures), " exposure", if (length(exposures) == 1) "" else "s",
        "; ", length(adjustment), " adjustment variable", if (length(adjustment) == 1) "" else "s",
        "; ", mode, "; N = ", nrow(data), "."
      )
    )
  })

  output$export_size_hint <- renderUI({
    preset <- input$export_preset %||% "standard"
    plot_dims <- gtx_export_size(preset, is_forest = FALSE)
    forest_dims <- gtx_export_size(preset, is_forest = TRUE)
    gtx_inline_help(
      paste0(
        "Downloads use approximately ",
        plot_dims$width, " x ", plot_dims$height,
        " inches for standard plots and ",
        forest_dims$width, " x ", forest_dims$height,
        " inches for forest plots."
      )
    )
  })

  load_template <- function(template, data, name, updates = list()) {
    rv$data <- data
    rv$data_name <- name
    rv$data_code <- gtx_template_code(template)
    rv$last_message <- "Original data is active. Open Data Prep only if you want to make changes."
    showNotification(paste(name, "template loaded. Original data is ready for analysis."), type = "message")
    session$onFlushed(function() {
      for (id in names(updates)) {
        value <- updates[[id]]
        if (id %in% c("desc_exposures", "reg_exposures", "reg_adjust", "surv_exposures", "surv_adjust", "adv_exposures", "adv_covariates", "med_covariates")) {
          updateCheckboxGroupInput(session, id, selected = value)
        } else {
          updateSelectInput(session, id, selected = value)
        }
      }
    }, once = TRUE)
  }

  observeEvent(input$template_birthwt, {
    vars <- c("age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat", "ftv_cat")
    load_template(
      "birthwt",
      gtx_template_birthwt(),
      "Birthweight regression",
      list(
        desc_exposures = vars,
        desc_by = "low",
        reg_outcome = "low",
        reg_exposures = vars,
        reg_adjust = c("age", "lwt", "race", "ptl_cat", "ftv_cat"),
        plot_source = "uni"
      )
    )
  })

  observeEvent(input$template_lung, {
    vars <- c("age", "karno", "trt", "celltype", "prior")
    load_template(
      "lung",
      gtx_template_lung(),
      "Lung survival",
      list(
        desc_exposures = vars,
        desc_by = "trt",
        surv_time = "time",
        surv_event = "status",
        surv_exposures = vars,
        surv_adjust = c("age", "karno"),
        km_by = "trt",
        adv_time = "time",
        adv_event = "status",
        adv_exposures = vars,
        adv_covariates = c("age", "karno")
      )
    )
  })

  observeEvent(input$template_diabetes, {
    covars <- c("age", "blood_pressure", "pregnancies", "diabetes_pedigree")
    load_template(
      "diabetes",
      gtx_dataset("data_diabetes_mediation"),
      "Diabetes mediation",
      list(
        med_exposure = "obesity",
        med_mediator = "glucose",
        med_outcome = "diabetes",
        med_covariates = covars,
        desc_exposures = c("obesity", "glucose", "diabetes", covars),
        desc_by = "diabetes"
      )
    )
  })

  observeEvent(input$template_endometrial, {
    df <- gtx_dataset("data_endometrial")
    load_template(
      "endometrial",
      df,
      "Endometrial Firth logistic",
      list(
        reg_outcome = names(df)[1],
        reg_exposures = setdiff(names(df), names(df)[1]),
        reg_approach = "firth"
      )
    )
  })

  observeEvent(input$load_dataset, {
    req(gtx_has_text(input$dataset_name))
    rv$data <- gtx_dataset(input$dataset_name)
    rv$data_name <- input$dataset_name
    rv$data_code <- paste0('data("', input$dataset_name, '", package = "gtregression")\ndf <- ', input$dataset_name)
    rv$last_message <- "Original data is active. Open Data Prep only if you want to make changes."
    showNotification("Dataset loaded. Original data is ready for analysis.", type = "message")
  })

  observeEvent(input$load_upload, {
    req(input$data_file)
    path <- input$data_file$datapath
    ext <- tolower(tools::file_ext(input$data_file$name))
    if (ext == "csv") {
      rv$data <- utils::read.csv(path, header = input$csv_header, sep = input$csv_sep, check.names = FALSE)
      rv$data_code <- sprintf('df <- read.csv("path/to/file.csv", header = %s, sep = "%s", check.names = FALSE)', input$csv_header, input$csv_sep)
    } else if (ext %in% c("xls", "xlsx")) {
      if (!requireNamespace("readxl", quietly = TRUE)) {
        showNotification("Install readxl to import Excel files.", type = "error")
        return(NULL)
      }
      sheet <- input$xlsx_sheet
      sheet <- if (gtx_has_text(sheet)) sheet else 1
      rv$data <- readxl::read_excel(path, sheet = sheet) |> as.data.frame()
      rv$data_code <- sprintf('df <- readxl::read_excel("path/to/file.xlsx", sheet = %s)', shQuote(as.character(sheet)))
    } else if (ext == "rds") {
      imported <- readRDS(path)
      if (!is.data.frame(imported)) {
        showNotification("The RDS file must contain a data.frame or tibble.", type = "error")
        return(NULL)
      }
      rv$data <- as.data.frame(imported)
      rv$data_code <- 'df <- readRDS("path/to/file.rds")'
    } else if (ext %in% c("dta", "sav", "zsav", "por", "sas7bdat", "xpt")) {
      if (!requireNamespace("haven", quietly = TRUE)) {
        showNotification("Install haven to import Stata, SPSS, or SAS files.", type = "error")
        return(NULL)
      }
      reader <- switch(
        ext,
        dta = "read_dta",
        sav = "read_sav",
        zsav = "read_sav",
        por = "read_por",
        sas7bdat = "read_sas",
        xpt = "read_xpt"
      )
      rv$data <- as.data.frame(getExportedValue("haven", reader)(path))
      rv$data_code <- sprintf('df <- haven::%s("path/to/file.%s")', reader, ext)
    } else {
      showNotification("Unsupported file type. Use CSV, Excel, RDS, Stata, SPSS, or SAS.", type = "error")
      return(NULL)
    }
    rv$data_name <- input$data_file$name
    rv$last_message <- "Original data is active. Open Data Prep only if you want to make changes."
    showNotification("Uploaded data loaded. Original data is ready for analysis.", type = "message")
  })

  output$data_preview <- gtx_render_data(function() {
    req(rv$data)
    utils::head(rv$data, 10)
  }, page_length = 10)

  output$data_summary <- renderPrint({
    req(rv$data)
    cat("Dataset:", rv$data_name %||% "data", "\n")
    cat("Rows:", nrow(rv$data), "\n")
    cat("Columns:", ncol(rv$data), "\n\n")
    print(utils::str(rv$data))
  })

  output$data_code <- renderText(rv$data_code)

  output$desc_inputs <- renderUI({
    req(analysis_data())
    vars <- names(analysis_data())
    tagList(
      selectInput("desc_by", "Group by", choices = c("None" = "", vars)),
      checkboxGroupInput("desc_exposures", "Variables to summarise", choices = vars),
      gtx_select_buttons("desc_exposures")
    )
  })

  output$reg_inputs <- renderUI({
    req(analysis_data())
    vars <- names(analysis_data())
    tagList(
      selectInput("reg_outcome", "Outcome", choices = c("", vars)),
      checkboxGroupInput("reg_exposures", "Exposures", choices = vars),
      gtx_select_buttons("reg_exposures"),
      checkboxGroupInput("reg_adjust", "Adjust for", choices = vars),
      gtx_select_buttons("reg_adjust"),
      textInput("reg_interaction", "Interaction term", placeholder = "Optional: smoke*race")
    )
  })

  reference_candidates <- reactive({
    req(analysis_data())
    gtx_reference_candidates(
      analysis_data(),
      unique(c(input$reg_exposures %||% character(0), input$reg_adjust %||% character(0))),
      outcome = input$reg_outcome
    )
  })

  output$reg_reference_inputs <- renderUI({
    candidates <- reference_candidates()
    if (!length(candidates)) return(NULL)
    controls <- Map(function(variable, levels, index) {
      selectInput(
        paste0("reg_reference_", index),
        paste0(variable, " reference category"),
        choices = levels,
        selected = levels[[1]]
      )
    }, names(candidates), candidates, seq_along(candidates))
    tagList(
      tags$hr(),
      h4("Reference categories"),
      div(class = "gtx-help", "The selected category is the baseline. Every reported category is interpreted relative to this baseline."),
      controls
    )
  })

  selected_references <- reactive({
    candidates <- reference_candidates()
    if (!length(candidates)) return(list())
    values <- lapply(seq_along(candidates), function(index) input[[paste0("reg_reference_", index)]] %||% candidates[[index]][[1]])
    stats::setNames(values, names(candidates))
  })

  regression_data <- reactive({
    req(analysis_data())
    gtx_relevel_predictors(analysis_data(), selected_references())
  })

  regression_code_prefix <- reactive({
    candidates <- reference_candidates()
    references <- selected_references()
    lines <- c(
      analysis_code_prefix(),
      gtx_reference_code(references, candidates, data_name = "analysis_data")
    )
    paste(lines, collapse = "\n")
  })

  analysis_code_prefix <- reactive({
    source_label <- if (isTRUE(data_prep$using_prepared())) "prepared" else "original"
    lines <- c(
      paste0("# The app was set to use the ", source_label, " dataset."),
      "analysis_data <- df"
    )
    if (isTRUE(data_prep$using_prepared()) && isTRUE(data_prep$changed())) {
      prep <- gsub("\\bdata\\b", "analysis_data", data_prep$code(), perl = TRUE)
      lines <- c(lines, "# Reproduce the preparation choices made in the app.", prep)
    }
    paste(lines, collapse = "\n")
  })

  output$reg_strata_inputs <- renderUI({
    req(analysis_data())
    selectInput("reg_stratifier", "Stratifier", choices = c("None" = "", names(analysis_data())))
  })

  output$surv_inputs <- renderUI({
    req(analysis_data())
    vars <- names(analysis_data())
    tagList(
      selectInput("surv_time", "Time", choices = c("", vars)),
      selectInput("surv_event", "Event", choices = c("", vars)),
      checkboxGroupInput("surv_exposures", "Exposures", choices = vars),
      gtx_select_buttons("surv_exposures"),
      checkboxGroupInput("surv_adjust", "Adjust for", choices = vars),
      gtx_select_buttons("surv_adjust"),
      selectInput("surv_stratifier", "Stratifier", choices = c("None" = "", vars)),
      conditionalPanel(
        "input.surv_mode == 'survreg'",
        selectInput("surv_dist", "Parametric distribution", choices = c("weibull", "exponential", "lognormal", "loglogistic"), selected = "weibull")
      )
    )
  })

  output$km_group_input <- renderUI({
    req(analysis_data())
    selectInput("km_by", "Group curves by", choices = c("None" = "", names(analysis_data())))
  })

  output$surv_mode_intro <- renderUI({
    if (identical(input$surv_mode, "survreg")) {
      tagList(
        h3("Parametric Survival"),
        div(class = "gtx-help", "Estimate time ratios with a selected survival distribution. Choose the distribution below before running the model.")
      )
    } else {
      tagList(
        h3("Cox Regression"),
        div(class = "gtx-help", "Estimate crude or adjusted hazard ratios without specifying the baseline hazard distribution.")
      )
    }
  })

  output$advanced_config <- renderUI({
    req(analysis_data())
    vars <- names(analysis_data())
    tool <- input$advanced_tool %||% "selection"
    model_choices <- names(Filter(Negate(is.null), list(
      "Univariate regression" = rv$uni,
      "Multivariable regression" = rv$multi,
      "Cox exposure models" = rv$cox_exposure,
      "Cox multivariable model" = rv$cox_multi,
      "Parametric exposure models" = rv$survreg_exposure,
      "Parametric multivariable model" = rv$survreg_multi
    )))
    model_values <- unname(c(
      "Univariate regression" = "uni",
      "Multivariable regression" = "multi",
      "Cox exposure models" = "cox_exposure",
      "Cox multivariable model" = "cox_multi",
      "Parametric exposure models" = "survreg_exposure",
      "Parametric multivariable model" = "survreg_multi"
    )[model_choices])
    named_model_choices <- stats::setNames(model_values, model_choices)
    collinearity_choices <- named_model_choices[named_model_choices %in% c("multi", "cox_exposure", "cox_multi", "survreg_exposure", "survreg_multi")]
    tool_copy <- switch(
      tool,
      selection = list("Model selection", "Explore forward, backward, or bidirectional candidate-model paths. Selection is a screening aid and should be combined with clinical reasoning."),
      comparison = list("Model comparison", "Build two or more named candidate models here, then compare their samples, nesting, fit statistics, and primary effect estimate."),
      confounder = list("Confounder assessment", "Estimate how a candidate variable changes the exposure effect. Use this as a transparent screening aid alongside a prespecified DAG."),
      interaction = list("Interaction assessment", "Test whether the exposure association differs across levels of a possible effect modifier."),
      convergence = list("Convergence check", "Check whether regression models were fitted reliably before interpreting their estimates."),
      collinearity = list("Collinearity check", "Inspect a fitted multivariable model for predictors that contain overlapping information.")
    )

    approach_choices <- if (tool == "convergence") {
      gtx_approaches
    } else {
      c(gtx_approaches, "Cox" = "cox", "Surv Reg" = "survreg")
    }

    model_roles <- function(include_covariates = FALSE, include_exposures = FALSE,
                            covariate_label = "Adjustment variables") {
      current_approach <- input$adv_approach %||% "logit"
      survival_advanced <- isTRUE(current_approach %in% c("cox", "survreg"))
      tagList(
        selectInput("adv_approach", "Regression approach", choices = approach_choices, selected = current_approach),
        if (survival_advanced) tagList(
          selectInput("adv_time", "Follow-up time", choices = c("Choose a variable" = "", vars), selected = input$adv_time %||% ""),
          selectInput("adv_event", "Event status", choices = c("Choose a variable" = "", vars), selected = input$adv_event %||% "")
        ) else selectInput("adv_outcome", "Outcome", choices = c("Choose a variable" = "", vars), selected = input$adv_outcome %||% ""),
        if (identical(current_approach, "survreg"))
          selectInput("adv_distribution", "Survival distribution", choices = c("weibull", "exponential", "lognormal", "loglogistic"), selected = input$adv_distribution %||% "weibull"),
        if (include_exposures) tagList(
          checkboxGroupInput("adv_exposures", "Candidate predictors", choices = vars, selected = input$adv_exposures %||% character(0)),
          gtx_select_buttons("adv_exposures")
        ),
        if (include_covariates) tagList(
          checkboxGroupInput("adv_covariates", covariate_label, choices = vars, selected = input$adv_covariates %||% character(0)),
          gtx_select_buttons("adv_covariates")
        )
      )
    }

    comparison_roles <- function() {
      current_approach <- input$compare_approach %||% "logit"
      survival_comparison <- current_approach %in% c("cox", "survreg")
      excluded <- if (survival_comparison) {
        c(input$compare_time, input$compare_event)
      } else {
        input$compare_outcome
      }
      predictors <- setdiff(vars, excluded[nzchar(excluded)])
      cards <- lapply(seq_len(rv$compare_count), function(i) {
        exposure_id <- paste0("compare_exposures_", i)
        adjust_id <- paste0("compare_adjust_", i)
        int_a_id <- paste0("compare_interaction_a_", i)
        int_b_id <- paste0("compare_interaction_b_", i)
        selected_exposures <- intersect(input[[exposure_id]] %||% character(0), predictors)
        selected_adjust <- intersect(input[[adjust_id]] %||% character(0), predictors)
        interaction_choices <- unique(c(selected_exposures, selected_adjust))
        div(
          class = "gtx-model-card",
          h4(paste("Candidate model", i)),
          textInput(
            paste0("compare_name_", i),
            "Model name",
            value = input[[paste0("compare_name_", i)]] %||% paste("Model", i)
          ),
          checkboxGroupInput(exposure_id, "Reported exposures", choices = predictors, selected = selected_exposures),
          checkboxGroupInput(adjust_id, "Adjustment variables", choices = predictors, selected = selected_adjust),
          gtx_inline_help("The app combines reported exposures and adjustment variables into one fitted candidate model."),
          selectInput(int_a_id, "Interaction term 1 (optional)", choices = c("None" = "", interaction_choices), selected = input[[int_a_id]] %||% ""),
          selectInput(int_b_id, "Interaction term 2 (optional)", choices = c("None" = "", interaction_choices), selected = input[[int_b_id]] %||% "")
        )
      })
      tagList(
        selectInput("compare_approach", "Regression approach", choices = c(gtx_approaches, "Cox" = "cox", "Surv Reg" = "survreg"), selected = current_approach),
        if (survival_comparison) tagList(
          selectInput("compare_time", "Follow-up time", choices = c("Choose a variable" = "", vars), selected = input$compare_time %||% ""),
          selectInput("compare_event", "Event status", choices = c("Choose a variable" = "", vars), selected = input$compare_event %||% "")
        ) else selectInput("compare_outcome", "Outcome", choices = c("Choose a variable" = "", vars), selected = input$compare_outcome %||% ""),
        if (identical(current_approach, "survreg"))
          selectInput("compare_distribution", "Survival distribution", choices = c("weibull", "exponential", "lognormal", "loglogistic"), selected = input$compare_distribution %||% "weibull"),
        selectInput("adv_primary_exposure", "Primary exposure to track (optional)", choices = c("Not specified" = "", predictors), selected = input$adv_primary_exposure %||% ""),
        div(
          class = "gtx-inline-actions",
          actionButton("compare_add_model", icon("plus"), " Add model"),
          actionButton("compare_remove_model", icon("minus"), " Remove last", disabled = if (rv$compare_count <= 2L) "disabled" else NULL)
        ),
        cards
      )
    }

    controls <- switch(
      tool,
      selection = tagList(
        div(class = "gtx-section-label", "Build a candidate model path"),
        gtx_inline_help("Choose one outcome and the predictors that may enter the model. This tool fits the candidate sequence for you; it does not require a previously fitted model."),
        model_roles(include_exposures = TRUE),
        selectInput("select_direction", "Selection direction", choices = c("Forward" = "forward", "Backward" = "backward", "Both directions" = "both"), selected = input$select_direction %||% "both"),
        gtx_inline_help("The output shows the route taken and marks the model preferred by AIC."),
        actionButton("run_select_models", icon("play"), " Run model selection", class = "btn-primary btn-block")
      ),
      comparison = tagList(
        div(class = "gtx-section-label", "Build and compare candidate models"),
        gtx_inline_help("Choose the common outcome and approach once. Give each model a meaningful name, then choose its exposures, adjustment variables, and optional interaction."),
        comparison_roles(),
        gtx_inline_help("Interactions can use only variables included in that candidate. The comparison table flags different analysis samples and non-nested model pairs."),
        actionButton("run_compare", icon("table-columns"), " Fit and compare models", class = "btn-primary btn-block")
      ),
      confounder = tagList(
        model_roles(),
        checkboxGroupInput("conf_exposures", "Exposures of interest", choices = vars, selected = input$conf_exposures %||% character(0)),
        gtx_select_buttons("conf_exposures"),
        selectInput("conf_candidate", "Potential confounder", choices = c("Choose a variable" = "", vars)),
        gtx_inline_help("The app screens every selected exposure against this candidate. A change-in-estimate result cannot establish causality; confirm decisions with a DAG and subject-matter knowledge."),
        actionButton("run_confounder", icon("magnifying-glass-chart"), " Assess confounding", class = "btn-primary btn-block")
      ),
      interaction = tagList(
        model_roles(
          include_covariates = TRUE,
          covariate_label = "Covariates included in both interaction models"
        ),
        selectInput("conf_exposure", "Exposure of interest", choices = c("Choose a variable" = "", vars)),
        selectInput("conf_candidate", "Possible effect modifier", choices = c("Choose a variable" = "", vars)),
        gtx_inline_help("Covariates are included in both the model with and without the interaction. A small interaction p-value suggests that the exposure effect may differ between groups; interpret stratum-specific estimates as well."),
        actionButton("run_interaction", icon("code-branch"), " Test interaction", class = "btn-primary btn-block")
      ),
      convergence = tagList(
        model_roles(include_exposures = TRUE),
        gtx_inline_help("Convergence checks apply to the standard regression approaches shown here. Resolve warnings before reporting estimates."),
        actionButton("run_convergence", icon("circle-check"), " Check convergence", class = "btn-primary btn-block")
      ),
      collinearity = tagList(
        if (!length(collinearity_choices)) div(class = "gtx-note", "Fit a multivariable, Cox, or parametric survival model first."),
        selectInput("adv_collinearity_model", "Fitted model", choices = collinearity_choices),
        gtx_inline_help("Use the fitted model that contains all predictors you want to assess. Collinearity is a property of predictors fitted together."),
        if (length(collinearity_choices)) {
          actionButton("run_collinearity", icon("arrows-left-right-to-line"), " Check collinearity", class = "btn-primary btn-block")
        } else {
          actionButton("run_collinearity", icon("arrows-left-right-to-line"), " Check collinearity", class = "btn-primary btn-block", disabled = "disabled")
        }
      )
    )

    tagList(
      h3(tool_copy[[1]]),
      div(class = "gtx-help", tool_copy[[2]]),
      controls
    )
  })

  output$med_inputs <- renderUI({
    req(analysis_data())
    vars <- names(analysis_data())
    tagList(
      selectInput("med_exposure", "Exposure", choices = c("", vars)),
      selectInput("med_mediator", "Mediator", choices = c("", vars)),
      selectInput("med_outcome", "Outcome", choices = c("", vars)),
      checkboxGroupInput("med_covariates", "Covariates", choices = vars),
      gtx_select_buttons("med_covariates")
    )
  })

  observeEvent(input$run_desc, {
    data <- require_analysis_data()
    req(data, input$desc_exposures)
    rv$desc <- tryCatch(
      gtx_call(
        "descriptive_table",
        data = data,
        exposures = input$desc_exposures,
        by = if (gtx_has_text(input$desc_by)) input$desc_by else NULL,
        percent = input$desc_percent,
        show_overall = input$desc_overall,
        show_missing = input$desc_missing,
        show_dichotomous = input$desc_dich,
        statistic = gtx_parse_named_stats(input$desc_statistic),
        format = "flextable"
      ),
      error = function(e) {
        showNotification(conditionMessage(e), type = "error")
        NULL
      }
    )
    if (!is.null(rv$desc)) {
      rv$last_message <- "Run univariable regression, survival analysis, or export the descriptive table."
      showNotification("Descriptive table ready. Run univariable models when you are ready.", type = "message")
    }
  })

  output$desc_table <- gtx_render_table(function() rv$desc)
  gtx_table_downloads(output, "desc", reactive(rv$desc))
  output$desc_code <- renderText({
    req(input$desc_exposures)
    paste(analysis_code_prefix(), gtx_code_assign(
      "desc_result",
      "descriptive_table",
      list(
        data = "analysis_data",
        exposures = gtx_vec_code(input$desc_exposures),
        by = gtx_null_code(input$desc_by),
        percent = shQuote(input$desc_percent),
        show_overall = shQuote(input$desc_overall),
        show_missing = shQuote(input$desc_missing),
        show_dichotomous = shQuote(input$desc_dich),
        statistic = gtx_statistic_code(input$desc_statistic),
        format = shQuote("flextable")
      )
    ), sep = "\n\n")
  })

  run_regression <- function(kind) {
    data <- require_analysis_data()
    req(data, input$reg_outcome, input$reg_exposures)
    gtx_validate_roles(
      outcome = input$reg_outcome,
      exposures = input$reg_exposures,
      adjust_for = if (kind == "multi") input$reg_adjust %||% character(0) else character(0),
      context = if (kind == "uni") "univariable regression" else "multivariable regression"
    )
    fun <- if (kind == "uni") "uni_reg" else "multi_reg"
    gtx_call(
      fun,
      data = regression_data(),
      outcome = input$reg_outcome,
      exposures = input$reg_exposures,
      adjust_for = if (kind == "multi" && length(input$reg_adjust)) input$reg_adjust else NULL,
      approach = input$reg_approach,
      interaction = if (gtx_has_text(input$reg_interaction)) input$reg_interaction else NULL,
      show_ref = input$reg_show_ref,
      model_stats = input$reg_model_stats,
      format = "flextable"
    )
  }

  observeEvent(input$run_uni, {
    rv$uni <- tryCatch(run_regression("uni"), error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL })
    if (!is.null(rv$uni)) {
      rv$last_message <- "Choose final variables, then run the multivariable model or visualise crude estimates."
      showNotification("Univariable table ready. Choose final variables for multivariable analysis.", type = "message")
    }
  })

  observeEvent(input$run_multi, {
    rv$multi <- tryCatch(run_regression("multi"), error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL })
    if (!is.null(rv$multi)) {
      rv$last_message <- "Merge tables, visualise estimates, check diagnostics, or export outputs."
      showNotification("Multivariable table ready. Merge tables or visualise results.", type = "message")
    }
  })

  observeEvent(input$run_strat, {
    data <- require_analysis_data()
    req(data, input$reg_outcome, input$reg_exposures, gtx_has_text(input$reg_stratifier))
    gtx_validate_roles(
      outcome = input$reg_outcome,
      exposures = input$reg_exposures,
      adjust_for = if (isTRUE(input$reg_strata_multi)) input$reg_adjust %||% character(0) else character(0),
      stratifier = input$reg_stratifier,
      context = "stratified regression"
    )
    fun <- if (isTRUE(input$reg_strata_multi)) "stratified_multi_reg" else "stratified_uni_reg"
    rv$strat <- tryCatch(
      gtx_call(
        fun,
        data = regression_data(),
        outcome = input$reg_outcome,
        exposures = input$reg_exposures,
        adjust_for = if (isTRUE(input$reg_strata_multi) && length(input$reg_adjust)) input$reg_adjust else NULL,
        stratifier = input$reg_stratifier,
        approach = input$reg_approach,
        show_ref = input$reg_show_ref,
        format = "flextable"
      ),
      error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL }
    )
    if (!is.null(rv$strat)) {
      rv$last_message <- "Review strata-specific estimates, then export the table or use forest_reg()."
      showNotification("Stratified table ready. Review strata-specific estimates.", type = "message")
    }
  })

  output$uni_table <- gtx_render_table(function() rv$uni)
  output$multi_table <- gtx_render_table(function() rv$multi)
  output$strat_table <- gtx_render_table(function() rv$strat)
  gtx_table_downloads(output, "uni", reactive(rv$uni))
  gtx_table_downloads(output, "multi", reactive(rv$multi))
  gtx_table_downloads(output, "strat", reactive(rv$strat))

  output$reg_stats <- gtx_render_data(function() {
    stats <- lapply(
      list(rv$uni, rv$multi, rv$cox_exposure, rv$cox_multi, rv$survreg_exposure, rv$survreg_multi),
      function(x) if (is.null(x)) NULL else x$model_stats
    )
    stats <- Filter(Negate(is.null), stats)
    req(length(stats))
    dplyr::bind_rows(stats)
  })

  output$uni_code <- renderText({
    req(input$reg_outcome, input$reg_exposures)
    paste(regression_code_prefix(), gtx_code_assign(
      "uni_result",
      "uni_reg",
      list(
        data = "analysis_data",
        outcome = gtx_null_code(input$reg_outcome),
        exposures = gtx_vec_code(input$reg_exposures),
        approach = shQuote(input$reg_approach),
        interaction = if (gtx_has_text(input$reg_interaction)) shQuote(input$reg_interaction) else NULL,
        show_ref = gtx_bool_code(input$reg_show_ref),
        model_stats = gtx_bool_code(input$reg_model_stats),
        format = shQuote("flextable")
      )
    ), sep = "\n\n")
  })
  output$multi_code <- renderText({
    req(input$reg_outcome, input$reg_exposures)
    paste(regression_code_prefix(), gtx_code_assign(
      "multi_result",
      "multi_reg",
      list(
        data = "analysis_data",
        outcome = gtx_null_code(input$reg_outcome),
        exposures = gtx_vec_code(input$reg_exposures),
        adjust_for = gtx_nonempty_vec_code(input$reg_adjust %||% character(0)),
        approach = shQuote(input$reg_approach),
        interaction = if (gtx_has_text(input$reg_interaction)) shQuote(input$reg_interaction) else NULL,
        show_ref = gtx_bool_code(input$reg_show_ref),
        model_stats = gtx_bool_code(input$reg_model_stats),
        format = shQuote("flextable")
      )
    ), sep = "\n\n")
  })
  output$strat_code <- renderText({
    req(input$reg_outcome, input$reg_exposures)
    fun <- if (isTRUE(input$reg_strata_multi)) "stratified_multi_reg" else "stratified_uni_reg"
    paste(regression_code_prefix(), gtx_code_assign(
      "stratified_result",
      fun,
      list(
        data = "analysis_data",
        outcome = gtx_null_code(input$reg_outcome),
        exposures = gtx_vec_code(input$reg_exposures),
        adjust_for = if (isTRUE(input$reg_strata_multi)) gtx_nonempty_vec_code(input$reg_adjust %||% character(0)) else NULL,
        stratifier = gtx_null_code(input$reg_stratifier),
        approach = shQuote(input$reg_approach),
        show_ref = gtx_bool_code(input$reg_show_ref),
        format = shQuote("flextable")
      )
    ), sep = "\n\n")
  })

  parse_limits <- function(x) {
    if (!gtx_has_text(x)) return(NULL)
    vals <- suppressWarnings(as.numeric(strsplit(x, ",")[[1]]))
    if (length(vals) == 2 && all(!is.na(vals))) vals else NULL
  }

  observeEvent(input$run_cox, {
    data <- require_analysis_data()
    req(data, input$surv_time, input$surv_event, input$surv_exposures)
    gtx_validate_roles(
      time = input$surv_time,
      event = input$surv_event,
      exposures = input$surv_exposures,
      adjust_for = input$surv_adjust %||% character(0),
      stratifier = if (gtx_has_text(input$surv_stratifier)) input$surv_stratifier else NULL,
      context = "Cox regression"
    )
    result <- tryCatch(
      gtx_call(
        "cox_reg",
        data = data,
        time = input$surv_time,
        event = input$surv_event,
        exposures = input$surv_exposures,
        adjust_for = input$surv_adjust,
        multivariable = input$surv_multivariable,
        multivariate = input$surv_multivariable,
        stratifier = if (gtx_has_text(input$surv_stratifier)) input$surv_stratifier else NULL,
        show_ref = input$surv_show_ref,
        model_stats = input$reg_model_stats,
        format = "flextable"
      ),
      error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL }
    )
    if (!is.null(result)) {
      rv$cox <- result
      if (isTRUE(input$surv_multivariable)) {
        rv$cox_multi <- result
        updateTabsetPanel(session, "cox_result_tab", selected = "Multivariable model")
      } else {
        rv$cox_exposure <- result
        updateTabsetPanel(session, "cox_result_tab", selected = "Exposure models")
      }
      rv$last_message <- "Draw KM curves, compare candidate models, or create a forest plot."
      updateTabsetPanel(session, "surv_output_tab", selected = "Model Result")
      showNotification("Cox regression table ready. Draw KM curves or visualise estimates.", type = "message")
    }
  })

  observeEvent(input$run_survreg, {
    data <- require_analysis_data()
    req(data, input$surv_time, input$surv_event, input$surv_exposures)
    gtx_validate_roles(
      time = input$surv_time,
      event = input$surv_event,
      exposures = input$surv_exposures,
      adjust_for = input$surv_adjust %||% character(0),
      stratifier = if (gtx_has_text(input$surv_stratifier)) input$surv_stratifier else NULL,
      context = "parametric survival regression"
    )
    result <- tryCatch(
      gtx_call(
        "surv_reg",
        data = data,
        time = input$surv_time,
        event = input$surv_event,
        exposures = input$surv_exposures,
        adjust_for = input$surv_adjust,
        multivariable = input$surv_multivariable,
        multivariate = input$surv_multivariable,
        stratifier = if (gtx_has_text(input$surv_stratifier)) input$surv_stratifier else NULL,
        distribution = input$surv_dist,
        show_ref = input$surv_show_ref,
        model_stats = input$reg_model_stats,
        format = "flextable"
      ),
      error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL }
    )
    if (!is.null(result)) {
      rv$survreg <- result
      if (isTRUE(input$surv_multivariable)) {
        rv$survreg_multi <- result
        updateTabsetPanel(session, "survreg_result_tab", selected = "Multivariable model")
      } else {
        rv$survreg_exposure <- result
        updateTabsetPanel(session, "survreg_result_tab", selected = "Exposure models")
      }
      rv$last_message <- "Compare survival models, visualise estimates, or create a forest plot."
      updateTabsetPanel(session, "surv_output_tab", selected = "Model Result")
      showNotification("Parametric survival table ready. Compare or visualise results.", type = "message")
    }
  })

  observeEvent(input$run_km, {
    data <- require_analysis_data()
    req(data, input$surv_time, input$surv_event)
    rv$km <- tryCatch(
      gtregression::km_plot(
        data = data,
        time = input$surv_time,
        event = input$surv_event,
        by = if (gtx_has_text(input$km_by)) input$km_by else NULL,
        conf.int = input$km_conf,
        risk_table = input$km_risk,
        p_value = input$km_p,
        break_time_by = gtx_km_break_value(input$km_break),
        xlim = parse_limits(input$km_xlim),
        ylim = parse_limits(input$km_ylim),
        y_percent = input$km_y_percent,
        theme = input$km_theme,
        grid = input$km_grid
      ),
      error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL }
    )
    if (!is.null(rv$km)) {
      rv$last_message <- "Tune x/y limits for publication or export the plot."
      updateTabsetPanel(session, "surv_output_tab", selected = "KM Plot")
      showNotification("KM plot ready. Tune x/y limits or export.", type = "message")
    }
  })

  output$cox_exposure_table <- gtx_render_table(function() rv$cox_exposure)
  output$cox_multi_table <- gtx_render_table(function() rv$cox_multi)
  output$survreg_exposure_table <- gtx_render_table(function() rv$survreg_exposure)
  output$survreg_multi_table <- gtx_render_table(function() rv$survreg_multi)
  output$km_plot <- renderPlot({ req(rv$km); print(rv$km) })
  gtx_table_downloads(output, "cox_exposure", reactive(rv$cox_exposure))
  gtx_table_downloads(output, "cox_multi", reactive(rv$cox_multi))
  gtx_table_downloads(output, "survreg_exposure", reactive(rv$survreg_exposure))
  gtx_table_downloads(output, "survreg_multi", reactive(rv$survreg_multi))
  gtx_plot_downloads(output, "km", reactive(rv$km))

  output$logrank_table <- gtx_render_table(function() {
    req(analysis_data(), input$surv_time, input$surv_event, gtx_has_text(input$km_by))
    gtregression::logrank_test(analysis_data(), time = input$surv_time, event = input$surv_event, by = input$km_by, format = "flextable")
  })
  output$surv_summary_table <- gtx_render_table(function() {
    req(analysis_data(), input$surv_time, input$surv_event)
    gtregression::survival_summary(analysis_data(), time = input$surv_time, event = input$surv_event, by = if (gtx_has_text(input$km_by)) input$km_by else NULL, format = "flextable")
  })
  output$surv_code <- renderText({
    ylim_code <- gtx_limits_code(parse_limits(input$km_ylim))
    xlim_code <- gtx_limits_code(parse_limits(input$km_xlim))
    result_name <- if (identical(input$surv_mode, "survreg")) {
      if (isTRUE(input$surv_multivariable)) "survreg_multi_result" else "survreg_exposure_result"
    } else {
      if (isTRUE(input$surv_multivariable)) "cox_multi_result" else "cox_exposure_result"
    }
    model_code <- if (identical(input$surv_mode, "survreg")) {
      gtx_code_assign(
        result_name,
        "surv_reg",
        list(
          data = "analysis_data",
          time = gtx_null_code(input$surv_time),
          event = gtx_null_code(input$surv_event),
          exposures = gtx_vec_code(input$surv_exposures %||% character(0)),
          adjust_for = gtx_nonempty_vec_code(input$surv_adjust %||% character(0)),
          multivariable = gtx_bool_code(input$surv_multivariable),
          stratifier = gtx_null_code(input$surv_stratifier),
          distribution = shQuote(input$surv_dist %||% "weibull"),
          show_ref = gtx_bool_code(input$surv_show_ref),
          model_stats = gtx_bool_code(input$reg_model_stats),
          format = shQuote("flextable")
        )
      )
    } else {
      gtx_code_assign(
        result_name,
        "cox_reg",
        list(
          data = "analysis_data",
          time = gtx_null_code(input$surv_time),
          event = gtx_null_code(input$surv_event),
          exposures = gtx_vec_code(input$surv_exposures %||% character(0)),
          adjust_for = gtx_nonempty_vec_code(input$surv_adjust %||% character(0)),
          multivariable = gtx_bool_code(input$surv_multivariable),
          stratifier = gtx_null_code(input$surv_stratifier),
          show_ref = gtx_bool_code(input$surv_show_ref),
          model_stats = gtx_bool_code(input$reg_model_stats),
          format = shQuote("flextable")
        )
      )
    }
    paste0(
      analysis_code_prefix(),
      "\n\n",
      model_code,
      "\n\n",
      gtx_code_assign(
        "km_result",
        "km_plot",
        list(
          data = "analysis_data",
          time = gtx_null_code(input$surv_time),
          event = gtx_null_code(input$surv_event),
          by = gtx_null_code(input$km_by),
          conf.int = gtx_bool_code(input$km_conf),
          risk_table = gtx_bool_code(input$km_risk),
          p_value = gtx_bool_code(input$km_p),
          break_time_by = if (is.null(gtx_km_break_value(input$km_break))) NULL else as.character(gtx_km_break_value(input$km_break)),
          xlim = if (identical(xlim_code, "NULL")) NULL else xlim_code,
          ylim = if (identical(ylim_code, "NULL")) NULL else ylim_code,
          y_percent = gtx_bool_code(input$km_y_percent),
          theme = shQuote(input$km_theme),
          grid = gtx_bool_code(input$km_grid)
        )
      ),
      "\n",
      "save_plot(km_result, filename = \"km_plot.png\", width = 8, height = 6)"
    )
  })

  result_for <- function(source) {
    switch(source,
           uni = rv$uni, multi = rv$multi,
           cox_exposure = rv$cox_exposure, cox_multi = rv$cox_multi,
           survreg_exposure = rv$survreg_exposure, survreg_multi = rv$survreg_multi,
           cox = rv$cox, survreg = rv$survreg, strat = rv$strat)
  }

  is_stratified_result <- function(x) {
    if (is.null(x)) return(FALSE)
    isTRUE(x$stratified) || inherits(
      x,
      c(
        "stratified_uni_reg", "stratified_multi_reg",
        "stratified_cox_reg", "stratified_surv_reg"
      )
    )
  }

  pair_can_combine <- function(first, second) {
    !is.null(first) && !is.null(second) &&
      !is_stratified_result(first) && !is_stratified_result(second)
  }

  selected_result <- reactive({
    source <- input$plot_source
    if (is.null(source)) {
      choices <- single_plot_choices()
      source <- if (length(choices)) unname(choices[1]) else NULL
    }
    result_for(source)
  })

  available_merge_tables <- reactive({
    Filter(Negate(is.null), list(
      "Descriptive" = rv$desc,
      "Crude" = rv$uni,
      "Adjusted" = rv$multi,
      "Cox exposure models" = rv$cox_exposure,
      "Cox multivariable model" = rv$cox_multi,
      "Parametric exposure models" = rv$survreg_exposure,
      "Parametric multivariable model" = rv$survreg_multi
    ))
  })

  output$merge_table_choices <- renderUI({
    tbls <- available_merge_tables()
    if (!length(tbls)) {
      return(div(class = "gtx-note", "No completed tables yet. Run descriptive or regression analyses first."))
    }
    checkboxGroupInput(
      "merge_selection",
      "Available tables",
      choices = stats::setNames(names(tbls), names(tbls)),
      selected = names(tbls)
    )
  })

  output$modify_source_controls <- renderUI({
    tables <- available_merge_tables()
    if (!is.null(rv$merged)) tables[["Merged table"]] <- rv$merged
    if (!length(tables)) {
      return(div(class = "gtx-note", "No completed table yet. Run a descriptive or regression analysis first."))
    }
    selectInput("modify_source", "Completed table", choices = stats::setNames(names(tables), names(tables)))
  })

  modify_source_object <- reactive({
    tables <- available_merge_tables()
    if (!is.null(rv$merged)) tables[["Merged table"]] <- rv$merged
    tables[[input$modify_source %||% ""]]
  })

  single_plot_choices <- reactive({
    choices <- c()
    if (!is.null(rv$uni)) choices["Univariate (crude)"] <- "uni"
    if (!is.null(rv$multi)) choices["Multivariable (adjusted)"] <- "multi"
    if (!is.null(rv$cox_exposure)) {
      label <- if (is_stratified_result(rv$cox_exposure)) "Cox exposure models (stratified)" else "Cox exposure models"
      choices[label] <- "cox_exposure"
    }
    if (!is.null(rv$cox_multi)) {
      label <- if (is_stratified_result(rv$cox_multi)) "Cox multivariable model (stratified)" else "Cox multivariable model"
      choices[label] <- "cox_multi"
    }
    if (!is.null(rv$survreg_exposure)) {
      label <- if (is_stratified_result(rv$survreg_exposure)) "Parametric exposure models (stratified)" else "Parametric exposure models"
      choices[label] <- "survreg_exposure"
    }
    if (!is.null(rv$survreg_multi)) {
      label <- if (is_stratified_result(rv$survreg_multi)) "Parametric multivariable model (stratified)" else "Parametric multivariable model"
      choices[label] <- "survreg_multi"
    }
    if (!is.null(rv$strat)) choices["Regression result (stratified)"] <- "strat"
    choices
  })

  output$plot_content_controls <- renderUI({
    choices <- single_plot_choices()
    combined_ready <- pair_can_combine(rv$uni, rv$multi)
    cox_combined_ready <- pair_can_combine(rv$cox_exposure, rv$cox_multi)
    survreg_combined_ready <- pair_can_combine(rv$survreg_exposure, rv$survreg_multi)
    stratified_ready <- any(vapply(unname(choices), function(source) {
      is_stratified_result(result_for(source))
    }, logical(1)))
    if (!length(choices)) {
      return(div(class = "gtx-note", "No regression results yet. Run a regression table first."))
    }
    tagList(
      radioButtons(
        "plot_layout",
        "Plot content",
        choices = c(
          "Selected result" = "single",
          if (combined_ready) c("Regression crude + adjusted" = "combined"),
          if (cox_combined_ready) c("Cox exposure + multivariable" = "cox_combined"),
          if (survreg_combined_ready) c("Parametric exposure + multivariable" = "survreg_combined")
        ),
        selected = if (identical(input$surv_mode, "cox") && cox_combined_ready) {
          "cox_combined"
        } else if (identical(input$surv_mode, "survreg") && survreg_combined_ready) {
          "survreg_combined"
        } else if (combined_ready) {
          "combined"
        } else {
          "single"
        }
      ),
      conditionalPanel(
        "input.plot_layout == 'single'",
        selectInput("plot_source", "Result", choices = choices, selected = unname(choices[1]))
      ),
      if (stratified_ready) div(
        class = "gtx-note",
        "Stratified results use one source: plot_reg() creates strata-specific panels. Crude + adjusted combination is intentionally unavailable for stratified results."
      )
    )
  })

  output$forest_content_controls <- renderUI({
    choices <- single_plot_choices()
    combined_ready <- pair_can_combine(rv$uni, rv$multi)
    cox_combined_ready <- pair_can_combine(rv$cox_exposure, rv$cox_multi)
    survreg_combined_ready <- pair_can_combine(rv$survreg_exposure, rv$survreg_multi)
    stratified_ready <- any(vapply(unname(choices), function(source) {
      is_stratified_result(result_for(source))
    }, logical(1)))
    desc_ready <- !is.null(rv$desc)
    forest_choices <- c("Selected result" = "single")
    if (combined_ready) forest_choices["Regression crude + adjusted"] <- "combined"
    if (combined_ready && desc_ready) forest_choices["Descriptive + regression crude + adjusted"] <- "all"
    if (cox_combined_ready) forest_choices["Cox exposure + multivariable"] <- "cox_combined"
    if (cox_combined_ready && desc_ready) forest_choices["Descriptive + Cox exposure + multivariable"] <- "cox_all"
    if (survreg_combined_ready) forest_choices["Parametric exposure + multivariable"] <- "survreg_combined"
    if (survreg_combined_ready && desc_ready) forest_choices["Descriptive + parametric exposure + multivariable"] <- "survreg_all"
    if (!length(choices)) {
      return(div(class = "gtx-note", "No compatible results yet. Run descriptive and regression analyses first."))
    }
    tagList(
      radioButtons(
        "forest_content",
        "Forest content",
        choices = forest_choices,
        selected = if (identical(input$surv_mode, "cox") && cox_combined_ready && desc_ready) {
          "cox_all"
        } else if (identical(input$surv_mode, "cox") && cox_combined_ready) {
          "cox_combined"
        } else if (identical(input$surv_mode, "survreg") && survreg_combined_ready && desc_ready) {
          "survreg_all"
        } else if (identical(input$surv_mode, "survreg") && survreg_combined_ready) {
          "survreg_combined"
        } else if (combined_ready && desc_ready) {
          "all"
        } else if (combined_ready) {
          "combined"
        } else {
          "single"
        }
      ),
      conditionalPanel(
        "input.forest_content == 'single'",
        selectInput("forest_source", "Result", choices = choices, selected = unname(choices[1]))
      ),
      if (stratified_ready) div(
        class = "gtx-note",
        "Stratified results use one source: forest_reg() places strata side by side and keeps the characteristic labels once. Multiple stratified objects cannot be combined."
      )
    )
  })

  output$fit_source_controls <- renderUI({
    choices <- single_plot_choices()
    if (!length(choices)) {
      return(div(class = "gtx-note", "No fitted model yet. Run a regression analysis first."))
    }
    selectInput("fit_source", "Fitted result", choices = choices, selected = unname(choices[1]))
  })

  observeEvent(input$visual_tool, {
    tab <- switch(
      input$visual_tool,
      merge = "Merged Table",
      modify = "Modified Table",
      plot = "plot_reg",
      forest = "forest_reg",
      fit = "Model fit",
      "Merged Table"
    )
    updateTabsetPanel(session, "visual_output_tab", selected = tab)
  }, ignoreInit = TRUE)

  observeEvent(input$run_plot_reg, {
    if (input$plot_layout %in% c("combined", "cox_combined", "survreg_combined")) {
      pair <- switch(
        input$plot_layout,
        combined = list(rv$uni, rv$multi),
        cox_combined = list(rv$cox_exposure, rv$cox_multi),
        survreg_combined = list(rv$survreg_exposure, rv$survreg_multi)
      )
      if (any(vapply(pair, is.null, logical(1)))) {
        showNotification("Run both the exposure models and multivariable model before creating the combined plot.", type = "warning")
        return(NULL)
      }
      rv$plotreg <- tryCatch(
        gtregression::plot_reg_combine(pair[[1]], pair[[2]], log_x = input$plot_log_x),
        error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL }
      )
    } else {
      req(selected_result())
      rv$plotreg <- tryCatch(
        gtregression::plot_reg(selected_result(), log_x = input$plot_log_x),
        error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL }
      )
    }
    if (!is.null(rv$plotreg)) {
      rv$last_message <- "Copy the plot code or export with a wider preset."
      showNotification("Regression plot preview ready. Use the Code tab for reproducible export.", type = "message")
    }
  })

  observeEvent(input$run_merge, {
    tbls <- available_merge_tables()
    selected <- input$merge_selection %||% character(0)
    selected <- names(tbls)[names(tbls) %in% selected]
    if (length(selected) < 2L) {
      showNotification("Run at least two tables before merging. For example: descriptive, univariable, then multivariable.", type = "message")
      return(NULL)
    }
    tbls <- tbls[selected]
    rv$merged <- tryCatch(
      do.call(gtregression::merge_tables, c(unname(tbls), list(spanners = names(tbls)))),
      error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL }
    )
    if (!is.null(rv$merged)) {
      rv$last_message <- "Download DOCX/HTML/PDF or apply modify_table() in RStudio."
      showNotification("Merged table ready. Tip: use show_ref = TRUE in regression tables when you want reference rows displayed.", type = "message")
    }
  })

  parse_named_mapping <- function(text, argument) {
    lines <- trimws(unlist(strsplit(text %||% "", "\n", fixed = TRUE)))
    lines <- lines[nzchar(lines)]
    if (!length(lines)) return(NULL)
    pieces <- strsplit(lines, "=", fixed = TRUE)
    valid <- vapply(pieces, function(x) {
      length(x) == 2L && nzchar(trimws(x[[1L]])) && nzchar(trimws(x[[2L]]))
    }, logical(1))
    if (!all(valid)) stop(paste0(argument, " must use one `name = label` mapping per line."), call. = FALSE)
    keys <- trimws(vapply(pieces, `[[`, character(1), 1L))
    values <- trimws(vapply(pieces, `[[`, character(1), 2L))
    if (anyDuplicated(keys)) stop(paste0(argument, " cannot repeat a name."), call. = FALSE)
    stats::setNames(values, keys)
  }

  parse_level_mapping <- function(text) {
    lines <- trimws(unlist(strsplit(text %||% "", "\n", fixed = TRUE)))
    lines <- lines[nzchar(lines)]
    if (!length(lines)) return(NULL)
    parsed <- lapply(lines, function(line) {
      sides <- strsplit(line, "=", fixed = TRUE)[[1L]]
      if (length(sides) != 2L) return(NULL)
      left <- strsplit(trimws(sides[[1L]]), ":", fixed = TRUE)[[1L]]
      if (length(left) != 2L || !all(nzchar(trimws(left))) || !nzchar(trimws(sides[[2L]]))) return(NULL)
      list(variable = trimws(left[[1L]]), level = trimws(left[[2L]]), label = trimws(sides[[2L]]))
    })
    if (any(vapply(parsed, is.null, logical(1)))) {
      stop("Level labels must use `variable: current level = new level` on each line.", call. = FALSE)
    }
    variables <- vapply(parsed, `[[`, character(1), "variable")
    levels <- vapply(parsed, `[[`, character(1), "level")
    if (anyDuplicated(paste(variables, levels, sep = "\r"))) {
      stop("Level labels cannot repeat the same variable and level.", call. = FALSE)
    }
    result <- lapply(unique(variables), function(variable) {
      matches <- which(variables == variable)
      stats::setNames(vapply(parsed[matches], `[[`, character(1), "label"), levels[matches])
    })
    stats::setNames(result, unique(variables))
  }

  observeEvent(input$run_modify_table, {
    source <- modify_source_object()
    if (is.null(source)) {
      showNotification("Choose a completed table to modify.", type = "warning")
      return(NULL)
    }
    rv$modified <- tryCatch(
      gtregression::modify_table(
        source,
        variable_labels = parse_named_mapping(input$modify_variable_labels, "Variable labels"),
        level_labels = parse_level_mapping(input$modify_level_labels),
        header_labels = parse_named_mapping(input$modify_header_labels, "Header labels"),
        caption = if (gtx_has_text(input$modify_caption)) input$modify_caption else NULL,
        bold_labels = isTRUE(input$modify_bold_labels),
        bold_levels = isTRUE(input$modify_bold_levels),
        italic_labels = isTRUE(input$modify_italic_labels),
        italic_levels = isTRUE(input$modify_italic_levels),
        remove_N = isTRUE(input$modify_remove_n),
        remove_N_obs = isTRUE(input$modify_remove_n_obs),
        remove_abbreviations = isTRUE(input$modify_remove_abbreviations),
        remove_adjustment_note = isTRUE(input$modify_remove_adjustment_note),
        caveat = if (gtx_has_text(input$modify_caveat)) input$modify_caveat else NULL
      ),
      error = function(e) {
        showNotification(conditionMessage(e), type = "error")
        NULL
      }
    )
    if (!is.null(rv$modified)) {
      rv$last_message <- "Modified table ready. Download it or keep refining its wording."
      showNotification("Modified table ready.", type = "message")
    }
  })

  observeEvent(input$run_forest, {
    rv$forest <- tryCatch({
      forest_xlim <- gtx_parse_numeric_vector(input$forest_xlim, expected = 2, name = "forest x limits")
      forest_ticks <- gtx_parse_numeric_vector(input$forest_ticks, name = "forest tick marks")
      ci_width <- input$forest_ci_width %||% 20
      if (input$forest_content %in% c("combined", "all", "cox_combined", "cox_all", "survreg_combined", "survreg_all")) {
        pair <- switch(
          input$forest_content,
          combined =, all = list(rv$uni, rv$multi),
          cox_combined =, cox_all = list(rv$cox_exposure, rv$cox_multi),
          survreg_combined =, survreg_all = list(rv$survreg_exposure, rv$survreg_multi)
        )
        if (any(vapply(pair, is.null, logical(1)))) {
          stop("Run both the exposure models and multivariable model before creating the combined forest plot.", call. = FALSE)
        }
        desc <- if (input$forest_content %in% c("all", "cox_all", "survreg_all")) rv$desc else NULL
        fd <- gtregression::forest_df(pair[[1]], pair[[2]], desc = desc)
      } else {
        source <- result_for(input$forest_source)
        req(source)
        fd <- gtregression::forest_df(source)
      }
      gtregression::forest_reg(
        fd,
        side = input$forest_side,
        ci_col_width = ci_width,
        xlim = forest_xlim,
        ticks_at = forest_ticks
      )
    }, error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL })
    if (!is.null(rv$forest)) {
      rv$last_message <- "Export with Wide or Many rows preset if the browser preview is cramped."
      showNotification("Forest preview ready. For publication, export with a wider preset or use save_forest() code.", type = "message")
    }
  })

  observeEvent(input$run_fit_plot, {
    fit_result <- result_for(input$fit_source)
    req(fit_result)
    rv$fitplot <- tryCatch(
      gtregression::plot_model_fit(
        fit_result,
        model_name = if (gtx_has_text(input$fit_model_name)) input$fit_model_name else NULL,
        type = input$fit_type,
        bins = input$fit_bins %||% 10,
        base_size = input$fit_base_size
      ),
      error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL }
    )
    if (!is.null(rv$fitplot)) {
      rv$last_message <- "Review diagnostics and copy the code into your analysis script."
      showNotification("Model fit plot ready. Use this as a diagnostic aid, not a final model decision alone.", type = "message")
    }
  })

  output$plotreg_plot <- renderPlot({ req(rv$plotreg); print(rv$plotreg) })
  output$forest_plot <- renderPlot({ req(rv$forest); print(rv$forest) })
  output$fit_plot <- renderPlot({ req(rv$fitplot); print(rv$fitplot) })
  output$merged_table <- gtx_render_table(function() rv$merged)
  output$modified_table <- gtx_render_table(function() rv$modified)
  gtx_table_downloads(output, "merged", reactive(rv$merged))
  gtx_table_downloads(output, "modified", reactive(rv$modified))
  gtx_plot_downloads(output, "plotreg", reactive(rv$plotreg), size_get = reactive(input$export_preset %||% "standard"))
  gtx_plot_downloads(output, "forest", reactive(rv$forest), is_forest = TRUE, size_get = reactive(input$export_preset %||% "standard"))
  gtx_plot_downloads(output, "fitplot", reactive(rv$fitplot), size_get = reactive(input$export_preset %||% "standard"))
  output$visual_code <- renderText({
    if (identical(input$visual_tool, "modify")) {
      source_name <- switch(
        input$modify_source %||% "",
        "Descriptive" = "desc_result",
        "Crude" = "uni_result",
        "Adjusted" = "multi_result",
        "Cox exposure models" = "cox_exposure_result",
        "Cox multivariable model" = "cox_multi_result",
        "Parametric exposure models" = "survreg_exposure_result",
        "Parametric multivariable model" = "survreg_multi_result",
        "Merged table" = "merged_table",
        "table_result"
      )
      object_code <- function(x) paste(utils::capture.output(dput(x)), collapse = "")
      variable_labels <- tryCatch(parse_named_mapping(input$modify_variable_labels, "Variable labels"), error = function(e) NULL)
      level_labels <- tryCatch(parse_level_mapping(input$modify_level_labels), error = function(e) NULL)
      header_labels <- tryCatch(parse_named_mapping(input$modify_header_labels, "Header labels"), error = function(e) NULL)
      return(paste0(
        "modified_table <- modify_table(\n",
        "  ", source_name,
        if (!is.null(variable_labels)) paste0(",\n  variable_labels = ", object_code(variable_labels)) else "",
        if (!is.null(level_labels)) paste0(",\n  level_labels = ", object_code(level_labels)) else "",
        if (!is.null(header_labels)) paste0(",\n  header_labels = ", object_code(header_labels)) else "",
        if (gtx_has_text(input$modify_caption)) paste0(",\n  caption = ", object_code(input$modify_caption)) else "",
        ",\n  bold_labels = ", gtx_bool_code(input$modify_bold_labels),
        ",\n  bold_levels = ", gtx_bool_code(input$modify_bold_levels),
        ",\n  italic_labels = ", gtx_bool_code(input$modify_italic_labels),
        ",\n  italic_levels = ", gtx_bool_code(input$modify_italic_levels),
        ",\n  remove_N = ", gtx_bool_code(input$modify_remove_n),
        ",\n  remove_N_obs = ", gtx_bool_code(input$modify_remove_n_obs),
        ",\n  remove_abbreviations = ", gtx_bool_code(input$modify_remove_abbreviations),
        ",\n  remove_adjustment_note = ", gtx_bool_code(input$modify_remove_adjustment_note),
        if (gtx_has_text(input$modify_caveat)) paste0(",\n  caveat = ", object_code(input$modify_caveat)) else "",
        "\n)\n",
        "save_table(modified_table, filename = \"modified_table.docx\")"
      ))
    }
    if (identical(input$visual_tool, "fit")) {
      fit_result_name <- switch(
        input$fit_source %||% "uni",
        uni = "uni_result", multi = "multi_result",
        cox_exposure = "cox_exposure_result", cox_multi = "cox_multi_result",
        survreg_exposure = "survreg_exposure_result", survreg_multi = "survreg_multi_result",
        strat = "stratified_result", "model_result"
      )
      return(paste0(
        "fit_plot <- plot_model_fit(\n  ", fit_result_name,
        if (gtx_has_text(input$fit_model_name)) paste0(",\n  model_name = ", shQuote(input$fit_model_name)) else "",
        ",\n  type = ", shQuote(input$fit_type),
        ",\n  bins = ", input$fit_bins %||% 10,
        ",\n  base_size = ", input$fit_base_size,
        "\n)\n",
        "save_plot(fit_plot, filename = \"model_fit.png\", width = 10, height = 7)"
      ))
    }
    plot_result_name <- switch(
      input$plot_source %||% "uni",
      uni = "uni_result",
      multi = "multi_result",
      cox_exposure = "cox_exposure_result",
      cox_multi = "cox_multi_result",
      survreg_exposure = "survreg_exposure_result",
      survreg_multi = "survreg_multi_result",
      cox = "cox_result",
      survreg = "survreg_result",
      strat = "stratified_result"
    )
    forest_result_name <- switch(
      input$forest_source %||% "uni",
      uni = "uni_result",
      multi = "multi_result",
      cox_exposure = "cox_exposure_result",
      cox_multi = "cox_multi_result",
      survreg_exposure = "survreg_exposure_result",
      survreg_multi = "survreg_multi_result",
      strat = "stratified_result"
    )
    forest_xlim_code <- gtx_numeric_vector_code(input$forest_xlim, expected = 2)
    forest_ticks_code <- gtx_numeric_vector_code(input$forest_ticks)
    ci_width <- input$forest_ci_width %||% 20
    plot_pair <- switch(
      input$plot_layout %||% "single",
      combined = c("uni_result", "multi_result"),
      cox_combined = c("cox_exposure_result", "cox_multi_result"),
      survreg_combined = c("survreg_exposure_result", "survreg_multi_result"),
      NULL
    )
    plot_code <- if (!is.null(plot_pair)) {
      paste0(
        "combined_plot <- plot_reg_combine(", plot_pair[1], ", ", plot_pair[2], ", log_x = ",
        gtx_bool_code(input$plot_log_x), ")\n",
        "save_plot(combined_plot, filename = \"plot_reg_combined.png\", width = 10, height = 7)"
      )
    } else {
      paste0(
        "reg_plot <- plot_reg(", plot_result_name, ", log_x = ", gtx_bool_code(input$plot_log_x), ")\n",
        "save_plot(reg_plot, filename = \"plot_reg.png\", width = 9, height = 6)"
      )
    }
    forest_data_code <- switch(
      input$forest_content %||% "single",
      all = "forest_data <- forest_df(uni_result, multi_result, desc = desc_result)",
      combined = "forest_data <- forest_df(uni_result, multi_result)",
      cox_all = "forest_data <- forest_df(cox_exposure_result, cox_multi_result, desc = desc_result)",
      cox_combined = "forest_data <- forest_df(cox_exposure_result, cox_multi_result)",
      survreg_all = "forest_data <- forest_df(survreg_exposure_result, survreg_multi_result, desc = desc_result)",
      survreg_combined = "forest_data <- forest_df(survreg_exposure_result, survreg_multi_result)",
      paste0("forest_data <- forest_df(", forest_result_name, ")")
    )
    forest_args <- paste0(
      "forest_plot <- forest_reg(\n",
      "  forest_data,\n",
      "  side = \"", input$forest_side, "\",\n",
      "  ci_col_width = ", ci_width,
      if (!is.null(forest_xlim_code)) paste0(",\n  xlim = ", forest_xlim_code) else "",
      if (!is.null(forest_ticks_code)) paste0(",\n  ticks_at = ", forest_ticks_code) else "",
      "\n)"
    )
    paste0(
      "# Browser previews are quick checks. For final output, run this in RStudio.\n",
      "# First run the Code tabs that create every result object used below.\n\n",
      plot_code, "\n\n",
      "# Merge tables for manuscript output after descriptive/univariable/multivariable steps.\n",
      "merged_table <- merge_tables(\n",
      "  desc_result,\n",
      "  uni_result,\n",
      "  multi_result,\n",
      "  spanners = c(\"Descriptive\", \"Crude\", \"Adjusted\")\n",
      ")\n",
      "save_table(merged_table, filename = \"merged_table.docx\")\n\n",
      forest_data_code, "\n",
      forest_args, "\n",
      "save_forest(forest_plot, filename = \"forest_reg.pdf\", width = 12, height = 8)\n\n",
      "# If labels or confidence intervals are crowded, increase width/height.\n",
      "# For overlapping x-axis text, set forest_reg(xlim = c(...), ticks_at = c(...)).\n",
      "# For narrow forest panels, increase ci_col_width."
    )
  })

  complete_workflow_code <- reactive({
    sections <- character(0)

    if (gtx_has_text(rv$data_code)) {
      sections <- c(sections, paste("# 1. Load data", rv$data_code, sep = "\n"))
      sections <- c(sections, paste("# 2. Select and prepare analysis data", analysis_code_prefix(), sep = "\n"))
    }

    reference_lines <- gtx_reference_code(
      selected_references(),
      reference_candidates(),
      data_name = "analysis_data"
    )
    if (length(reference_lines)) {
      sections <- c(sections, paste(
        "# 3. Set categorical predictor reference categories",
        paste(reference_lines, collapse = "\n"),
        sep = "\n"
      ))
    }

    if (!is.null(rv$desc) && length(input$desc_exposures %||% character(0))) {
      desc_args <- list(
        data = "analysis_data",
        exposures = gtx_vec_code(input$desc_exposures),
        by = gtx_null_code(input$desc_by),
        percent = shQuote(input$desc_percent),
        show_overall = shQuote(input$desc_overall),
        show_missing = shQuote(input$desc_missing),
        show_dichotomous = shQuote(input$desc_dich),
        statistic = gtx_statistic_code(input$desc_statistic),
        format = shQuote("flextable")
      )
      sections <- c(sections, paste("# 3. Descriptive table", gtx_code_assign("desc_result", "descriptive_table", desc_args), sep = "\n"))
    }

    if (!is.null(rv$uni) && gtx_has_text(input$reg_outcome) && length(input$reg_exposures %||% character(0))) {
      sections <- c(sections, paste(
        "# 4. Univariable regression",
        gtx_code_assign(
          "uni_result",
          "uni_reg",
          list(
            data = "analysis_data",
            outcome = gtx_null_code(input$reg_outcome),
            exposures = gtx_vec_code(input$reg_exposures),
            approach = shQuote(input$reg_approach),
            interaction = if (gtx_has_text(input$reg_interaction)) shQuote(input$reg_interaction) else NULL,
            show_ref = gtx_bool_code(input$reg_show_ref),
            model_stats = gtx_bool_code(input$reg_model_stats %||% FALSE),
            format = shQuote("flextable")
          )
        ),
        sep = "\n"
      ))
    }

    if (!is.null(rv$multi) && gtx_has_text(input$reg_outcome) && length(input$reg_exposures %||% character(0))) {
      sections <- c(sections, paste(
        "# 5. Multivariable regression",
        gtx_code_assign(
          "multi_result",
          "multi_reg",
          list(
            data = "analysis_data",
            outcome = gtx_null_code(input$reg_outcome),
            exposures = gtx_vec_code(input$reg_exposures),
            adjust_for = gtx_nonempty_vec_code(input$reg_adjust %||% character(0)),
            approach = shQuote(input$reg_approach),
            interaction = if (gtx_has_text(input$reg_interaction)) shQuote(input$reg_interaction) else NULL,
            show_ref = gtx_bool_code(input$reg_show_ref),
            model_stats = gtx_bool_code(input$reg_model_stats %||% FALSE),
            format = shQuote("flextable")
          )
        ),
        sep = "\n"
      ))
    }

    if (!is.null(rv$strat) && gtx_has_text(input$reg_stratifier)) {
      fun <- if (isTRUE(input$reg_strata_multi)) "stratified_multi_reg" else "stratified_uni_reg"
      sections <- c(sections, paste(
        "# 5. Stratified regression",
        gtx_code_assign(
          "stratified_result",
          fun,
          list(
            data = "analysis_data",
            outcome = gtx_null_code(input$reg_outcome),
            exposures = gtx_vec_code(input$reg_exposures),
            adjust_for = if (isTRUE(input$reg_strata_multi)) gtx_nonempty_vec_code(input$reg_adjust %||% character(0)) else NULL,
            stratifier = gtx_null_code(input$reg_stratifier),
            approach = shQuote(input$reg_approach),
            show_ref = gtx_bool_code(input$reg_show_ref),
            format = shQuote("flextable")
          )
        ),
        sep = "\n"
      ))
    }

    if ((!is.null(rv$cox) || !is.null(rv$survreg) || !is.null(rv$km)) && gtx_has_text(input$surv_time) && gtx_has_text(input$surv_event)) {
      survival_function <- if (!is.null(rv$survreg)) "surv_reg" else "cox_reg"
      survival_result <- if (identical(survival_function, "surv_reg")) "survreg_result" else "cox_result"
      sections <- c(sections, paste(
        "# 6. Survival analysis",
        gtx_code_assign(
          survival_result,
          survival_function,
          list(
            data = "analysis_data",
            time = gtx_null_code(input$surv_time),
            event = gtx_null_code(input$surv_event),
            exposures = gtx_vec_code(input$surv_exposures %||% character(0)),
            adjust_for = gtx_nonempty_vec_code(input$surv_adjust %||% character(0)),
            multivariable = gtx_bool_code(input$surv_multivariable),
            stratifier = gtx_null_code(input$surv_stratifier),
            distribution = if (identical(survival_function, "surv_reg")) shQuote(input$surv_dist %||% "weibull") else NULL,
            show_ref = gtx_bool_code(input$surv_show_ref),
            model_stats = gtx_bool_code(input$reg_model_stats %||% TRUE),
            format = shQuote("flextable")
          )
        ),
        "",
        gtx_code_assign(
          "km_result",
          "km_plot",
          list(
            data = "analysis_data",
            time = gtx_null_code(input$surv_time),
            event = gtx_null_code(input$surv_event),
            by = gtx_null_code(input$km_by),
            y_percent = gtx_bool_code(input$km_y_percent),
            grid = gtx_bool_code(input$km_grid),
            p_value = gtx_bool_code(input$km_p)
          )
        ),
        sep = "\n"
      ))
    }

    if (!is.null(rv$merged) || !is.null(rv$plotreg) || !is.null(rv$forest) || !is.null(rv$fitplot)) {
      result_names <- c(
        uni = "uni_result", multi = "multi_result", strat = "stratified_result",
        cox_exposure = "cox_exposure_result", cox_multi = "cox_multi_result",
        survreg_exposure = "survreg_exposure_result", survreg_multi = "survreg_multi_result"
      )
      visual_lines <- character(0)
      if (!is.null(rv$merged)) {
        merge_labels <- input$merge_selection %||% character(0)
        merge_objects <- c(
          "Descriptive" = "desc_result", "Crude" = "uni_result", "Adjusted" = "multi_result",
          "Cox exposure models" = "cox_exposure_result", "Cox multivariable model" = "cox_multi_result",
          "Parametric exposure models" = "survreg_exposure_result", "Parametric multivariable model" = "survreg_multi_result"
        )[merge_labels]
        if (length(merge_objects) >= 2L) {
          visual_lines <- c(visual_lines, paste0(
            "merged_table <- merge_tables(\n  ", paste(unname(merge_objects), collapse = ",\n  "),
            ",\n  spanners = ", gtx_vec_code(merge_labels), "\n)",
            "save_table(merged_table, filename = \"merged_table.docx\")"
          ))
        }
      }
      if (!is.null(rv$plotreg)) {
        layout <- input$plot_layout %||% "single"
        pair <- switch(layout, combined = c("uni_result", "multi_result"), cox_combined = c("cox_exposure_result", "cox_multi_result"), survreg_combined = c("survreg_exposure_result", "survreg_multi_result"), NULL)
        if (is.null(pair)) {
          source <- result_names[[input$plot_source %||% "uni"]] %||% "uni_result"
          visual_lines <- c(visual_lines, paste0("reg_plot <- plot_reg(", source, ", log_x = ", gtx_bool_code(input$plot_log_x), ")", "\nsave_plot(reg_plot, filename = \"plot_reg.png\", width = 9, height = 6)"))
        } else {
          visual_lines <- c(visual_lines, paste0("reg_plot <- plot_reg_combine(", pair[[1]], ", ", pair[[2]], ", log_x = ", gtx_bool_code(input$plot_log_x), ")", "\nsave_plot(reg_plot, filename = \"plot_reg_combined.png\", width = 10, height = 7)"))
        }
      }
      if (!is.null(rv$forest)) {
        content <- input$forest_content %||% "single"
        forest_data <- switch(content,
          all = "forest_df(uni_result, multi_result, desc = desc_result)",
          combined = "forest_df(uni_result, multi_result)",
          cox_all = "forest_df(cox_exposure_result, cox_multi_result, desc = desc_result)",
          cox_combined = "forest_df(cox_exposure_result, cox_multi_result)",
          survreg_all = "forest_df(survreg_exposure_result, survreg_multi_result, desc = desc_result)",
          survreg_combined = "forest_df(survreg_exposure_result, survreg_multi_result)",
          paste0("forest_df(", result_names[[input$forest_source %||% "uni"]] %||% "uni_result", ")")
        )
        visual_lines <- c(visual_lines, paste0(
          "forest_data <- ", forest_data, "\n",
          "forest_plot <- forest_reg(forest_data, side = ", shQuote(input$forest_side),
          ", ci_col_width = ", input$forest_ci_width %||% 20,
          if (!is.null(gtx_numeric_vector_code(input$forest_xlim, 2))) paste0(", xlim = ", gtx_numeric_vector_code(input$forest_xlim, 2)) else "",
          if (!is.null(gtx_numeric_vector_code(input$forest_ticks))) paste0(", ticks_at = ", gtx_numeric_vector_code(input$forest_ticks)) else "", ")\n",
          "save_forest(forest_plot, filename = \"forest_reg.pdf\", width = 13, height = 9)"
        ))
      }
      if (length(visual_lines)) sections <- c(sections, paste("# 7. Visualise and export", paste(visual_lines, collapse = "\n\n"), sep = "\n"))
    }

    if (!is.null(rv$fitplot)) {
      fit_result_name <- switch(
        input$fit_source %||% "uni",
        uni = "uni_result", multi = "multi_result",
        cox_exposure = "cox_exposure_result", cox_multi = "cox_multi_result",
        survreg_exposure = "survreg_exposure_result", survreg_multi = "survreg_multi_result",
        strat = "stratified_result", "model_result"
      )
      sections <- c(sections, paste(
        "# 7a. Model-fit diagnostic",
        gtx_code_assign(
          "fit_plot",
          "plot_model_fit",
          list(
            model = fit_result_name,
            model_name = if (gtx_has_text(input$fit_model_name)) shQuote(input$fit_model_name) else NULL,
            type = shQuote(input$fit_type),
            bins = as.character(input$fit_bins %||% 10),
            base_size = as.character(input$fit_base_size)
          )
        ),
        "save_plot(fit_plot, filename = \"model_fit.png\", width = 10, height = 7)",
        sep = "\n"
      ))
    }

    if (!is.null(rv$modified)) {
      modified_source <- switch(
        input$modify_source %||% "",
        "Descriptive" = "desc_result",
        "Crude" = "uni_result",
        "Adjusted" = "multi_result",
        "Cox exposure models" = "cox_exposure_result",
        "Cox multivariable model" = "cox_multi_result",
        "Parametric exposure models" = "survreg_exposure_result",
        "Parametric multivariable model" = "survreg_multi_result",
        "Merged table" = "merged_table",
        "table_result"
      )
      object_code <- function(x) paste(utils::capture.output(dput(x)), collapse = "")
      variable_labels <- tryCatch(parse_named_mapping(input$modify_variable_labels, "Variable labels"), error = function(e) NULL)
      level_labels <- tryCatch(parse_level_mapping(input$modify_level_labels), error = function(e) NULL)
      header_labels <- tryCatch(parse_named_mapping(input$modify_header_labels, "Header labels"), error = function(e) NULL)
      sections <- c(sections, paste0(
        "# 7a. Modify table\n",
        "modified_table <- modify_table(\n",
        "  ", modified_source,
        if (!is.null(variable_labels)) paste0(",\n  variable_labels = ", object_code(variable_labels)) else "",
        if (!is.null(level_labels)) paste0(",\n  level_labels = ", object_code(level_labels)) else "",
        if (!is.null(header_labels)) paste0(",\n  header_labels = ", object_code(header_labels)) else "",
        if (gtx_has_text(input$modify_caption)) paste0(",\n  caption = ", object_code(input$modify_caption)) else "",
        ",\n  bold_labels = ", gtx_bool_code(input$modify_bold_labels),
        ",\n  bold_levels = ", gtx_bool_code(input$modify_bold_levels),
        ",\n  italic_labels = ", gtx_bool_code(input$modify_italic_labels),
        ",\n  italic_levels = ", gtx_bool_code(input$modify_italic_levels),
        ",\n  remove_N = ", gtx_bool_code(input$modify_remove_n),
        ",\n  remove_N_obs = ", gtx_bool_code(input$modify_remove_n_obs),
        ",\n  remove_abbreviations = ", gtx_bool_code(input$modify_remove_abbreviations),
        ",\n  remove_adjustment_note = ", gtx_bool_code(input$modify_remove_adjustment_note),
        if (gtx_has_text(input$modify_caveat)) paste0(",\n  caveat = ", object_code(input$modify_caveat)) else "",
        "\n)\n",
        "save_table(modified_table, filename = \"modified_table.docx\")"
      ))
    }

    if (!is.null(rv$advanced) && gtx_has_text(rv$advanced_code)) {
      sections <- c(sections, paste("# 8. Advanced analysis", rv$advanced_code, sep = "\n"))
    }

    if (!is.null(rv$mediation)) {
      sections <- c(sections, paste(
        "# 9. Causal mediation",
        gtx_code_assign(
          "mediation_result",
          "mediation_analysis",
          list(
            data = "analysis_data",
            exposure = gtx_null_code(input$med_exposure),
            mediator = gtx_null_code(input$med_mediator),
            outcome = gtx_null_code(input$med_outcome),
            covariates = gtx_nonempty_vec_code(input$med_covariates %||% character(0)),
            outcome_approach = shQuote(input$med_approach),
            sims = as.character(input$med_sims),
            seed = as.character(input$med_seed),
            format = shQuote("flextable")
          )
        ),
        "mediation_plot <- plot_mediation(mediation_result)",
        sep = "\n"
      ))
    }

    if (!length(sections)) {
      return("# Load data and run an analysis. The app will build a full reproducible workflow here.")
    }

    paste(
      "# Generated by the gtregression app",
      "library(gtregression)",
      paste(sections, collapse = "\n\n"),
      sep = "\n\n"
    )
  })

  output$full_workflow_code <- renderText(complete_workflow_code())

  output$download_session_script <- downloadHandler(
    filename = function() "gtregression-session.R",
    content = function(file) {
      writeLines(complete_workflow_code(), file, useBytes = TRUE)
    }
  )

  observeEvent(input$run_select_models, {
    data <- require_analysis_data()
    req(data, input$adv_exposures)
    survival_advanced <- input$adv_approach %in% c("cox", "survreg")
    if (survival_advanced) {
      req(input$adv_time, input$adv_event)
    } else {
      req(input$adv_outcome)
    }
    res <- run_safely("Model selection", gtx_capture(gtx_call(
      "select_models",
      data = data,
      outcome = if (survival_advanced) NULL else input$adv_outcome,
      time = if (survival_advanced) input$adv_time else NULL,
      event = if (survival_advanced) input$adv_event else NULL,
      exposures = input$adv_exposures,
      approach = input$adv_approach,
      distribution = if (identical(input$adv_approach, "survreg")) input$adv_distribution else NULL,
      direction = input$select_direction,
      format = "flextable"
    )), on_error = clear_advanced_error)
    if (is.null(res)) return(NULL)
    rv$advanced <- res$value
    rv$advanced_text <- res$text
    rv$advanced_title <- "Model-selection results"
    rv$advanced_guidance <- "The highlighted model has the preferred AIC within this candidate path. Retain variables using study design, clinical relevance, and diagnostics rather than automated selection alone."
    rv$advanced_code <- paste(analysis_code_prefix(), gtx_code_assign(
      "selected_models",
      "select_models",
      list(
        data = "analysis_data",
        outcome = if (input$adv_approach %in% c("cox", "survreg")) NULL else gtx_null_code(input$adv_outcome),
        time = if (input$adv_approach %in% c("cox", "survreg")) gtx_null_code(input$adv_time) else NULL,
        event = if (input$adv_approach %in% c("cox", "survreg")) gtx_null_code(input$adv_event) else NULL,
        exposures = gtx_vec_code(input$adv_exposures),
        approach = shQuote(input$adv_approach),
        distribution = if (identical(input$adv_approach, "survreg")) shQuote(input$adv_distribution) else NULL,
        direction = shQuote(input$select_direction),
        format = shQuote("flextable")
      )
    ), sep = "\n\n")
    updateTabsetPanel(session, "advanced_output_tab", selected = "Output")
  })

  observeEvent(input$advanced_tool, {
    tool <- input$advanced_tool %||% "selection"
    labels <- c(
      selection = "Model selection",
      comparison = "Model comparison",
      confounder = "Confounder assessment",
      interaction = "Interaction assessment",
      convergence = "Convergence diagnostics",
      collinearity = "Collinearity diagnostics"
    )
    rv$advanced <- NULL
    rv$advanced_text <- ""
    rv$advanced_code <- ""
    rv$advanced_title <- unname(labels[[tool]] %||% "Advanced analysis")
    rv$advanced_guidance <- "Complete the choices on the left and run this tool. Results from another advanced tool are cleared when you switch tools."
  }, ignoreInit = TRUE)

  observeEvent(input$compare_add_model, {
    rv$compare_count <- min(6L, rv$compare_count + 1L)
    if (rv$compare_count == 6L) {
      showNotification("A maximum of six candidate models keeps the comparison readable.", type = "message")
    }
  }, ignoreInit = TRUE)

  observeEvent(input$compare_remove_model, {
    rv$compare_count <- max(2L, rv$compare_count - 1L)
  }, ignoreInit = TRUE)

  observeEvent(input$run_compare, {
    data <- require_analysis_data()
    req(data)
    approach <- input$compare_approach %||% "logit"
    survival_comparison <- approach %in% c("cox", "survreg")
    if (survival_comparison) {
      req(input$compare_time, input$compare_event)
    } else {
      req(input$compare_outcome)
    }

    specifications <- lapply(seq_len(rv$compare_count), function(i) {
      exposures <- input[[paste0("compare_exposures_", i)]] %||% character(0)
      adjust <- input[[paste0("compare_adjust_", i)]] %||% character(0)
      int_a <- input[[paste0("compare_interaction_a_", i)]] %||% ""
      int_b <- input[[paste0("compare_interaction_b_", i)]] %||% ""
      list(
        name = trimws(input[[paste0("compare_name_", i)]] %||% paste("Model", i)),
        exposures = exposures,
        adjust = adjust,
        predictors = unique(c(exposures, adjust)),
        int_a = int_a,
        int_b = int_b,
        interaction = if (nzchar(int_a) && nzchar(int_b)) paste(int_a, int_b, sep = "*") else NULL
      )
    })

    model_names <- vapply(specifications, `[[`, character(1), "name")
    if (any(!nzchar(model_names))) {
      showNotification("Give every candidate model a name.", type = "warning")
      return(NULL)
    }
    if (anyDuplicated(model_names)) {
      showNotification("Candidate model names must be unique.", type = "warning")
      return(NULL)
    }
    if (any(vapply(specifications, function(x) !length(x$exposures), logical(1)))) {
      showNotification("Choose at least one reported exposure for every candidate model.", type = "warning")
      return(NULL)
    }
    overlap <- vapply(specifications, function(x) length(intersect(x$exposures, x$adjust)) > 0L, logical(1))
    if (any(overlap)) {
      showNotification("A variable cannot be both a reported exposure and an adjustment variable in the same model.", type = "warning")
      return(NULL)
    }
    incomplete_interaction <- vapply(specifications, function(x) xor(nzchar(x$int_a), nzchar(x$int_b)), logical(1))
    invalid_interaction <- vapply(specifications, function(x) {
      both <- nzchar(x$int_a) && nzchar(x$int_b)
      both && (identical(x$int_a, x$int_b) || !all(c(x$int_a, x$int_b) %in% x$predictors))
    }, logical(1))
    if (any(incomplete_interaction)) {
      showNotification("Choose both interaction terms, or leave both as None.", type = "warning")
      return(NULL)
    }
    if (any(invalid_interaction)) {
      showNotification("Interaction terms must be different variables already included in that candidate model.", type = "warning")
      return(NULL)
    }
    if (gtx_has_text(input$adv_primary_exposure) && any(!vapply(
      specifications,
      function(x) input$adv_primary_exposure %in% x$predictors,
      logical(1)
    ))) {
      showNotification("The primary exposure must be included in every candidate model.", type = "warning")
      return(NULL)
    }

    fit_one <- function(specification) {
      if (identical(approach, "cox")) {
        return(gtx_call(
          "cox_reg", data = data, time = input$compare_time, event = input$compare_event,
          exposures = specification$predictors, interaction = specification$interaction,
          multivariable = TRUE, format = "flextable", model_stats = TRUE
        ))
      }
      if (identical(approach, "survreg")) {
        return(gtx_call(
          "surv_reg", data = data, time = input$compare_time, event = input$compare_event,
          exposures = specification$predictors, interaction = specification$interaction,
          multivariable = TRUE, distribution = input$compare_distribution,
          format = "flextable", model_stats = TRUE
        ))
      }
      gtx_call(
        "multi_reg", data = data, outcome = input$compare_outcome,
        exposures = specification$predictors, interaction = specification$interaction,
        approach = approach, format = "flextable", model_stats = TRUE
      )
    }

    models <- tryCatch(
      withProgress(message = "Fitting candidate models", value = 0, {
        fitted <- lapply(seq_along(specifications), function(i) {
          incProgress(1 / length(specifications), detail = model_names[i])
          fit_one(specifications[[i]])
        })
        names(fitted) <- model_names
        fitted
      }),
      error = function(e) {
        showNotification(paste("A candidate model could not be fitted:", conditionMessage(e)), type = "error", duration = 10)
        NULL
      }
    )
    if (is.null(models)) return(NULL)

    compare_options <- list(model_names = model_names, format = "flextable")
    if (gtx_has_text(input$adv_primary_exposure)) compare_options$primary_exposure <- input$adv_primary_exposure
    res <- tryCatch(do.call(gtregression::compare_models, c(models, compare_options)), error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL })
    if (is.null(res)) return(NULL)
    rv$advanced <- res
    rv$advanced_text <- ""
    rv$advanced_title <- "Model-comparison results"
    rv$advanced_guidance <- "Read the comparison-status panel first. Likelihood-based comparisons are strongest for nested models fitted to the same observations; the primary estimate helps assess robustness across candidate models."
    object_names <- make.unique(vapply(seq_along(specifications), function(i) {
      gtx_object_name(model_names[i], paste0("candidate_model_", i))
    }, character(1)))
    model_code <- vapply(seq_along(specifications), function(i) {
      specification <- specifications[[i]]
      fun <- if (identical(approach, "cox")) "cox_reg" else if (identical(approach, "survreg")) "surv_reg" else "multi_reg"
      args <- list(
        data = "analysis_data",
        outcome = if (!survival_comparison) gtx_null_code(input$compare_outcome) else NULL,
        time = if (survival_comparison) gtx_null_code(input$compare_time) else NULL,
        event = if (survival_comparison) gtx_null_code(input$compare_event) else NULL,
        exposures = gtx_vec_code(specification$predictors),
        interaction = if (!is.null(specification$interaction)) shQuote(specification$interaction) else NULL,
        multivariable = if (survival_comparison) "TRUE" else NULL,
        distribution = if (identical(approach, "survreg")) shQuote(input$compare_distribution) else NULL,
        approach = if (!survival_comparison) shQuote(approach) else NULL,
        format = shQuote("flextable"),
        model_stats = "TRUE"
      )
      paste0(
        "# ", model_names[i], ": reported exposures = ", paste(specification$exposures, collapse = ", "),
        if (length(specification$adjust)) paste0("; adjusted for = ", paste(specification$adjust, collapse = ", ")) else "",
        "\n", gtx_code_assign(object_names[i], fun, args)
      )
    }, character(1))
    model_lines <- paste0("  ", object_names, collapse = ",\n")
    rv$advanced_code <- paste0(
      analysis_code_prefix(), "\n\n",
      paste(model_code, collapse = "\n\n"), "\n\n",
      "comparison_table <- compare_models(\n",
      model_lines, ",\n",
      "  model_names = ", gtx_vec_code(model_names), ",\n",
      if (gtx_has_text(input$adv_primary_exposure)) paste0("  primary_exposure = ", shQuote(input$adv_primary_exposure), ",\n") else "",
      "  format = \"flextable\"\n",
      ")"
    )
    updateTabsetPanel(session, "advanced_output_tab", selected = "Output")
  })

  observeEvent(input$run_confounder, {
    data <- require_analysis_data()
    exposures <- input$conf_exposures %||% character(0)
    req(data, length(exposures), input$conf_candidate)
    survival_advanced <- input$adv_approach %in% c("cox", "survreg")
    if (survival_advanced) {
      req(input$adv_time, input$adv_event)
    } else {
      req(input$adv_outcome)
    }
    res <- run_safely("Confounder assessment", gtx_capture(gtx_call(
      "identify_confounder",
      data = data,
      outcome = if (survival_advanced) NULL else input$adv_outcome,
      time = if (survival_advanced) input$adv_time else NULL,
      event = if (survival_advanced) input$adv_event else NULL,
      exposure = exposures,
      potential_confounder = input$conf_candidate,
      approach = input$adv_approach,
      distribution = if (identical(input$adv_approach, "survreg")) input$adv_distribution else NULL,
      format = "flextable"
    )), on_error = clear_advanced_error)
    if (is.null(res)) return(NULL)
    rv$advanced <- res$value
    rv$advanced_text <- res$text
    rv$advanced_title <- "Confounder assessment"
    rv$advanced_guidance <- "This is a viewing and decision-support result, not a publication-ready claim that a variable is a confounder. Base the final decision on a prespecified DAG, temporal order, and subject-matter knowledge."
    rv$advanced_code <- paste(analysis_code_prefix(), gtx_code_assign(
      "confounder_check",
      "identify_confounder",
      list(
        data = "analysis_data",
        outcome = if (input$adv_approach %in% c("cox", "survreg")) NULL else gtx_null_code(input$adv_outcome),
        time = if (input$adv_approach %in% c("cox", "survreg")) gtx_null_code(input$adv_time) else NULL,
        event = if (input$adv_approach %in% c("cox", "survreg")) gtx_null_code(input$adv_event) else NULL,
        exposure = gtx_vec_code(exposures),
        potential_confounder = gtx_null_code(input$conf_candidate),
        approach = shQuote(input$adv_approach),
        distribution = if (identical(input$adv_approach, "survreg")) shQuote(input$adv_distribution) else NULL,
        format = shQuote("flextable")
      )
    ), sep = "\n\n")
    updateTabsetPanel(session, "advanced_output_tab", selected = "Output")
  })

  observeEvent(input$run_interaction, {
    data <- require_analysis_data()
    req(data, input$conf_exposure, input$conf_candidate)
    survival_advanced <- input$adv_approach %in% c("cox", "survreg")
    if (survival_advanced) {
      req(input$adv_time, input$adv_event)
    } else {
      req(input$adv_outcome)
    }
    res <- run_safely("Interaction assessment", gtx_capture(gtx_call(
      "interaction_models",
      data = data,
      outcome = if (survival_advanced) NULL else input$adv_outcome,
      time = if (survival_advanced) input$adv_time else NULL,
      event = if (survival_advanced) input$adv_event else NULL,
      exposure = input$conf_exposure,
      effect_modifier = input$conf_candidate,
      covariates = input$adv_covariates,
      approach = input$adv_approach,
      distribution = if (identical(input$adv_approach, "survreg")) input$adv_distribution else NULL,
      format = "flextable"
    )), on_error = clear_advanced_error)
    if (is.null(res)) return(NULL)
    rv$advanced <- res$value
    rv$advanced_text <- res$text
    rv$advanced_title <- "Interaction assessment"
    rv$advanced_guidance <- "Use the interaction test together with stratum-specific estimates and confidence intervals. Statistical interaction depends on the chosen effect scale."
    rv$advanced_code <- paste(analysis_code_prefix(), gtx_code_assign(
      "interaction_check",
      "interaction_models",
      list(
        data = "analysis_data",
        outcome = if (input$adv_approach %in% c("cox", "survreg")) NULL else gtx_null_code(input$adv_outcome),
        time = if (input$adv_approach %in% c("cox", "survreg")) gtx_null_code(input$adv_time) else NULL,
        event = if (input$adv_approach %in% c("cox", "survreg")) gtx_null_code(input$adv_event) else NULL,
        exposure = gtx_null_code(input$conf_exposure),
        effect_modifier = gtx_null_code(input$conf_candidate),
        covariates = gtx_nonempty_vec_code(input$adv_covariates %||% character(0)),
        approach = shQuote(input$adv_approach),
        distribution = if (identical(input$adv_approach, "survreg")) shQuote(input$adv_distribution) else NULL,
        format = shQuote("flextable")
      )
    ), sep = "\n\n")
    updateTabsetPanel(session, "advanced_output_tab", selected = "Output")
  })

  observeEvent(input$run_convergence, {
    data <- require_analysis_data()
    req(data, input$adv_outcome, input$adv_exposures)
    res <- run_safely(
      "Convergence diagnostics",
      gtx_capture(gtx_call("check_convergence", data = data, outcome = input$adv_outcome, exposures = input$adv_exposures, approach = input$adv_approach, multivariate = TRUE, multivariable = TRUE, format = "flextable")),
      on_error = clear_advanced_error
    )
    if (is.null(res)) return(NULL)
    rv$advanced <- res$value
    rv$advanced_text <- res$text
    rv$advanced_title <- "Convergence diagnostics"
    rv$advanced_guidance <- "Models flagged as non-convergent or unstable should not be interpreted until the data, specification, or modelling approach has been reviewed."
    rv$advanced_code <- paste(analysis_code_prefix(), gtx_code_assign(
      "convergence_check",
      "check_convergence",
      list(
        data = "analysis_data",
        outcome = gtx_null_code(input$adv_outcome),
        exposures = gtx_vec_code(input$adv_exposures),
        approach = shQuote(input$adv_approach),
        multivariable = "TRUE",
        format = shQuote("flextable")
      )
    ), sep = "\n\n")
    updateTabsetPanel(session, "advanced_output_tab", selected = "Output")
  })

  observeEvent(input$run_collinearity, {
    available <- Filter(Negate(is.null), list(
      uni = rv$uni,
      multi = rv$multi,
      cox_exposure = rv$cox_exposure,
      cox_multi = rv$cox_multi,
      survreg_exposure = rv$survreg_exposure,
      survreg_multi = rv$survreg_multi
    ))
    req(input$adv_collinearity_model, available[[input$adv_collinearity_model]])
    res <- run_safely(
      "Collinearity diagnostics",
      gtx_capture(gtx_call("check_collinearity", model = available[[input$adv_collinearity_model]], format = "flextable")),
      on_error = clear_advanced_error
    )
    if (is.null(res)) return(NULL)
    rv$advanced <- res$value
    rv$advanced_text <- res$text
    rv$advanced_title <- "Collinearity diagnostics"
    rv$advanced_guidance <- "Large collinearity measures indicate that predictors may be difficult to distinguish in the fitted model. Review coding, clinical overlap, and model purpose before removing variables."
    rv$advanced_code <- paste0(
      "collinearity_check <- check_collinearity(\n",
      "  model = ", input$adv_collinearity_model, "_result,\n",
      "  format = \"flextable\"\n",
      ")"
    )
    updateTabsetPanel(session, "advanced_output_tab", selected = "Output")
  })

  output$advanced_table <- gtx_render_table(function() rv$advanced)
  output$advanced_text <- renderText(rv$advanced_text)
  output$advanced_code <- renderText(rv$advanced_code)
  output$advanced_result_heading <- renderUI(tagList(h3(rv$advanced_title)))
  output$advanced_guidance <- renderUI(div(class = "gtx-help", strong("How to use this result: "), rv$advanced_guidance))
  output$advanced_downloads <- renderUI({
    if (is.null(rv$advanced)) return(NULL)
    tagListDownload("advanced", table = TRUE)
  })
  gtx_table_downloads(output, "advanced", reactive(rv$advanced))

  observeEvent(input$run_mediation, {
    data <- require_analysis_data()
    req(data, input$med_exposure, input$med_mediator, input$med_outcome)
    rv$mediation <- tryCatch(
      gtregression::mediation_analysis(
        data = data,
        exposure = input$med_exposure,
        mediator = input$med_mediator,
        outcome = input$med_outcome,
        covariates = input$med_covariates,
        outcome_approach = input$med_approach,
        sims = input$med_sims,
        seed = input$med_seed,
        format = "flextable"
      ),
      error = function(e) { showNotification(conditionMessage(e), type = "error"); NULL }
    )
  })

  output$med_table <- gtx_render_table(function() rv$mediation)
  output$med_plot <- renderPlot({
    req(rv$mediation)
    print(gtregression::plot_mediation(rv$mediation))
  })
  gtx_table_downloads(output, "med", reactive(rv$mediation))
  gtx_plot_downloads(output, "medplot", reactive({
    req(rv$mediation)
    gtregression::plot_mediation(rv$mediation)
  }))
  output$med_code <- renderText({
    paste0(
      analysis_code_prefix(),
      "\n\n",
      gtx_code_assign(
        "mediation_result",
        "mediation_analysis",
        list(
          data = "analysis_data",
          exposure = gtx_null_code(input$med_exposure),
          mediator = gtx_null_code(input$med_mediator),
          outcome = gtx_null_code(input$med_outcome),
          covariates = gtx_nonempty_vec_code(input$med_covariates %||% character(0)),
          outcome_approach = shQuote(input$med_approach),
          sims = as.character(input$med_sims),
          seed = as.character(input$med_seed),
          format = shQuote("flextable")
        )
      ),
      "\n\n",
      "mediation_plot <- plot_mediation(mediation_result)\n",
      "save_table(mediation_result$table, filename = \"mediation_table.docx\")\n",
      "save_plot(mediation_plot, filename = \"mediation_path.png\", width = 8, height = 6)"
    )
  })
}

shinyApp(ui, server)

test_that("gtregression app is bundled with the package", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")

  expect_true(nzchar(app_file))
  expect_true(file.exists(app_file))
  expect_true("gtregression_app" %in% getNamespaceExports("gtregression"))
})

test_that("app organises visualisation tools and available combinations", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, '"visual_tool"', fixed = TRUE)
  expect_match(app_code, '"merge_table_choices"', fixed = TRUE)
  expect_match(app_code, '"plot_content_controls"', fixed = TRUE)
  expect_match(app_code, '"forest_content_controls"', fixed = TRUE)
  expect_match(app_code, '"Descriptive + regression crude + adjusted"', fixed = TRUE)
  expect_match(app_code, "gtregression::plot_reg_combine(pair[[1]], pair[[2]]", fixed = TRUE)
  expect_match(app_code, "gtregression::forest_df(pair[[1]], pair[[2]], desc = desc)", fixed = TRUE)
  expect_match(app_code, "Run both the exposure models and multivariable model", fixed = TRUE)
})

test_that("app treats stratified plots and forests as single sources", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, "is_stratified_result <- function", fixed = TRUE)
  expect_match(app_code, "pair_can_combine <- function", fixed = TRUE)
  expect_match(app_code, "Cox exposure models (stratified)", fixed = TRUE)
  expect_match(app_code, "plot_reg() creates strata-specific panels", fixed = TRUE)
  expect_match(app_code, "forest_reg() places strata side by side", fixed = TRUE)
  expect_match(app_code, "combined_ready <- pair_can_combine(rv$uni, rv$multi)", fixed = TRUE)
  expect_match(app_code, "cox_combined_ready <- pair_can_combine(rv$cox_exposure, rv$cox_multi)", fixed = TRUE)
  expect_match(app_code, "survreg_combined_ready <- pair_can_combine(rv$survreg_exposure, rv$survreg_multi)", fixed = TRUE)
})

test_that("app exposes broad imports and essential forest controls", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, '".rds"', fixed = TRUE)
  expect_match(app_code, '".dta"', fixed = TRUE)
  expect_match(app_code, '".sav"', fixed = TRUE)
  expect_match(app_code, '".sas7bdat"', fixed = TRUE)
  expect_match(app_code, 'readRDS(path)', fixed = TRUE)
  expect_match(app_code, 'requireNamespace("haven"', fixed = TRUE)
  expect_match(app_code, 'numericInput("forest_ci_width"', fixed = TRUE)
  expect_match(app_code, 'textInput("forest_xlim"', fixed = TRUE)
  expect_match(app_code, 'textInput("forest_ticks"', fixed = TRUE)
  expect_match(app_code, 'downloadButton(paste0(id, "_html"), "HTML")', fixed = TRUE)
})

test_that("survival workspace separates Cox and parametric workflows", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, '"surv_mode"', fixed = TRUE)
  expect_match(app_code, '"Cox regression" = "cox"', fixed = TRUE)
  expect_match(app_code, '"Parametric survival" = "survreg"', fixed = TRUE)
  expect_match(app_code, "input.surv_mode == 'cox'", fixed = TRUE)
  expect_match(app_code, "input.surv_mode == 'survreg'", fixed = TRUE)
  expect_match(app_code, 'id = "surv_output_tab"', fixed = TRUE)
  expect_match(app_code, '"Model Result",', fixed = TRUE)
  expect_match(app_code, 'actionButton("run_cox", "Run Cox regression"', fixed = TRUE)
  expect_match(app_code, 'actionButton("run_survreg", "Run parametric survival regression"', fixed = TRUE)
  expect_match(app_code, 'id = "cox_result_tab"', fixed = TRUE)
  expect_match(app_code, 'id = "survreg_result_tab"', fixed = TRUE)
  expect_match(app_code, 'gtx_table_output("cox_exposure_table")', fixed = TRUE)
  expect_match(app_code, 'gtx_table_output("cox_multi_table")', fixed = TRUE)
  expect_match(app_code, 'gtx_table_output("survreg_exposure_table")', fixed = TRUE)
  expect_match(app_code, 'gtx_table_output("survreg_multi_table")', fixed = TRUE)
  expect_match(app_code, 'uiOutput("km_group_input")', fixed = TRUE)
  expect_match(app_code, 'identical(input$surv_mode, "survreg")', fixed = TRUE)
})

test_that("survival results are retained by model role", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, "rv$cox_exposure <- result", fixed = TRUE)
  expect_match(app_code, "rv$cox_multi <- result", fixed = TRUE)
  expect_match(app_code, "rv$survreg_exposure <- result", fixed = TRUE)
  expect_match(app_code, "rv$survreg_multi <- result", fixed = TRUE)
  expect_match(app_code, '"Cox exposure models" = rv$cox_exposure', fixed = TRUE)
  expect_match(app_code, '"Cox multivariable model" = rv$cox_multi', fixed = TRUE)
  expect_match(app_code, '"Parametric exposure models" = rv$survreg_exposure', fixed = TRUE)
  expect_match(app_code, '"Parametric multivariable model" = rv$survreg_multi', fixed = TRUE)
  expect_match(app_code, '"cox_multi_result" else "cox_exposure_result"', fixed = TRUE)
  expect_match(app_code, '"survreg_multi_result" else "survreg_exposure_result"', fixed = TRUE)
  expect_match(app_code, '"Cox exposure + multivariable" = "cox_combined"', fixed = TRUE)
  expect_match(app_code, '"Parametric exposure + multivariable" = "survreg_combined"', fixed = TRUE)
  expect_match(app_code, "cox_combined = list(rv$cox_exposure, rv$cox_multi)", fixed = TRUE)
  expect_match(app_code, "survreg_combined = list(rv$survreg_exposure, rv$survreg_multi)", fixed = TRUE)
  expect_match(app_code, "forest_data <- forest_df(cox_exposure_result, cox_multi_result", fixed = TRUE)
  expect_match(app_code, "forest_data <- forest_df(survreg_exposure_result, survreg_multi_result", fixed = TRUE)
})

test_that("advanced tools use a guided contextual workflow", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, '"advanced_tool"', fixed = TRUE)
  expect_match(app_code, '"Select models" = "selection"', fixed = TRUE)
  expect_match(app_code, '"Compare models" = "comparison"', fixed = TRUE)
  expect_match(app_code, '"Confounder" = "confounder"', fixed = TRUE)
  expect_match(app_code, '"Interaction" = "interaction"', fixed = TRUE)
  expect_match(app_code, '"Convergence" = "convergence"', fixed = TRUE)
  expect_match(app_code, '"Collinearity" = "collinearity"', fixed = TRUE)
  expect_match(app_code, '"compare_approach"', fixed = TRUE)
  expect_match(app_code, '"compare_outcome"', fixed = TRUE)
  expect_match(app_code, '"compare_exposures_"', fixed = TRUE)
  expect_match(app_code, '"compare_adjust_"', fixed = TRUE)
  expect_match(app_code, '"compare_interaction_a_"', fixed = TRUE)
  expect_match(app_code, '"compare_name_"', fixed = TRUE)
  expect_match(app_code, '"compare_add_model"', fixed = TRUE)
  expect_match(app_code, '"adv_collinearity_model"', fixed = TRUE)
  expect_match(app_code, 'uiOutput("advanced_downloads")', fixed = TRUE)
  expect_match(app_code, 'current_approach <- input$adv_approach %||% "logit"', fixed = TRUE)
  expect_match(app_code, 'isTRUE(current_approach %in% c("cox", "survreg"))', fixed = TRUE)
  expect_match(app_code, 'observeEvent(input$advanced_tool', fixed = TRUE)
  expect_match(app_code, 'Fit and compare models', fixed = TRUE)
  expect_match(app_code, 'fit_one <- function(specification)', fixed = TRUE)
  expect_match(app_code, '"multi_reg", data = data', fixed = TRUE)
  expect_match(app_code, '"cox_reg", data = data', fixed = TRUE)
  expect_match(app_code, '"surv_reg", data = data', fixed = TRUE)
  expect_match(app_code, 'gtx_table_downloads(output, "advanced"', fixed = TRUE)
  expect_match(app_code, 'updateTabsetPanel(session, "advanced_output_tab"', fixed = TRUE)
})

test_that("app errors are contained without pausing the Shiny session", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, "run_safely <- function", fixed = TRUE)
  expect_match(app_code, "shiny.silent.error = function(e) NULL", fixed = TRUE)
  expect_match(app_code, 'type = "error"', fixed = TRUE)
  expect_match(app_code, "clear_advanced_error", fixed = TRUE)
  expect_match(app_code, 'run_safely("Model selection"', fixed = TRUE)
  expect_match(app_code, 'run_safely("Confounder assessment"', fixed = TRUE)
  expect_match(app_code, 'run_safely("Interaction assessment"', fixed = TRUE)
  expect_match(app_code, '"Convergence diagnostics",\n      gtx_capture', fixed = TRUE)
  expect_match(app_code, '"Collinearity diagnostics",\n      gtx_capture', fixed = TRUE)
  expect_match(app_code, "This result could not be displayed", fixed = TRUE)
})

test_that("the complete app workflow can be downloaded as an R script", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, 'tabPanel(\n    "Session Code"', fixed = TRUE)
  expect_match(app_code, 'downloadButton("download_session_script", "Download complete R script"', fixed = TRUE)
  expect_match(app_code, 'complete_workflow_code <- reactive({', fixed = TRUE)
  expect_match(app_code, '"# Generated by the gtregression app"', fixed = TRUE)
  expect_match(app_code, '"library(gtregression)"', fixed = TRUE)
  expect_match(app_code, 'output$full_workflow_code <- renderText(complete_workflow_code())', fixed = TRUE)
  expect_match(app_code, 'output$download_session_script <- downloadHandler(', fixed = TRUE)
  expect_match(app_code, 'filename = function() "gtregression-session.R"', fixed = TRUE)
  expect_match(app_code, 'writeLines(complete_workflow_code(), file, useBytes = TRUE)', fixed = TRUE)
})

test_that("app keeps table modification and display-label controls discoverable", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, '"Modify table" = "modify"', fixed = TRUE)
  expect_match(app_code, 'actionButton("run_modify_table", "Apply table changes"', fixed = TRUE)
  expect_match(app_code, 'gtregression::modify_table(', fixed = TRUE)
  expect_match(app_code, 'remove_adjustment_note = isTRUE(input$modify_remove_adjustment_note)', fixed = TRUE)
  expect_match(app_code, 'tabPanel(\n            "Modified Table"', fixed = TRUE)
  expect_match(app_code, "Level-label format:", fixed = TRUE)
})

test_that("app defaults regression plots to a linear axis and supports advanced screening roles", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, 'checkboxInput("plot_log_x", "Use a log x-axis", FALSE)', fixed = TRUE)
  expect_match(app_code, 'checkboxInput("reg_model_stats", "Store model statistics", value = TRUE)', fixed = TRUE)
  expect_match(app_code, 'model_stats = input$reg_model_stats', fixed = TRUE)
  expect_match(app_code, 'checkboxGroupInput("conf_exposures", "Exposures of interest"', fixed = TRUE)
  expect_match(app_code, 'covariate_label = "Covariates included in both interaction models"', fixed = TRUE)
  expect_match(app_code, 'exposure = exposures,', fixed = TRUE)
})

test_that("app exposes the complete plot_model_fit API and records it in session code", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, '"Auto (recommended)" = "auto"', fixed = TRUE)
  expect_match(app_code, '"Normal Q-Q" = "qq"', fixed = TRUE)
  expect_match(app_code, '"Scale-location" = "scale_location"', fixed = TRUE)
  expect_match(app_code, '"Observed vs predicted" = "observed_predicted"', fixed = TRUE)
  expect_match(app_code, 'numericInput("fit_base_size", "Base font size"', fixed = TRUE)
  expect_match(app_code, 'base_size = input$fit_base_size', fixed = TRUE)
  expect_match(app_code, '"# 7a. Model-fit diagnostic"', fixed = TRUE)
})

test_that("app keeps data preview compact and descriptive statistics visible", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_code <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_match(app_code, "utils::head(rv$data, 10)", fixed = TRUE)
  expect_match(app_code, "First 10 rows only.", fixed = TRUE)
  expect_match(app_code, 'textInput("desc_statistic", "Numeric statistic override"', fixed = TRUE)
  expect_match(app_code, "If x-axis labels overlap, enter fewer ticks", fixed = TRUE)
})

## ----export-setup, message=FALSE, warning=FALSE-------------------------------
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
  )

birthwt_exposures <- c("age", "lwt", "race", "smoke", "ht", "ui")

attr(birthwt_data$age, "label") <- "Maternal age"
attr(birthwt_data$lwt, "label") <- "Maternal weight"
attr(birthwt_data$race, "label") <- "Maternal race"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"
attr(birthwt_data$ui, "label") <- "Uterine irritability"

birthwt_desc <- descriptive_table(
  birthwt_data,
  exposures = birthwt_exposures,
  by = low
)
birthwt_uni <- uni_reg(
  birthwt_data,
  outcome = low,
  exposures = birthwt_exposures,
  approach = logit
)
birthwt_multi <- multi_reg(
  birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui"),
  adjust_for = c("age", "lwt", "race"),
  approach = logit
)

## ----modify-table, message=FALSE, warning=FALSE-------------------------------
birthwt_custom <- modify_table(
  birthwt_multi,
  variable_labels = c(
    smoke = "Smoked during pregnancy",
    ht = "History of hypertension",
    ui = "Uterine irritability"
  ),
  level_labels = list(
    smoke = c(Yes = "Smoker"),
    ht = c(Yes = "Hypertensive")
  ),
  header_labels = c(estimate = "Adjusted OR", p.value = "P"),
  caption = "Adjusted regression for low birth weight",
  caveat = "Adjusted for maternal age, maternal weight, and maternal race."
)

birthwt_custom

## ----merge-tables, message=FALSE, warning=FALSE-------------------------------
birthwt_merged <- merge_tables(
  birthwt_desc,
  birthwt_uni,
  birthwt_multi,
  spanners = c("Descriptive", "Crude", "Adjusted")
)

birthwt_merged$table

## ----merged-polish, message=FALSE, warning=FALSE------------------------------
birthwt_merged_paper <- modify_table(
  birthwt_merged,
  variable_labels = c(
    age = "Maternal age",
    lwt = "Maternal weight",
    race = "Maternal race",
    smoke = "Smoking during pregnancy",
    ht = "Hypertension",
    ui = "Uterine irritability"
  ),
  caption = "Clinical profile and regression estimates for low birth weight",
  caveat = "Adjusted estimates are adjusted for maternal age, maternal weight, and maternal race."
)

birthwt_merged_paper$table

## ----save-examples, eval=FALSE------------------------------------------------
# table_path <- save_table(
#   birthwt_merged_paper,
#   filename = "birthwt-table",
#   format = html
# )
# 
# birthwt_plot <- plot_reg(
#   birthwt_multi,
#   title = "Adjusted Regression for Low Birth Weight"
# )
# 
# plot_path <- save_plot(
#   birthwt_plot,
#   filename = "birthwt-forest",
#   format = png
# )

## ----word-example, eval=FALSE-------------------------------------------------
# birthwt_multi_ft <- multi_reg(
#   birthwt_data,
#   outcome = low,
#   exposures = c("smoke", "ht", "ui"),
#   adjust_for = c("age", "lwt", "race"),
#   approach = logit,
#   format = flextable
# )
# 
# docx_path <- save_docx(
#   tables = list(birthwt_multi_ft),
#   filename = "birthwt-report",
#   titles = "Adjusted Regression",
#   table_width = 6.5
# )

## ----reg-setup, message=FALSE, warning=FALSE----------------------------------
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes")),
    ftv_cat = factor(case_when(
      ftv == 0 ~ "None",
      ftv == 1 ~ "One",
      ftv >= 2 ~ "Two or more"
    ), levels = c("None", "One", "Two or more"))
  )

birthwt_exposures <- c(
  "age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat", "ftv_cat"
)

attr(birthwt_data$age, "label") <- "Maternal age"
attr(birthwt_data$lwt, "label") <- "Maternal weight"
attr(birthwt_data$race, "label") <- "Maternal race"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"
attr(birthwt_data$ui, "label") <- "Uterine irritability"
attr(birthwt_data$ptl_cat, "label") <- "Previous preterm labour"
attr(birthwt_data$ftv_cat, "label") <- "First trimester visits"

## ----uni-logit, message=FALSE, warning=FALSE----------------------------------
birthwt_uni <- uni_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = birthwt_exposures,
  approach = "logit",
  theme = clinical
)

birthwt_uni

## ----multi-full, message=FALSE, warning=FALSE---------------------------------
birthwt_full <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("age", "lwt", "race", "smoke", "ht", "ui"),
  approach = logit,
  theme = clinical
)

birthwt_full

## ----multi-adjusted, message=FALSE, warning=FALSE-----------------------------
birthwt_multi <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat", "ftv_cat"),
  adjust_for = c("age", "lwt", "race"),
  approach = logit,
  theme = striped
)

birthwt_multi

## ----model-stats, message=FALSE, warning=FALSE--------------------------------
birthwt_uni_stats <- uni_reg(
  data = birthwt_data,
  outcome = low,
  exposures = birthwt_exposures,
  approach = logit,
  model_stats = TRUE
)

birthwt_uni_stats$model_stats

## ----adjusted-model-stats, message=FALSE, warning=FALSE-----------------------
birthwt_multi_stats <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat"),
  adjust_for = c("age", "lwt", "race"),
  approach = logit,
  model_stats = TRUE
)

birthwt_multi_stats$model_stats

## ----reg-risk-ratio, message=FALSE, warning=FALSE-----------------------------
uni_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat"),
  approach = logbinomial
)

## ----reg-firth, message=FALSE, warning=FALSE----------------------------------
data("data_endometrial", package = "gtregression")

endometrial_data <- data_endometrial |>
  mutate(
    HG = factor(HG, levels = c(0, 1),
                labels = c("Low grade", "High grade")),
    NV = factor(NV, levels = c(0, 1), labels = c("Absent", "Present"))
  )

multi_reg(
  data = endometrial_data,
  outcome = HG,
  exposures = c(NV, PI, EH),
  approach = firth
)

## ----cox-setup, message=FALSE, warning=FALSE----------------------------------
data("data_lungcancer", package = "gtregression")

lung_data <- data_lungcancer |>
  mutate(
    trt = factor(trt, levels = c(1, 2),
                 labels = c("Standard treatment", "Test treatment")),
    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
  )

attr(lung_data$trt, "label") <- "Treatment group"
attr(lung_data$celltype, "label") <- "Cancer cell type"
attr(lung_data$karno, "label") <- "Karnofsky performance score"
attr(lung_data$age, "label") <- "Age"
attr(lung_data$prior, "label") <- "Prior therapy"

## ----cox-crude, message=FALSE, warning=FALSE----------------------------------
lung_hr <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c("trt", "celltype", "karno", "age"),
  theme = clinical
)

lung_hr

## ----cox-adjusted, message=FALSE, warning=FALSE-------------------------------
lung_adj_hr <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno),
  model_stats = TRUE,
  theme = striped
)

lung_adj_hr
lung_adj_hr$model_stats

## ----cox-interaction, message=FALSE, warning=FALSE----------------------------
cox_interaction <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno),
  interaction = trt*prior
)

cox_interaction

## ----surv-crude, message=FALSE, warning=FALSE---------------------------------
lung_time_ratio <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c("trt", "celltype", "karno", "age"),
  distribution = weibull,
  theme = clinical
)

lung_time_ratio

## ----surv-adjusted, message=FALSE, warning=FALSE------------------------------
lung_adj_time_ratio <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno),
  distribution = lognormal,
  model_stats = TRUE,
  theme = striped
)

lung_adj_time_ratio
lung_adj_time_ratio$model_stats

## ----surv-interaction, message=FALSE, warning=FALSE---------------------------
surv_interaction <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno),
  interaction = trt*prior,
  distribution = weibull
)

surv_interaction

## ----reg-linear, message=FALSE, warning=FALSE---------------------------------
birthwt_linear <- multi_reg(
  data = birthwt_data,
  outcome = bwt,
  exposures = c("age", "lwt", "race", "smoke", "ht", "ui"),
  approach = linear
)

birthwt_linear

## ----diag-setup, message=FALSE, warning=FALSE---------------------------------
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes"))
  )

exposures <- c("age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat")

## ----diag-convergence, message=FALSE, warning=FALSE---------------------------
check_convergence(
  data = birthwt_data,
  exposures = exposures,
  outcome = low,
  approach = logit,
  multivariate = TRUE
)

## ----diag-convergence-logbin, message=FALSE, warning=FALSE--------------------
check_convergence(
  data = birthwt_data,
  exposures = c("smoke", "ht", "ui", "ptl_cat"),
  outcome = low,
  approach = logbinomial,
  multivariate = TRUE
)

## ----diag-collinearity, message=FALSE, warning=FALSE--------------------------
birthwt_multi <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = exposures,
  approach = logit
)

check_collinearity(birthwt_multi, format = gt)

## ----diag-collinearity-adjusted, message=FALSE, warning=FALSE-----------------
birthwt_adjusted <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat"),
  adjust_for = c("age", "lwt", "race"),
  approach = logit
)

check_collinearity(birthwt_adjusted, format = tibble)

## ----diag-fit-logistic, message=FALSE, warning=FALSE--------------------------
plot_model_fit(
  birthwt_multi,
  type = calibration,
  bins = 6
)

## ----diag-fit-uni, message=FALSE, warning=FALSE-------------------------------
birthwt_uni <- uni_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("age", "lwt", "smoke"),
  approach = logit
)

plot_model_fit(
  birthwt_uni,
  model_name = smoke,
  type = residual
)

## ----diag-fit-linear, message=FALSE, warning=FALSE----------------------------
fit_lm <- lm(bwt ~ age + lwt, data = birthwt_data)
plot_model_fit(fit_lm)

## ----diag-ph-setup, message=FALSE, warning=FALSE------------------------------
data("data_lungcancer", package = "gtregression")

lung_data <- data_lungcancer |>
  mutate(
    trt = factor(trt, levels = c(1, 2),
                 labels = c("Standard treatment", "Test treatment")),
    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes")),
    celltype = factor(
      celltype,
      levels = c("squamous", "smallcell", "adeno", "large"),
      labels = c("Squamous", "Small cell", "Adenocarcinoma", "Large cell")
    )
  )

cox_fit <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno)
)

## ----diag-ph-table, message=FALSE, warning=FALSE------------------------------
check_ph(cox_fit, format = gt)

## ----diag-ph-tibble, message=FALSE, warning=FALSE-----------------------------
check_ph(cox_fit, transform = rank, format = tibble)

## ----diag-compare-logit, message=FALSE, warning=FALSE-------------------------
logit_m0 <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = smoke,
  approach = logit
)

logit_m1 <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c(smoke, age, lwt),
  approach = logit
)

logit_m2 <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c(smoke, age, lwt, race, ht, ui),
  approach = logit
)

logit_model_comparison <- compare_models(
  logit_m0,
  logit_m1,
  logit_m2,
  model_names = c(
    "Smoking only",
    "Add age and weight",
    "Full clinical model"
  ),
  primary_exposure = smoke,
  format = gt
)

logit_model_comparison$table

## ----diag-compare-cox, message=FALSE, warning=FALSE---------------------------
cox_m0 <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt
)

cox_m1 <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno)
)

cox_m2 <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, age, karno, celltype, prior),
  multivariable = TRUE
)

cox_model_comparison <- compare_models(
  list(
    "Treatment only" = cox_m0,
    "Add age and performance" = cox_m1,
    "Full clinical model" = cox_m2
  ),
  primary_exposure = trt,
  format = gt
)

cox_model_comparison$table

## ----diag-select, message=FALSE, warning=FALSE--------------------------------
selected <- select_models(
  data = birthwt_data,
  outcome = low,
  exposures = exposures,
  approach = logit,
  direction = forward
)

selected

## ----diag-select-other-directions, message=FALSE, warning=FALSE---------------
select_models(
  data = birthwt_data,
  outcome = low,
  exposures = exposures,
  approach = logit,
  direction = backward,
  format = tibble
)$results_table

select_models(
  data = birthwt_data,
  outcome = low,
  exposures = exposures,
  approach = logit,
  direction = both,
  format = tibble
)$results_table

## ----plot-setup, message=FALSE, warning=FALSE---------------------------------
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")
data("data_lungcancer", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes")),
    ftv_cat = factor(case_when(
      ftv == 0 ~ "None",
      ftv == 1 ~ "One",
      ftv >= 2 ~ "Two or more"
    ), levels = c("None", "One", "Two or more"))
  )

birthwt_exposures <- c(
  "age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat", "ftv_cat"
)

attr(birthwt_data$age, "label") <- "Maternal age"
attr(birthwt_data$lwt, "label") <- "Maternal weight"
attr(birthwt_data$race, "label") <- "Maternal race"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"
attr(birthwt_data$ui, "label") <- "Uterine irritability"
attr(birthwt_data$ptl_cat, "label") <- "Previous preterm labour"
attr(birthwt_data$ftv_cat, "label") <- "First trimester visits"

birthwt_desc <- descriptive_table(
  birthwt_data,
  exposures = birthwt_exposures,
  by = "low",
  show_overall = "last"
)
birthwt_uni <- uni_reg(
  birthwt_data,
  outcome = "low",
  exposures = birthwt_exposures,
  approach = "logit"
)
birthwt_multi <- multi_reg(
  birthwt_data,
  outcome = "low",
  exposures = c("smoke", "ht", "ui", "ptl_cat", "ftv_cat"),
  adjust_for = c("age", "lwt", "race"),
  approach = "logit"
)

lung_data <- data_lungcancer |>
  mutate(
    trt = factor(trt, levels = c(1, 2),
                 labels = c("Standard treatment", "Test treatment")),
    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
  )

lung_surv <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = "trt",
  adjust_for = c("age", "karno"),
  distribution = weibull
)

## ----km-plot, message=FALSE, warning=FALSE------------------------------------
km_plot(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  break_time_by = 200,
  ylim = c(50, 100),
  title = "Kaplan-Meier Survival by Treatment"
)

## ----km-panel-plot, fig.width=7, fig.height=5, message=FALSE, warning=FALSE----
km_treatment <- km_plot(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  risk_table = FALSE,
  title = "A. Treatment",
  title_size = 10,
  title_face = plain,
  legend_position = bottom,
  base_size = 10
)

km_prior <- km_plot(
  data = lung_data,
  time = time,
  event = status,
  by = prior,
  risk_table = FALSE,
  title = "B. Prior therapy",
  title_size = 10,
  title_face = plain,
  legend_position = bottom,
  base_size = 10
)

patchwork::wrap_plots(km_treatment, km_prior, ncol = 2) +
  patchwork::plot_layout(guides = "collect") &
  ggplot2::theme(legend.position = "bottom")

## ----km-risk-table, message=FALSE, warning=FALSE------------------------------
km_risk_table(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  times = c(0, 90, 180, 365)
)

## ----rmst-table, message=FALSE, warning=FALSE---------------------------------
rmst_table(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  tau = 365
)

## ----survival-summary, message=FALSE, warning=FALSE---------------------------
survival_summary(
  data = lung_data,
  time = time,
  event = status,
  by = trt
)

## ----survival-quantiles, message=FALSE, warning=FALSE-------------------------
survival_quantiles(
  data = lung_data,
  time = time,
  event = status,
  by = trt
)

## ----survival-prob, message=FALSE, warning=FALSE------------------------------
survival_prob(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  times = c(90, 180, 365)
)

## ----logrank-test, message=FALSE, warning=FALSE-------------------------------
logrank_test(
  data = lung_data,
  time = time,
  event = status,
  by = trt
)

## ----surv-model-compare, message=FALSE, warning=FALSE-------------------------
surv_model_compare(
  data = lung_data,
  time = time,
  event = status,
  exposures = c("trt", "celltype"),
  adjust_for = c("age", "karno")
)

## ----plot-surv-fit, fig.width=7, fig.height=5, message=FALSE, warning=FALSE----
plot_surv_fit(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  distributions = c(weibull, lognormal),
  break_time_by = 200
)

## ----plot-surv-fit-adjusted, fig.width=7, fig.height=5, message=FALSE, warning=FALSE----
plot_surv_fit(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  adjust_for = c(age, karno),
  distributions = loglogistic,
  xlim = c(0, 800)
)

## ----surv-predict, fig.width=7, fig.height=5, message=FALSE, warning=FALSE----
surv_predict(
  model = lung_surv$models$trt,
  newdata = data.frame(
    trt = factor("Test treatment", levels = levels(lung_data$trt)),
    age = 60,
    karno = 70
  ),
  times = c(90, 180, 365)
)

## ----one-plot, fig.width=7, fig.height=5, message=FALSE, warning=FALSE--------
plot_reg(
  birthwt_uni,
  title = "Crude Associations With Low Birth Weight"
)

## ----plot-style, fig.width=7, fig.height=5, message=FALSE, warning=FALSE------
plot_reg(
  birthwt_uni,
  show_ref = FALSE,
  point_size = 3.5,
  point_stroke = 0.7,
  ci_linewidth = 0.75,
  base_size = 13,
  title = "Crude Associations With Low Birth Weight"
)

## ----adjusted-plot, fig.width=10, fig.height=5, message=FALSE, warning=FALSE----
plot_reg(
  birthwt_multi,
  show_ref = FALSE,
  log_x = TRUE,
  title = "Adjusted Associations With Low Birth Weight"
)

## ----compact-binary-plot, fig.width=10, fig.height=7, message=FALSE, warning=FALSE----
plot_reg(
  birthwt_uni,
  show_ref = FALSE,
  title = "Crude Associations With Reference Rows Hidden"
)

## ----log-axis-plot,fig.width=10, fig.height=7, message=FALSE, warning=FALSE----
plot_reg(
  birthwt_uni,
  log_x = TRUE,
  title = "Crude Associations on a Log Scale"
)

## ----custom-axis-plot, fig.width=10, fig.height=7, message=FALSE, warning=FALSE----
plot_reg(
  birthwt_uni,
  show_ref = FALSE,
  log_x = TRUE,
  xlim = c(0.25, 12),
  breaks = c(0.5, 1, 2, 4, 8),
  title = "Crude Associations With Custom Axis"
)

## ----combined-plot, fig.width=12, fig.height=7, message=FALSE, warning=FALSE----
plot_reg_combine(
  tbl_uni = birthwt_uni,
  tbl_multi = birthwt_multi,
  show_ref = FALSE,
  log_x = TRUE,
  xlim_uni = c(0.25, 12),
  breaks_uni = c(0.5, 1, 2, 4, 8),
  xlim_multi = c(0.25, 16),
  breaks_multi = c(0.5, 1, 2, 4, 8),
  title_uni = "Crude Effects",
  title_multi = "Adjusted Effects"
)

## ----forest-table, fig.width=13, fig.height=10, message=FALSE, warning=FALSE----
forest_data <- forest_df(
  uni = birthwt_uni,
  multi = birthwt_multi,
  desc = birthwt_desc
)
forest_reg(forest_data)

## ----forest-axis-control, fig.width=13, fig.height=10,message=FALSE, warning=FALSE----
forest_reg(
  forest_data,
  xlim = list(c(0.25, 8), c(0.8, 25)),
  ticks_at = list(
    c(0.5, 1, 2, 4, 8),
    c(1, 2, 4, 8, 16)
  ),
  quiet = TRUE
)

## ----forest-ci-width, fig.width=13, fig.height=10, message=FALSE, warning=FALSE----
forest_reg(
  forest_data,
  ci_col_width = c(18, 22),
  xlim = list(c(0.25, 8), c(0.8, 25)),
  ticks_at = list(
    c(0.5, 1, 2, 4, 8),
    c(1, 2, 4, 8, 16)
  )
)

## ----forest-one-call, fig.width=13, fig.height=10,message=FALSE, warning=FALSE----
forest_reg(
  uni = birthwt_uni,
  multi = birthwt_multi,
  desc = birthwt_desc,
  side = "left"
)

## ----ci-setup, message=FALSE, warning=FALSE-----------------------------------
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
  )

attr(birthwt_data$race, "label") <- "Maternal race"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"

## ----confounder, message=FALSE, warning=FALSE---------------------------------
confounder_check <- identify_confounder(
  data = birthwt_data,
  outcome = low,
  exposure = smoke,
  potential_confounder = c("race", "ht"),
  approach = logit,
  method = both,
  format = gt
)

confounder_check$table

## ----confounder-summary, message=FALSE, warning=FALSE-------------------------
confounder_check$summary

## ----confounder-mh, message=FALSE, warning=FALSE------------------------------
identify_confounder(
  data = birthwt_data,
  outcome = low,
  exposure = smoke,
  potential_confounder = race,
  approach = logit,
  method = mh,
  format = flextable
)$table

## ----interaction, message=FALSE, warning=FALSE--------------------------------
interaction_check <- interaction_models(
  data = birthwt_data,
  outcome = low,
  exposure = smoke,
  effect_modifier = race,
  covariates = c("age", "lwt"),
  approach = logit,
  test = LRT,
  format = gt
)

interaction_check$table

## ----survival-confounder, message=FALSE, warning=FALSE------------------------
lung_data <- data_lungcancer |>
  dplyr::mutate(
    trt = factor(trt, levels = c(1, 2),
                 labels = c("Standard treatment", "Test treatment")),
    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
  )

survival_confounder <- identify_confounder(
  data = lung_data,
  time = time,
  event = status,
  exposure = trt,
  potential_confounder = prior,
  approach = cox,
  method = change,
  format = gt
)

survival_confounder$table

## ----survival-interaction, message=FALSE, warning=FALSE-----------------------
survival_interaction <- interaction_models(
  data = lung_data,
  time = time,
  event = status,
  exposure = trt,
  effect_modifier = prior,
  covariates = c(age, karno),
  approach = cox,
  test = LRT,
  format = gt
)

survival_interaction$table

## ----mediation, message=FALSE, warning=FALSE----------------------------------
data("data_diabetes_mediation", package = "gtregression")

diabetes_med <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = obesity,
  mediator = glucose,
  outcome = diabetes,
  covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
  outcome_approach = logit,
  sims = 100,
  seed = 123
)

diabetes_med

## ----mediation-body-----------------------------------------------------------
diabetes_med$table_body

## ----mediation-gt, message=FALSE, warning=FALSE-------------------------------
med_gt <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = obesity,
  mediator = glucose,
  outcome = diabetes,
  covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
  outcome_approach = logit,
  format = gt,
  sims = 100,
  seed = 123
)

med_gt$table

## ----mediation-plot, message=FALSE, warning=FALSE, fig.width=7, fig.height=4----
plot_mediation(diabetes_med)

## ----mediation-plot-simple, message=FALSE, warning=FALSE, fig.width=7, fig.height=4----
plot_mediation(diabetes_med, show_estimates = FALSE)

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(gtregression)
library(dplyr)

data("data_diabetes_mediation", package = "gtregression")

dissect(data_diabetes_mediation)

## ----mediation-logit, message=FALSE, warning=FALSE----------------------------
diabetes_med <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = obesity,
  mediator = glucose,
  outcome = diabetes,
  covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
  outcome_approach = logit,
  sims = 100,
  seed = 123
)

diabetes_med

## ----inspect-object-----------------------------------------------------------
diabetes_med$table_body
diabetes_med$values
diabetes_med$models$mediator
diabetes_med$models$outcome
head(diabetes_med$boot)

## ----mediation-plot, fig.width=7, fig.height=5--------------------------------
plot_mediation(diabetes_med)

## ----mediation-plot-no-estimates, fig.width=7, fig.height=5-------------------
plot_mediation(diabetes_med, show_estimates = FALSE)

## ----mediation-quoted, message=FALSE, warning=FALSE---------------------------
exposure_var <- "obesity"
mediator_var <- "glucose"
outcome_var <- "diabetes"
covariate_vars <- c(
  "age", "blood_pressure", "pregnancies", "diabetes_pedigree"
)

med_quoted <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = exposure_var,
  mediator = mediator_var,
  outcome = outcome_var,
  covariates = covariate_vars,
  outcome_approach = "logit",
  sims = 100,
  seed = 456
)

med_quoted

## ----mediation-linear, message=FALSE, warning=FALSE---------------------------
med_linear <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = obesity,
  mediator = glucose,
  outcome = bmi,
  covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
  outcome_approach = linear,
  sims = 100,
  seed = 789
)

med_linear

## ----mediation-linear-plot, fig.width=7, fig.height=5-------------------------
plot_mediation(med_linear)

## ----strata-setup, message=FALSE, warning=FALSE-------------------------------
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes"))
  )

attr(birthwt_data$age, "label") <- "Maternal age"
attr(birthwt_data$lwt, "label") <- "Maternal weight"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"
attr(birthwt_data$ui, "label") <- "Uterine irritability"
attr(birthwt_data$ptl_cat, "label") <- "Previous preterm labour"

## ----strata-desc, message=FALSE, warning=FALSE--------------------------------
strata_desc <- descriptive_table(
  data = birthwt_data,
  exposures = c("age", "lwt", "smoke", "ht", "ui", "ptl_cat"),
  by = race,
  percent = column,
  show_overall = last,
  theme = clinical
)

strata_desc$table

## ----strata-uni, message=FALSE, warning=FALSE---------------------------------
strata_uni <- stratified_uni_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("age", "lwt", "smoke", "ht", "ui", "ptl_cat"),
  stratifier = race,
  approach = logit,
  theme = clinical
)

strata_uni$table

## ----strata-multi-full, message=FALSE, warning=FALSE--------------------------
strata_full <- stratified_multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("age", "lwt", "smoke", "ht", "ui", "ptl_cat"),
  stratifier = race,
  approach = logit,
  theme = clinical
)

strata_full$table

## ----strata-multi, message=FALSE, warning=FALSE-------------------------------
strata_multi <- stratified_multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat"),
  stratifier = race,
  adjust_for = c("age", "lwt"),
  approach = logit,
  theme = striped
)

strata_multi$table

## ----strata-forest, message=FALSE, warning=FALSE, fig.width=8, fig.height=7----
strata_forest_data <- forest_df(strata_multi)

forest_reg(
  strata_forest_data,
  ci_col_width = 18
)

## ----echo=FALSE, out.width='160px'--------------------------------------------
logo_candidates <- c(
  "man/figures/gtregression_hex.png",
  "../man/figures/gtregression_hex.png"
)
logo_path <- logo_candidates[file.exists(logo_candidates)][1]

if (!is.na(logo_path)) {
  knitr::include_graphics(logo_path)
}

## ----setup-birthwt, message=FALSE, warning=FALSE------------------------------
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = ifelse(ptl > 0, "Yes", "No"),
    ftv_cat = case_when(
      ftv == 0 ~ "None",
      ftv == 1 ~ "One",
      ftv >= 2 ~ "Two or more"
    )
  ) |>
  mutate(
    ptl_cat = factor(ptl_cat, levels = c("No", "Yes")),
    ftv_cat = factor(ftv_cat, levels = c("None", "One", "Two or more"))
  )

birthwt_exposures <- c(
  "age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat", "ftv_cat"
)

attr(birthwt_data$age, "label") <- "Maternal age"
attr(birthwt_data$lwt, "label") <- "Maternal weight"
attr(birthwt_data$race, "label") <- "Maternal race"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"
attr(birthwt_data$ui, "label") <- "Uterine irritability"
attr(birthwt_data$ptl_cat, "label") <- "Previous preterm labour"
attr(birthwt_data$ftv_cat, "label") <- "First trimester visits"

## ----quick-describe, message=FALSE, warning=FALSE-----------------------------
birthwt_summary <- descriptive_table(
  data = birthwt_data,
  exposures = birthwt_exposures,
  by = low,
  percent = column,
  show_overall = last,
  theme = clinical
)

birthwt_summary$table

## ----quick-model, message=FALSE, warning=FALSE--------------------------------
birthwt_uni <- uni_reg(
  data = birthwt_data,
  outcome = low,
  exposures = birthwt_exposures,
  approach = logit,
  theme = clinical
)

birthwt_multi <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat", "ftv_cat"),
  adjust_for = c("age", "lwt", "race"),
  approach = logit,
  theme = striped
)

birthwt_multi$table

## ----quick-plot, fig.width=12, fig.height=8,message=FALSE, warning=FALSE------
plot_reg(
  birthwt_multi,
  title = "Adjusted Regression for Low Birth Weight"
)

forest_reg(forest_df(birthwt_uni, birthwt_multi))

## ----quick-merge, message=FALSE, warning=FALSE--------------------------------
birthwt_final <- merge_tables(
  birthwt_summary,
  birthwt_uni,
  birthwt_multi,
  spanners = c("Clinical profile", "Crude OR", "Adjusted OR")
)

birthwt_final <- modify_table(
  birthwt_final,
  caption = "Clinical profile and regression estimates for low birth weight",
  caveat = "Adjusted estimates are adjusted for maternal age, maternal weight, and maternal race."
)

birthwt_final$table

## ----quick-save, eval=FALSE---------------------------------------------------
# save_table(birthwt_final, filename = "birthwt-table", format = html)
# save_docx(tables = birthwt_final, filename = "birthwt-report")

# ---------- minimal theme resolver ----------
.theme_presets_desc <- list(
  minimal  = c("plain","lines","labels_bold"),
  clinical = c("plain","labels_bold","compact"),
  striped  = c("zebra","labels_bold","compact"),
  shaded   = c("header_shaded","labels_bold","lines"),
  jama     = c("plain","lines","labels_bold","compact")
)
.resolve_theme_desc <- function(theme) {
  if (length(theme) == 1 && !is.na(theme) && theme %in% names(.theme_presets_desc))
    return(.theme_presets_desc[[theme]])
  unique(tolower(theme))
}

.statistic_arg_desc <- function(expr, env = parent.frame()) {
  allowed <- c("mean", "median", "mode", "count", "categorical")
  aliases <- c(
    category = "categorical",
    catagory = "categorical",
    catagorical = "categorical",
    factor = "categorical"
  )

  normalize_one <- function(x) {
    if (is.character(x)) {
      out <- tolower(x)
    } else if (is.symbol(x)) {
      out <- tolower(as.character(x))
    } else {
      out <- eval(x, envir = env)
      if (!is.character(out)) {
        stop(
          "`statistic` must use values such as mean, median, count, or categorical.",
          call. = FALSE
        )
      }
      out <- tolower(out)
    }

    idx <- match(out, names(aliases))
    out[!is.na(idx)] <- unname(aliases[idx[!is.na(idx)]])
    out
  }

  if (identical(expr, quote(NULL))) {
    return(NULL)
  }

  if (is.call(expr) && identical(expr[[1L]], as.name("c"))) {
    parts <- as.list(expr)[-1L]
    out <- unlist(lapply(parts, normalize_one), use.names = FALSE)
    names(out) <- names(parts)
  } else {
    out <- normalize_one(expr)
  }

  if (!is.character(out) || anyNA(out) || any(!nzchar(out))) {
    stop(
      "`statistic` must use values such as mean, median, count, or categorical.",
      call. = FALSE
    )
  }

  bad_stats <- setdiff(out, allowed)
  if (length(bad_stats)) {
    stop("Unsupported statistic: ", paste(bad_stats, collapse = ", "), call. = FALSE)
  }

  named <- names(out)
  if (length(out) > 1L && (is.null(named) || any(!nzchar(named)))) {
    stop(
      "`statistic` must be a single summary type or a named vector such as c(age = mean, ftv = categorical).",
      call. = FALSE
    )
  }

  out
}

# ---------- descriptive_table ----------
#' Descriptive Summary Table (no gtsummary) using gt/flextable
#'
#' Publication-ready summary of categorical and continuous variables
#' (optionally stratified). Mimics the OG gtsummary style:
#' * column headers include N, e.g. "Overall, N=200"
#' * categorical rows shown as n (\%)
#' * continuous rows default to Median (IQR) (footnote reflects summary)
#'
#' @param data data.frame
#' @param exposures Character vector of variables to summarise. Quoted names are
#'   recommended in scripts, and bare names are also accepted.
#' @param by Optional single grouping variable. Quoted and bare names are
#'   accepted.
#' @param percent "column" (default) or "row"; aliases like "col"/"rows" accepted
#' @param digits integer; decimals for \% and continuous stats (default 1)
#' @param show_missing "ifany" (default) or "no"
#' @param show_dichotomous "all_levels" (default) or "single_row"
#' @param show_overall "no" (default), "first", or "last"
#' @param statistic Optional summary type for numeric variables. Use a single
#'   value such as \code{"mean"} or \code{mean} for all numeric variables, or a
#'   named vector such as \code{c(age = mean, ftv = categorical)}. Supported
#'   values are \code{"mean"}, \code{"median"}, \code{"mode"},
#'   \code{"count"}, and \code{"categorical"}. Use \code{"categorical"} for
#'   numeric ordinal variables that should be shown as n (\%).
#' @param value optional named list for single-row binaries (e.g., list(sex="Female"));
#'   formula entries like list(sex ~ "Female") are also accepted
#' @param format "flextable" (default) or "gt"
#' @param theme preset or primitives
#'
#' @details
#' If variables have a \code{"label"} attribute, for example from
#' \code{labelled::var_label()}, those labels are used automatically in the
#' displayed table. Internal matching still uses the original column names.
#'
#' @return A list with class \code{c("gtregression", "descriptive_table", ...)}
#'   containing:
#'   \describe{
#'     \item{\code{table}}{A \code{gt_tbl} or \code{flextable}.}
#'     \item{\code{table_display}}{Display-ready data.}
#'     \item{\code{table_body}}{Long audit data with variable, level, and type.}
#'     \item{\code{variable_labels}}{Named character vector of display labels
#'     used for variables.}
#'     \item{metadata}{Additional metadata fields.}
#'   }
#' @export
descriptive_table <- function(data,
                              exposures,
                              by = NULL,
                              percent = c("column","row"),
                              digits = 1,
                              show_missing = c("ifany","no"),
                              show_dichotomous = c("all_levels","single_row"),
                              show_overall = c("no","first","last"),
                              statistic = NULL,
                              value = NULL,
                              format = c("flextable","gt"),
                              theme = c("minimal")) {

  exposures <- .vars_arg(substitute(exposures), env = parent.frame())
  by <- .vars_arg(substitute(by), env = parent.frame(), allow_null = TRUE)
  statistic <- .statistic_arg_desc(substitute(statistic), env = parent.frame())

  # ---- normalize choices (accept aliases) ----
  percent <- .choice_arg(
    substitute(percent),
    env = parent.frame(),
    choices = c("column","row"),
    aliases = c(col = "column", cols = "column", columns = "column",
                rows = "row", rowwise = "row")
  )
  show_missing <- .choice_arg(substitute(show_missing), env = parent.frame(), choices = c("ifany","no"))
  show_dichotomous <- .choice_arg(
    substitute(show_dichotomous),
    env = parent.frame(),
    choices = c("all_levels", "single_row")
  )
  show_overall <- .choice_arg(substitute(show_overall), env = parent.frame(), choices = c("no","first","last"))
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("flextable","gt"))
  theme <- .choice_arg(substitute(theme), env = parent.frame())

  percent <- percent[1]
  show_missing     <- match.arg(tolower(show_missing[1]),     c("ifany","no"))
  show_dichotomous <- match.arg(tolower(show_dichotomous[1]), c("all_levels","single_row"))
  show_overall     <- match.arg(tolower(show_overall[1]),     c("no","first","last"))
  percent          <- match.arg(percent, c("column","row"))
  format           <- match.arg(tolower(format[1]), c("flextable","gt"))
  theme            <- .resolve_theme_desc(theme)

  # ---- checks ----
  if (!is.character(exposures) || length(exposures) < 1) {
    stop("`exposures` must be a non-empty character vector.", call. = FALSE)
  }
  if (!is.null(by) && (!is.character(by) || length(by) != 1L)) {
    stop("`by` must be NULL or a single character variable name.", call. = FALSE)
  }
  if (!is.numeric(digits) || length(digits) != 1L || is.na(digits) || digits < 0) {
    stop("`digits` must be a single non-negative number.", call. = FALSE)
  }
  digits <- as.integer(digits)
  data <- as.data.frame(data)
  missing_vars <- setdiff(c(exposures, by), names(data))
  if (length(missing_vars)) stop("Variables not found: ", paste(missing_vars, collapse=", "), call.=FALSE)
  if (!is.null(statistic) && length(statistic) > 1L) {
    unused_stats <- setdiff(names(statistic), exposures)
    if (length(unused_stats)) {
      stop(
        "`statistic` names must also be included in `exposures`: ",
        paste(unused_stats, collapse = ", "),
        call. = FALSE
      )
    }
  } else if (!is.null(statistic) && !is.null(names(statistic)) &&
             any(nzchar(names(statistic)))) {
    unused_stats <- setdiff(names(statistic), exposures)
    if (length(unused_stats)) {
      stop(
        "`statistic` names must also be included in `exposures`: ",
        paste(unused_stats, collapse = ", "),
        call. = FALSE
      )
    }
  }
  if (!is.null(statistic) && length(statistic) > 1L &&
      (is.null(names(statistic)) || any(!nzchar(names(statistic))))) {
    stop(
      "`statistic` must be a single summary type or a named vector such as c(age = mean, ftv = categorical).",
      call. = FALSE
    )
  }
  if (percent == "row" && is.null(by)) {
    warning("`percent = \"row\"` requires `by`; using column-wise % instead.")
    percent <- "column"
  }
  variable_labels <- .var_label_map(data, exposures)

  # ---- helpers ----
  fmt_num <- function(x, d = digits) formatC(x, digits = d, format="f", big.mark=",")
  fmt_pct <- function(p, d = digits) if (is.na(p)) paste0(fmt_num(0, d), "%") else paste0(fmt_num(100*p, d), "%")
  is_binary <- function(x) { ux <- unique(x[!is.na(x)]); length(ux) == 2 }
  pick_single_level <- function(x) {
    ux <- unique(x[!is.na(x)]); if (length(ux) != 2) return(NA_character_)
    if (is.factor(x)) return(levels(x)[2])
    if (is.logical(x)) return("TRUE")
    if (is.numeric(x)) return(as.character(sort(ux)[2]))
    sort(as.character(ux))[2]
  }
  # default continuous summary: median (IQR)
  cont_summary <- function(x, stat = "median", d = digits) {
    x <- x[!is.na(x)]
    if (!length(x)) return("")
    stat <- match.arg(stat, c("mean","median","mode","count"))
    if (stat == "mean") {
      m <- mean(x); s <- stats::sd(x)
      paste0(fmt_num(m, d), " (", fmt_num(s, d), ")")
    } else if (stat == "median") {
      md <- stats::median(x); q <- stats::quantile(x, c(.25,.75), names=FALSE)
      paste0(fmt_num(md, d), " (", fmt_num(q[1], d), "-", fmt_num(q[2], d), ")")
    } else if (stat == "mode") {
      tab <- sort(table(x), decreasing = TRUE); md <- as.numeric(names(tab)[1])
      paste0(fmt_num(md, d))
    } else {
      paste0("N = ", length(x))
    }
  }
  stat_label <- function(s) switch(s,
                                   mean   = "Mean (SD)",
                                   median = "Median (IQR)",
                                   mode   = "Mode",
                                   count  = "N",
                                   categorical = "n (%)",
                                   "Median (IQR)"
  )
  get_user_stat <- function(v) {
    if (is.null(statistic)) return(NA_character_)
    if (is.null(names(statistic)) || all(!nzchar(names(statistic)))) {
      return(statistic[[1]])
    }
    idx <- match(v, names(statistic))
    if (is.na(idx)) NA_character_ else statistic[[idx]]
  }
  is_forced_categorical <- function(v) {
    identical(get_user_stat(v), "categorical")
  }
  get_single_level <- function(vname) {
    if (is.null(value)) return(NA_character_)
    if (!is.null(names(value)) && vname %in% names(value)) return(as.character(value[[vname]]))
    for (entry in value) {
      if (inherits(entry, "formula") && length(entry) == 3L && identical(as.character(entry[[2]]), vname)) {
        return(as.character(entry[[3]]))
      }
    }
    NA_character_
  }

  # ---- by levels and group Ns (for headers) ----
  by_levels <- NULL
  group_names <- "Overall"
  group_idx   <- list(Overall = rep(TRUE, nrow(data)))
  if (!is.null(by)) {
    bv <- data[[by]]
    by_levels <- if (is.factor(bv)) levels(stats::na.omit(bv)) else unique(stats::na.omit(bv))
    group_names <- as.character(by_levels)
    group_idx   <- lapply(by_levels, function(g) (data[[by]] == g) & !is.na(data[[by]]))
    names(group_idx) <- group_names
    if (show_overall != "no") {
      where <- if (show_overall == "first") 1L else length(group_names) + 1L
      group_names <- append(group_names, values = "Overall", after = where - 1L)
      group_idx   <- append(group_idx,   values = list(Overall = rep(TRUE, nrow(data))), after = where - 1L)
    }
  }
  # ---- construct long body (header + levels; NO N CELLS) ----
  body_rows <- list()
  compact_dichotomous_vars <- character()
  for (v in exposures) {
    x <- data[[v]]
    is_cat <- is.factor(x) || is.character(x) || is_forced_categorical(v)
    is_bin <- is_binary(x)

    if (is_bin && show_dichotomous == "single_row") {
      pick <- get_single_level(v); if (is.na(pick) || !nzchar(pick)) pick <- pick_single_level(x)
      compact_dichotomous_vars <- c(compact_dichotomous_vars, v)
      body_rows[[length(body_rows)+1]] <- data.frame(
        var=v, level=pick, type="dichotomous", stringsAsFactors=FALSE
      )
      if ((show_missing == "ifany") && any(is.na(x))) {
        body_rows[[length(body_rows)+1]] <- data.frame(
          var=v, level="(Missing)", type="categorical_missing", stringsAsFactors=FALSE
        )
      }
      next
    }

    # header row
    body_rows[[length(body_rows)+1]] <- data.frame(
      var=v, level=NA_character_, type="header", stringsAsFactors=FALSE
    )

    if (!is_cat && !is_bin) {
      body_rows[[length(body_rows)+1]] <- data.frame(
        var=v, level=v, type="continuous", stringsAsFactors=FALSE
      ); next
    }
    levels_here <- if (is.factor(x)) levels(x) else sort(unique(as.character(x)))
    add_missing <- (show_missing == "ifany") && any(is.na(x))
    for (lv in levels_here) {
      body_rows[[length(body_rows)+1]] <- data.frame(
        var=v, level=lv, type="categorical", stringsAsFactors=FALSE
      )
    }
    if (add_missing) {
      body_rows[[length(body_rows)+1]] <- data.frame(
        var=v, level="(Missing)", type="categorical_missing", stringsAsFactors=FALSE
      )
    }
  }
  table_body <- do.call(rbind, body_rows)
  if (length(compact_dichotomous_vars)) {
    message(
      "descriptive_table(): dichotomous variables are displayed as ",
      "single rows for ",
      paste(unique(compact_dichotomous_vars), collapse = ", "),
      ". To display all levels, use `show_dichotomous = \"all_levels\"`."
    )
  }

  # ---- display skeleton ----
  display <- do.call(rbind, lapply(
    split(table_body, factor(table_body$var, levels = exposures)),
    function(df) {
    var_nm <- unique(df$var)
    var_lab <- .label_var(var_nm, variable_labels)
    hdr <- data.frame(
      Characteristic = var_lab,
      is_header = TRUE, var = unique(df$var), level = NA_character_,
      type = "header", stringsAsFactors = FALSE
    )
    levs <- subset(df, type != "header")
    if (!nrow(levs)) return(hdr)

    # --- NEW: continuous-only variables -> single row, no duplicate header ---
    if (all(levs$type == "continuous")) {
      levs$Characteristic <- var_lab            # no indent
      levs$is_header      <- TRUE               # bold like a header
      # keep type == "continuous" so add_group_col() will compute summaries
      levs <- levs[, c("Characteristic","is_header","var","level","type"), drop = FALSE]
      return(levs)                              # <- do NOT prepend hdr
    }

    if (any(levs$type == "dichotomous")) {
      levs$Characteristic <- ifelse(levs$type == "dichotomous",
                                    var_lab,
                                    paste0("  ", levs$level))
      levs$is_header <- levs$type == "dichotomous"
      levs <- levs[, c("Characteristic","is_header","var","level","type"), drop = FALSE]
      return(levs)
    }

    # categorical (and mixed) path unchanged
    levs$Characteristic <- ifelse(levs$type == "continuous",
                                  var_lab,
                                  paste0("  ", levs$level))
    levs$is_header <- FALSE
    levs <- levs[, c("Characteristic","is_header","var","level","type"), drop = FALSE]
      rbind(hdr, levs)
    }
  ))

  rownames(display) <- NULL
  display <- .attach_display_metadata(
    display,
    row_exposure = display$var,
    variable_labels = variable_labels
  )

  # ---- fill cells (no header cells content) ----
  add_group_col <- function(df, grp_idx) {
    cells <- character(nrow(df))
    for (i in seq_len(nrow(df))) {
      typ <- df$type[i]; v <- df$var[i]; lv <- df$level[i]
      x <- data[[v]][grp_idx]

      if (typ == "header") { cells[i] <- "" ; next }

      if (typ == "continuous") {
        st <- get_user_stat(v); if (is.na(st) || !nzchar(st)) st <- "median"
        cells[i] <- cont_summary(x, stat = st, d = digits)
        next
      }
      if (typ == "categorical_missing") {
        n <- sum(is.na(data[[v]][grp_idx]))
        denom <- sum(!is.na(data[[v]][grp_idx]))
        p <- if (denom > 0) n/denom else NA_real_
        cells[i] <- paste0(n, " (", fmt_pct(p, digits), ")"); next
      }
      n <- sum(as.character(x) == as.character(lv), na.rm=TRUE)
      if (is.null(by) || percent == "column") {
        denom <- sum(!is.na(x))
        p <- if (denom > 0) n/denom else NA_real_
        cells[i] <- paste0(n, " (", fmt_pct(p, digits), ")")
      } else {
        cells[i] <- as.character(n) # temporarily; will convert to row %
      }
    }
    cells
  }
  for (gn in group_names) {
    display[[gn]] <- add_group_col(display, group_idx[[gn]])
  }

  # row % conversion
  if (!is.null(by) && percent == "row") {
    grp_cols <- setdiff(group_names, "Overall")
    for (i in seq_len(nrow(display))) {
      if (display$type[i] %in% c("categorical","categorical_missing","dichotomous") &&
          (!display$is_header[i] || display$type[i] == "dichotomous")) {
        v <- display$var[i]; lv <- display$level[i]
        counts <- integer(length(grp_cols))
        for (j in seq_along(grp_cols)) {
          idx <- group_idx[[grp_cols[j]]]; x <- data[[v]][idx]
          counts[j] <- if (display$type[i] == "categorical_missing") sum(is.na(x)) else
            sum(as.character(x) == as.character(lv), na.rm=TRUE)
        }
        row_tot <- sum(counts)
        for (j in seq_along(grp_cols)) {
          p <- if (row_tot > 0) counts[j]/row_tot else NA_real_
          display[[grp_cols[j]]][i] <- paste0(counts[j], " (", fmt_pct(p, digits), ")")
        }
        if ("Overall" %in% group_names) display[["Overall"]][i] <- as.character(row_tot)
      }
    }
  }

  # ---- header labels with N ----
  header_labels <- setNames(
    object = paste0(group_names, ", N=", vapply(group_names, function(gn) {
      # Use non-missing over all exposures per group as a pragmatic denominator
      sum(rowSums(!is.na(data[exposures])[group_idx[[gn]], , drop=FALSE]) > 0)
    }, numeric(1))),
    nm = group_names
  )

  # ---- footnotes ----
  # build continuous stat note
  cont_vars <- exposures[!vapply(names(data[exposures]), function(v) {
    x <- data[[v]]
    is.factor(x) || is.character(x) || is_binary(x) || is_forced_categorical(v)
  }, logical(1))]
  stat_used <- if (length(cont_vars)) {
    lbls <- vapply(
      cont_vars,
      function(v) stat_label(ifelse(is.na(get_user_stat(v)), "median", get_user_stat(v))),
      character(1)
    )
    if (length(unique(lbls)) == 1L) {
      paste0("Continuous variables shown as ", unique(lbls), ".")
    } else {
      paste0("Continuous summaries: ",
             paste(paste0(cont_vars, " = ", lbls), collapse = "; "), ".")
    }
  } else NULL

  pct_note <- if (!is.null(by)) {
    if (percent == "column") "Categorical variables shown as n (%); percentages are by column."
    else "Categorical variables shown as n (%); percentages are by row (Overall shows counts)."
  } else {
    "Categorical variables shown as n (%)."
  }

  foot <- c(pct_note, stat_used)

  # ---- builders (gt / flextable) ----
  build_gt <- function(display_df, header_labels, footnote_lines, theme) {
    if (!requireNamespace("gt", quietly = TRUE))
      stop("Install 'gt' or use format='flextable'.", call. = FALSE)
    out <- display_df[, c("Characteristic", names(header_labels)), drop=FALSE]
    tb <- gt::gt(out) |>
      gt::cols_label(.list = c(list(Characteristic = "Characteristic"), as.list(header_labels))) |>
      gt::cols_align("left",   columns = "Characteristic") |>
      gt::cols_align("center", columns = setdiff(names(out), "Characteristic")) |>
      gt::tab_options(
        table.font.names         = "system-ui",
        data_row.padding         = gt::px(4),
        heading.background.color = "white",
        table.background.color   = "white"
      )
    # bold labels + spanners, bold var headers; indent levels
    tb <- gt::tab_style(tb, gt::cell_text(weight="bold"), locations = gt::cells_column_labels())
    tb <- gt::tab_style(tb, gt::cell_text(weight="bold"),
                        gt::cells_body(rows = display_df$is_header %in% TRUE, columns = "Characteristic"))
    tb <- gt::tab_style(tb, gt::cell_text(indent = gt::px(12)),
                        gt::cells_body(rows = display_df$is_header %in% FALSE, columns = "Characteristic"))
    if ("header_shaded" %in% theme) tb <- gt::tab_options(tb, column_labels.background.color = "#f6f8fa")
    if ("zebra" %in% theme) tb <- gt::opt_row_striping(tb)
    if ("lines" %in% theme) {
      tb <- gt::tab_style(tb, gt::cell_borders(sides="top", color="#DADADA", weight=gt::px(1)),
                          gt::cells_body(rows=TRUE))
      tb <- gt::tab_options(tb,
                            table.border.top.style="solid",    table.border.top.color="#DADADA",
                            table.border.bottom.style="solid", table.border.bottom.color="#DADADA")
    }
    if ("compact" %in% theme) tb <- gt::tab_options(tb, data_row.padding = gt::px(2))
    if (length(footnote_lines)) {
      tb <- gt::tab_source_note(tb, source_note = footnote_lines)
      tb <- gt::tab_options(tb, source_notes.padding = gt::px(2))
    }
    tb
  }

  build_flex <- function(display_df, header_labels, footnote_lines, theme) {
    if (!requireNamespace("flextable", quietly = TRUE))
      stop("Install 'flextable' or use format='gt'.", call. = FALSE)
    out <- display_df[, c("Characteristic", names(header_labels)), drop=FALSE]
    colnames(out) <- c("Characteristic", unname(header_labels))
    ft <- flextable::flextable(out)
    ft <- flextable::align(ft, j="Characteristic", align="left",  part="all")
    ft <- flextable::align(ft, j=setdiff(names(out), "Characteristic"), align="center", part="all")
    ft <- flextable::bold(ft, part="header", bold=TRUE)
    hdr_idx <- which(display_df$is_header %in% TRUE)
    lvl_idx <- which(display_df$is_header %in% FALSE)
    if (length(hdr_idx)) ft <- flextable::bold(ft, i=hdr_idx, j="Characteristic", bold=TRUE, part="body")
    if (length(lvl_idx)) ft <- flextable::padding(ft, i=lvl_idx, j="Characteristic", padding.left=14, part="body")
    if ("header_shaded" %in% theme) ft <- flextable::bg(ft, part="header", bg="#f6f8fa")
    if ("zebra" %in% theme && length(lvl_idx)) {
      ft <- flextable::bg(ft, i=lvl_idx[seq(1,length(lvl_idx),by=2)], bg="#f6f8fa", part="body")
    }
    if ("lines" %in% theme) {
      n_body <- nrow(out); if (!is.null(n_body) && n_body>0) {
        ft <- flextable::hline(ft, i=n_body, part="body")
      }
    }
    if ("compact" %in% theme) ft <- flextable::padding(ft, padding=2, part="body")
    ft <- flextable::autofit(ft)
    if (length(footnote_lines)) {
      ft <- flextable::add_footer_lines(ft, values=footnote_lines)
      ft <- flextable::fontsize(ft, size=8, part="footer")
      ft <- flextable::padding(ft, part="footer", padding.top=0, padding.bottom=0)
      ft <- flextable::line_spacing(ft, part="footer", space=0.9)
      ft <- flextable::align(ft, part="footer", align="left")
    }
    ft
  }
  tbl <- if (format == "gt") build_gt(display, header_labels, foot, theme)
  else                build_flex(display, header_labels, foot, theme)

  table_display <- display[, c("Characteristic","is_header", names(header_labels)), drop=FALSE]
  table_display <- .attach_display_metadata(
    table_display,
    row_exposure = display$var,
    variable_labels = variable_labels
  )

  res <- list(
    table         = tbl,
    table_display = table_display,
    table_body    = table_body,
    variable_labels = variable_labels,
    by            = by,
    levels        = by_levels,
    format        = format,
    source        = "descriptive_table"
  )
  # keep the same notes the table shows
  res$footnotes <- unique(c(foot))

  engine_class <- if (format == "gt") "gt_desc" else "ft_desc"
  class(res) <- c("gtregression","descriptive_table", engine_class, class(res))
  res
}

birthwt_modify_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
    )
}

lung_modify_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes")),
      celltype = factor(
        celltype,
        levels = c("squamous", "smallcell", "adeno", "large"),
        labels = c("Squamous", "Small cell", "Adenocarcinoma", "Large cell")
      )
    )
}

test_that("modify_table updates package-native univariable gt tables", {
  df <- birthwt_modify_data()

  tbl <- uni_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "smoke", "ht"),
    approach = logit,
    format = gt
  )

  modified <- modify_table(
    tbl,
    variable_labels = c(age = "Maternal age", smoke = "Smoking", ht = "Hypertension"),
    level_labels = list(smoke = c(Yes = "Smoker"), ht = c(No = "No hypertension")),
    header_labels = c(estimate = "Crude OR", p.value = "P"),
    caption = "Table 1. Univariable regression",
    remove_N = TRUE,
    caveat = "Interpret with clinical context."
  )

  expect_s3_class(modified, "gtregression")
  expect_s3_class(modified, "uni_reg")
  expect_s3_class(modified$table, "gt_tbl")
  expect_false("N" %in% names(modified$table_display))
  expect_true("Maternal age" %in% modified$table_display$Characteristic)
  expect_true("Smoking" %in% modified$table_display$Characteristic)
  expect_true("  Smoker" %in% modified$table_display$Characteristic)
  expect_true("  No hypertension" %in% modified$table_display$Characteristic)
  expect_equal(modified$caption, "Table 1. Univariable regression")
  expect_true(any(grepl("Interpret with clinical context", modified$footnotes, fixed = TRUE)))
})

test_that("modify_table validates header aliases and preserves visible headers", {
  df <- birthwt_modify_data()
  tbl <- uni_reg(df, outcome = "low", exposures = c("age", "smoke"), approach = logit, format = gt)

  modified <- modify_table(
    tbl,
    header_labels = c("OR (95% CI)" = "Odds ratio", "p-value" = "P value")
  )

  expect_s3_class(modified$table, "gt_tbl")
  expect_true("OR (95% CI)" %in% names(modified$table_display))
  expect_error(
    modify_table(tbl, header_labels = c(not_a_column = "Nope")),
    "columns not found"
  )
})

test_that("modify_table works for adjusted multivariable tables and footnote options", {
  df <- birthwt_modify_data()

  tbl <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("smoke", "ht"),
    adjust_for = c("age", "lwt"),
    approach = logit,
    format = gt
  )

  keep_notes <- modify_table(tbl, caveat = "Adjusted model.")
  renamed <- modify_table(tbl, header_labels = c(estimate = "Adjusted OR", p.value = "P"))
  drop_notes <- modify_table(
    tbl,
    remove_abbreviations = TRUE,
    remove_N_obs = TRUE,
    caveat = "Adjusted model."
  )

  expect_s3_class(keep_notes, "multi_reg")
  expect_s3_class(renamed$table, "gt_tbl")
  expect_true(any(grepl("OR = Odds Ratio", keep_notes$footnotes, fixed = TRUE)))
  expect_true(any(grepl("Adjusted model", keep_notes$footnotes, fixed = TRUE)))
  relabelled <- modify_table(
    tbl,
    variable_labels = c(age = "Maternal age", lwt = "Maternal weight")
  )
  expect_true(any(relabelled$footnotes == "Adjusted for Maternal age and Maternal weight"))
  expect_false(any(relabelled$footnotes == "Adjusted for age and lwt"))
  custom_adjustment <- modify_table(
    tbl,
    remove_adjustment_note = TRUE,
    caveat = "Adjusted for baseline maternal characteristics."
  )
  hidden_adjustment <- modify_table(tbl, remove_adjustment_note = TRUE)
  expect_true(any(custom_adjustment$footnotes == "Adjusted for baseline maternal characteristics."))
  expect_false(any(custom_adjustment$footnotes == "Adjusted for age and lwt"))
  expect_false(any(grepl("^Adjusted for ", hidden_adjustment$footnotes)))
  expect_false(any(grepl("OR = Odds Ratio", drop_notes$footnotes, fixed = TRUE)))
  expect_false(any(grepl("complete observations included", drop_notes$footnotes, fixed = TRUE)))
})

test_that("modify_table works for Cox and survival regression tables", {
  skip_if_not_installed("survival")
  skip_if_not_installed("flextable")

  df <- lung_modify_data()

  cox_tbl <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior),
    adjust_for = c(age, karno),
    format = gt
  )
  cox_mod <- modify_table(
    cox_tbl,
    variable_labels = c(trt = "Treatment group", celltype = "Cancer cell type"),
    header_labels = c(estimate = "Adjusted HR", p.value = "P")
  )

  aft_tbl <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior),
    adjust_for = c(age, karno),
    distribution = weibull,
    format = flextable
  )
  aft_mod <- modify_table(
    aft_tbl,
    variable_labels = c(trt = "Treatment group", celltype = "Cancer cell type"),
    header_labels = c(estimate = "Adjusted time ratio", p.value = "P")
  )

  expect_s3_class(cox_mod, "cox_reg")
  expect_s3_class(cox_mod$table, "gt_tbl")
  expect_true("Treatment group" %in% cox_mod$table_display$Characteristic)
  expect_true("Cancer cell type" %in% cox_mod$table_display$Characteristic)
  expect_s3_class(aft_mod, "surv_reg")
  expect_s3_class(aft_mod$table, "flextable")
  expect_true("Treatment group" %in% aft_mod$table_display$Characteristic)
  expect_true("Cancer cell type" %in% aft_mod$table_display$Characteristic)

  cox_no_adjustment_note <- modify_table(
    cox_tbl,
    remove_adjustment_note = TRUE,
    caveat = "Adjusted survival model."
  )
  aft_no_adjustment_note <- modify_table(
    aft_tbl,
    remove_adjustment_note = TRUE,
    caveat = "Adjusted parametric survival model."
  )
  expect_false(any(grepl("^Adjusted for ", cox_no_adjustment_note$footnotes)))
  expect_false(any(grepl("^Adjusted for ", aft_no_adjustment_note$footnotes)))
  expect_true(any(cox_no_adjustment_note$footnotes == "Adjusted survival model."))
  expect_true(any(aft_no_adjustment_note$footnotes == "Adjusted parametric survival model."))
})

test_that("modify_table preserves stratified Cox sample-column controls", {
  skip_if_not_installed("survival")

  df <- lung_modify_data()

  tbl <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(celltype, prior),
    stratifier = trt,
    show_sample = both,
    format = gt
  )

  modified <- modify_table(
    tbl,
    variable_labels = c(celltype = "Cancer cell type", prior = "Prior therapy"),
    header_labels = c(estimate = "Crude HR"),
    remove_N = TRUE,
    caveat = "Stratified Cox table."
  )

  expect_s3_class(modified, "stratified_cox_reg")
  expect_s3_class(modified$table, "gt_tbl")
  expect_false(any(startsWith(names(modified$table_display), "..N__")))
  expect_true(any(startsWith(names(modified$table_display), "..Events__")))
  expect_true("Cancer cell type" %in% modified$table_display$Characteristic)
  expect_true(any(grepl("Stratified Cox table", modified$footnotes, fixed = TRUE)))
  expect_error(
    modify_table(tbl, header_labels = c(p.value = "P")),
    "columns not found in the stratified table"
  )
})

test_that("modify_table preserves stratified survival sample-column controls", {
  skip_if_not_installed("survival")
  skip_if_not_installed("flextable")

  df <- lung_modify_data()

  tbl <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(celltype, prior),
    adjust_for = c(age, karno),
    stratifier = trt,
    distribution = weibull,
    show_sample = both,
    format = flextable
  )

  modified <- modify_table(
    tbl,
    variable_labels = c(celltype = "Cancer cell type", prior = "Prior therapy"),
    header_labels = c(estimate = "Adjusted time ratio"),
    remove_N = TRUE
  )

  expect_s3_class(modified, "stratified_surv_reg")
  expect_s3_class(modified$table, "flextable")
  expect_false(any(startsWith(names(modified$table_display), "..N__")))
  expect_true(any(startsWith(names(modified$table_display), "..Events__")))
  expect_true("Cancer cell type" %in% modified$table_display$Characteristic)
})

test_that("modify_table works for descriptive and merged tables", {
  df <- birthwt_modify_data()

  desc <- descriptive_table(
    data = df,
    exposures = c("age", "smoke"),
    by = "low",
    show_overall = "last"
  )
  uni <- uni_reg(df, outcome = "low", exposures = c("age", "smoke"), approach = logit)
  merged <- merge_tables(desc, uni, spanners = c("Descriptive", "Univariable"))

  desc_mod <- modify_table(
    desc,
    variable_labels = c(age = "Age", smoke = "Smoking"),
    level_labels = list(smoke = c(Yes = "Smoker"))
  )
  merged_mod <- modify_table(
    merged,
    variable_labels = c(age = "Age", smoke = "Smoking")
  )

  expect_s3_class(desc_mod, "descriptive_table")
  expect_s3_class(merged_mod, "merged_table")
  expect_true("Age" %in% desc_mod$table_display$Characteristic)
  expect_true("  Smoker" %in% desc_mod$table_display$Characteristic)
  expect_true("Smoking" %in% merged_mod$table_display$Characteristic)
})

test_that("modified input labels still align when tables are merged", {
  skip_if_not_installed("flextable")

  df <- birthwt_modify_data()

  desc <- descriptive_table(
    data = df,
    exposures = c("age", "smoke"),
    by = "low",
    show_overall = "last",
    format = flextable
  )
  desc <- modify_table(desc, variable_labels = c(smoke = "Smoking status"))

  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = logit,
    format = flextable
  )

  merged <- merge_tables(desc, uni, spanners = c("Descriptive", "Crude"))
  merged <- modify_table(merged, variable_labels = c(smoke = "Smoking status"))

  expect_equal(sum(trimws(merged$table_display$Characteristic) == "Smoking status"), 1L)

  header_text <- vapply(
    merged$table$header$content$data,
    function(part) part$txt[1],
    character(1)
  )
  expect_false(any(grepl("_p[0-9]+$", header_text)))
  expect_true(all(c("Descriptive", "Crude", "Low BW", "OR (95% CI)", "p-value") %in%
                    header_text))
})

test_that("modify_table works for flextable outputs", {
  skip_if_not_installed("flextable")

  df <- birthwt_modify_data()
  tbl <- uni_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = logit,
    format = flextable
  )

  modified <- modify_table(
    tbl,
    variable_labels = c(smoke = "Smoking"),
    header_labels = c(estimate = "Crude OR"),
    bold_labels = TRUE,
    bold_levels = TRUE,
    italic_labels = TRUE,
    italic_levels = TRUE,
    caption = "Flextable regression"
  )

  expect_s3_class(modified, "ft_uni")
  expect_s3_class(modified$table, "flextable")
  expect_true("Smoking" %in% modified$table_display$Characteristic)
})

test_that("modify_table preserves existing publication notes", {
  df <- birthwt_modify_data()
  tbl <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("smoke", "ht"),
    adjust_for = "age",
    approach = logit,
    format = flextable
  )

  modified <- modify_table(tbl, caption = "Adjusted model")

  expect_true(any(grepl("Abbreviations:", modified$footnotes)))
  expect_true(any(grepl("Ref\\. = reference category", modified$footnotes)))
  expect_true(any(grepl("complete observations", modified$footnotes)))
  expect_true(any(grepl("Adjusted for", modified$footnotes)))
})

test_that("modify_table validates inputs clearly", {
  df <- birthwt_modify_data()
  tbl <- uni_reg(df, outcome = "low", exposures = "smoke", approach = logit)

  expect_error(modify_table(data.frame(x = 1)), "gtregression object")
  expect_error(modify_table(tbl, variable_labels = c("Smoking")), "variable_labels")
  expect_error(modify_table(tbl, level_labels = c(Yes = "Smoker")), "level_labels")
  expect_error(modify_table(tbl, level_labels = list(smoke = c("Smoker"))), "level_labels\\$smoke")
  expect_error(modify_table(tbl, bold_labels = NA), "`bold_labels` must be")
  expect_error(modify_table(tbl, bold_levels = 1), "`bold_levels` must be")
  expect_error(modify_table(tbl, italic_labels = 1), "`italic_labels` must be")
  expect_error(modify_table(tbl, italic_levels = NA), "`italic_levels` must be")
  expect_error(modify_table(tbl, remove_N = NA), "`remove_N` must be")
  expect_error(modify_table(tbl, remove_N_obs = c(TRUE, FALSE)), "`remove_N_obs` must be")
  expect_error(modify_table(tbl, remove_abbreviations = "yes"), "`remove_abbreviations` must be")
  expect_error(modify_table(tbl, caption = NA_character_), "`caption` must be")
  expect_error(modify_table(tbl, caveat = c("a", "b")), "`caveat` must be")
})

test_that("descriptive_table works with basic categorical inputs", {
  df <- data.frame(
    group = factor(c("A", "A", "B", "B")),
    status = factor(c("No", "Yes", "No", "Yes")),
    risk = factor(c("Low", "High", "High", NA), levels = c("Low", "High"))
  )

  tbl <- descriptive_table(
    data = df,
    exposures = c("status", "risk"),
    by = "group"
  )

  expect_s3_class(tbl, "descriptive_table")
  expect_s3_class(tbl$table, "flextable")
  expect_equal(tbl$by, "group")
  expect_equal(tbl$levels, c("A", "B"))
  expect_equal(tbl$table_display$A[tbl$table_display$Characteristic == "  Yes"], "1 (50.0%)")
  expect_equal(tbl$table_display$B[tbl$table_display$Characteristic == "  Low"], "0 (0.0%)")
  expect_true("(Missing)" %in% trimws(tbl$table_display$Characteristic))
  expect_match(tbl$footnotes[1], "percentages are by column")
})

test_that("descriptive_table handles row percentages and overall position", {
  df <- data.frame(
    group = factor(c("A", "A", "B", "B", "B"), levels = c("A", "B")),
    status = factor(c("No", "Yes", "No", "No", "Yes"))
  )

  tbl <- descriptive_table(
    data = df,
    exposures = "status",
    by = "group",
    percent = "row",
    show_overall = "last"
  )

  expect_named(tbl$table_display, c("Characteristic", "is_header", "A", "B", "Overall"))
  expect_equal(tbl$table_display$A[tbl$table_display$Characteristic == "  No"], "1 (33.3%)")
  expect_equal(tbl$table_display$B[tbl$table_display$Characteristic == "  No"], "2 (66.7%)")
  expect_equal(tbl$table_display$Overall[tbl$table_display$Characteristic == "  No"], "3")
  expect_match(tbl$footnotes[1], "percentages are by row")
})

test_that("descriptive_table falls back to column percentages without by", {
  df <- data.frame(status = factor(c("No", "Yes", "No")))

  expect_warning(
    tbl <- descriptive_table(df, "status", percent = "rows"),
    "`percent = \"row\"` requires `by`"
  )

  expect_named(tbl$table_display, c("Characteristic", "is_header", "Overall"))
  expect_equal(tbl$table_display$Overall[tbl$table_display$Characteristic == "  No"], "2 (66.7%)")
})

test_that("descriptive_table handles dichotomous single-row values", {
  df <- data.frame(
    named = factor(c("No", "Yes", "No", "Yes")),
    formula = factor(c("Low", "High", "High", "Low"))
  )

  expect_message(
    tbl_named <- descriptive_table(
      df,
      "named",
      show_dichotomous = "single_row",
      value = list(named = "Yes")
    ),
    "show_dichotomous = \"all_levels\""
  )
  expect_message(
    tbl_formula <- descriptive_table(
      df,
      "formula",
      show_dichotomous = "single_row",
      value = list(formula ~ "High")
    ),
    "show_dichotomous = \"all_levels\""
  )

  expect_equal(tbl_named$table_display$Characteristic, "named")
  expect_equal(tbl_named$table_display$Overall, "2 (50.0%)")
  expect_equal(tbl_formula$table_display$Characteristic, "formula")
  expect_equal(tbl_formula$table_display$Overall, "2 (50.0%)")
})

test_that("descriptive_table chooses sensible default single-row levels", {
  df <- data.frame(
    logical_var = c(TRUE, FALSE, TRUE),
    numeric_var = c(0, 1, 1),
    char_var = c("a", "b", "a")
  )

  tbl_logical <- suppressMessages(
    descriptive_table(df, "logical_var", show_dichotomous = "single_row")
  )
  tbl_numeric <- suppressMessages(
    descriptive_table(df, "numeric_var", show_dichotomous = "single_row")
  )
  tbl_char <- suppressMessages(
    descriptive_table(df, "char_var", show_dichotomous = "single_row")
  )

  expect_equal(tbl_logical$table_display$Characteristic, "logical_var")
  expect_equal(tbl_numeric$table_display$Characteristic, "numeric_var")
  expect_equal(tbl_char$table_display$Characteristic, "char_var")
  expect_equal(tbl_logical$table_display$Overall, "2 (66.7%)")
  expect_equal(tbl_numeric$table_display$Overall, "2 (66.7%)")
  expect_equal(tbl_char$table_display$Overall, "1 (33.3%)")
})

test_that("descriptive_table single-row dichotomous variables are compact and ordered", {
  df <- data.frame(
    group = factor(c("A", "A", "B", "B")),
    smoke = factor(c("No", "Yes", "No", "Yes"), levels = c("No", "Yes")),
    ht = factor(c("No", "No", "Yes", "Yes"), levels = c("No", "Yes"))
  )

  expect_message(
    tbl <- descriptive_table(
      df,
      exposures = c("smoke", "ht"),
      by = "group",
      show_dichotomous = "single_row",
      value = list(smoke = "Yes", ht = "Yes"),
      show_overall = "last",
      percent = "row"
    ),
    "smoke, ht"
  )

  expect_equal(tbl$table_display$Characteristic, c("smoke", "ht"))
  expect_true(all(tbl$table_display$is_header))
  expect_false(any(grepl("^\\s", tbl$table_display$Characteristic)))
  expect_equal(tbl$table_display$A, c("1 (50.0%)", "0 (0.0%)"))
  expect_equal(tbl$table_display$B, c("1 (50.0%)", "2 (100.0%)"))
  expect_equal(tbl$table_display$Overall, c("2", "2"))
})

test_that("descriptive_table can treat numeric ordinal variables as categorical", {
  df <- data.frame(
    group = factor(c("A", "A", "B", "B", "B")),
    ordinal_score = c(0, 1, 0, 2, 2),
    continuous_score = c(10, 12, 15, 18, 21)
  )

  default_tbl <- descriptive_table(
    df,
    exposures = c("ordinal_score", "continuous_score"),
    by = "group",
    show_overall = "last"
  )
  ordinal_tbl <- descriptive_table(
    df,
    exposures = c("ordinal_score", "continuous_score"),
    by = "group",
    statistic = c(ordinal_score = categorical),
    show_overall = "last"
  )
  ordinal_tbl_quoted <- descriptive_table(
    df,
    exposures = c("ordinal_score", "continuous_score"),
    by = "group",
    statistic = c(ordinal_score = "categorical", continuous_score = "mean"),
    show_overall = "last"
  )
  ordinal_tbl_typo <- descriptive_table(
    df,
    exposures = c("ordinal_score", "continuous_score"),
    by = "group",
    statistic = c(ordinal_score = catagorical),
    show_overall = "last"
  )

  expect_equal(
    default_tbl$table_display$Overall[
      default_tbl$table_display$Characteristic == "ordinal_score"
    ],
    "1.0 (0.0-2.0)"
  )
  expect_true("ordinal_score" %in% ordinal_tbl$table_display$Characteristic)
  expect_true(all(c("  0", "  1", "  2") %in% ordinal_tbl$table_display$Characteristic))
  expect_equal(
    ordinal_tbl$table_display$Overall[
      ordinal_tbl$table_display$Characteristic == "  2"
    ],
    "2 (40.0%)"
  )
  expect_equal(
    ordinal_tbl_quoted$table_display$Overall[
      ordinal_tbl_quoted$table_display$Characteristic == "  2"
    ],
    "2 (40.0%)"
  )
  expect_equal(
    ordinal_tbl_typo$table_display$Overall[
      ordinal_tbl_typo$table_display$Characteristic == "  2"
    ],
    "2 (40.0%)"
  )
  expect_equal(
    ordinal_tbl_quoted$table_display$Overall[
      ordinal_tbl_quoted$table_display$Characteristic == "continuous_score"
    ],
    "15.2 (4.4)"
  )
  expect_match(ordinal_tbl$footnotes[2], "Continuous variables shown as Median")
})

test_that("descriptive_table handles continuous summaries", {
  df <- data.frame(
    mean_var = c(1, 2, 3, 4),
    median_var = c(1, 2, 10, NA),
    mode_var = c(1, 1, 2, 3),
    count_var = c(1, 2, NA, 4),
    empty_var = c(NA_real_, NA_real_, NA_real_, NA_real_)
  )

  tbl <- descriptive_table(
    df,
    c("mean_var", "median_var", "mode_var", "count_var", "empty_var"),
    statistic = c(
      mean_var = "mean",
      median_var = "median",
      mode_var = "mode",
      count_var = "count"
    )
  )

  expect_equal(tbl$table_display$Overall[tbl$table_display$Characteristic == "mean_var"], "2.5 (1.3)")
  expect_equal(tbl$table_display$Overall[tbl$table_display$Characteristic == "median_var"], "2.0 (1.5-6.0)")
  expect_equal(tbl$table_display$Overall[tbl$table_display$Characteristic == "mode_var"], "1.0")
  expect_equal(tbl$table_display$Overall[tbl$table_display$Characteristic == "count_var"], "N = 3")
  expect_equal(tbl$table_display$Overall[tbl$table_display$Characteristic == "empty_var"], "")
  expect_match(tbl$footnotes[2], "Continuous summaries")

  mean_all <- descriptive_table(
    df,
    c("mean_var", "median_var"),
    statistic = mean
  )
  expect_equal(mean_all$table_display$Overall[mean_all$table_display$Characteristic == "mean_var"], "2.5 (1.3)")
  expect_match(mean_all$footnotes[2], "Continuous variables shown as Mean")
})

test_that("descriptive_table supports flextable and theme options", {
  skip_if_not_installed("flextable")

  df <- data.frame(
    group = factor(c("A", "A", "B")),
    status = factor(c("No", "Yes", "No"))
  )

  tbl <- descriptive_table(
    df,
    "status",
    by = "group",
    format = "flextable",
    theme = "striped",
    show_overall = "first"
  )

  expect_s3_class(tbl, "ft_desc")
  expect_s3_class(tbl$table, "flextable")
  expect_named(tbl$table_display, c("Characteristic", "is_header", "Overall", "A", "B"))

  body_bottom_borders <- tbl$table$body$styles$cells$border.width.bottom$data
  expect_true(all(body_bottom_borders[1, ] == 0))
  expect_true(all(body_bottom_borders[nrow(body_bottom_borders), ] > 0))
})

test_that("descriptive_table accepts custom theme primitives", {
  df <- data.frame(status = factor(c("No", "Yes", "No")))

  tbl <- descriptive_table(df, "status", theme = c("HEADER_SHADED", "compact"))

  expect_s3_class(tbl, "ft_desc")
  expect_equal(tbl$table_display$Overall[tbl$table_display$Characteristic == "  No"], "2 (66.7%)")
})

test_that("descriptive_table can hide missing rows", {
  df <- data.frame(status = factor(c("No", "Yes", NA)))

  tbl <- descriptive_table(df, "status", show_missing = "no")

  expect_false("(Missing)" %in% trimws(tbl$table_display$Characteristic))
})

test_that("descriptive_table gives clear errors for invalid inputs", {
  df <- data.frame(x = 1:3, y = 4:6, group = c("A", "B", "A"))

  expect_error(descriptive_table(df, character(0)), "`exposures` must be")
  expect_error(descriptive_table(df, 1), "`exposures` must be")
  expect_error(descriptive_table(df, "x", by = c("group", "y")), "`by` must be")
  expect_error(descriptive_table(df, "not_in_data"), "Variables not found")
  expect_error(descriptive_table(df, "x", digits = -1), "`digits` must be")
  expect_error(descriptive_table(df, "x", statistic = c(x = "range")), "Unsupported statistic")
  expect_error(descriptive_table(df, "x", statistic = c("mean", "median")), "`statistic` must be")
  expect_error(descriptive_table(df, "x", statistic = c(y = categorical)), "must also be included")
})

labelled_birthwt_data <- function() {
  df <- data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
      ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes"))
    )

  attr(df$smoke, "label") <- "Smoking during pregnancy"
  attr(df$ht, "label") <- "Hypertension"
  attr(df$age, "label") <- "Maternal age"
  attr(df$race, "label") <- "Maternal race"
  df
}

test_that("variable label attributes are used in descriptive and regression displays", {
  df <- labelled_birthwt_data()

  desc <- descriptive_table(
    data = df,
    exposures = c("age", "race", "smoke", "ht"),
    by = "low",
    show_dichotomous = "single_row"
  )
  uni <- uni_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "race", "smoke", "ht"),
    approach = logit
  )
  multi <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "race", "smoke", "ht"),
    approach = logit
  )

  expect_true("Maternal age" %in% desc$table_display$Characteristic)
  expect_true("Smoking during pregnancy" %in% desc$table_display$Characteristic)
  expect_true("Maternal race" %in% uni$table_display$Characteristic)
  expect_true("Hypertension" %in% multi$table_display$Characteristic)

  expect_true("age" %in% desc$table_body$var)
  expect_true("smoke" %in% uni$table_body$exposure)
  expect_equal(uni$variable_labels[["smoke"]], "Smoking during pregnancy")
  expect_equal(multi$variable_labels[["ht"]], "Hypertension")
})

test_that("variable label attributes remain compatible with modify_table, plots, merge, and forest_df", {
  df <- labelled_birthwt_data()

  desc <- descriptive_table(
    data = df,
    exposures = c("age", "race", "smoke", "ht"),
    by = "low"
  )
  uni <- uni_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "race", "smoke", "ht"),
    approach = logit
  )
  multi <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "race", "smoke", "ht"),
    approach = logit
  )

  modified <- modify_table(
    uni,
    variable_labels = c(smoke = "Smoker"),
    level_labels = list(smoke = c(Yes = "Smoked"))
  )
  expect_true("Smoker" %in% modified$table_display$Characteristic)
  expect_true("  Smoked" %in% modified$table_display$Characteristic)

  p <- plot_reg(uni, show_ref = FALSE)
  expect_true("Smoking during pregnancy" %in% p$data$label)
  expect_true("Hypertension" %in% p$data$label)

  merged <- merge_tables(desc, uni, multi)
  expect_true("Maternal age" %in% merged$table_display$Characteristic)
  expect_true("Smoking during pregnancy" %in% merged$table_display$Characteristic)

  forest <- forest_df(uni, multi, desc = desc)
  expect_true("Maternal age" %in% forest$Characteristic)
  expect_true("Smoking during pregnancy" %in% forest$Characteristic)
})

test_that("variable label attributes are used in stratified outputs", {
  df <- labelled_birthwt_data()

  strata_uni <- suppressMessages(
    stratified_uni_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "smoke", "ht"),
      stratifier = "race",
      approach = logit
    )
  )
  strata_multi <- suppressMessages(
    stratified_multi_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "smoke", "ht"),
      stratifier = "race",
      approach = logit
    )
  )

  expect_true("Maternal age" %in% strata_uni$table_display$Characteristic)
  expect_true("Smoking during pregnancy" %in% strata_uni$table_display$Characteristic)
  expect_true("Hypertension" %in% strata_multi$table_display$Characteristic)
  expect_equal(strata_uni$variable_labels[["smoke"]], "Smoking during pregnancy")
})

test_that("choice-style arguments accept bare values in descriptive_table", {
  df <- data.frame(
    group = factor(c("A", "A", "B", "B")),
    status = factor(c("No", "Yes", "No", NA))
  )

  tbl <- descriptive_table(
    df,
    exposures = status,
    by = group,
    percent = row,
    show_missing = no,
    show_overall = first,
    format = flextable,
    theme = shaded
  )

  expect_s3_class(tbl, "descriptive_table")
  expect_s3_class(tbl$table, "flextable")
  expect_equal(tbl$format, "flextable")
  expect_named(tbl$table_display, c("Characteristic", "is_header", "Overall", "A", "B"))
  expect_false("(Missing)" %in% trimws(tbl$table_display$Characteristic))
})

test_that("choice-style arguments still accept objects", {
  df <- data.frame(status = factor(c("No", "Yes", "No")))
  pct <- "column"
  overall <- "last"
  fmt <- "gt"
  thm <- c("header_shaded", "compact")

  tbl <- descriptive_table(
    df,
    exposures = "status",
    percent = pct,
    show_overall = overall,
    format = fmt,
    theme = thm
  )

  expect_s3_class(tbl, "gt_desc")
  expect_equal(tbl$format, "gt")
})

test_that("regression table options accept bare values", {
  df <- mtcars
  df$am <- factor(df$am)

  tbl <- uni_reg(
    data = df,
    outcome = mpg,
    exposures = hp,
    approach = linear,
    theme = minimal
  )

  expect_s3_class(tbl, "uni_reg")
  expect_equal(tbl$approach, "linear")
  expect_equal(tbl$format, "flextable")
  expect_s3_class(tbl$table, "flextable")
})

test_that("logbinomial is accepted bare and old hyphen spelling remains an alias", {
  df <- data.frame(
    y = c(0, 1, 0, 1, 0, 1),
    x = c(0, 0, 1, 1, 1, 0)
  )

  tbl_bare <- uni_reg(
    data = df,
    outcome = "y",
    exposures = "x",
    approach = logbinomial,
    format = gt
  )
  tbl_old <- uni_reg(
    data = df,
    outcome = "y",
    exposures = "x",
    approach = "log-binomial",
    format = gt
  )

  expect_s3_class(tbl_bare, "uni_reg")
  expect_equal(tbl_bare$approach, "logbinomial")
  expect_equal(tbl_old$approach, "logbinomial")
})

test_that("selection and diagnostics options accept bare values", {
  df <- mtcars

  sel <- select_models(
    data = df,
    outcome = "mpg",
    exposures = c("hp", "wt"),
    approach = linear,
    direction = forward
  )

  expect_type(sel, "list")

  conv <- check_convergence(
    data = data.frame(y = c(0, 1, 0, 1), x = c(1, 2, 3, 4)),
    exposures = "x",
    outcome = "y",
    approach = logit,
    format = tibble
  )

  expect_s3_class(conv, "data.frame")
  expect_equal(conv$Model, "logit")
})

test_that("interaction test option accepts bare values", {
  df <- data.frame(
    y = c(1, 2, 3, 4, 5, 6),
    x = c(1, 1, 2, 2, 3, 3),
    z = factor(c("A", "B", "A", "B", "A", "B"))
  )

  res <- interaction_models(
    data = df,
    outcome = "y",
    exposure = "x",
    effect_modifier = "z",
    approach = linear,
    test = LRT,
    verbose = FALSE
  )

  expect_type(res, "list")
  expect_equal(res$test, "Likelihood Ratio Test")
})

lung_cox_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )
}

test_that("cox_reg returns crude hazard ratio tables", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()
  attr(df$trt, "label") <- "Treatment group"

  res <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, karno),
    format = gt
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "cox_reg")
  expect_s3_class(res, "gt_cox")
  expect_s3_class(res$table, "gt_tbl")
  expect_equal(res$source, "cox_reg")
  expect_equal(res$approach, "cox")
  expect_false(res$adjusted_mode)
  expect_null(res$adjust_for)
  expect_equal(res$time, "time")
  expect_equal(res$event, "status")
  expect_named(res$models, c("trt", "celltype", "karno"))
  expect_true(all(vapply(res$models, inherits, logical(1), what = "coxph")))
  expect_true("N" %in% names(res$table_display))
  expect_true("HR (95% CI)" %in% names(res$table_display))
  expect_true("Treatment group" %in% res$table_display$Characteristic)
  expect_true("Ref." %in% res$table_display[["HR (95% CI)"]])
  expect_null(res$model_stats)
  expect_equal(
    attr(res$table_display, "row_exposure")[res$table_display$is_header],
    c("trt", "celltype", "karno")
  )
})

test_that("cox_reg crude tables display survival complete-case N consistently", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()
  df$karno[1:3] <- NA

  res <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, karno)
  )

  karno_header <- which(attr(res$table_display, "row_exposure") == "karno" &
                          res$table_display$is_header)[1]

  expect_equal(
    res$table_display$N[karno_header],
    sum(stats::complete.cases(df[, c("time", "status", "karno")]))
  )

  expect_s3_class(modify_table(res, remove_N = TRUE), "cox_reg")
  expect_false("N" %in% names(modify_table(res, remove_N = TRUE)$table_display))
})

test_that("cox_reg adjustment notes use display labels", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()
  attr(df$age, "label") <- "Patient age"
  attr(df$karno, "label") <- "Performance score"

  res <- cox_reg(
    data = df, time = time, event = status, exposures = trt,
    adjust_for = c(age, karno)
  )

  expect_true(any(res$footnotes == "Adjusted for Patient age and Performance score"))
  expect_false(any(res$footnotes == "Adjusted for age and karno"))
  expect_false(any(grepl("^Event variable:", res$footnotes)))
})

test_that("cox_reg explains how to show reference rows when hidden", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()

  expect_message(
    res <- cox_reg(
      data = df,
      time = time,
      event = status,
      exposures = trt,
      show_ref = FALSE
    ),
    "To display Ref., use `show_ref = TRUE`.",
    fixed = TRUE
  )

  expect_true(any(res$table_body$ref))
  expect_equal(nrow(res$table_display), 1L)
  expect_false("Ref." %in% res$table_display[["HR (95% CI)"]])
  expect_false(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                         fixed = TRUE)))
})

test_that("cox_reg returns adjusted hazard ratio tables and model stats", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()

  res <- cox_reg(
    data = df,
    time = "time",
    event = "status",
    exposures = c("trt", "celltype", "prior"),
    adjust_for = c("age", "karno"),
    model_stats = TRUE
  )

  expect_s3_class(res, "ft_cox")
  expect_s3_class(res$table, "flextable")
  expect_true(res$adjusted_mode)
  expect_equal(res$adjust_for, c("age", "karno"))
  expect_true("Adjusted HR (95% CI)" %in% names(res$table_display))
  expect_s3_class(res$model_stats, "data.frame")
  expect_equal(res$model_stats$model, c("trt", "celltype", "prior"))
  expect_true(all(c("AIC", "BIC", "logLik", "concordance", "events", "n") %in%
                    names(res$model_stats)))
  expect_true(all(is.finite(res$model_stats$AIC)))
  expect_true(all(res$model_stats$events > 0))
})

test_that("cox_reg supports a single multivariable Cox model", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()

  res <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior, age, karno),
    multivariable = TRUE,
    model_stats = TRUE
  )

  direct <- survival::coxph(
    survival::Surv(time, status) ~ trt + celltype + prior + age + karno,
    data = df,
    model = TRUE
  )

  expect_s3_class(res, "cox_reg")
  expect_false(res$adjusted_mode)
  expect_true(res$multivariable)
  expect_null(res$adjust_for)
  expect_named(res$models, "multivariable_model")
  expect_equal(res$models$multivariable_model$n, direct$n)
  expect_equal(res$models$multivariable_model$nevent, direct$nevent)
  expect_equal(stats::coef(res$models$multivariable_model), stats::coef(direct), tolerance = 1e-8)
  expect_equal(res$model_stats$model, "multivariable_model")
  expect_true("Adjusted HR (95% CI)" %in% names(res$table_display))
  expect_true(all(c("trt", "celltype", "prior", "age", "karno") %in% res$table_body$exposure))
  expect_equal(
    attr(res$table_display, "row_exposure")[res$table_display$is_header],
    c("trt", "celltype", "prior", "age", "karno")
  )
})

test_that("cox_reg supports interaction terms", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()
  attr(df$trt, "label") <- "Treatment"
  attr(df$prior, "label") <- "Prior therapy"

  adjusted <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = trt,
    adjust_for = c(age, karno),
    interaction = trt*prior
  )

  direct_adjusted <- survival::coxph(
    survival::Surv(time, status) ~ trt + age + karno + prior + trt:prior,
    data = df,
    model = TRUE
  )

  expect_equal(adjusted$interaction, "trt*prior")
  expect_true("Adjusted HR (95% CI)" %in% names(adjusted$table_display))
  expect_true(any(grepl(" x ", adjusted$table_body$level, fixed = TRUE)))
  expect_true(any(adjusted$table_body$level == "Treatment: Test treatment x Prior therapy: Yes"))
  expect_equal(stats::coef(adjusted$models$trt), stats::coef(direct_adjusted), tolerance = 1e-8)

  multivariable <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, age, karno),
    interaction = "trt*prior",
    multivariable = TRUE
  )

  direct_multi <- survival::coxph(
    survival::Surv(time, status) ~ trt + age + karno + prior + trt:prior,
    data = df,
    model = TRUE
  )

  expect_true(multivariable$multivariable)
  expect_equal(multivariable$interaction, "trt*prior")
  expect_true(any(grepl(":", names(stats::coef(multivariable$models$multivariable_model)), fixed = TRUE)))
  expect_equal(stats::coef(multivariable$models$multivariable_model), stats::coef(direct_multi), tolerance = 1e-8)

  expect_error(
    cox_reg(
      data = df,
      time = time,
      event = status,
      exposures = c(trt, prior),
      interaction = trt:prior
    ),
    "Use standard interaction syntax with"
  )
  expect_error(
    cox_reg(
      data = df,
      time = time,
      event = status,
      exposures = c(trt, prior),
      interaction = trt*prior
    ),
    "exposure-by-exposure mode"
  )
})

test_that("cox_reg accepts multivariate alias for multivariable mode", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()

  res <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, age, karno),
    multivariate = TRUE
  )

  expect_true(res$multivariable)
  expect_named(res$models, "multivariable_model")
  expect_true("Adjusted HR (95% CI)" %in% names(res$table_display))
})

test_that("cox_reg validates survival inputs", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()
  df$bad_event <- 1

  expect_error(
    cox_reg(df, time = "time", event = "bad_event", exposures = "age"),
    "`event` must include at least one censored observation"
  )
  expect_error(
    cox_reg(df, time = "time", event = "status", exposures = "missing_var"),
    "exposure variables were not found"
  )
  expect_error(
    cox_reg(df, time = "time", event = "status", exposures = "age", model_stats = NA),
    "`model_stats` must be TRUE or FALSE"
  )
  expect_error(
    cox_reg(df, time = "time", event = "status", exposures = "age", multivariable = NA),
    "`multivariable` must be TRUE or FALSE"
  )
  expect_error(
    cox_reg(df, time = "time", event = "status", exposures = "age", multivariate = NA),
    "`multivariate` must be TRUE or FALSE"
  )
  expect_error(
    cox_reg(
      df,
      time = "time",
      event = "status",
      exposures = "age",
      adjust_for = "karno",
      multivariable = TRUE
    ),
    "`adjust_for` is not used when `multivariable = TRUE`"
  )
})

test_that("cox_reg accepts zero follow-up times like survival::coxph", {
  skip_if_not_installed("survival")

  df <- data.frame(
    time = c(0, 2, 4, 6, 8, 10, 12, 14),
    status = c(1, 1, 0, 1, 0, 1, 0, 1),
    group = factor(c("A", "B", "A", "B", "A", "B", "A", "B"))
  )

  res <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = group
  )

  direct <- survival::coxph(survival::Surv(time, status) ~ group, data = df)
  direct_hr <- unname(exp(stats::coef(direct)[["groupB"]]))
  got_hr <- res$table_body$estimate[res$table_body$exposure == "group" &
                                      res$table_body$level == "B"]

  expect_s3_class(res, "cox_reg")
  expect_equal(got_hr, direct_hr, tolerance = 1e-8)
})

test_that("cox_reg uses exposure-specific complete cases for crude models", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()
  df$marker <- df$karno
  df$marker[seq_len(20)] <- NA

  res <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(age, marker),
    model_stats = TRUE
  )

  direct_age <- survival::coxph(survival::Surv(time, status) ~ age, data = df)
  direct_marker <- survival::coxph(survival::Surv(time, status) ~ marker, data = df)

  expect_equal(res$models$age$n, direct_age$n)
  expect_equal(res$models$marker$n, direct_marker$n)
  expect_gt(res$models$age$n, res$models$marker$n)
  expect_equal(res$model_stats$n, c(direct_age$n, direct_marker$n))
  expect_equal(stats::coef(res$models$age), stats::coef(direct_age), tolerance = 1e-8)
  expect_equal(stats::coef(res$models$marker), stats::coef(direct_marker), tolerance = 1e-8)
})

test_that("cox_reg accepts two-level factor exposures after complete-case filtering", {
  skip_if_not_installed("survival")

  df <- data.frame(
    time_90 = c(0, 1, 3, 5, 9, 12, 20, 40, 60, 90),
    cs_event = c(1, 1, 0, 1, 0, 1, 0, 1, 0, 1),
    within_admission = factor(
      c("No", "Yes", "No", "Yes", "No", "Yes", "No", "Yes", "No", "Yes"),
      levels = c("No", "Yes")
    )
  )

  res <- cox_reg(
    data = df,
    time = time_90,
    event = cs_event,
    exposures = within_admission
  )

  direct <- survival::coxph(
    survival::Surv(time_90, cs_event) ~ within_admission,
    data = df
  )
  direct_hr <- unname(exp(stats::coef(direct)[["within_admissionYes"]]))
  got_hr <- res$table_body$estimate[
    res$table_body$exposure == "within_admission" &
      res$table_body$level == "Yes"
  ]

  expect_s3_class(res, "cox_reg")
  expect_equal(got_hr, direct_hr, tolerance = 1e-8)
})

test_that("cox_reg rejects negative follow-up times", {
  skip_if_not_installed("survival")

  df <- data.frame(
    time = c(-1, 2, 4, 6),
    status = c(1, 1, 0, 1),
    age = c(50, 60, 55, 70)
  )

  expect_error(
    cox_reg(df, time = time, event = status, exposures = age),
    "`time` must contain non-negative follow-up times"
  )
})

test_that("cox_reg aligns correctly when merged with descriptive tables", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()
  df$celltype <- factor(
    df$celltype,
    levels = c("squamous", "smallcell", "adeno", "large"),
    labels = c("Squamous", "Small cell", "Adenocarcinoma", "Large cell")
  )
  attr(df$trt, "label") <- "Treatment group"
  attr(df$celltype, "label") <- "Cancer cell type"
  attr(df$karno, "label") <- "Karnofsky performance score"
  attr(df$age, "label") <- "Age"
  attr(df$prior, "label") <- "Prior therapy"

  desc <- descriptive_table(
    data = df,
    exposures = c("time", "status", "celltype", "karno", "age", "prior"),
    by = trt,
    statistic = c(time = median, karno = mean, age = mean),
    percent = column,
    show_overall = last
  )
  crude <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c("trt", "celltype", "karno", "age", "prior")
  )
  adjusted <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c("trt", "celltype", "prior"),
    adjust_for = c("age", "karno")
  )

  merged <- merge_tables(
    desc,
    modify_table(crude, header_labels = c(estimate = "Crude HR")),
    modify_table(adjusted, header_labels = c(estimate = "Adjusted HR")),
    spanners = c("Baseline profile", "Crude Cox", "Adjusted Cox")
  )

  out <- merged$table_display
  crude_col <- grep("^HR|^Crude", names(out), value = TRUE)[1]
  adj_col <- grep("^Adjusted", names(out), value = TRUE)[1]
  ch <- trimws(out$Characteristic)

  expect_match(out[ch == "Cancer cell type", crude_col], "^$")
  expect_match(out[ch == "Squamous", crude_col], "Ref\\.")
  expect_match(out[ch == "Small cell", crude_col], "2\\.72")
  expect_match(out[ch == "Adenocarcinoma", crude_col], "3\\.15")
  expect_match(out[ch == "Large cell", crude_col], "1\\.26")
  expect_match(out[ch == "Age", crude_col], "1\\.01")
  expect_match(out[ch == "No", crude_col], "Ref\\.")
  expect_match(out[ch == "Yes", crude_col], "0\\.87")
  expect_match(out[ch == "Test treatment", crude_col], "1\\.02")

  expect_match(out[ch == "Cancer cell type", adj_col], "^$")
  expect_match(out[ch == "Squamous", adj_col], "Ref\\.")
  expect_match(out[ch == "Small cell", adj_col], "2\\.06")
  expect_match(out[ch == "Adenocarcinoma", adj_col], "3\\.23")
  expect_match(out[ch == "Large cell", adj_col], "1\\.38")
  expect_match(out[ch == "No", adj_col], "Ref\\.")
  expect_match(out[ch == "Yes", adj_col], "0\\.96")
  expect_match(out[ch == "Test treatment", adj_col], "1\\.21")
})

test_that("cox_reg works with plotting and forest helpers", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("forestploter")

  df <- lung_cox_data()
  df$celltype <- factor(
    df$celltype,
    levels = c("squamous", "smallcell", "adeno", "large"),
    labels = c("Squamous", "Small cell", "Adenocarcinoma", "Large cell")
  )
  attr(df$trt, "label") <- "Treatment group"
  attr(df$celltype, "label") <- "Cancer cell type"
  attr(df$karno, "label") <- "Karnofsky performance score"
  attr(df$age, "label") <- "Age"
  attr(df$prior, "label") <- "Prior therapy"

  crude <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c("trt", "celltype", "karno", "age", "prior")
  )
  adjusted <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c("trt", "celltype", "prior"),
    adjust_for = c("age", "karno")
  )

  expect_s3_class(plot_reg(crude, log_x = TRUE), "ggplot")
  expect_s3_class(plot_reg(adjusted, log_x = TRUE), "ggplot")
  expect_s3_class(plot_reg_combine(crude, adjusted, log_x = TRUE), "patchwork")

  df_crude <- forest_df(crude)
  expect_true("HR (95% CI)" %in% names(df_crude))
  expect_equal(attr(df_crude, "forest_meta")$x_trans, "log")
  expect_equal(attr(df_crude, "forest_meta")$ref_line, 1)

  df_both <- forest_df(crude, adjusted)
  expect_true("HR (95% CI)" %in% names(df_both))
  expect_true("Adjusted HR (95% CI)" %in% names(df_both))
  expect_s3_class(forest_reg(df_both), "gtregression_forest")

  desc <- descriptive_table(
    data = df,
    exposures = c("time", "status", "celltype", "karno", "age", "prior"),
    by = trt,
    statistic = c(time = median, karno = mean, age = mean),
    percent = column,
    show_overall = last
  )
  df_desc <- forest_df(crude, adjusted, desc = desc)
  display_cols <- setdiff(names(df_desc), c("se_uni", "se_adj"))
  ch <- trimws(df_desc$Characteristic)

  expect_equal(df_desc[ch == "Karnofsky performance score", "Overall"], "58.6 (20.0)")
  expect_equal(df_desc[ch == "Age", "Standard treatment"], "57.5 (10.8)")
  expect_equal(df_desc[ch == "Small cell", "Standard treatment"], "30 (43.5%)")
  expect_equal(df_desc[ch == "Yes", "Overall"], "40 (29.2%)")
  expect_false(any(is.na(df_desc[display_cols])))
  expect_true(all(nchar(df_desc[[" "]]) >= 20))
  expect_true(all(nchar(df_desc[["  "]]) >= 20))
  expect_s3_class(forest_reg(df_desc), "gtregression_forest")
})

test_that("cox_reg supports stratified crude and adjusted Cox tables", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()

  crude <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(celltype, karno),
    stratifier = trt,
    format = gt
  )

  expect_s3_class(crude, "gtregression")
  expect_s3_class(crude, "stratified_cox_reg")
  expect_s3_class(crude, "cox_reg")
  expect_equal(crude$source, "stratified_cox_reg")
  expect_true(crude$stratified)
  expect_equal(crude$show_sample, "events")
  expect_false(crude$adjusted_mode)
  expect_equal(crude$by, "trt")
  expect_equal(crude$levels, c("Standard treatment", "Test treatment"))
  expect_false("..N__Standard treatment" %in% names(crude$table_display))
  expect_true("..Events__Standard treatment" %in% names(crude$table_display))
  expect_true("..eff__Standard treatment" %in% names(crude$table_display))
  expect_true("..p__Test treatment" %in% names(crude$table_display))
  expect_named(crude$per_stratum, c("Standard treatment", "Test treatment"))

  adjusted <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(celltype, prior),
    adjust_for = c(age, karno),
    stratifier = trt,
    model_stats = TRUE
  )

  expect_s3_class(adjusted, "stratified_cox_reg")
  expect_true(adjusted$adjusted_mode)
  expect_equal(adjusted$adjust_for, c("age", "karno"))
  expect_false("..N__Standard treatment" %in% names(adjusted$table_display))
  expect_true("..Events__Standard treatment" %in% names(adjusted$table_display))
  expect_true("..eff__Standard treatment" %in% names(adjusted$table_display))
  expect_s3_class(adjusted$model_stats, "data.frame")
  expect_true(all(c("stratum", "model", "AIC", "events", "n") %in% names(adjusted$model_stats)))
})

test_that("cox_reg explains hidden reference rows in stratified tables", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()

  expect_message(
    res <- cox_reg(
      data = df,
      time = time,
      event = status,
      exposures = prior,
      stratifier = trt,
      show_ref = FALSE
    ),
    "To display Ref., use `show_ref = TRUE`.",
    fixed = TRUE
  )

  expect_true(any(vapply(res$per_stratum, function(x) any(x$table_body$ref), logical(1))))
  eff_cols <- startsWith(names(res$table_display), "..eff__")
  expect_false("Ref." %in% unlist(res$table_display[eff_cols]))
  expect_false(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                         fixed = TRUE)))
})

test_that("cox_reg controls displayed sample columns in stratified tables", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()

  with_n <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = celltype,
    stratifier = trt,
    show_sample = n
  )
  expect_true("..N__Standard treatment" %in% names(with_n$table_display))
  expect_false("..Events__Standard treatment" %in% names(with_n$table_display))

  with_both <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = celltype,
    stratifier = trt,
    show_sample = both
  )
  expect_true("..N__Standard treatment" %in% names(with_both$table_display))
  expect_true("..Events__Standard treatment" %in% names(with_both$table_display))

  with_none <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = celltype,
    stratifier = trt,
    show_sample = none
  )
  expect_false(any(startsWith(names(with_none$table_display), "..N__")))
  expect_false(any(startsWith(names(with_none$table_display), "..Events__")))
  expect_true("..eff__Standard treatment" %in% names(with_none$table_display))

  expect_error(
    cox_reg(df, time = time, event = status, exposures = celltype, stratifier = trt, show_sample = "bad"),
    "show_sample"
  )
})

test_that("cox_reg supports stratified multivariable Cox tables", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()

  res <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(celltype, prior, age, karno),
    stratifier = trt,
    multivariable = TRUE
  )

  expect_s3_class(res, "stratified_cox_reg")
  expect_true(res$multivariable)
  expect_equal(res$show_sample, "events")
  expect_false(res$adjusted_mode)
  expect_named(res$models, c("Standard treatment", "Test treatment"))
  expect_named(res$models[["Standard treatment"]], "multivariable_model")
  expect_false("..N__Standard treatment" %in% names(res$table_display))
  expect_true("..Events__Standard treatment" %in% names(res$table_display))
  expect_true("..eff__Standard treatment" %in% names(res$table_display))
  expect_true(any(trimws(res$table_display$Characteristic) %in% c("Age", "age")))
})

test_that("cox_reg validates stratifier inputs and supports stratified forest data", {
  skip_if_not_installed("survival")

  df <- lung_cox_data()

  expect_error(
    cox_reg(df, time = time, event = status, exposures = trt, stratifier = trt),
    "`stratifier` cannot also be used"
  )
  expect_error(
    cox_reg(df, time = time, event = status, exposures = trt, adjust_for = age, stratifier = age),
    "`stratifier` cannot also be used"
  )
  expect_error(
    cox_reg(df, time = time, event = status, exposures = trt, stratifier = "time"),
    "`stratifier` cannot also be used"
  )
  expect_error(
    cox_reg(df, time = time, event = status, exposures = trt, interaction = trt*prior, stratifier = prior),
    "`stratifier` cannot also be used"
  )

  stratified <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(celltype, karno),
    stratifier = trt
  )

  strat_plot <- plot_reg(stratified)
  expect_s3_class(strat_plot, "ggplot")
  expect_true("stratum" %in% names(strat_plot$data))
  expect_error(plot_reg_combine(stratified, stratified), "does not support stratified")

  forest_data <- forest_df(stratified)
  expect_s3_class(forest_data, "data.frame")
  expect_true(all(paste0(
    c("Standard treatment", "Test treatment"),
    "\nHR (95% CI)"
  ) %in% names(forest_data)))
  expect_false(any(forest_data$Characteristic %in% c(
    "trt = Standard treatment",
    "trt = Test treatment"
  )))
  expect_equal(length(attr(forest_data, "est")), nrow(forest_data))
  expect_equal(attr(forest_data, "forest_meta")$stratifier, "trt")
  expect_true(attr(forest_data, "forest_meta")$side_by_side_strata)
  expect_equal(length(attr(forest_data, "forest_estimates")$est), 2L)
})
test_that("cox_reg displays every level of character predictors", {
  set.seed(412)
  d <- data.frame(
    time = stats::rexp(180),
    event = stats::rbinom(180, 1, 0.7),
    classification = rep(c("Associated", "Elementary", "Combined"), each = 60)
  )

  res <- cox_reg(d, time, event, classification, format = "gt")
  rows <- res$table_body[res$table_body$exposure == "classification", ]

  expect_equal(rows$level, c("Associated", "Combined", "Elementary"))
  expect_equal(sum(rows$ref), 1L)
  expect_equal(nrow(rows), 3L)
})

birthwt_merge_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
      ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes"))
    )
}

flextable_header_text <- function(x) {
  vapply(x$header$content$data, function(part) part$txt[1], character(1))
}

test_that("merge_tables combines native gtregression objects", {
  df <- birthwt_merge_data()

  desc_tbl <- descriptive_table(
    data = df,
    exposures = c("age", "lwt", "race", "smoke", "ht"),
    by = "low",
    show_overall = "last",
    format = gt
  )
  uni_tbl <- uni_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "lwt", "race", "smoke", "ht"),
    approach = logit,
    format = gt
  )

  merged <- merge_tables(
    desc_tbl,
    uni_tbl,
    spanners = c("Descriptive", "Univariable"),
    theme = shaded,
    format = gt
  )

  expect_s3_class(merged, "gtregression")
  expect_s3_class(merged, "merged_table")
  expect_s3_class(merged, "gt_merge")
  expect_s3_class(merged$table, "gt_tbl")
  expect_equal(merged$spanners, c("Descriptive", "Univariable"))
  expect_equal(merged$engine, "gt")
  expect_equal(merged$part_sources, c("descriptive_table", "uni_reg"))
  expect_true(all(c("Characteristic", "is_header") %in% names(merged$table_display)))
  expect_true(any(trimws(merged$table_display$Characteristic) == "smoke"))
  expect_true(any(grepl("OR", names(merged$table_display), fixed = TRUE)))
  expect_true(any(grepl("percentages are by column", merged$footnotes)))
  expect_true(any(grepl("OR = Odds Ratio", merged$footnotes, fixed = TRUE)))

  gt_labels <- as.character(merged$table$`_boxhead`$column_label)
  expect_false(any(grepl("_p[0-9]+$", gt_labels)))
  expect_true(all(c("Normal BW", "Low BW", "Overall", "N", "OR (95% CI)", "p-value") %in%
                    gt_labels))
})

test_that("merge_tables defaults to flextable independently of input format", {
  skip_if_not_installed("flextable")

  df <- birthwt_merge_data()
  uni_gt <- uni_reg(
    df,
    outcome = "low",
    exposures = "smoke",
    approach = logit,
    format = gt
  )
  multi_gt <- multi_reg(
    df,
    outcome = "low",
    exposures = "smoke",
    approach = logit,
    format = gt
  )

  merged <- merge_tables(uni_gt, multi_gt)

  expect_equal(merged$format, "flextable")
  expect_s3_class(merged, "ft_merge")
  expect_s3_class(merged$table, "flextable")
})

test_that("merge_tables aligns univariable and adjusted multivariable tables", {
  df <- birthwt_merge_data()

  exposures <- c("ui", "smoke", "ht")
  uni_tbl <- uni_reg(
    data = df,
    outcome = "low",
    exposures = exposures,
    approach = logit
  )
  multi_tbl <- multi_reg(
    data = df,
    outcome = "low",
    exposures = exposures,
    adjust_for = c("age", "lwt"),
    approach = logit
  )

  merged <- merge_tables(uni_tbl, multi_tbl, spanners = c("Crude", "Adjusted"))

  expect_equal(merged$spanners, c("Crude", "Adjusted"))
  expect_equal(merged$part_sources, c("uni_reg", "multi_reg"))
  expect_true(any(trimws(merged$table_display$Characteristic) == "Yes"))
  expect_true(any(grepl("Adjusted.OR", names(merged$table_display), fixed = TRUE)))
  expect_true(any(grepl("Adjusted for age and lwt", merged$footnotes, fixed = TRUE)))
  expect_equal(
    trimws(merged$table_display$Characteristic[merged$table_display$is_header]),
    exposures
  )
})

test_that("merge_tables carries stored multivariable footnotes unchanged", {
  df <- birthwt_merge_data()
  attr(df$age, "label") <- "Maternal age"
  attr(df$lwt, "label") <- "Maternal weight"
  uni_tbl <- uni_reg(df, outcome = "low", exposures = c("smoke", "ht"), approach = logit)
  multi_tbl <- multi_reg(
    df,
    outcome = "low",
    exposures = c("smoke", "ht"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )
  merged <- merge_tables(uni_tbl, multi_tbl, spanners = c("Crude", "Adjusted"))

  expect_true(any(multi_tbl$footnotes == "Adjusted for Maternal age and Maternal weight"))
  expect_true(all(multi_tbl$footnotes %in% merged$footnotes))
  expect_equal(
    merged$footnotes[grepl("^Adjusted for ", merged$footnotes)],
    multi_tbl$footnotes[grepl("^Adjusted for ", multi_tbl$footnotes)]
  )
})

test_that("merge_tables supports three-table merges and default spanners", {
  df <- birthwt_merge_data()

  desc_tbl <- descriptive_table(df, exposures = c("age", "smoke"), by = "low")
  uni_tbl <- uni_reg(df, outcome = "low", exposures = c("age", "smoke"), approach = logit)
  multi_tbl <- multi_reg(df, outcome = "low", exposures = c("age", "smoke"), approach = logit)

  merged <- merge_tables(desc_tbl, uni_tbl, multi_tbl)

  expect_s3_class(merged, "merged_table")
  expect_equal(merged$spanners, c("Table 1", "Table 2", "Table 3"))
  expect_equal(merged$part_sources, c("descriptive_table", "uni_reg", "multi_reg"))
  expect_true(ncol(merged$table_display) > ncol(desc_tbl$table_display))
})

test_that("merge_tables warns for mixed binary rows in descriptive-regression merges", {
  df <- birthwt_merge_data()

  desc_tbl <- descriptive_table(
    df,
    exposures = "smoke",
    by = "low",
    show_dichotomous = "all_levels"
  )
  uni_compact <- suppressMessages(uni_reg(
    df,
    outcome = "low",
    exposures = "smoke",
    approach = logit,
    show_ref = FALSE
  ))
  multi_compact <- suppressMessages(multi_reg(
    df,
    outcome = "low",
    exposures = "smoke",
    approach = logit,
    show_ref = FALSE
  ))

  expect_warning(
    suppressMessages(merge_tables(desc_tbl, uni_compact, multi_compact)),
    "show_dichotomous = \"all_levels\".*show_ref = TRUE"
  )

  uni_expanded <- uni_reg(
    df,
    outcome = "low",
    exposures = "smoke",
    approach = logit,
    show_ref = TRUE
  )
  multi_expanded <- multi_reg(
    df,
    outcome = "low",
    exposures = "smoke",
    approach = logit,
    show_ref = TRUE
  )

  expect_no_warning(suppressMessages(
    merge_tables(desc_tbl, uni_expanded, multi_expanded)
  ))
})

test_that("merge_tables aligns compact binary regression rows to descriptive levels", {
  df <- birthwt_merge_data()

  desc_tbl <- descriptive_table(
    df,
    exposures = c("age", "smoke"),
    by = "low",
    format = gt
  )
  uni_tbl <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = logit,
    format = gt
  )

  compact_uni <- uni_tbl
  display <- uni_tbl$table_display
  body <- uni_tbl$table_body

  smoke_header <- which(display$Characteristic == "smoke" & display$is_header)[1]
  smoke_yes <- which(trimws(display$Characteristic) == "Yes" & !display$is_header)[1]
  age_row <- which(display$Characteristic == "age" & display$is_header)[1]
  effect_cols <- setdiff(names(display), c("Characteristic", "is_header"))

  display[smoke_header, effect_cols] <- display[smoke_yes, effect_cols]
  compact_uni$table_display <- display[c(age_row, smoke_header), , drop = FALSE]
  compact_uni$table_body <- body[
    body$exposure == "age" | (body$exposure == "smoke" & body$level == "Yes"),
    ,
    drop = FALSE
  ]

  merged <- NULL
  expect_warning(
    expect_message(
      merged <- merge_tables(
        desc_tbl,
        compact_uni,
        spanners = c("Descriptive", "Crude")
      ),
      "show_ref = TRUE"
    ),
    "binary display mode mismatch"
  )

  out <- merged$table_display
  or_cols <- grep("OR", names(out), value = TRUE)
  p_cols <- grep("p.value", names(out), value = TRUE)
  smoke_header_row <- which(trimws(out$Characteristic) == "smoke" & (out$is_header %in% TRUE))
  smoke_yes_row <- which(trimws(out$Characteristic) == "Yes" & (out$is_header %in% FALSE))

  expect_length(smoke_header_row, 1L)
  expect_length(smoke_yes_row, 1L)
  expect_equal(out[smoke_header_row, or_cols], "")
  expect_equal(out[smoke_header_row, p_cols], "")
  expect_true(any(nzchar(unlist(out[smoke_yes_row, or_cols], use.names = FALSE))))
  expect_false(any(trimws(out$Characteristic) == "smoke" & (out$is_header %in% FALSE)))
})

test_that("merge_tables keeps compact descriptive and regression binary rows aligned", {
  df <- birthwt_merge_data()

  desc_tbl <- suppressMessages(
    descriptive_table(
      df,
      exposures = c("age", "smoke"),
      by = "low",
      show_dichotomous = "single_row",
      value = list(smoke = "Yes"),
      format = gt
    )
  )
  uni_tbl <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = logit,
    format = gt
  )

  compact_uni <- uni_tbl
  display <- uni_tbl$table_display
  body <- uni_tbl$table_body

  smoke_header <- which(display$Characteristic == "smoke" & display$is_header)[1]
  smoke_yes <- which(trimws(display$Characteristic) == "Yes" & !display$is_header)[1]
  age_row <- which(display$Characteristic == "age" & display$is_header)[1]
  effect_cols <- setdiff(names(display), c("Characteristic", "is_header"))

  display[smoke_header, effect_cols] <- display[smoke_yes, effect_cols]
  compact_uni$table_display <- display[c(age_row, smoke_header), , drop = FALSE]
  compact_uni$table_body <- body[
    body$exposure == "age" | (body$exposure == "smoke" & body$level == "Yes"),
    ,
    drop = FALSE
  ]

  merged <- NULL
  expect_no_message(
    merged <- merge_tables(desc_tbl, compact_uni, spanners = c("Descriptive", "Crude"))
  )

  out <- merged$table_display
  smoke_rows <- which(trimws(out$Characteristic) == "smoke")
  or_cols <- grep("OR", names(out), value = TRUE)

  expect_length(smoke_rows, 1L)
  expect_true(out$is_header[smoke_rows])
  expect_true(any(nzchar(unlist(out[smoke_rows, or_cols], use.names = FALSE))))
})

test_that("merge_tables supports flextable output", {
  skip_if_not_installed("flextable")

  df <- birthwt_merge_data()

  uni_tbl <- uni_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = logit,
    format = flextable
  )
  multi_tbl <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = logit,
    format = flextable
  )

  merged <- merge_tables(uni_tbl, multi_tbl, theme = striped)

  expect_s3_class(merged, "ft_merge")
  expect_s3_class(merged$table, "flextable")
  expect_equal(merged$engine, "flextable")

  header_text <- flextable_header_text(merged$table)
  expect_false(any(grepl("_p[0-9]+$", header_text)))
  expect_true(all(c("N", "OR (95% CI)", "p-value", "Adjusted OR (95% CI)") %in%
                    header_text))
})

test_that("merge_tables validates inputs", {
  df <- birthwt_merge_data()

  uni_gt <- uni_reg(df, outcome = "low", exposures = "smoke", approach = logit, format = gt)
  multi_gt <- multi_reg(df, outcome = "low", exposures = "smoke", approach = logit, format = gt)

  expect_error(merge_tables(uni_gt), "at least two tables")
  expect_error(merge_tables(data.frame(x = 1), multi_gt), "All inputs must be outputs")
  expect_error(merge_tables(uni_gt, multi_gt, spanners = "Only one"), "Length of `spanners`")

  skip_if_not_installed("flextable")
  uni_ft <- uni_reg(
    df,
    outcome = "low",
    exposures = "smoke",
    approach = logit,
    format = flextable
  )
  expect_error(merge_tables(uni_gt, uni_ft), "same engine")
})

test_that("merge_tables rejects malformed package-like inputs", {
  bad <- structure(
    list(
      table_display = data.frame(Characteristic = "age", is_header = TRUE),
      table = list()
    ),
    class = "gtregression"
  )

  good <- uni_reg(
    data = birthwt_merge_data(),
    outcome = "low",
    exposures = "age",
    approach = logit
  )

  expect_error(merge_tables(bad, good), "same engine")
})

#' Check collinearity using VIF for fitted models
#'
#' Computes Variance Inflation Factors (VIF) for fitted multivariable models
#' returned by \code{multi_reg()} or related functions.
#'
#' @param model A fitted model object returned by \code{multi_reg()},
#'   \code{stratified_multi_reg()}, or compatible \code{gtregression}
#'   functions. Univariable model objects are rejected because VIF is not
#'   applicable.
#' @param format Output format. One of \code{"flextable"} (default),
#'   \code{"gt"}, or \code{"tibble"}. Use \code{format = "tibble"} to preserve
#'   the original tibble or nested-list output.
#'
#' @return For multivariable models, a tibble if a single fitted model is
#' present, or a named list of tibbles if multiple fitted models are present.
#' With \code{format = "gt"} or \code{format = "flextable"}, leaf tibbles are
#' converted to formatted tables while preserving any list nesting. The tibble
#' contains:
#' \describe{
#'   \item{\code{Variable}}{Model term.}
#'   \item{\code{VIF}}{Variance inflation factor. For multi-degree-of-freedom
#'   terms, this is the adjusted GVIF: \code{GVIF^(1 / (2 * Df))}.}
#'   \item{\code{Interpretation}}{Simple interpretation based on common cut
#'   points: no collinearity, moderate, or high.}
#' }
#'
#' For univariate models, an error is raised indicating that VIF is not applicable.
#'
#' @details If the \pkg{car} package is installed, \code{check_collinearity()}
#' uses \code{car::vif()}. Otherwise, it computes VIF/GVIF from the fitted model
#' matrix so that diagnostics remain available without an additional dependency.
#'
#' @importFrom stats complete.cases cor model.matrix terms
#' @export
check_collinearity <- function(model,
                               format = c("flextable", "gt", "tibble")) {
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  format <- match.arg(format, c("flextable", "gt", "tibble"))

  valid_sources <- c(
    "uni_reg", "multi_reg", "uni_reg_nbin", "multi_reg_nbin",
    "stratified_multi_reg"
  )

  if (!inherits(model, "gtregression")) {
    stop("Input must be a gtregression model object.", call. = FALSE)
  }

  model_source <- model$source
  model_list <- model$models

  if (is.null(model_source) || !(model_source %in% valid_sources)) {
    stop(
      "Input must be a fitted model from uni_reg, multi_reg, uni_reg_nbin, ",
      "multi_reg_nbin, or stratified_multi_reg.",
      call. = FALSE
    )
  }

  if (is.null(model_list) || !length(model_list)) {
    stop("Model list not found in object. Cannot compute VIF.", call. = FALSE)
  }

  if (model_source %in% c("uni_reg", "uni_reg_nbin")) {
    stop(
      "VIF is not applicable for univariate models. ",
      "Use multi_reg() to check collinearity among predictors.",
      call. = FALSE
    )
  }

  .empty_vif_table <- function() {
    tibble::tibble(
      Variable = character(0),
      VIF = numeric(0),
      Interpretation = character(0)
    )
  }

  .interpret_vif <- function(vif) {
    dplyr::case_when(
      is.na(vif) ~ NA_character_,
      vif < 2 ~ "No collinearity",
      vif >= 2 & vif < 5 ~ "Moderate",
      vif >= 5 ~ "High"
    )
  }

  .from_car_vif <- function(vif_vals) {
    if (is.matrix(vif_vals) || is.data.frame(vif_vals)) {
      vif_df <- as.data.frame(vif_vals)
      if (all(c("GVIF", "Df") %in% names(vif_df))) {
        vals <- vif_df$GVIF^(1 / (2 * vif_df$Df))
      } else {
        vals <- vif_df[[1]]
      }
      names(vals) <- rownames(vif_df)
      return(vals)
    }
    vif_vals
  }

  .manual_vif <- function(fit_model) {
    term_labels <- attr(stats::terms(fit_model), "term.labels")
    if (length(term_labels) < 2) return(stats::setNames(numeric(0), character(0)))

    mm <- stats::model.matrix(fit_model)
    assign <- attr(mm, "assign")
    keep <- assign > 0
    mm <- mm[, keep, drop = FALSE]
    assign <- assign[keep]

    if (ncol(mm) < 2) return(stats::setNames(numeric(0), character(0)))

    complete <- stats::complete.cases(mm)
    mm <- mm[complete, , drop = FALSE]
    non_constant <- apply(mm, 2, function(x) stats::var(x, na.rm = TRUE) > 0)
    mm <- mm[, non_constant, drop = FALSE]
    assign <- assign[non_constant]

    if (ncol(mm) < 2) return(stats::setNames(numeric(0), character(0)))

    cmat <- stats::cor(mm)
    if (anyNA(cmat)) return(NULL)

    out <- vapply(seq_along(term_labels), function(i) {
      cols <- which(assign == i)
      other <- which(assign != i)
      if (!length(cols) || !length(other)) return(NA_real_)

      if (length(cols) == 1L) {
        fit <- stats::lm.fit(x = cbind(1, mm[, other, drop = FALSE]),
                             y = mm[, cols])
        rss <- sum(fit$residuals^2)
        y <- mm[, cols]
        tss <- sum((y - mean(y))^2)
        if (!is.finite(tss) || tss <= 0) return(NA_real_)
        r_squared <- max(0, min(1, 1 - rss / tss))
        if (isTRUE(all.equal(r_squared, 1))) return(Inf)
        return(1 / (1 - r_squared))
      }

      det_term <- det(cmat[cols, cols, drop = FALSE])
      det_other <- det(cmat[other, other, drop = FALSE])
      det_all <- det(cmat)
      if (!is.finite(det_term) || !is.finite(det_other) ||
          !is.finite(det_all) || det_term <= 0 || det_other <= 0 ||
          det_all <= 0) {
        return(NA_real_)
      }
      gvif <- (det_term * det_other) / det_all
      gvif^(1 / (2 * length(cols)))
    }, numeric(1))

    stats::setNames(out, term_labels)
  }

  .make_vif_table <- function(fit_model, model_name = NULL) {
    if (!inherits(fit_model, c("glm", "lm", "negbin"))) {
      stop("Unsupported model type for VIF calculation.", call. = FALSE)
    }

    term_labels <- attr(stats::terms(fit_model), "term.labels")

    if (length(term_labels) < 2) {
      return(.empty_vif_table())
    }

    vif_vals <- NULL
    vif_fun <- getOption("gtregression.vif_fun", NULL)
    if (is.function(vif_fun)) {
      vif_vals <- tryCatch(
        .from_car_vif(vif_fun(fit_model)),
        error = function(e) NULL
      )
    } else if (requireNamespace("car", quietly = TRUE)) {
      vif_vals <- tryCatch(
        .from_car_vif(car::vif(fit_model)),
        error = function(e) NULL
      )
    }
    if (is.null(vif_vals)) {
      vif_vals <- tryCatch(
        .manual_vif(fit_model),
        error = function(e) NULL
      )
    }

    if (is.null(vif_vals)) {
      return(
        tibble::tibble(
          Variable = NA_character_,
          VIF = NA_real_,
          Interpretation = "VIF could not be computed"
        )
      )
    }

    vif_names <- names(vif_vals)
    vif_vals <- as.numeric(vif_vals)
    names(vif_vals) <- vif_names

    tibble::tibble(
      Variable = names(vif_vals),
      VIF = round(as.numeric(vif_vals), 2),
      Interpretation = .interpret_vif(as.numeric(vif_vals))
    )
  }

  # multi_reg default mode: usually list(multivariable_model = fit)
  # multi_reg adjusted mode: named list of fits
  # stratified_multi_reg: nested list by stratum -> list(multivariable_model=fit) or more

  if (model_source == "multi_reg") {
    out <- lapply(names(model_list), function(nm) {
      .make_vif_table(model_list[[nm]], nm)
    })
    names(out) <- names(model_list)

    if (length(out) == 1) {
      out <- out[[1]]
      if (format == "tibble") {
        return(out)
      }
      return(.format_vif_output(out, format = format))
    }
    if (format == "tibble") {
      return(out)
    }
    return(.format_vif_output(out, format = format))
  }

  if (model_source == "multi_reg_nbin") {
    out <- lapply(names(model_list), function(nm) {
      .make_vif_table(model_list[[nm]], nm)
    })
    names(out) <- names(model_list)

    if (length(out) == 1) {
      out <- out[[1]]
      if (format == "tibble") {
        return(out)
      }
      return(.format_vif_output(out, format = format))
    }
    if (format == "tibble") {
      return(out)
    }
    return(.format_vif_output(out, format = format))
  }

  if (model_source == "stratified_multi_reg") {
    out <- lapply(names(model_list), function(stratum_name) {
      stratum_models <- model_list[[stratum_name]]

      vif_stratum <- lapply(names(stratum_models), function(nm) {
        .make_vif_table(stratum_models[[nm]], nm)
      })
      names(vif_stratum) <- names(stratum_models)

      if (length(vif_stratum) == 1) {
        vif_stratum[[1]]
      } else {
        vif_stratum
      }
    })
    names(out) <- names(model_list)
    if (format == "tibble") {
      return(out)
    }
    return(.format_vif_output(out, format = format))
  }

  stop("Unsupported model source for VIF calculation.", call. = FALSE)
}

#' Format VIF output recursively
#' @keywords internal
#' @noRd
.format_vif_output <- function(x, format = c("flextable", "gt")) {
  format <- match.arg(format, c("flextable", "gt"))

  if (is.data.frame(x)) {
    return(.build_check_collinearity_table(x, format = format))
  }

  if (is.list(x)) {
    return(lapply(x, .format_vif_output, format = format))
  }

  x
}

#' Build formatted VIF table
#' @keywords internal
#' @noRd
.build_check_collinearity_table <- function(result,
                                            format = c("flextable", "gt")) {
  format <- match.arg(format, c("flextable", "gt"))
  note <- paste(
    "Screening aid only; interpret VIF with model purpose, coding choices,",
    "sample size, and subject-matter knowledge."
  )

  display <- result |>
    dplyr::mutate(
      VIF = dplyr::if_else(
        is.na(.data$VIF),
        "",
        formatC(.data$VIF, digits = 2, format = "f")
      )
    )

  if (format == "gt") {
    return(
      gt::gt(display) |>
        gt::tab_header(title = "Collinearity check") |>
        gt::cols_align(align = "left", columns = c("Variable", "Interpretation")) |>
        gt::cols_align(align = "center", columns = "VIF") |>
        gt::tab_style(
          style = gt::cell_text(weight = "bold"),
          locations = gt::cells_column_labels()
        ) |>
        gt::tab_style(
          style = gt::cell_fill(color = "#e7f5ec"),
          locations = gt::cells_body(rows = .data$Interpretation == "No collinearity")
        ) |>
        gt::tab_style(
          style = gt::cell_fill(color = "#fff4d6"),
          locations = gt::cells_body(rows = .data$Interpretation == "Moderate")
        ) |>
        gt::tab_style(
          style = gt::cell_fill(color = "#fde2e2"),
          locations = gt::cells_body(rows = .data$Interpretation == "High")
        ) |>
        gt::tab_source_note(gt::md(note)) |>
        .compact_gt_source_notes()
    )
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Collinearity check")
  ft <- flextable::align(ft, j = c("Variable", "Interpretation"), align = "left", part = "all")
  ft <- flextable::align(ft, j = "VIF", align = "center", part = "all")
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  ft <- flextable::bg(
    ft,
    i = which(display$Interpretation == "No collinearity"),
    bg = "#e7f5ec",
    part = "body"
  )
  ft <- flextable::bg(
    ft,
    i = which(display$Interpretation == "Moderate"),
    bg = "#fff4d6",
    part = "body"
  )
  ft <- flextable::bg(
    ft,
    i = which(display$Interpretation == "High"),
    bg = "#fde2e2",
    part = "body"
  )
  ft <- flextable::add_footer_lines(ft, values = note)
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

collinearity_birthwt_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
    )
}

test_that("check_collinearity returns VIF table for numeric predictors", {
  fit <- multi_reg(
    data = mtcars,
    outcome = "mpg",
    exposures = c("hp", "wt"),
    approach = linear
  )

  out <- check_collinearity(fit, format = tibble)

  expect_s3_class(out, "tbl_df")
  expect_named(out, c("Variable", "VIF", "Interpretation"))
  expect_equal(out$Variable, c("hp", "wt"))
  expect_true(all(is.finite(out$VIF)))
  expect_true(all(out$VIF >= 1))
  expect_true(all(out$Interpretation %in% c("No collinearity", "Moderate", "High")))
})

test_that("check_collinearity handles factor predictors with adjusted GVIF values", {
  df <- mtcars
  df$cyl <- factor(df$cyl)
  df$gear <- factor(df$gear)

  fit <- multi_reg(
    data = df,
    outcome = "mpg",
    exposures = c("cyl", "gear"),
    approach = linear
  )

  out <- check_collinearity(fit, format = tibble)

  expect_s3_class(out, "tbl_df")
  expect_equal(out$Variable, c("cyl", "gear"))
  expect_true(all(is.finite(out$VIF) | is.na(out$VIF)))
  expect_true(all(out$VIF >= 1 | is.na(out$VIF)))
})

test_that("check_collinearity returns list for adjusted multivariable models", {
  df <- collinearity_birthwt_data()

  fit <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("smoke", "ht"),
    adjust_for = c("age", "lwt", "race"),
    approach = logit
  )

  out <- check_collinearity(fit, format = tibble)

  expect_type(out, "list")
  expect_equal(names(out), c("smoke", "ht"))
  expect_true(all(vapply(out, inherits, logical(1), what = "tbl_df")))
  expect_true(all(c("age", "lwt", "race") %in% out$smoke$Variable))
})

test_that("check_collinearity handles stratified multivariable models", {
  df <- collinearity_birthwt_data()

  fit <- stratified_multi_reg(
    data = df,
    outcome = "bwt",
    exposures = c("age", "lwt", "smoke"),
    stratifier = "race",
    approach = linear
  )

  out <- check_collinearity(fit, format = tibble)

  expect_type(out, "list")
  expect_equal(names(out), levels(df$race))
  expect_true(all(vapply(out, inherits, logical(1), what = "tbl_df")))
  expect_true(all(vapply(out, function(x) all(c("Variable", "VIF", "Interpretation") %in% names(x)),
                         logical(1))))
})

test_that("check_collinearity rejects univariable and unsupported objects clearly", {
  uni <- uni_reg(
    data = mtcars,
    outcome = "mpg",
    exposures = c("hp", "wt"),
    approach = linear
  )

  expect_error(
    check_collinearity(uni),
    "VIF is not applicable for univariate models"
  )
  expect_error(
    check_collinearity(stats::lm(mpg ~ hp + wt, data = mtcars)),
    "Input must be a gtregression model object"
  )

  bad <- list(source = "unknown", models = list(stats::lm(mpg ~ hp + wt, data = mtcars)))
  class(bad) <- c("gtregression", "list")
  expect_error(
    check_collinearity(bad),
    "Input must be a fitted model"
  )
})

test_that("check_collinearity validates model list and inner model types", {
  fit <- multi_reg(
    data = mtcars,
    outcome = "mpg",
    exposures = c("hp", "wt"),
    approach = linear
  )

  missing_models <- fit
  missing_models$models <- NULL
  expect_error(
    check_collinearity(missing_models),
    "Model list not found"
  )

  bad_inner <- fit
  bad_inner$models <- list(multivariable_model = structure(list(), class = "not_a_model"))
  expect_error(
    check_collinearity(bad_inner),
    "Unsupported model type"
  )
})

test_that("check_collinearity returns empty tibble when fewer than two predictors are present", {
  one <- multi_reg(
    data = mtcars,
    outcome = "mpg",
    exposures = "hp",
    approach = linear
  )

  out <- check_collinearity(one, format = tibble)

  expect_s3_class(out, "tbl_df")
  expect_equal(nrow(out), 0L)
  expect_named(out, c("Variable", "VIF", "Interpretation"))
})

test_that("check_collinearity reports when VIF cannot be computed", {
  fit_model <- list(terms = stats::terms(y ~ x1 + x2))
  class(fit_model) <- "lm"
  obj <- list(
    source = "multi_reg",
    models = list(multivariable_model = fit_model)
  )
  class(obj) <- c("gtregression", "multi_reg", "list")

  out <- check_collinearity(obj, format = tibble)

  expect_s3_class(out, "tbl_df")
  expect_equal(out$Variable, NA_character_)
  expect_true(is.na(out$VIF))
  expect_equal(out$Interpretation, "VIF could not be computed")
})

test_that("check_collinearity handles simulated car::vif vector and GVIF matrix outputs", {
  fit <- multi_reg(
    data = mtcars,
    outcome = "mpg",
    exposures = c("hp", "wt"),
    approach = linear
  )

  old <- options(gtregression.vif_fun = function(model) c(hp = 1.5, wt = 6.2))
  on.exit(options(old), add = TRUE)
  vector_out <- check_collinearity(fit, format = tibble)

  expect_equal(vector_out$Variable, c("hp", "wt"))
  expect_equal(vector_out$VIF, c(1.5, 6.2))
  expect_equal(vector_out$Interpretation, c("No collinearity", "High"))

  options(gtregression.vif_fun = function(model) {
    out <- matrix(c(4, 1, 9, 2), ncol = 2, byrow = TRUE)
    colnames(out) <- c("GVIF", "Df")
    rownames(out) <- c("hp", "wt")
    out
  })
  gvif_out <- check_collinearity(fit, format = tibble)

  expect_equal(gvif_out$Variable, c("hp", "wt"))
  expect_equal(gvif_out$VIF, c(2, round(9^(1 / 4), 2)))

  options(gtregression.vif_fun = function(model) {
    out <- matrix(c(2, 3), ncol = 1)
    rownames(out) <- c("hp", "wt")
    out
  })
  matrix_out <- check_collinearity(fit, format = tibble)

  expect_equal(matrix_out$Variable, c("hp", "wt"))
  expect_equal(matrix_out$VIF, c(2, 3))
})

test_that("check_collinearity handles legacy multi_reg_nbin source", {
  data("quine", package = "MASS")
  fit_model <- MASS::glm.nb(Days ~ Eth + Sex, data = quine)
  obj <- list(
    source = "multi_reg_nbin",
    models = list(multivariable_model = fit_model)
  )
  class(obj) <- c("gtregression", "multi_reg_nbin", "list")

  out <- check_collinearity(obj, format = tibble)

  expect_s3_class(out, "tbl_df")
  expect_true(all(c("Eth", "Sex") %in% out$Variable))
})

test_that("check_collinearity preserves nested adjusted stratified output", {
  df <- collinearity_birthwt_data()
  fit <- stratified_multi_reg(
    data = df,
    outcome = "bwt",
    exposures = c("smoke", "ht"),
    stratifier = "race",
    adjust_for = c("age", "lwt"),
    approach = linear
  )

  out <- check_collinearity(fit, format = tibble)

  expect_type(out, "list")
  expect_true(is.list(out$White))
  expect_true(all(c("smoke", "ht") %in% names(out$White)))
  expect_s3_class(out$White$smoke, "tbl_df")
})

test_that("check_collinearity can return gt and flextable outputs", {
  fit <- multi_reg(
    data = mtcars,
    outcome = "mpg",
    exposures = c("hp", "wt"),
    approach = linear
  )

  gt_out <- check_collinearity(fit, format = gt)
  ft_out <- check_collinearity(fit, format = flextable)

  expect_s3_class(gt_out, "gt_tbl")
  expect_s3_class(ft_out, "flextable")
  expect_equal(names(gt_out$`_data`), c("Variable", "VIF", "Interpretation"))
  expect_match(
    as.character(gt_out$`_source_notes`[[1]]),
    "Screening aid only"
  )
})

test_that("check_collinearity formats nested list outputs without flattening", {
  df <- collinearity_birthwt_data()
  fit <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("smoke", "ht"),
    adjust_for = c("age", "lwt", "race"),
    approach = logit
  )

  out <- check_collinearity(fit, format = gt)

  expect_type(out, "list")
  expect_equal(names(out), c("smoke", "ht"))
  expect_s3_class(out$smoke, "gt_tbl")
  expect_s3_class(out$ht, "gt_tbl")
})

#' Kaplan-Meier survival quantile table
#'
#' Estimate Kaplan-Meier survival time quantiles, such as the 25th percentile,
#' median, and 75th percentile survival times.
#'
#' @param data A \code{data.frame} containing survival time and event status.
#' @param time Survival follow-up time. Quoted and bare names are accepted.
#' @param event Event indicator. Quoted and bare names are accepted. Numeric
#'   \code{0/1}, numeric \code{1/2}, logical, character, and factor variables
#'   are accepted. For two-level character or factor variables, the second level
#'   is treated as the event.
#' @param by Optional grouping variable for separate Kaplan-Meier quantiles.
#'   Quoted and bare names are accepted.
#' @param probs Numeric vector of event-time quantiles to estimate. The default
#'   \code{c(0.25, 0.5, 0.75)} reports the 25th percentile, median, and 75th
#'   percentile event times.
#' @param digits Number of digits for survival time summaries.
#' @param format Output format. One of \code{"flextable"} (default),
#'   \code{"gt"}, or \code{"tibble"}.
#' @param theme Table styling preset.
#'
#' @return A list of class \code{c("gtregression","survival_quantiles", ...)}
#' with elements:
#' \describe{
#'   \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
#'   when \code{format = "tibble"}.}
#'   \item{\code{table_body}}{Tibble with Kaplan-Meier quantiles.}
#'   \item{\code{table_display}}{Display data frame used to render the table.}
#'   \item{\code{fit}}{Fitted \code{survfit} object.}
#'   \item{\code{time,event,by,probs,format,source}}{Metadata fields.}
#' }
#'
#' @details
#' A probability of \code{0.50} is the median event time: the estimated time by
#' which 50% of participants have had the event, corresponding to 50% survival.
#' A probability of \code{0.25} is the time by which 25% have had the event,
#' corresponding to 75% survival. If a quantile is not reached during observed
#' follow-up, the display table shows \code{"Not reached"}.
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#'
#' survival_quantiles(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   by = trt
#' )
#'
#' survival_quantiles(
#'   data = lung_data,
#'   time = "time",
#'   event = "status",
#'   probs = c(0.25, 0.5),
#'   format = tibble
#' )
#'
#' @importFrom survival survfit
#' @importFrom stats complete.cases quantile
#' @export
survival_quantiles <- function(data,
                               time,
                               event,
                               by = NULL,
                               probs = c(0.25, 0.5, 0.75),
                               digits = 1,
                               format = c("flextable", "gt", "tibble"),
                               theme = c("minimal")) {

  time <- .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  event <- .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  by <- .vars_arg(substitute(by), env = parent.frame(), allow_null = TRUE)
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  theme <- .choice_arg(substitute(theme), env = parent.frame())

  format <- match.arg(format, c("flextable", "gt", "tibble"))
  theme <- .resolve_theme(theme)

  data_clean <- .validate_survival_quantiles_inputs(
    data = data,
    time = time,
    event = event,
    by = by,
    probs = probs,
    digits = digits
  )

  if (!is.null(by)) {
    data_clean[[by]] <- factor(data_clean[[by]])
  }

  fit <- survival::survfit(.km_formula(time, event, by), data = data_clean)
  q <- stats::quantile(fit, probs = probs)
  table_body <- .survival_quantiles_table_body(q, probs = probs, by = by)

  if (format == "tibble") {
    return(tibble::as_tibble(table_body))
  }

  table_display <- .survival_quantiles_display(table_body, digits = digits)
  tbl <- .build_survival_quantiles_table(
    table_display,
    format = format,
    theme = theme
  )

  res <- list(
    table = tbl,
    table_body = tibble::as_tibble(table_body),
    table_display = table_display,
    fit = fit,
    time = time,
    event = event,
    by = by,
    probs = probs,
    format = format,
    source = "survival_quantiles"
  )

  class(res) <- c("gtregression", "survival_quantiles", class(res))
  res
}

#' @keywords internal
#' @noRd
.validate_survival_quantiles_inputs <- function(data, time, event, by, probs, digits) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }
  if (!is.character(time) || length(time) != 1L || !time %in% names(data)) {
    stop("`time` must be a single survival time variable in `data`.", call. = FALSE)
  }
  if (!is.character(event) || length(event) != 1L || !event %in% names(data)) {
    stop("`event` must be a single event indicator variable in `data`.", call. = FALSE)
  }
  if (!is.null(by) && (!is.character(by) || length(by) != 1L || !by %in% names(data))) {
    stop("`by` must be NULL or a single grouping variable in `data`.", call. = FALSE)
  }
  if (!is.numeric(data[[time]])) {
    stop("`time` must be numeric.", call. = FALSE)
  }
  if (any(data[[time]] < 0, na.rm = TRUE)) {
    stop("`time` must contain non-negative follow-up times.", call. = FALSE)
  }
  if (!is.numeric(probs) || length(probs) < 1L || anyNA(probs) ||
      any(probs <= 0 | probs >= 1)) {
    stop("`probs` must be a numeric vector with values between 0 and 1.", call. = FALSE)
  }
  if (!is.numeric(digits) || length(digits) != 1L || is.na(digits) ||
      digits < 0 || digits != floor(digits)) {
    stop("`digits` must be a non-negative whole number.", call. = FALSE)
  }

  vars_needed <- unique(c(time, event, by))
  data_clean <- data[stats::complete.cases(data[, vars_needed, drop = FALSE]), , drop = FALSE]
  if (nrow(data_clean) == 0) {
    stop("No complete cases available for survival quantiles.", call. = FALSE)
  }

  data_clean[[event]] <- .cox_event01(data_clean[[event]])
  if (sum(data_clean[[event]] == 1) == 0) {
    stop("`event` must include at least one event.", call. = FALSE)
  }
  if (!is.null(by) && length(unique(data_clean[[by]])) < 2L) {
    stop("`by` must contain at least two non-missing groups.", call. = FALSE)
  }

  data_clean
}

#' @keywords internal
#' @noRd
.survival_quantiles_table_body <- function(q, probs, by = NULL) {
  if (is.null(dim(q$quantile))) {
    out <- data.frame(
      Group = "Overall",
      Probability = probs,
      Survival.probability = 1 - probs,
      Time = as.numeric(q$quantile),
      CI.lower = as.numeric(q$lower),
      CI.upper = as.numeric(q$upper),
      stringsAsFactors = FALSE
    )
    return(out)
  }

  groups <- sub("^.*=", "", rownames(q$quantile))
  do.call(
    rbind,
    lapply(seq_along(groups), function(i) {
      data.frame(
        Group = groups[[i]],
        Probability = probs,
        Survival.probability = 1 - probs,
        Time = as.numeric(q$quantile[i, ]),
        CI.lower = as.numeric(q$lower[i, ]),
        CI.upper = as.numeric(q$upper[i, ]),
        stringsAsFactors = FALSE
      )
    })
  )
}

#' @keywords internal
#' @noRd
.survival_quantiles_display <- function(table_body, digits = 1) {
  pct <- function(x) paste0(formatC(100 * x, digits = 0, format = "f"), "%")
  fmt_num <- function(x) {
    ifelse(
      is.na(x),
      NA_character_,
      formatC(x, digits = digits, format = "f", big.mark = ",")
    )
  }
  time_txt <- ifelse(
    is.na(table_body$Time),
    "Not reached",
    paste0(
      fmt_num(table_body$Time),
      " (",
      ifelse(is.na(table_body$CI.lower), "NA", fmt_num(table_body$CI.lower)),
      "-",
      ifelse(is.na(table_body$CI.upper), "NA", fmt_num(table_body$CI.upper)),
      ")"
    )
  )

  data.frame(
    Group = table_body$Group,
    `Event percentile` = pct(table_body$Probability),
    `Survival probability` = pct(table_body$Survival.probability),
    `Time (95% CI)` = time_txt,
    check.names = FALSE,
    stringsAsFactors = FALSE
  )
}

#' @keywords internal
#' @noRd
.build_survival_quantiles_table <- function(display,
                                            format = c("flextable", "gt"),
                                            theme = c("minimal")) {
  format <- match.arg(format, c("flextable", "gt"))
  note <- paste(
    "Kaplan-Meier quantiles estimate the time by which the event percentile has occurred.",
    "For example, the 50% event percentile is the median survival time."
  )

  if (format == "gt") {
    tbl <- gt::gt(display) |>
      gt::tab_header(title = "Kaplan-Meier survival quantiles") |>
      gt::cols_align(align = "left", columns = "Group") |>
      gt::cols_align(
        align = "center",
        columns = c("Event percentile", "Survival probability", "Time (95% CI)")
      ) |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_column_labels()
      ) |>
      gt::tab_source_note(gt::md(note)) |>
      .compact_gt_source_notes()

    if ("header_shaded" %in% theme) {
      tbl <- gt::tab_options(tbl, column_labels.background.color = "#f6f8fa")
    }
    if ("zebra" %in% theme) {
      tbl <- gt::opt_row_striping(tbl)
    }
    if ("compact" %in% theme) {
      tbl <- gt::tab_options(tbl, data_row.padding = gt::px(2))
    }
    return(tbl)
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Kaplan-Meier survival quantiles")
  ft <- flextable::align(ft, j = "Group", align = "left", part = "all")
  ft <- flextable::align(
    ft,
    j = c("Event percentile", "Survival probability", "Time (95% CI)"),
    align = "center",
    part = "all"
  )
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  if ("header_shaded" %in% theme) {
    ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  }
  if ("zebra" %in% theme && nrow(display) > 1L) {
    ft <- flextable::bg(ft, i = seq(1, nrow(display), by = 2), bg = "#f6f8fa", part = "body")
  }
  if ("compact" %in% theme) {
    ft <- flextable::padding(ft, padding = 2, part = "body")
  }
  ft <- flextable::add_footer_lines(ft, values = note)
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

lung_survival_quantiles_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment"))
    )
}

test_that("survival_quantiles returns grouped Kaplan-Meier quantiles", {
  skip_if_not_installed("survival")

  df <- lung_survival_quantiles_data()

  res <- survival_quantiles(
    data = df,
    time = time,
    event = status,
    by = trt
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "survival_quantiles")
  expect_s3_class(res$table, "flextable")
  expect_s3_class(res$fit, "survfit")
  expect_equal(res$source, "survival_quantiles")
  expect_equal(res$time, "time")
  expect_equal(res$event, "status")
  expect_equal(res$by, "trt")
  expect_equal(res$probs, c(0.25, 0.5, 0.75))
  expect_equal(
    res$table_body$Group,
    rep(c("Standard treatment", "Test treatment"), each = 3)
  )
  expect_equal(res$table_body$Probability[1:3], c(0.25, 0.5, 0.75))
  expect_equal(res$table_body$Survival.probability[1:3], c(0.75, 0.5, 0.25))
  expect_equal(res$table_body$Time, c(27, 103, 162, 24.5, 52.5, 140))
  expect_true("Time (95% CI)" %in% names(res$table_display))
})

test_that("survival_quantiles returns overall tibble with quoted names", {
  skip_if_not_installed("survival")

  df <- lung_survival_quantiles_data()
  time_var <- "time"
  event_var <- "status"

  out <- survival_quantiles(
    data = df,
    time = time_var,
    event = event_var,
    probs = c(0.25, 0.5),
    format = tibble
  )

  expect_s3_class(out, "tbl_df")
  expect_equal(out$Group, c("Overall", "Overall"))
  expect_equal(out$Probability, c(0.25, 0.5))
  expect_equal(out$Time, c(25, 80))
})

test_that("survival_quantiles returns gt table", {
  skip_if_not_installed("survival")
  skip_if_not_installed("gt")

  df <- lung_survival_quantiles_data()

  res <- survival_quantiles(
    data = df,
    time = time,
    event = status,
    by = trt,
    format = gt
  )

  expect_s3_class(res$table, "gt_tbl")
})

test_that("survival_quantiles validates inputs", {
  skip_if_not_installed("survival")

  df <- lung_survival_quantiles_data()
  df$bad_event <- 0
  df$one_group <- "A"

  expect_error(
    survival_quantiles(data = "not data", time = time, event = status),
    "`data` must be"
  )
  expect_error(
    survival_quantiles(data = df, time = missing_time, event = status),
    "`time` must be"
  )
  expect_error(
    survival_quantiles(data = df, time = time, event = missing_event),
    "`event` must be"
  )
  expect_error(
    survival_quantiles(data = df, time = time, event = bad_event),
    "`event` must include at least one event"
  )
  expect_error(
    survival_quantiles(data = df, time = time, event = status, by = one_group),
    "`by` must contain at least two"
  )
  expect_error(
    survival_quantiles(data = df, time = time, event = status, probs = c(0, 0.5)),
    "`probs` must be"
  )
  expect_error(
    survival_quantiles(data = df, time = time, event = status, digits = -1),
    "`digits` must be"
  )
})

#' Cox proportional hazards regression
#'
#' Fit Cox proportional hazards models and report hazard ratios.
#'
#' @param data A \code{data.frame} containing survival time, event status, and
#'   exposure variables.
#' @param time Survival follow-up time. Quoted and bare names are accepted.
#' @param event Event indicator. Quoted and bare names are accepted. Numeric
#'   \code{0/1}, numeric \code{1/2}, logical, character, and factor variables
#'   are accepted. For two-level character or factor variables, the second level
#'   is treated as the event.
#' @param exposures Character vector of exposure variable names. Quoted names
#'   are recommended in scripts, and bare names are also accepted.
#' @param adjust_for Optional character vector of adjustment variables. When
#'   supplied, one adjusted Cox model is fitted per exposure.
#' @param stratifier Optional single stratifying variable. When supplied,
#'   stratum-specific Cox tables are produced using the same crude, adjusted,
#'   or multivariable workflow requested by the other arguments. The
#'   stratifier cannot also be used as the time, event, exposure, adjustment,
#'   or interaction variable.
#' @param interaction Optional character scalar specifying one interaction term
#'   using standard formula syntax, e.g. \code{"trt*prior"}. Quoted and bare
#'   interaction syntax are accepted. In exposure-by-exposure mode, supply a
#'   single exposure; in \code{multivariable = TRUE} mode, the interaction is
#'   added to the single multivariable model.
#' @param multivariable Logical; if \code{FALSE} (default), the current
#'   exposure-by-exposure workflow is used. If \code{TRUE}, one multivariable
#'   Cox model is fitted using all variables in \code{exposures}, and all
#'   exposure coefficients are reported.
#' @param multivariate Optional logical alias for \code{multivariable}. This is
#'   accepted for convenience; \code{multivariable} is used internally.
#' @param format Output table format; one of \code{"flextable"} (default) or
#'   \code{"gt"}.
#' @param theme Table styling preset.
#' @param show_sample For stratified Cox tables, controls which sample-size
#'   columns are shown in the publication table. One of \code{"events"}
#'   (default), \code{"n"}, \code{"both"}, or \code{"none"}. Model statistics,
#'   when requested, still retain both N and event counts.
#' @param model_stats Logical; if \code{TRUE}, extract model-fit statistics
#'   including AIC, BIC, log-likelihood, concordance, number of events, and N.
#' @param show_ref Logical; if \code{TRUE} (default), display reference-category
#'   rows as \code{"Ref."}. If \code{FALSE}, hide reference rows; a message
#'   reminds users to use \code{show_ref = TRUE} when reference rows are needed.
#'
#' @details
#' By default, \code{cox_reg()} keeps the exposure-by-exposure workflow:
#' without \code{adjust_for}, one crude Cox model is fitted per exposure; with
#' \code{adjust_for}, one adjusted Cox model is fitted per exposure and only the
#' exposure estimate is reported. This is useful for screening or for reporting
#' several adjusted exposure effects.
#'
#' With \code{multivariable = TRUE}, all variables in \code{exposures} are
#' included in a single Cox model and all coefficients are reported. This mirrors
#' the multivariable workflow used by \code{multi_reg()}. The
#' \code{adjust_for} argument is not used in this mode; include every variable
#' that belongs in the model inside \code{exposures}. Since these estimates are
#' adjusted for the other variables in the same model, the table reports
#' \code{Adjusted HR (95\% CI)}.
#'
#' Interaction terms specified via \code{interaction} are included using
#' standard formula expansion (for example, \code{trt*prior}). Interaction
#' effects are displayed as additional rows beneath the corresponding exposure.
#'
#' The proportional hazards assumption should be assessed separately, for
#' example with \code{check_ph()}.
#'
#' Stratified Cox tables show event counts by default. Use
#' \code{show_sample = "n"}, \code{show_sample = "both"}, or
#' \code{show_sample = "none"} to control the displayed sample columns. Crude
#' stratified tables calculate these counts for each exposure-specific model;
#' adjusted and multivariable stratified tables use the corresponding fitted
#' model within each stratum.
#'
#' If exposure variables have a \code{"label"} attribute, for example from
#' \code{labelled::var_label()}, those labels are used automatically in the
#' displayed table.
#'
#' @return A list of class \code{c("gtregression","cox_reg", ...)} with elements:
#' \describe{
#'   \item{table}{A \code{flextable} or \code{gt_tbl}.}
#'   \item{table_body}{Data frame of hazard ratios and confidence intervals.}
#'   \item{table_display}{Data frame used to render the publication table.}
#'   \item{models}{List of fitted \code{coxph} models.}
#'   \item{model_summaries}{Summary output for the fitted models.}
#'   \item{model_stats}{Model-fit statistics when \code{model_stats = TRUE};
#'   otherwise \code{NULL}.}
#'   \item{variable_labels}{Named character vector of display labels.}
#'   \item{footnotes}{Character vector used by the rendered table. Adjustment
#'   notes use the same display labels as the table.}
#'   \item{time,event,approach,format,source,adjust_for,exposures,interaction}{Metadata fields.}
#' }
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#' lung_data$prior <- factor(lung_data$prior, levels = c(0, 10),
#'                           labels = c("No", "Yes"))
#'
#' cox_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = c("trt", "celltype", "karno", "age")
#' )
#'
#' cox_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = c(trt, celltype, prior),
#'   adjust_for = c(age, karno)
#' )
#'
#' # Interaction in an adjusted exposure model
#' cox_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = trt,
#'   adjust_for = c(age, karno),
#'   interaction = trt*prior
#' )
#'
#' cox_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = c(trt, celltype, prior, age, karno),
#'   multivariable = TRUE
#' )
#'
#' # multivariate is accepted as an alias
#' cox_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = c(trt, age, karno),
#'   multivariate = TRUE
#' )
#'
#' @importFrom survival Surv coxph
#' @export
cox_reg <- function(data,
                    time,
                    event,
                    exposures,
                    adjust_for = NULL,
                    stratifier = NULL,
                    interaction = NULL,
                    multivariable = FALSE,
                    multivariate = NULL,
                    format = c("flextable", "gt"),
                    theme = c("minimal"),
                    show_sample = "events",
                    model_stats = FALSE,
                    show_ref = TRUE) {

  time <- .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  event <- .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  exposures <- .vars_arg(substitute(exposures), env = parent.frame())
  adjust_for <- .vars_arg(substitute(adjust_for), env = parent.frame(), allow_null = TRUE)
  stratifier <- .survival_optional_single_var_arg(
    substitute(stratifier),
    data = data,
    env = parent.frame()
  )
  interaction <- .interaction_arg(substitute(interaction), env = parent.frame(), allow_null = TRUE)
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("flextable", "gt"))
  theme <- .choice_arg(substitute(theme), env = parent.frame())
  show_sample <- .choice_arg(
    substitute(show_sample),
    env = parent.frame(),
    choices = c("events", "n", "both", "none")
  )

  if (!is.logical(model_stats) || length(model_stats) != 1L || is.na(model_stats)) {
    stop("`model_stats` must be TRUE or FALSE.", call. = FALSE)
  }
  .validate_show_ref(show_ref)
  if (!is.null(multivariate)) {
    if (!is.logical(multivariate) || length(multivariate) != 1L || is.na(multivariate)) {
      stop("`multivariate` must be TRUE or FALSE.", call. = FALSE)
    }
    if (!identical(multivariable, FALSE) && !identical(multivariable, multivariate)) {
      stop("Use only one of `multivariable` or `multivariate`, or give them the same value.", call. = FALSE)
    }
    multivariable <- multivariate
  }
  if (!is.logical(multivariable) || length(multivariable) != 1L || is.na(multivariable)) {
    stop("`multivariable` must be TRUE or FALSE.", call. = FALSE)
  }
  if (isTRUE(multivariable) && !is.null(adjust_for) && length(adjust_for) > 0) {
    stop(
      "`adjust_for` is not used when `multivariable = TRUE`; include all model variables in `exposures`.",
      call. = FALSE
    )
  }

  format <- match.arg(format, c("flextable", "gt"))
  show_sample_choices <- c("events", "n", "both", "none")
  if (!is.character(show_sample) || length(show_sample) != 1L || !show_sample %in% show_sample_choices) {
    stop("`show_sample` must be one of 'events', 'n', 'both', or 'none'.", call. = FALSE)
  }
  show_sample <- match.arg(show_sample, show_sample_choices)
  theme <- .resolve_theme(theme)
  adjusted_mode <- !is.null(adjust_for) && length(adjust_for) > 0
  fmt_class <- if (format == "gt") "gt_cox" else "ft_cox"

  if (!is.null(stratifier)) {
    .validate_survival_stratifier(
      data = data,
      time = time,
      event = event,
      exposures = exposures,
      adjust_for = adjust_for,
      interaction = interaction,
      stratifier = stratifier
    )

    return(.run_stratified_cox_reg(
      data = data,
      time = time,
      event = event,
      exposures = exposures,
      adjust_for = adjust_for,
      stratifier = stratifier,
      interaction = interaction,
      multivariable = multivariable,
      format = format,
      theme = theme,
      show_sample = show_sample,
      model_stats = model_stats,
      show_ref = show_ref,
      fmt_class = fmt_class
    ))
  }

  core <- .run_cox_core(
    data = data,
    time = time,
    event = event,
    exposures = exposures,
    adjust_for = adjust_for,
    interaction = interaction,
    multivariable = multivariable
  )

  effect_label <- if (adjusted_mode || isTRUE(multivariable)) "Adjusted HR (95% CI)" else "HR (95% CI)"
  variable_labels <- .var_label_map(
    data,
    unique(c(exposures, adjust_for, .interaction_vars(interaction)))
  )
  core$table_body <- .format_interaction_levels(
    core$table_body, core$data_clean, variable_labels
  )
  crude_mode <- !adjusted_mode && !isTRUE(multivariable)

  if (crude_mode) {
    display_df <- .make_display_survival_uni(
      core$table_body,
      core$data_clean,
      time = time,
      event = event,
      effect_label = effect_label,
      variable_labels = variable_labels,
      show_ref = show_ref
    )
    .must_be_display_df(display_df)
  } else {
    display_df <- .make_display_multi(
      core$table_body,
      core$data_clean,
      outcome = event,
      effect_label = effect_label,
      variable_labels = variable_labels,
      show_ref = show_ref
    )
    .must_be_display_df_multi(display_df)
  }

  footnotes <- c(
    .abbrev_note("cox"),
    if (isTRUE(show_ref) && any(core$table_body$ref %in% TRUE)) .ref_note() else NULL,
    if (adjusted_mode) .display_adjustment_note(adjust_for, variable_labels) else NULL,
    if (isTRUE(multivariable)) "Adjusted for the other variables in the model." else NULL,
    if (!is.null(interaction)) .interaction_note(interaction) else NULL,
    NULL
  )

  .message_hidden_ref_rows("cox_reg", core$table_body, show_ref)

  tbl <- if (format == "gt") {
    if (crude_mode) {
      .build_gt(display_df, effect_label, footnotes, theme)
    } else {
      .build_gt_multi(display_df, effect_label, footnotes, theme)
    }
  } else {
    if (crude_mode) {
      .build_flextable(display_df, effect_label, footnotes, theme)
    } else {
      .build_flextable_multi(display_df, effect_label, footnotes, theme)
    }
  }

  res <- list(
    table = tbl,
    table_body = core$table_body,
    table_display = display_df,
    models = core$models,
    model_summaries = core$model_summaries,
    model_stats = if (isTRUE(model_stats)) .cox_model_stats_table(core$models) else NULL,
    variable_labels = variable_labels,
    footnotes = footnotes,
    time = time,
    event = event,
    approach = "cox",
    format = format,
    source = "cox_reg",
    adjusted_mode = adjusted_mode,
    multivariable = isTRUE(multivariable),
    adjust_for = if (adjusted_mode) unique(adjust_for) else NULL,
    exposures = unique(exposures),
    interaction = interaction,
    show_ref = isTRUE(show_ref)
  )

  class(res) <- c("gtregression", "cox_reg", fmt_class, class(res))
  res
}

#' @keywords internal
#' @noRd
.run_cox_core <- function(data,
                          time,
                          event,
                          exposures,
                          adjust_for = NULL,
                          interaction = NULL,
                          multivariable = FALSE) {
  data_valid <- .validate_cox_inputs(
    data,
    time,
    event,
    exposures,
    adjust_for,
    interaction = interaction,
    multivariable = multivariable
  )
  adjusted_mode <- !is.null(adjust_for) && length(adjust_for) > 0
  interaction_vars <- .interaction_vars(interaction)

  if (isTRUE(multivariable)) {
    predictors <- unique(c(exposures, interaction_vars))
    data_model <- .cox_model_data(data_valid, time, event, predictors)
    fit <- .fit_cox_model(data_model, time, event, predictors, interaction = interaction)

    if (is.null(fit)) {
      stop("Cox model fitting failed for the multivariable model.", call. = FALSE)
    }

    td_list <- lapply(unique(exposures), function(exposure) .tidy_cox(fit, exposure, interaction = interaction))
    td_list <- Filter(Negate(is.null), td_list)
    if (!length(td_list)) {
      stop("No estimable Cox coefficients for the multivariable model.", call. = FALSE)
    }

    fits <- list(multivariable_model = fit)
    return(list(
      data_clean = data_valid,
      table_body = do.call(rbind, td_list),
      models = fits,
      model_summaries = lapply(fits, summary)
    ))
  }

  fits <- vector("list", length(exposures))
  names(fits) <- exposures
  tds <- vector("list", length(exposures))
  names(tds) <- exposures

  for (i in seq_along(exposures)) {
    exposure <- exposures[i]
    predictors <- unique(c(if (adjusted_mode) c(exposure, adjust_for) else exposure, interaction_vars))
    data_model <- .cox_model_data(data_valid, time, event, predictors)
    fit <- .fit_cox_model(data_model, time, event, predictors, interaction = interaction)

    if (is.null(fit)) {
      stop("Cox model fitting failed for exposure '", exposure, "'.", call. = FALSE)
    }

    td <- .tidy_cox(fit, exposure, interaction = interaction)
    if (is.null(td) || !nrow(td)) {
      stop("No estimable Cox coefficients for exposure '", exposure, "'.", call. = FALSE)
    }

    fits[[i]] <- fit
    tds[[i]] <- td
  }

  list(
    data_clean = data_valid,
    table_body = do.call(rbind, tds),
    models = fits,
    model_summaries = lapply(fits, summary)
  )
}

#' @keywords internal
#' @noRd
.cox_single_var_arg <- function(expr, data, env = parent.frame()) {
  if (is.symbol(expr)) {
    nm <- as.character(expr)
    if (nm %in% names(data)) {
      return(nm)
    }
    if (exists(nm, envir = env, inherits = TRUE)) {
      val <- get(nm, envir = env, inherits = TRUE)
      if (is.character(val) && length(val) == 1L) {
        return(val)
      }
    }
    return(nm)
  }

  out <- eval(expr, envir = env)
  if (!is.character(out) || length(out) != 1L) {
    stop("Survival variable arguments must be single column names.", call. = FALSE)
  }
  out
}

#' @keywords internal
#' @noRd
.validate_cox_inputs <- function(data,
                                 time,
                                 event,
                                 exposures,
                                 adjust_for = NULL,
                                 interaction = NULL,
                                 multivariable = FALSE) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }

  if (!is.character(time) || length(time) != 1L || !time %in% names(data)) {
    stop("`time` must be a single survival time variable in `data`.", call. = FALSE)
  }

  if (!is.character(event) || length(event) != 1L || !event %in% names(data)) {
    stop("`event` must be a single event indicator variable in `data`.", call. = FALSE)
  }

  exposures <- unique(exposures)
  if (!is.character(exposures) || length(exposures) < 1L) {
    stop("`exposures` must contain at least one variable.", call. = FALSE)
  }

  if (!all(exposures %in% names(data))) {
    stop("One or more exposure variables were not found in the dataset.", call. = FALSE)
  }

  adjusted_mode <- !is.null(adjust_for) && length(adjust_for) > 0
  if (adjusted_mode) {
    adjust_for <- unique(adjust_for)
    if (!is.character(adjust_for) || !all(adjust_for %in% names(data))) {
      stop("One or more adjustment variables were not found in the dataset.", call. = FALSE)
    }
    if (any(exposures %in% adjust_for)) {
      stop("In adjusted mode, `exposures` and `adjust_for` must not overlap.", call. = FALSE)
    }
  } else {
    adjust_for <- NULL
  }

  interaction_vars <- .validate_interaction_term(
    data = data,
    exposures = exposures,
    interaction = interaction,
    adjusted_mode = adjusted_mode,
    exposure_by_exposure = !isTRUE(multivariable)
  )

  if (!is.numeric(data[[time]])) {
    stop("`time` must be numeric.", call. = FALSE)
  }

  if (any(data[[time]] < 0, na.rm = TRUE)) {
    stop("`time` must contain non-negative follow-up times.", call. = FALSE)
  }

  data_valid <- data

  data_valid[[event]] <- .cox_event01(data_valid[[event]])
  if (sum(data_valid[[event]] == 1, na.rm = TRUE) == 0) {
    stop("`event` must include at least one event.", call. = FALSE)
  }
  if (sum(data_valid[[event]] == 0, na.rm = TRUE) == 0) {
    stop("`event` must include at least one censored observation.", call. = FALSE)
  }

  .validate_exposures(data_valid, unique(c(exposures, adjust_for, interaction_vars)))
  data_valid
}

#' @keywords internal
#' @noRd
.cox_model_data <- function(data, time, event, predictors) {
  vars_needed <- unique(c(time, event, predictors))
  cc_idx <- stats::complete.cases(data[, vars_needed, drop = FALSE])
  data_model <- data[cc_idx, , drop = FALSE]

  if (nrow(data_model) == 0) {
    stop("No complete cases available for this Cox model.", call. = FALSE)
  }

  if (sum(data_model[[event]] == 1, na.rm = TRUE) == 0) {
    stop("`event` must include at least one event for this Cox model.", call. = FALSE)
  }
  if (sum(data_model[[event]] == 0, na.rm = TRUE) == 0) {
    stop("`event` must include at least one censored observation for this Cox model.", call. = FALSE)
  }

  .validate_exposures(data_model, predictors)
  data_model
}

#' @keywords internal
#' @noRd
.cox_event01 <- function(x) {
  if (is.logical(x)) {
    return(as.integer(x))
  }

  if (is.factor(x) || is.character(x)) {
    xf <- factor(x)
    if (nlevels(xf) != 2L) {
      stop("`event` must have exactly two levels.", call. = FALSE)
    }
    return(as.integer(xf == levels(xf)[2L]))
  }

  if (is.numeric(x)) {
    vals <- sort(unique(stats::na.omit(x)))
    if (length(vals) > 0 && all(vals %in% c(0, 1))) {
      return(as.integer(x))
    }
    if (length(vals) > 0 && all(vals %in% c(1, 2))) {
      return(as.integer(x == 2))
    }
  }

  stop("`event` must be coded as 0/1, 1/2, logical, or a two-level factor/character variable.", call. = FALSE)
}

#' @keywords internal
#' @noRd
.fit_cox_model <- function(data, time, event, predictors, interaction = NULL) {
  bt <- .surv_bt
  rhs <- .survival_rhs(predictors, interaction)
  fml <- stats::as.formula(paste0("survival::Surv(", bt(time), ", ", bt(event), ") ~ ", rhs))

  tryCatch(
    survival::coxph(fml, data = data, model = TRUE),
    error = function(e) {
      warning("Cox model failed: ", e$message, call. = FALSE)
      NULL
    }
  )
}

#' @keywords internal
#' @noRd
.tidy_cox <- function(fit, exposure, interaction = NULL) {
  smry <- summary(fit)
  coefs <- smry$coefficients
  if (is.null(coefs) || nrow(coefs) == 0) {
    return(NULL)
  }

  rn <- rownames(coefs)

  is_exposure_term <- function(term, exposure) {
    term_clean <- gsub("`", "", term, fixed = TRUE)
    exp_clean <- gsub("`", "", exposure, fixed = TRUE)
    if (identical(term_clean, exp_clean) ||
        (startsWith(term_clean, exp_clean) && !grepl(":", term_clean, fixed = TRUE))) {
      return(TRUE)
    }
    if (grepl(":", term_clean, fixed = TRUE) && !is.null(interaction)) {
      parts <- strsplit(term_clean, ":", fixed = TRUE)[[1]]
      return(any(startsWith(parts, exp_clean) | parts == exp_clean))
    }
    FALSE
  }

  clean_level <- function(term, exposure) {
    term_clean <- gsub("`", "", term, fixed = TRUE)
    exp_clean <- gsub("`", "", exposure, fixed = TRUE)
    if (identical(term_clean, exp_clean)) {
      return(exp_clean)
    }
    if (grepl(":", term_clean, fixed = TRUE)) {
      return(paste(strsplit(term_clean, ":", fixed = TRUE)[[1]], collapse = " x "))
    }
    lvl <- sub(paste0("^", exp_clean), "", term_clean)
    if (identical(lvl, "")) exp_clean else lvl
  }

  idx <- which(vapply(rn, is_exposure_term, logical(1), exposure = exposure))
  df_nonref <- NULL

  if (length(idx)) {
    est_log <- coefs[idx, "coef"]
    se_col <- if ("robust se" %in% colnames(coefs)) "robust se" else "se(coef)"
    se <- coefs[idx, se_col]
    p_col <- grep("^Pr\\(", colnames(coefs), value = TRUE)
    p <- coefs[idx, p_col[1]]
    z <- stats::qnorm(0.975)

    df_nonref <- data.frame(
      exposure = exposure,
      level = vapply(rn[idx], clean_level, character(1), exposure = exposure),
      estimate = exp(est_log),
      conf.low = exp(est_log - z * se),
      conf.high = exp(est_log + z * se),
      p.value = p,
      ref = FALSE,
      stringsAsFactors = FALSE
    )
  }

  ref_row <- NULL
  exposure_data <- .model_data_col(fit, exposure)
  if (!is.null(exposure_data) && is.factor(exposure_data)) {
    levs <- levels(exposure_data)
    ref_row <- data.frame(
      exposure = exposure,
      level = levs[1],
      estimate = 1,
      conf.low = NA_real_,
      conf.high = NA_real_,
      p.value = NA_real_,
      ref = TRUE,
      stringsAsFactors = FALSE
    )

    if (!is.null(df_nonref)) {
      df_nonref$..ord <- match(df_nonref$level, levs)
      df_nonref <- df_nonref[order(df_nonref$..ord), , drop = FALSE]
      df_nonref$..ord <- NULL
    }
  }

  if (is.null(ref_row) && is.null(df_nonref)) {
    return(NULL)
  }
  if (!is.null(ref_row) && !is.null(df_nonref)) {
    return(rbind(ref_row, df_nonref))
  }
  if (!is.null(ref_row)) ref_row else df_nonref
}

#' @keywords internal
#' @noRd
.surv_bt <- function(x) {
  paste0("`", gsub("`", "", x, fixed = TRUE), "`")
}

#' @keywords internal
#' @noRd
.interaction_vars <- function(interaction) {
  if (is.null(interaction) || !length(interaction)) {
    return(character(0))
  }
  vars <- trimws(unlist(strsplit(interaction, "\\*")))
  vars[nzchar(vars)]
}

#' @keywords internal
#' @noRd
.interaction_formula_term <- function(interaction) {
  vars <- .interaction_vars(interaction)
  if (!length(vars)) {
    return(NULL)
  }
  paste(.surv_bt(vars), collapse = " * ")
}

#' @keywords internal
#' @noRd
.survival_rhs <- function(predictors, interaction = NULL) {
  terms <- if (length(predictors)) .surv_bt(predictors) else character(0)
  if (!is.null(interaction)) {
    terms <- c(terms, .interaction_formula_term(interaction))
  }
  if (!length(terms)) {
    return("1")
  }
  paste(unique(terms), collapse = " + ")
}

#' @keywords internal
#' @noRd
.cox_model_stats_table <- function(models) {
  out <- lapply(names(models), function(model_name) {
    fit <- models[[model_name]]
    smry <- tryCatch(summary(fit), error = function(e) NULL)
    concordance <- if (!is.null(smry) && !is.null(smry$concordance)) {
      suppressWarnings(as.numeric(smry$concordance[1]))
    } else {
      NA_real_
    }

    data.frame(
      model = model_name,
      AIC = .safe_numeric(stats::AIC(fit)),
      BIC = .safe_numeric(stats::BIC(fit)),
      logLik = .safe_numeric(as.numeric(stats::logLik(fit))),
      concordance = .safe_numeric(concordance),
      events = .safe_numeric(fit$nevent),
      n = .safe_numeric(fit$n),
      stringsAsFactors = FALSE
    )
  })

  out <- Filter(Negate(is.null), out)
  if (!length(out)) {
    return(NULL)
  }
  do.call(rbind, out)
}

lung_ph_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )
}

test_that("check_ph works with direct coxph models", {
  skip_if_not_installed("survival")

  df <- lung_ph_data()
  fit <- survival::coxph(survival::Surv(time, status) ~ trt + age + prior, data = df)

  out <- check_ph(fit, format = tibble)

  expect_s3_class(out, "tbl_df")
  expect_true(all(c("Model", "Term", "Test", "Chi.square", "df", "p.value",
                    "Interpretation") %in% names(out)))
  expect_equal(unique(out$Model), "cox_model")
  expect_true("GLOBAL" %in% out$Term)
  expect_true(all(is.finite(out$p.value)))
  expect_true(all(out$Interpretation %in%
                    c("Possible PH violation", "No evidence of PH violation")))
})

test_that("check_ph works with cox_reg objects", {
  skip_if_not_installed("survival")

  df <- lung_ph_data()

  cox_fit <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior),
    adjust_for = c(age, karno)
  )

  out <- check_ph(cox_fit, transform = rank, format = tibble)

  expect_s3_class(out, "tbl_df")
  expect_true(all(c("trt", "celltype", "prior") %in% out$Model))
  expect_equal(sum(out$Term == "GLOBAL"), 3)
  expect_true(all(out$Test %in% c("Term", "Global")))
})

test_that("check_ph returns formatted tables", {
  skip_if_not_installed("survival")
  skip_if_not_installed("flextable")
  skip_if_not_installed("gt")

  df <- lung_ph_data()
  fit <- survival::coxph(survival::Surv(time, status) ~ trt + age, data = df)

  expect_s3_class(check_ph(fit), "flextable")
  expect_s3_class(check_ph(fit, format = gt), "gt_tbl")
})

test_that("check_ph validates inputs", {
  skip_if_not_installed("survival")

  df <- lung_ph_data()
  fit <- survival::coxph(survival::Surv(time, status) ~ trt + age, data = df)

  expect_error(
    check_ph(stats::lm(mpg ~ hp, data = mtcars)),
    "`model` must be"
  )
  expect_error(
    check_ph(fit, alpha = 2),
    "`alpha` must be"
  )
  expect_error(
    check_ph(fit, transform = invalid_transform),
    "km|rank|identity"
  )
})

birthwt_plot_combine_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
      ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes"))
    )
}

test_that("plot_reg_combine returns patchwork for current regression objects", {
  df <- birthwt_plot_combine_data()
  exposures <- c("smoke", "age", "ht", "lwt", "ui")

  tbl_uni <- uni_reg(df, outcome = "low", exposures = exposures, approach = logit)
  tbl_multi <- multi_reg(df, outcome = "low", exposures = exposures, approach = logit)

  p <- plot_reg_combine(tbl_uni, tbl_multi)

  expect_s3_class(p, "patchwork")
  expect_equal(p$patches$annotation$caption, "Ref. = reference category.")
  expect_equal(p[[1]]$data$exposure[p[[1]]$data$is_header], exposures)
  expect_equal(p[[2]]$data$exposure[p[[2]]$data$is_header], exposures)
})

test_that("plot_reg_combine inherits compact binary rows from both tables", {
  df <- birthwt_plot_combine_data()
  exposures <- c("smoke", "ht", "age")

  tbl_uni <- uni_reg(
    df, outcome = "low", exposures = exposures, approach = logit,
    show_ref = FALSE
  )
  tbl_multi <- multi_reg(
    df, outcome = "low", exposures = exposures, approach = logit,
    show_ref = FALSE
  )

  p <- plot_reg_combine(tbl_uni, tbl_multi)

  expect_false(any(grepl("(Ref.)", p[[1]]$data$label_clean, fixed = TRUE)))
  expect_false(any(grepl("(Ref.)", p[[2]]$data$label_clean, fixed = TRUE)))
  expect_false(any(p[[1]]$data$label %in% c("Yes", "No")))
  expect_false(any(p[[2]]$data$label %in% c("Yes", "No")))
  expect_true(all(c("smoke", "ht") %in% p[[1]]$data$label))
  expect_true(all(c("smoke", "ht") %in% p[[2]]$data$label))
})

test_that("plot_reg_combine supports multi_reg adjust_for mode", {
  df <- birthwt_plot_combine_data()

  tbl_uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("smoke", "ht", "ui"),
    approach = logit
  )
  tbl_multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("smoke", "ht", "ui"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )

  p <- plot_reg_combine(
    tbl_uni,
    tbl_multi,
    order_y = c("smoke", "ht", "ui"),
    log_x = TRUE,
    show_ref = FALSE,
    point_color = "#1b9e77",
    errorbar_color = "#4d4d4d",
    sig_color = "#d95f02",
    sig_errorbar_color = "#7570b3",
    xlim_uni = c(0.2, 20),
    breaks_uni = c(0.5, 1, 2, 4, 8),
    xlim_multi = c(0.2, 20),
    breaks_multi = c(0.5, 1, 2, 4, 8)
  )

  expect_s3_class(p, "patchwork")
  expect_equal(p$patches$annotation$caption, "Adjusted for age and lwt")
  expect_true("smoke" %in% p[[1]]$data$label)
  expect_true("ht" %in% p[[1]]$data$label)
  expect_true(any(p[[1]]$data$label == "smoke" & p[[1]]$data$is_header))
  expect_true(any(grepl("<b>smoke</b>", p[[1]]$data$label_clean, fixed = TRUE)))
  expect_equal(p[[1]]$coordinates$limits$x, c(0.2, 20))
  expect_equal(p[[1]]$scales$get_scales("x")$breaks, c(0.5, 1, 2, 4, 8))
  expect_equal(
    plot_reg_combine(tbl_uni, tbl_multi, show_adjustment_note = FALSE)$patches$annotation$caption,
    "Ref. = reference category."
  )
  expect_equal(
    plot_reg_combine(tbl_uni, tbl_multi, caption = "Custom note")$patches$annotation$caption,
    "Custom note"
  )

  p_default_breaks <- plot_reg_combine(tbl_uni, tbl_multi, log_x = TRUE)
  expect_true(length(p_default_breaks[[1]]$scales$get_scales("x")$breaks) > 1L)
  expect_true(1 %in% p_default_breaks[[1]]$scales$get_scales("x")$breaks)
})

test_that("plot_reg_combine supports survival multivariable counterparts", {
  skip_if_not_installed("survival")

  df <- data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )

  cox_crude <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior)
  )
  cox_multi <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior, age, karno),
    multivariable = TRUE
  )
  surv_crude <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior)
  )
  surv_multi <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior, age, karno),
    multivariable = TRUE
  )

  p_cox <- plot_reg_combine(cox_crude, cox_multi)
  p_surv <- plot_reg_combine(surv_crude, surv_multi)

  expect_s3_class(p_cox, "patchwork")
  expect_s3_class(p_surv, "patchwork")
  expect_match(p_cox[[2]]$labels$x, "Adjusted Hazard Ratio", fixed = TRUE)
  expect_match(p_surv[[2]]$labels$x, "Adjusted Time Ratio", fixed = TRUE)
  expect_equal(
    p_cox$patches$annotation$caption,
    "Adjusted for the other variables in the model. Ref. = reference category."
  )
})

test_that("plot_reg_combine supports mismatched exposures and linear models", {
  df <- birthwt_plot_combine_data()

  tbl_uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "lwt", "smoke", "ht"),
    approach = logit
  )
  tbl_multi_subset <- multi_reg(
    df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = logit
  )
  p_subset <- plot_reg_combine(tbl_uni, tbl_multi_subset)
  expect_s3_class(p_subset, "patchwork")

  tbl_uni_linear <- uni_reg(
    df,
    outcome = "bwt",
    exposures = c("age", "lwt", "smoke"),
    approach = linear
  )
  tbl_multi_linear <- multi_reg(
    df,
    outcome = "bwt",
    exposures = c("age", "lwt", "smoke"),
    approach = linear
  )
  p_linear <- plot_reg_combine(tbl_uni_linear, tbl_multi_linear, log_x = TRUE, alpha = 0.5)
  expect_s3_class(p_linear, "patchwork")
})

test_that("plot_reg_combine validates object classes and sources", {
  df <- birthwt_plot_combine_data()

  tbl_uni <- uni_reg(df, outcome = "low", exposures = "smoke", approach = logit)
  tbl_multi <- multi_reg(df, outcome = "low", exposures = "smoke", approach = logit)

  expect_error(plot_reg_combine(mtcars, tbl_multi), "`tbl_uni` must be")
  expect_error(plot_reg_combine(tbl_uni, mtcars), "`tbl_multi` must be")
  expect_error(plot_reg_combine(tbl_multi, tbl_multi), "must come from uni_reg")
  expect_error(plot_reg_combine(tbl_uni, tbl_uni), "must come from multi_reg")
})

test_that("compare_models reports multi_reg statistics without refitting", {
  df <- data.frame(
    y = c(0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0),
    x1 = c(1, 2, 1, 3, 4, 2, 5, 1, 4, 2, 3, 5),
    x2 = factor(c("A", "B", "A", "B", "B", "A", "B", "A", "B", "A", "A", "B"))
  )

  m0 <- multi_reg(data = df, outcome = y, exposures = x1, approach = logit)
  m1 <- multi_reg(data = df, outcome = y, exposures = c(x1, x2), approach = logit)
  fit0 <- m0$models[[1]]
  fit1 <- m1$models[[1]]

  res <- compare_models(
    m0,
    m1,
    model_names = c("Clinical score", "Clinical score + group"),
    primary_exposure = x1,
    format = tibble
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "compare_models")
  expect_null(res$table)
  expect_equal(res$table_body$model, c("Clinical score", "Clinical score + group"))
  expect_equal(res$table_body$model_terms, c("x1", "x1 + x2"))
  expect_equal(res$table_display$Variables, c("x1", "x1 + x2"))
  expect_equal(res$table_body$n, c(stats::nobs(fit0), stats::nobs(fit1)))
  expect_equal(res$table_body$AIC, c(stats::AIC(fit0), stats::AIC(fit1)))
  expect_equal(res$table_body$BIC, c(stats::BIC(fit0), stats::BIC(fit1)))
  expect_equal(res$table_body$logLik, c(as.numeric(stats::logLik(fit0)), as.numeric(stats::logLik(fit1))))
  expect_equal(res$comparison_status$status, "Same analysis sample")
  expect_equal(unique(res$table_body$comparison_status), "Same analysis sample")
  expect_true(res$table_body$nested_comparison[2])
  expect_false(any(grepl("Different analysis sample", res$comparison_warnings, fixed = TRUE)))
  expect_false(any(grepl("Non-nested comparison", res$comparison_warnings, fixed = TRUE)))
  expect_true(any(grepl("Comparison status: Same analysis sample", res$footnotes, fixed = TRUE)))
  expect_true(any(grepl("Nested-model status: sequential models are nested", res$footnotes, fixed = TRUE)))
  expect_true(is.na(res$table_body$LR_chisq[1]))
  expect_true(is.finite(res$table_body$LR_chisq[2]))
  expect_equal(sum(res$table_body$best_AIC), 1)
  expect_true("Best AIC" %in% names(res$table_display))
  expect_true("Variables" %in% names(res$table_display))
  expect_true("Primary estimate" %in% names(res$table_display))
})

test_that("compare_models uses object names by default", {
  df <- data.frame(
    y = c(0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0),
    x1 = c(1, 2, 1, 3, 4, 2, 5, 1, 4, 2, 3, 5),
    x2 = factor(c("A", "B", "A", "B", "B", "A", "B", "A", "B", "A", "A", "B"))
  )

  m0 <- multi_reg(data = df, outcome = y, exposures = x1, approach = logit)
  m1 <- multi_reg(data = df, outcome = y, exposures = c(x1, x2), approach = logit)

  default_names <- compare_models(m0, m1, format = tibble)
  explicit_names <- compare_models(
    m0,
    m1,
    model_names = c("Clinical", "Clinical plus group"),
    format = tibble
  )

  expect_equal(default_names$table_body$model, c("m0", "m1"))
  expect_equal(explicit_names$table_body$model, c("Clinical", "Clinical plus group"))
})

test_that("compare_models flags different analysis samples without hiding statistics", {
  df <- data.frame(
    y = c(2.1, 3.4, 4.8, 6.2, 7.4, 8.9, 10.5, 11.7, 12.8, 14.1),
    x1 = c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10),
    x2 = c(0, 1, 0, 1, NA, 0, 1, NA, 1, 0)
  )

  m0 <- multi_reg(data = df, outcome = y, exposures = x1, approach = linear)
  m1 <- multi_reg(data = df, outcome = y, exposures = c(x1, x2), approach = linear)

  res <- compare_models(
    m0,
    m1,
    primary_exposure = x1,
    format = tibble
  )

  expect_equal(res$comparison_status$status, "Different analysis sample")
  expect_equal(unique(res$table_body$comparison_status), "Different analysis sample")
  expect_true(res$table_body$nested_comparison[2])
  expect_true(length(unique(res$table_body$n)) > 1)
  expect_true(all(is.finite(res$table_body$AIC)))
  expect_true(all(is.finite(res$table_body$BIC)))
  expect_true(all(is.finite(res$table_body$logLik)))
  expect_true(is.finite(res$table_body$primary_estimate[2]))
  expect_true(is.finite(res$table_body$primary_pct_change[2]))
  expect_true(length(res$comparison_warnings) >= 1)
  expect_match(res$comparison_warnings[1], "Different analysis sample: Models were fitted")
  expect_false(any(grepl("Non-nested comparison", res$comparison_warnings, fixed = TRUE)))
  expect_true(any(grepl("Comparison status: Different analysis sample", res$footnotes, fixed = TRUE)))
  expect_true(any(grepl("Nested-model status: sequential models are nested", res$footnotes, fixed = TRUE)))
})

test_that("compare_models only warns about non-nested models when needed", {
  df <- data.frame(
    y = c(0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0),
    x1 = c(1, 2, 1, 3, 4, 2, 5, 1, 4, 2, 3, 5),
    x2 = c(0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0),
    x3 = c(1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6)
  )

  m0 <- multi_reg(data = df, outcome = y, exposures = c(x1, x2), approach = logit)
  m1 <- multi_reg(data = df, outcome = y, exposures = c(x1, x3), approach = logit)

  res <- compare_models(m0, m1, format = tibble)

  expect_equal(res$comparison_status$status, "Same analysis sample")
  expect_false(res$table_body$nested_comparison[2])
  expect_false(any(grepl("Different analysis sample", res$comparison_warnings, fixed = TRUE)))
  expect_true(any(grepl("Non-nested comparison", res$comparison_warnings, fixed = TRUE)))
  expect_true(any(grepl("Nested-model status: one or more sequential model comparisons are not nested", res$footnotes, fixed = TRUE)))
})

test_that("compare_models reports Cox events and concordance from cox_reg objects", {
  skip_if_not_installed("survival")

  df <- data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )
  m0 <- cox_reg(data = df, time = time, event = status, exposures = trt)
  m1 <- cox_reg(data = df, time = time, event = status, exposures = trt, adjust_for = c(age, karno))
  fit0 <- m0$models[[1]]
  fit1 <- m1$models[[1]]

  res <- compare_models(
    m0,
    m1,
    model_names = c("Treatment", "Treatment + clinical covariates"),
    primary_exposure = trt,
    nested = TRUE,
    format = tibble
  )

  expect_equal(res$table_body$n, c(fit0$n, fit1$n))
  expect_equal(res$table_body$model_terms, c("trt", "trt + age + karno"))
  expect_false(any(grepl("Surv", res$table_display$Variables, fixed = TRUE)))
  expect_equal(res$table_body$events, c(fit0$nevent, fit1$nevent))
  expect_equal(res$table_body$AIC, c(stats::AIC(fit0), stats::AIC(fit1)))
  expect_equal(res$table_body$BIC, c(stats::BIC(fit0), stats::BIC(fit1)))
  expect_equal(res$table_body$logLik, c(as.numeric(stats::logLik(fit0)), as.numeric(stats::logLik(fit1))))
  expect_equal(
    res$table_body$concordance,
    unname(c(summary(fit0)$concordance[1], summary(fit1)$concordance[1]))
  )
  expect_equal(
    res$table_body$primary_estimate,
    c(exp(stats::coef(fit0)[["trtTest treatment"]]), exp(stats::coef(fit1)[["trtTest treatment"]])),
    tolerance = 1e-8
  )
  expect_true("Events" %in% names(res$table_display))
  expect_true("Concordance" %in% names(res$table_display))
})

test_that("compare_models supports list input and formatted tables", {
  skip_if_not_installed("flextable")

  df <- data.frame(
    y = c(2.1, 3.4, 4.8, 6.2, 7.4, 8.9, 10.5, 11.7, 12.8, 14.1),
    x1 = c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10),
    x2 = c(0, 1, 0, 1, 1, 0, 1, 0, 1, 0)
  )

  m0 <- multi_reg(data = df, outcome = y, exposures = x1, approach = linear)
  m1 <- multi_reg(data = df, outcome = y, exposures = c(x1, x2), approach = linear)

  res <- compare_models(
    list(base = m0, adjusted = m1),
    nested = FALSE
  )

  expect_s3_class(res$table, "flextable")
  expect_equal(res$table_body$model, c("base", "adjusted"))
  expect_true(all(is.na(res$table_body$LR_chisq)))
  expect_false("Events" %in% names(res$table_display))
})

test_that("compare_models renders caution notes prominently in flextable output", {
  skip_if_not_installed("flextable")

  df <- data.frame(
    y = c(2.1, 3.4, 4.8, 6.2, 7.4, 8.9, 10.5, 11.7, 12.8, 14.1),
    x1 = c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10),
    x2 = c(0, 1, 0, 1, NA, 0, 1, NA, 1, 0)
  )

  m0 <- multi_reg(data = df, outcome = y, exposures = x1, approach = linear)
  m1 <- multi_reg(data = df, outcome = y, exposures = c(x1, x2), approach = linear)

  res <- compare_models(m0, m1, primary_exposure = x1)

  expect_s3_class(res$table, "flextable")
  expect_match(res$table$footer$dataset$Model[1], "Comparison status: Different analysis sample")
  expect_match(res$table$footer$dataset$Model[2], "Nested-model status: sequential models are nested")
  expect_match(res$table$footer$dataset$Model[3], "Different analysis sample: Models were fitted")
})

test_that("compare_models renders caution notes without raw HTML in gt output", {
  skip_if_not_installed("gt")

  df <- data.frame(
    y = c(2.1, 3.4, 4.8, 6.2, 7.4, 8.9, 10.5, 11.7, 12.8, 14.1),
    x1 = c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10),
    x2 = c(0, 1, 0, 1, NA, 0, 1, NA, 1, 0)
  )

  m0 <- multi_reg(data = df, outcome = y, exposures = x1, approach = linear)
  m1 <- multi_reg(data = df, outcome = y, exposures = c(x1, x2), approach = linear)

  res <- compare_models(m0, m1, primary_exposure = x1, format = gt)
  source_notes <- unlist(res$table$`_source_notes`, recursive = TRUE, use.names = FALSE)

  expect_true(any(grepl("Comparison status: Different analysis sample", source_notes, fixed = TRUE)))
  expect_true(any(grepl("Different analysis sample: Models were fitted", source_notes, fixed = TRUE)))
  expect_false(any(grepl("<div", source_notes, fixed = TRUE)))
  expect_false(any(grepl("background-color", source_notes, fixed = TRUE)))
})

test_that("compare_models rejects raw fitted model objects", {
  df <- data.frame(
    y = c(2, 3, 5, 6, 8, 9, 11, 12),
    x1 = c(1, 2, 3, 4, 5, 6, 7, 8),
    x2 = c(0, 1, 0, 1, 0, 1, 0, 1)
  )

  m0 <- stats::lm(y ~ x1, data = df)
  m1 <- stats::lm(y ~ x1 + x2, data = df)

  expect_error(
    compare_models(m0, m1),
    "gtregression objects"
  )
})

test_that("compare_models supports multivariable Cox and parametric survival outputs", {
  skip_if_not_installed("survival")

  df <- data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes")),
      celltype = factor(
        celltype,
        levels = c("squamous", "smallcell", "adeno", "large"),
        labels = c("Squamous", "Small cell", "Adenocarcinoma", "Large cell")
      )
    )

  cox_m0 <- cox_reg(data = df, time = time, event = status, exposures = trt)
  cox_m1 <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, age, karno, celltype, prior),
    multivariable = TRUE
  )
  cox_res <- compare_models(cox_m0, cox_m1, primary_exposure = trt, format = tibble)

  expect_equal(cox_res$table_body$model_type, c("Cox regression", "Cox regression"))
  expect_equal(cox_res$table_body$model_terms, c("trt", "trt + age + karno + celltype + prior"))
  expect_equal(cox_res$table_body$events, c(cox_m0$models[[1]]$nevent, cox_m1$models[[1]]$nevent))
  expect_true(is.finite(cox_res$table_body$primary_estimate[2]))

  aft_m0 <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = trt,
    distribution = weibull
  )
  aft_m1 <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, age, karno, celltype, prior),
    distribution = weibull,
    multivariable = TRUE
  )
  aft_res <- compare_models(aft_m0, aft_m1, primary_exposure = trt, format = tibble)

  expect_equal(aft_res$table_body$model_type, c("Parametric survival", "Parametric survival"))
  expect_equal(aft_res$table_body$model_terms, c("trt", "trt + age + karno + celltype + prior"))
  expect_equal(aft_res$table_body$events, c(128, 128))
  expect_true(is.finite(aft_res$table_body$primary_estimate[2]))
})

test_that("compare_models rejects gtregression objects with multiple fitted models", {
  df <- data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )

  multi_exposure <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, prior)
  )

  expect_error(
    compare_models(multi_exposure, multi_exposure),
    "must contain one fitted model"
  )
})

birthwt_plot_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
      ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes")),
      ftv_cat = dplyr::case_when(
        ftv == 0 ~ "None",
        ftv == 1 ~ "One",
        ftv >= 2 ~ "Two or more"
      ),
      ftv_cat = factor(ftv_cat, levels = c("None", "One", "Two or more"))
    )
}

test_that("plot_reg returns ggplot and correct x-axis labels", {
  df <- birthwt_plot_data()

  tbl_logit <- uni_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "lwt", "smoke", "ht"),
    approach = logit
  )
  tbl_linear <- uni_reg(
    data = df,
    outcome = "bwt",
    exposures = c("age", "lwt", "smoke"),
    approach = linear
  )

  p_logit <- plot_reg(tbl_logit)
  p_linear <- plot_reg(tbl_linear)

  expect_s3_class(p_logit, "ggplot")
  expect_s3_class(p_linear, "ggplot")
  expect_match(p_logit$labels$x, "Odds Ratio", fixed = TRUE)
  expect_match(p_linear$labels$x, "Beta Coefficient", fixed = TRUE)
  expect_false(grepl("log scale", p_logit$labels$x, fixed = TRUE))

  expect_true(grepl("log scale", plot_reg(tbl_logit, log_x = TRUE)$labels$x, fixed = TRUE))
  expect_false(grepl("log scale", plot_reg(tbl_linear, log_x = TRUE)$labels$x, fixed = TRUE))
})

test_that("plot_reg supports multi_reg adjusted mode", {
  df <- birthwt_plot_data()

  tbl <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("smoke", "ht", "ui"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )

  p <- plot_reg(tbl)

  expect_s3_class(p, "ggplot")
  expect_match(p$labels$x, "Adjusted Odds Ratio", fixed = TRUE)
  expect_equal(p$labels$caption, "Adjusted for age and lwt. Ref. = reference category.")
  expect_equal(
    plot_reg(tbl, show_adjustment_note = FALSE)$labels$caption,
    "Ref. = reference category."
  )
  expect_equal(plot_reg(tbl, caption = "Custom note")$labels$caption, "Custom note")
  expect_true(all(c("smoke", "ht", "ui") %in% p$data$exposure))
})

test_that("plot_reg treats survival multivariable models as adjusted outputs", {
  skip_if_not_installed("survival")

  df <- data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )

  cox_tbl <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior, age, karno),
    multivariable = TRUE
  )
  surv_tbl <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior, age, karno),
    multivariable = TRUE
  )

  p_cox <- plot_reg(cox_tbl)
  p_surv <- plot_reg(surv_tbl)

  expect_match(p_cox$labels$x, "Adjusted Hazard Ratio", fixed = TRUE)
  expect_match(p_surv$labels$x, "Adjusted Time Ratio", fixed = TRUE)
  expect_equal(
    p_cox$labels$caption,
    "Adjusted for the other variables in the model. Ref. = reference category."
  )
  expect_equal(
    p_surv$labels$caption,
    "Adjusted for the other variables in the model. Ref. = reference category."
  )
})

test_that("plot_reg handles reference rows, ordering, significance, and x limits", {
  df <- birthwt_plot_data()

  tbl <- uni_reg(
    data = df,
    outcome = "low",
    exposures = c("smoke", "ht", "age"),
    approach = logit
  )

  p_ref <- plot_reg(tbl, show_ref = TRUE)
  p_no_ref <- plot_reg(tbl, show_ref = FALSE)

  expect_true(any(grepl("(Ref.)", p_ref$data$label_clean, fixed = TRUE)))
  expect_false(any(grepl("(Ref.)", p_no_ref$data$label_clean, fixed = TRUE)))
  expect_equal(p_ref$data$exposure[p_ref$data$is_header], c("smoke", "ht", "age"))
  expect_equal(p_ref$labels$caption, "Ref. = reference category.")
  expect_null(p_no_ref$labels$caption)
  expect_true("smoke" %in% p_no_ref$data$label)
  expect_true("ht" %in% p_no_ref$data$label)
  expect_true(any(p_no_ref$data$label == "smoke" & p_no_ref$data$is_header))
  expect_true(any(grepl("<b>smoke</b>", p_no_ref$data$label_clean, fixed = TRUE)))

  p_ordered <- plot_reg(tbl, order_y = c("age", "smoke", "ht"))
  expect_equal(p_ordered$data$exposure[p_ordered$data$is_header], c("age", "smoke", "ht"))

  p_breaks <- plot_reg(tbl, breaks = c(0.5, 1, 2, 4), xlim = c(0.25, 6))
  expect_s3_class(p_breaks, "ggplot")
  expect_equal(p_breaks$coordinates$limits$x, c(0.25, 6))

  p_log_breaks <- plot_reg(
    tbl,
    log_x = TRUE,
    breaks = c(0.5, 1, 2, 4),
    xlim = c(0.25, 6)
  )
  expect_equal(p_log_breaks$coordinates$limits$x, c(0.25, 6))
  expect_equal(p_log_breaks$scales$get_scales("x")$breaks, c(0.5, 1, 2, 4))

  p_log_default <- plot_reg(tbl, log_x = TRUE)
  expect_true(length(p_log_default$scales$get_scales("x")$breaks) > 1L)
  expect_lte(length(p_log_default$scales$get_scales("x")$breaks), 5L)
  expect_true(1 %in% p_log_default$scales$get_scales("x")$breaks)

  expect_true(all(!p_ref$data$is_header[p_ref$data$is_sig]))
  expect_true(all(!p_ref$data$ref[p_ref$data$is_sig]))

  lims <- p_ref$coordinates$limits$x
  expect_true(is.numeric(lims) && length(lims) == 2)
  expect_true(1 >= lims[1] && 1 <= lims[2])

  gclasses <- vapply(p_ref$layers, function(layer) class(layer$geom)[1], character(1))
  expect_true(any(grepl("GeomErrorbarh|GeomErrorbar", gclasses)))

  styled <- plot_reg(
    tbl,
    point_size = 4,
    point_stroke = 0.8,
    ci_linewidth = 0.9,
    base_size = 13
  )
  point_layer <- styled$layers[[which(grepl("GeomPoint", vapply(
    styled$layers, function(layer) class(layer$geom)[1], character(1)
  )))[1]]]
  expect_equal(point_layer$aes_params$size, 4)
  expect_equal(point_layer$aes_params$stroke, 0.8)
})

test_that("plot_reg inherits the table reference-row setting", {
  df <- birthwt_plot_data()

  compact_tbl <- uni_reg(
    data = df,
    outcome = "low",
    exposures = c("smoke", "ht", "age"),
    approach = logit,
    show_ref = FALSE
  )
  expanded_tbl <- uni_reg(
    data = df,
    outcome = "low",
    exposures = c("smoke", "ht", "age"),
    approach = logit,
    show_ref = TRUE
  )

  compact_plot <- plot_reg(compact_tbl)
  expanded_plot <- plot_reg(expanded_tbl)

  expect_false(any(grepl("(Ref.)", compact_plot$data$label_clean, fixed = TRUE)))
  expect_false(any(compact_plot$data$label %in% c("Yes", "No")))
  expect_true(all(c("smoke", "ht") %in% compact_plot$data$label))
  expect_true(all(compact_plot$data$is_header[compact_plot$data$label %in% c("smoke", "ht")]))
  expect_true(any(grepl("(Ref.)", expanded_plot$data$label_clean, fixed = TRUE)))

  override_plot <- plot_reg(expanded_tbl, show_ref = FALSE)
  expect_false(any(grepl("(Ref.)", override_plot$data$label_clean, fixed = TRUE)))
})

test_that("plot_reg uses p values for linear significance", {
  df <- birthwt_plot_data()

  tbl <- uni_reg(
    data = df,
    outcome = "bwt",
    exposures = c("age", "lwt"),
    approach = linear
  )

  p_alpha0 <- plot_reg(tbl, alpha = 0)
  p_alpha1 <- plot_reg(tbl, alpha = 1)

  sig0 <- sum(p_alpha0$data$is_sig & p_alpha0$data$is_data, na.rm = TRUE)
  sig1 <- sum(p_alpha1$data$is_sig & p_alpha1$data$is_data, na.rm = TRUE)

  expect_equal(sig0, 0L)
  expect_true(sig1 >= sig0)
})

test_that("plot_reg supports stratified objects and validates unsupported inputs", {
  df <- birthwt_plot_data()

  stratified <- suppressMessages(
    stratified_uni_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "smoke"),
      stratifier = "race",
      approach = logit
    )
  )

  p <- plot_reg(stratified)

  expect_s3_class(p, "ggplot")
  expect_true("stratum" %in% names(p$data))
  expect_equal(levels(p$data$stratum), levels(df$race))
  expect_true(any(vapply(p$facet$params$facets, rlang::as_label, character(1)) == "stratum"))

  expect_error(
    plot_reg_combine(stratified, stratified),
    "does not support stratified objects"
  )

  expect_error(plot_reg(mtcars), "gtregression object")
})

test_that("plot_reg keeps parent variable labels in stratified facets", {
  df <- birthwt_plot_data()
  attr(df$smoke, "label") <- "Smoking during pregnancy"
  attr(df$ht, "label") <- "Maternal hypertension"

  stratified <- suppressMessages(
    stratified_uni_reg(
      data = df,
      outcome = "low",
      exposures = c("smoke", "ht"),
      stratifier = "race",
      approach = logit
    )
  )

  p <- plot_reg(stratified)

  expect_true(any(
    p$data$is_header & p$data$label == "Smoking during pregnancy"
  ))
  expect_true(any(
    p$data$is_header & p$data$label == "Maternal hypertension"
  ))
  expect_false(any(p$data$is_header & p$data$label %in% c("smoke", "ht")))
  expect_true(all(c("Smoking during pregnancy", "Maternal hypertension") %in%
                    p$data$label[p$data$is_header]))

  built <- ggplot2::ggplot_build(p)
  rendered_labels <- unlist(lapply(built$layout$panel_params, function(panel) {
    panel$y$get_labels()
  }), use.names = FALSE)
  expect_true(any(grepl("Smoking during pregnancy", rendered_labels, fixed = TRUE)))
  expect_true(any(grepl("Maternal hypertension", rendered_labels, fixed = TRUE)))
})

test_that("plot_reg supports adjusted stratified and survival outputs consistently", {
  skip_if_not_installed("survival")

  df <- birthwt_plot_data()
  stratified_multi <- suppressMessages(
    stratified_multi_reg(
      data = df,
      outcome = "low",
      exposures = c("smoke", "ht"),
      adjust_for = c("age", "lwt"),
      stratifier = "race",
      approach = logit
    )
  )

  p_multi <- plot_reg(
    stratified_multi,
    show_ref = FALSE,
    log_x = TRUE,
    xlim = c(0.2, 20),
    breaks = c(0.5, 1, 2, 4, 8)
  )

  expect_s3_class(p_multi, "ggplot")
  expect_match(p_multi$labels$x, "Adjusted Odds Ratio", fixed = TRUE)
  expect_true("stratum" %in% names(p_multi$data))
  expect_equal(levels(p_multi$data$stratum), levels(df$race))
  expect_equal(p_multi$coordinates$limits$x, c(0.2, 20))
  expect_equal(p_multi$scales$get_scales("x")$breaks, c(0.5, 1, 2, 4, 8))
  expect_true(any(p_multi$data$label == "smoke" & p_multi$data$is_header))
  expect_false(any(grepl("(Ref.)", p_multi$data$label_clean, fixed = TRUE)))

  lung <- data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )

  stratified_cox <- cox_reg(
    data = lung,
    time = time,
    event = status,
    exposures = c(celltype, prior),
    adjust_for = c(age, karno),
    stratifier = trt
  )
  stratified_surv <- surv_reg(
    data = lung,
    time = time,
    event = status,
    exposures = c(celltype, prior),
    adjust_for = c(age, karno),
    stratifier = trt
  )

  p_cox <- plot_reg(stratified_cox, log_x = TRUE)
  p_surv <- plot_reg(stratified_surv, log_x = TRUE)

  expect_match(p_cox$labels$x, "Adjusted Hazard Ratio", fixed = TRUE)
  expect_match(p_surv$labels$x, "Adjusted Time Ratio", fixed = TRUE)
  expect_equal(levels(p_cox$data$stratum), levels(lung$trt))
  expect_equal(levels(p_surv$data$stratum), levels(lung$trt))
  expect_true(any(vapply(p_cox$facet$params$facets, rlang::as_label, character(1)) == "stratum"))
  expect_true(any(vapply(p_surv$facet$params$facets, rlang::as_label, character(1)) == "stratum"))
})

test_that("plot_reg validates malformed gtregression objects", {
  good_body <- data.frame(
    exposure = "age",
    level = "age",
    estimate = 1,
    conf.low = 0.8,
    conf.high = 1.2,
    p.value = 0.5,
    ref = FALSE
  )
  good_display <- data.frame(Characteristic = "age", is_header = TRUE)

  no_source <- structure(
    list(table_body = good_body, table_display = good_display, approach = "logit"),
    class = "gtregression"
  )
  no_body <- structure(
    list(table_display = good_display, source = "uni_reg", approach = "logit"),
    class = "gtregression"
  )
  no_display <- structure(
    list(table_body = good_body, source = "uni_reg", approach = "logit"),
    class = "gtregression"
  )
  bad_body <- structure(
    list(
      table_body = good_body[, setdiff(names(good_body), "ref"), drop = FALSE],
      table_display = good_display,
      source = "uni_reg",
      approach = "logit"
    ),
    class = "gtregression"
  )
  bad_display <- structure(
    list(
      table_body = good_body,
      table_display = data.frame(Characteristic = "age"),
      source = "uni_reg",
      approach = "logit"
    ),
    class = "gtregression"
  )

  expect_error(plot_reg(no_source), "source")
  expect_error(plot_reg(no_body), "table_body")
  expect_error(plot_reg(no_display), "table_display")
  expect_error(plot_reg(bad_body), "requires `table_body`")
  expect_error(plot_reg(bad_display), "requires `table_display`")
})

test_that("plot_reg includes non-factor extra rows when display has only headers", {
  body <- data.frame(
    exposure = c("age", "age"),
    level = c("age", "age:smokeYes"),
    estimate = c(1.1, 1.3),
    conf.low = c(0.9, 1.0),
    conf.high = c(1.4, 1.8),
    p.value = c(0.2, 0.04),
    ref = c(FALSE, FALSE)
  )
  display <- data.frame(Characteristic = "age", is_header = TRUE)
  tbl <- structure(
    list(
      table_body = body,
      table_display = display,
      source = "uni_reg",
      approach = "logit"
    ),
    class = "gtregression"
  )

  p <- plot_reg(tbl)

  expect_true("age:smokeYes" %in% p$data$label)
})

#' Compare Prespecified Regression Models
#'
#' Compare gtregression candidate models side by side using model-fit statistics.
#' This is intended for transparent model comparison after you have already
#' fitted the candidate models with functions such as \code{multi_reg()},
#' \code{cox_reg()}, or \code{surv_reg()}.
#'
#' @param ... Two or more gtregression model objects, or one list containing
#'   them. Inputs should be outputs from \code{multi_reg()}, \code{cox_reg()},
#'   or \code{surv_reg()}.
#' @param model_names Optional character vector of names to display. If omitted,
#'   names supplied in \code{...} are used; otherwise models are labelled
#'   \code{Model 1}, \code{Model 2}, etc.
#' @param nested Logical. If \code{TRUE}, likelihood-ratio statistics are
#'   calculated sequentially by comparing each model with the previous model.
#'   Use this only when models are nested and supplied in the intended order.
#' @param primary_exposure Optional exposure or exact coefficient name to track
#'   across models. For Cox models this can be used to show the hazard ratio
#'   and percentage change in the log-effect estimate across candidate models.
#' @param exponentiate Logical. If \code{NULL}, Cox, logistic, Poisson,
#'   negative-binomial, and parametric survival models are exponentiated by
#'   default, while linear models are not.
#' @param digits Number of digits for model statistics and estimates.
#' @param p_digits Number of digits for p-values.
#' @param format Output format. Defaults to \code{"flextable"}.
#' @param theme Table theme preset.
#'
#' @return A \code{gtregression} object with:
#' \itemize{
#'   \item \code{table}: publication-ready table
#'   \item \code{table_body}: raw comparison statistics
#'   \item \code{table_display}: formatted display data
#'   \item \code{models}: fitted models compared
#'   \item \code{comparison_status}: whether models appear to use the same
#'     analysis sample
#'   \item \code{comparison_warnings}: caution messages that are highlighted
#'     in rendered tables when interpretation needs extra care
#' }
#'
#' @details
#' \code{compare_models()} does not refit models and does not perform hidden
#' complete-case filtering. It compares models already fitted by gtregression
#' and extracts the single fitted model stored in each object's \code{models}
#' element. The reported N, event counts, and fit statistics therefore come from
#' the model already fitted by \code{multi_reg()}, \code{cox_reg()}, or
#' \code{surv_reg()}. This keeps model comparison separate from model
#' selection: compare candidate models first, then choose the final model using
#' clinical, epidemiological, and statistical judgement.
#' The guided builder in \code{gtregression_app()} can create the fitted
#' gtregression objects before calling this function; the generated app code
#' exposes every model formula and comparison call.
#'
#' Likelihood-ratio p-values are meaningful only for nested models fitted to
#' the same analysis sample. \code{compare_models()} checks whether the fitted
#' models appear to use the same analysis sample using retained model row
#' identifiers when available; otherwise it compares N and event counts. It
#' also checks whether sequential model pairs appear to be nested when
#' \code{nested = TRUE}. Rendered warnings are context-aware: no warning about
#' different analysis samples is shown when the compared models use the same
#' observations, and no nested-model warning is shown when sequential models
#' appear nested. AIC, BIC, log-likelihood, and likelihood-ratio statistics
#' remain visible when warnings are needed, but should then be interpreted with
#' the displayed caution.
#'
#' @examples
#' data("data_lungcancer", package = "gtregression")
#'
#' lung_data <- data_lungcancer
#'
#' cox_1 <- cox_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = trt
#' )
#'
#' cox_2 <- cox_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = trt,
#'   adjust_for = c(age, karno)
#' )
#'
#' compare_models(
#'   cox_1,
#'   cox_2,
#'   primary_exposure = trt
#' )
#'
#' compare_models(
#'   cox_1,
#'   cox_2,
#'   model_names = c("Treatment only", "Treatment + age + performance"),
#'   primary_exposure = trt
#' )
#'
#' @seealso \code{gtregression_app()}, \code{multi_reg()}, \code{cox_reg()},
#'   \code{surv_reg()}, \code{select_models()}
#'
#' @export
compare_models <- function(...,
                           model_names = NULL,
                           nested = TRUE,
                           primary_exposure = NULL,
                           exponentiate = NULL,
                           digits = 2,
                           p_digits = 3,
                           format = c("flextable", "gt", "tibble"),
                           theme = c("minimal")) {
  input_names <- .compare_models_input_names(substitute(list(...)))
  models <- .compare_models_list(...)
  if (length(models) < 2L) {
    stop("Supply at least two gtregression model objects to compare.", call. = FALSE)
  }

  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  format <- match.arg(format, c("flextable", "gt", "tibble"))
  theme <- .choice_arg(substitute(theme), env = parent.frame())
  theme <- theme[1]
  theme <- .resolve_theme(theme)

  primary_exposure <- .vars_arg(
    substitute(primary_exposure),
    env = parent.frame(),
    allow_null = TRUE
  )

  model_names <- .compare_model_names(models, model_names, input_names = input_names)
  names(models) <- model_names

  table_body <- .compare_models_body(
    models = models,
    nested = nested,
    primary_exposure = primary_exposure,
    exponentiate = exponentiate
  )
  comparison_status <- .compare_models_status(models, table_body)
  table_body$comparison_status <- comparison_status$status
  table_body$comparison_status_detail <- comparison_status$detail

  table_display <- .compare_models_display(
    table_body,
    digits = digits,
    p_digits = p_digits,
    primary_exposure = primary_exposure
  )
  table_notes <- .compare_models_notes(
    body = table_body,
    nested = nested,
    primary_exposure = primary_exposure,
    comparison_status = comparison_status
  )

  out <- list(
    table = NULL,
    table_body = table_body,
    table_display = table_display,
    models = models,
    model_names = model_names,
    primary_exposure = primary_exposure,
    comparison_status = comparison_status,
    comparison_warnings = table_notes$warnings,
    footnotes = c(table_notes$warnings, table_notes$notes),
    format = format,
    source = "compare_models"
  )

  if (format != "tibble") {
    out$table <- .build_compare_models_table(
      display = table_display,
      body = table_body,
      format = format,
      theme = theme,
      nested = nested,
      primary_exposure = primary_exposure,
      comparison_status = comparison_status,
      table_notes = table_notes
    )
  }

  class(out) <- c("gtregression", "compare_models")
  out
}

#' @keywords internal
#' @noRd
.compare_models_input_names <- function(expr) {
  exprs <- as.list(expr)[-1L]
  if (!length(exprs)) {
    return(character(0))
  }

  explicit <- names(exprs)
  if (is.null(explicit)) {
    explicit <- rep("", length(exprs))
  }

  if (length(exprs) == 1L && identical(explicit, "")) {
    return(character(0))
  }

  labels <- vapply(seq_along(exprs), function(i) {
    if (nzchar(explicit[i])) {
      return(explicit[i])
    }

    if (is.symbol(exprs[[i]])) {
      return(as.character(exprs[[i]]))
    }

    ""
  }, character(1))

  labels
}

#' @keywords internal
#' @noRd
.compare_models_list <- function(...) {
  inputs <- list(...)
  if (
    length(inputs) == 1L &&
      is.list(inputs[[1L]]) &&
      !.is_compare_gtregression_model(inputs[[1L]])
  ) {
    inputs <- inputs[[1L]]
  }

  if (!length(inputs)) {
    stop("Supply gtregression model objects to compare.", call. = FALSE)
  }

  invalid_inputs <- vapply(inputs, function(x) !.is_compare_gtregression_model(x), logical(1))
  if (any(invalid_inputs)) {
    stop(
      "All inputs must be gtregression objects from multi_reg(), cox_reg(), or surv_reg().",
      call. = FALSE
    )
  }

  models <- lapply(inputs, .compare_extract_model)
  model_terms <- Map(.compare_gtregression_terms, inputs, models)
  for (i in seq_along(models)) {
    attr(models[[i]], "gtregression_model_terms") <- model_terms[[i]]
  }

  invalid <- vapply(models, function(x) !.is_compare_fitted_model(x), logical(1))
  if (any(invalid)) {
    stop(
      "The supplied gtregression objects must contain fitted models with a logLik() method.",
      call. = FALSE
    )
  }

  names(models) <- names(inputs)
  models
}

#' @keywords internal
#' @noRd
.is_compare_gtregression_model <- function(x) {
  inherits(x, "gtregression") &&
    (inherits(x, "multi_reg") || inherits(x, "cox_reg") || inherits(x, "surv_reg")) &&
    is.list(x$models) &&
    length(x$models) > 0L
}

#' @keywords internal
#' @noRd
.is_compare_fitted_model <- function(x) {
  is.list(x) && !is.null(tryCatch(stats::logLik(x), error = function(e) NULL))
}

#' @keywords internal
#' @noRd
.compare_extract_model <- function(x) {
  if (.is_compare_gtregression_model(x)) {
    if (length(x$models) != 1L) {
      stop(
        paste0(
          "Each gtregression object supplied to compare_models() must contain one fitted model. ",
          "Use multivariable = TRUE for a single full model, or supply one exposure per ",
          "candidate model when using adjusted-exposure mode."
        ),
        call. = FALSE
      )
    }
    return(x$models[[1L]])
  }

  x
}

#' @keywords internal
#' @noRd
.compare_model_names <- function(models, model_names = NULL, input_names = NULL) {
  if (!is.null(model_names)) {
    if (!is.character(model_names) || length(model_names) != length(models)) {
      stop("`model_names` must be a character vector with one name per model.", call. = FALSE)
    }
    return(model_names)
  }

  nm <- names(models)
  if (!is.null(nm) && all(nzchar(nm))) {
    return(nm)
  }

  if (!is.null(input_names) && length(input_names) == length(models) && all(nzchar(input_names))) {
    return(input_names)
  }

  paste("Model", seq_along(models))
}

#' @keywords internal
#' @noRd
.compare_models_body <- function(models,
                                 nested = TRUE,
                                 primary_exposure = NULL,
                                 exponentiate = NULL) {
  rows <- lapply(seq_along(models), function(i) {
    fit <- models[[i]]
    ll <- .safe_loglik(fit)
    type <- .compare_model_type(fit)
    exp_flag <- .compare_exponentiate(fit, exponentiate)
    primary <- .compare_primary_effect(fit, primary_exposure, exp_flag)

    data.frame(
      model = names(models)[i],
      model_type = type,
      formula = .compare_model_formula(fit),
      model_terms = .compare_model_terms_label(fit),
      n = .compare_model_n(fit),
      events = .compare_model_events(fit),
      parameters = .safe_numeric(attr(ll, "df", exact = TRUE)),
      AIC = .safe_numeric(stats::AIC(fit)),
      BIC = .safe_numeric(stats::BIC(fit)),
      logLik = .safe_numeric(as.numeric(ll)),
      LR_chisq = NA_real_,
      LR_df = NA_real_,
      p.value = NA_real_,
      nested_comparison = NA,
      concordance = .compare_model_concordance(fit),
      primary_term = primary$term,
      primary_log_estimate = primary$log_estimate,
      primary_estimate = primary$estimate,
      primary_pct_change = NA_real_,
      stringsAsFactors = FALSE
    )
  })

  out <- do.call(rbind, rows)

  if (isTRUE(nested) && nrow(out) > 1L) {
    for (i in 2:nrow(out)) {
      out$nested_comparison[i] <- .compare_models_nested_pair(models[[i - 1L]], models[[i]])
      lr <- 2 * (out$logLik[i] - out$logLik[i - 1L])
      df <- out$parameters[i] - out$parameters[i - 1L]
      if (is.finite(lr) && is.finite(df) && lr >= 0 && df > 0) {
        out$LR_chisq[i] <- lr
        out$LR_df[i] <- df
        out$p.value[i] <- stats::pchisq(lr, df = df, lower.tail = FALSE)
      }
    }
  }

  ref_coef <- out$primary_log_estimate[is.finite(out$primary_log_estimate)][1]
  if (!is.na(ref_coef) && is.finite(ref_coef) && !isTRUE(all.equal(ref_coef, 0))) {
    out$primary_pct_change <- ((out$primary_log_estimate - ref_coef) / abs(ref_coef)) * 100
  }

  out$best_AIC <- is.finite(out$AIC) & out$AIC == min(out$AIC, na.rm = TRUE)
  out$best_BIC <- is.finite(out$BIC) & out$BIC == min(out$BIC, na.rm = TRUE)

  out
}

#' @keywords internal
#' @noRd
.compare_models_nested_pair <- function(smaller, larger) {
  small_terms <- .compare_model_terms(smaller)
  large_terms <- .compare_model_terms(larger)

  if (is.null(small_terms) || is.null(large_terms)) {
    return(NA)
  }

  all(small_terms %in% large_terms)
}

#' @keywords internal
#' @noRd
.compare_model_terms <- function(model) {
  out <- tryCatch(attr(stats::terms(model), "term.labels"), error = function(e) NULL)
  if (is.null(out)) {
    return(NULL)
  }
  as.character(out)
}

#' @keywords internal
#' @noRd
.gtregression_terms_label <- function(vars) {
  vars <- unique(as.character(vars))
  vars <- vars[!is.na(vars) & nzchar(vars)]
  if (!length(vars)) {
    return("Intercept only")
  }
  paste(vars, collapse = " + ")
}

#' @keywords internal
#' @noRd
.compare_gtregression_terms <- function(x, fit) {
  terms <- character(0)
  if (.is_compare_gtregression_model(x)) {
    terms <- c(terms, x$exposures)
    if (isTRUE(x$adjusted_mode)) {
      terms <- c(terms, x$adjust_for)
    }
    terms <- c(terms, x$interaction)
  }

  if (!length(terms)) {
    terms <- .compare_model_terms(fit)
  }

  .gtregression_terms_label(terms)
}

#' @keywords internal
#' @noRd
.compare_model_terms_label <- function(model) {
  stored <- attr(model, "gtregression_model_terms", exact = TRUE)
  if (!is.null(stored) && length(stored) == 1L && nzchar(stored)) {
    return(stored)
  }

  .gtregression_terms_label(.compare_model_terms(model))
}

#' @keywords internal
#' @noRd
.compare_models_status <- function(models, table_body) {
  rows <- lapply(models, .compare_model_rows)
  have_rows <- vapply(rows, function(x) length(x) > 0L, logical(1))

  same_rows <- NA
  if (all(have_rows) && length(rows) > 1L) {
    first_rows <- rows[[1L]]
    same_rows <- all(vapply(rows[-1L], identical, logical(1), y = first_rows))
  }

  same_n <- length(unique(table_body$n[is.finite(table_body$n)])) <= 1L
  finite_events <- table_body$events[is.finite(table_body$events)]
  same_events <- !length(finite_events) || length(unique(finite_events)) <= 1L

  same_sample <- if (!is.na(same_rows)) {
    same_rows
  } else {
    same_n && same_events
  }

  if (isTRUE(same_sample)) {
    detail <- if (!is.na(same_rows)) {
      "Same analysis sample; assessed using retained model row identifiers."
    } else {
      "Same analysis sample; assessed using N and event counts because retained row identifiers were unavailable."
    }

    return(list(
      status = "Same analysis sample",
      same_sample = TRUE,
      row_check_available = all(have_rows),
      detail = detail
    ))
  }

  detail <- if (!is.na(same_rows)) {
    "Different analysis sample; retained model row identifiers differed between models."
  } else {
    "Different analysis sample; N or event counts differed between models."
  }

  list(
    status = "Different analysis sample",
    same_sample = FALSE,
    row_check_available = all(have_rows),
    detail = detail
  )
}

#' @keywords internal
#' @noRd
.compare_model_rows <- function(model) {
  mf <- tryCatch(stats::model.frame(model), error = function(e) NULL)
  rn <- if (!is.null(mf)) rownames(mf) else NULL
  if (!is.null(rn) && length(rn)) {
    return(as.character(rn))
  }

  stored <- tryCatch(model$model, error = function(e) NULL)
  rn <- if (!is.null(stored)) rownames(stored) else NULL
  if (!is.null(rn) && length(rn)) {
    return(as.character(rn))
  }

  character(0)
}

#' @keywords internal
#' @noRd
.safe_loglik <- function(model) {
  tryCatch(stats::logLik(model), error = function(e) structure(NA_real_, df = NA_real_))
}

#' @keywords internal
#' @noRd
.compare_model_type <- function(model) {
  if (inherits(model, "coxph")) return("Cox regression")
  if (inherits(model, "survreg")) return("Parametric survival")
  if (inherits(model, "negbin")) return("Negative binomial")
  if (inherits(model, "glm")) {
    fam <- tryCatch(model$family$family, error = function(e) "")
    if (identical(fam, "binomial")) return("Logistic regression")
    if (identical(fam, "poisson")) return("Poisson regression")
    return("Generalized linear model")
  }
  if (inherits(model, "lm")) return("Linear regression")
  class(model)[1]
}

#' @keywords internal
#' @noRd
.compare_exponentiate <- function(model, exponentiate = NULL) {
  if (!is.null(exponentiate)) {
    return(isTRUE(exponentiate))
  }

  if (inherits(model, c("coxph", "survreg", "negbin"))) {
    return(TRUE)
  }

  if (inherits(model, "glm")) {
    fam <- tryCatch(model$family$family, error = function(e) "")
    return(fam %in% c("binomial", "poisson"))
  }

  FALSE
}

#' @keywords internal
#' @noRd
.compare_model_formula <- function(model) {
  out <- tryCatch(stats::formula(model), error = function(e) NULL)
  if (is.null(out)) {
    return(NA_character_)
  }
  paste(deparse(out), collapse = " ")
}

#' @keywords internal
#' @noRd
.compare_model_n <- function(model) {
  if (inherits(model, "coxph") && !is.null(model$n)) {
    return(.safe_numeric(model$n))
  }
  .safe_numeric(stats::nobs(model))
}

#' @keywords internal
#' @noRd
.compare_model_events <- function(model) {
  if (inherits(model, "coxph") && !is.null(model$nevent)) {
    return(.safe_numeric(model$nevent))
  }

  if (inherits(model, "survreg")) {
    attr_events <- attr(model, "gtregression_events", exact = TRUE)
    if (!is.null(attr_events)) {
      return(.safe_numeric(attr_events))
    }
  }

  NA_real_
}

#' @keywords internal
#' @noRd
.compare_model_concordance <- function(model) {
  if (!inherits(model, "coxph")) {
    return(NA_real_)
  }
  out <- tryCatch(summary(model)$concordance[1], error = function(e) NA_real_)
  .safe_numeric(out)
}

#' @keywords internal
#' @noRd
.compare_primary_effect <- function(model, primary_exposure = NULL, exponentiate = TRUE) {
  empty <- list(term = NA_character_, log_estimate = NA_real_, estimate = NA_real_)
  if (is.null(primary_exposure) || !length(primary_exposure)) {
    return(empty)
  }

  coefs <- tryCatch(stats::coef(model), error = function(e) NULL)
  if (is.null(coefs) || !length(coefs)) {
    return(empty)
  }

  names_coefs <- names(coefs)
  primary_exposure <- as.character(primary_exposure)[1]
  idx <- which(names_coefs == primary_exposure)

  if (!length(idx)) {
    idx <- which(startsWith(names_coefs, primary_exposure))
  }

  if (length(idx) != 1L) {
    return(empty)
  }

  log_est <- suppressWarnings(as.numeric(coefs[idx]))
  est <- if (isTRUE(exponentiate)) exp(log_est) else log_est

  list(
    term = names_coefs[idx],
    log_estimate = log_est,
    estimate = est
  )
}

#' @keywords internal
#' @noRd
.compare_models_display <- function(table_body,
                                    digits = 2,
                                    p_digits = 3,
                                    primary_exposure = NULL) {
  display <- data.frame(
    Model = table_body$model,
    Variables = table_body$model_terms,
    N = .fmt_count(table_body$n),
    Parameters = .fmt_count(table_body$parameters),
    AIC = .fmt_num_compare(table_body$AIC, digits),
    BIC = .fmt_num_compare(table_body$BIC, digits),
    `Best AIC` = ifelse(table_body$best_AIC, "Yes", "No"),
    `Best BIC` = ifelse(table_body$best_BIC, "Yes", "No"),
    `Log-likelihood` = .fmt_num_compare(table_body$logLik, digits),
    `LR chi-square` = .fmt_num_compare(table_body$LR_chisq, digits),
    df = .fmt_count(table_body$LR_df),
    `p-value` = .fmt_p_compare(table_body$p.value, p_digits),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  if (any(is.finite(table_body$events))) {
    display <- data.frame(
      display[, c("Model", "Variables", "N"), drop = FALSE],
      Events = .fmt_count(table_body$events),
      display[, setdiff(names(display), c("Model", "Variables", "N")), drop = FALSE],
      check.names = FALSE
    )
  }

  if (any(is.finite(table_body$concordance))) {
    display$Concordance <- .fmt_num_compare(table_body$concordance, 3)
  }

  if (!is.null(primary_exposure) && any(is.finite(table_body$primary_estimate))) {
    display$`Primary estimate` <- .fmt_num_compare(table_body$primary_estimate, digits)
    display$`Change from first` <- ifelse(
      is.finite(table_body$primary_pct_change),
      paste0(.fmt_num_compare(table_body$primary_pct_change, digits), "%"),
      ""
    )
  }

  display
}

#' @keywords internal
#' @noRd
.fmt_num_compare <- function(x, digits = 2) {
  out <- ifelse(is.finite(x), formatC(x, format = "f", digits = digits), "")
  as.character(out)
}

#' @keywords internal
#' @noRd
.fmt_count <- function(x) {
  out <- ifelse(is.finite(x), formatC(x, format = "f", digits = 0), "")
  as.character(out)
}

#' @keywords internal
#' @noRd
.fmt_p_compare <- function(x, digits = 3) {
  out <- rep("", length(x))
  ok <- is.finite(x)
  if (any(ok)) {
    threshold <- 10^(-digits)
    out[ok & x < threshold] <- paste0("<", formatC(threshold, format = "f", digits = digits))
    out[ok & x >= threshold] <- formatC(x[ok & x >= threshold], format = "f", digits = digits)
  }
  out
}

#' @keywords internal
#' @noRd
.compare_status_note <- function(comparison_status = NULL) {
  if (is.null(comparison_status) || is.null(comparison_status$status)) {
    return("Comparison status: not assessed.")
  }

  paste0(
    "Comparison status: ",
    comparison_status$status,
    ". ",
    comparison_status$detail
  )
}

#' @keywords internal
#' @noRd
.compare_nested_status_note <- function(body, nested = TRUE) {
  if (!isTRUE(nested)) {
    return("Nested-model status: not assessed because `nested = FALSE`.")
  }

  checked <- body$nested_comparison[!is.na(body$nested_comparison)]
  if (!length(checked)) {
    return("Nested-model status: not assessed for the first model.")
  }

  if (any(checked %in% FALSE)) {
    return("Nested-model status: one or more sequential model comparisons are not nested.")
  }

  "Nested-model status: sequential models are nested."
}

#' @keywords internal
#' @noRd
.compare_models_notes <- function(body,
                                  nested = TRUE,
                                  primary_exposure = NULL,
                                  comparison_status = NULL) {
  notes <- c(
    .compare_status_note(comparison_status),
    .compare_nested_status_note(body, nested = nested),
    "Compare prespecified candidate models; lower AIC or BIC indicates better relative fit among the compared models."
  )

  warnings <- character(0)

  if (!is.null(comparison_status) && isFALSE(comparison_status$same_sample)) {
    warnings <- c(
      warnings,
      paste0(
        paste0(
          "Different analysis sample: Models were fitted to different analysis samples because of ",
          "missing data or differing inclusion criteria. "
        ),
        paste0(
          "AIC, BIC, log-likelihood and likelihood-ratio statistics are presented for completeness ",
          "but should not be interpreted as formal model-selection criteria across different datasets."
        )
      )
    )
  } else {
    notes <- c(
      notes,
      paste0(
        "Models were fitted to the same analysis sample. AIC, BIC, log-likelihood and ",
        "likelihood-ratio tests may be interpreted as formal model-comparison statistics when ",
        "the models are nested as required."
      )
    )
  }

  if (isTRUE(nested) && any(body$nested_comparison %in% FALSE)) {
    warnings <- c(
      warnings,
      paste0(
        "Non-nested comparison: Likelihood-ratio statistics should be interpreted with caution ",
        "because one or more sequential model pairs do not appear to be nested based on their ",
        "model terms."
      )
    )
  }

  if (!is.null(primary_exposure) && any(is.finite(body$primary_estimate))) {
    notes <- c(
      notes,
      paste0(
        "Primary estimate change is calculated on the coefficient/log-effect scale before ",
        "exponentiation and can help assess robustness across candidate models."
      )
    )
  }

  list(
    warnings = unique(warnings[nzchar(warnings)]),
    notes = unique(notes[nzchar(notes)])
  )
}

#' @keywords internal
#' @noRd
.build_compare_models_table <- function(display,
                                        body,
                                        format = c("flextable", "gt"),
                                        theme = c("minimal"),
                                        nested = TRUE,
                                        primary_exposure = NULL,
                                        comparison_status = NULL,
                                        table_notes = NULL) {
  format <- match.arg(format, c("flextable", "gt"))
  if (is.null(table_notes)) {
    table_notes <- .compare_models_notes(
      body = body,
      nested = nested,
      primary_exposure = primary_exposure,
      comparison_status = comparison_status
    )
  }
  warnings <- table_notes$warnings
  notes <- table_notes$notes
  status_notes <- notes[grepl("^(Comparison status|Nested-model status):", notes)]
  routine_notes <- notes[!notes %in% status_notes]

  if (format == "gt") {
    tbl <- gt::gt(display) |>
      gt::tab_header(title = "Model comparison") |>
      gt::cols_align(align = "left", columns = c("Model", "Variables")) |>
      gt::cols_align(align = "center", columns = setdiff(names(display), c("Model", "Variables"))) |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_column_labels()
      )

    if (length(status_notes)) {
      tbl <- gt::tab_source_note(tbl, gt::md(paste(status_notes, collapse = "<br>")))
    }

    if (length(warnings)) {
      warning_md <- paste(paste0("**", warnings, "**"), collapse = "<br>")
      tbl <- gt::tab_source_note(tbl, gt::md(warning_md))
    }

    if (length(routine_notes)) {
      tbl <- gt::tab_source_note(tbl, gt::md(paste(routine_notes, collapse = "<br>")))
    }

    tbl <- .compact_gt_source_notes(tbl)

    best_rows <- which(body$best_AIC | body$best_BIC)
    if (length(best_rows)) {
      tbl <- gt::tab_style(
        tbl,
        style = gt::cell_fill(color = "#e8f5e9"),
        locations = gt::cells_body(rows = best_rows)
      )
    }

    if ("header_shaded" %in% theme) {
      tbl <- gt::tab_options(tbl, column_labels.background.color = "#f6f8fa")
    }
    if ("zebra" %in% theme) {
      tbl <- gt::opt_row_striping(tbl)
    }
    if ("compact" %in% theme) {
      tbl <- gt::tab_options(tbl, data_row.padding = gt::px(2))
    }
    return(tbl)
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Model comparison")
  ft <- flextable::align(ft, j = c("Model", "Variables"), align = "left", part = "all")
  ft <- flextable::align(ft, j = setdiff(names(display), c("Model", "Variables")), align = "center", part = "all")
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  if ("header_shaded" %in% theme) {
    ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  }
  if ("zebra" %in% theme && nrow(display) > 1L) {
    ft <- flextable::bg(ft, i = seq(1, nrow(display), by = 2), bg = "#f6f8fa", part = "body")
  }
  best_rows <- which(body$best_AIC | body$best_BIC)
  if (length(best_rows)) {
    ft <- flextable::bg(ft, i = best_rows, bg = "#e8f5e9", part = "body")
  }
  if ("compact" %in% theme) {
    ft <- flextable::padding(ft, padding = 2, part = "body")
  }
  footer_lines <- c(status_notes, warnings, routine_notes)
  ft <- flextable::add_footer_lines(ft, values = footer_lines)
  if (length(warnings)) {
    warning_rows <- length(status_notes) + seq_len(length(warnings))
    ft <- flextable::bold(ft, i = warning_rows, bold = TRUE, part = "footer")
    ft <- flextable::bg(ft, i = warning_rows, bg = "#fff3cd", part = "footer")
    ft <- flextable::color(ft, i = warning_rows, color = "#5f4700", part = "footer")
  }
  ft <- .compact_flex_footer(ft)
  note_rows <- if (length(footer_lines)) seq_len(length(footer_lines)) else integer(0)
  note_rows <- setdiff(note_rows, if (length(warnings)) warning_rows else integer(0))
  if (length(note_rows)) {
    ft <- flextable::italic(ft, i = note_rows, italic = TRUE, part = "footer")
  }
  flextable::autofit(ft)
}

#' @keywords internal
#' @noRd
.model_data_col <- function(fit, variable) {
  mf <- fit$model
  if (is.null(mf)) {
    mf <- attr(fit, "gtregression_model_frame", exact = TRUE)
  }
  if (is.null(mf) || !variable %in% names(mf)) {
    return(NULL)
  }
  x <- mf[[variable]]

  # R's modelling machinery expands character predictors as treatment-coded
  # categorical variables, but model.frame() may retain their character class.
  # Normalise only for table reconstruction so every fitted level is displayed.
  if (is.character(x)) {
    x <- factor(x)
  }

  x
}

#' @keywords internal
#' @noRd
.tidy_coefficients <- function(fit, approach) {
  approach <- .normalize_approach(approach)

  if (identical(approach, "firth")) {
    est <- tryCatch(stats::coef(fit), error = function(e) fit$coefficients)
    if (is.null(est)) return(NULL)
    term <- names(est)
    if (is.null(term)) term <- names(fit$coefficients)
    if (is.null(term)) return(NULL)

    lo <- fit$ci.lower
    hi <- fit$ci.upper
    p  <- fit$prob

    if (is.null(lo) || is.null(hi)) {
      se <- tryCatch(sqrt(diag(fit$var)), error = function(e) rep(NA_real_, length(est)))
      z <- stats::qnorm(0.975)
      lo <- est - z * se
      hi <- est + z * se
    }

    if (is.null(p)) {
      se <- tryCatch(sqrt(diag(fit$var)), error = function(e) rep(NA_real_, length(est)))
      p <- 2 * stats::pnorm(abs(est / se), lower.tail = FALSE)
    }

    out <- data.frame(
      term = term,
      estimate = as.numeric(est),
      conf.low = as.numeric(lo[term]),
      conf.high = as.numeric(hi[term]),
      p.value = as.numeric(p[term]),
      stringsAsFactors = FALSE
    )

    missing_ci <- is.na(out$conf.low) | is.na(out$conf.high)
    if (any(missing_ci)) {
      out$conf.low[missing_ci] <- as.numeric(lo)[missing_ci]
      out$conf.high[missing_ci] <- as.numeric(hi)[missing_ci]
    }

    missing_p <- is.na(out$p.value)
    if (any(missing_p)) {
      out$p.value[missing_p] <- as.numeric(p)[missing_p]
    }

    return(out)
  }

  smry  <- summary(fit)
  coefs <- smry$coefficients
  if (is.null(coefs) || nrow(coefs) == 0) return(NULL)

  rn <- rownames(coefs)
  est <- coefs[, 1]
  se <- coefs[, 2]
  p <- coefs[, 4]
  z <- stats::qnorm(0.975)

  data.frame(
    term = rn,
    estimate = as.numeric(est),
    conf.low = as.numeric(est - z * se),
    conf.high = as.numeric(est + z * se),
    p.value = as.numeric(p),
    stringsAsFactors = FALSE
  )
}

#' @keywords internal
#' @noRd
.tidy_uni <- function(fit, exposure, approach) {
  coefs <- .tidy_coefficients(fit, approach)
  if (is.null(coefs) || nrow(coefs) == 0) return(NULL)

  rn <- coefs$term
  keep <- grepl(paste0("^", exposure), rn) |
    grepl(paste0("^`", exposure, "`"), rn) |
    rn %in% exposure | rn %in% paste0("`", exposure, "`")
  idx <- which(keep)

  df_nonref <- NULL
  if (length(idx)) {
    est <- coefs$estimate[idx]
    lo  <- coefs$conf.low[idx]
    hi  <- coefs$conf.high[idx]
    p   <- coefs$p.value[idx]
    if (.is_ratio(approach)) { est <- exp(est); lo <- exp(lo); hi <- exp(hi) }
    term <- rn[idx]
    lvl  <- sub(paste0("^`?", exposure, "`?"), "", term)
    lvl[lvl == ""] <- exposure  # continuous -> single row at header
    lvl  <- sub("^[:\\.]?", "", lvl)

    df_nonref <- data.frame(
      exposure  = exposure,
      level     = lvl,
      estimate  = est,
      conf.low  = lo,
      conf.high = hi,
      p.value   = p,
      ref       = FALSE,
      stringsAsFactors = FALSE
    )
  }

  ref_row <- NULL
  x <- .model_data_col(fit, exposure)
  if (!is.null(x) && is.factor(x)) {
    levs <- levels(x)
    ref_level <- levs[1]
    ref_row <- data.frame(
      exposure  = exposure,
      level     = ref_level,
      estimate  = if (.is_ratio(approach)) 1 else 0,
      conf.low  = NA_real_,
      conf.high = NA_real_,
      p.value   = NA_real_,
      ref       = TRUE,
      stringsAsFactors = FALSE
    )
    if (!is.null(df_nonref)) {
      df_nonref$..ord <- match(df_nonref$level, levs)
      df_nonref <- df_nonref[order(df_nonref$..ord), , drop = FALSE]
      df_nonref$..ord <- NULL
    }
  }

  if (is.null(ref_row) && is.null(df_nonref)) return(NULL)
  if (!is.null(ref_row) && !is.null(df_nonref)) rbind(ref_row, df_nonref) else
    if (!is.null(ref_row)) ref_row else df_nonref
}

#' @keywords internal
#' @noRd
.tidy_multi <- function(fit, exposures, approach) {
  coefs <- .tidy_coefficients(fit, approach)

  if (is.null(coefs) || nrow(coefs) == 0) {
    return(NULL)
  }

  rn <- coefs$term

  .is_exposure_term <- function(term, exposure) {
    term_clean <- gsub("`", "", term, fixed = TRUE)
    exp_clean  <- gsub("`", "", exposure, fixed = TRUE)

    # 1. Exact match (continuous)
    if (identical(term_clean, exp_clean)) {
      return(TRUE)
    }

    # 2. Factor main effects (e.g. sexMale)
    if (startsWith(term_clean, exp_clean) && !grepl(":", term_clean, fixed = TRUE)) {
      return(TRUE)
    }

    # 3. Interaction term: must explicitly contain exposure as a separate part
    if (grepl(":", term_clean, fixed = TRUE)) {
      parts <- strsplit(term_clean, ":", fixed = TRUE)[[1]]
      parts <- gsub("`", "", parts, fixed = TRUE)

      # KEY: exact match, not partial
      return(any(parts == exp_clean))
    }

    FALSE
  }

  .clean_term_label <- function(term, exposure) {
    term_clean <- gsub("`", "", term, fixed = TRUE)
    exp_clean  <- gsub("`", "", exposure, fixed = TRUE)

    # Main effect
    if (identical(term_clean, exp_clean)) {
      return(exp_clean)
    }

    # Factor level (e.g. sexMale -> Male)
    if (startsWith(term_clean, exp_clean) && !grepl(":", term_clean, fixed = TRUE)) {
      lvl <- sub(paste0("^", exp_clean), "", term_clean)
      if (identical(lvl, "")) return(exp_clean)
      return(lvl)
    }

    # Interaction term (KEY FIX)
    if (grepl(":", term_clean, fixed = TRUE)) {
      parts <- strsplit(term_clean, ":", fixed = TRUE)[[1]]
      parts <- gsub("`", "", parts, fixed = TRUE)

      # Always return full readable interaction
      return(paste(parts, collapse = " x "))
    }

    term_clean
  }

  out_list <- lapply(exposures, function(exposure) {
    idx <- which(vapply(rn, .is_exposure_term, logical(1), exposure = exposure))

    df_nonref <- NULL

    if (length(idx) > 0) {
      est <- coefs$estimate[idx]
      lo  <- coefs$conf.low[idx]
      hi  <- coefs$conf.high[idx]
      p   <- coefs$p.value[idx]

      if (.is_ratio(approach)) {
        est <- exp(est)
        lo  <- exp(lo)
        hi  <- exp(hi)
      }

      term <- rn[idx]
      lvl <- vapply(term, .clean_term_label, character(1), exposure = exposure)

      df_nonref <- data.frame(
        exposure  = exposure,
        level     = lvl,
        estimate  = est,
        conf.low  = lo,
        conf.high = hi,
        p.value   = p,
        ref       = FALSE,
        stringsAsFactors = FALSE
      )

      df_nonref <- df_nonref[!duplicated(df_nonref$level), , drop = FALSE]
    }

    ref_row <- NULL

    x <- .model_data_col(fit, exposure)
    if (!is.null(x) && is.factor(x)) {
      levs <- levels(x)
      ref_level <- levs[1]

      ref_row <- data.frame(
        exposure  = exposure,
        level     = ref_level,
        estimate  = if (.is_ratio(approach)) 1 else 0,
        conf.low  = NA_real_,
        conf.high = NA_real_,
        p.value   = NA_real_,
        ref       = TRUE,
        stringsAsFactors = FALSE
      )

      if (!is.null(df_nonref)) {
        is_interaction <- grepl(" x ", df_nonref$level, fixed = TRUE)

        ord_main <- match(df_nonref$level, levs)
        ord_main[is.na(ord_main)] <- Inf

        df_nonref$..interaction <- is_interaction
        df_nonref$..ord <- ord_main

        df_nonref <- df_nonref[
          order(df_nonref$..interaction, df_nonref$..ord, df_nonref$level),
          ,
          drop = FALSE
        ]

        df_nonref$..interaction <- NULL
        df_nonref$..ord <- NULL
      }
    }

    if (is.null(ref_row) && is.null(df_nonref)) {
      return(NULL)
    }

    if (!is.null(ref_row) && !is.null(df_nonref)) {
      return(rbind(ref_row, df_nonref))
    }

    if (!is.null(ref_row)) {
      return(ref_row)
    }

    df_nonref
  })

  out_list <- Filter(Negate(is.null), out_list)

  if (!length(out_list)) {
    return(NULL)
  }

  do.call(rbind, out_list)
}

#' @keywords internal
#' @noRd
.make_display <- function(td, data, outcome, approach, effect_label, show_ref = TRUE) {
  stopifnot(all(c("exposure","level","estimate","conf.low","conf.high","p.value","ref") %in% names(td)))
  label_map <- .var_label_map(data, unique(td$exposure))
  fmt_est_ci <- function(est, lo, hi, digits = 2) {
    f <- function(x) formatC(x, digits = digits, format = "f", big.mark = ",")
    paste0(f(est), " (", f(lo), "-", f(hi), ")")
  }
  fmt_p <- function(p) ifelse(is.na(p), "", ifelse(p < 0.001, "<0.001", formatC(p, format = "f", digits = 3)))
  complete_n <- function(d, y, x) sum(stats::complete.cases(d[, c(y, x), drop = FALSE]))

  exposure_order <- unique(td$exposure)
  blocks <- lapply(split(td, factor(td$exposure, levels = exposure_order)), function(df) {
    exp_nm  <- unique(df$exposure)[1]
    exp_lab <- .label_var(exp_nm, label_map)
    N_here  <- complete_n(data, outcome, exp_nm)
    is_fact <- any(df$ref)

    if (!is_fact) {
      est <- df$estimate[1]; lo <- df$conf.low[1]; hi <- df$conf.high[1]; p <- df$p.value[1]
      header <- data.frame(Characteristic = exp_lab, stringsAsFactors = FALSE)
      header[[effect_label]] <- fmt_est_ci(est, lo, hi)
      header[["p-value"]]    <- fmt_p(p)
      header$N               <- N_here
      header$is_header       <- TRUE
      header[, c("Characteristic", effect_label, "p-value", "N", "is_header"), drop = FALSE]
    } else {
      header <- data.frame(Characteristic = exp_lab, stringsAsFactors = FALSE)
      header[[effect_label]] <- ""
      header[["p-value"]]    <- ""
      header$N               <- N_here
      header$is_header       <- TRUE

      lev <- df
      if (!isTRUE(show_ref) && nrow(lev) == 2L && sum(lev$ref %in% TRUE) == 1L) {
        row <- lev[!(lev$ref %in% TRUE), , drop = FALSE][1, ]
        header[[effect_label]] <- fmt_est_ci(row$estimate, row$conf.low, row$conf.high)
        header[["p-value"]] <- fmt_p(row$p.value)
        return(header[, c("Characteristic", effect_label, "p-value", "N", "is_header"),
                      drop = FALSE])
      }

      if (!isTRUE(show_ref)) {
        lev <- lev[!(lev$ref %in% TRUE), , drop = FALSE]
      }

      lev$Characteristic <- ifelse(lev$ref, lev$level, paste0("  ", lev$level))
      lev[[effect_label]] <- ifelse(lev$ref, "Ref.",
                                    fmt_est_ci(lev$estimate, lev$conf.low, lev$conf.high))
      lev[["p-value"]] <- ifelse(lev$ref, "", fmt_p(lev$p.value))
      lev$N <- NA_integer_
      lev$is_header <- FALSE
      lev <- lev[, c("Characteristic", effect_label, "p-value", "N", "is_header"), drop = FALSE]

      rbind(header[, names(lev)], lev, make.row.names = FALSE)
    }
  })
  out <- do.call(rbind, blocks)
  row_exposure <- rep(names(blocks), vapply(blocks, nrow, integer(1)))
  .attach_display_metadata(out, row_exposure = row_exposure, variable_labels = label_map)
}

#' @keywords internal
#' @noRd
.must_be_display_df <- function(df) {
  need <- c("Characteristic", "is_header", "N")
  if (!all(need %in% names(df))) {
    stop("Internal: display frame missing required columns (",
         paste(setdiff(need, names(df)), collapse = ", "),
         "). Did you call .make_display() first?", call. = FALSE)
  }
  invisible(TRUE)
}

#' @keywords internal
#' @noRd
.make_display_survival_uni <- function(td,
                                       data,
                                       time,
                                       event,
                                       effect_label,
                                       variable_labels = NULL,
                                       show_ref = TRUE) {
  stopifnot(all(c(
    "exposure", "level", "estimate", "conf.low", "conf.high", "p.value", "ref"
  ) %in% names(td)))

  label_map <- variable_labels
  if (is.null(label_map)) {
    label_map <- .var_label_map(data, unique(td$exposure))
  }

  fmt_est_ci <- function(est, lo, hi, digits = 2) {
    f <- function(x) formatC(x, digits = digits, format = "f", big.mark = ",")
    paste0(f(est), " (", f(lo), "\u2013", f(hi), ")")
  }

  complete_n <- function(d, time, event, x) {
    sum(stats::complete.cases(d[, c(time, event, x), drop = FALSE]))
  }

  exposure_order <- unique(td$exposure)
  blocks <- lapply(split(td, factor(td$exposure, levels = exposure_order)), function(df) {
    exp_nm <- unique(df$exposure)[1]
    exp_lab <- .label_var(exp_nm, label_map)
    N_here <- complete_n(data, time, event, exp_nm)
    is_fact <- any(df$ref)

    if (!is_fact && nrow(df) == 1) {
      header <- data.frame(Characteristic = exp_lab, stringsAsFactors = FALSE)
      header[[effect_label]] <- fmt_est_ci(df$estimate[1], df$conf.low[1], df$conf.high[1])
      header[["p-value"]] <- .fmt_p(df$p.value[1])
      header$N <- N_here
      header$is_header <- TRUE
      return(header[, c("Characteristic", effect_label, "p-value", "N", "is_header"),
                    drop = FALSE])
    }

    header <- data.frame(Characteristic = exp_lab, stringsAsFactors = FALSE)
    header[[effect_label]] <- ""
    header[["p-value"]] <- ""
    header$N <- N_here
    header$is_header <- TRUE

    lev <- df
    if (!isTRUE(show_ref) && nrow(lev) == 2L && sum(lev$ref %in% TRUE) == 1L) {
      row <- lev[!(lev$ref %in% TRUE), , drop = FALSE][1, ]
      header[[effect_label]] <- fmt_est_ci(row$estimate, row$conf.low, row$conf.high)
      header[["p-value"]] <- .fmt_p(row$p.value)
      return(header[, c("Characteristic", effect_label, "p-value", "N", "is_header"),
                    drop = FALSE])
    }

    if (!isTRUE(show_ref)) {
      lev <- lev[!(lev$ref %in% TRUE), , drop = FALSE]
    }

    lev$Characteristic <- ifelse(lev$ref, lev$level, paste0("  ", lev$level))
    lev[[effect_label]] <- ifelse(
      lev$ref,
      "Ref.",
      fmt_est_ci(lev$estimate, lev$conf.low, lev$conf.high)
    )
    lev[["p-value"]] <- ifelse(lev$ref, "", .fmt_p(lev$p.value))
    lev$N <- NA_integer_
    lev$is_header <- FALSE
    lev <- lev[, c("Characteristic", effect_label, "p-value", "N", "is_header"),
               drop = FALSE]

    rbind(header[, names(lev), drop = FALSE], lev, make.row.names = FALSE)
  })

  out <- do.call(rbind, blocks)
  row_exposure <- rep(names(blocks), vapply(blocks, nrow, integer(1)))
  .attach_display_metadata(out, row_exposure = row_exposure, variable_labels = label_map)
}

#' @keywords internal
#' @noRd
.make_display_multi <- function(td,
                                data,
                                outcome,
                                effect_label,
                                variable_labels = NULL,
                                show_ref = TRUE) {
  stopifnot(all(c(
    "exposure", "level", "estimate", "conf.low", "conf.high", "p.value", "ref"
  ) %in% names(td)))

  label_map <- variable_labels
  if (is.null(label_map)) {
    label_map <- .var_label_map(data, unique(td$exposure))
  }

  fmt_est_ci <- function(est, lo, hi, digits = 2) {
    f <- function(x) formatC(x, digits = digits, format = "f", big.mark = ",")
    paste0(f(est), " (", f(lo), "\u2013", f(hi), ")")
  }

  fmt_p <- .fmt_p

  exposure_order <- unique(td$exposure)
  blocks <- lapply(split(td, factor(td$exposure, levels = exposure_order)), function(df) {
    exp_nm <- unique(df$exposure)[1]
    exp_lab <- .label_var(exp_nm, label_map)
    is_fact <- any(df$ref)

    # Case 1: continuous exposure with a single row
    if (!is_fact && nrow(df) == 1) {
      est <- df$estimate[1]
      lo  <- df$conf.low[1]
      hi  <- df$conf.high[1]
      p   <- df$p.value[1]

      header <- data.frame(Characteristic = exp_lab, stringsAsFactors = FALSE)
      header[[effect_label]] <- fmt_est_ci(est, lo, hi)
      header[["p-value"]] <- fmt_p(p)
      header$is_header <- TRUE

      return(header[, c("Characteristic", effect_label, "p-value", "is_header"),
                    drop = FALSE])
    }

    # Case 2: factor exposure OR continuous exposure with interaction rows
    header <- data.frame(Characteristic = exp_lab, stringsAsFactors = FALSE)
    header[[effect_label]] <- if (is_fact) "" else fmt_est_ci(
      df$estimate[df$level == exp_nm][1],
      df$conf.low[df$level == exp_nm][1],
      df$conf.high[df$level == exp_nm][1]
    )
    header[["p-value"]] <- if (is_fact) "" else fmt_p(
      df$p.value[df$level == exp_nm][1]
    )
    header$is_header <- TRUE

    lev <- df

    # For factor exposures: show reference and levels underneath
    if (is_fact) {
      if (!isTRUE(show_ref) && nrow(lev) == 2L && sum(lev$ref %in% TRUE) == 1L) {
        row <- lev[!(lev$ref %in% TRUE), , drop = FALSE][1, ]
        header[[effect_label]] <- fmt_est_ci(row$estimate, row$conf.low, row$conf.high)
        header[["p-value"]] <- fmt_p(row$p.value)
        return(header[, c("Characteristic", effect_label, "p-value", "is_header"),
                      drop = FALSE])
      }

      if (!isTRUE(show_ref)) {
        lev <- lev[!(lev$ref %in% TRUE), , drop = FALSE]
      }

      lev$Characteristic <- ifelse(lev$ref, lev$level, paste0("  ", lev$level))
      lev[[effect_label]] <- ifelse(
        lev$ref,
        "Ref.",
        fmt_est_ci(lev$estimate, lev$conf.low, lev$conf.high)
      )
      lev[["p-value"]] <- ifelse(lev$ref, "", fmt_p(lev$p.value))
    } else {
      # For continuous exposures with interaction rows:
      # keep only non-main rows under the header
      lev <- lev[lev$level != exp_nm, , drop = FALSE]

      if (nrow(lev) > 0) {
        lev$Characteristic <- paste0("  ", lev$level)
        lev[[effect_label]] <- fmt_est_ci(lev$estimate, lev$conf.low, lev$conf.high)
        lev[["p-value"]] <- fmt_p(lev$p.value)
      }
    }

    lev$is_header <- FALSE

    lev <- lev[, c("Characteristic", effect_label, "p-value", "is_header"),
               drop = FALSE]

    if (nrow(lev) == 0) {
      return(header[, names(lev), drop = FALSE])
    }

    rbind(header[, names(lev), drop = FALSE], lev, make.row.names = FALSE)
  })

  out <- do.call(rbind, blocks)
  row_exposure <- rep(names(blocks), vapply(blocks, nrow, integer(1)))
  .attach_display_metadata(out, row_exposure = row_exposure, variable_labels = label_map)
}

#' @keywords internal
#' @noRd
.must_be_display_df_multi <- function(df) {
  need <- c("Characteristic", "is_header")
  if (!all(need %in% names(df))) {
    stop("Internal: display frame missing required columns (",
         paste(setdiff(need, names(df)), collapse = ", "),
         ").", call. = FALSE)
  }
  invisible(TRUE)
}

#' Restricted mean survival time table
#'
#' Estimate restricted mean survival time (RMST) up to a user-specified follow-up
#' time. RMST is the average survival time observed within a fixed time window,
#' such as 365 days.
#'
#' @param data A \code{data.frame} containing survival time and event status.
#' @param time Survival follow-up time. Quoted and bare names are accepted.
#' @param event Event indicator. Quoted and bare names are accepted. Numeric
#'   \code{0/1}, numeric \code{1/2}, logical, character, and factor variables
#'   are accepted. For two-level character or factor variables, the second level
#'   is treated as the event.
#' @param by Optional grouping variable for separate RMST estimates. Quoted and
#'   bare names are accepted.
#' @param tau Restriction time for RMST. For example, \code{tau = 365} reports
#'   mean survival time restricted to the first 365 days of follow-up.
#' @param digits Number of digits for displayed survival time summaries.
#' @param conf.level Confidence level for RMST confidence intervals.
#' @param format Output format. One of \code{"flextable"} (default),
#'   \code{"gt"}, or \code{"tibble"}.
#' @param theme Table styling preset.
#'
#' @return A list of class \code{c("gtregression","rmst_table", ...)} with
#' elements:
#' \describe{
#'   \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
#'   when \code{format = "tibble"}.}
#'   \item{\code{table_body}}{Tibble with numeric RMST summaries.}
#'   \item{\code{table_display}}{Display data frame used to render the table.}
#'   \item{\code{fit}}{Fitted \code{survfit} object.}
#'   \item{\code{time,event,by,tau,format,source}}{Metadata fields.}
#' }
#'
#' @details
#' RMST is estimated from \code{survival::survfit()} using Kaplan-Meier methods
#' and \code{summary.survfit(rmean = tau)}. When \code{by} has exactly two
#' groups, the table includes the RMST difference as the second group minus the
#' first group, with a Wald confidence interval and p-value based on the
#' reported RMST standard errors.
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#'
#' rmst_table(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   by = trt,
#'   tau = 365
#' )
#'
#' rmst_table(
#'   data = lung_data,
#'   time = "time",
#'   event = "status",
#'   tau = 180,
#'   format = tibble
#' )
#'
#' @importFrom survival survfit
#' @importFrom stats complete.cases pnorm qnorm
#' @export
rmst_table <- function(data,
                       time,
                       event,
                       by = NULL,
                       tau,
                       digits = 1,
                       conf.level = 0.95,
                       format = c("flextable", "gt", "tibble"),
                       theme = c("minimal")) {

  time <- .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  event <- .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  by <- .vars_arg(substitute(by), env = parent.frame(), allow_null = TRUE)
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  theme <- .choice_arg(substitute(theme), env = parent.frame())

  format <- match.arg(format, c("flextable", "gt", "tibble"))
  theme <- .resolve_theme(theme)

  data_clean <- .validate_rmst_inputs(
    data = data,
    time = time,
    event = event,
    by = by,
    tau = tau,
    digits = digits,
    conf.level = conf.level
  )

  if (!is.null(by)) {
    data_clean[[by]] <- factor(data_clean[[by]])
  }

  fit <- survival::survfit(.km_formula(time, event, by), data = data_clean)
  sm <- summary(fit, rmean = tau)
  table_body <- .rmst_table_body(sm, by = by, tau = tau, conf.level = conf.level)

  if (format == "tibble") {
    return(tibble::as_tibble(table_body))
  }

  table_display <- .rmst_table_display(table_body, digits = digits)
  tbl <- .build_rmst_table(
    table_display,
    format = format,
    theme = theme
  )

  res <- list(
    table = tbl,
    table_body = tibble::as_tibble(table_body),
    table_display = table_display,
    fit = fit,
    time = time,
    event = event,
    by = by,
    tau = tau,
    conf.level = conf.level,
    format = format,
    source = "rmst_table"
  )

  class(res) <- c("gtregression", "rmst_table", class(res))
  res
}

#' @keywords internal
#' @noRd
.validate_rmst_inputs <- function(data, time, event, by, tau, digits, conf.level) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }
  if (!is.character(time) || length(time) != 1L || !time %in% names(data)) {
    stop("`time` must be a single survival time variable in `data`.", call. = FALSE)
  }
  if (!is.character(event) || length(event) != 1L || !event %in% names(data)) {
    stop("`event` must be a single event indicator variable in `data`.", call. = FALSE)
  }
  if (!is.null(by) && (!is.character(by) || length(by) != 1L || !by %in% names(data))) {
    stop("`by` must be NULL or a single grouping variable in `data`.", call. = FALSE)
  }
  if (!is.numeric(data[[time]])) {
    stop("`time` must be numeric.", call. = FALSE)
  }
  if (any(data[[time]] < 0, na.rm = TRUE)) {
    stop("`time` must contain non-negative follow-up times.", call. = FALSE)
  }
  if (missing(tau) || !is.numeric(tau) || length(tau) != 1L ||
      is.na(tau) || tau <= 0) {
    stop("`tau` must be a single positive follow-up time.", call. = FALSE)
  }
  if (!is.numeric(digits) || length(digits) != 1L || is.na(digits) ||
      digits < 0 || digits != floor(digits)) {
    stop("`digits` must be a non-negative whole number.", call. = FALSE)
  }
  if (!is.numeric(conf.level) || length(conf.level) != 1L || is.na(conf.level) ||
      conf.level <= 0 || conf.level >= 1) {
    stop("`conf.level` must be a number between 0 and 1.", call. = FALSE)
  }

  vars_needed <- unique(c(time, event, by))
  data_clean <- data[stats::complete.cases(data[, vars_needed, drop = FALSE]), , drop = FALSE]
  if (nrow(data_clean) == 0) {
    stop("No complete cases available for RMST estimation.", call. = FALSE)
  }

  data_clean[[event]] <- .cox_event01(data_clean[[event]])
  if (sum(data_clean[[event]] == 1) == 0) {
    stop("`event` must include at least one event.", call. = FALSE)
  }
  if (!is.null(by) && length(unique(data_clean[[by]])) < 2L) {
    stop("`by` must contain at least two non-missing groups.", call. = FALSE)
  }

  data_clean
}

#' @keywords internal
#' @noRd
.rmst_table_body <- function(sm, by = NULL, tau, conf.level = 0.95) {
  tab <- as.data.frame(sm$table)
  if (is.null(dim(sm$table))) {
    tab <- as.data.frame(t(sm$table))
    rownames(tab) <- "Overall"
  }

  group <- if (is.null(by)) {
    "Overall"
  } else {
    sub("^.*=", "", rownames(tab))
  }

  z <- stats::qnorm(1 - (1 - conf.level) / 2)
  rmst <- as.numeric(tab$rmean)
  se <- as.numeric(tab[["se(rmean)"]])

  out <- data.frame(
    Type = "Group",
    Group = group,
    Tau = tau,
    N = as.numeric(tab$records),
    Events = as.numeric(tab$events),
    RMST = rmst,
    SE = se,
    CI.lower = rmst - z * se,
    CI.upper = rmst + z * se,
    Difference = NA_real_,
    Difference.SE = NA_real_,
    Difference.CI.lower = NA_real_,
    Difference.CI.upper = NA_real_,
    p.value = NA_real_,
    stringsAsFactors = FALSE
  )

  if (!is.null(by) && nrow(out) == 2L) {
    diff <- out$RMST[2] - out$RMST[1]
    diff_se <- sqrt(sum(out$SE^2))
    diff_p <- if (diff_se > 0) {
      2 * stats::pnorm(abs(diff / diff_se), lower.tail = FALSE)
    } else {
      NA_real_
    }

    out <- rbind(
      out,
      data.frame(
        Type = "Difference",
        Group = paste0("Difference (", out$Group[2], " - ", out$Group[1], ")"),
        Tau = tau,
        N = NA_real_,
        Events = NA_real_,
        RMST = NA_real_,
        SE = diff_se,
        CI.lower = NA_real_,
        CI.upper = NA_real_,
        Difference = diff,
        Difference.SE = diff_se,
        Difference.CI.lower = diff - z * diff_se,
        Difference.CI.upper = diff + z * diff_se,
        p.value = diff_p,
        stringsAsFactors = FALSE
      )
    )
  }

  out
}

#' @keywords internal
#' @noRd
.rmst_table_display <- function(table_body, digits = 1) {
  fmt_num <- function(x) {
    ifelse(
      is.na(x),
      NA_character_,
      formatC(x, digits = digits, format = "f", big.mark = ",")
    )
  }
  fmt_p <- function(x) {
    ifelse(
      is.na(x),
      "",
      ifelse(x < 0.001, "<0.001", formatC(x, digits = 3, format = "f"))
    )
  }

  rmst_txt <- ifelse(
    is.na(table_body$RMST),
    "",
    paste0(
      fmt_num(table_body$RMST),
      " (",
      fmt_num(table_body$CI.lower),
      "-",
      fmt_num(table_body$CI.upper),
      ")"
    )
  )

  diff_txt <- ifelse(
    is.na(table_body$Difference),
    "",
    paste0(
      fmt_num(table_body$Difference),
      " (",
      fmt_num(table_body$Difference.CI.lower),
      "-",
      fmt_num(table_body$Difference.CI.upper),
      ")"
    )
  )

  data.frame(
    Group = table_body$Group,
    Tau = fmt_num(table_body$Tau),
    N = ifelse(is.na(table_body$N), "", as.character(table_body$N)),
    Events = ifelse(is.na(table_body$Events), "", as.character(table_body$Events)),
    `RMST (95% CI)` = rmst_txt,
    `RMST difference (95% CI)` = diff_txt,
    `p-value` = fmt_p(table_body$p.value),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )
}

#' @keywords internal
#' @noRd
.build_rmst_table <- function(display,
                              format = c("flextable", "gt"),
                              theme = c("minimal")) {
  format <- match.arg(format, c("flextable", "gt"))
  note <- paste(
    "RMST is restricted mean survival time up to tau.",
    "For two groups, the difference is the second group minus the first group."
  )

  if (format == "gt") {
    tbl <- gt::gt(display) |>
      gt::tab_header(title = "Restricted mean survival time") |>
      gt::cols_align(align = "left", columns = "Group") |>
      gt::cols_align(
        align = "center",
        columns = c("Tau", "N", "Events", "RMST (95% CI)", "RMST difference (95% CI)", "p-value")
      ) |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_column_labels()
      ) |>
      gt::tab_source_note(gt::md(note)) |>
      .compact_gt_source_notes()

    if ("header_shaded" %in% theme) {
      tbl <- gt::tab_options(tbl, column_labels.background.color = "#f6f8fa")
    }
    if ("zebra" %in% theme) {
      tbl <- gt::opt_row_striping(tbl)
    }
    if ("compact" %in% theme) {
      tbl <- gt::tab_options(tbl, data_row.padding = gt::px(2))
    }
    return(tbl)
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Restricted mean survival time")
  ft <- flextable::align(ft, j = "Group", align = "left", part = "all")
  ft <- flextable::align(
    ft,
    j = c("Tau", "N", "Events", "RMST (95% CI)", "RMST difference (95% CI)", "p-value"),
    align = "center",
    part = "all"
  )
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  if ("header_shaded" %in% theme) {
    ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  }
  if ("zebra" %in% theme && nrow(display) > 1L) {
    ft <- flextable::bg(ft, i = seq(1, nrow(display), by = 2), bg = "#f6f8fa", part = "body")
  }
  if ("compact" %in% theme) {
    ft <- flextable::padding(ft, padding = 2, part = "body")
  }
  ft <- flextable::add_footer_lines(ft, values = note)
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

lung_rmst_table_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment"))
    )
}

test_that("rmst_table returns grouped RMST table with two-group difference", {
  skip_if_not_installed("survival")

  df <- lung_rmst_table_data()

  res <- rmst_table(
    data = df,
    time = time,
    event = status,
    by = trt,
    tau = 365
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "rmst_table")
  expect_s3_class(res$table, "flextable")
  expect_s3_class(res$fit, "survfit")
  expect_equal(res$source, "rmst_table")
  expect_equal(res$time, "time")
  expect_equal(res$event, "status")
  expect_equal(res$by, "trt")
  expect_equal(res$tau, 365)
  expect_equal(res$conf.level, 0.95)

  expect_equal(res$table_body$Type, c("Group", "Group", "Difference"))
  expect_equal(
    res$table_body$Group,
    c(
      "Standard treatment",
      "Test treatment",
      "Difference (Test treatment - Standard treatment)"
    )
  )
  expect_equal(res$table_body$N, c(69, 68, NA))
  expect_equal(res$table_body$Events, c(64, 64, NA))
  expect_equal(round(res$table_body$RMST, 3), c(118.972, 112.404, NA))
  expect_equal(round(res$table_body$SE, 3), c(13.020, 14.875, 19.768))
  expect_equal(round(res$table_body$CI.lower, 3), c(93.452, 83.250, NA))
  expect_equal(round(res$table_body$CI.upper, 3), c(144.491, 141.558, NA))
  expect_equal(round(res$table_body$Difference, 3), c(NA, NA, -6.567))
  expect_equal(round(res$table_body$Difference.CI.lower, 3), c(NA, NA, -45.313))
  expect_equal(round(res$table_body$Difference.CI.upper, 3), c(NA, NA, 32.178))
  expect_equal(round(res$table_body$p.value, 3), c(NA, NA, 0.740))
  expect_true("RMST (95% CI)" %in% names(res$table_display))
  expect_true("RMST difference (95% CI)" %in% names(res$table_display))
})

test_that("rmst_table returns overall tibble with quoted names", {
  skip_if_not_installed("survival")

  df <- lung_rmst_table_data()
  time_var <- "time"
  event_var <- "status"

  out <- rmst_table(
    data = df,
    time = time_var,
    event = event_var,
    tau = 180,
    format = tibble
  )

  expect_s3_class(out, "tbl_df")
  expect_equal(out$Type, "Group")
  expect_equal(out$Group, "Overall")
  expect_equal(out$Tau, 180)
  expect_equal(out$N, 137)
  expect_equal(out$Events, 128)
  expect_equal(round(out$RMST, 3), 88.521)
  expect_equal(round(out$SE, 3), 5.639)
})

test_that("rmst_table returns gt table", {
  skip_if_not_installed("survival")
  skip_if_not_installed("gt")

  df <- lung_rmst_table_data()

  res <- rmst_table(
    data = df,
    time = time,
    event = status,
    by = trt,
    tau = 365,
    format = gt
  )

  expect_s3_class(res$table, "gt_tbl")
})

test_that("rmst_table validates inputs", {
  skip_if_not_installed("survival")

  df <- lung_rmst_table_data()
  df$bad_event <- 0
  df$one_group <- "A"

  expect_error(
    rmst_table(data = "not data", time = time, event = status, tau = 365),
    "`data` must be"
  )
  expect_error(
    rmst_table(data = df, time = missing_time, event = status, tau = 365),
    "`time` must be"
  )
  expect_error(
    rmst_table(data = df, time = time, event = missing_event, tau = 365),
    "`event` must be"
  )
  expect_error(
    rmst_table(data = df, time = time, event = status),
    "`tau` must be"
  )
  expect_error(
    rmst_table(data = df, time = time, event = status, tau = 0),
    "`tau` must be"
  )
  expect_error(
    rmst_table(data = df, time = time, event = bad_event, tau = 365),
    "`event` must include at least one event"
  )
  expect_error(
    rmst_table(data = df, time = time, event = status, by = one_group, tau = 365),
    "`by` must contain at least two"
  )
  expect_error(
    rmst_table(data = df, time = time, event = status, tau = 365, digits = -1),
    "`digits` must be"
  )
  expect_error(
    rmst_table(data = df, time = time, event = status, tau = 365, conf.level = 1),
    "`conf.level` must be"
  )
})

#' Predict survival probabilities from a parametric survival model
#'
#' Estimate model-based survival probabilities at user-specified follow-up
#' times from a fitted \code{survival::survreg()} model.
#'
#' @param model A fitted \code{survreg} model, or a \code{surv_reg()} object
#'   containing exactly one fitted model.
#' @param newdata Optional \code{data.frame} of profiles for prediction. If
#'   \code{NULL}, one typical profile is built from the model data using medians
#'   for numeric variables and the most common level for categorical variables.
#' @param times Numeric vector of follow-up times at which survival probability
#'   should be predicted.
#' @param digits Number of digits for displayed follow-up times and
#'   probabilities.
#' @param format Output format. One of \code{"flextable"} (default),
#'   \code{"gt"}, or \code{"tibble"}.
#' @param theme Table styling preset.
#'
#' @return A list of class \code{c("gtregression","surv_predict", ...)}
#' with elements:
#' \describe{
#'   \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
#'   when \code{format = "tibble"}.}
#'   \item{\code{table_body}}{Tibble with predicted survival probabilities.}
#'   \item{\code{table_display}}{Display data frame used to render the table.}
#'   \item{\code{model}}{The fitted \code{survreg} model used for prediction.}
#'   \item{\code{newdata,times,distribution,format,source}}{Metadata fields.}
#' }
#'
#' @details
#' \code{surv_predict()} is for parametric survival regression models fitted by
#' \code{surv_reg()} or \code{survival::survreg()}. It is not a Kaplan-Meier
#' estimate and it is not a Cox prediction helper.
#'
#' Supported distributions are \code{"weibull"}, \code{"exponential"},
#' \code{"lognormal"}, and \code{"loglogistic"}, matching
#' \code{surv_reg()}. Predictions are conditional on the supplied profile and
#' the chosen parametric distribution.
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#'
#' fit <- surv_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = "trt",
#'   adjust_for = c("age", "karno"),
#'   distribution = weibull
#' )
#'
#' surv_predict(
#'   model = fit$models$trt,
#'   newdata = data.frame(
#'     trt = factor("Test", levels = levels(lung_data$trt)),
#'     age = 60,
#'     karno = 70
#'   ),
#'   times = c(90, 180, 365)
#' )
#'
#' surv_predict(
#'   model = fit,
#'   times = c(90, 180),
#'   format = tibble
#' )
#'
#' @importFrom stats median predict terms delete.response model.frame
#' @export
surv_predict <- function(model,
                         newdata = NULL,
                         times,
                         digits = 1,
                         format = c("flextable", "gt", "tibble"),
                         theme = c("minimal")) {

  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  theme <- .choice_arg(substitute(theme), env = parent.frame())

  format <- match.arg(format, c("flextable", "gt", "tibble"))
  theme <- .resolve_theme(theme)

  fit <- .surv_predict_model(model)
  .validate_surv_predict_inputs(fit, newdata = newdata, times = times, digits = digits)

  if (is.null(newdata)) {
    newdata <- .surv_predict_typical_newdata(fit)
  }

  table_body <- .surv_predict_table_body(fit, newdata = newdata, times = times)

  if (format == "tibble") {
    return(tibble::as_tibble(table_body))
  }

  table_display <- .surv_predict_display(table_body, digits = digits)
  tbl <- .build_surv_predict_table(
    table_display,
    format = format,
    theme = theme,
    distribution = fit$dist
  )

  res <- list(
    table = tbl,
    table_body = tibble::as_tibble(table_body),
    table_display = table_display,
    model = fit,
    newdata = newdata,
    times = times,
    distribution = fit$dist,
    format = format,
    source = "surv_predict"
  )

  class(res) <- c("gtregression", "surv_predict", class(res))
  res
}

#' @keywords internal
#' @noRd
.surv_predict_model <- function(model) {
  if (inherits(model, "survreg")) {
    return(model)
  }

  if (inherits(model, "surv_reg")) {
    if (is.null(model$models) || length(model$models) != 1L) {
      stop(
        "`model` is a surv_reg() object with multiple fitted models. ",
        "Pass one model, for example `model$models$trt`.",
        call. = FALSE
      )
    }
    return(model$models[[1L]])
  }

  stop("`model` must be a survreg model or a single-model surv_reg() object.", call. = FALSE)
}

#' @keywords internal
#' @noRd
.validate_surv_predict_inputs <- function(model, newdata, times, digits) {
  supported <- c("weibull", "exponential", "lognormal", "loglogistic")
  if (is.null(model$dist) || !model$dist %in% supported) {
    stop(
      "`model` must use one of these survreg distributions: ",
      paste(supported, collapse = ", "),
      call. = FALSE
    )
  }
  if (!is.null(newdata) && !is.data.frame(newdata)) {
    stop("`newdata` must be NULL or a data.frame.", call. = FALSE)
  }
  if (missing(times) || !is.numeric(times) || length(times) < 1L ||
      anyNA(times) || any(times < 0)) {
    stop("`times` must be a numeric vector of non-negative follow-up times.", call. = FALSE)
  }
  if (!is.numeric(digits) || length(digits) != 1L || is.na(digits) ||
      digits < 0 || digits != floor(digits)) {
    stop("`digits` must be a non-negative whole number.", call. = FALSE)
  }
  if (is.null(newdata) && is.null(model$model)) {
    stop("`model` must be fitted with `model = TRUE` to support default profiles.", call. = FALSE)
  }

  if (!is.null(newdata)) {
    tryCatch(
      stats::predict(model, newdata = newdata, type = "lp"),
      error = function(e) {
        stop("`newdata` is not compatible with `model`: ", e$message, call. = FALSE)
      }
    )
  }

  invisible(TRUE)
}

#' @keywords internal
#' @noRd
.surv_predict_typical_newdata <- function(model) {
  mf <- model$model
  term_labels <- attr(stats::terms(model), "term.labels")
  vars <- unique(gsub("`", "", term_labels, fixed = TRUE))
  vars <- vars[vars %in% names(mf)]

  if (!length(vars)) {
    return(data.frame(.profile = "Overall", stringsAsFactors = FALSE))
  }

  out <- data.frame(.profile = "Typical profile", stringsAsFactors = FALSE)
  for (var in vars) {
    out[[var]] <- .surv_predict_typical_value(mf[[var]])
  }
  out
}

#' @keywords internal
#' @noRd
.surv_predict_typical_value <- function(x) {
  if (is.numeric(x)) {
    return(stats::median(x, na.rm = TRUE))
  }
  if (is.factor(x)) {
    tab <- table(x)
    val <- names(tab)[which.max(tab)]
    return(factor(val, levels = levels(x), ordered = is.ordered(x)))
  }
  if (is.logical(x)) {
    tab <- table(x)
    return(as.logical(names(tab)[which.max(tab)]))
  }
  tab <- table(x)
  names(tab)[which.max(tab)]
}

#' @keywords internal
#' @noRd
.surv_predict_table_body <- function(model, newdata, times) {
  lp <- as.numeric(stats::predict(model, newdata = newdata, type = "lp"))
  profiles <- seq_len(nrow(newdata))
  grid <- expand.grid(
    Profile = profiles,
    Time = times,
    KEEP.OUT.ATTRS = FALSE
  )

  eta <- lp[grid$Profile]
  surv <- .surv_predict_probability(
    time = grid$Time,
    eta = eta,
    scale = model$scale,
    distribution = model$dist
  )

  out <- data.frame(
    Profile = grid$Profile,
    Time = as.numeric(grid$Time),
    Survival.probability = as.numeric(surv),
    Distribution = model$dist,
    stringsAsFactors = FALSE
  )

  profile_cols <- newdata[grid$Profile, , drop = FALSE]
  rownames(profile_cols) <- NULL
  cbind(
    out[, "Profile", drop = FALSE],
    profile_cols,
    out[, c("Time", "Survival.probability", "Distribution"), drop = FALSE]
  )
}

#' @keywords internal
#' @noRd
.surv_predict_probability <- function(time, eta, scale, distribution) {
  out <- rep(NA_real_, length(time))
  zero <- time == 0
  out[zero] <- 1

  idx <- !zero
  z <- (log(time[idx]) - eta[idx]) / scale

  out[idx] <- switch(
    distribution,
    weibull = exp(-exp(z)),
    exponential = exp(-exp(z)),
    lognormal = stats::pnorm(z, lower.tail = FALSE),
    loglogistic = 1 / (1 + exp(z))
  )
  pmin(pmax(out, 0), 1)
}

#' @keywords internal
#' @noRd
.surv_predict_display <- function(table_body, digits = 1) {
  fmt_time <- function(x) {
    formatC(x, digits = digits, format = "f", big.mark = ",")
  }
  fmt_pct <- function(x) {
    ifelse(
      is.na(x),
      "NA",
      paste0(formatC(100 * x, digits = digits, format = "f"), "%")
    )
  }

  out <- table_body
  out$Time <- fmt_time(out$Time)
  out$Survival.probability <- fmt_pct(out$Survival.probability)
  names(out)[names(out) == "Survival.probability"] <- "Predicted survival"
  names(out)[names(out) == "Distribution"] <- "Model distribution"
  out
}

#' @keywords internal
#' @noRd
.build_surv_predict_table <- function(display,
                                      format = c("flextable", "gt"),
                                      theme = c("minimal"),
                                      distribution = NULL) {
  format <- match.arg(format, c("flextable", "gt"))
  note <- paste(
    "Model-based predictions from a parametric survival regression model.",
    if (!is.null(distribution)) paste0("Distribution: ", distribution, ".") else NULL,
    "Predictions depend on the supplied profile and model specification."
  )

  center_cols <- setdiff(names(display), names(display)[names(display) %in% c("Profile", ".profile")])

  if (format == "gt") {
    tbl <- gt::gt(display) |>
      gt::tab_header(title = "Predicted survival probabilities") |>
      gt::cols_align(align = "center", columns = center_cols) |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_column_labels()
      ) |>
      gt::tab_source_note(gt::md(note)) |>
      .compact_gt_source_notes()

    if ("header_shaded" %in% theme) {
      tbl <- gt::tab_options(tbl, column_labels.background.color = "#f6f8fa")
    }
    if ("zebra" %in% theme) {
      tbl <- gt::opt_row_striping(tbl)
    }
    if ("compact" %in% theme) {
      tbl <- gt::tab_options(tbl, data_row.padding = gt::px(2))
    }
    return(tbl)
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Predicted survival probabilities")
  ft <- flextable::align(ft, j = center_cols, align = "center", part = "all")
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  if ("header_shaded" %in% theme) {
    ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  }
  if ("zebra" %in% theme && nrow(display) > 1L) {
    ft <- flextable::bg(ft, i = seq(1, nrow(display), by = 2), bg = "#f6f8fa", part = "body")
  }
  if ("compact" %in% theme) {
    ft <- flextable::padding(ft, padding = 2, part = "body")
  }
  ft <- flextable::add_footer_lines(ft, values = note)
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

lung_surv_predict_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )
}

test_that("surv_predict returns profile-specific predicted probabilities", {
  skip_if_not_installed("survival")

  df <- lung_surv_predict_data()
  fit <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = "trt",
    adjust_for = c("age", "karno"),
    distribution = weibull
  )

  res <- surv_predict(
    model = fit$models$trt,
    newdata = data.frame(
      trt = factor("Test treatment", levels = levels(df$trt)),
      age = 60,
      karno = 70
    ),
    times = c(90, 180, 365)
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "surv_predict")
  expect_s3_class(res$table, "flextable")
  expect_s3_class(res$model, "survreg")
  expect_equal(res$source, "surv_predict")
  expect_equal(res$times, c(90, 180, 365))
  expect_equal(res$distribution, "weibull")
  expect_equal(res$table_body$Time, c(90, 180, 365))
  expect_equal(round(res$table_body$Survival.probability, 3), c(0.553, 0.311, 0.097))
  expect_true("Predicted survival" %in% names(res$table_display))
})

test_that("surv_predict supports single-model surv_reg objects and tibble output", {
  skip_if_not_installed("survival")

  df <- lung_surv_predict_data()
  fit <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = "trt",
    adjust_for = c("age", "karno"),
    distribution = lognormal
  )

  out <- surv_predict(
    model = fit,
    times = c(90, 180),
    format = tibble
  )

  expect_s3_class(out, "tbl_df")
  expect_equal(out$Profile, c(1, 1))
  expect_equal(out$Time, c(90, 180))
  expect_equal(out$Distribution, c("lognormal", "lognormal"))
  expect_equal(round(out$Survival.probability, 3), c(0.439, 0.218))
  expect_true(".profile" %in% names(out))
})

test_that("surv_predict supports multiple profiles and gt output", {
  skip_if_not_installed("survival")
  skip_if_not_installed("gt")

  df <- lung_surv_predict_data()
  fit <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = "trt",
    adjust_for = c("age", "karno"),
    distribution = loglogistic
  )

  res <- surv_predict(
    model = fit$models$trt,
    newdata = data.frame(
      trt = factor(c("Standard treatment", "Test treatment"), levels = levels(df$trt)),
      age = c(60, 60),
      karno = c(70, 70)
    ),
    times = c(90, 180),
    format = gt
  )

  expect_s3_class(res$table, "gt_tbl")
  expect_equal(res$table_body$Profile, c(1, 2, 1, 2))
  expect_equal(res$table_body$Time, c(90, 90, 180, 180))
  expect_equal(round(res$table_body$Survival.probability, 3), c(0.567, 0.545, 0.299, 0.281))
})

test_that("surv_predict supports direct survreg models with supplied newdata", {
  skip_if_not_installed("survival")

  df <- lung_surv_predict_data()
  fit <- survival::survreg(
    survival::Surv(time, status) ~ trt + age,
    data = df,
    dist = "weibull"
  )

  out <- surv_predict(
    model = fit,
    newdata = data.frame(
      trt = factor("Test treatment", levels = levels(df$trt)),
      age = 60
    ),
    times = c(90, 180),
    format = tibble
  )

  expect_s3_class(out, "tbl_df")
  expect_equal(out$Time, c(90, 180))
  expect_equal(out$Distribution, c("weibull", "weibull"))
  expect_true(all(out$Survival.probability >= 0 & out$Survival.probability <= 1))
})

test_that("surv_predict validates inputs", {
  skip_if_not_installed("survival")

  df <- lung_surv_predict_data()
  one_fit <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = "trt",
    adjust_for = c("age", "karno")
  )
  multi_fit <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c("trt", "prior")
  )

  expect_error(
    surv_predict(stats::lm(mpg ~ hp, data = mtcars), times = 90),
    "`model` must be"
  )
  expect_error(
    surv_predict(multi_fit, times = 90),
    "multiple fitted models"
  )
  expect_error(
    surv_predict(one_fit, newdata = "bad", times = 90),
    "`newdata` must be"
  )
  expect_error(
    surv_predict(one_fit, times = -1),
    "`times` must be"
  )
  expect_error(
    surv_predict(one_fit, times = 90, digits = -1),
    "`digits` must be"
  )
  expect_error(
    surv_predict(
      one_fit,
      newdata = data.frame(trt = "Test treatment", age = 60),
      times = 90
    ),
    "`newdata` is not compatible"
  )
})

#' Compare Models With and Without an Interaction Term
#'
#' Fits two models, one with and one without an interaction term between an
#' exposure and a potential effect modifier. The models are compared using a
#' likelihood ratio test or Wald test to assess statistical evidence of
#' interaction.
#'
#' Use this function when the interaction is planned or clinically/causally
#' motivated and you want a focused model comparison. Mantel-Haenszel estimation
#' is not used here because this function tests an explicit interaction term in
#' a regression model. For broader screening of candidate confounders or effect
#' modifiers, including Mantel-Haenszel-supported checks when appropriate, use
#' \code{\link{identify_confounder}()}.
#'
#' With \code{covariates = c(age, sex)}, the two fitted models are
#' \code{outcome ~ exposure + effect_modifier + age + sex} and
#' \code{outcome ~ exposure + effect_modifier + age + sex +
#' exposure:effect_modifier}. Both models use the same complete-case analysis
#' data, so covariate adjustment is applied consistently to the interaction
#' comparison.
#'
#' @param data A data frame containing all required variables.
#' @param outcome Outcome variable name. Quoted and bare names are accepted.
#'   Required for ordinary regression approaches. Leave unset for Cox and
#'   parametric survival approaches and supply \code{time} and \code{event}
#'   instead.
#' @param exposure Main exposure variable name. Quoted and bare names are
#'   accepted.
#' @param covariates Optional character vector of adjustment covariates. They
#'   are included in both the model without and the model with the interaction
#'   term. Quoted names are recommended in scripts, and bare names are also
#'   accepted.
#' @param effect_modifier Variable name for the potential effect modifier.
#'   Quoted and bare names are accepted.
#' @param approach Regression approach. One of \code{"logit"},
#'   \code{"logbinomial"}, \code{"poisson"}, \code{"robpoisson"},
#'   \code{"negbin"}, \code{"linear"}, \code{"cox"}, or
#'   \code{"survreg"}.
#' @param time Survival time variable name for \code{approach = "cox"} or
#'   \code{approach = "survreg"}. Quoted and bare names are accepted.
#' @param event Event indicator variable name for survival approaches.
#' @param distribution Parametric survival distribution for
#'   \code{approach = "survreg"}. One of \code{"weibull"},
#'   \code{"exponential"}, \code{"lognormal"}, or \code{"loglogistic"}.
#' @param test Statistical test for model comparison. One of \code{"LRT"} or
#'   \code{"Wald"}.
#' @param alpha Significance threshold used to classify the interaction result.
#' @param verbose Logical; if \code{TRUE}, prints a short interpretation.
#' @param format Output format for the viewing table. One of
#'   \code{"flextable"} (default), \code{"gt"}, or \code{"tibble"}. Use
#'   \code{format = "tibble"} to keep only the original list structure.
#'
#' @return A list with model objects, formulas, p-value, decision, and a
#'   one-row summary tibble. When \code{format} is \code{"gt"} or
#'   \code{"flextable"}, the list also includes \code{table}.
#'
#' @examples
#' birthwt_data <- data_birthwt |>
#'   dplyr::mutate(
#'     low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
#'     smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
#'     race = factor(race, levels = c(1, 2, 3),
#'                   labels = c("White", "Black", "Other"))
#'   )
#'
#' interaction_models(
#'   data = birthwt_data,
#'   outcome = low,
#'   exposure = smoke,
#'   effect_modifier = race,
#'   covariates = c(age, lwt),
#'   approach = logit
#' )
#'
#' lung_data <- data_lungcancer |>
#'   dplyr::mutate(
#'     trt = factor(trt, levels = c(1, 2),
#'                  labels = c("Standard treatment", "Test treatment")),
#'     prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
#'   )
#'
#' interaction_models(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposure = trt,
#'   effect_modifier = prior,
#'   covariates = c(age, karno),
#'   approach = cox
#' )
#'
#' @importFrom stats glm anova binomial coef lm poisson reformulate terms
#' @importFrom survival coxph survreg Surv
#' @importFrom MASS glm.nb
#' @seealso \code{\link{identify_confounder}()} for broader confounding and
#'   effect-modification screening.
#' @export
interaction_models <- function(data,
                               outcome = NULL,
                               exposure,
                               covariates = NULL,
                               effect_modifier,
                               approach = "logit",
                               time = NULL,
                               event = NULL,
                               distribution = "weibull",
                               test = c("LRT", "Wald"),
                               alpha = 0.05,
                               verbose = FALSE,
                               format = c("flextable", "gt", "tibble")) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }
  outcome <- if (missing(outcome) || identical(substitute(outcome), quote(NULL))) {
    NULL
  } else {
    .vars_arg(substitute(outcome), env = parent.frame())
  }
  exposure <- .vars_arg(substitute(exposure), env = parent.frame())
  covariates <- .vars_arg(substitute(covariates), env = parent.frame(), allow_null = TRUE)
  effect_modifier <- .vars_arg(substitute(effect_modifier), env = parent.frame())
  if (!is.character(exposure) || length(exposure) != 1L ||
      is.na(exposure) || !nzchar(exposure)) {
    stop("`exposure` must be a single character variable name.", call. = FALSE)
  }
  if (!is.character(effect_modifier) || length(effect_modifier) != 1L ||
      is.na(effect_modifier) || !nzchar(effect_modifier)) {
    stop("`effect_modifier` must be a single character variable name.",
         call. = FALSE)
  }
  if (!is.null(covariates) &&
      (!is.character(covariates) || anyNA(covariates) ||
       any(!nzchar(covariates)))) {
    stop("`covariates` must be NULL or a character vector of variable names.",
         call. = FALSE)
  }
  if (!is.numeric(alpha) || length(alpha) != 1L ||
      is.na(alpha) || alpha <= 0 || alpha >= 1) {
    stop("`alpha` must be a single number between 0 and 1.", call. = FALSE)
  }
  if (!is.logical(verbose) || length(verbose) != 1L || is.na(verbose)) {
    stop("`verbose` must be TRUE or FALSE.", call. = FALSE)
  }

  approach <- .choice_arg(
    substitute(approach),
    env = parent.frame(),
    choices = c("logit", "logbinomial", "poisson", "robpoisson", "negbin",
                "linear", "cox", "surv", "survreg")
  )
  approach <- .normalize_approach(approach)
  .validate_approach(approach, context = "interaction_models")
  is_survival <- approach %in% c("cox", "survreg")
  if (!is_survival &&
      (!is.character(outcome) || length(outcome) != 1L ||
       is.na(outcome) || !nzchar(outcome))) {
    stop("`outcome` must be a single character variable name.", call. = FALSE)
  }
  time <- if (missing(time) || identical(substitute(time), quote(NULL))) {
    NULL
  } else {
    .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  }
  event <- if (missing(event) || identical(substitute(event), quote(NULL))) {
    NULL
  } else {
    .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  }
  distribution <- if (identical(approach, "survreg")) {
    .surv_distribution_arg(
      substitute(distribution),
      env = parent.frame(),
      multiple = FALSE,
      arg = "distribution"
    )
  } else {
    "weibull"
  }

  test <- .choice_arg(
    substitute(test),
    env = parent.frame(),
    choices = c("LRT", "Wald"),
    lower = FALSE
  )
  test <- match.arg(test, c("LRT", "Wald"))
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  format <- match.arg(format, c("flextable", "gt", "tibble"))

  if (is_survival && (is.null(time) || is.null(event))) {
    stop("`time` and `event` are required when `approach` is cox or survreg.", call. = FALSE)
  }

  needed <- if (is_survival) {
    unique(c(time, event, exposure, effect_modifier, covariates))
  } else {
    unique(c(outcome, exposure, effect_modifier, covariates))
  }
  missing_vars <- setdiff(needed, names(data))
  if (length(missing_vars)) {
    stop("Variables not found: ", paste(missing_vars, collapse = ", "),
         call. = FALSE)
  }

  if (!is_survival) {
    .validate_outcome_by_approach(data[[outcome]], approach)
  } else {
    .validate_cox_inputs(
      data = data,
      time = time,
      event = event,
      exposures = unique(c(exposure, effect_modifier, covariates)),
      adjust_for = NULL
    )
  }

  model_data <- stats::na.omit(data[, needed, drop = FALSE])
  if (nrow(model_data) == 0L) {
    stop("No complete cases available for model fitting.", call. = FALSE)
  }

  if (is_survival) {
    model_data[[event]] <- .cox_event01(model_data[[event]])
  }

  if (approach == "robpoisson" &&
      (is.factor(model_data[[outcome]]) || is.character(model_data[[outcome]]))) {
    lev <- .interaction_binary_levels(model_data[[outcome]])
    if (length(lev) == 2L) {
      model_data[[outcome]] <- ifelse(model_data[[outcome]] == lev[[2]], 1L, 0L)
    }
  }

  base_terms <- unique(c(exposure, effect_modifier, covariates))
  interaction_term <- paste0(exposure, ":", effect_modifier)
  if (is_survival) {
    response <- paste0("survival::Surv(", .surv_bt(time), ", ", .surv_bt(event), ")")
    base_formula <- stats::as.formula(
      paste(response, "~", .survival_rhs(base_terms))
    )
    interaction_formula <- stats::as.formula(
      paste(response, "~", .survival_rhs(base_terms, paste0(exposure, "*", effect_modifier)))
    )
  } else {
    base_formula <- stats::reformulate(base_terms, response = outcome)
    interaction_formula <- stats::reformulate(
      unique(c(base_terms, interaction_term)),
      response = outcome
    )
  }

  fit_model <- function(formula) {
    switch(
      approach,
      "logit" = stats::glm(formula, data = model_data,
                           family = stats::binomial("logit")),
      "logbinomial" = stats::glm(formula, data = model_data,
                                 family = stats::binomial("log")),
      "poisson" = stats::glm(formula, data = model_data,
                             family = stats::poisson("log")),
      "robpoisson" = stats::glm(formula, data = model_data,
                                family = stats::poisson("log")),
      "negbin" = MASS::glm.nb(formula, data = model_data),
      "linear" = stats::lm(formula, data = model_data),
      "cox" = survival::coxph(formula, data = model_data, model = TRUE),
      "survreg" = survival::survreg(formula, data = model_data, dist = distribution, model = TRUE),
      stop("Unsupported regression approach.", call. = FALSE)
    )
  }

  model_no_interaction <- tryCatch(
    fit_model(base_formula),
    error = function(e) structure(list(error = conditionMessage(e)),
                                  class = "interaction_fit_error")
  )
  model_with_interaction <- tryCatch(
    fit_model(interaction_formula),
    error = function(e) structure(list(error = conditionMessage(e)),
                                  class = "interaction_fit_error")
  )

  if (inherits(model_no_interaction, "interaction_fit_error") ||
      inherits(model_with_interaction, "interaction_fit_error")) {
    stop("Model fitting failed for one or both models.", call. = FALSE)
  }

  comparison <- .interaction_compare_models(
    model_no_interaction,
    model_with_interaction,
    test = test,
    approach = approach
  )

  p_value <- comparison$p_value
  has_interaction <- if (is.na(p_value)) NA else p_value < alpha
  decision <- if (is.na(has_interaction)) {
    "not_estimable"
  } else if (has_interaction) {
    "interaction"
  } else {
    "no_interaction"
  }
  interpretation <- .interaction_interpretation(
    decision = decision,
    p_value = p_value,
    alpha = alpha,
    exposure = exposure,
    effect_modifier = effect_modifier
  )

  interaction_terms <- .interaction_terms(model_with_interaction, interaction_term)

  robust_no_interaction <- NULL
  robust_with_interaction <- NULL
  if (approach == "robpoisson") {
    robust_no_interaction <- lmtest::coeftest(
      model_no_interaction,
      vcov. = sandwich::vcovHC(model_no_interaction, type = "HC0")
    )
    robust_with_interaction <- lmtest::coeftest(
      model_with_interaction,
      vcov. = sandwich::vcovHC(model_with_interaction, type = "HC0")
    )
  }

  summary <- tibble::tibble(
    outcome = if (is_survival) paste0(time, "/", event) else outcome,
    exposure = exposure,
    effect_modifier = effect_modifier,
    covariates = if (length(covariates)) paste(covariates, collapse = ", ") else "None",
    approach = approach,
    test = comparison$test_label,
    p_value = p_value,
    alpha = alpha,
    has_interaction = has_interaction,
    decision = decision,
    interpretation = interpretation
  )

  result <- list(
    summary = summary,
    model_no_interaction = model_no_interaction,
    model_with_interaction = model_with_interaction,
    robust_no_interaction = robust_no_interaction,
    robust_with_interaction = robust_with_interaction,
    formula_no_interaction = base_formula,
    formula_with_interaction = interaction_formula,
    interaction_terms = interaction_terms,
    comparison = comparison$comparison,
    p_value = p_value,
    alpha = alpha,
    has_interaction = has_interaction,
    decision = decision,
    interpretation = interpretation,
    test = comparison$test_label,
    approach = approach,
    covariates = covariates,
    source = "interaction_models"
  )
  if (format != "tibble") {
    result$table <- .build_interaction_models_table(
      summary,
      formula_no_interaction = base_formula,
      formula_with_interaction = interaction_formula,
      format = format
    )
  }
  class(result) <- c("interaction_models_result", class(result))

  if (verbose) {
    message(interpretation)
  }

  result
}

#' Build formatted interaction_models table
#' @keywords internal
#' @noRd
.build_interaction_models_table <- function(summary_tbl,
                                            formula_no_interaction,
                                            formula_with_interaction,
                                            format = c("flextable", "gt")) {
  format <- match.arg(format, c("flextable", "gt"))
  note <- paste(
    "Screening aid only; interaction decisions should be interpreted with",
    "subject-matter knowledge, study design, and stratum-specific estimates."
  )
  model_note <- paste0(
    "Models tested: ", paste(deparse(formula_no_interaction), collapse = " "),
    "; ", paste(deparse(formula_with_interaction), collapse = " "), "."
  )

  display <- summary_tbl |>
    dplyr::mutate(
      p_value = dplyr::if_else(
        is.na(.data$p_value),
        "",
        ifelse(.data$p_value < 0.001, "<0.001",
               formatC(.data$p_value, digits = 3, format = "f"))
      ),
      alpha = formatC(.data$alpha, digits = 3, format = "f"),
      has_interaction = dplyr::case_when(
        is.na(.data$has_interaction) ~ "",
        .data$has_interaction ~ "Yes",
        TRUE ~ "No"
      ),
      decision = dplyr::recode(
        .data$decision,
        interaction = "Interaction",
        no_interaction = "No interaction",
        not_estimable = "Not estimable",
        .default = .data$decision
      )
    ) |>
    dplyr::transmute(
      "Outcome" = .data$outcome,
      "Exposure" = .data$exposure,
      "Effect modifier tested" = .data$effect_modifier,
      "Adjustment covariates" = .data$covariates,
      "Approach" = .data$approach,
      "Test" = .data$test,
      "p-value" = .data$p_value,
      "Alpha" = .data$alpha,
      "Interaction?" = .data$has_interaction,
      "Decision" = .data$decision,
      "Interpretation" = .data$interpretation
    )

  if (format == "gt") {
    return(
      gt::gt(display) |>
        gt::tab_header(title = "Interaction screening") |>
        gt::cols_align(
          align = "left",
          columns = c("Outcome", "Exposure", "Effect modifier tested", "Adjustment covariates",
                      "Approach", "Test", "Decision", "Interpretation")
        ) |>
        gt::cols_align(
          align = "center",
          columns = c("p-value", "Alpha", "Interaction?")
        ) |>
        gt::tab_style(
          style = gt::cell_text(weight = "bold"),
          locations = gt::cells_column_labels()
        ) |>
        gt::tab_source_note(gt::md(model_note)) |>
        gt::tab_source_note(gt::md(note)) |>
        .compact_gt_source_notes()
    )
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Interaction screening")
  ft <- flextable::align(
    ft,
    j = c("Outcome", "Exposure", "Effect modifier tested", "Adjustment covariates",
          "Approach", "Test", "Decision", "Interpretation"),
    align = "left",
    part = "all"
  )
  ft <- flextable::align(
    ft,
    j = c("p-value", "Alpha", "Interaction?"),
    align = "center",
    part = "all"
  )
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  ft <- flextable::add_footer_lines(ft, values = c(model_note, note))
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

.interaction_binary_levels <- function(x) {
  x <- x[!is.na(x)]
  if (is.factor(x)) {
    levels(droplevels(x))
  } else {
    sort(unique(x))
  }
}

.interaction_compare_models <- function(model1, model2, test, approach) {
  if (test == "LRT") {
    comp <- tryCatch(
      suppressWarnings(stats::anova(model1, model2, test = "LRT")),
      error = function(e) NULL
    )
    p_value <- .interaction_extract_lrt_p(comp)
    if (is.na(p_value)) {
      p_value <- .interaction_lrt_from_loglik(model1, model2)
    }
    return(list(
      comparison = comp,
      p_value = p_value,
      test_label = "Likelihood Ratio Test"
    ))
  }

  comp <- tryCatch(
    suppressWarnings(lmtest::waldtest(model1, model2)),
    error = function(e) NULL
  )
  p_value <- if (!is.null(comp) && nrow(comp) >= 2L) {
    cols <- grep("^Pr\\(", names(comp), value = TRUE)
    if (length(cols)) suppressWarnings(as.numeric(comp[[cols[[1]]]][2])) else NA_real_
  } else {
    NA_real_
  }

  list(
    comparison = comp,
    p_value = p_value,
    test_label = "Wald Test"
  )
}

.interaction_extract_lrt_p <- function(comp) {
  if (is.null(comp) || nrow(comp) < 2L) {
    return(NA_real_)
  }
  cols <- grep("^Pr\\(", names(comp), value = TRUE)
  if (!length(cols)) {
    return(NA_real_)
  }
  suppressWarnings(as.numeric(comp[[cols[[1]]]][2]))
}

.interaction_lrt_from_loglik <- function(model1, model2) {
  ll1 <- tryCatch(stats::logLik(model1), error = function(e) NULL)
  ll2 <- tryCatch(stats::logLik(model2), error = function(e) NULL)
  if (is.null(ll1) || is.null(ll2)) {
    return(NA_real_)
  }
  df <- attr(ll2, "df") - attr(ll1, "df")
  chisq <- 2 * (as.numeric(ll2) - as.numeric(ll1))
  if (!is.finite(df) || !is.finite(chisq) || df <= 0 || chisq < 0) {
    return(NA_real_)
  }
  stats::pchisq(chisq, df = df, lower.tail = FALSE)
}

.interaction_terms <- function(model, interaction_term) {
  coef_names <- names(stats::coef(model))
  if (is.null(coef_names)) {
    return(character(0))
  }
  parts <- strsplit(interaction_term, ":", fixed = TRUE)[[1]]
  coef_names[vapply(
    coef_names,
    function(x) all(vapply(parts, grepl, logical(1), x = x, fixed = TRUE)),
    logical(1)
  )]
}

.interaction_interpretation <- function(decision,
                                        p_value,
                                        alpha,
                                        exposure,
                                        effect_modifier) {
  if (decision == "interaction") {
    return(paste0(
      "Evidence of interaction between ", exposure, " and ", effect_modifier,
      " (p = ", formatC(p_value, digits = 4, format = "f"),
      "); consider reporting stratum-specific effects or including the interaction term."
    ))
  }
  if (decision == "no_interaction") {
    return(paste0(
      "No statistical evidence of interaction between ", exposure, " and ",
      effect_modifier, " at alpha = ", alpha, "."
    ))
  }
  paste0(
    "Interaction could not be estimated for ", exposure, " and ",
    effect_modifier, "."
  )
}

birthwt_interaction_data <- function() {
  data("data_birthwt", package = "gtregression")

  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
    )
}

lung_interaction_data <- function() {
  data("data_lungcancer", package = "gtregression")

  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )
}

test_that("interaction_models returns focused result for logit LRT", {
  df <- birthwt_interaction_data()

  result <- interaction_models(
    data = df,
    outcome = "low",
    exposure = "smoke",
    effect_modifier = "race",
    covariates = c("age", "lwt"),
    approach = logit,
    test = LRT,
    format = tibble
  )

  expect_s3_class(result, "interaction_models_result")
  expect_named(
    result,
    c(
      "summary", "model_no_interaction", "model_with_interaction",
      "robust_no_interaction", "robust_with_interaction",
      "formula_no_interaction", "formula_with_interaction",
      "interaction_terms", "comparison", "p_value", "alpha",
      "has_interaction", "decision", "interpretation", "test",
      "approach", "covariates", "source"
    )
  )
  expect_s3_class(result$summary, "tbl_df")
  expect_equal(result$covariates, c("age", "lwt"))
  expect_equal(result$test, "Likelihood Ratio Test")
  expect_true(is.numeric(result$p_value) || is.na(result$p_value))
  expect_true(result$decision %in%
                c("interaction", "no_interaction", "not_estimable"))
  expect_true(length(result$interaction_terms) >= 1)
})

test_that("interaction_models supports Wald test and verbose message", {
  df <- birthwt_interaction_data()

  expect_message(
    result <- interaction_models(
      data = df,
      outcome = "low",
      exposure = "smoke",
      effect_modifier = "race",
      approach = "logit",
      test = "Wald",
      verbose = TRUE
    ),
    "interaction"
  )

  expect_equal(result$test, "Wald Test")
  expect_s3_class(result$summary, "tbl_df")
})

test_that("interaction_models can add gt and flextable viewing tables", {
  df <- birthwt_interaction_data()

  gt_result <- interaction_models(
    data = df,
    outcome = "low",
    exposure = "smoke",
    effect_modifier = "race",
    approach = logit,
    format = gt
  )
  ft_result <- interaction_models(
    data = df,
    outcome = "low",
    exposure = "smoke",
    effect_modifier = "race",
    approach = logit,
    format = flextable
  )

  expect_s3_class(gt_result$summary, "tbl_df")
  expect_s3_class(gt_result$table, "gt_tbl")
  expect_s3_class(ft_result$table, "flextable")
  gt_notes <- vapply(gt_result$table$`_source_notes`, as.character, character(1))
  expect_true(any(grepl("Models tested:", gt_notes, fixed = TRUE)))
  expect_true(any(grepl("Screening aid only", gt_notes, fixed = TRUE)))
  expect_true("Effect modifier tested" %in% names(ft_result$table$header$dataset))
  expect_true(any(grepl(
    "Models tested:",
    unlist(ft_result$table$footer$dataset, use.names = FALSE),
    fixed = TRUE
  )))
})

test_that("interaction_models supports robpoisson and robust coefficient tables", {
  df <- birthwt_interaction_data()

  result <- interaction_models(
    data = df,
    outcome = "low",
    exposure = "smoke",
    effect_modifier = "race",
    approach = robpoisson,
    test = Wald
  )

  expect_equal(result$approach, "robpoisson")
  expect_s3_class(result$robust_no_interaction, "coeftest")
  expect_s3_class(result$robust_with_interaction, "coeftest")
  expect_s3_class(result$model_no_interaction, "glm")
})

test_that("interaction_models supports linear models with covariates", {
  set.seed(2)
  n <- 150
  df <- data.frame(
    y = rnorm(n),
    x = factor(sample(c("A", "B"), n, TRUE)),
    z = factor(sample(c("Low", "High"), n, TRUE)),
    age = rnorm(n)
  )

  result <- interaction_models(
    data = df,
    outcome = "y",
    exposure = "x",
    effect_modifier = "z",
    covariates = "age",
    approach = linear,
    test = LRT
  )

  expect_s3_class(result$model_no_interaction, "lm")
  expect_equal(result$summary$approach, "linear")
  expect_match(deparse(result$formula_with_interaction), "x:z")
})

test_that("interaction_models supports poisson and negative binomial models", {
  data("quine", package = "MASS")
  quine_data <- quine |>
    dplyr::mutate(dplyr::across(c(Eth, Sex, Age, Lrn), as.factor))

  pois <- interaction_models(
    data = quine_data,
    outcome = "Days",
    exposure = "Sex",
    effect_modifier = "Eth",
    covariates = "Age",
    approach = poisson
  )
  nb <- interaction_models(
    data = quine_data,
    outcome = "Days",
    exposure = "Sex",
    effect_modifier = "Eth",
    covariates = "Age",
    approach = negbin
  )

  expect_s3_class(pois$model_no_interaction, "glm")
  expect_s3_class(nb$model_no_interaction, "negbin")
  expect_true(is.numeric(pois$p_value) || is.na(pois$p_value))
  expect_true(is.numeric(nb$p_value) || is.na(nb$p_value))
})

test_that("interaction_models supports Cox and parametric survival models", {
  skip_if_not_installed("survival")

  df <- lung_interaction_data()

  cox_result <- interaction_models(
    data = df,
    time = time,
    event = status,
    exposure = trt,
    effect_modifier = prior,
    covariates = c(age, karno),
    approach = cox,
    test = LRT,
    format = tibble
  )

  expect_s3_class(cox_result, "interaction_models_result")
  expect_s3_class(cox_result$model_no_interaction, "coxph")
  expect_s3_class(cox_result$model_with_interaction, "coxph")
  expect_equal(cox_result$summary$approach, "cox")
  expect_equal(cox_result$summary$outcome, "time/status")
  expect_true(length(cox_result$interaction_terms) >= 1)
  expect_true(is.numeric(cox_result$p_value) || is.na(cox_result$p_value))

  surv_result <- interaction_models(
    data = df,
    time = "time",
    event = "status",
    exposure = "trt",
    effect_modifier = "prior",
    covariates = c("age", "karno"),
    approach = surv,
    distribution = weibull,
    test = LRT,
    format = tibble
  )

  expect_s3_class(surv_result$model_no_interaction, "survreg")
  expect_s3_class(surv_result$model_with_interaction, "survreg")
  expect_equal(surv_result$summary$approach, "survreg")
  expect_true(is.numeric(surv_result$p_value) || is.na(surv_result$p_value))
})

test_that("interaction_models classifies detected interactions using alpha", {
  df <- birthwt_interaction_data()

  result <- interaction_models(
    data = df,
    outcome = "low",
    exposure = "smoke",
    effect_modifier = "race",
    approach = logit,
    alpha = 0.99
  )

  expect_true(result$has_interaction)
  expect_equal(result$decision, "interaction")
  expect_match(result$interpretation, "Evidence of interaction")
})

test_that("interaction_models handles not-estimable comparison", {
  model1 <- lm(mpg ~ wt, data = mtcars)
  comp <- gtregression:::.interaction_compare_models(
    model1,
    model1,
    test = "LRT",
    approach = "linear"
  )

  expect_true(is.na(comp$p_value))
})

test_that("interaction_models validates inputs clearly", {
  df <- birthwt_interaction_data()

  expect_error(
    interaction_models(list(), "low", "smoke", effect_modifier = "race"),
    "`data` must be a data frame"
  )
  expect_error(
    interaction_models(df, c("low", "bwt"), "smoke", effect_modifier = "race"),
    "`outcome` must be a single character variable name"
  )
  expect_error(
    interaction_models(df, "low", c("smoke", "ht"), effect_modifier = "race"),
    "`exposure` must be a single character variable name"
  )
  expect_error(
    interaction_models(df, "low", "smoke", effect_modifier = c("race", "ht")),
    "`effect_modifier` must be a single character variable name"
  )
  expect_error(
    interaction_models(df, "low", "smoke", effect_modifier = "race",
                       covariates = c("age", NA_character_)),
    "`covariates` must be NULL or a character vector"
  )
  expect_error(
    interaction_models(df, "low", "smoke", effect_modifier = "race",
                       alpha = 1),
    "`alpha` must be a single number between 0 and 1"
  )
  expect_error(
    interaction_models(df, "low", "smoke", effect_modifier = "race",
                       verbose = NA),
    "`verbose` must be TRUE or FALSE"
  )
  expect_error(
    interaction_models(df, "missing", "smoke", effect_modifier = "race"),
    "Variables not found: missing"
  )
  expect_error(
    interaction_models(df, "low", "smoke", effect_modifier = "race",
                       approach = "bad"),
    "not a valid approach"
  )
})

test_that("interaction_models errors when no complete cases are available", {
  df <- birthwt_interaction_data()
  df$smoke <- NA

  expect_error(
    interaction_models(
      data = df,
      outcome = "low",
      exposure = "smoke",
      effect_modifier = "race",
      approach = logit
    ),
    "No complete cases available"
  )
})

#' Stratified multivariable regression
#'
#' Fits multivariable regression models within each stratum and returns a
#' unified wide table with one "Characteristic" column and, under bold
#' spanners for each stratum, model N, "Adjusted <effect>", and "p-value".
#'
#' If \code{adjust_for = NULL}, all \code{exposures} are included in one
#' multivariable model within each stratum. If \code{adjust_for} is supplied,
#' one adjusted model is fitted per exposure within each stratum.
#'
#' @param data A data frame containing the variables.
#' @param outcome Character scalar; name of the outcome variable. Quoted and
#'   bare names are accepted.
#' @param exposures Character vector of exposure variables to report. Quoted
#'   names are recommended in scripts, and bare names are also accepted.
#' @param stratifier Character scalar; name of the stratifying variable. Quoted
#'   and bare names are accepted.
#' @param adjust_for Optional character vector of adjustment variables.
#'   Quoted and bare names are accepted. This argument works the same way as in
#'   \code{multi_reg()}.
#' @param interaction Optional character scalar specifying one interaction term
#'   using standard formula syntax, e.g. \code{"bmi*sex"}
#' @param approach One of \code{"logit"}, \code{"firth"}, \code{"logbinomial"}, \code{"poisson"},
#'   \code{"linear"}, \code{"robpoisson"}, or \code{"negbin"}
#' @param format One of \code{"flextable"} (default) or \code{"gt"}.
#' @param theme Preset name (e.g. \code{"minimal"}, \code{"striped"}, \code{"clinical"},
#'   \code{"shaded"}, \code{"jama"}) or primitives
#'   \code{c("plain","zebra","lines","labels_bold","compact","header_shaded")}
#' @param show_ref Logical; if \code{TRUE} (default), display reference-category
#'   rows as \code{"Ref."}. If \code{FALSE}, hide reference rows; a message
#'   reminds users to use \code{show_ref = TRUE} when reference rows are needed.
#'
#' @details
#' If exposure variables have a \code{"label"} attribute, for example from
#' \code{labelled::var_label()}, those labels are used automatically in the
#' displayed table. Internal matching still uses the original column names.
#'
#' @return A list of class \code{c("gtregression","stratified_multi_reg", ...)} with:
#' \describe{
#'   \item{\code{table}}{A \code{flextable} (format = \code{"flextable"}) or
#'   \code{gt_tbl} (format = \code{"gt"}).}
#'   \item{\code{table_display}}{Wide data frame used to build the table.}
#'   \item{\code{per_stratum}}{Named list of per-stratum regression results.}
#'   \item{\code{models}}{Named list of fitted models by stratum.}
#'   \item{\code{model_summaries}}{Named list of model summaries by stratum.}
#'   \item{\code{variable_labels}}{Named character vector of display labels used
#'   for exposure variables.}
#'   \item{\code{reg_check}}{Named list of diagnostics by stratum.}
#'   \item{\code{by}, \code{levels}, \code{approach}, \code{format}, \code{source}}{Metadata fields.}
#' }
#'
#' @examples
#' birthwt_data <- data_birthwt |>
#'   transform(
#'     race = factor(race, levels = c(1, 2, 3),
#'                   labels = c("White", "Black", "Other")),
#'     smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
#'     ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
#'     ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
#'     low = factor(low, levels = c(0, 1),
#'                  labels = c("Normal BW", "Low BW"))
#'   )
#'
#' stratified_multi <- stratified_multi_reg(
#'   data = birthwt_data,
#'   outcome = "low",
#'   exposures = c("age", "lwt", "smoke", "ht"),
#'   stratifier = "race",
#'   approach = "logit",
#'   format = "gt"
#' )
#'
#' stratified_adjusted <- stratified_multi_reg(
#'   data = birthwt_data,
#'   outcome = "low",
#'   exposures = c("smoke", "ht", "ui"),
#'   stratifier = "race",
#'   adjust_for = c("age", "lwt"),
#'   approach = "logit",
#'   format = "gt"
#' )
#' @importFrom stats nobs
#' @export
stratified_multi_reg <- function(data,
                                 outcome,
                                 exposures,
                                 stratifier,
                                 adjust_for = NULL,
                                 interaction = NULL,
                                 approach = "logit",
                                 format = c("flextable", "gt"),
                                 theme = c("minimal"),
                                 show_ref = TRUE) {

  outcome <- .vars_arg(substitute(outcome), env = parent.frame())
  exposures <- .vars_arg(substitute(exposures), env = parent.frame())
  stratifier <- .vars_arg(substitute(stratifier), env = parent.frame())
  adjust_for <- .vars_arg(substitute(adjust_for), env = parent.frame(), allow_null = TRUE)
  interaction <- .interaction_arg(substitute(interaction), env = parent.frame(), allow_null = TRUE)
  approach <- .choice_arg(
    substitute(approach),
    env = parent.frame(),
    choices = c("logit","firth","logbinomial","poisson","robpoisson","linear","negbin")
  )
  approach <- .normalize_approach(approach)
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("flextable","gt"))
  theme <- .choice_arg(substitute(theme), env = parent.frame())
  .validate_show_ref(show_ref)

  format <- match.arg(format, c("flextable","gt"))
  theme <- .resolve_theme(theme)
  variable_labels <- .var_label_map(
    data,
    unique(c(exposures, adjust_for, .interaction_vars(interaction)))
  )

  if (!stratifier %in% names(data)) {
    stop("Stratifier not found in dataset.", call. = FALSE)
  }

  data <- data[!is.na(data[[stratifier]]), , drop = FALSE]
  levs <- .strata_levels(data, stratifier)

  message("Running stratified multivariable regression by: ", stratifier)

  per_stratum <- list()
  tds <- list()
  models <- list()
  sums <- list()
  diags <- list()
  n_by_stratum <- list()

  for (lv in levs) {
    message("  > Stratum: ", stratifier, " = ", lv)

    dlev <- data[data[[stratifier]] == lv, , drop = FALSE]
    key <- as.character(lv)

    res_i <- tryCatch(
      .run_multi_core(
        data = dlev,
        outcome = outcome,
        exposures = exposures,
        approach = approach,
        adjust_for = adjust_for,
        interaction = interaction
      ),
      error = function(e) {
        warning("Skipping stratum ", lv, ": ", e$message, call. = FALSE)
        NULL
      }
    )

    if (is.null(res_i)) {
      next
    }

    res_i$table_body <- .format_interaction_levels(
      res_i$table_body, dlev, variable_labels
    )

    tds[[key]] <- res_i$table_body
    models[[key]] <- res_i$models
    sums[[key]] <- res_i$model_summaries
    diags[[key]] <- res_i$reg_check
    n_by_stratum[[key]] <- res_i$n_used

    per_stratum[[key]] <- list(
      table_body = res_i$table_body,
      models = res_i$models,
      model_summaries = res_i$model_summaries,
      reg_check = res_i$reg_check,
      n_used = res_i$n_used,
      adjusted_mode = res_i$adjusted_mode
    )
  }

  if (!length(tds)) {
    stop("No valid models across strata.", call. = FALSE)
  }

  built <- .strata_build_wide_multi(
    data,
    exposures,
    stratifier,
    tds,
    variable_labels = variable_labels,
    show_ref = show_ref
  )
  wide <- built$wide
  spanners <- built$spanners
  eff_lab <- paste("Adjusted", .get_effect_label(approach))

  footnotes <- c(
    .stratified_by_note(stratifier),
    .abbrev_note(approach),
    if (
      isTRUE(show_ref) &&
        any(unlist(lapply(tds, function(x) x$ref %in% TRUE), use.names = FALSE))
    ) .ref_note() else NULL,
    if (!is.null(adjust_for) && length(adjust_for) > 0) .adjustment_note(adjust_for) else NULL,
    if (!is.null(interaction)) .interaction_note(interaction) else NULL,
    .n_note_multi_strata(stratifier, n_by_stratum)
  )

  .message_hidden_ref_rows(
    "stratified_multi_reg",
    do.call(rbind, tds),
    show_ref
  )

  tbl <- if (format == "gt") {
    .build_gt_strata_wide_multi(wide, spanners, eff_lab, theme, footnotes)
  } else {
    .build_flex_strata_wide_multi(wide, spanners, eff_lab, theme, footnotes)
  }

  fmt_class <- if (format == "gt") "gt_strata_multi" else "ft_strata_multi"

  out <- list(
    table = tbl,
    table_display = wide,
    per_stratum = per_stratum,
    models = models,
    model_summaries = sums,
    variable_labels = variable_labels,
    reg_check = .as_reg_check_result(diags, format = format),
    by = stratifier,
    levels = levs,
    approach = approach,
    format = format,
    source = "stratified_multi_reg",
    show_ref = isTRUE(show_ref)
  )

  class(out) <- c("gtregression", "stratified_multi_reg", fmt_class, class(out))
  out
}

birthwt_strata_multi_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
      ptl_cat = ifelse(ptl > 0, "Yes", "No"),
      ftv_cat = dplyr::case_when(
        ftv == 0 ~ "None",
        ftv == 1 ~ "One",
        ftv >= 2 ~ "Two or more"
      )
    ) |>
    dplyr::mutate(
      ptl_cat = factor(ptl_cat, levels = c("No", "Yes")),
      ftv_cat = factor(ftv_cat, levels = c("None", "One", "Two or more"))
    )
}

test_that("stratified_multi_reg returns a gtregression object for full logit models", {
  df <- birthwt_strata_multi_data()

  res <- suppressMessages(
    stratified_multi_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "lwt", "smoke"),
      stratifier = "race",
      approach = logit,
      format = gt
    )
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "stratified_multi_reg")
  expect_s3_class(res, "gt_strata_multi")
  expect_s3_class(res$table, "gt_tbl")
  expect_equal(res$approach, "logit")
  expect_equal(res$format, "gt")
  expect_equal(res$source, "stratified_multi_reg")
  expect_equal(res$by, "race")
  expect_equal(res$levels, c("White", "Black", "Other"))
  expect_named(
    res,
    c("table", "table_display", "per_stratum", "models",
      "model_summaries", "variable_labels", "reg_check", "by", "levels", "approach",
      "format", "source", "show_ref")
  )
  expect_named(res$models, c("White", "Black", "Other"))
  expect_named(res$models$White, "multivariable_model")
  expect_true(all(c("Characteristic", "is_header", "..eff__White",
                    "..p__White") %in% names(res$table_display)))
  expect_false("..N__White" %in% names(res$table_display))
  eff_cols <- startsWith(names(res$table_display), "..eff__")
  expect_true("Ref." %in% unlist(res$table_display[eff_cols]))
  expect_true(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                        fixed = TRUE)))
})

test_that("stratified_multi_reg adjust_for fits one adjusted model per exposure", {
  df <- birthwt_strata_multi_data()

  res <- suppressMessages(
    stratified_multi_reg(
      data = df,
      outcome = "low",
      exposures = c("smoke", "ht", "ui"),
      stratifier = "race",
      adjust_for = c("age", "lwt"),
      approach = logit,
      theme = clinical
    )
  )

  expect_s3_class(res, "stratified_multi_reg")
  expect_named(res$models$White, c("smoke", "ht", "ui"))
  expect_named(res$model_summaries$White, c("smoke", "ht", "ui"))
  expect_true(res$per_stratum$White$adjusted_mode)
  expect_true(res$per_stratum$Black$adjusted_mode)
  expect_false("..N__White" %in% names(res$table_display))
  expect_true(all(c("smoke", "ht", "ui") %in% res$table_display$Characteristic))
})

test_that("stratified_multi_reg preserves factor strata order and excludes missing strata", {
  df <- birthwt_strata_multi_data()
  df$race <- factor(df$race, levels = c("Other", "White", "Black"))
  df$race[1:3] <- NA

  res <- suppressMessages(
    stratified_multi_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "smoke"),
      stratifier = "race",
      approach = "logit"
    )
  )

  expect_equal(res$levels, c("Other", "White", "Black"))
  expect_named(res$models, c("Other", "White", "Black"))
  expect_true(all(!grepl("NA", names(res$table_display), fixed = TRUE)))
})

test_that("stratified_multi_reg explains how to show reference rows when hidden", {
  df <- birthwt_strata_multi_data()

  expect_message(
    res <- stratified_multi_reg(
      data = df,
      outcome = low,
      exposures = smoke,
      stratifier = ht,
      adjust_for = age,
      approach = logit,
      show_ref = FALSE
    ),
    "To display Ref., use `show_ref = TRUE`.",
    fixed = TRUE
  )

  expect_true(any(vapply(res$per_stratum, function(x) any(x$table_body$ref), logical(1))))
  eff_cols <- startsWith(names(res$table_display), "..eff__")
  expect_false("Ref." %in% unlist(res$table_display[eff_cols]))
  expect_false(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                         fixed = TRUE)))
})

test_that("stratified_multi_reg accepts bare and quoted options", {
  df <- birthwt_strata_multi_data()

  bare <- suppressMessages(
    stratified_multi_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "smoke"),
      stratifier = "race",
      approach = logit,
      format = gt,
      theme = clinical
    )
  )
  quoted <- suppressMessages(
    stratified_multi_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "smoke"),
      stratifier = "race",
      approach = "logit",
      format = "gt",
      theme = "clinical"
    )
  )

  expect_equal(bare$approach, quoted$approach)
  expect_equal(bare$format, quoted$format)
  expect_equal(bare$table_display, quoted$table_display)
})

test_that("stratified_multi_reg supports flextable output", {
  skip_if_not_installed("flextable")

  df <- birthwt_strata_multi_data()

  res <- suppressMessages(
    stratified_multi_reg(
      data = df,
      outcome = "low",
      exposures = c("smoke", "ht"),
      stratifier = "race",
      adjust_for = c("age", "lwt"),
      approach = logit,
      format = flextable
    )
  )

  expect_s3_class(res, "ft_strata_multi")
  expect_s3_class(res$table, "flextable")
  expect_equal(res$format, "flextable")
})

test_that("stratified_multi_reg passes interactions to stratum-specific models", {
  df <- birthwt_strata_multi_data()

  res <- suppressMessages(
    stratified_multi_reg(
      data = df,
      outcome = "low",
      exposures = "smoke",
      stratifier = "race",
      adjust_for = c("age", "lwt"),
      interaction = "smoke*ht",
      approach = logit
    )
  )

  expect_s3_class(res, "stratified_multi_reg")
  expect_named(res$models$White, "smoke")
  expect_match(
    paste(deparse(stats::formula(res$models$White$smoke)), collapse = " "),
    "smoke \\* ht"
  )
})

test_that("stratified_multi_reg returns diagnostics for linear models", {
  df <- birthwt_strata_multi_data()

  res <- suppressMessages(
    stratified_multi_reg(
      data = df,
      outcome = "bwt",
      exposures = c("age", "lwt", "smoke"),
      stratifier = "race",
      approach = linear
    )
  )

  expect_s3_class(res, "stratified_multi_reg")
  expect_equal(res$approach, "linear")
  expect_s3_class(res$reg_check, "gtregression_reg_check")
  expect_true("..eff__White" %in% names(res$table_display))
  expect_true("multivariable_model" %in% names(res$reg_check$White))
  expect_true("Test" %in% names(res$reg_check$White$multivariable_model))
})

test_that("stratified_multi_reg validates invalid inputs through stratum failures", {
  df <- birthwt_strata_multi_data()

  expect_error(
    stratified_multi_reg(
      df,
      outcome = "low",
      exposures = "age",
      stratifier = "not_here",
      approach = logit
    ),
    "Stratifier not found"
  )
  expect_error(
    suppressWarnings(
      suppressMessages(
        stratified_multi_reg(
          df,
          outcome = "low",
          exposures = "not_here",
          stratifier = "race",
          approach = logit
        )
      )
    ),
    "No valid models across strata"
  )
  expect_error(
    suppressWarnings(
      suppressMessages(
        stratified_multi_reg(
          df,
          outcome = "bwt",
          exposures = "age",
          stratifier = "race",
          approach = logit
        )
      )
    ),
    "No valid models across strata"
  )
})

test_that("stratified_multi_reg validates adjustment and interaction inputs", {
  df <- birthwt_strata_multi_data()

  expect_error(
    suppressWarnings(
      suppressMessages(
        stratified_multi_reg(
          df,
          outcome = "low",
          exposures = "smoke",
          stratifier = "race",
          adjust_for = "smoke",
          approach = logit
        )
      )
    ),
    "No valid models across strata"
  )
  expect_error(
    suppressWarnings(
      suppressMessages(
        stratified_multi_reg(
          df,
          outcome = "low",
          exposures = "smoke",
          stratifier = "race",
          adjust_for = "low",
          approach = logit
        )
      )
    ),
    "No valid models across strata"
  )
  expect_error(
    suppressWarnings(
      suppressMessages(
        stratified_multi_reg(
          df,
          outcome = "low",
          exposures = "smoke",
          stratifier = "race",
          interaction = "smoke:ht",
          approach = logit
        )
      )
    ),
    "No valid models across strata"
  )
})

test_that("stratified_multi_reg errors when no valid strata remain", {
  df <- birthwt_strata_multi_data()
  df$race <- NA_character_

  expect_error(
    suppressWarnings(
      suppressMessages(
        stratified_multi_reg(
          df,
          outcome = "low",
          exposures = c("age", "lwt"),
          stratifier = "race",
          approach = logit
        )
      )
    ),
    "No valid models across strata"
  )
})

#' Get a display label from a variable label attribute
#'
#' @keywords internal
#' @noRd
.var_label_one <- function(data, var) {
  if (is.null(data) || !var %in% names(data)) {
    return(var)
  }
  lab <- attr(data[[var]], "label", exact = TRUE)
  if (is.null(lab) || length(lab) < 1L || is.na(lab[1]) || !nzchar(as.character(lab[1]))) {
    return(var)
  }
  as.character(lab[1])
}

#' Build a named variable-label map
#'
#' @keywords internal
#' @noRd
.var_label_map <- function(data, vars) {
  vars <- unique(as.character(vars))
  stats::setNames(vapply(vars, .var_label_one, character(1), data = data), vars)
}

#' Look up a display label from a named map
#'
#' @keywords internal
#' @noRd
.label_var <- function(var, label_map = NULL) {
  if (is.null(label_map) || !length(label_map) || is.na(match(var, names(label_map)))) {
    return(var)
  }
  out <- unname(label_map[[var]])
  if (is.null(out) || is.na(out) || !nzchar(out)) var else out
}

#' Convert model-matrix interaction names into readable display labels
#' @keywords internal
#' @noRd
.format_interaction_levels <- function(table_body, data, variable_labels = NULL) {
  if (!is.data.frame(table_body) || !"level" %in% names(table_body)) {
    return(table_body)
  }

  idx <- grepl(" x ", table_body$level, fixed = TRUE)
  if (!any(idx)) return(table_body)

  label_map <- variable_labels
  if (is.null(label_map)) {
    label_map <- .var_label_map(data, names(data))
  }
  vars <- names(data)[order(nchar(names(data)), decreasing = TRUE)]

  format_piece <- function(piece) {
    piece <- gsub("`", "", piece, fixed = TRUE)
    for (var in vars) {
      if (!startsWith(piece, var)) next
      level <- substr(piece, nchar(var) + 1L, nchar(piece))
      label <- .label_var(var, label_map)
      return(if (nzchar(level)) paste0(label, ": ", level) else label)
    }
    piece
  }

  table_body$level[idx] <- vapply(
    strsplit(table_body$level[idx], " x ", fixed = TRUE),
    function(parts) paste(vapply(parts, format_piece, character(1)), collapse = " x "),
    character(1)
  )
  table_body
}

#' Attach raw row metadata to display data frames
#'
#' @keywords internal
#' @noRd
.attach_display_metadata <- function(display_df, row_exposure = NULL, variable_labels = NULL) {
  if (!is.null(row_exposure) && length(row_exposure) == nrow(display_df)) {
    attr(display_df, "row_exposure") <- as.character(row_exposure)
  }
  if (!is.null(variable_labels)) {
    attr(display_df, "variable_labels") <- variable_labels
  }
  display_df
}

#' Plot observed and fitted parametric survival curves
#'
#' Compare the observed Kaplan-Meier survival curve with fitted parametric
#' survival curves from \code{survival::survreg()}.
#'
#' @param data A \code{data.frame} containing survival time, event status, and
#'   optional grouping or adjustment variables.
#' @param time Survival follow-up time. Quoted and bare names are accepted.
#' @param event Event indicator. Quoted and bare names are accepted. Numeric
#'   \code{0/1}, numeric \code{1/2}, logical, character, and factor variables
#'   are accepted. For two-level character or factor variables, the second level
#'   is treated as the event.
#' @param by Optional grouping variable for observed and fitted curves. Quoted
#'   and bare names are accepted.
#' @param adjust_for Optional character vector of adjustment variables included
#'   in the fitted parametric model. Fitted curves are predicted at typical
#'   adjustment values.
#' @param distributions Parametric survival distributions to overlay. One or
#'   more of \code{"weibull"}, \code{"exponential"}, \code{"lognormal"}, or
#'   \code{"loglogistic"}. Quoted and bare values are accepted. Common
#'   spellings such as \code{"log-normal"} and \code{"log-logistic"} are also
#'   accepted.
#' @param break_time_by Optional numeric interval for x-axis breaks. If
#'   \code{NULL}, breaks are chosen automatically.
#' @param xlim Optional numeric vector of length 2 specifying x-axis limits.
#' @param xlab,ylab Axis labels.
#' @param title Optional plot title.
#' @param legend_title Optional legend title. If \code{NULL}, the labelled
#'   \code{by} variable name is used.
#' @param palette Optional character vector of colors for observed groups.
#' @param base_size Base font size.
#' @param n_points Number of points used to draw each fitted curve.
#'
#' @details
#' \code{plot_surv_fit()} is a visual diagnostic for parametric survival
#' modelling. It is useful after \code{surv_model_compare()} and before treating
#' a final \code{surv_reg()} model as the preferred model. It is not a Cox-model
#' diagnostic; use \code{check_ph()} for Cox proportional hazards assumptions.
#'
#' When \code{adjust_for} is supplied, fitted curves are predicted at typical
#' adjustment values: medians for numeric variables and the most common level for
#' categorical variables. Use this as a model-fit screen, not as a replacement
#' for clinical or subject-matter judgement.
#'
#' @return A \code{ggplot2} object with attributes \code{km_fit},
#'   \code{model_fits}, \code{observed_data}, \code{fitted_data}, and
#'   \code{prediction_data}.
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#'
#' plot_surv_fit(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   by = trt,
#'   distributions = c(weibull, lognormal),
#'   break_time_by = 200
#' )
#'
#' plot_surv_fit(
#'   data = lung_data,
#'   time = "time",
#'   event = "status",
#'   by = "trt",
#'   adjust_for = c(age, karno),
#'   distributions = "log-logistic"
#' )
#'
#' @importFrom survival Surv survfit survreg
#' @importFrom stats complete.cases median predict
#' @importFrom ggplot2 ggplot aes geom_step geom_line labs theme_minimal theme element_text
#' @importFrom ggplot2 scale_y_continuous scale_x_continuous coord_cartesian
#' @importFrom ggplot2 scale_color_manual scale_linetype_manual
#' @export
plot_surv_fit <- function(data,
                          time,
                          event,
                          by = NULL,
                          adjust_for = NULL,
                          distributions = c("weibull", "exponential", "lognormal", "loglogistic"),
                          break_time_by = NULL,
                          xlim = NULL,
                          xlab = "Time",
                          ylab = "Survival probability",
                          title = NULL,
                          legend_title = NULL,
                          palette = NULL,
                          base_size = 13,
                          n_points = 200) {

  time <- .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  event <- .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  by <- .vars_arg(substitute(by), env = parent.frame(), allow_null = TRUE)
  adjust_for <- .vars_arg(substitute(adjust_for), env = parent.frame(), allow_null = TRUE)
  distributions <- .surv_distribution_arg(
    substitute(distributions),
    env = parent.frame(),
    multiple = TRUE,
    arg = "distributions"
  )

  data_clean <- .validate_plot_surv_fit_inputs(
    data = data,
    time = time,
    event = event,
    by = by,
    adjust_for = adjust_for,
    distributions = distributions,
    break_time_by = break_time_by,
    xlim = xlim,
    base_size = base_size,
    n_points = n_points
  )

  if (!is.null(by)) {
    data_clean[[by]] <- factor(data_clean[[by]])
  }

  km_fit <- survival::survfit(.km_formula(time, event, by), data = data_clean)
  observed <- .km_tidy_survfit(km_fit)
  observed$Curve <- "Observed KM"
  observed$Distribution <- "Kaplan-Meier"

  predictors <- unique(c(by, adjust_for))
  fitted_models <- vector("list", length(distributions))
  names(fitted_models) <- distributions

  for (dist in distributions) {
    fitted_models[[dist]] <- .fit_surv_model(data_clean, time, event, predictors, dist)
    if (is.null(fitted_models[[dist]])) {
      stop("Parametric survival model fitting failed for distribution '", dist, "'.", call. = FALSE)
    }
  }

  prediction_data <- .plot_surv_fit_newdata(data_clean, by = by, adjust_for = adjust_for)
  fitted <- .plot_surv_fit_fitted_curves(
    models = fitted_models,
    newdata = prediction_data,
    by = by,
    n_points = n_points
  )

  breaks <- .km_time_breaks(data_clean[[time]], break_time_by, xlim)
  strata_count <- length(unique(observed$strata))
  if (is.null(palette)) {
    palette <- .km_default_palette(strata_count)
  }
  strata_levels <- unique(as.character(observed$strata))

  if (is.null(legend_title)) {
    legend_title <- if (is.null(by)) "Curve" else .label_var(by, .var_label_map(data, by))
  }
  if (is.null(title)) {
    title <- "Observed and fitted survival curves"
  }

  plot_data <- rbind(
    observed[, c("time", "survival", "strata", "Curve", "Distribution"), drop = FALSE],
    fitted[, c("time", "survival", "strata", "Curve", "Distribution"), drop = FALSE]
  )
  plot_data$strata <- factor(plot_data$strata, levels = strata_levels)

  p <- ggplot2::ggplot(
    plot_data,
    ggplot2::aes(
      x = .data$time,
      y = .data$survival,
      color = .data$strata,
      linetype = .data$Curve
    )
  ) +
    ggplot2::geom_step(
      data = plot_data[plot_data$Curve == "Observed KM", , drop = FALSE],
      linewidth = 0.9
    ) +
    ggplot2::geom_line(
      data = plot_data[plot_data$Curve != "Observed KM", , drop = FALSE],
      linewidth = 0.8
    ) +
    ggplot2::scale_y_continuous(labels = scales::percent_format(accuracy = 1)) +
    ggplot2::scale_x_continuous(breaks = breaks) +
    ggplot2::scale_linetype_manual(
      values = .plot_surv_fit_linetypes(unique(plot_data$Curve))
    ) +
    ggplot2::labs(
      title = title,
      x = xlab,
      y = ylab,
      color = legend_title,
      linetype = "Curve"
    ) +
    ggplot2::theme_minimal(base_size = base_size) +
    ggplot2::theme(
      legend.position = "bottom",
      plot.title = ggplot2::element_text(face = "bold")
    ) +
    ggplot2::coord_cartesian(xlim = xlim, ylim = c(0, 1))

  if (!is.null(palette)) {
    p <- p + ggplot2::scale_color_manual(
      values = stats::setNames(palette[seq_along(strata_levels)], strata_levels)
    )
  }

  attr(p, "km_fit") <- km_fit
  attr(p, "model_fits") <- fitted_models
  attr(p, "observed_data") <- observed
  attr(p, "fitted_data") <- fitted
  attr(p, "prediction_data") <- prediction_data
  p
}

#' @keywords internal
#' @noRd
.validate_plot_surv_fit_inputs <- function(data,
                                           time,
                                           event,
                                           by,
                                           adjust_for,
                                           distributions,
                                           break_time_by,
                                           xlim,
                                           base_size,
                                           n_points) {
  choices <- c("weibull", "exponential", "lognormal", "loglogistic")
  if (!is.character(distributions) || length(distributions) < 1L ||
      anyNA(distributions) || any(!distributions %in% choices)) {
    stop(
      "`distributions` must contain one or more of: ",
      paste(choices, collapse = ", "),
      call. = FALSE
    )
  }
  if (!is.numeric(n_points) || length(n_points) != 1L || is.na(n_points) ||
      n_points < 25 || n_points != floor(n_points)) {
    stop("`n_points` must be a whole number of at least 25.", call. = FALSE)
  }

  .validate_km_inputs(
    data = data,
    time = time,
    event = event,
    by = by,
    conf.int = FALSE,
    risk_table = FALSE,
    p_value = FALSE,
    p_value_position = NULL,
    censor = FALSE,
    break_time_by = break_time_by,
    xlim = xlim,
    ylim = NULL,
    base_size = base_size
  )

  if (!is.null(adjust_for)) {
    missing_adjust <- setdiff(adjust_for, names(data))
    if (length(missing_adjust)) {
      stop(
        "The following adjustment variables were not found in `data`: ",
        paste(missing_adjust, collapse = ", "),
        call. = FALSE
      )
    }
  }

  vars_needed <- unique(c(time, event, by, adjust_for))
  data_clean <- data[stats::complete.cases(data[, vars_needed, drop = FALSE]), , drop = FALSE]
  data_clean[[event]] <- .cox_event01(data_clean[[event]])
  data_clean
}

#' @keywords internal
#' @noRd
.plot_surv_fit_newdata <- function(data, by = NULL, adjust_for = NULL) {
  typical <- function(x) {
    if (is.numeric(x)) {
      return(stats::median(x, na.rm = TRUE))
    }
    if (is.factor(x)) {
      tab <- table(x)
      val <- names(tab)[which.max(tab)]
      return(factor(val, levels = levels(x), ordered = is.ordered(x)))
    }
    if (is.logical(x)) {
      tab <- table(x)
      return(as.logical(names(tab)[which.max(tab)]))
    }
    tab <- table(x)
    names(tab)[which.max(tab)]
  }

  if (is.null(by)) {
    out <- data.frame(.curve_id = "Overall", stringsAsFactors = FALSE)
  } else {
    levs <- levels(factor(data[[by]]))
    out <- data.frame(.curve_id = levs, stringsAsFactors = FALSE)
    out[[by]] <- factor(levs, levels = levels(factor(data[[by]])))
  }

  if (!is.null(adjust_for)) {
    for (var in adjust_for) {
      out[[var]] <- typical(data[[var]])
    }
  }

  out
}

#' @keywords internal
#' @noRd
.plot_surv_fit_fitted_curves <- function(models, newdata, by = NULL, n_points = 200) {
  p_grid <- seq(0.001, 0.999, length.out = n_points)

  pieces <- list()
  idx <- 1L
  for (dist in names(models)) {
    fit <- models[[dist]]
    pred <- stats::predict(fit, newdata = newdata, type = "quantile", p = p_grid)
    pred <- as.matrix(pred)
    if (nrow(pred) != nrow(newdata) && ncol(pred) == nrow(newdata)) {
      pred <- t(pred)
    }

    for (i in seq_len(nrow(newdata))) {
      times <- as.numeric(pred[i, ])
      ok <- is.finite(times) & times >= 0
      curve <- data.frame(
        time = c(0, times[ok]),
        survival = c(1, 1 - p_grid[ok]),
        strata = newdata$.curve_id[i],
        Curve = paste0(.plot_surv_fit_distribution_label(dist), " fit"),
        Distribution = dist,
        stringsAsFactors = FALSE
      )
      curve <- curve[order(curve$time, decreasing = FALSE), , drop = FALSE]
      pieces[[idx]] <- curve
      idx <- idx + 1L
    }
  }

  do.call(rbind, pieces)
}

#' @keywords internal
#' @noRd
.plot_surv_fit_distribution_label <- function(distribution) {
  stats::setNames(
    c("Weibull", "Exponential", "Lognormal", "Log-logistic"),
    c("weibull", "exponential", "lognormal", "loglogistic")
  )[[distribution]]
}

#' @keywords internal
#' @noRd
.plot_surv_fit_linetypes <- function(curves) {
  vals <- c("solid", "dashed", "dotdash", "twodash", "longdash")
  stats::setNames(rep(vals, length.out = length(curves)), curves)
}

lung_plot_surv_fit_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )
}

test_that("plot_surv_fit overlays grouped Kaplan-Meier and fitted curves", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")

  df <- lung_plot_surv_fit_data()

  res <- plot_surv_fit(
    data = df,
    time = time,
    event = status,
    by = trt,
    distributions = c(weibull, lognormal),
    break_time_by = 200
  )

  expect_s3_class(res, "ggplot")
  expect_s3_class(attr(res, "km_fit"), "survfit")
  expect_true(all(vapply(attr(res, "model_fits"), inherits, logical(1), what = "survreg")))

  observed <- attr(res, "observed_data")
  fitted <- attr(res, "fitted_data")

  expect_true(all(c("time", "survival", "strata", "Curve", "Distribution") %in% names(fitted)))
  expect_true(all(c("Standard treatment", "Test treatment") %in%
                    unique(as.character(observed$strata))))
  expect_equal(unique(fitted$Distribution), c("weibull", "lognormal"))
  expect_true(all(fitted$survival >= 0 & fitted$survival <= 1))
  expect_warning(ggplot2::ggplot_build(res), NA)
})

test_that("plot_surv_fit supports quoted names, adjustment, and single distribution", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")

  df <- lung_plot_surv_fit_data()
  time_var <- "time"
  event_var <- "status"

  res <- plot_surv_fit(
    data = df,
    time = time_var,
    event = event_var,
    by = "trt",
    adjust_for = c(age, karno),
    distributions = loglogistic,
    xlim = c(0, 800)
  )

  expect_s3_class(res, "ggplot")
  expect_named(attr(res, "model_fits"), "loglogistic")
  expect_true(all(c("age", "karno") %in% names(attr(res, "prediction_data"))))
  expect_equal(attr(res, "prediction_data")$age, rep(stats::median(df$age, na.rm = TRUE), 2))
  expect_warning(ggplot2::ggplot_build(res), NA)
})

test_that("plot_surv_fit normalizes common distribution spellings", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")

  df <- lung_plot_surv_fit_data()

  res <- plot_surv_fit(
    data = df,
    time = time,
    event = status,
    by = trt,
    distributions = c("Weibull", "log-logistic")
  )

  expect_named(attr(res, "model_fits"), c("weibull", "loglogistic"))
  expect_equal(unique(attr(res, "fitted_data")$Distribution), c("weibull", "loglogistic"))
})

test_that("plot_surv_fit works without grouping", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")

  df <- lung_plot_surv_fit_data()

  res <- plot_surv_fit(
    data = df,
    time = time,
    event = status,
    distributions = exponential
  )

  expect_s3_class(res, "ggplot")
  expect_equal(unique(as.character(attr(res, "observed_data")$strata)), "Overall")
  expect_equal(unique(attr(res, "fitted_data")$strata), "Overall")
})

test_that("plot_surv_fit validates inputs", {
  skip_if_not_installed("survival")

  df <- lung_plot_surv_fit_data()
  df$bad_event <- 0
  df$one_group <- "A"

  expect_error(
    plot_surv_fit(data = "not data", time = time, event = status),
    "`data` must be"
  )
  expect_error(
    plot_surv_fit(data = df, time = missing_time, event = status),
    "`time` must be"
  )
  expect_error(
    plot_surv_fit(data = df, time = time, event = missing_event),
    "`event` must be"
  )
  expect_error(
    plot_surv_fit(data = df, time = time, event = status, by = one_group),
    "`by` must contain at least two"
  )
  expect_error(
    plot_surv_fit(data = df, time = time, event = bad_event),
    "`event` must include at least one event"
  )
  expect_error(
    plot_surv_fit(data = df, time = time, event = status, adjust_for = missing_var),
    "adjustment variables were not found"
  )
  expect_error(
    plot_surv_fit(data = df, time = time, event = status, distributions = gaussian),
    "`distributions` must contain"
  )
  expect_error(
    plot_surv_fit(data = df, time = time, event = status, n_points = 10),
    "`n_points` must be"
  )
})

# =========================
# DESC EXTRACTOR (INTERNAL)
# =========================
.df_from_desc <- function(desc, drop_is_header = TRUE) {
  if (!is.list(desc) || is.null(desc$table_display) || is.null(desc$table_body)) {
    stop("`desc` must be a descriptive_table object with `table_display` and `table_body`.",
         call. = FALSE)
  }
  td <- desc$table_display
  tb <- desc$table_body

  hdr <- if ("is_header" %in% names(td)) as.logical(td$is_header) else rep(FALSE, nrow(td))

  expo <- attr(td, "row_exposure", exact = TRUE)
  if (!is.null(expo) && length(expo) == nrow(td)) {
    expo <- as.character(expo)
  } else {
    expo <- NULL
  }
  if (is.data.frame(tb) && all(c("var", "type") %in% names(tb))) {
    skeleton <- do.call(rbind, lapply(unique(tb$var), function(var) {
      x <- tb[tb$var == var, , drop = FALSE]
      levs <- x[x$type != "header", , drop = FALSE]
      if (!nrow(levs)) {
        return(data.frame(exposure = unique(x$var), stringsAsFactors = FALSE))
      }
      if (all(levs$type == "continuous")) {
        return(data.frame(exposure = unique(x$var), stringsAsFactors = FALSE))
      }
      data.frame(
        exposure = c(unique(x$var), rep(unique(x$var), nrow(levs))),
        stringsAsFactors = FALSE
      )
    }))
    if (nrow(skeleton) == nrow(td)) {
      expo <- as.character(skeleton$exposure)
    }
  }

  if (is.null(expo)) {
    expo <- as.character(td$Characteristic)
    expo[!hdr] <- NA_character_
    last <- NA_character_
    for (i in seq_along(expo)) {
      if (!is.na(expo[i]) && nzchar(expo[i])) last <- expo[i] else expo[i] <- last
    }
  }

  out <- td
  out$exposure <- expo

  out$Characteristic <- as.character(out$Characteristic)
  out$exposure       <- as.character(out$exposure)
  if (drop_is_header && "is_header" %in% names(out)) out$is_header <- NULL
  rownames(out) <- NULL
  out
}

.merge_forest_desc <- function(left, desc) {
  ddf <- if (is.list(desc) && !is.null(desc$table_display)) .df_from_desc(desc) else desc
  stopifnot(is.data.frame(ddf), all(c("Characteristic","exposure") %in% names(ddf)))

  left$..forest_desc_order <- seq_len(nrow(left))
  joined <- dplyr::left_join(left, ddf, by = c("exposure","Characteristic"))
  joined <- joined[order(joined$..forest_desc_order), , drop = FALSE]
  desc_cols <- setdiff(names(ddf), c("Characteristic", "exposure"))
  desc_cols <- desc_cols[desc_cols %in% names(joined)]
  if (!length(desc_cols)) {
    joined$..forest_desc_order <- NULL
    return(joined)
  }

  fallback <- ddf[!duplicated(ddf$exposure), c("exposure", desc_cols), drop = FALSE]
  fallback <- fallback[match(joined$exposure, fallback$exposure), , drop = FALSE]

  for (col in desc_cols) {
    needs_value <- is.na(joined[[col]]) | !nzchar(as.character(joined[[col]]))
    fallback_value <- fallback[[col]]
    has_fallback <- !is.na(fallback_value) & nzchar(as.character(fallback_value))
    continuous_or_header_row <- joined$Characteristic == joined$exposure
    idx <- needs_value & has_fallback & continuous_or_header_row
    joined[[col]][idx] <- fallback_value[idx]
  }

  joined$..forest_desc_order <- NULL
  joined
}

# =========================================
# CLASS-BASED INPUT PICKER (ORDER-AGNOSTIC)
# =========================================
.pick_inputs <- function(uni, multi = NULL, desc = NULL) {
  inputs <- Filter(Negate(is.null), list(uni = uni, multi = multi, desc = desc))

  is_uni   <- function(x) inherits(x, "gtregression") &&
    any(c("uni_reg", "gt_uni", "cox_reg", "gt_cox", "surv_reg", "gt_surv") %in% class(x)) &&
    !.is_adjusted_reg_output(x)
  is_multi <- function(x) inherits(x, "gtregression") &&
    (any(c("multi_reg", "gt_multi") %in% class(x)) ||
       (any(c("cox_reg", "gt_cox", "surv_reg", "gt_surv") %in% class(x)) &&
          .is_adjusted_reg_output(x)))
  is_desc  <- function(x) any(c("descriptive_table","gt_desc") %in% class(x)) ||
    (is.list(x) && !is.null(x$table_display) && !is.null(x$table_body)) ||
    (is.data.frame(x) && "Characteristic" %in% names(x))

  got_uni <- got_multi <- got_desc <- NULL

  for (nm in names(inputs)) {
    x <- inputs[[nm]]
    if (is_uni(x)) {
      if (!is.null(got_uni)) stop("Two univariate objects supplied.", call. = FALSE)
      got_uni <- x
      next
    }
    if (is_multi(x)) {
      if (!is.null(got_multi)) stop("Two multivariable objects supplied.", call. = FALSE)
      got_multi <- x
      next
    }
    if (is_desc(x)) {
      if (!is.null(got_desc)) stop("Two descriptive tables supplied.", call. = FALSE)
      got_desc <- x
      next
    }
    stop("Unknown input type for argument '", nm, "'.", call. = FALSE)
  }

  list(uni = got_uni, multi = got_multi, desc = got_desc)
}

.is_stratified_forest_input <- function(x) {
  inherits(x, "gtregression") &&
    !is.null(x$per_stratum) &&
    any(c(
      "stratified_uni_reg",
      "stratified_multi_reg",
      "stratified_cox_reg",
      "stratified_surv_reg"
    ) %in% class(x))
}

.forest_or <- function(a, b) {
  if (!is.null(a)) a else b
}

.forest_stratum_object <- function(parent, x) {
  if (inherits(x, "gtregression")) {
    return(x)
  }

  out <- x
  out$approach <- parent$approach
  out$variable_labels <- parent$variable_labels
  out$adjusted_mode <- .forest_or(parent$adjusted_mode, x$adjusted_mode)
  out$multivariable <- .forest_or(parent$multivariable, x$multivariable)

  if (inherits(parent, "stratified_multi_reg")) {
    class(out) <- c("gtregression", "multi_reg", "gt_multi", class(out))
  } else if (inherits(parent, "stratified_cox_reg")) {
    class(out) <- c("gtregression", "cox_reg", "gt_cox", class(out))
  } else if (inherits(parent, "stratified_surv_reg")) {
    class(out) <- c("gtregression", "surv_reg", "gt_surv", class(out))
  }

  out
}

.forest_df_from_stratified <- function(x, digits = 2) {
  strata <- x$per_stratum
  if (!length(strata)) {
    stop("`stratified` regression objects must contain at least one stratum.", call. = FALSE)
  }

  labels <- names(strata)
  if (is.null(labels) || any(!nzchar(labels))) {
    labels <- paste0("Stratum ", seq_along(strata))
  }

  stratifier <- .forest_or(x$by, "Stratum")
  parts <- lapply(strata, function(item) {
    stratum_object <- .forest_stratum_object(x, item)
    forest_df(stratum_object, digits = digits)
  })

  first <- parts[[1]]
  text_cols_first <- grep("\\(95% CI\\)$", names(first), value = TRUE)
  if (length(text_cols_first) != 1L) {
    stop(
      "Stratified forest plots expect one effect column per stratum. ",
      "Use a single stratified regression object created by the package stratified functions.",
      call. = FALSE
    )
  }

  helper_cols <- c("se_uni", "se_adj")
  anchor_cols_first <- names(first)[grepl("^ +$", names(first))]
  left_cols <- setdiff(names(first), c(text_cols_first, anchor_cols_first, helper_cols))
  out <- first[, left_cols, drop = FALSE]

  est_list <- lo_list <- hi_list <- vector("list", length(parts))
  anchor_names <- character(length(parts))
  text_names <- character(length(parts))

  for (i in seq_along(parts)) {
    part <- parts[[i]]
    text_cols <- grep("\\(95% CI\\)$", names(part), value = TRUE)
    if (length(text_cols) != 1L) {
      stop(
        "Stratified forest plots expect one effect column per stratum. ",
        "Use a single stratified regression object created by the package stratified functions.",
        call. = FALSE
      )
    }

    effect_label <- text_cols[[1]]
    text_name <- paste0(labels[[i]], "\n", effect_label)
    anchor_name <- paste(rep(" ", i), collapse = "")

    out[[text_name]] <- part[[effect_label]]
    out[[anchor_name]] <- ""

    est_list[[i]] <- attr(part, "est", exact = TRUE)
    lo_list[[i]] <- attr(part, "lo", exact = TRUE)
    hi_list[[i]] <- attr(part, "hi", exact = TRUE)
    anchor_names[[i]] <- anchor_name
    text_names[[i]] <- text_name
  }

  if ("se_uni" %in% names(first)) {
    out$se_uni <- first$se_uni
  }

  # Keep the old single/two-model attributes for backward compatibility, while
  # also exposing a general list for side-by-side stratified plots.
  attr(out, "est") <- est_list[[1]]
  attr(out, "lo") <- lo_list[[1]]
  attr(out, "hi") <- hi_list[[1]]
  if (length(est_list) >= 2L) {
    attr(out, "est2") <- est_list[[2]]
    attr(out, "lo2") <- lo_list[[2]]
    attr(out, "hi2") <- hi_list[[2]]
  }
  attr(out, "forest_estimates") <- list(
    est = est_list,
    lo = lo_list,
    hi = hi_list,
    anchors = anchor_names,
    text_cols = text_names
  )

  meta <- .forest_or(attr(out, "forest_meta", exact = TRUE), list())
  first_meta <- attr(first, "forest_meta", exact = TRUE)
  if (!is.null(first_meta$x_trans)) meta$x_trans <- first_meta$x_trans
  if (!is.null(first_meta$ref_line)) meta$ref_line <- first_meta$ref_line
  meta$stratified <- TRUE
  meta$stratifier <- stratifier
  meta$strata <- labels
  meta$side_by_side_strata <- TRUE
  meta$effect_sets <- list(anchors = anchor_names, text_cols = text_names)
  attr(out, "forest_meta") <- meta
  out
}

#' Build a compatible data frame for forest plots
#'
#' Creates the tabular input used by \code{forest_reg()} from \code{uni_reg()},
#' \code{multi_reg()}, and optionally \code{descriptive_table()} objects. This
#' function is useful when users want to inspect, edit, or reuse the exact data
#' that will be passed to the forest plot. Stratified regression objects are
#' also supported when supplied one at a time; the returned data frame keeps the
#' characteristic rows once and places strata side by side as separate effect
#' columns for \code{forest_reg()}.
#'
#' @param uni A \code{gtregression} object from \code{uni_reg()}. If
#'   \code{multi} is supplied without \code{uni}, the multivariable object is
#'   plotted as a single effect column.
#' @param multi Optional \code{gtregression} object from \code{multi_reg()}.
#' @param desc Optional descriptive table object from \code{descriptive_table()}.
#' @param digits Number of digits used when formatting confidence intervals.
#'
#' @return A data frame with display columns, formatted effect-size columns,
#'   standard-error columns, and plotting attributes used by \code{forest_reg()}
#'   to draw confidence intervals.
#'
#' @examples
#' birthwt_data <- data_birthwt |>
#'   dplyr::mutate(
#'     smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
#'     ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
#'     low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
#'   )
#'
#' uni_or <- uni_reg(
#'   birthwt_data,
#'   outcome = "low",
#'   exposures = c("age", "lwt", "smoke", "ht"),
#'   approach = "logit"
#' )
#' multi_or <- multi_reg(
#'   birthwt_data,
#'   outcome = "low",
#'   exposures = c("smoke", "ht"),
#'   adjust_for = c("age", "lwt"),
#'   approach = "logit"
#' )
#'
#' forest_data <- forest_df(uni_or, multi_or)
#' head(forest_data)
#' @export
forest_df <- function(uni, multi = NULL, desc = NULL, digits = 2) {
  if (!is.numeric(digits) || length(digits) != 1L || is.na(digits) || digits < 0) {
    stop("`digits` must be a single non-negative number.", call. = FALSE)
  }
  digits <- as.integer(digits)

  inputs_raw <- list(uni = uni, multi = multi, desc = desc)
  supplied <- Filter(Negate(is.null), inputs_raw)
  stratified_inputs <- Filter(.is_stratified_forest_input, supplied)
  if (length(stratified_inputs)) {
    if (length(stratified_inputs) > 1L || length(supplied) > 1L) {
      stop(
        "forest_df() supports one stratified regression object at a time. ",
        "Do not combine stratified objects with `multi`, `desc`, or another stratified object.",
        call. = FALSE
      )
    }
    return(.forest_df_from_stratified(stratified_inputs[[1]], digits = digits))
  }

  `%||%`   <- function(a,b) if (!is.null(a)) a else b
  approach <- function(o) .normalize_approach(o$approach %||% "logit")
  is_ratio <- function(a) .normalize_approach(a) %in% c("logit","firth","logbinomial","poisson","robpoisson","negbin",
                                   "cox", "survreg", "margstd_boot","margstd_delta")
  effname  <- function(a) {
    a <- .normalize_approach(a)
    if (a == "linear") return("Beta")
    if (a == "cox") return("HR")
    if (a == "survreg") return("Time Ratio")
    if (a %in% c("poisson","negbin")) return("IRR")
    if (a %in% c("logbinomial","robpoisson","margstd_boot","margstd_delta")) return("RR")
    "OR"
  }
  fmt_ci <- function(e,l,h,d){
    out <- rep("", length(e))
    ok <- is.finite(e) & is.finite(l) & is.finite(h)
    out[ok] <- sprintf(paste0("%.",d,"f (%.",d,"f-%." ,d,"f)"), e[ok], l[ok], h[ok])
    out
  }
  read_tb <- function(x){
    tb <- x$table_body
    need <- c("exposure","level","estimate","conf.low","conf.high","p.value","ref")
    miss <- setdiff(need, names(tb))
    if (length(miss)) stop("Missing columns in table_body: ", paste(miss, collapse=", "), call. = FALSE)
    transform(tb[ , need, drop=FALSE],
              ref       = as.logical(ref),
              estimate  = as.numeric(estimate),
              conf.low  = as.numeric(conf.low),
              conf.high = as.numeric(conf.high))
  }
  .se_from_ci <- function(est, lo, hi, ratio = TRUE) {
    se <- rep(NA_real_, length(est))
    ok <- is.finite(est) & is.finite(lo) & is.finite(hi)
    if (ratio) se[ok] <- (log(hi[ok]) - log(lo[ok])) / (2 * 1.96)
    else       se[ok] <- (hi[ok] - lo[ok]) / (2 * 1.96)
    se
  }

  picked <- .pick_inputs(uni, multi, desc)
  uni   <- picked$uni
  multi <- picked$multi
  desc  <- picked$desc

  multi_only <- is.null(uni) && !is.null(multi)
  if (is.null(uni) && !is.null(multi) && inherits(multi, "gtregression")) {
    uni   <- multi
    multi <- NULL
  }
  if (is.null(uni) && is.null(multi) && !is.null(desc)) return(.df_from_desc(desc))

  has_multi <- !is.null(multi)
  u    <- read_tb(uni)
  app1 <- approach(uni)
  eff1 <- if (isTRUE(multi_only)) paste("Adjusted", effname(app1)) else effname(app1)
  if (has_multi) {
    m    <- read_tb(multi)
    app2 <- approach(multi); eff2 <- effname(app2)
  }

  ref_line <- if (is_ratio(app1)) 1 else 0

  # Build layout from univariate
  u$is_cont <- (u$exposure == u$level)
  blocks <- list()

  for (ex in unique(u$exposure)) {
    df <- u[u$exposure == ex, , drop = FALSE]

    if (all(df$is_cont)) {
      df$..row_type <- "level"
      df$..label <- df$exposure
      blocks[[length(blocks) + 1L]] <- df
    } else {
      hdr <- df[1, , drop = FALSE]
      hdr$level <- ex
      hdr$estimate <- hdr$conf.low <- hdr$conf.high <- hdr$p.value <- NA_real_
      hdr$ref <- FALSE
      hdr$..row_type <- "header"
      hdr$..label <- ex
      df$..row_type <- "level"
      df$..label <- paste0("  ", df$level)
      blocks[[length(blocks) + 1L]] <- hdr
      blocks[[length(blocks) + 1L]] <- df
    }
  }
  lay <- dplyr::bind_rows(blocks)
  lay$..row_order <- seq_len(nrow(lay))

  # Reference rows lie on ref_line (text "Ref." added later)
  is_ref <- lay$ref & lay$..row_type == "level"
  lay$estimate [is_ref] <- ref_line
  lay$conf.low [is_ref] <- ref_line
  lay$conf.high[is_ref] <- ref_line

  # Align multi
  if (has_multi) {
    key_m <- paste(m$exposure, m$level, sep="||")
    key_l <- paste(lay$exposure, lay$level, sep="||")
    pos   <- match(key_l, key_m)
    est2 <- lo2 <- hi2 <- rep(NA_real_, nrow(lay))
    hit <- which(!is.na(pos))
    has_adj_row <- !is.na(pos)
    est2[hit] <- m$estimate[pos[hit]]
    lo2 [hit] <- m$conf.low[pos[hit]]
    hi2 [hit] <- m$conf.high[pos[hit]]
    adj_ref <- is_ref & has_adj_row
    est2[adj_ref] <- ref_line
    lo2 [adj_ref] <- ref_line
    hi2 [adj_ref] <- ref_line
  }

  # Left table + optional desc merge
  forest_blank <- paste(rep(" ", 20), collapse = " ")
  label_map <- uni$variable_labels
  if ((is.null(label_map) || !length(label_map)) && has_multi) {
    label_map <- multi$variable_labels
  }
  left <- data.frame(
    Characteristic   = lay$..label,
    exposure         = lay$exposure,
    ..row_order      = lay$..row_order,
    stringsAsFactors = FALSE
  )
  if (!is.null(desc)) {
    left <- .merge_forest_desc(left, desc)
  }
  left <- left[order(left$..row_order), , drop = FALSE]
  label_idx <- lay$..row_type == "header" | (lay$exposure == lay$level)
  left$Characteristic[label_idx] <- vapply(
    lay$exposure[label_idx],
    .label_var,
    character(1),
    label_map = label_map
  )
  is_header <- lay$..row_type == "header"
  # CI text cols + BLANK anchor headers
  if (has_multi) {
    left[[" "]] <- forest_blank   # uni anchor
    ci1 <- fmt_ci(lay$estimate, lay$conf.low, lay$conf.high, digits)
    ci1[is_header | is_ref] <- ""              # <-- make headers & refs blank
    ci1[is_ref] <- "Ref."
    left[[paste0(eff1, " (95% CI)")]] <- ci1

    left[["  "]] <- forest_blank  # adj anchor
    ci2 <- fmt_ci(est2, lo2, hi2, digits)
    ci2[is_header | is_ref] <- ""              # <-- same here
    ci2[is_ref & has_adj_row] <- "Ref."
    left[[paste0("Adjusted ", eff2, " (95% CI)")]] <- ci2
  } else {
    left[[" "]] <- forest_blank
    ci1 <- fmt_ci(lay$estimate, lay$conf.low, lay$conf.high, digits)
    ci1[is_header | is_ref] <- ""              # <-- and here
    ci1[is_ref] <- "Ref."
    left[[paste0(eff1, " (95% CI)")]] <- ci1
  }


  # Remove helper join key from visible cols
  left <- left[, setdiff(names(left), c("exposure", "..row_order")), drop = FALSE]

  # Standard errors (not for display)
  is_header <- lay$..row_type == "header"
  se_uni <- .se_from_ci(lay$estimate, lay$conf.low, lay$conf.high, ratio = is_ratio(app1))
  se_uni[is_header | is_ref] <- NA_real_
  left$se_uni <- se_uni

  if (has_multi) {
    se_adj <- .se_from_ci(est2, lo2, hi2, ratio = is_ratio(app2))
    se_adj[is_header | is_ref] <- NA_real_
    left$se_adj <- se_adj
  }

  display_cols <- setdiff(names(left), c("se_uni", "se_adj"))
  for (col in display_cols) {
    left[[col]] <- as.character(left[[col]])
    left[[col]][is.na(left[[col]])] <- ""
  }

  # Keep only minimal, plotting-relevant attributes (optional).
  # If you want *no* attributes at all, comment these three lines too.
  attr(left, "est") <- lay$estimate
  attr(left, "lo")  <- lay$conf.low
  attr(left, "hi")  <- lay$conf.high
  if (has_multi) {
    attr(left, "est2") <- est2
    attr(left, "lo2")  <- lo2
    attr(left, "hi2")  <- hi2
  }

  attr(left, "forest_meta") <- list(
    x_trans = if (is_ratio(app1)) "log" else "none",
    ref_line = ref_line
  )

  left
}

lung_surv_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )
}

test_that("surv_reg returns crude time-ratio tables", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()
  attr(df$trt, "label") <- "Treatment group"

  res <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, karno),
    distribution = weibull,
    format = gt
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "surv_reg")
  expect_s3_class(res, "gt_surv")
  expect_s3_class(res$table, "gt_tbl")
  expect_equal(res$source, "surv_reg")
  expect_equal(res$approach, "survreg")
  expect_equal(res$distribution, "weibull")
  expect_false(res$adjusted_mode)
  expect_null(res$adjust_for)
  expect_named(res$models, c("trt", "celltype", "karno"))
  expect_true(all(vapply(res$models, inherits, logical(1), what = "survreg")))
  expect_true("N" %in% names(res$table_display))
  expect_true("Time Ratio (95% CI)" %in% names(res$table_display))
  expect_true("Treatment group" %in% res$table_display$Characteristic)
  expect_true("Ref." %in% res$table_display[["Time Ratio (95% CI)"]])
  expect_null(res$model_stats)
  expect_equal(
    attr(res$table_display, "row_exposure")[res$table_display$is_header],
    c("trt", "celltype", "karno")
  )
})

test_that("surv_reg crude tables display survival complete-case N consistently", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()
  df$karno[1:3] <- NA

  res <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, karno),
    distribution = weibull
  )

  karno_header <- which(attr(res$table_display, "row_exposure") == "karno" &
                          res$table_display$is_header)[1]

  expect_equal(
    res$table_display$N[karno_header],
    sum(stats::complete.cases(df[, c("time", "status", "karno")]))
  )

  expect_s3_class(modify_table(res, remove_N = TRUE), "surv_reg")
  expect_false("N" %in% names(modify_table(res, remove_N = TRUE)$table_display))
})

test_that("surv_reg adjustment notes use display labels", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()
  attr(df$age, "label") <- "Patient age"
  attr(df$karno, "label") <- "Performance score"

  res <- surv_reg(
    data = df, time = time, event = status, exposures = trt,
    adjust_for = c(age, karno), distribution = weibull
  )

  expect_true(any(res$footnotes == "Adjusted for Patient age and Performance score"))
  expect_false(any(res$footnotes == "Adjusted for age and karno"))
  expect_false(any(grepl("^Event variable:", res$footnotes)))
})

test_that("surv_reg explains how to show reference rows when hidden", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()

  expect_message(
    res <- surv_reg(
      data = df,
      time = time,
      event = status,
      exposures = trt,
      distribution = weibull,
      show_ref = FALSE
    ),
    "To display Ref., use `show_ref = TRUE`.",
    fixed = TRUE
  )

  expect_true(any(res$table_body$ref))
  expect_equal(nrow(res$table_display), 1L)
  expect_false("Ref." %in% res$table_display[["Time Ratio (95% CI)"]])
  expect_false(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                         fixed = TRUE)))
})

test_that("surv_reg returns adjusted time-ratio tables and model stats", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()

  res <- surv_reg(
    data = df,
    time = "time",
    event = "status",
    exposures = c("trt", "celltype", "prior"),
    adjust_for = c("age", "karno"),
    distribution = lognormal,
    model_stats = TRUE
  )

  expect_s3_class(res, "ft_surv")
  expect_s3_class(res$table, "flextable")
  expect_true(res$adjusted_mode)
  expect_equal(res$adjust_for, c("age", "karno"))
  expect_equal(res$distribution, "lognormal")
  expect_true("Adjusted Time Ratio (95% CI)" %in% names(res$table_display))
  expect_s3_class(res$model_stats, "data.frame")
  expect_equal(res$model_stats$model, c("trt", "celltype", "prior"))
  expect_true(all(c("distribution", "AIC", "BIC", "logLik", "scale", "events", "n") %in%
                    names(res$model_stats)))
  expect_true(all(res$model_stats$distribution == "lognormal"))
  expect_true(all(is.finite(res$model_stats$AIC)))
  expect_true(all(res$model_stats$events > 0))
})

test_that("surv_reg supports a single multivariable parametric survival model", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()

  res <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior, age, karno),
    distribution = weibull,
    multivariable = TRUE,
    model_stats = TRUE
  )

  direct <- survival::survreg(
    survival::Surv(time, status) ~ trt + celltype + prior + age + karno,
    data = df,
    dist = "weibull",
    model = TRUE
  )

  expect_s3_class(res, "surv_reg")
  expect_false(res$adjusted_mode)
  expect_true(res$multivariable)
  expect_null(res$adjust_for)
  expect_named(res$models, "multivariable_model")
  expect_equal(stats::nobs(res$models$multivariable_model), stats::nobs(direct))
  expect_equal(stats::coef(res$models$multivariable_model), stats::coef(direct), tolerance = 1e-8)
  expect_equal(res$model_stats$model, "multivariable_model")
  expect_true("Adjusted Time Ratio (95% CI)" %in% names(res$table_display))
  expect_true(all(c("trt", "celltype", "prior", "age", "karno") %in% res$table_body$exposure))
  expect_equal(
    attr(res$table_display, "row_exposure")[res$table_display$is_header],
    c("trt", "celltype", "prior", "age", "karno")
  )
})

test_that("surv_reg supports interaction terms", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()
  attr(df$trt, "label") <- "Treatment"
  attr(df$prior, "label") <- "Prior therapy"

  adjusted <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = trt,
    adjust_for = c(age, karno),
    interaction = trt*prior,
    distribution = weibull
  )

  direct_adjusted <- survival::survreg(
    survival::Surv(time, status) ~ trt + age + karno + prior + trt:prior,
    data = df,
    dist = "weibull",
    model = TRUE
  )

  expect_equal(adjusted$interaction, "trt*prior")
  expect_true("Adjusted Time Ratio (95% CI)" %in% names(adjusted$table_display))
  expect_true(any(grepl(" x ", adjusted$table_body$level, fixed = TRUE)))
  expect_true(any(adjusted$table_body$level == "Treatment: Test treatment x Prior therapy: Yes"))
  expect_equal(stats::coef(adjusted$models$trt), stats::coef(direct_adjusted), tolerance = 1e-8)

  multivariable <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, age, karno),
    interaction = "trt*prior",
    distribution = lognormal,
    multivariable = TRUE
  )

  direct_multi <- survival::survreg(
    survival::Surv(time, status) ~ trt + age + karno + prior + trt:prior,
    data = df,
    dist = "lognormal",
    model = TRUE
  )

  expect_true(multivariable$multivariable)
  expect_equal(multivariable$interaction, "trt*prior")
  expect_true(any(grepl(":", names(stats::coef(multivariable$models$multivariable_model)), fixed = TRUE)))
  expect_equal(stats::coef(multivariable$models$multivariable_model), stats::coef(direct_multi), tolerance = 1e-8)

  expect_error(
    surv_reg(
      data = df,
      time = time,
      event = status,
      exposures = c(trt, prior),
      interaction = trt:prior
    ),
    "Use standard interaction syntax with"
  )
  expect_error(
    surv_reg(
      data = df,
      time = time,
      event = status,
      exposures = c(trt, prior),
      interaction = trt*prior
    ),
    "exposure-by-exposure mode"
  )
})

test_that("surv_reg accepts multivariate alias for multivariable mode", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()

  res <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, age, karno),
    multivariate = TRUE
  )

  expect_true(res$multivariable)
  expect_named(res$models, "multivariable_model")
  expect_true("Adjusted Time Ratio (95% CI)" %in% names(res$table_display))
})

test_that("surv_reg uses exposure-specific complete cases for crude models", {
  skip_if_not_installed("survival")

  df <- data.frame(
    time = c(1, 2, 3, 5, 9, 12, 20, 40),
    status = c(1, 1, 0, 1, 0, 1, 0, 1),
    age = c(40, 44, 50, 52, 60, 62, 70, 72),
    marker = c(1, 0, NA, 1, NA, 0, 1, 0)
  )

  res <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(age, marker),
    distribution = weibull,
    model_stats = TRUE
  )

  direct_age <- survival::survreg(survival::Surv(time, status) ~ age, data = df, dist = "weibull")
  direct_marker <- survival::survreg(survival::Surv(time, status) ~ marker, data = df, dist = "weibull")

  expect_equal(stats::nobs(res$models$age), stats::nobs(direct_age))
  expect_equal(stats::nobs(res$models$marker), stats::nobs(direct_marker))
  expect_gt(stats::nobs(res$models$age), stats::nobs(res$models$marker))
  expect_equal(res$model_stats$n, c(stats::nobs(direct_age), stats::nobs(direct_marker)))
  expect_equal(stats::coef(res$models$age), stats::coef(direct_age), tolerance = 1e-8)
  expect_equal(stats::coef(res$models$marker), stats::coef(direct_marker), tolerance = 1e-8)
})

test_that("surv_reg normalizes common distribution spellings", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()

  res <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, prior),
    distribution = "log-logistic"
  )

  expect_equal(res$distribution, "loglogistic")
  expect_true(all(vapply(res$models, function(x) identical(x$dist, "loglogistic"), logical(1))))
})

test_that("surv_reg validates survival and distribution inputs", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()
  df$bad_event <- 1

  expect_error(
    surv_reg(df, time = "time", event = "bad_event", exposures = "age"),
    "`event` must include at least one censored observation"
  )
  expect_error(
    surv_reg(df, time = "time", event = "status", exposures = "missing_var"),
    "exposure variables were not found"
  )
  expect_error(
    surv_reg(df, time = "time", event = "status", exposures = "age", distribution = gaussian),
    "Invalid distribution"
  )
  expect_error(
    surv_reg(df, time = "time", event = "status", exposures = "age", model_stats = NA),
    "`model_stats` must be TRUE or FALSE"
  )
  expect_error(
    surv_reg(df, time = "time", event = "status", exposures = "age", multivariable = NA),
    "`multivariable` must be TRUE or FALSE"
  )
  expect_error(
    surv_reg(df, time = "time", event = "status", exposures = "age", multivariate = NA),
    "`multivariate` must be TRUE or FALSE"
  )
  expect_error(
    surv_reg(
      df,
      time = "time",
      event = "status",
      exposures = "age",
      adjust_for = "karno",
      multivariable = TRUE
    ),
    "`adjust_for` is not used when `multivariable = TRUE`"
  )
})

test_that("surv_reg works with plotting and forest helpers", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("forestploter")

  df <- lung_surv_data()
  df$celltype <- factor(
    df$celltype,
    levels = c("squamous", "smallcell", "adeno", "large"),
    labels = c("Squamous", "Small cell", "Adenocarcinoma", "Large cell")
  )

  crude <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c("trt", "celltype", "karno", "age", "prior"),
    distribution = weibull
  )
  adjusted <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c("trt", "celltype", "prior"),
    adjust_for = c("age", "karno"),
    distribution = lognormal
  )

  expect_s3_class(plot_reg(crude, log_x = TRUE), "ggplot")
  expect_s3_class(plot_reg(adjusted, log_x = TRUE), "ggplot")
  expect_s3_class(plot_reg_combine(crude, adjusted, log_x = TRUE), "patchwork")

  df_crude <- forest_df(crude)
  expect_true("Time Ratio (95% CI)" %in% names(df_crude))
  expect_equal(attr(df_crude, "forest_meta")$x_trans, "log")
  expect_equal(attr(df_crude, "forest_meta")$ref_line, 1)

  df_both <- forest_df(crude, adjusted)
  expect_true("Time Ratio (95% CI)" %in% names(df_both))
  expect_true("Adjusted Time Ratio (95% CI)" %in% names(df_both))
  expect_s3_class(forest_reg(df_both), "gtregression_forest")
})

test_that("surv_reg supports stratified crude and adjusted survival tables", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()

  crude <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(celltype, karno),
    stratifier = trt,
    distribution = weibull,
    format = gt
  )

  expect_s3_class(crude, "gtregression")
  expect_s3_class(crude, "stratified_surv_reg")
  expect_s3_class(crude, "surv_reg")
  expect_equal(crude$source, "stratified_surv_reg")
  expect_true(crude$stratified)
  expect_equal(crude$show_sample, "events")
  expect_false(crude$adjusted_mode)
  expect_equal(crude$by, "trt")
  expect_equal(crude$distribution, "weibull")
  expect_equal(crude$levels, c("Standard treatment", "Test treatment"))
  expect_false("..N__Standard treatment" %in% names(crude$table_display))
  expect_true("..Events__Standard treatment" %in% names(crude$table_display))
  expect_true("..eff__Standard treatment" %in% names(crude$table_display))
  expect_named(crude$per_stratum, c("Standard treatment", "Test treatment"))

  adjusted <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(celltype, prior),
    adjust_for = c(age, karno),
    stratifier = trt,
    distribution = lognormal,
    model_stats = TRUE
  )

  expect_s3_class(adjusted, "stratified_surv_reg")
  expect_true(adjusted$adjusted_mode)
  expect_equal(adjusted$adjust_for, c("age", "karno"))
  expect_equal(adjusted$distribution, "lognormal")
  expect_false("..N__Standard treatment" %in% names(adjusted$table_display))
  expect_true("..Events__Standard treatment" %in% names(adjusted$table_display))
  expect_true("..eff__Standard treatment" %in% names(adjusted$table_display))
  expect_s3_class(adjusted$model_stats, "data.frame")
  expect_true(all(c("stratum", "model", "distribution", "AIC", "events", "n") %in%
                    names(adjusted$model_stats)))
})

test_that("surv_reg explains hidden reference rows in stratified tables", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()

  expect_message(
    res <- surv_reg(
      data = df,
      time = time,
      event = status,
      exposures = prior,
      stratifier = trt,
      distribution = weibull,
      show_ref = FALSE
    ),
    "To display Ref., use `show_ref = TRUE`.",
    fixed = TRUE
  )

  expect_true(any(vapply(res$per_stratum, function(x) any(x$table_body$ref), logical(1))))
  eff_cols <- startsWith(names(res$table_display), "..eff__")
  expect_false("Ref." %in% unlist(res$table_display[eff_cols]))
  expect_false(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                         fixed = TRUE)))
})

test_that("surv_reg controls displayed sample columns in stratified tables", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()

  with_n <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = celltype,
    stratifier = trt,
    show_sample = n
  )
  expect_true("..N__Standard treatment" %in% names(with_n$table_display))
  expect_false("..Events__Standard treatment" %in% names(with_n$table_display))

  with_both <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = celltype,
    stratifier = trt,
    show_sample = both
  )
  expect_true("..N__Standard treatment" %in% names(with_both$table_display))
  expect_true("..Events__Standard treatment" %in% names(with_both$table_display))

  with_none <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = celltype,
    stratifier = trt,
    show_sample = none
  )
  expect_false(any(startsWith(names(with_none$table_display), "..N__")))
  expect_false(any(startsWith(names(with_none$table_display), "..Events__")))
  expect_true("..eff__Standard treatment" %in% names(with_none$table_display))

  expect_error(
    surv_reg(df, time = time, event = status, exposures = celltype, stratifier = trt, show_sample = "bad"),
    "show_sample"
  )
})

test_that("surv_reg supports stratified multivariable survival tables", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()

  res <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(celltype, prior, age, karno),
    stratifier = trt,
    distribution = weibull,
    multivariable = TRUE
  )

  expect_s3_class(res, "stratified_surv_reg")
  expect_true(res$multivariable)
  expect_equal(res$show_sample, "events")
  expect_false(res$adjusted_mode)
  expect_named(res$models, c("Standard treatment", "Test treatment"))
  expect_named(res$models[["Standard treatment"]], "multivariable_model")
  expect_false("..N__Standard treatment" %in% names(res$table_display))
  expect_true("..Events__Standard treatment" %in% names(res$table_display))
  expect_true("..eff__Standard treatment" %in% names(res$table_display))
  expect_true(any(trimws(res$table_display$Characteristic) %in% c("Age", "age")))
})

test_that("surv_reg validates stratifier inputs and supports stratified forest data", {
  skip_if_not_installed("survival")

  df <- lung_surv_data()

  expect_error(
    surv_reg(df, time = time, event = status, exposures = trt, stratifier = trt),
    "`stratifier` cannot also be used"
  )
  expect_error(
    surv_reg(df, time = time, event = status, exposures = trt, adjust_for = age, stratifier = age),
    "`stratifier` cannot also be used"
  )
  expect_error(
    surv_reg(df, time = time, event = status, exposures = trt, stratifier = "status"),
    "`stratifier` cannot also be used"
  )
  expect_error(
    surv_reg(df, time = time, event = status, exposures = trt, interaction = trt*prior, stratifier = prior),
    "`stratifier` cannot also be used"
  )

  stratified <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(celltype, karno),
    stratifier = trt,
    distribution = weibull
  )

  strat_plot <- plot_reg(stratified)
  expect_s3_class(strat_plot, "ggplot")
  expect_true("stratum" %in% names(strat_plot$data))
  expect_error(plot_reg_combine(stratified, stratified), "does not support stratified")

  forest_data <- forest_df(stratified)
  expect_s3_class(forest_data, "data.frame")
  expect_true(all(paste0(
    c("Standard treatment", "Test treatment"),
    "\nTime Ratio (95% CI)"
  ) %in% names(forest_data)))
  expect_false(any(forest_data$Characteristic %in% c(
    "trt = Standard treatment",
    "trt = Test treatment"
  )))
  expect_equal(length(attr(forest_data, "est")), nrow(forest_data))
  expect_equal(attr(forest_data, "forest_meta")$stratifier, "trt")
  expect_true(attr(forest_data, "forest_meta")$side_by_side_strata)
  expect_equal(length(attr(forest_data, "forest_estimates")$est), 2L)
})
test_that("surv_reg displays every level of character predictors", {
  set.seed(413)
  d <- data.frame(
    time = stats::rexp(180),
    event = stats::rbinom(180, 1, 0.7),
    classification = rep(c("Associated", "Elementary", "Combined"), each = 60)
  )

  res <- surv_reg(d, time, event, classification, format = "gt")
  rows <- res$table_body[res$table_body$exposure == "classification", ]

  expect_equal(rows$level, c("Associated", "Combined", "Elementary"))
  expect_equal(sum(rows$ref), 1L)
  expect_equal(nrow(rows), 3L)
})

birthwt_forest_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
    )
}

test_that("forest_df builds univariate and adjusted forest data", {
  df <- birthwt_forest_data()

  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "lwt", "smoke", "ht"),
    approach = logit
  )
  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("smoke", "ht"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )

  out <- forest_df(uni = uni, multi = multi)
  meta <- attr(out, "forest_meta")

  expect_s3_class(out, "data.frame")
  expect_true(all(c("Characteristic", "OR (95% CI)", "Adjusted OR (95% CI)",
                    " ", "  ", "se_uni", "se_adj") %in% names(out)))
  expect_true(all(c("est", "lo", "hi", "est2", "lo2", "hi2") %in% names(attributes(out))))
  expect_equal(meta$x_trans, "log")
  expect_equal(meta$ref_line, 1)
  expect_true("Ref." %in% out[["OR (95% CI)"]])
  expect_true("Ref." %in% out[["Adjusted OR (95% CI)"]])
})

test_that("forest_df leaves adjusted reference rows blank for omitted variables", {
  df <- birthwt_forest_data()

  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("race", "smoke", "ht"),
    approach = logit
  )
  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = "smoke",
    adjust_for = c("age", "lwt"),
    approach = logit
  )

  out <- forest_df(uni = uni, multi = multi)
  adj_col <- "Adjusted OR (95% CI)"

  ht_header <- which(out$Characteristic == "ht")
  smoke_header <- which(out$Characteristic == "smoke")

  expect_equal(out[[adj_col]][ht_header + 1L], "")
  expect_equal(out[[adj_col]][ht_header + 2L], "")
  expect_equal(out[[adj_col]][smoke_header + 1L], "Ref.")
  expect_match(out[[adj_col]][smoke_header + 2L], "\\(")
  expect_true(all(is.na(attr(out, "est2")[c(ht_header + 1L, ht_header + 2L)])))
})

test_that("forest_df preserves regression table exposure order", {
  df <- birthwt_forest_data()

  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("smoke", "age", "race", "lwt", "ht"),
    approach = logit
  )

  out <- forest_df(uni)

  expect_equal(
    out$Characteristic,
    c(
      "smoke", "  No", "  Yes",
      "age",
      "race", "  White", "  Black", "  Other",
      "lwt",
      "ht", "  No", "  Yes"
    )
  )
})

test_that("forest_df preserves exposure order after descriptive and adjusted merges", {
  df <- birthwt_forest_data()
  exposures <- c("smoke", "age", "race", "lwt", "ht")

  desc <- descriptive_table(
    data = df,
    exposures = exposures,
    by = "low",
    percent = column
  )
  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = exposures,
    approach = logit
  )
  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("ht", "smoke"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )

  out <- forest_df(uni, multi, desc = desc)
  headers <- trimws(out$Characteristic[!startsWith(out$Characteristic, "  ")])

  expect_equal(headers, exposures)
})

test_that("forest_df handles linear metadata and descriptive table merge", {
  df <- birthwt_forest_data()

  desc <- descriptive_table(
    data = df,
    exposures = c("age", "lwt", "smoke"),
    by = "low",
    percent = column
  )
  uni <- uni_reg(
    df,
    outcome = "bwt",
    exposures = c("age", "lwt", "smoke"),
    approach = linear
  )

  out <- forest_df(uni = uni, desc = desc)
  meta <- attr(out, "forest_meta")

  expect_s3_class(out, "data.frame")
  expect_true("Beta (95% CI)" %in% names(out))
  expect_equal(meta$x_trans, "none")
  expect_equal(meta$ref_line, 0)
  expect_true(any(grepl("Normal BW", names(out), fixed = TRUE)))
  expect_false(is.na(out[out$Characteristic == "age", "Normal BW"]))
  expect_false(is.na(out[out$Characteristic == "lwt", "Normal BW"]))
})

test_that("forest_df pulls continuous descriptive values even when labels differ", {
  df <- birthwt_forest_data()

  desc <- descriptive_table(
    data = df,
    exposures = c("age", "lwt", "smoke"),
    by = "low",
    percent = column
  )
  desc$table_display$Characteristic[desc$table_display$Characteristic == "age"] <- "Maternal age"
  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "lwt", "smoke"),
    approach = logit
  )

  out <- forest_df(uni = uni, desc = desc)

  expect_false(is.na(out[out$Characteristic == "age", "Normal BW"]))
  expect_false(is.na(out[out$Characteristic == "age", "Low BW"]))
  expect_match(out[out$Characteristic == "age", "Normal BW"], "\\(")
})

test_that("forest_df preserves continuous descriptive summaries for multivariable tables", {
  df <- birthwt_forest_data()

  desc <- descriptive_table(
    data = df,
    exposures = c("age", "lwt", "smoke", "ht"),
    by = "low",
    percent = column
  )
  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("age", "lwt", "smoke", "ht"),
    approach = logit
  )

  out <- forest_df(multi, desc = desc)

  expect_true("Adjusted OR (95% CI)" %in% names(out))
  expect_false(is.na(out[out$Characteristic == "age", "Normal BW"]))
  expect_false(is.na(out[out$Characteristic == "lwt", "Low BW"]))
  expect_true(any(grepl("\\(", out[out$Characteristic == "age", "Normal BW"])))
})

test_that("forest_df handles user-facing positional descriptive workflows", {
  df <- birthwt_forest_data() |>
    dplyr::mutate(
      ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes")),
      ftv_cat = factor(
        dplyr::case_when(
          ftv == 0 ~ "None",
          ftv == 1 ~ "One",
          ftv >= 2 ~ "Two or more"
        ),
        levels = c("None", "One", "Two or more")
      )
    )
  exposures <- c("age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat", "ftv_cat")

  desc <- descriptive_table(
    data = df,
    exposures = exposures,
    by = "low",
    percent = column,
    show_overall = last
  )
  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = exposures,
    approach = logit
  )
  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("race", "smoke", "ht", "ui", "ptl_cat", "ftv_cat"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )
  multi_full <- multi_reg(
    df,
    outcome = "low",
    exposures = exposures,
    approach = logit
  )

  uni_desc <- forest_df(uni, desc)
  multi_desc <- forest_df(multi_full, desc = desc)
  both_desc <- forest_df(uni, multi, desc = desc)

  expect_false(is.na(uni_desc[uni_desc$Characteristic == "age", "Normal BW"]))
  expect_false(is.na(uni_desc[uni_desc$Characteristic == "lwt", "Low BW"]))
  expect_true("Adjusted OR (95% CI)" %in% names(multi_desc))
  expect_false("OR (95% CI)" %in% names(multi_desc))
  expect_false(is.na(multi_desc[multi_desc$Characteristic == "age", "Normal BW"]))
  expect_false(is.na(multi_desc[multi_desc$Characteristic == "lwt", "Low BW"]))
  expect_true(all(c("OR (95% CI)", "Adjusted OR (95% CI)") %in% names(both_desc)))
  expect_false(is.na(both_desc[both_desc$Characteristic == "age", "Overall"]))
  expect_equal(both_desc[both_desc$Characteristic == "age", "Adjusted OR (95% CI)"], "")
  expect_equal(both_desc[both_desc$Characteristic == "lwt", "Adjusted OR (95% CI)"], "")
})

test_that("forest_reg builds forest objects from data frames and regression objects", {
  skip_if_not_installed("forestploter")

  df <- birthwt_forest_data()

  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "lwt", "smoke", "ht"),
    approach = logit
  )
  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("smoke", "ht"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )

  forest_data <- forest_df(uni = uni, multi = multi)

  res_df <- forest_reg(df = forest_data, side = left, quiet = TRUE)
  res_obj <- forest_reg(uni = uni, multi = multi, side = right, quiet = TRUE)

  expect_s3_class(res_df, "gtregression_forest")
  expect_s3_class(res_obj, "gtregression_forest")
  expect_equal(res_df$meta$ref_line, 1)
  expect_equal(res_df$meta$x_trans, "log")
  expect_s3_class(res_df$data, "data.frame")
  expect_equal(res_df$meta$ci_col_width, c(20L, 20L))
  expect_identical(formals(print.gtregression_forest)$autofit, FALSE)
  expect_invisible(suppressWarnings(print(res_df)))
  expect_warning(
    expect_invisible(print(res_df, autofit = TRUE)),
    "`autofit = TRUE` is ignored"
  )
  expect_error(print(res_df, autofit = NA), "`autofit` must be")
})

test_that("forest_reg preserves forest_df effect headers in plotted data", {
  skip_if_not_installed("forestploter")

  df <- birthwt_forest_data()

  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "lwt", "race", "smoke"),
    approach = logit
  )
  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("race", "smoke"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )

  res <- forest_reg(uni = uni, multi = multi, quiet = TRUE)

  expect_true(all(c("OR (95% CI)", "Adjusted OR (95% CI)") %in%
                    names(res$data)))
  expect_true(all(c("se_uni", "se_adj") %in% names(res$input_data)))
  expect_false(any(grepl("Odds Ratio", names(res$data), fixed = TRUE)))
})

test_that("forest_reg preserves adjusted header for multi-only plotted data", {
  skip_if_not_installed("forestploter")

  df <- birthwt_forest_data()
  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("race", "smoke"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )

  res <- forest_reg(multi = multi, quiet = TRUE)

  expect_true("Adjusted OR (95% CI)" %in% names(res$data))
  expect_false("OR (95% CI)" %in% names(res$data))
  expect_true("se_uni" %in% names(res$input_data))
  expect_false(any(grepl("Odds Ratio", names(res$data), fixed = TRUE)))
})

test_that("forest_df recognises survival multivariable outputs as adjusted", {
  skip_if_not_installed("survival")

  df <- data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )

  cox_crude <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior)
  )
  cox_multi <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior, age, karno),
    multivariable = TRUE
  )
  surv_crude <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior)
  )
  surv_multi <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, celltype, prior, age, karno),
    multivariable = TRUE
  )

  cox_both <- forest_df(cox_crude, cox_multi)
  cox_only <- forest_df(cox_multi)
  surv_both <- forest_df(surv_crude, surv_multi)
  surv_only <- forest_df(surv_multi)

  expect_true(all(c("HR (95% CI)", "Adjusted HR (95% CI)") %in% names(cox_both)))
  expect_true("Adjusted HR (95% CI)" %in% names(cox_only))
  expect_false("HR (95% CI)" %in% names(cox_only))
  expect_true(all(c("Time Ratio (95% CI)", "Adjusted Time Ratio (95% CI)") %in% names(surv_both)))
  expect_true("Adjusted Time Ratio (95% CI)" %in% names(surv_only))
  expect_false("Time Ratio (95% CI)" %in% names(surv_only))
})

test_that("forest_df builds side-by-side data for stratified regression outputs", {
  df <- birthwt_forest_data()

  strat_uni <- stratified_uni_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "smoke", "ht"),
    stratifier = "race",
    approach = logit
  )
  strat_multi <- stratified_multi_reg(
    data = df,
    outcome = "low",
    exposures = c("smoke", "ht"),
    stratifier = "race",
    approach = logit
  )

  uni_out <- forest_df(strat_uni)
  multi_out <- forest_df(strat_multi)

  expect_s3_class(uni_out, "data.frame")
  expect_true(all(paste0(c("White", "Black", "Other"), "\nOR (95% CI)") %in% names(uni_out)))
  expect_equal(length(attr(uni_out, "est")), nrow(uni_out))
  expect_false(any(uni_out$Characteristic %in% c("race = White", "race = Black", "race = Other")))
  expect_equal(sum(uni_out$Characteristic == "age"), 1L)
  expect_equal(attr(uni_out, "forest_meta")$stratifier, "race")
  expect_true(attr(uni_out, "forest_meta")$side_by_side_strata)
  expect_equal(length(attr(uni_out, "forest_estimates")$est), 3L)

  uni_plot <- forest_reg(uni_out, quiet = TRUE)
  expect_s3_class(uni_plot, "gtregression_forest")
  expect_equal(uni_plot$meta$strata_rows, integer())
  expect_true(uni_plot$meta$style_strata)

  plain_plot <- forest_reg(uni_out, style_strata = FALSE, quiet = TRUE)
  expect_false(plain_plot$meta$style_strata)
  expect_equal(plain_plot$meta$strata_rows, uni_plot$meta$strata_rows)

  expect_s3_class(multi_out, "data.frame")
  expect_true(all(paste0(c("White", "Black", "Other"), "\nAdjusted OR (95% CI)") %in% names(multi_out)))
  expect_false("OR (95% CI)" %in% names(multi_out))
  expect_equal(length(attr(multi_out, "est")), nrow(multi_out))
  expect_equal(attr(multi_out, "forest_meta")$strata, c("White", "Black", "Other"))

  expect_error(
    forest_df(strat_uni, multi = strat_multi),
    "one stratified regression object"
  )
})

test_that("forest_df builds side-by-side data for stratified Cox and surv outputs", {
  skip_if_not_installed("survival")

  df <- data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )

  cox_strat <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(age, prior),
    stratifier = trt
  )
  cox_adjusted_strat <- cox_reg(
    data = df,
    time = time,
    event = status,
    exposures = prior,
    adjust_for = c(age, karno),
    stratifier = trt
  )
  surv_strat <- surv_reg(
    data = df,
    time = time,
    event = status,
    exposures = c(age, prior),
    stratifier = trt
  )

  cox_out <- forest_df(cox_strat)
  cox_adjusted_out <- forest_df(cox_adjusted_strat)
  surv_out <- forest_df(surv_strat)

  expect_true(all(paste0(c("Standard treatment", "Test treatment"), "\nHR (95% CI)") %in% names(cox_out)))
  expect_true(all(paste0(c("Standard treatment", "Test treatment"), "\nAdjusted HR (95% CI)") %in% names(cox_adjusted_out)))
  expect_true(all(paste0(c("Standard treatment", "Test treatment"), "\nTime Ratio (95% CI)") %in% names(surv_out)))
  expect_equal(length(attr(cox_out, "est")), nrow(cox_out))
  expect_equal(length(attr(cox_adjusted_out, "est")), nrow(cox_adjusted_out))
  expect_equal(length(attr(surv_out, "est")), nrow(surv_out))
  expect_equal(attr(cox_out, "forest_meta")$x_trans, "log")
  expect_equal(attr(surv_out, "forest_meta")$stratifier, "trt")
  expect_equal(length(attr(cox_out, "forest_estimates")$est), 2L)
  cox_plot <- forest_reg(cox_out, quiet = TRUE)
  expect_s3_class(cox_plot, "gtregression_forest")
  expect_equal(cox_plot$meta$strata_rows, integer())
})

test_that("forest_reg leaves default axis ticks to forestploter and respects overrides", {
  skip_if_not_installed("forestploter")

  df <- birthwt_forest_data()
  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "lwt", "race", "smoke", "ht", "ui"),
    approach = logit
  )
  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("race", "smoke", "ht", "ui"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )

  auto <- forest_reg(uni = uni, multi = multi, quiet = TRUE)
  custom <- forest_reg(
    uni = uni,
    multi = multi,
    ci_col_width = c(12, 16),
    ticks_at = list(c(1, 2, 4), c(1, 3, 9)),
    ticks_digits = 1,
    xlim = list(c(0.25, 8), c(0.25, 12)),
    quiet = TRUE
  )

  expect_null(auto$meta$ticks_at)
  expect_null(auto$meta$xlim)
  expect_equal(custom$meta$ticks_at, list(c(1, 2, 4), c(1, 3, 9)))
  expect_equal(custom$meta$ticks_digits, 1L)
  expect_equal(custom$meta$xlim, list(c(0.25, 8), c(0.25, 12)))
  expect_equal(custom$meta$ci_col_width, c(12L, 16L))
})

test_that("forest_reg validates missing inputs", {
  expect_error(
    forest_reg(df = NULL, uni = NULL, quiet = TRUE),
    "Provide `df = forest_df"
  )
  expect_error(forest_reg(df = mtcars), "`Characteristic` column")
  expect_error(forest_reg(df = "not data"), "`df` must be a data frame")
  expect_error(forest_reg(df = data.frame(Characteristic = "x"), quiet = NA), "`quiet` must be")
  expect_error(
    forest_reg(df = data.frame(Characteristic = "x"), effects = 1),
    "`effects` must be"
  )
  expect_error(
    forest_reg(df = data.frame(Characteristic = "x"), ci_col_width = 0),
    "`ci_col_width` must be"
  )
  expect_error(
    forest_reg(df = data.frame(Characteristic = "x"), xlim = c(1, 1)),
    "`xlim` must be"
  )
  expect_error(
    forest_reg(df = data.frame(Characteristic = "x"), style_strata = NA),
    "`style_strata` must be"
  )
  expect_error(
    forest_reg(df = data.frame(Characteristic = "x"), strata_fill = NA_character_),
    "`strata_fill` must be"
  )
})

test_that("forest_df accepts multi-only and desc-only inputs", {
  df <- birthwt_forest_data()

  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("smoke", "ht"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )
  desc <- descriptive_table(
    data = df,
    exposures = c("age", "smoke"),
    by = "low"
  )

  multi_only <- forest_df(uni = multi)
  desc_only <- forest_df(uni = desc)

  expect_s3_class(multi_only, "data.frame")
  expect_true("Adjusted OR (95% CI)" %in% names(multi_only))
  expect_false("OR (95% CI)" %in% names(multi_only))
  expect_equal(
    attr(multi_only, "est")[is.finite(attr(multi_only, "est")) &
                              attr(multi_only, "est") != 1][1],
    multi$table_body$estimate[is.finite(multi$table_body$estimate) &
                                !multi$table_body$ref][1]
  )
  expect_s3_class(desc_only, "data.frame")
  expect_true("exposure" %in% names(desc_only))
})

test_that("forest_reg handles multi-only and descriptive-only workflows clearly", {
  skip_if_not_installed("forestploter")

  df <- birthwt_forest_data()
  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("smoke", "ht"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )
  desc <- descriptive_table(
    data = df,
    exposures = c("age", "smoke"),
    by = "low"
  )

  df_multi <- forest_df(multi)
  df_desc <- forest_df(desc)

  expect_s3_class(forest_reg(df_multi), "gtregression_forest")
  expect_s3_class(forest_reg(multi = multi), "gtregression_forest")
  expect_error(forest_reg(df_desc), "descriptive-only")
  expect_error(forest_reg(df_desc, df_multi), "do not pass another data frame")
})

test_that("forest_df rejects duplicate and unknown inputs", {
  df <- birthwt_forest_data()
  uni <- uni_reg(df, outcome = "low", exposures = "smoke", approach = logit)
  multi <- multi_reg(df, outcome = "low", exposures = "smoke", approach = logit)

  expect_error(forest_df(uni = uni, digits = -1), "`digits` must be")
  expect_error(forest_df(uni = uni, digits = NA_real_), "`digits` must be")
  expect_error(forest_df(uni = uni, multi = uni), "Two univariate")
  expect_error(forest_df(uni = multi, multi = multi), "Two multivariable")
  expect_error(forest_df(uni = mtcars), "Unknown input type")
  expect_error(.df_from_desc(list()), "`desc` must be")
})

test_that("forest_reg applies default metadata when forest_meta is absent", {
  skip_if_not_installed("forestploter")

  df <- birthwt_forest_data()
  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "lwt", "smoke"),
    approach = logit
  )
  forest_data <- forest_df(uni)
  attr(forest_data, "forest_meta") <- NULL

  res <- forest_reg(df = forest_data, side = right, quiet = TRUE)

  expect_s3_class(res, "gtregression_forest")
  expect_equal(res$meta$x_trans, "none")
  expect_equal(res$meta$ref_line, 0)
})

# -------------------------------------------------------------------
# merge_tables internals
# -------------------------------------------------------------------

#' @keywords internal
#' @noRd
.theme_presets_merge <- list(
  minimal  = c("plain", "lines", "labels_bold"),
  clinical = c("plain", "labels_bold", "compact"),
  striped  = c("zebra", "labels_bold", "compact"),
  shaded   = c("header_shaded", "labels_bold", "lines"),
  jama     = c("plain", "lines", "labels_bold", "compact")
)

#' @keywords internal
#' @noRd
.resolve_theme_merge <- function(theme) {
  if (length(theme) == 1 && !is.na(theme) &&
      theme %in% names(.theme_presets_merge)) {
    return(.theme_presets_merge[[theme]])
  }
  unique(tolower(theme))
}

#' Build canonical row-id map for a table_display object
#'
#' Rules:
#' - every header row is retained
#' - header followed immediately by another header (or end of table) is treated
#'   as a continuous/main-effect row and gets id var||var
#' - categorical headers get id var||__header__
#' - non-header rows get id current_var||level
#'
#' @keywords internal
#' @noRd
.canonical_row_ids <- function(obj, df) {
  body <- obj[["table_body"]]

  if (is.data.frame(body) && all(c("var", "level", "type") %in% names(body))) {
    ids <- character()
    for (v in unique(body$var)) {
      rows <- body[body$var == v, , drop = FALSE]
      data_rows <- rows[rows$type != "header", , drop = FALSE]

      if (!nrow(data_rows)) {
        ids <- c(ids, paste0(v, "||", v))
      } else if (all(data_rows$type == "continuous")) {
        ids <- c(ids, paste0(v, "||", v))
      } else if (any(data_rows$type == "dichotomous")) {
        ids <- c(ids, paste0(v, "||", data_rows$level))
      } else {
        ids <- c(
          ids,
          paste0(v, "||__header__"),
          paste0(v, "||", data_rows$level)
        )
      }
    }
    if (length(ids) == nrow(df)) {
      return(ids)
    }
  }

  if (is.data.frame(body) && all(c("exposure", "level", "ref") %in% names(body))) {
    ids <- character()
    for (v in unique(body$exposure)) {
      rows <- body[body$exposure == v, , drop = FALSE]
      is_fact <- any(rows$ref %in% TRUE)

      if (!is_fact && nrow(rows) == 1L) {
        ids <- c(ids, paste0(v, "||", v))
      } else {
        levels <- rows$level
        if (!is_fact) {
          levels <- levels[levels != v]
        }
        ids <- c(
          ids,
          paste0(v, "||__header__"),
          paste0(v, "||", levels)
        )
      }
    }
    if (length(ids) == nrow(df)) {
      return(ids)
    }
  }

  NULL
}

.canonical_map <- function(obj) {
  df <- if (is.list(obj) && !is.null(obj[["table_display"]])) {
    obj[["table_display"]]
  } else {
    obj
  }
  stopifnot(all(c("Characteristic", "is_header") %in% names(df)))

  ids <- .canonical_row_ids(obj, df)

  if (is.null(ids)) {
    ids <- rep(NA_character_, nrow(df))
    current_var <- NA_character_

    for (i in seq_len(nrow(df))) {
      ch <- trimws(df$Characteristic[i])
      ih <- isTRUE(df$is_header[i])

      if (ih) {
        current_var <- ch

        # Continuous/main-effect rows live on the header line; categorical
        # headers are structural rows and should still be preserved.
        if (i == nrow(df) || isTRUE(df$is_header[i + 1])) {
          ids[i] <- paste0(current_var, "||", current_var)
        } else {
          ids[i] <- paste0(current_var, "||__header__")
        }
      } else {
        lvl <- trimws(df$Characteristic[i])
        lvl <- sub("\\s*\\((Ref\\.|ref)\\)$", "", lvl)
        ids[i] <- paste0(current_var, "||", lvl)
      }
    }
  }

  content_cols <- setdiff(names(df), c("Characteristic", "is_header"))
  if ("N" %in% content_cols) {
    content_cols <- c("N", setdiff(content_cols, "N"))
  }

  lookup <- new.env(parent = emptyenv())
  for (i in which(!is.na(ids))) {
    lookup[[ids[i]]] <- df[i, content_cols, drop = FALSE]
  }

  labels <- stats::setNames(as.character(df$Characteristic), ids)
  headers <- stats::setNames(as.logical(df$is_header), ids)

  list(
    row_ids_order = ids,
    content_cols = content_cols,
    lookup = lookup,
    labels = labels,
    headers = headers
  )
}

#' Align compact binary regression rows to descriptive level rows
#'
#' Regression tables that do not display reference rows may represent a binary
#' exposure as one compact row (var||var). When merged with a descriptive table
#' that has explicit level rows (var||No, var||Yes), the compact row should live
#' on the non-reference level rather than creating a separate variable row.
#'
#' @keywords internal
#' @noRd
.align_compact_binary_maps <- function(maps) {
  parse_ids <- function(ids) {
    parts <- strsplit(ids, "\\|\\|")
    data.frame(
      id = ids,
      var = vapply(parts, `[`, character(1), 1),
      level = vapply(parts, `[`, character(1), 2),
      stringsAsFactors = FALSE
    )
  }

  all_ids <- unique(unlist(lapply(maps, `[[`, "row_ids_order"), use.names = FALSE))
  all_ids <- all_ids[!is.na(all_ids)]
  id_df <- parse_ids(all_ids)

  positive_values <- c(
    "yes", "y", "true", "1", "present", "positive", "exposed",
    "case", "event"
  )

  choose_level <- function(var, current_label) {
    candidates <- id_df$level[
      id_df$var == var &
        !id_df$level %in% c(var, "__header__")
    ]
    candidates <- unique(candidates[nzchar(candidates)])
    if (!length(candidates)) {
      return(NA_character_)
    }

    current_label <- trimws(current_label)
    label_match <- candidates[tolower(candidates) == tolower(current_label)]
    if (length(label_match)) {
      return(label_match[1])
    }

    positive_match <- candidates[tolower(candidates) %in% positive_values]
    if (length(positive_match)) {
      return(positive_match[1])
    }

    if (length(candidates) == 2L) {
      return(candidates[2])
    }

    NA_character_
  }

  remapped <- character()

  for (m in seq_along(maps)) {
    ids <- maps[[m]]$row_ids_order
    if (!length(ids)) {
      next
    }

    parsed <- parse_ids(ids)
    compact <- which(parsed$var == parsed$level)

    for (i in compact) {
      from <- parsed$id[i]
      value <- maps[[m]]$lookup[[from]]
      if (is.null(value)) {
        next
      }

      label <- unname(maps[[m]]$labels[from])
      target_level <- choose_level(parsed$var[i], label)
      if (is.na(target_level)) {
        next
      }

      to <- paste0(parsed$var[i], "||", target_level)
      if (identical(from, to)) {
        next
      }

      maps[[m]]$lookup[[to]] <- value
      rm(list = from, envir = maps[[m]]$lookup)

      maps[[m]]$row_ids_order[maps[[m]]$row_ids_order == from] <- to
      names(maps[[m]]$labels)[names(maps[[m]]$labels) == from] <- to
      if (!is.null(maps[[m]]$headers)) {
        names(maps[[m]]$headers)[names(maps[[m]]$headers) == from] <- to
        maps[[m]]$headers[to] <- FALSE
      }
      remapped <- c(remapped, parsed$var[i])
    }
  }

  list(
    maps = maps,
    remapped = unique(remapped)
  )
}

#' Inspect binary row representations across tables before merging.
#'
#' - `compact` indicates the variable is represented by one row (for example
#'   `smoke` as a single row in regression output with reference rows hidden).
#' - `expanded` indicates the variable is represented with explicit levels,
#'   e.g. `smoke = No`, `smoke = Yes`.
#'
#' @keywords internal
#' @noRd
.binary_mode_profile <- function(tbl) {
  body <- tbl[["table_body"]]
  display <- tbl[["table_display"]]
  if (!is.data.frame(body) || !nrow(body)) {
    return(list(compact = character(), expanded = character()))
  }
  body <- as.data.frame(body, stringsAsFactors = FALSE)
  if (is.data.frame(display)) {
    display <- as.data.frame(display, stringsAsFactors = FALSE)
  }

  compact <- character()
  expanded <- character()

  if (all(c("var", "type", "level") %in% names(body))) {
    for (v in unique(body$var)) {
      rows <- body[which(body$var == v), , drop = FALSE]
      data_rows <- rows[which(rows$type != "header"), , drop = FALSE]

      if (any(data_rows$type %in% "dichotomous")) {
        compact <- c(compact, v)
        next
      }

      level_rows <- data_rows[
        which(data_rows$type %in% c(
          "categorical", "dichotomous", "categorical_missing"
        )),
        ,
        drop = FALSE
      ]
      if (nrow(level_rows) == 0L) {
        next
      }

      level_vals <- trimws(as.character(level_rows$level))
      level_vals <- level_vals[nzchar(level_vals)]
      if (length(level_vals) %in% c(2L, 3L)) {
        # Two levels for all-level binary, or two levels + (Missing).
        expanded <- c(expanded, v)
      }
    }
    return(list(compact = unique(compact), expanded = unique(expanded)))
  }

  if (all(c("exposure", "level", "ref") %in% names(body))) {
    for (v in unique(body$exposure)) {
      rows <- body[which(body$exposure == v), , drop = FALSE]
      is_factor <- any(rows$ref %in% TRUE)

      if (is_factor && is.data.frame(display) &&
          all(c("Characteristic", "is_header") %in% names(display))) {
        header <- which(
          display$is_header %in% TRUE &
            trimws(as.character(display$Characteristic)) == v
        )
        if (length(header) == 1L) {
          next_header <- which(
            display$is_header %in% TRUE & seq_len(nrow(display)) > header
          )
          last_row <- if (length(next_header)) {
            next_header[1L] - 1L
          } else {
            nrow(display)
          }
          visible_levels <- if (header < last_row) {
            sum(!display$is_header[(header + 1L):last_row])
          } else {
            0L
          }

          # `show_ref = FALSE` represents a binary predictor on its header row.
          if (visible_levels == 0L && nrow(rows) == 2L) {
            compact <- c(compact, v)
          } else {
            expanded <- c(expanded, v)
          }
          next
        }
      }

      if (nrow(rows) == 1L) {
        # Regression compact binary row usually has level different from variable
        # name because reference rows are removed.
        lev <- as.character(rows$level[[1]])
        if (!identical(trimws(lev), trimws(v))) {
          compact <- c(compact, v)
        }
      } else if (is_factor && any(!rows$ref %in% TRUE)) {
        expanded <- c(expanded, v)
      }
    }
  }

  list(compact = unique(compact), expanded = unique(expanded))
}

#' Warn when binary rows are mixed as compact versus expanded across inputs.
#'
#' @keywords internal
#' @noRd
.binary_merge_mismatch <- function(tbls) {
  profiles <- lapply(tbls, .binary_mode_profile)

  compact_vars <- list()
  expanded_vars <- list()

  for (i in seq_along(profiles)) {
    profile <- profiles[[i]]

    for (v in profile$compact) {
      previous <- compact_vars[[v]]
      if (is.null(previous)) {
        previous <- integer()
      }
      compact_vars[[v]] <- unique(c(previous, i))
    }

    for (v in profile$expanded) {
      previous <- expanded_vars[[v]]
      if (is.null(previous)) {
        previous <- integer()
      }
      expanded_vars[[v]] <- unique(c(previous, i))
    }
  }

  vars_compact <- names(compact_vars)
  vars_expanded <- names(expanded_vars)
  mismatch <- intersect(vars_compact, vars_expanded)

  unique(mismatch)
}

#' Warn when binary rows are mixed as compact versus expanded across inputs.
#'
#' @keywords internal
#' @noRd
.warn_binary_merge_mode <- function(tbls) {
  mismatch <- .binary_merge_mismatch(tbls)

  if (!length(mismatch)) return(invisible(NULL))

  warning(
    "merge_tables(): binary display mode mismatch for variable(s): ",
    paste(mismatch, collapse = ", "),
    ". When merging descriptive, univariable, and multivariable tables, use ",
    "`show_dichotomous = \"all_levels\"` in descriptive_table() and ",
    "`show_ref = TRUE` in every regression table to prevent extra binary rows. ",
    "Alternatively, use compact display consistently: `show_dichotomous = ",
    "\"single_row\"` and `show_ref = FALSE`.",
    call. = FALSE
  )
}

#' Build merged skeleton using the union of canonical row ids
#' across all input tables
#' @keywords internal
#' @noRd
.build_merged_skeleton <- function(maps) {
  all_ids <- unique(unlist(
    lapply(maps, `[[`, "row_ids_order"),
    use.names = FALSE
  ))
  all_ids <- all_ids[!is.na(all_ids)]

  rows <- lapply(all_ids, function(id) {
    parts <- strsplit(id, "\\|\\|")[[1]]
    var <- parts[1]
    lvl <- parts[2]

    is_source_header <- any(vapply(
      maps,
      function(map) {
        if (is.null(map$headers)) {
          return(FALSE)
        }
        isTRUE(unname(map$headers[id]))
      },
      logical(1)
    ))
    is_header <- identical(var, lvl) || identical(lvl, "__header__") ||
      is_source_header
    label <- NA_character_
    for (map in maps) {
      hit <- unname(map$labels[id])
      if (!is.na(hit) && nzchar(hit)) {
        label <- hit
        break
      }
    }
    char <- if (!is.na(label)) label else if (is_header) var else paste0("  ", lvl)

    data.frame(
      Characteristic = char,
      is_header = is_header,
      row_id = id,
      stringsAsFactors = FALSE
    )
  })

  do.call(rbind, rows)
}

#' Collect footnotes from gtregression objects
#' @keywords internal
#' @noRd
.collect_footnotes <- function(obj) {
  # Prefer the notes created by the source function. A merge must preserve the
  # source table's reporting decisions rather than recreate them.
  stored_footnotes <- obj[["footnotes"]]
  if (!is.null(stored_footnotes)) {
    stored_footnotes <- as.character(stored_footnotes)
    return(unique(stored_footnotes[nzchar(stored_footnotes)]))
  }

  # Compatibility fallback for objects created by earlier gtregression
  # versions that did not retain their rendered notes.
  out <- character()

  approach <- obj[["approach"]]
  if (!is.null(approach) && exists(".abbrev_note", mode = "function")) {
    out <- c(out, .abbrev_note(approach))
  }

  adjust_for <- obj[["adjust_for"]]
  if (!is.null(adjust_for) && length(adjust_for) &&
      exists(".adjustment_note", mode = "function")) {
    out <- c(out, .adjustment_note(adjust_for))
  }

  models <- obj[["models"]]
  is_single_multi <- inherits(obj, "multi_reg") &&
    !is.null(models) &&
    length(models) == 1L &&
    !is.null(models[[1]])

  if (is_single_multi) {
    n_used <- tryCatch(
      stats::nobs(models[[1]]),
      error = function(e) NA_integer_
    )
    if (!is.na(n_used)) {
      out <- c(
        out,
        paste0(
          "N = ", n_used,
          " complete observations included in the model."
        )
      )
    }
  }

  unique(out[nzchar(out)])
}

# -------------------------------------------------------------------
# merge_tables main
# -------------------------------------------------------------------

#' Merge gtregression tables and preserve structure and notes
#'
#' @param ... Two or more \code{gtregression} objects containing
#'   \code{$table_display}.
#' @param spanners Character vector of spanner labels, one per table.
#'   If \code{NULL}, defaults to \code{"Table 1"}, \code{"Table 2"}, etc.
#' @param theme Merge theme preset or vector of primitives.
#' @param format Output table format. One of \code{"flextable"} (default) or
#'   \code{"gt"}. The merged display is rebuilt in this format, independently
#'   of the formats used by the input tables.
#' @details
#' Binary variables should use the same row display across all input tables.
#' For a descriptive, univariable, and multivariable merge, the clearest
#' publication layout is usually \code{show_dichotomous = "all_levels"} in
#' \code{descriptive_table()} and \code{show_ref = TRUE} in each regression
#' function. Mixing these settings can create additional binary rows; in that
#' situation \code{merge_tables()} issues a warning with the compatible
#' settings.
#'
#' Footnotes created by each input table are retained unchanged. Exact duplicate
#' notes are shown once in the merged table.
#'
#' @return A merged table object of class
#'   \code{c("gtregression", "merged_table", ...)}.
#'
#' @examples
#' birthwt_data <- data_birthwt |>
#'   dplyr::mutate(
#'     race = factor(race, levels = c(1, 2, 3),
#'                   labels = c("White", "Black", "Other")),
#'     smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
#'     ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
#'     low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
#'   )
#'
#' uni_tbl <- uni_reg(
#'   data = birthwt_data,
#'   outcome = "low",
#'   exposures = c("age", "lwt", "smoke", "ht"),
#'   approach = "logit"
#' )
#'
#' multi_tbl <- multi_reg(
#'   data = birthwt_data,
#'   outcome = "low",
#'   exposures = c("smoke", "ht"),
#'   adjust_for = c("age", "lwt"),
#'   approach = "logit"
#' )
#'
#' merge_tables(
#'   uni_tbl,
#'   multi_tbl,
#'   spanners = c("Univariable", "Adjusted")
#' )
#' @export
merge_tables <- function(...,
                         spanners = NULL,
                         theme = "minimal",
                         format = c("flextable", "gt")) {
  theme <- .choice_arg(substitute(theme), env = parent.frame())
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt")
  )
  format <- match.arg(format, c("flextable", "gt"))

  tbls <- list(...)

  if (length(tbls) < 2L) {
    stop("Provide at least two tables to merge.", call. = FALSE)
  }

  ok <- vapply(
    tbls,
    function(x) inherits(x, "gtregression") && !is.null(x[["table_display"]]),
    logical(1)
  )
  if (!all(ok)) {
    stop(
      "All inputs must be outputs of this package (contain $table_display).",
      call. = FALSE
    )
  }

  engines <- vapply(
    tbls,
    function(x) {
      fmt <- x[["format"]]
      tbl <- x[["table"]]

      if (!is.null(fmt)) {
        return(tolower(fmt))
      }
      if (inherits(tbl, "gt_tbl")) {
        return("gt")
      }
      if (inherits(tbl, "flextable")) {
        return("flextable")
      }
      NA_character_
    },
    character(1)
  )

  if (anyNA(engines) || length(unique(engines)) != 1L) {
    stop(
      "All inputs must use the same engine ('gt' or 'flextable').",
      call. = FALSE
    )
  }

  engine <- format
  theme <- .resolve_theme_merge(theme)

  if (is.null(spanners)) {
    spanners <- paste("Table", seq_along(tbls))
  }
  if (length(spanners) != length(tbls)) {
    stop("Length of `spanners` must equal number of tables.", call. = FALSE)
  }

  .warn_binary_merge_mode(tbls)
  binary_mismatch <- .binary_merge_mismatch(tbls)

  maps <- lapply(tbls, .canonical_map)
  aligned <- .align_compact_binary_maps(maps)
  maps <- aligned$maps
  aligned_mismatch <- intersect(aligned$remapped, binary_mismatch)
  if (length(aligned_mismatch)) {
    message(
      "merge_tables(): compact binary regression rows were aligned to ",
      "descriptive level rows for ",
      paste(aligned_mismatch, collapse = ", "),
      ". To display Ref., use `show_ref = TRUE` in the regression ",
      "function before merging."
    )
  }
  skeleton <- .build_merged_skeleton(maps)
  base_ids <- skeleton$row_id

  merged_cols <- list()
  col_labels <- list()

  for (i in seq_along(tbls)) {
    mi <- maps[[i]]

    blank_row <- as.data.frame(
      as.list(rep("", length(mi$content_cols))),
      stringsAsFactors = FALSE
    )
    names(blank_row) <- mi$content_cols

    aligned <- do.call(
      rbind,
      lapply(seq_along(base_ids), function(r) {
        rid <- base_ids[r]
        got <- mi$lookup[[rid]]
        if (is.null(got)) {
          blank_row
        } else {
          got[, mi$content_cols, drop = FALSE]
        }
      })
    )

    safe_names <- make.unique(
      paste0(make.names(mi$content_cols, unique = FALSE), "_p", i)
    )
    names(aligned) <- safe_names
    merged_cols[[i]] <- aligned
    col_labels[[i]] <- setNames(mi$content_cols, nm = safe_names)
  }

  merged_display <- cbind(
    skeleton[, c("Characteristic", "is_header"), drop = FALSE],
    do.call(cbind, merged_cols)
  )

  footnotes <- unique(unlist(
    lapply(tbls, .collect_footnotes),
    use.names = FALSE
  ))
  footnotes <- footnotes[nzchar(footnotes)]

  # ---------------- gt rendering ----------------

  apply_theme_gt <- function(tb, df, theme) {
    tb <- gt::tab_style(
      tb,
      gt::cell_text(weight = "bold"),
      locations = gt::cells_column_labels()
    )

    tb <- gt::tab_style(
      tb,
      gt::cell_text(weight = "bold"),
      gt::cells_body(rows = df$is_header %in% TRUE, columns = "Characteristic")
    )

    tb <- gt::tab_style(
      tb,
      gt::cell_text(indent = gt::px(12)),
      gt::cells_body(rows = df$is_header %in% FALSE, columns = "Characteristic")
    )

    if ("header_shaded" %in% theme) {
      tb <- gt::tab_options(
        tb,
        column_labels.background.color = "#f6f8fa"
      )
    }

    if ("zebra" %in% theme) {
      tb <- gt::opt_row_striping(tb)
    }

    if ("lines" %in% theme) {
      tb <- gt::tab_style(
        tb,
        gt::cell_borders(
          sides = "top",
          color = "#DADADA",
          weight = gt::px(1)
        ),
        gt::cells_body(rows = TRUE)
      )

      tb <- gt::tab_options(
        tb,
        table.border.top.style = "solid",
        table.border.top.color = "#DADADA",
        table.border.bottom.style = "solid",
        table.border.bottom.color = "#DADADA"
      )
    }

    if ("compact" %in% theme) {
      tb <- gt::tab_options(tb, data_row.padding = gt::px(2))
    }

    tb
  }

  build_gt_merge <- function(df, spanners, col_labels, theme, footnotes) {
    stopifnot(requireNamespace("gt", quietly = TRUE))

    content_cols <- unlist(lapply(col_labels, names), use.names = FALSE)
    out <- df[, c("Characteristic", content_cols), drop = FALSE]

    labels_list <- c(
      list(Characteristic = "Characteristic"),
      setNames(
        as.list(unlist(col_labels, use.names = FALSE)),
        nm = unlist(lapply(col_labels, names), use.names = FALSE)
      )
    )

    tb <- gt::gt(out) |>
      gt::cols_label(.list = labels_list) |>
      gt::cols_align("left", columns = "Characteristic") |>
      gt::cols_align(
        "center",
        columns = setdiff(names(out), "Characteristic")
      ) |>
      gt::sub_missing(
        columns = setdiff(names(out), "Characteristic"),
        missing_text = ""
      ) |>
      gt::tab_options(
        table.font.names = "system-ui",
        data_row.padding = gt::px(4),
        heading.background.color = "white",
        table.background.color = "white"
      )

    start <- 2L
    for (i in seq_along(spanners)) {
      k <- length(col_labels[[i]])
      if (k > 0) {
        tb <- gt::tab_spanner(
          tb,
          label = spanners[i],
          columns = names(out)[start:(start + k - 1L)]
        )
        start <- start + k
      }
    }

    tb <- gt::tab_style(
      tb,
      gt::cell_text(weight = "bold"),
      locations = gt::cells_column_spanners()
    )

    tb <- apply_theme_gt(tb, df, theme)

    if (length(footnotes)) {
      for (ln in footnotes) {
        tb <- gt::tab_source_note(tb, source_note = ln)
      }
      tb <- gt::tab_options(tb, source_notes.padding = gt::px(2))
    }

    tb
  }

  # ---------------- flextable rendering ----------------

  build_flex_merge <- function(df, spanners, col_labels, theme, footnotes) {
    stopifnot(requireNamespace("flextable", quietly = TRUE))

    content_cols <- unlist(lapply(col_labels, names), use.names = FALSE)
    out <- df[, c("Characteristic", content_cols), drop = FALSE]
    suppressWarnings(out[is.na(out)] <- "")

    keys <- colnames(out)
    labs <- c("Characteristic", unlist(col_labels, use.names = FALSE))

    ft <- flextable::flextable(out)
    ft <- flextable::set_header_labels(ft, values = stats::setNames(labs, keys))
    ft <- flextable::align(ft, j = "Characteristic", align = "left", part = "all")
    ft <- flextable::align(
      ft,
      j = setdiff(keys, "Characteristic"),
      align = "center",
      part = "all"
    )
    ft <- flextable::bold(ft, part = "header", bold = TRUE)

    widths <- c(1, vapply(col_labels, length, integer(1)))
    ft <- flextable::add_header_row(
      ft,
      values = c("", spanners),
      colwidths = widths
    )
    ft <- flextable::bold(ft, part = "header", bold = TRUE)

    hdr_idx <- which(df$is_header %in% TRUE)
    lvl_idx <- which(df$is_header %in% FALSE)

    if (length(hdr_idx)) {
      ft <- flextable::bold(
        ft,
        i = hdr_idx,
        j = "Characteristic",
        bold = TRUE,
        part = "body"
      )
    }

    if (length(lvl_idx)) {
      ft <- flextable::padding(
        ft,
        i = lvl_idx,
        j = "Characteristic",
        padding.left = 14,
        part = "body"
      )
    }

    if ("header_shaded" %in% theme) {
      ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
    }

    if ("zebra" %in% theme && length(lvl_idx)) {
      ft <- flextable::bg(
        ft,
        i = lvl_idx[seq(1, length(lvl_idx), by = 2)],
        bg = "#f6f8fa",
        part = "body"
      )
    }

    if ("lines" %in% theme) {
      n_body <- nrow(out)
      if (!is.null(n_body) && n_body > 0) {
        ft <- flextable::hline(ft, i = 1, part = "body")
        ft <- flextable::hline(ft, i = n_body, part = "body")
      }
    }

    if ("compact" %in% theme) {
      ft <- flextable::padding(ft, padding = 2, part = "body")
    }

    ft <- flextable::autofit(ft)

    if (length(footnotes)) {
      ft <- flextable::add_footer_lines(ft, values = footnotes)
      ft <- flextable::align(ft, part = "footer", align = "left")
      ft <- flextable::padding(
        ft,
        part = "footer",
        padding.top = 0,
        padding.bottom = 0
      )
      ft <- flextable::line_spacing(ft, part = "footer", space = 0.9)
    }

    ft
  }

  merged_tbl <- if (engine == "gt") {
    build_gt_merge(merged_display, spanners, col_labels, theme, footnotes)
  } else {
    build_flex_merge(merged_display, spanners, col_labels, theme, footnotes)
  }

  engine_class <- if (engine == "gt") "gt_merge" else "ft_merge"

  part_sources <- vapply(
    tbls,
    function(x) {
      src <- x[["source"]]
      if (!is.null(src)) src else NA_character_
    },
    character(1)
  )

  part_formats <- vapply(
    tbls,
    function(x) {
      fmt <- x[["format"]]
      if (!is.null(fmt)) fmt else NA_character_
    },
    character(1)
  )

  res <- list(
    table = merged_tbl,
    table_display = merged_display,
    column_labels = col_labels,
    spanners = spanners,
    engine = engine,
    format = format,
    footnotes = footnotes,
    part_sources = part_sources,
    part_formats = part_formats,
    source = "merge_tables"
  )

  class(res) <- c("gtregression", "merged_table", engine_class, "list")
  res
}

# -------------------------------------------------------------------
# print method
# -------------------------------------------------------------------

#' @export
print.merged_table <- function(x, ...) {
  tbl <- x[["table"]]
  if (inherits(tbl, "gt_tbl") || inherits(tbl, "flextable")) {
    base::print(tbl)
  } else {
    utils::str(x)
  }
  invisible(x)
}

#' Log-rank test for Kaplan-Meier survival curves
#'
#' Compare survival curves between groups using the log-rank test.
#'
#' @param data A \code{data.frame} containing survival time, event status, and
#'   grouping variable.
#' @param time Survival follow-up time. Quoted and bare names are accepted.
#' @param event Event indicator. Quoted and bare names are accepted. Numeric
#'   \code{0/1}, numeric \code{1/2}, logical, character, and factor variables
#'   are accepted. For two-level character or factor variables, the second level
#'   is treated as the event.
#' @param by Grouping variable used to compare survival curves. Quoted and bare
#'   names are accepted.
#' @param digits Number of digits for the chi-square statistic and expected
#'   events.
#' @param format Output format. One of \code{"flextable"} (default),
#'   \code{"gt"}, or \code{"tibble"}.
#' @param theme Table styling preset.
#'
#' @return A list of class \code{c("gtregression","logrank_test", ...)} with
#' elements:
#' \describe{
#'   \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
#'   when \code{format = "tibble"}.}
#'   \item{\code{table_body}}{Tibble with group-level log-rank components and
#'   overall test statistics.}
#'   \item{\code{test}}{One-row tibble with chi-square statistic, degrees of
#'   freedom, and p-value.}
#'   \item{\code{fit}}{The \code{survdiff} object.}
#'   \item{\code{time,event,by,format,source}}{Metadata fields.}
#' }
#'
#' @details
#' The log-rank test compares the observed number of events with the expected
#' number of events in each group under the null hypothesis that the survival
#' curves are the same. It is a group comparison, not an effect-size estimate;
#' use \code{cox_reg()} when a hazard ratio is needed.
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#'
#' logrank_test(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   by = trt
#' )
#'
#' logrank_test(
#'   data = lung_data,
#'   time = "time",
#'   event = "status",
#'   by = "trt",
#'   format = tibble
#' )
#'
#' @importFrom survival survdiff
#' @importFrom stats pchisq complete.cases
#' @export
logrank_test <- function(data,
                         time,
                         event,
                         by,
                         digits = 2,
                         format = c("flextable", "gt", "tibble"),
                         theme = c("minimal")) {

  time <- .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  event <- .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  by <- .cox_single_var_arg(substitute(by), data = data, env = parent.frame())
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  theme <- .choice_arg(substitute(theme), env = parent.frame())

  format <- match.arg(format, c("flextable", "gt", "tibble"))
  theme <- .resolve_theme(theme)

  data_clean <- .validate_logrank_inputs(
    data = data,
    time = time,
    event = event,
    by = by,
    digits = digits
  )
  data_clean[[by]] <- factor(data_clean[[by]])

  fit <- survival::survdiff(.km_formula(time, event, by), data = data_clean)
  test <- .logrank_test_summary(fit)
  table_body <- .logrank_table_body(fit, test)

  if (format == "tibble") {
    return(tibble::as_tibble(table_body))
  }

  table_display <- .logrank_table_display(table_body, digits = digits)
  tbl <- .build_logrank_table(
    table_display,
    test = test,
    digits = digits,
    format = format,
    theme = theme
  )

  res <- list(
    table = tbl,
    table_body = tibble::as_tibble(table_body),
    table_display = table_display,
    test = tibble::as_tibble(test),
    fit = fit,
    time = time,
    event = event,
    by = by,
    format = format,
    source = "logrank_test"
  )

  class(res) <- c("gtregression", "logrank_test", class(res))
  res
}

#' @keywords internal
#' @noRd
.validate_logrank_inputs <- function(data, time, event, by, digits) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }
  if (!is.character(time) || length(time) != 1L || !time %in% names(data)) {
    stop("`time` must be a single survival time variable in `data`.", call. = FALSE)
  }
  if (!is.character(event) || length(event) != 1L || !event %in% names(data)) {
    stop("`event` must be a single event indicator variable in `data`.", call. = FALSE)
  }
  if (!is.character(by) || length(by) != 1L || !by %in% names(data)) {
    stop("`by` must be a single grouping variable in `data`.", call. = FALSE)
  }
  if (!is.numeric(data[[time]])) {
    stop("`time` must be numeric.", call. = FALSE)
  }
  if (any(data[[time]] < 0, na.rm = TRUE)) {
    stop("`time` must contain non-negative follow-up times.", call. = FALSE)
  }
  if (!is.numeric(digits) || length(digits) != 1L || is.na(digits) ||
      digits < 0 || digits != floor(digits)) {
    stop("`digits` must be a non-negative whole number.", call. = FALSE)
  }

  vars_needed <- unique(c(time, event, by))
  data_clean <- data[stats::complete.cases(data[, vars_needed, drop = FALSE]), , drop = FALSE]
  if (nrow(data_clean) == 0) {
    stop("No complete cases available for log-rank test.", call. = FALSE)
  }

  data_clean[[event]] <- .cox_event01(data_clean[[event]])
  if (sum(data_clean[[event]] == 1) == 0) {
    stop("`event` must include at least one event.", call. = FALSE)
  }
  if (length(unique(data_clean[[by]])) < 2L) {
    stop("`by` must contain at least two non-missing groups.", call. = FALSE)
  }

  data_clean
}

#' @keywords internal
#' @noRd
.logrank_test_summary <- function(fit) {
  chisq <- as.numeric(fit$chisq)
  df <- length(fit$n) - 1L
  data.frame(
    Chi.square = chisq,
    df = df,
    p.value = stats::pchisq(chisq, df = df, lower.tail = FALSE),
    stringsAsFactors = FALSE
  )
}

#' @keywords internal
#' @noRd
.logrank_table_body <- function(fit, test) {
  data.frame(
    Group = sub("^.*=", "", names(fit$n)),
    N = as.numeric(fit$n),
    Observed = as.numeric(fit$obs),
    Expected = as.numeric(fit$exp),
    Chi.square = test$Chi.square,
    df = test$df,
    p.value = test$p.value,
    stringsAsFactors = FALSE
  )
}

#' @keywords internal
#' @noRd
.logrank_table_display <- function(table_body, digits = 2) {
  data.frame(
    Group = table_body$Group,
    N = as.character(table_body$N),
    `Observed events` = as.character(table_body$Observed),
    `Expected events` = formatC(table_body$Expected, digits = digits, format = "f"),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )
}

#' @keywords internal
#' @noRd
.build_logrank_table <- function(display,
                                 test,
                                 digits = 2,
                                 format = c("flextable", "gt"),
                                 theme = c("minimal")) {
  format <- match.arg(format, c("flextable", "gt"))
  p_txt <- if (is.na(test$p.value)) {
    "NA"
  } else if (test$p.value < 0.001) {
    "<0.001"
  } else {
    formatC(test$p.value, digits = 3, format = "f")
  }
  note <- paste0(
    "Log-rank test: chi-square = ",
    formatC(test$Chi.square, digits = digits, format = "f"),
    ", df = ",
    test$df,
    ", p-value = ",
    p_txt,
    ". This compares survival curves; use cox_reg() when a hazard ratio is needed."
  )

  if (format == "gt") {
    tbl <- gt::gt(display) |>
      gt::tab_header(title = "Log-rank test") |>
      gt::cols_align(align = "left", columns = "Group") |>
      gt::cols_align(
        align = "center",
        columns = c("N", "Observed events", "Expected events")
      ) |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_column_labels()
      ) |>
      gt::tab_source_note(gt::md(note)) |>
      .compact_gt_source_notes()

    if ("header_shaded" %in% theme) {
      tbl <- gt::tab_options(tbl, column_labels.background.color = "#f6f8fa")
    }
    if ("zebra" %in% theme) {
      tbl <- gt::opt_row_striping(tbl)
    }
    if ("compact" %in% theme) {
      tbl <- gt::tab_options(tbl, data_row.padding = gt::px(2))
    }
    return(tbl)
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Log-rank test")
  ft <- flextable::align(ft, j = "Group", align = "left", part = "all")
  ft <- flextable::align(
    ft,
    j = c("N", "Observed events", "Expected events"),
    align = "center",
    part = "all"
  )
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  if ("header_shaded" %in% theme) {
    ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  }
  if ("zebra" %in% theme && nrow(display) > 1L) {
    ft <- flextable::bg(ft, i = seq(1, nrow(display), by = 2), bg = "#f6f8fa", part = "body")
  }
  if ("compact" %in% theme) {
    ft <- flextable::padding(ft, padding = 2, part = "body")
  }
  ft <- flextable::add_footer_lines(ft, values = note)
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

lung_logrank_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment"))
    )
}

test_that("logrank_test returns grouped survival comparison table", {
  skip_if_not_installed("survival")

  df <- lung_logrank_data()

  res <- logrank_test(
    data = df,
    time = time,
    event = status,
    by = trt
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "logrank_test")
  expect_s3_class(res$table, "flextable")
  expect_s3_class(res$fit, "survdiff")
  expect_equal(res$source, "logrank_test")
  expect_equal(res$time, "time")
  expect_equal(res$event, "status")
  expect_equal(res$by, "trt")
  expect_equal(res$table_body$Group, c("Standard treatment", "Test treatment"))
  expect_equal(res$table_body$N, c(69, 68))
  expect_equal(res$table_body$Observed, c(64, 64))
  expect_equal(res$table_body$Expected, c(64.50315, 63.49685), tolerance = 1e-4)
  expect_equal(res$test$Chi.square, 0.008227343, tolerance = 1e-6)
  expect_equal(res$test$df, 1)
  expect_equal(res$test$p.value, 0.9277272, tolerance = 1e-6)
})

test_that("logrank_test supports quoted names and tibble output", {
  skip_if_not_installed("survival")

  df <- lung_logrank_data()
  time_var <- "time"
  event_var <- "status"

  out <- logrank_test(
    data = df,
    time = time_var,
    event = event_var,
    by = "trt",
    format = tibble
  )

  expect_s3_class(out, "tbl_df")
  expect_equal(out$Group, c("Standard treatment", "Test treatment"))
  expect_true(all(c("Chi.square", "df", "p.value") %in% names(out)))
})

test_that("logrank_test returns gt table", {
  skip_if_not_installed("survival")
  skip_if_not_installed("gt")

  df <- lung_logrank_data()

  res <- logrank_test(
    data = df,
    time = time,
    event = status,
    by = trt,
    format = gt
  )

  expect_s3_class(res$table, "gt_tbl")
})

test_that("logrank_test validates inputs", {
  skip_if_not_installed("survival")

  df <- lung_logrank_data()
  df$bad_event <- 0
  df$one_group <- "A"

  expect_error(
    logrank_test(data = "not data", time = time, event = status, by = trt),
    "`data` must be"
  )
  expect_error(
    logrank_test(data = df, time = missing_time, event = status, by = trt),
    "`time` must be"
  )
  expect_error(
    logrank_test(data = df, time = time, event = missing_event, by = trt),
    "`event` must be"
  )
  expect_error(
    logrank_test(data = df, time = time, event = status, by = missing_group),
    "`by` must be"
  )
  expect_error(
    logrank_test(data = df, time = time, event = bad_event, by = trt),
    "`event` must include at least one event"
  )
  expect_error(
    logrank_test(data = df, time = time, event = status, by = one_group),
    "`by` must contain at least two"
  )
  expect_error(
    logrank_test(data = df, time = time, event = status, by = trt, digits = -1),
    "`digits` must be"
  )
})

# R/print.R

#' Print gtregression objects (unified)
#'
#' Prints the rendered table for any object produced by this package
#' (objects that include class \code{"gtregression"}), regardless of subtype
#' (\code{uni_reg}, \code{multi_reg}, \code{stratified_*}, \code{merged_table},
#' \code{descriptive_table}, ...). If no rendered table is found, a compact
#' structure of the object (or its display data) is shown.
#'
#' @param x An object with class \code{"gtregression"}.
#' @param ... Ignored. Present for compatibility with the generic.
#'
#' @export
print.gtregression <- function(x, ...) {
  # Preferred inner table if present
  tb <- NULL
  if (!is.null(x$table)) tb <- x$table
  # In case someone passes the raw gt/flextable itself but it still has the class
  if (is.null(tb) && (inherits(x, "gt_tbl") || inherits(x, "flextable"))) tb <- x

  if (inherits(tb, "gt_tbl") || inherits(tb, "flextable")) {
    print(tb)
  } else if (!is.null(tb)) {
    # Some other table-like object
    print(tb)
  } else if (!is.null(x$table_display)) {
    # Fallback: show a compact peek of the display data so users see "something"
    utils::str(utils::head(x$table_display, 10))
  } else {
    # Last resort
    utils::str(x)
  }
  invisible(x)
}

#' Print identify_confounder results
#'
#' @param x An object returned by \code{identify_confounder()}.
#' @param ... Ignored. Present for compatibility with the generic.
#'
#' @noRd
#' @export
print.identify_confounder_result <- function(x, ...) {
  if (!is.null(x$table)) {
    print(x$table)
  } else if (!is.null(x$summary)) {
    print(x$summary)
  } else {
    utils::str(x)
  }
  invisible(x)
}

#' Univariate regression
#'
#' Fit one model per exposure and return a clean regression table in
#' \pkg{flextable} or \pkg{gt} format.
#'
#' @param data A data frame containing the outcome and exposure variables.
#' @param outcome Character scalar; outcome column name. Quoted and bare names
#'   are accepted.
#' @param exposures Character vector; exposure column names. Quoted names are
#'   recommended in scripts, and bare names are also accepted.
#' @param approach Regression approach. One of \code{"logit"}, \code{"firth"},
#'   \code{"logbinomial"}, \code{"poisson"}, \code{"robpoisson"},
#'   \code{"linear"}, or \code{"negbin"}. Use \code{"firth"} for Firth
#'   penalized logistic regression, especially with sparse cells or separation.
#' @param format One of \code{"flextable"} (default) or \code{"gt"}.
#' @param theme Preset name (e.g. \code{"minimal"}, \code{"striped"}, \code{"clinical"},
#'   \code{"shaded"}, \code{"jama"}) or primitives
#'   \code{c("plain","zebra","lines","labels_bold","compact","header_shaded")}
#' @param model_stats Logical; if \code{TRUE}, extract model-fit statistics
#'   such as AIC, BIC, log-likelihood, deviance, pseudo R-squared for
#'   non-linear models, and R-squared for linear models. Statistics are stored
#'   in the returned object's \code{model_stats} element and are not added to
#'   the publication table.
#' @param show_ref Logical; if \code{TRUE} (default), display reference-category
#'   rows as \code{"Ref."}. If \code{FALSE}, hide reference rows; a message
#'   reminds users to use \code{show_ref = TRUE} when reference rows are needed.
#'
#' @details
#' Use this when you want a quick crude association table before building an
#' adjusted model. The fitted models are kept in the returned object, so the
#' formatted table does not hide the underlying analysis.
#'
#' If exposure variables have a \code{"label"} attribute, for example from
#' \code{labelled::var_label()}, those labels are used automatically in the
#' displayed table and plots. Internal matching still uses the original column
#' names.
#'
#' @return A list of class \code{c("gtregression","uni_reg", ...)} with elements:
#' \describe{
#'   \item{table}{A \code{flextable} (when \code{format="flextable"}) or
#'   \code{gt_tbl} (when \code{format="gt"}).}
#'   \item{table_body}{Data frame of numeric estimates and CIs.}
#'   \item{table_display}{Data frame for display (headers + levels).}
#'   \item{models}{List of fitted univariate models.}
#'   \item{model_summaries}{Per-model \code{summary()} results.}
#'   \item{model_stats}{Model-fit statistics when \code{model_stats = TRUE};
#'   otherwise \code{NULL}.}
#'   \item{variable_labels}{Named character vector of display labels used for
#'   exposure variables.}
#'   \item{reg_check}{Diagnostics for linear models. Printing this element
#'   renders a publication-ready table; individual diagnostic rows remain
#'   available for programmatic use. Other approaches return a message.}
#'   \item{approach, format, source}{Metadata fields.}
#' }
#'
#' @examples
#' d <- mtcars
#' if (requireNamespace("gt", quietly = TRUE)) {
#'   uni_reg(d, "am", c("mpg","cyl"), approach = "logit", format = "gt")$table
#' }
#' if (requireNamespace("flextable", quietly = TRUE)) {
#'   ft <- uni_reg(d, "am", c("mpg","cyl"), approach = "logit", format = "flextable")
#'   class(ft$table)
#' }
#'
#' endometrial_data <- data_endometrial
#' endometrial_data$HG <- factor(endometrial_data$HG, levels = c(0, 1))
#' endometrial_data$NV <- factor(endometrial_data$NV, levels = c(0, 1))
#' uni_reg(endometrial_data, HG, c(NV, PI, EH), approach = firth, format = gt)$table
#'
#' @importFrom stats qnorm residuals
#' @export
uni_reg <- function(data,
                    outcome,
                    exposures,
                    approach = "logit",
                    format = c("flextable","gt"),
                    theme = c("minimal"),
                    model_stats = FALSE,
                    show_ref = TRUE) {

  outcome <- .vars_arg(
    substitute(outcome),
    env = parent.frame(),
    data_names = names(data)
  )
  exposures <- .vars_arg(
    substitute(exposures),
    env = parent.frame(),
    data_names = names(data)
  )
  approach <- .choice_arg(
    substitute(approach),
    env = parent.frame(),
    choices = c("logit","firth","logbinomial","poisson","robpoisson","linear","negbin")
  )
  approach <- .normalize_approach(approach)
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("flextable","gt"))
  theme <- .choice_arg(substitute(theme), env = parent.frame())
  if (!is.logical(model_stats) || length(model_stats) != 1L || is.na(model_stats)) {
    stop("`model_stats` must be TRUE or FALSE.", call. = FALSE)
  }
  .validate_show_ref(show_ref)

  format <- match.arg(format, c("flextable","gt"))
  theme  <- .resolve_theme(theme)

  # ---- validate inputs (package helper) -------------------------------------
  .validate_uni_inputs(data, outcome, exposures, approach)

  # engine tag for S3 class chain
  fmt_class <- if (format == "gt") "gt_uni" else "ft_uni"

  # ---- fit, tidy, stack ------------------------------------------------------
  model_list <- lapply(exposures, function(x) .fit_uni_model(data, outcome, x, approach))
  names(model_list) <- exposures
  model_list <- Filter(Negate(is.null), model_list)
  if (!length(model_list))
    stop("All models failed. Check data and exposure specifications.", call. = FALSE)

  tidy_list <- Map(function(fit, var) .tidy_uni(fit, var, approach), model_list, names(model_list))
  tidy_list <- Filter(Negate(is.null), tidy_list)
  if (!length(tidy_list))
    stop("No estimable coefficients for supplied exposures.", call. = FALSE)

  td <- do.call(rbind, tidy_list)

  # ---- display + build table -------------------------------------------------
  effect_label <- .get_effect_label(approach)
  .message_hidden_ref_rows("uni_reg", td, show_ref)
  display_df   <- .make_display(td, data, outcome, approach, effect_label, show_ref = show_ref)
  .must_be_display_df(display_df)

  source_note  <- c(
    .abbrev_note(approach),
    if (isTRUE(show_ref) && any(td$ref %in% TRUE)) .ref_note() else NULL
  )

  tbl <- if (format == "gt") {
    .build_gt(display_df, effect_label, source_note, theme)
  } else {
    .build_flextable(display_df, effect_label, source_note, theme)
  }

  # ---- summaries & diagnostics ----------------------------------------------
  model_summaries <- lapply(model_list, .model_summary)
  fit_stats <- if (isTRUE(model_stats)) .model_stats_table(model_list, approach) else NULL

  # Always define a default first (belt-and-braces)
  reg_diagnostics <- list(message = "Regression diagnostics available only for 'linear' models.")

  if (identical(approach, "linear")) {
    reg_diagnostics <- lapply(names(model_list), function(v) {
      .reg_check_linear(model_list[[v]], exposure = v)
    })
    names(reg_diagnostics) <- names(model_list)
  }


  # ---- return ----------------------------------------------------------------
  res <- list(
    table           = tbl,
    table_body      = td,
    table_display   = display_df,
    models          = model_list,
    model_summaries = model_summaries,
    model_stats     = fit_stats,
    variable_labels = attr(display_df, "variable_labels", exact = TRUE),
    footnotes       = source_note,
    reg_check       = .as_reg_check_result(reg_diagnostics, format = format),
    approach        = approach,
    format          = format,
    source          = "uni_reg",
    show_ref        = isTRUE(show_ref)
  )
  class(res) <- c("gtregression", "uni_reg", fmt_class, class(res))
  res
}

test_that("uni_reg returns a gtregression object for binary logistic models", {
  df <- mtcars
  df$am <- as.integer(df$am)
  df$cyl <- factor(df$cyl)

  res <- uni_reg(
    data = df,
    outcome = "am",
    exposures = c("mpg", "cyl"),
    approach = logit,
    format = gt
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "uni_reg")
  expect_s3_class(res, "gt_uni")
  expect_s3_class(res$table, "gt_tbl")
  expect_equal(res$approach, "logit")
  expect_equal(res$format, "gt")
  expect_equal(res$source, "uni_reg")
  expect_named(
    res,
    c("table", "table_body", "table_display", "models",
      "model_summaries", "model_stats", "variable_labels", "footnotes", "reg_check",
      "approach", "format", "source", "show_ref")
  )
  expect_null(res$model_stats)
  expect_true(all(c("exposure", "level", "estimate", "conf.low",
                    "conf.high", "p.value", "ref") %in% names(res$table_body)))
  expect_true(any(res$table_body$ref))
  expect_true(all(c("Characteristic", "OR (95% CI)", "p-value", "N",
                    "is_header") %in% names(res$table_display)))
  expect_true("Ref." %in% res$table_display[["OR (95% CI)"]])
  expect_true(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                        fixed = TRUE)))
})

test_that("uni_reg publication table preserves user exposure order", {
  df <- mtcars
  df$am <- as.integer(df$am)
  df$cyl <- factor(df$cyl)
  exposures <- c("hp", "cyl", "mpg")

  res <- uni_reg(
    data = df,
    outcome = "am",
    exposures = exposures,
    approach = logit
  )

  headers <- res$table_display$Characteristic[res$table_display$is_header]
  expect_equal(headers, exposures)
  expect_equal(unique(res$table_body$exposure), exposures)
})

test_that("uni_reg explains how to show reference rows when hidden", {
  df <- mtcars
  df$am <- as.integer(df$am)
  df$vs <- factor(df$vs, levels = c(0, 1), labels = c("No", "Yes"))

  expect_message(
    res <- uni_reg(
      data = df,
      outcome = am,
      exposures = vs,
      approach = logit,
      show_ref = FALSE
    ),
    "To display Ref., use `show_ref = TRUE`.",
    fixed = TRUE
  )

  expect_true(any(res$table_body$ref))
  expect_equal(nrow(res$table_display), 1L)
  expect_false("Ref." %in% res$table_display[["OR (95% CI)"]])
  expect_false(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                         fixed = TRUE)))
})

test_that("uni_reg optionally returns model-fit statistics", {
  df <- mtcars
  df$am <- as.integer(df$am)
  df$cyl <- factor(df$cyl)

  res <- uni_reg(
    data = df,
    outcome = "am",
    exposures = c("mpg", "cyl"),
    approach = logit,
    model_stats = TRUE
  )

  expect_s3_class(res, "uni_reg")
  expect_s3_class(res$model_stats, "data.frame")
  expect_equal(res$model_stats$model, c("mpg", "cyl"))
  expect_true(all(c("AIC", "BIC", "logLik", "deviance", "null_deviance",
                    "pseudo_r2", "r_squared", "adj_r_squared", "n") %in%
                    names(res$model_stats)))
  expect_true(all(is.finite(res$model_stats$AIC)))
  expect_true(all(is.finite(res$model_stats$BIC)))
  expect_true(all(is.na(res$model_stats$r_squared)))
  expect_error(
    uni_reg(df, outcome = "am", exposures = "mpg", model_stats = NA),
    "`model_stats` must be TRUE or FALSE"
  )
})

test_that("uni_reg matches direct model fits with exposure-specific complete cases", {
  df <- data.frame(
    y = c(0, 1, 0, 1, 0, 1, 1, 0),
    y_cont = c(1.1, 1.4, 1.8, 2.2, 2.5, 3.0, 3.4, 3.9),
    age = c(40, 44, 50, 52, 60, 62, 70, 72),
    marker = c(1, 0, NA, 1, NA, 0, 1, 0)
  )

  logit_res <- uni_reg(
    data = df,
    outcome = y,
    exposures = c(age, marker),
    approach = logit,
    model_stats = TRUE
  )
  direct_logit_age <- stats::glm(y ~ age, data = df, family = stats::binomial("logit"))
  direct_logit_marker <- stats::glm(y ~ marker, data = df, family = stats::binomial("logit"))

  expect_equal(stats::coef(logit_res$models$age), stats::coef(direct_logit_age), tolerance = 1e-8)
  expect_equal(stats::coef(logit_res$models$marker), stats::coef(direct_logit_marker), tolerance = 1e-8)
  expect_equal(logit_res$model_stats$n, c(stats::nobs(direct_logit_age), stats::nobs(direct_logit_marker)))
  expect_gt(stats::nobs(logit_res$models$age), stats::nobs(logit_res$models$marker))

  linear_res <- uni_reg(
    data = df,
    outcome = y_cont,
    exposures = c(age, marker),
    approach = linear,
    model_stats = TRUE
  )
  direct_lm_age <- stats::lm(y_cont ~ age, data = df)
  direct_lm_marker <- stats::lm(y_cont ~ marker, data = df)

  expect_equal(stats::coef(linear_res$models$age), stats::coef(direct_lm_age), tolerance = 1e-8)
  expect_equal(stats::coef(linear_res$models$marker), stats::coef(direct_lm_marker), tolerance = 1e-8)
  expect_equal(linear_res$model_stats$n, c(stats::nobs(direct_lm_age), stats::nobs(direct_lm_marker)))
})

test_that("uni_reg supports logbinomial and old hyphenated alias", {
  df <- data.frame(
    y = c(0, 1, 0, 1, 0, 1),
    x = c(0, 0, 1, 1, 1, 0)
  )

  res_new <- uni_reg(df, outcome = "y", exposures = "x", approach = logbinomial)
  res_old <- uni_reg(df, outcome = "y", exposures = "x", approach = "log-binomial")

  expect_equal(res_new$approach, "logbinomial")
  expect_equal(res_old$approach, "logbinomial")
  expect_named(res_new$table_display, c("Characteristic", "N", "RR (95% CI)", "p-value", "is_header"), ignore.order = TRUE)
})

test_that("uni_reg supports poisson and negative binomial count models", {
  df <- data.frame(
    count = c(0, 1, 2, 3, 1, 4, 2, 5, 3, 6),
    exposure = c(0, 1, 0, 1, 0, 1, 0, 1, 0, 1)
  )

  pois <- uni_reg(df, outcome = "count", exposures = "exposure", approach = poisson)
  nb <- uni_reg(df, outcome = "count", exposures = "exposure", approach = negbin)

  expect_equal(pois$approach, "poisson")
  expect_equal(nb$approach, "negbin")
  expect_true("IRR (95% CI)" %in% names(pois$table_display))
  expect_true("IRR (95% CI)" %in% names(nb$table_display))
})

test_that("uni_reg supports robust poisson when risks is available", {
  skip_if_not_installed("risks")

  df <- data.frame(
    y = c(0, 1, 0, 1, 0, 1, 1, 0),
    x = c(0, 0, 1, 1, 1, 0, 1, 0)
  )

  res <- uni_reg(df, outcome = "y", exposures = "x", approach = robpoisson)

  expect_equal(res$approach, "robpoisson")
  expect_true("RR (95% CI)" %in% names(res$table_display))
})

test_that("uni_reg supports Firth logistic regression", {
  skip_if_not_installed("logistf")

  df <- data_endometrial
  df$HG <- factor(df$HG, levels = c(0, 1),
                  labels = c("Low grade", "High grade"))
  df$NV <- factor(df$NV, levels = c(0, 1),
                  labels = c("Absent", "Present"))

  res <- uni_reg(df, outcome = HG, exposures = c(NV, PI, EH), approach = firth)

  expect_s3_class(res, "uni_reg")
  expect_equal(res$approach, "firth")
  expect_s3_class(res$models$NV, "logistf")
  expect_true("OR (95% CI)" %in% names(res$table_display))
  expect_true(any(res$table_body$ref))
  expect_true(any(is.finite(res$table_body$estimate[!res$table_body$ref])))
  expect_equal(unname(table(df$HG, df$NV)["Low grade", "Present"]), 0)
})

test_that("uni_reg returns diagnostics for linear model", {
  df <- mtcars
  df$cyl <- factor(df$cyl)

  res <- uni_reg(
    data = df,
    outcome = "mpg",
    exposures = c("hp", "cyl"),
    approach = linear
  )

  expect_s3_class(res, "uni_reg")
  expect_type(res$reg_check, "list")
  expect_named(res$reg_check, c("hp", "cyl"))
  expect_match(res$reg_check$hp$Test[1], "Breusch-Pagan")
  expect_true("Beta (95% CI)" %in% names(res$table_display))

  stats_res <- uni_reg(
    data = df,
    outcome = "mpg",
    exposures = c("hp", "cyl"),
    approach = linear,
    model_stats = TRUE
  )
  expect_true(all(is.na(stats_res$model_stats$pseudo_r2)))
  expect_true(all(is.finite(stats_res$model_stats$r_squared)))
  expect_true(all(is.finite(stats_res$model_stats$adj_r_squared)))
})

test_that("uni_reg supports flextable output", {
  skip_if_not_installed("flextable")

  df <- mtcars
  df$am <- as.integer(df$am)

  res <- uni_reg(
    data = df,
    outcome = "am",
    exposures = "mpg",
    approach = logit,
    format = flextable
  )

  expect_s3_class(res, "ft_uni")
  expect_s3_class(res$table, "flextable")
  expect_equal(res$format, "flextable")
})

test_that("uni_reg errors clearly when all models fail", {
  df <- mtcars
  df$onelevel <- factor(rep("only", nrow(df)))

  expect_error(
    suppressWarnings(
      uni_reg(df, outcome = "mpg", exposures = "onelevel", approach = linear)
    ),
    "Exposure with <2 levels"
  )
})

test_that("uni_reg handles bad outcome type gracefully", {
  df <- mtcars
  df$mpg_char <- as.character(df$mpg)

  expect_error(
    uni_reg(df, outcome = "mpg_char", exposures = "hp", approach = linear),
    "requires a continuous"
  )
})

test_that("uni_reg accessors return current components", {
  df <- mtcars
  df$am <- as.integer(df$am)

  res <- uni_reg(
    data = df,
    outcome = "am",
    exposures = "mpg",
    approach = logit
  )

  expect_s3_class(res$models[[1]], "glm")
  expect_s3_class(res$model_summaries[[1]], "summary.glm")
  expect_s3_class(res$table, "flextable")
  expect_equal(res$format, "flextable")
  expect_identical(res$engine, res$format)
})
test_that("uni_reg displays every level of character predictors", {
  set.seed(410)
  d <- data.frame(
    outcome = stats::rbinom(180, 1, 0.4),
    classification = rep(c("Associated", "Elementary", "Combined"), each = 60)
  )

  res <- uni_reg(d, outcome, classification, approach = "logit", format = "gt")
  rows <- res$table_body[res$table_body$exposure == "classification", ]

  expect_equal(rows$level, c("Associated", "Combined", "Elementary"))
  expect_equal(sum(rows$ref), 1L)
  expect_equal(nrow(rows), 3L)
  expect_true(all(c("Associated", "Combined", "Elementary") %in%
                    trimws(res$table_display$Characteristic)))
})

test_that("plot_model_fit works for raw lm models", {
  fit <- stats::lm(mpg ~ wt + hp, data = mtcars)

  p_all <- plot_model_fit(fit)
  expect_s3_class(p_all, "patchwork")
  expect_equal(attr(p_all, "source"), "plot_model_fit")

  p_resid <- plot_model_fit(fit, type = residual)
  expect_s3_class(p_resid, "ggplot")
})

test_that("plot_model_fit works for raw binomial glm models", {
  fit <- stats::glm(am ~ mpg + wt, data = mtcars, family = stats::binomial())

  p_all <- plot_model_fit(fit)
  expect_s3_class(p_all, "patchwork")

  p_cal <- plot_model_fit(fit, type = calibration, bins = 4)
  expect_s3_class(p_cal, "ggplot")
})

test_that("plot_model_fit calibration handles few distinct fitted values", {
  d <- data.frame(
    y = c(rep(0, 8), rep(1, 6), rep(0, 5), rep(1, 9)),
    x = factor(rep(c("No", "Yes"), each = 14))
  )
  fit <- stats::glm(y ~ x, data = d, family = stats::binomial())

  p <- plot_model_fit(fit, type = calibration, bins = 6)
  expect_s3_class(p, "ggplot")
})

test_that("plot_model_fit works for raw non-binomial glm models", {
  fit <- stats::glm(carb ~ mpg + wt, data = mtcars, family = stats::poisson())

  p <- plot_model_fit(fit, type = observed_predicted)
  expect_s3_class(p, "ggplot")
})

test_that("plot_model_fit selects models from uni_reg results", {
  fit <- uni_reg(
    data = mtcars,
    outcome = am,
    exposures = c(mpg, wt),
    approach = logit
  )

  expect_message(
    p_default <- plot_model_fit(fit),
    "Multiple fitted models found"
  )
  expect_s3_class(p_default, "patchwork")
  expect_equal(attr(p_default, "model_name"), "mpg")

  p_named <- plot_model_fit(fit, model_name = wt, type = calibration, bins = 4)
  expect_s3_class(p_named, "ggplot")
  expect_equal(attr(p_named, "model_name"), "wt")
})

test_that("plot_model_fit selects models from multi_reg results", {
  fit <- multi_reg(
    data = mtcars,
    outcome = am,
    exposures = c(mpg, wt),
    approach = logit
  )

  p <- plot_model_fit(fit, type = cooks)
  expect_s3_class(p, "ggplot")
  expect_equal(attr(p, "model_name"), "multivariable_model")
})

test_that("plot_model_fit validates inputs clearly", {
  expect_error(plot_model_fit(list()), "fitted model")

  fit <- stats::lm(mpg ~ wt, data = mtcars)
  expect_error(plot_model_fit(fit, bins = 1), "`bins`")
  expect_error(plot_model_fit(fit, base_size = 0), "`base_size`")
  expect_error(plot_model_fit(fit, type = calibration), "binomial")
})

test_that("plot_model_fit directs survival objects to survival helpers", {
  fit <- list(models = list(model = structure(list(), class = "coxph")))
  class(fit) <- c("gtregression", "cox_reg", "list")

  expect_error(
    plot_model_fit(fit),
    "check_ph\\(\\).*plot_surv_fit\\(\\)"
  )
})

#' Plot regression estimates
#'
#' Draw a forest-style plot from a fitted \code{gtregression} object produced by
#' functions such as \code{uni_reg()}, \code{multi_reg()}, \code{cox_reg()}, or
#' \code{surv_reg()}. Stratified regression outputs are displayed as one
#' faceted quick-view plot; use \code{forest_df()} and \code{forest_reg()} for
#' publication-style stratified forest tables.
#'
#' @param tbl A fitted \code{gtregression} object.
#' @param title Optional plot title.
#' @param caption Optional plot caption. If \code{NULL}, notes are added
#'   automatically for adjusted \code{multi_reg()} objects and displayed
#'   reference categories.
#' @param ref_line Optional numeric value for the reference line.
#'   Defaults to 0 for linear models and 1 otherwise.
#' @param order_y Optional character vector specifying exposure order.
#' @param log_x Logical. If \code{TRUE}, use a log-scaled x-axis for ratio
#'   measures. The default \code{FALSE} uses a linear x-axis.
#' @param xlim Optional numeric vector of length 2 specifying x-axis limits.
#' @param breaks Optional numeric vector of x-axis tick breaks.
#' @param point_color Fill color for non-significant estimate points.
#' @param errorbar_color Color for non-significant confidence intervals.
#' @param point_size Diameter of estimate points in millimetres.
#' @param point_stroke Outline width of estimate points.
#' @param ci_linewidth Line width of confidence intervals.
#' @param base_size Base font size. The default is designed for a clear report
#'   or slide figure; increase it for a single large plot.
#' @param show_ref Logical or \code{NULL}. The default \code{NULL} inherits the
#'   reference-row choice used to create \code{tbl}. If \code{TRUE}, reference
#'   rows are shown. If \code{FALSE}, binary exposures are shown as compact rows
#'   for the estimated non-reference category; affirmative levels such as
#'   \code{Yes} or \code{1} are displayed using the variable name.
#' @param sig_color Optional fill color for significant points.
#' @param sig_errorbar_color Optional color for significant error bars.
#' @param alpha Significance level for linear models when \code{p.value} is available.
#' @param show_adjustment_note Logical; if \code{TRUE}, add a default caption
#'   describing \code{adjust_for} variables when available.
#'
#' @return A \code{ggplot2} object.
#' @importFrom rlang .data
#' @importFrom stats ave setNames
#' @export
plot_reg <- function(tbl,
                     title = NULL,
                     caption = NULL,
                     ref_line = NULL,
                     order_y = NULL,
                     log_x = FALSE,
                     xlim = NULL,
                     breaks = NULL,
                     point_color = "#6B7280",
                     errorbar_color = "#6B7280",
                     point_size = 2.8,
                     point_stroke = 0.55,
                     ci_linewidth = 0.55,
                     base_size = 12,
                     show_ref = NULL,
                     sig_color = "#0072B2",
                     sig_errorbar_color = "#0072B2",
                     alpha = 0.05,
                     show_adjustment_note = TRUE) {

  if (!inherits(tbl, "gtregression")) {
    stop("`tbl` must be a gtregression object.", call. = FALSE)
  }

  source_type <- tbl$source
  approach <- .normalize_approach(tbl$approach)

  if (is.null(source_type) || is.null(approach)) {
    stop("`tbl` must contain `source` and `approach`.", call. = FALSE)
  }

  log_x <- .plot_resolve_log_x(log_x, approach)

  show_ref <- .plot_resolve_show_ref(show_ref, tbl)

  if (.plot_is_stratified(tbl, source_type)) {
    return(.plot_reg_stratified(
      tbl = tbl,
      title = title,
      caption = caption,
      ref_line = ref_line,
      order_y = order_y,
      log_x = log_x,
      xlim = xlim,
      breaks = breaks,
      point_color = point_color,
      errorbar_color = errorbar_color,
      point_size = point_size,
      point_stroke = point_stroke,
      ci_linewidth = ci_linewidth,
      base_size = base_size,
      show_ref = show_ref,
      sig_color = sig_color,
      sig_errorbar_color = sig_errorbar_color,
      alpha = alpha,
      show_adjustment_note = show_adjustment_note
    ))
  }

  if (is.null(tbl$table_body) || !nrow(tbl$table_body)) {
    stop("`tbl$table_body` is missing or empty.", call. = FALSE)
  }

  if (is.null(tbl$table_display) || !nrow(tbl$table_display)) {
    stop("`tbl$table_display` is missing or empty.", call. = FALSE)
  }

  df_body <- tbl$table_body
  df_disp <- tbl$table_display

  req_body <- c("exposure", "level", "estimate", "conf.low", "conf.high", "p.value", "ref")
  if (!all(req_body %in% names(df_body))) {
    stop(
      "plot_reg() requires `table_body` to contain: ",
      paste(req_body, collapse = ", "),
      call. = FALSE
    )
  }

  if (!all(c("Characteristic", "is_header") %in% names(df_disp))) {
    stop(
      "plot_reg() requires `table_display` to contain `Characteristic` and `is_header`.",
      call. = FALSE
    )
  }

  df_body$estimate  <- suppressWarnings(as.numeric(df_body$estimate))
  df_body$conf.low  <- suppressWarnings(as.numeric(df_body$conf.low))
  df_body$conf.high <- suppressWarnings(as.numeric(df_body$conf.high))
  df_body$p.value   <- suppressWarnings(as.numeric(df_body$p.value))

  if (is.null(ref_line)) {
    ref_line <- if (identical(approach, "linear")) 0 else 1
  }

  is_multi <- source_type %in% c("multi_reg", "multi_reg_nbin") ||
    .is_adjusted_reg_output(tbl)

  base_label <- dplyr::case_when(
    approach %in% c("logit", "firth") ~ "Odds Ratio",
    approach == "logbinomial" ~ "Risk Ratio",
    approach %in% c("poisson", "negbin") ~ "Incidence Rate Ratio",
    approach == "robpoisson" ~ "Risk Ratio",
    approach == "cox" ~ "Hazard Ratio",
    approach == "survreg" ~ "Time Ratio",
    approach == "linear" ~ "Beta Coefficient",
    TRUE ~ "Effect Size"
  )

  x_axis_label <- if (isTRUE(is_multi)) paste("Adjusted", base_label) else base_label

  if (log_x) {
    x_axis_label <- paste0(x_axis_label, " (log scale)")
  }

  # ---- build header order from raw table_body names ----
  exposure_order <- unique(df_body$exposure)

  if (!is.null(order_y)) {
    exposure_order <- c(
      intersect(order_y, exposure_order),
      setdiff(exposure_order, order_y)
    )
  }

  # ---- build plotting rows using table_display skeleton + table_body values ----
  plot_rows <- list()

  for (ex in exposure_order) {
    dfx <- df_body[df_body$exposure == ex, , drop = FALSE]
    if (!nrow(dfx)) next

    ex_label <- .label_var(ex, tbl$variable_labels)
    is_factor_exp <- any(dfx$ref)
    is_binary_exp <- is_factor_exp && nrow(dfx) == 2L && sum(dfx$ref %in% TRUE) == 1L

    # display rows for this exposure
    disp_idx <- which(df_disp$is_header & trimws(df_disp$Characteristic) == ex_label)
    if (!length(disp_idx)) next

    start_i <- disp_idx[1]
    end_i <- if (start_i < nrow(df_disp)) {
      next_headers <- which(df_disp$is_header & seq_len(nrow(df_disp)) > start_i)
      if (length(next_headers)) min(next_headers) - 1 else nrow(df_disp)
    } else {
      nrow(df_disp)
    }

    disp_block <- df_disp[start_i:end_i, , drop = FALSE]

    # header row
    main_row <- dfx[!dfx$ref & dfx$level == ex, , drop = FALSE]

    header_plot <- data.frame(
      exposure = ex,
      label = ex_label,
      estimate = NA_real_,
      conf.low = NA_real_,
      conf.high = NA_real_,
      p.value = NA_real_,
      ref = FALSE,
      is_header = TRUE,
      is_data = FALSE,
      stringsAsFactors = FALSE
    )

    # continuous exposure: plot estimate on header row
    if (!is_factor_exp && nrow(main_row)) {
      header_plot$estimate <- main_row$estimate[1]
      header_plot$conf.low <- main_row$conf.low[1]
      header_plot$conf.high <- main_row$conf.high[1]
      header_plot$p.value <- main_row$p.value[1]
      header_plot$is_data <- TRUE
    }

    compact_binary <- is_binary_exp && !isTRUE(show_ref)
    if (compact_binary) {
      binary_row <- dfx[!dfx$ref, , drop = FALSE]
      if (nrow(binary_row)) {
        header_plot$label <- .plot_binary_label(ex_label, binary_row$level[1])
        header_plot$estimate <- binary_row$estimate[1]
        header_plot$conf.low <- binary_row$conf.low[1]
        header_plot$conf.high <- binary_row$conf.high[1]
        header_plot$p.value <- binary_row$p.value[1]
        header_plot$is_header <- TRUE
        header_plot$is_data <- TRUE
        plot_rows[[length(plot_rows) + 1]] <- header_plot
        next
      }
    }

    plot_rows[[length(plot_rows) + 1]] <- header_plot

    # categorical levels OR extra rows (e.g. interaction rows)
    if (nrow(disp_block) > 1) {
      sub_block <- disp_block[-1, , drop = FALSE]

      for (i in seq_len(nrow(sub_block))) {
        char_i <- sub_block$Characteristic[i]
        lab_i <- trimws(char_i)

        row_match <- dfx[dfx$level == lab_i, , drop = FALSE]

        if (!nrow(row_match)) next

        if (isTRUE(row_match$ref[1]) && !show_ref) next

        plot_rows[[length(plot_rows) + 1]] <- data.frame(
          exposure = ex,
          label = lab_i,
          estimate = row_match$estimate[1],
          conf.low = row_match$conf.low[1],
          conf.high = row_match$conf.high[1],
          p.value = row_match$p.value[1],
          ref = isTRUE(row_match$ref[1]),
          is_header = FALSE,
          is_data = !isTRUE(row_match$ref[1]),
          stringsAsFactors = FALSE
        )
      }
    } else {
      # handle non-factor extras in table_body not represented in table_display
      extra_rows <- dfx[!dfx$ref & dfx$level != ex, , drop = FALSE]
      if (nrow(extra_rows)) {
        for (i in seq_len(nrow(extra_rows))) {
          plot_rows[[length(plot_rows) + 1]] <- data.frame(
            exposure = ex,
            label = extra_rows$level[i],
            estimate = extra_rows$estimate[i],
            conf.low = extra_rows$conf.low[i],
            conf.high = extra_rows$conf.high[i],
            p.value = extra_rows$p.value[i],
            ref = FALSE,
            is_header = FALSE,
            is_data = TRUE,
            stringsAsFactors = FALSE
          )
        }
      }
    }
  }

  plot_df <- do.call(rbind, plot_rows)

  # ---- OG-style label rendering ----
  indent_html <- "<span style='color:white;'>..</span>"

  plot_df$label_clean <- dplyr::case_when(
    plot_df$is_header ~ paste0("<b>", plot_df$label, "</b>"),
    plot_df$ref & show_ref ~ paste0(
      indent_html,
      plot_df$label,
      " <span style='color:gray;'>(Ref.)</span>"
    ),
    !plot_df$is_header ~ paste0(indent_html, plot_df$label),
    TRUE ~ NA_character_
  )

  # ---- significance ----
  if (identical(approach, "linear")) {
    plot_df$is_sig <- dplyr::case_when(
      !plot_df$is_data ~ FALSE,
      !is.na(plot_df$p.value) & plot_df$p.value < alpha ~ TRUE,
      TRUE ~ FALSE
    )
  } else {
    plot_df$is_sig <- dplyr::case_when(
      !plot_df$is_data ~ FALSE,
      is.finite(plot_df$conf.low) & is.finite(plot_df$conf.high) &
        ((plot_df$conf.low > ref_line) | (plot_df$conf.high < ref_line)) ~ TRUE,
      TRUE ~ FALSE
    )
  }

  plot_df$row_id <- factor(seq_len(nrow(plot_df)), levels = rev(seq_len(nrow(plot_df))))
  label_map <- stats::setNames(plot_df$label_clean, as.character(plot_df$row_id))

  fill_vals <- c(
    "FALSE" = point_color,
    "TRUE" = if (!is.null(sig_color)) sig_color else point_color
  )
  line_vals <- c(
    "FALSE" = errorbar_color,
    "TRUE" = if (!is.null(sig_errorbar_color)) sig_errorbar_color else errorbar_color
  )

  caption <- .plot_caption(
    tbl = tbl,
    caption = caption,
    show_adjustment_note = show_adjustment_note,
    show_ref = show_ref,
    has_ref = any(plot_df$ref %in% TRUE)
  )

  if (is.null(xlim)) {
    vals <- c(plot_df$conf.low, plot_df$conf.high, plot_df$estimate, ref_line)
    vals <- vals[is.finite(vals)]
    if (length(vals) >= 2L) {
      rng <- range(vals, na.rm = TRUE)
      if (log_x) {
        lower <- max(min(rng[1], ref_line, na.rm = TRUE), .Machine$double.eps * 10)
        upper <- max(rng[2], ref_line, na.rm = TRUE)
      } else {
        span <- diff(rng)
        pad <- if (is.finite(span) && span > 0) 0.05 * span else 0.1
        lower <- min(rng[1], ref_line, na.rm = TRUE) - pad
        upper <- max(rng[2], ref_line, na.rm = TRUE) + pad
      }
      xlim <- c(lower, upper)
    }
  }

  p <- ggplot2::ggplot(
    plot_df,
    ggplot2::aes(x = .data$estimate, y = .data$row_id)
  ) +
    .add_h_ci(plot_df[plot_df$is_data, , drop = FALSE], linewidth = ci_linewidth) +
    ggplot2::geom_point(
      data = plot_df[plot_df$is_data, , drop = FALSE],
      ggplot2::aes(fill = .data$is_sig),
      shape = 21,
      size = point_size,
      stroke = point_stroke,
      show.legend = FALSE,
      na.rm = TRUE
    ) +
    ggplot2::geom_vline(
      xintercept = ref_line,
      linetype = "dashed",
      colour = "gray60"
    ) +
    ggplot2::scale_fill_manual(values = fill_vals, guide = "none") +
    ggplot2::scale_color_manual(values = line_vals, guide = "none") +
    ggplot2::scale_y_discrete(
      labels = label_map,
      limits = levels(plot_df$row_id),
      drop = FALSE
    ) +
    ggplot2::labs(title = title, x = x_axis_label, y = NULL, caption = caption) +
    .plot_reg_theme(base_size = base_size)

  if (log_x) {
    if (is.null(breaks)) {
      breaks <- .plot_log_breaks(xlim)
    }
    p <- p + ggplot2::scale_x_log10(breaks = breaks)
  } else if (!is.null(breaks)) {
    p <- p + ggplot2::scale_x_continuous(breaks = breaks)
  }
  if (!is.null(xlim)) {
    p <- p + ggplot2::coord_cartesian(xlim = xlim)
  }

  p
}

#' Resolve reference-row display for regression plots
#' @keywords internal
#' @noRd
.plot_resolve_show_ref <- function(show_ref, ...) {
  objects <- list(...)

  if (!is.null(show_ref)) {
    if (!is.logical(show_ref) || length(show_ref) != 1L || is.na(show_ref)) {
      stop("`show_ref` must be TRUE, FALSE, or NULL.", call. = FALSE)
    }
    return(isTRUE(show_ref))
  }

  stored <- vapply(objects, function(x) {
    if (is.null(x$show_ref)) NA else isTRUE(x$show_ref)
  }, logical(1))
  stored <- stored[!is.na(stored)]

  if (!length(stored)) return(TRUE)
  if (length(unique(stored)) == 1L) return(stored[[1]])

  warning(
    paste0(
      "The supplied tables use different `show_ref` settings; reference rows are shown to keep ",
      "panels aligned. Set `show_ref` explicitly to override this."
    ),
    call. = FALSE
  )
  TRUE
}

#' Plot stratified regression outputs (internal)
#' @keywords internal
#' @noRd
.plot_reg_stratified <- function(tbl,
                                 title = NULL,
                                 caption = NULL,
                                 ref_line = NULL,
                                 order_y = NULL,
                                 log_x = FALSE,
                                 xlim = NULL,
                                 breaks = NULL,
                                 point_color = "#6B7280",
                                 errorbar_color = "#6B7280",
                                 point_size = 2.8,
                                 point_stroke = 0.55,
                                 ci_linewidth = 0.55,
                                 base_size = 12,
                                 show_ref = TRUE,
                                 sig_color = "#0072B2",
                                 sig_errorbar_color = "#0072B2",
                                 alpha = 0.05,
                                 show_adjustment_note = TRUE) {
  if (is.null(tbl$per_stratum) || !length(tbl$per_stratum)) {
    stop("Stratified regression objects must contain `per_stratum` results.", call. = FALSE)
  }

  strata <- names(tbl$per_stratum)
  if (is.null(strata) || any(!nzchar(strata))) {
    strata <- paste0("Stratum ", seq_along(tbl$per_stratum))
  }

  parts <- Map(function(item, stratum) {
    stratum_tbl <- .plot_stratum_object(tbl, item)
    p <- plot_reg(
      stratum_tbl,
      title = NULL,
      caption = NULL,
      ref_line = ref_line,
      order_y = order_y,
      log_x = log_x,
      xlim = NULL,
      breaks = breaks,
      point_color = point_color,
      errorbar_color = errorbar_color,
      point_size = point_size,
      point_stroke = point_stroke,
      ci_linewidth = ci_linewidth,
      base_size = base_size,
      show_ref = show_ref,
      sig_color = sig_color,
      sig_errorbar_color = sig_errorbar_color,
      alpha = alpha,
      show_adjustment_note = show_adjustment_note
    )
    out <- p$data
    out$stratum <- stratum
    out$row_key <- paste(stratum, seq_len(nrow(out)), sep = "__")
    out
  }, tbl$per_stratum, strata)

  plot_df <- do.call(rbind, parts)
  plot_df$stratum <- factor(plot_df$stratum, levels = strata)
  row_count <- nrow(parts[[1]])
  if (any(vapply(parts, nrow, integer(1)) != row_count)) {
    stop("Each stratum must contain the same display rows for plot_reg().", call. = FALSE)
  }
  plot_df$row_index <- ave(
    seq_len(nrow(plot_df)),
    plot_df$stratum,
    FUN = seq_along
  )
  plot_df$row_key <- factor(plot_df$row_index, levels = rev(seq_len(row_count)))
  first_stratum <- plot_df[plot_df$stratum == strata[[1]], , drop = FALSE]
  label_map <- stats::setNames(first_stratum$label_clean, as.character(first_stratum$row_key))

  approach <- .normalize_approach(tbl$approach)
  if (is.null(ref_line)) {
    ref_line <- if (identical(approach, "linear")) 0 else 1
  }
  log_x <- .plot_resolve_log_x(log_x, approach)

  x_axis_label <- .plot_axis_label(
    approach = approach,
    adjusted = .plot_stratified_is_adjusted(tbl)
  )
  if (log_x) {
    x_axis_label <- paste0(x_axis_label, " (log scale)")
  }

  if (is.null(xlim)) {
    vals <- c(plot_df$conf.low, plot_df$conf.high, plot_df$estimate, ref_line)
    vals <- vals[is.finite(vals)]
    if (length(vals) >= 2L) {
      rng <- range(vals, na.rm = TRUE)
      if (log_x) {
        lower <- max(min(rng[1], ref_line, na.rm = TRUE), .Machine$double.eps * 10)
        upper <- max(rng[2], ref_line, na.rm = TRUE)
      } else {
        span <- diff(rng)
        pad <- if (is.finite(span) && span > 0) 0.05 * span else 0.1
        lower <- min(rng[1], ref_line, na.rm = TRUE) - pad
        upper <- max(rng[2], ref_line, na.rm = TRUE) + pad
      }
      xlim <- c(lower, upper)
    }
  }

  fill_vals <- c(
    "FALSE" = point_color,
    "TRUE" = if (!is.null(sig_color)) sig_color else point_color
  )
  line_vals <- c(
    "FALSE" = errorbar_color,
    "TRUE" = if (!is.null(sig_errorbar_color)) sig_errorbar_color else errorbar_color
  )
  caption <- .plot_caption(
    tbl = tbl,
    caption = caption,
    show_adjustment_note = show_adjustment_note,
    show_ref = show_ref,
    has_ref = any(plot_df$ref %in% TRUE)
  )
  # Facets otherwise drop structural header rows because their estimate is NA.
  plot_df$axis_anchor <- ref_line

  p <- ggplot2::ggplot(
    plot_df,
    ggplot2::aes(x = .data$estimate, y = .data$row_key)
  ) +
    ggplot2::geom_blank(ggplot2::aes(x = .data$axis_anchor, y = .data$row_key)) +
    .add_h_ci(plot_df[plot_df$is_data, , drop = FALSE], linewidth = ci_linewidth) +
    ggplot2::geom_point(
      data = plot_df[plot_df$is_data, , drop = FALSE],
      ggplot2::aes(fill = .data$is_sig),
      shape = 21,
      size = point_size,
      stroke = point_stroke,
      show.legend = FALSE,
      na.rm = TRUE
    ) +
    ggplot2::geom_vline(
      xintercept = ref_line,
      linetype = "dashed",
      colour = "gray60"
    ) +
    ggplot2::facet_wrap(ggplot2::vars(.data$stratum), scales = "fixed") +
    ggplot2::scale_fill_manual(values = fill_vals, guide = "none") +
    ggplot2::scale_color_manual(values = line_vals, guide = "none") +
    ggplot2::scale_y_discrete(
      labels = label_map
    ) +
    ggplot2::labs(title = title, x = x_axis_label, y = NULL, caption = caption) +
    .plot_reg_theme(base_size = base_size, stratified = TRUE)

  if (log_x) {
    if (is.null(breaks)) {
      breaks <- .plot_log_breaks(xlim)
    }
    p <- p + ggplot2::scale_x_log10(breaks = breaks)
  } else if (!is.null(breaks)) {
    p <- p + ggplot2::scale_x_continuous(breaks = breaks)
  }
  if (!is.null(xlim)) {
    p <- p + ggplot2::coord_cartesian(xlim = xlim)
  }

  p
}

#' Stratified plot input detector (internal)
#' @keywords internal
#' @noRd
.plot_is_stratified <- function(tbl, source_type = tbl$source) {
  isTRUE(tbl$stratified) ||
    identical(source_type, "stratified_multi_reg") ||
    identical(source_type, "stratified_uni_reg") ||
    identical(source_type, "stratified_cox_reg") ||
    identical(source_type, "stratified_surv_reg")
}

#' Recreate a plottable per-stratum regression object (internal)
#' @keywords internal
#' @noRd
.plot_stratum_object <- function(parent, x) {
  if (inherits(x, "gtregression")) {
    # Per-stratum uni_reg objects retain their own display data. Rebuild it
    # with the parent label map so faceted plots match the stratified table.
    label_map <- parent$variable_labels
    if (is.null(label_map)) {
      label_map <- x$variable_labels
    }
    x$table_display <- .plot_display_from_body(
      x$table_body,
      label_map
    )
    x$variable_labels <- label_map
    x$stratified <- FALSE
    return(x)
  }

  out <- x
  out$table_display <- .plot_display_from_body(x$table_body, parent$variable_labels)
  out$approach <- parent$approach
  out$variable_labels <- parent$variable_labels
  out$stratified <- FALSE
  out$adjusted_mode <- .forest_or(parent$adjusted_mode, x$adjusted_mode)
  out$multivariable <- .forest_or(parent$multivariable, x$multivariable)
  out$adjust_for <- parent$adjust_for

  if (inherits(parent, "stratified_multi_reg")) {
    out$source <- "multi_reg"
    class(out) <- c("gtregression", "multi_reg", "gt_multi", class(out))
  } else if (inherits(parent, "stratified_cox_reg")) {
    out$source <- "cox_reg"
    class(out) <- c("gtregression", "cox_reg", "gt_cox", class(out))
  } else if (inherits(parent, "stratified_surv_reg")) {
    out$source <- "surv_reg"
    class(out) <- c("gtregression", "surv_reg", "gt_surv", class(out))
  } else {
    out$source <- parent$source
    class(out) <- c("gtregression", class(out))
  }

  out
}

#' Build a minimal table display from table_body rows (internal)
#' @keywords internal
#' @noRd
.plot_display_from_body <- function(table_body, variable_labels = NULL) {
  rows <- list()
  for (ex in unique(table_body$exposure)) {
    dfx <- table_body[table_body$exposure == ex, , drop = FALSE]
    rows[[length(rows) + 1L]] <- data.frame(
      Characteristic = .label_var(ex, variable_labels),
      is_header = TRUE,
      stringsAsFactors = FALSE
    )
    if (any(dfx$ref %in% TRUE)) {
      for (lv in dfx$level) {
        rows[[length(rows) + 1L]] <- data.frame(
          Characteristic = as.character(lv),
          is_header = FALSE,
          stringsAsFactors = FALSE
        )
      }
    }
  }
  out <- do.call(rbind, rows)
  rownames(out) <- NULL
  out
}

#' Plot axis label helper (internal)
#' @keywords internal
#' @noRd
.plot_axis_label <- function(approach, adjusted = FALSE) {
  base <- dplyr::case_when(
    approach %in% c("logit", "firth") ~ "Odds Ratio",
    approach == "logbinomial" ~ "Risk Ratio",
    approach %in% c("poisson", "negbin") ~ "Incidence Rate Ratio",
    approach == "robpoisson" ~ "Risk Ratio",
    approach == "cox" ~ "Hazard Ratio",
    approach == "survreg" ~ "Time Ratio",
    approach == "linear" ~ "Beta Coefficient",
    TRUE ~ "Effect Size"
  )
  if (isTRUE(adjusted)) paste("Adjusted", base) else base
}

#' Resolve automatic log-axis choice for regression plots (internal)
#' @keywords internal
#' @noRd
.plot_resolve_log_x <- function(log_x, approach) {
  if (is.null(log_x)) {
    return(FALSE)
  }
  if (!is.logical(log_x) || length(log_x) != 1L || is.na(log_x)) {
    stop("`log_x` must be TRUE, FALSE, or NULL.", call. = FALSE)
  }
  if (identical(approach, "linear")) return(FALSE)
  isTRUE(log_x)
}

#' Stratified adjusted plot detector (internal)
#' @keywords internal
#' @noRd
.plot_stratified_is_adjusted <- function(tbl) {
  inherits(tbl, "stratified_multi_reg") ||
    isTRUE(tbl$adjusted_mode) ||
    isTRUE(tbl$multivariable) ||
    any(vapply(tbl$per_stratum, function(x) {
      isTRUE(x$adjusted_mode) || isTRUE(x$multivariable)
    }, logical(1)))
}

#' Plot adjustment caption helper (internal)
#' @keywords internal
#' @noRd
.plot_adjustment_caption <- function(tbl, caption = NULL, show_adjustment_note = TRUE) {
  if (!is.null(caption)) {
    return(caption)
  }
  if (!isTRUE(show_adjustment_note)) {
    return(NULL)
  }

  adjust_for <- tbl$adjust_for
  if (!is.null(adjust_for) && length(adjust_for)) {
    return(.adjustment_note(adjust_for))
  }

  if (isTRUE(tbl$multivariable)) {
    return("Adjusted for the other variables in the model")
  }

  NULL
}

#' Plot caption helper (internal)
#' @keywords internal
#' @noRd
.plot_caption <- function(tbl,
                          caption = NULL,
                          show_adjustment_note = TRUE,
                          show_ref = TRUE,
                          has_ref = FALSE) {
  if (!is.null(caption)) {
    return(caption)
  }

  notes <- character(0)
  adjustment_note <- .plot_adjustment_caption(
    tbl,
    caption = NULL,
    show_adjustment_note = show_adjustment_note
  )
  if (!is.null(adjustment_note)) {
    notes <- c(notes, adjustment_note)
  }
  if (isTRUE(show_ref) && isTRUE(has_ref)) {
    notes <- c(notes, .ref_note())
  }

  if (!length(notes)) {
    return(NULL)
  }
  paste(notes, collapse = ". ")
}

#' Compact binary label helper (internal)
#' @keywords internal
#' @noRd
.plot_binary_label <- function(exposure, level) {
  level_chr <- tolower(trimws(as.character(level)))
  affirmative <- level_chr %in% c(
    "yes", "y", "1", "true", "t", "positive", "pos", "present", "exposed"
  )
  if (isTRUE(affirmative)) {
    return(exposure)
  }
  paste0(exposure, ": ", as.character(level))
}

#' Log-scale breaks helper (internal)
#' @keywords internal
#' @noRd
.plot_log_breaks <- function(xlim = NULL) {
  if (is.null(xlim) || length(xlim) != 2L || any(!is.finite(xlim))) {
    return(c(0.25, 0.5, 1, 2, 4, 8, 16))
  }
  # A doubling sequence stays legible in narrow panels. Users can supply
  # `breaks` when clinically meaningful non-standard tick marks are needed.
  candidates <- 2^seq(-4, 7)
  out <- candidates[candidates >= min(xlim) & candidates <= max(xlim)]
  if (length(out) > 5L) {
    targets <- exp(seq(log(min(out)), log(max(out)), length.out = 5L))
    keep <- vapply(targets, function(target) {
      which.min(abs(log(out) - log(target)))
    }, integer(1))
    keep <- unique(keep)

    one_idx <- which(out == 1)
    if (length(one_idx) && !one_idx %in% keep) {
      replace_idx <- which.min(abs(log(out[keep])))
      keep[replace_idx] <- one_idx[1]
    }
    out <- sort(unique(out[keep]))
  }
  if (!length(out)) {
    out <- signif(exp(seq(log(min(xlim)), log(max(xlim)), length.out = 5)), 2)
  }
  out
}

#' Horizontal CI helper (internal)
#' @keywords internal
#' @noRd
.add_h_ci <- function(data, linewidth = 0.55) {
  v <- utils::packageVersion("ggplot2")

  if (v >= "4.0.0") {
    ggplot2::geom_errorbar(
      data = data,
      ggplot2::aes(
        xmin = .data$conf.low,
        xmax = .data$conf.high,
        colour = .data$is_sig
      ),
      orientation = "y",
      width = 0.2,
      linewidth = linewidth,
      na.rm = TRUE
    )
  } else if (v >= "3.5.0") {
    ggplot2::geom_errorbarh(
      data = data,
      ggplot2::aes(
        xmin = .data$conf.low,
        xmax = .data$conf.high,
        colour = .data$is_sig
      ),
      width = 0.2,
      linewidth = linewidth,
      na.rm = TRUE
    )
  } else {
    ggplot2::geom_errorbarh(
      data = data,
      ggplot2::aes(
        xmin = .data$conf.low,
        xmax = .data$conf.high,
        colour = .data$is_sig
      ),
      height = 0.2,
      size = linewidth,
      na.rm = TRUE
    )
  }
}

#' Regression-plot theme (internal)
#' @keywords internal
#' @noRd
.plot_reg_theme <- function(base_size = 12, stratified = FALSE) {
  ggplot2::theme_minimal(base_size = base_size) +
    ggplot2::theme(
      panel.grid.major.y = ggplot2::element_blank(),
      panel.grid.major.x = ggplot2::element_line(colour = "#D9E1E5", linewidth = 0.35),
      panel.grid.minor.x = ggplot2::element_blank(),
      axis.line.x = ggplot2::element_line(colour = "#6B7280", linewidth = 0.35),
      axis.ticks.x = ggplot2::element_line(colour = "#6B7280", linewidth = 0.35),
      axis.text.x = ggplot2::element_text(colour = "#374151", size = ggplot2::rel(0.9)),
      axis.title.x = ggplot2::element_text(face = "bold", colour = "#1F2937",
                                           margin = ggplot2::margin(t = 8)),
      axis.text.y = ggtext::element_markdown(hjust = 0, colour = "#1F2937"),
      axis.text.y.left = ggtext::element_markdown(hjust = 0, colour = "#1F2937"),
      strip.text = if (stratified) ggplot2::element_text(face = "bold",
                                                         colour = "#1F2937")
      else ggplot2::element_blank(),
      plot.title = ggplot2::element_text(face = "bold", colour = "#111827",
                                         margin = ggplot2::margin(b = 8)),
      plot.caption = ggplot2::element_text(colour = "#6B7280", hjust = 0,
                                           size = ggplot2::rel(0.85)),
      plot.margin = ggplot2::margin(12, 40, 12, 10)
    )
}

#' Kaplan-Meier survival plot
#'
#' Plot observed survival over time, with optional confidence intervals,
#' censoring marks, a log-rank p-value, and a number-at-risk table.
#'
#' @param data A \code{data.frame} containing survival time and event status.
#' @param time Survival follow-up time. Quoted and bare names are accepted.
#' @param event Event indicator. Quoted and bare names are accepted. Numeric
#'   \code{0/1}, numeric \code{1/2}, logical, character, and factor variables
#'   are accepted. For two-level character or factor variables, the second level
#'   is treated as the event.
#' @param by Optional grouping variable for separate Kaplan-Meier curves. Quoted
#'   and bare names are accepted.
#' @param conf.int Logical; if \code{TRUE}, show confidence limits.
#' @param risk_table Logical; if \code{TRUE}, add a number-at-risk table below
#'   the curve.
#' @param p_value Logical; if \code{TRUE}, show the log-rank p-value when
#'   \code{by} is supplied.
#' @param p_value_position Optional numeric vector of length 2 giving the
#'   \code{x} and \code{y} coordinates for the log-rank p-value inside the
#'   plotting panel. If \code{NULL}, a lower-left position is chosen
#'   automatically.
#' @param censor Logical; if \code{TRUE}, show censoring marks.
#' @param break_time_by Optional numeric interval for x-axis and risk-table time
#'   breaks. If \code{NULL}, breaks are chosen automatically.
#' @param xlim Optional numeric vector of length 2 specifying x-axis limits.
#' @param ylim Optional numeric vector of length 2 specifying y-axis limits.
#'   Values may be supplied on the survival-probability scale
#'   (for example \code{c(0.5, 1)}) or, when \code{y_percent = TRUE}, on the
#'   percentage scale (for example \code{c(50, 100)}).
#' @param xlab,ylab Axis labels.
#' @param title Optional plot title.
#' @param subtitle Optional plot subtitle.
#' @param caption Optional plot caption.
#' @param title_size Optional numeric title font size. If \code{NULL}, ggplot2's
#'   theme default is used.
#' @param title_face Font face for the title. One of \code{"plain"},
#'   \code{"bold"}, \code{"italic"}, or \code{"bold.italic"}. Quoted and bare
#'   values are accepted.
#' @param legend_title Optional legend title. If \code{NULL}, the labelled
#'   \code{by} variable name is used.
#' @param legend_position Legend position. One of \code{"bottom"}, \code{"top"},
#'   \code{"right"}, \code{"left"}, or \code{"none"}. If \code{NULL}, grouped
#'   plots use \code{"bottom"} and ungrouped plots hide the legend. Quoted and
#'   bare values are accepted.
#' @param palette Optional character vector of colors for grouped curves.
#' @param y_percent Logical; if \code{TRUE}, display survival probability as
#'   percentages. If \code{FALSE}, display the raw 0 to 1 probability scale.
#' @param theme Plot theme. One of \code{"classic"}, \code{"minimal"},
#'   \code{"bw"}, \code{"light"}, or \code{"none"}. Quoted and bare values are
#'   accepted.
#' @param grid Logical; if \code{TRUE}, show major grid lines. The default is
#'   \code{FALSE} for a cleaner publication-style Kaplan-Meier plot.
#' @param base_size Base font size.
#'
#' @return A \code{ggplot2} object when \code{risk_table = FALSE}; otherwise a
#'   \code{patchwork} object combining the survival curve and risk table. The
#'   returned object has attributes \code{fit}, \code{plot_data},
#'   \code{risk_table}, and \code{logrank_p}.
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#'
#' km_plot(
#'   data = lung_data,
#'   time = time,
#'   event = status
#' )
#'
#' km_plot(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   by = trt,
#'   break_time_by = 200,
#'   ylim = c(50, 100),
#'   title = "A. Treatment group",
#'   title_size = 11,
#'   legend_position = "none"
#' )
#'
#' @importFrom survival Surv survfit survdiff
#' @importFrom stats as.formula pchisq complete.cases
#' @importFrom ggplot2 ggplot aes geom_ribbon geom_step geom_point geom_text labs
#' @importFrom ggplot2 theme_minimal theme_classic theme_bw theme_light theme_void theme
#' @importFrom ggplot2 element_blank element_text scale_y_continuous scale_x_continuous
#' @importFrom ggplot2 coord_cartesian scale_color_manual scale_fill_manual guides guide_legend
#' @importFrom patchwork wrap_plots
#' @export
km_plot <- function(data,
                    time,
                    event,
                    by = NULL,
                    conf.int = TRUE,
                    risk_table = TRUE,
                    p_value = TRUE,
                    p_value_position = NULL,
                    censor = TRUE,
                    break_time_by = NULL,
                    xlim = NULL,
                    ylim = NULL,
                    xlab = "Time",
                    ylab = "Survival probability",
                    title = NULL,
                    subtitle = NULL,
                    caption = NULL,
                    title_size = NULL,
                    title_face = "bold",
                    legend_title = NULL,
                    legend_position = NULL,
                    palette = NULL,
                    y_percent = TRUE,
                    theme = "classic",
                    grid = FALSE,
                    base_size = 13) {

  time <- .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  event <- .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  by <- .vars_arg(substitute(by), env = parent.frame(), allow_null = TRUE)
  theme <- .km_theme_arg(substitute(theme), env = parent.frame())
  title_face <- .km_title_face_arg(substitute(title_face), env = parent.frame())
  legend_position <- .km_legend_position_arg(substitute(legend_position), env = parent.frame())

  .validate_km_inputs(
    data = data,
    time = time,
    event = event,
    by = by,
    conf.int = conf.int,
    risk_table = risk_table,
    p_value = p_value,
    p_value_position = p_value_position,
    censor = censor,
    break_time_by = break_time_by,
    xlim = xlim,
    ylim = ylim,
    y_percent = y_percent,
    theme = theme,
    title = title,
    subtitle = subtitle,
    caption = caption,
    title_size = title_size,
    title_face = title_face,
    legend_position = legend_position,
    grid = grid,
    base_size = base_size
  )

  vars_needed <- unique(c(time, event, by))
  data_clean <- data[stats::complete.cases(data[, vars_needed, drop = FALSE]), , drop = FALSE]
  data_clean[[event]] <- .cox_event01(data_clean[[event]])

  if (!is.null(by)) {
    data_clean[[by]] <- factor(data_clean[[by]])
  }

  fml <- .km_formula(time, event, by)
  fit <- survival::survfit(fml, data = data_clean)

  plot_data <- .km_tidy_survfit(fit)
  risk_breaks <- .km_time_breaks(data_clean[[time]], break_time_by, xlim)
  risk_data <- .km_risk_table(fit, risk_breaks)
  logrank_p <- .km_logrank_p(data_clean, time, event, by)
  ylim <- .km_normalize_ylim(ylim, y_percent = y_percent)

  strata_count <- length(unique(plot_data$strata))
  if (is.null(palette)) {
    palette <- .km_default_palette(strata_count)
  }

  by_label <- if (is.null(by)) NULL else .label_var(by, .var_label_map(data, by))
  if (is.null(legend_title)) {
    legend_title <- by_label
  }

  main_plot <- .km_main_plot(
    plot_data = plot_data,
    conf.int = conf.int,
    censor = censor,
    xlim = xlim,
    ylim = ylim,
    breaks = risk_breaks,
    xlab = xlab,
    ylab = ylab,
    title = title,
    subtitle = subtitle,
    caption = caption,
    title_size = title_size,
    title_face = title_face,
    legend_title = legend_title,
    legend_position = legend_position,
    palette = palette,
    y_percent = y_percent,
    theme = theme,
    grid = grid,
    base_size = base_size,
    logrank_p = if (isTRUE(p_value)) logrank_p else NA_real_,
    p_value_position = p_value_position
  )

  out <- main_plot
  if (isTRUE(risk_table)) {
    risk_plot <- .km_risk_plot(
      risk_data = risk_data,
      breaks = risk_breaks,
      xlim = xlim,
      xlab = xlab,
      palette = palette,
      theme = theme,
      grid = grid,
      base_size = base_size
    )
    out <- patchwork::wrap_plots(main_plot, risk_plot, ncol = 1, heights = c(3, 1))
  }

  attr(out, "fit") <- fit
  attr(out, "plot_data") <- plot_data
  attr(out, "risk_table") <- risk_data
  attr(out, "logrank_p") <- logrank_p
  out
}

#' @keywords internal
#' @noRd
.validate_km_inputs <- function(data, time, event, by, conf.int, risk_table,
                                p_value, p_value_position, censor,
                                break_time_by, xlim, ylim, y_percent = TRUE,
                                theme = "classic", title = NULL,
                                subtitle = NULL, caption = NULL,
                                title_size = NULL, title_face = "bold",
                                legend_position = NULL, grid = FALSE,
                                base_size) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }
  if (!is.character(time) || length(time) != 1L || !time %in% names(data)) {
    stop("`time` must be a single survival time variable in `data`.", call. = FALSE)
  }
  if (!is.character(event) || length(event) != 1L || !event %in% names(data)) {
    stop("`event` must be a single event indicator variable in `data`.", call. = FALSE)
  }
  if (!is.null(by) && (!is.character(by) || length(by) != 1L || !by %in% names(data))) {
    stop("`by` must be NULL or a single grouping variable in `data`.", call. = FALSE)
  }
  if (!is.numeric(data[[time]])) {
    stop("`time` must be numeric.", call. = FALSE)
  }
  if (any(data[[time]] < 0, na.rm = TRUE)) {
    stop("`time` must contain non-negative follow-up times.", call. = FALSE)
  }
  for (arg in c("conf.int", "risk_table", "p_value", "censor", "y_percent", "grid")) {
    val <- get(arg)
    if (!is.logical(val) || length(val) != 1L || is.na(val)) {
      stop("`", arg, "` must be TRUE or FALSE.", call. = FALSE)
    }
  }
  if (!is.character(theme) || length(theme) != 1L ||
      !theme %in% c("classic", "minimal", "bw", "light", "none")) {
    stop("`theme` must be one of 'classic', 'minimal', 'bw', 'light', or 'none'.",
         call. = FALSE)
  }
  for (arg in c("title", "subtitle", "caption")) {
    val <- get(arg)
    if (!is.null(val) && (!is.character(val) || length(val) != 1L || is.na(val))) {
      stop("`", arg, "` must be NULL or a single character string.", call. = FALSE)
    }
  }
  if (!is.null(title_size) &&
      (!is.numeric(title_size) || length(title_size) != 1L ||
       is.na(title_size) || title_size <= 0)) {
    stop("`title_size` must be NULL or a positive number.", call. = FALSE)
  }
  if (!is.character(title_face) || length(title_face) != 1L ||
      !title_face %in% c("plain", "bold", "italic", "bold.italic")) {
    stop("`title_face` must be one of 'plain', 'bold', 'italic', or 'bold.italic'.",
         call. = FALSE)
  }
  if (!is.null(legend_position) &&
      (!is.character(legend_position) || length(legend_position) != 1L ||
       !legend_position %in% c("bottom", "top", "right", "left", "none"))) {
    stop("`legend_position` must be NULL or one of 'bottom', 'top', 'right', 'left', or 'none'.",
         call. = FALSE)
  }
  if (!is.null(break_time_by) &&
      (!is.numeric(break_time_by) || length(break_time_by) != 1L ||
       is.na(break_time_by) || break_time_by <= 0)) {
    stop("`break_time_by` must be NULL or a positive number.", call. = FALSE)
  }
  if (!is.null(xlim) &&
      (!is.numeric(xlim) || length(xlim) != 2L || anyNA(xlim) || xlim[1] >= xlim[2])) {
    stop("`xlim` must be NULL or a numeric vector of length 2.", call. = FALSE)
  }
  .km_normalize_ylim(ylim, y_percent = y_percent)
  if (!is.null(p_value_position) &&
      (!is.numeric(p_value_position) || length(p_value_position) != 2L ||
       anyNA(p_value_position) || any(!is.finite(p_value_position)) ||
       p_value_position[2] < 0 || p_value_position[2] > 1)) {
    stop("`p_value_position` must be NULL or a numeric vector `c(x, y)` with y between 0 and 1.",
         call. = FALSE)
  }
  if (!is.numeric(base_size) || length(base_size) != 1L || is.na(base_size) || base_size <= 0) {
    stop("`base_size` must be a positive number.", call. = FALSE)
  }

  vars_needed <- unique(c(time, event, by))
  data_clean <- data[stats::complete.cases(data[, vars_needed, drop = FALSE]), , drop = FALSE]
  if (nrow(data_clean) == 0) {
    stop("No complete cases available for Kaplan-Meier estimation.", call. = FALSE)
  }
  event01 <- .cox_event01(data_clean[[event]])
  if (sum(event01 == 1) == 0) {
    stop("`event` must include at least one event.", call. = FALSE)
  }
  if (!is.null(by) && length(unique(data_clean[[by]])) < 2L) {
    stop("`by` must contain at least two non-missing groups.", call. = FALSE)
  }
  invisible(TRUE)
}

#' @keywords internal
#' @noRd
.km_normalize_ylim <- function(ylim = NULL, y_percent = TRUE) {
  if (is.null(ylim)) {
    return(c(0, 1))
  }
  if (!is.numeric(ylim) || length(ylim) != 2L || anyNA(ylim) ||
      any(!is.finite(ylim)) || ylim[1] >= ylim[2]) {
    stop("`ylim` must be NULL or a numeric vector of length 2.", call. = FALSE)
  }

  out <- ylim
  if (isTRUE(y_percent) && max(out) > 1) {
    out <- out / 100
  }
  if (out[1] < 0 || out[2] > 1) {
    stop("`ylim` must be within 0 to 1, or within 0 to 100 when `y_percent = TRUE`.",
         call. = FALSE)
  }
  out
}

#' @keywords internal
#' @noRd
.km_formula <- function(time, event, by = NULL) {
  bt <- function(x) paste0("`", gsub("`", "", x, fixed = TRUE), "`")
  rhs <- if (is.null(by)) "1" else bt(by)
  stats::as.formula(paste0("survival::Surv(", bt(time), ", ", bt(event), ") ~ ", rhs))
}

#' @keywords internal
#' @noRd
.km_tidy_survfit <- function(fit) {
  sm <- summary(fit, censored = TRUE)
  strata <- if (is.null(sm$strata)) {
    rep("Overall", length(sm$time))
  } else {
    sub("^.*=", "", as.character(sm$strata))
  }

  out <- data.frame(
    time = sm$time,
    n.risk = sm$n.risk,
    n.event = sm$n.event,
    n.censor = sm$n.censor,
    survival = sm$surv,
    conf.low = sm$lower,
    conf.high = sm$upper,
    strata = strata,
    stringsAsFactors = FALSE
  )

  starts <- lapply(unique(out$strata), function(s) {
    data.frame(
      time = 0,
      n.risk = max(out$n.risk[out$strata == s], na.rm = TRUE),
      n.event = 0,
      n.censor = 0,
      survival = 1,
      conf.low = 1,
      conf.high = 1,
      strata = s,
      stringsAsFactors = FALSE
    )
  })
  out <- rbind(do.call(rbind, starts), out)
  out$strata <- factor(out$strata, levels = unique(out$strata))
  out[order(out$strata, out$time), , drop = FALSE]
}

#' @keywords internal
#' @noRd
.km_time_breaks <- function(time, break_time_by = NULL, xlim = NULL) {
  rng <- if (is.null(xlim)) c(0, max(time, na.rm = TRUE)) else xlim
  if (!is.null(break_time_by)) {
    return(seq(rng[1], rng[2], by = break_time_by))
  }
  pretty(rng, n = 5)
}

#' @keywords internal
#' @noRd
.km_risk_table <- function(fit, times) {
  sm <- summary(fit, times = times, extend = TRUE)
  strata <- if (is.null(sm$strata)) {
    rep("Overall", length(sm$time))
  } else {
    sub("^.*=", "", as.character(sm$strata))
  }
  out <- data.frame(
    time = sm$time,
    n.risk = sm$n.risk,
    strata = strata,
    stringsAsFactors = FALSE
  )
  out$strata <- factor(out$strata, levels = unique(out$strata))
  out
}

#' @keywords internal
#' @noRd
.km_logrank_p <- function(data, time, event, by = NULL) {
  if (is.null(by)) {
    return(NA_real_)
  }
  fit <- survival::survdiff(.km_formula(time, event, by), data = data)
  stats::pchisq(fit$chisq, df = length(fit$n) - 1L, lower.tail = FALSE)
}

#' @keywords internal
#' @noRd
.km_fmt_p <- function(p) {
  if (is.na(p)) {
    return(NULL)
  }
  paste0("Log-rank p = ", if (p < 0.001) "<0.001" else formatC(p, format = "f", digits = 3))
}

#' @keywords internal
#' @noRd
.km_p_value_position <- function(plot_data, xlim = NULL, ylim = NULL, position = NULL) {
  if (!is.null(position)) {
    return(position)
  }
  xrng <- if (is.null(xlim)) range(plot_data$time, na.rm = TRUE) else xlim
  yrng <- if (is.null(ylim)) c(0, 1) else ylim
  c(xrng[1] + diff(xrng) * 0.05, yrng[1] + diff(yrng) * 0.08)
}

#' @keywords internal
#' @noRd
.km_default_palette <- function(n) {
  cols <- c("#1F77B4", "#D55E00", "#009E73", "#CC79A7", "#0072B2", "#E69F00")
  rep(cols, length.out = n)
}

#' @keywords internal
#' @noRd
.km_theme_arg <- function(expr, env = parent.frame()) {
  .choice_arg(
    expr,
    choices = c("classic", "minimal", "bw", "light", "none"),
    env = env
  )
}

#' @keywords internal
#' @noRd
.km_title_face_arg <- function(expr, env = parent.frame()) {
  .choice_arg(
    expr,
    choices = c("plain", "bold", "italic", "bold.italic"),
    env = env
  )
}

#' @keywords internal
#' @noRd
.km_legend_position_arg <- function(expr, env = parent.frame()) {
  if (identical(expr, quote(NULL))) {
    return(NULL)
  }
  .choice_arg(
    expr,
    choices = c("bottom", "top", "right", "left", "none"),
    env = env
  )
}

#' @keywords internal
#' @noRd
.km_resolve_legend_position <- function(plot_data, legend_position = NULL) {
  if (!is.null(legend_position)) {
    return(legend_position)
  }
  if (length(unique(plot_data$strata)) > 1L) {
    return("bottom")
  }
  "none"
}

#' @keywords internal
#' @noRd
.km_plot_theme <- function(theme, base_size, grid = FALSE) {
  p_theme <- switch(
    theme,
    classic = ggplot2::theme_classic(base_size = base_size),
    minimal = ggplot2::theme_minimal(base_size = base_size),
    bw = ggplot2::theme_bw(base_size = base_size),
    light = ggplot2::theme_light(base_size = base_size),
    none = ggplot2::theme_void(base_size = base_size)
  )

  if (!isTRUE(grid)) {
    p_theme <- p_theme +
      ggplot2::theme(
        panel.grid.major = ggplot2::element_blank(),
        panel.grid.minor = ggplot2::element_blank()
      )
  }

  p_theme
}

#' @keywords internal
#' @noRd
.km_main_plot <- function(plot_data, conf.int, censor, xlim, ylim, breaks, xlab, ylab,
                          title, subtitle, caption, title_size, title_face,
                          legend_title, legend_position, palette, y_percent, theme, grid,
                          base_size, logrank_p, p_value_position) {
  p <- ggplot2::ggplot(
    plot_data,
    ggplot2::aes(x = .data$time, y = .data$survival, color = .data$strata)
  )

  if (isTRUE(conf.int)) {
    ci_data <- plot_data[
      is.finite(plot_data$conf.low) & is.finite(plot_data$conf.high),
      ,
      drop = FALSE
    ]
    p <- p +
      ggplot2::geom_ribbon(
        data = ci_data,
        ggplot2::aes(ymin = .data$conf.low, ymax = .data$conf.high, fill = .data$strata),
        alpha = 0.18,
        color = NA
      )
  }

  p <- p +
    ggplot2::geom_step(linewidth = 0.9) +
    ggplot2::scale_y_continuous(
      labels = if (isTRUE(y_percent)) scales::percent_format(accuracy = 1) else ggplot2::waiver()
    ) +
    ggplot2::scale_x_continuous(breaks = breaks) +
    ggplot2::labs(
      title = title,
      subtitle = subtitle,
      caption = caption,
      x = xlab,
      y = ylab,
      color = legend_title
    ) +
    .km_plot_theme(theme, base_size = base_size, grid = grid) +
    ggplot2::theme(
      legend.position = .km_resolve_legend_position(plot_data, legend_position),
      plot.title = ggplot2::element_text(face = title_face, size = title_size)
    )

  if (isTRUE(censor)) {
    cens <- plot_data[plot_data$n.censor > 0, , drop = FALSE]
    if (nrow(cens)) {
      p <- p + ggplot2::geom_point(data = cens, shape = 3, size = 2, stroke = 0.8)
    }
  }

  p_label <- .km_fmt_p(logrank_p)
  if (!is.null(p_label)) {
    p_pos <- .km_p_value_position(
      plot_data,
      xlim = xlim,
      ylim = ylim,
      position = p_value_position
    )
    p <- p +
      ggplot2::annotate(
        "text",
        x = p_pos[1],
        y = p_pos[2],
        label = p_label,
        hjust = 0,
        vjust = 0,
        size = base_size / 3.4,
        fontface = "plain",
        color = "black"
      )
  }

  if (!is.null(palette)) {
    levs <- levels(plot_data$strata)
    pal <- stats::setNames(palette[seq_along(levs)], levs)
    p <- p +
      ggplot2::scale_color_manual(values = pal) +
      ggplot2::scale_fill_manual(values = pal, guide = "none")
  }

  p <- p + ggplot2::coord_cartesian(xlim = xlim, ylim = ylim)

  p
}

#' @keywords internal
#' @noRd
.km_risk_plot <- function(risk_data, breaks, xlim, xlab, palette, theme, grid, base_size) {
  p <- ggplot2::ggplot(
    risk_data,
    ggplot2::aes(x = .data$time, y = .data$strata, label = .data$n.risk,
                 color = .data$strata)
  ) +
    ggplot2::geom_text(size = base_size / 4) +
    ggplot2::scale_x_continuous(breaks = breaks) +
    ggplot2::labs(x = xlab, y = "Number at risk") +
    .km_plot_theme(theme, base_size = base_size, grid = grid) +
    ggplot2::theme(
      legend.position = "none",
      panel.grid.major.y = ggplot2::element_blank(),
      panel.grid.minor = ggplot2::element_blank(),
      axis.title.y = ggplot2::element_text(face = "bold")
    )

  if (!is.null(palette)) {
    levs <- levels(risk_data$strata)
    p <- p + ggplot2::scale_color_manual(values = stats::setNames(palette[seq_along(levs)], levs))
  }
  if (!is.null(xlim)) {
    p <- p + ggplot2::coord_cartesian(xlim = xlim)
  }
  p
}

lung_km_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )
}

test_that("km_plot returns an ungrouped ggplot when risk table is hidden", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")

  df <- lung_km_data()

  res <- km_plot(
    data = df,
    time = time,
    event = status,
    risk_table = FALSE
  )

  expect_s3_class(res, "ggplot")
  expect_s3_class(attr(res, "fit"), "survfit")
  expect_s3_class(attr(res, "plot_data"), "data.frame")
  expect_s3_class(attr(res, "risk_table"), "data.frame")
  expect_true(all(c("time", "survival", "conf.low", "conf.high", "strata") %in%
                    names(attr(res, "plot_data"))))
  expect_equal(unique(as.character(attr(res, "plot_data")$strata)), "Overall")
  expect_true(is.na(attr(res, "logrank_p")))
})

test_that("km_plot returns grouped curves with risk table and log-rank p-value", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("patchwork")

  df <- lung_km_data()
  attr(df$trt, "label") <- "Treatment group"

  res <- km_plot(
    data = df,
    time = "time",
    event = "status",
    by = "trt",
    break_time_by = 200,
    title = "Kaplan-Meier survival by treatment"
  )

  expect_s3_class(res, "patchwork")
  expect_s3_class(attr(res, "fit"), "survfit")
  expect_true(is.finite(attr(res, "logrank_p")))
  expect_true(all(c("Standard treatment", "Test treatment") %in%
                    unique(as.character(attr(res, "plot_data")$strata))))
  expect_true(all(c(0, 200, 400) %in% attr(res, "risk_table")$time))
})

test_that("km_plot builds without dropping confidence interval rows", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")

  df <- lung_km_data()

  res <- km_plot(
    data = df,
    time = time,
    event = status,
    by = trt,
    risk_table = FALSE,
    break_time_by = 200
  )

  expect_warning(ggplot2::ggplot_build(res), NA)
})

test_that("km_plot supports y-axis limits and shaded confidence intervals", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")

  df <- lung_km_data()

  res <- km_plot(
    data = df,
    time = time,
    event = status,
    by = trt,
    risk_table = FALSE,
    ylim = c(50, 100),
    conf.int = TRUE
  )

  expect_s3_class(res, "ggplot")
  expect_equal(.km_normalize_ylim(c(50, 100), y_percent = TRUE), c(0.5, 1))
  expect_equal(.km_normalize_ylim(c(0.5, 1), y_percent = FALSE), c(0.5, 1))
  expect_true(any(vapply(
    res$layers,
    function(layer) inherits(layer$geom, "GeomRibbon"),
    logical(1)
  )))
  expect_warning(ggplot2::ggplot_build(res), NA)
})

test_that("km_plot draws log-rank p-value inside the graph", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")

  df <- lung_km_data()

  res <- km_plot(
    data = df,
    time = time,
    event = status,
    by = trt,
    risk_table = FALSE,
    p_value_position = c(100, 0.12)
  )

  last_layer <- ggplot2::layer_data(res, length(res$layers))
  expect_true(any(grepl("Log-rank p", last_layer$label, fixed = TRUE)))
  expect_equal(last_layer$x[1], 100)
  expect_equal(last_layer$y[1], 0.12)
})

test_that("km_plot supports clean themes and raw probability y-axis", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")

  df <- lung_km_data()

  res <- km_plot(
    data = df,
    time = time,
    event = status,
    risk_table = FALSE,
    y_percent = FALSE,
    theme = classic
  )

  expect_s3_class(res, "ggplot")
  expect_s3_class(res$theme$panel.grid.major, "element_blank")
  expect_s3_class(res$theme$panel.grid.minor, "element_blank")

  res_grid <- km_plot(
    data = df,
    time = time,
    event = status,
    risk_table = FALSE,
    theme = "minimal",
    grid = TRUE
  )

  expect_s3_class(res_grid, "ggplot")
})

test_that("km_plot supports publication panel styling controls", {
  skip_if_not_installed("survival")
  skip_if_not_installed("ggplot2")

  df <- lung_km_data()

  res <- km_plot(
    data = df,
    time = time,
    event = status,
    by = trt,
    risk_table = FALSE,
    title = "A. Treatment group",
    subtitle = "Lung cancer trial",
    caption = "Log-rank test shown inside the panel.",
    title_size = 10,
    title_face = plain,
    legend_position = none
  )

  expect_s3_class(res, "ggplot")
  expect_equal(res$labels$title, "A. Treatment group")
  expect_equal(res$labels$subtitle, "Lung cancer trial")
  expect_equal(res$labels$caption, "Log-rank test shown inside the panel.")
  expect_equal(res$theme$plot.title$size, 10)
  expect_equal(res$theme$plot.title$face, "plain")
  expect_equal(res$theme$legend.position, "none")
})

test_that("km_plot validates inputs", {
  skip_if_not_installed("survival")

  df <- lung_km_data()
  df$bad_event <- 0
  df$one_group <- "A"

  expect_error(
    km_plot(data = "not data", time = time, event = status),
    "`data` must be"
  )
  expect_error(
    km_plot(data = df, time = missing_time, event = status),
    "`time` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = missing_event),
    "`event` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, by = one_group),
    "`by` must contain at least two"
  )
  expect_error(
    km_plot(data = df, time = time, event = bad_event),
    "`event` must include at least one event"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, break_time_by = 0),
    "`break_time_by` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, xlim = c(10, 1)),
    "`xlim` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, ylim = c(100, 50)),
    "`ylim` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, ylim = c(-10, 100)),
    "`ylim` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, p_value_position = c(10, 2)),
    "`p_value_position` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, title = c("A", "B")),
    "`title` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, title_size = 0),
    "`title_size` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, title_face = heavy),
    "`title_face` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, legend_position = middle),
    "`legend_position` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, risk_table = NA),
    "`risk_table` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, y_percent = NA),
    "`y_percent` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, grid = NA),
    "`grid` must be"
  )
  expect_error(
    km_plot(data = df, time = time, event = status, theme = ugly),
    "`theme` must be"
  )
})

#' @keywords internal
#' @noRd
.theme_presets <- list(
  minimal  = c("plain","lines","labels_bold"),
  clinical = c("plain","labels_bold","compact"),
  striped  = c("zebra","labels_bold","compact"),
  shaded   = c("header_shaded","labels_bold","lines"),
  # aliases ("inspired by", not replicas)
  jama     = c("plain","lines","labels_bold","compact")
)
#' @keywords internal
#' @noRd
.resolve_theme <- function(theme) {
  if (length(theme) == 1 && !is.na(theme) && theme %in% names(.theme_presets))
    return(.theme_presets[[theme]])
  unique(tolower(theme))
}

#' Plot mediation paths
#'
#' Draw the exposure, mediator, and outcome path from a
#' \code{mediation_analysis()} result.
#'
#' @param mediation_object Object returned by \code{mediation_analysis()}.
#' @param show_estimates Logical; if \code{TRUE}, show direct and indirect
#'   effect estimates on the plot.
#' @param base_size Base font size.
#'
#' @return A \code{ggplot2} object.
#' @importFrom rlang .data
#'
#' @examples
#' med <- mediation_analysis(
#'   data = data_diabetes_mediation,
#'   exposure = obesity,
#'   mediator = glucose,
#'   outcome = diabetes,
#'   covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
#'   outcome_approach = logit,
#'   sims = 50,
#'   seed = 123
#' )
#' plot_mediation(med)
#' plot_mediation(med, show_estimates = FALSE)
#'
#' @export
plot_mediation <- function(mediation_object,
                           show_estimates = TRUE,
                           base_size = 13) {
  if (!inherits(mediation_object, "mediation_analysis")) {
    stop("`mediation_object` must be returned by `mediation_analysis()`.", call. = FALSE)
  }
  if (!is.logical(show_estimates) || length(show_estimates) != 1L || is.na(show_estimates)) {
    stop("`show_estimates` must be TRUE or FALSE.", call. = FALSE)
  }
  if (!is.numeric(base_size) || length(base_size) != 1L || is.na(base_size) || base_size <= 0) {
    stop("`base_size` must be a single positive number.", call. = FALSE)
  }

  labels <- mediation_object$variable_labels
  if (is.null(labels)) {
    labels <- .var_label_map(
      mediation_object$complete_data,
      c(mediation_object$exposure, mediation_object$mediator, mediation_object$outcome)
    )
  }
  exposure_lab <- .label_var(mediation_object$exposure, labels)
  mediator_lab <- .label_var(mediation_object$mediator, labels)
  outcome_lab <- .label_var(mediation_object$outcome, labels)

  nodes <- data.frame(
    node = c("Exposure", "Mediator", "Outcome"),
    label = c(exposure_lab, mediator_lab, outcome_lab),
    x = c(0, 1, 2),
    y = c(0, 0.7, 0),
    stringsAsFactors = FALSE
  )
  paths <- data.frame(
    x = c(0.18, 1.18, 0.18),
    y = c(0.08, 0.62, -0.06),
    xend = c(0.82, 1.82, 1.82),
    yend = c(0.62, 0.08, -0.06),
    path = c("Exposure -> mediator", "Mediator -> outcome", "Direct path"),
    stringsAsFactors = FALSE
  )

  effects <- mediation_object$table_body
  direct <- effects$estimate[effects$effect == "direct"]
  indirect <- effects$estimate[effects$effect == "indirect"]
  total <- effects$estimate[effects$effect == "total"]

  ann <- data.frame(
    x = c(0.5, 1.5, 1.0),
    y = c(0.5, 0.5, -0.18),
    label = c(
      if (show_estimates) paste0("Indirect path\npart 1") else "",
      if (show_estimates) paste0("Indirect path\npart 2") else "",
      if (show_estimates) paste0("Direct = ", formatC(direct, digits = 3, format = "f"),
                                 "\nIndirect = ", formatC(indirect, digits = 3, format = "f"),
                                 "\nTotal = ", formatC(total, digits = 3, format = "f")) else ""
    ),
    stringsAsFactors = FALSE
  )

  ggplot2::ggplot() +
    ggplot2::geom_segment(
      data = paths,
      ggplot2::aes(
        x = .data$x,
        y = .data$y,
        xend = .data$xend,
        yend = .data$yend
      ),
      arrow = grid::arrow(length = grid::unit(0.22, "cm")),
      linewidth = 0.8,
      lineend = "round",
      color = "#2B5C8A"
    ) +
    ggplot2::geom_label(
      data = nodes,
      ggplot2::aes(x = .data$x, y = .data$y, label = .data$label),
      size = base_size / 4,
      linewidth = 0.35,
      fill = "white",
      color = "#222222"
    ) +
    ggplot2::geom_text(
      data = ann,
      ggplot2::aes(x = .data$x, y = .data$y, label = .data$label),
      size = base_size / 5,
      color = "#444444",
      lineheight = 0.9
    ) +
    ggplot2::coord_cartesian(xlim = c(-0.25, 2.25), ylim = c(-0.42, 0.98), expand = FALSE) +
    ggplot2::theme_void(base_size = base_size) +
    ggplot2::labs(
      title = "Mediation path diagram",
      subtitle = "Interpret causally only when mediation assumptions are justified"
    ) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold"),
      plot.subtitle = ggplot2::element_text(color = "#555555")
    )
}

test_that("plot_mediation returns a ggplot object", {
  med <- mediation_analysis(
    data = data_diabetes_mediation,
    exposure = obesity,
    mediator = glucose,
    outcome = diabetes,
    covariates = c(age, blood_pressure),
    outcome_approach = logit,
    sims = 20,
    seed = 123
  )

  p <- plot_mediation(med)
  expect_s3_class(p, "ggplot")
  expect_equal(p$layers[[2]]$data$label, c("Obesity", "Plasma glucose", "Diabetes"))

  p_no_estimates <- plot_mediation(med, show_estimates = FALSE, base_size = 11)
  expect_s3_class(p_no_estimates, "ggplot")
})

test_that("plot_mediation validates inputs", {
  expect_error(plot_mediation(list()), "`mediation_object` must be returned", fixed = TRUE)

  med <- mediation_analysis(
    data = data_diabetes_mediation,
    exposure = obesity,
    mediator = glucose,
    outcome = diabetes,
    outcome_approach = logit,
    sims = 20,
    seed = 123
  )

  expect_error(plot_mediation(med, show_estimates = NA), "`show_estimates`", fixed = TRUE)
  expect_error(plot_mediation(med, base_size = 0), "`base_size`", fixed = TRUE)
})

# R/accessors.R

#' Access fields on gtregression objects with `$`
#'
#' Works for any object from this package, since they all carry class
#' `"gtregression"`. Returns NULL (quietly) if the field is not present.
#'
#' Common fields:
#' - table, table_display, table_body
#' - models, model_summaries, reg_check
#' - approach, format (or engine), source
#' - parts, spanners (for merged tables)
#' - by, levels (for descriptive tables)
#'
#' @param x A \code{gtregression} object.
#' @param name Field name to access.
#'
#' @export
`$.gtregression` <- function(x, name) {
  # avoid recursive calls by using [[ ]] only
  fmt <- x[["format"]]
  eng <- x[["engine"]]

  # aliases
  if (identical(name, "engine") && !is.null(fmt)) {
    return(fmt)
  }
  if (identical(name, "format") && !is.null(eng)) {
    return(eng)
  }

  # ordinary field access
  x[[name]]
}

#' Lung Cancer Trial Data
#'
#' Survival data from a clinical trial of lung cancer patients conducted
#'  by the Veteran's Administration.
#'
#' @format A data frame with 137 observations and 8 variables:
#' \describe{
#'   \item{trt}{Treatment group (1 = standard, 2 = test)}
#'   \item{celltype}{Cell type (squamous, smallcell, adeno, large)}
#'   \item{time}{Survival time (in days)}
#'   \item{status}{Censoring status (1 = died, 0 = censored)}
#'   \item{karno}{Karnofsky performance score (higher = better)}
#'   \item{diagtime}{Months from diagnosis to randomization}
#'   \item{age}{Age in years}
#'   \item{prior}{Prior therapy (0 = no, 10 = yes)}
#' }
#' @source \url{https://CRAN.R-project.org/package=survival}
#' @references Kalbfleisch JD and Prentice RL (1980).
#'  The Statistical Analysis of Failure Time Data.
"data_lungcancer"

#' Endometrial Cancer Histology Grade Data
#'
#' A classic endometrial cancer dataset used to demonstrate separation in
#' logistic regression. The outcome is high histology grade. Neovascularization
#' is completely absent among low-grade cases in this dataset, making it useful
#' for teaching Firth penalized logistic regression.
#'
#' @format A data frame with 79 observations and 4 variables:
#' \describe{
#'   \item{NV}{Neovascularization status (0 = absent, 1 = present)}
#'   \item{PI}{Pulsatility index of the uterine artery}
#'   \item{EH}{Endometrium height}
#'   \item{HG}{Histology grade (0 = low grade, 1 = high grade)}
#' }
#'
#' @source \pkg{brglm2} package. The packaged dataset was sourced from
#' \url{https://users.stat.ufl.edu/~aa/glm/data/}, the data repository used in
#' Agresti (2015). Originally analyzed in Heinze and Schemper (2002).
#' @references
#' Agresti A (2015). \emph{Foundations of Linear and Generalized Linear Models}.
#' Wiley.
#'
#' Heinze G, Schemper M (2002). A solution to the problem of separation in
#' logistic regression. \emph{Statistics in Medicine}, 21, 2409-2419.
#' \doi{10.1002/sim.1047}
"data_endometrial"

#' Diabetes Mediation Teaching Dataset
#'
#' A health-related teaching dataset prepared from synthetic diabetes data.
#' It is designed for the practical question: could part of the obesity
#' and diabetes relationship operate through plasma glucose, after adjustment
#' for basic clinical covariates?
#'
#' @format A data frame with variables:
#' \describe{
#'   \item{diabetes}{Diabetes status (\code{No}, \code{Yes})}
#'   \item{obesity}{Obesity status based on BMI >= 30 (\code{No}, \code{Yes})}
#'   \item{glucose}{Plasma glucose concentration}
#'   \item{bmi}{Body mass index}
#'   \item{age}{Age in years}
#'   \item{blood_pressure}{Diastolic blood pressure}
#'   \item{pregnancies}{Number of pregnancies}
#'   \item{diabetes_pedigree}{Diabetes pedigree function}
#' }
#'
#' @details
#' This dataset is intended for practice and teaching. Mediation estimates from
#' observational data should be interpreted causally only when the temporal
#' ordering and no-unmeasured-confounding assumptions are justified by study
#' design, DAGs, and subject-matter knowledge.
#'
#' @source Derived from \code{data_SynthDiabetes}, which is based on
#' \code{mlbench::SynthDiabetes2} (\pkg{mlbench} version 2.1-11 or later).
"data_diabetes_mediation"

#' Infertility Matched Case-Control Study
#'
#' investigating the relationship between infertility and abortions.
#'
#' @format A data frame with 248 observations and 8 variables:
#' \describe{
#'   \item{education}{Education level (0 = 0–5 years, 1 = 6–11 years,
#'   2 = 12+ years)}
#'   \item{age}{Age in years}
#'   \item{parity}{Number of prior pregnancies}
#'   \item{induced}{Number of induced abortions}
#'   \item{case}{Infertility case status (1 = case, 0 = control)}
#'   \item{spontaneous}{Number of spontaneous abortions}
#'   \item{stratum}{Matched set ID}
#'   \item{pooled.stratum}{Pooled stratum ID used for conditional regression}
#' }
#' @source
#' \url{https://stat.ethz.ch/R-manual/R-devel/library/datasets/html/infert.html}
"data_infertility"


#' Epilepsy Treatment and Seizure Counts
#'
#' RCT on the effect of a drug on the seizures in patients with epilepsy.
#' Contains repeated measures data with treatment groups,
#' baseline seizure counts, and follow-up counts.
#'
#' @format A data frame with 236 observations and 9 variables:
#' \describe{
#'   \item{y}{Number of seizures in a 2-week period (count)}
#'   \item{trt}{Treatment group (factor): \code{placebo} or \code{progabide}}
#'   \item{base}{Seizure count during baseline period (numeric)}
#'   \item{age}{Age of patient (numeric)}
#'   \item{V4}{Indicator for 4th visit (binary)}
#'   \item{subject}{Patient ID (factor)}
#'   \item{period}{Follow-up period number (integer)}
#'   \item{lbase}{Log of baseline seizures (numeric)}
#'   \item{lage}{Log of age (numeric)}
#' }
#'
#' @source \pkg{MASS} package. Original data from Thall and Vail (1990)
"data_epilepsy"


#' Birth Weight Data
#'
#' A dataset from the \pkg{MASS} package containing risk factors
#' associated with low birth weight (LBW) in newborns.
#' Originally collected at Baystate Medical Center,
#' Springfield, Massachusetts, USA.
#'
#' The outcome variable is binary (`low`): birth weight < 2500g (yes = 1)
#' or not (no = 0).
#'
#' @format A data frame with 189 observations and 10 variables:
#' \describe{
#'   \item{low}{Indicator for birth weight < 2500g (binary):
#'   \code{0 = normal}, \code{1 = low birth weight}}
#'   \item{age}{Mother's age in years (numeric)}
#'   \item{lwt}{Mother's weight in pounds at last menstrual period (numeric)}
#'   \item{race}{Mother's race (factor): \code{1 = White}, \code{2 = Black},
#'   \code{3 = Other}}
#'   \item{smoke}{Smoking status during pregnancy (binary): \code{0 = No},
#'    \code{1 = Yes}}
#'   \item{ptl}{Number of previous premature labors (integer)}
#'   \item{ht}{History of hypertension (binary): \code{0 = No}, \code{1 = Yes}}
#'   \item{ui}{Presence of uterine irritability (binary): \code{0 = No},
#'   \code{1 = Yes}}
#'   \item{ftv}{no of physician visits during the 1st trimester (integer, 0–6)}
#'   \item{bwt}{Birth weight in grams (numeric)}
#' }
#'
#' @source Hosmer, D.W., Lemeshow, S. (1989). *Applied Logistic Regression.*
#' New York: Wiley.
#' Also available in \pkg{MASS} and described in detail in its documentation.
"data_birthwt"


#' Student Absenteeism in Rural Schools
#'
#' This dataset contains observations on the number of days absent from school
#' for children in rural Australia,
#' along with student characteristics. It's commonly used to demonstrate count
#' models such as Poisson and Negative Binomial regression.
#'
#' @format A data frame with 146 observations and 5 variables:
#' \describe{
#'   \item{Eth}{Ethnicity (\code{"A"} = Aboriginal,
#'   \code{"N"} = Non-Aboriginal)}
#'   \item{Sex}{Sex (\code{"F"} or \code{"M"})}
#'   \item{Age}{Age group (\code{"F0", "F1", "F2", "F3"})}
#'   \item{Lrn}{Learner status (\code{"AL"} = average learner,
#'    \code{"SL"} = slow learner)}
#'   \item{Days}{Number of days absent from school (count outcome)}
#' }
#'
#' @source \pkg{MASS} package. See also Venables and Ripley (2002),
#'  *Modern Applied Statistics with S*.
#'
"data_gt_quin"

#' Synthetic Diabetes Dataset
#'
#' A synthetic replacement for the former diabetes dataset from the
#' \pkg{mlbench} package. It has the same structure and number of observations
#' but does not represent real people.
#' Useful for demonstrating regression approaches for binary outcomes.
#'
#' @format A data frame with 768 observations and 9 variables:
#' \describe{
#'   \item{pregnant}{Number of times pregnant}
#'   \item{glucose}{Plasma glucose concentration (glucose tolerance test)}
#'   \item{pressure}{Diastolic blood pressure (mm Hg)}
#'   \item{triceps}{Triceps skin fold thickness (mm)}
#'   \item{insulin}{2-Hour serum insulin (mu U/ml)}
#'   \item{mass}{Body mass index (BMI)}
#'   \item{pedigree}{Diabetes pedigree function}
#'   \item{age}{Age in years}
#'   \item{diabetes}{Factor indicating diabetes status (pos/neg)}
#' }
#'
#' @source \code{mlbench::SynthDiabetes2}, available in \pkg{mlbench}
#' version 2.1-11 or later.
"data_SynthDiabetes"

#' @importFrom stats AIC BIC anova as.formula binomial coef cooks.distance
#' @importFrom stats deviance glm glm.control lm logLik na.omit nobs poisson
#' @importFrom stats predict residuals shapiro.test
#' @importFrom utils globalVariables
#' @importFrom dplyr bind_rows case_when filter mutate
#' @importFrom tibble tibble
#' @importFrom broom.helpers tidy_plus_plus tidy_add_reference_rows
#' @importFrom utils head

# Prevent global variable warnings
utils::globalVariables(c(
  ".data",
  "reference_row", "variable", "label_clean", "header_order", "row_id", "label",
  "estimate", "conf.low", "conf.high", "approach", "stat_n", "row_type", "level",
  "var_type", "p.value", "type", "ref"
))

#' Normalize quoted or bare option arguments
#'
#' @keywords internal
#' @noRd
.choice_arg <- function(expr,
                        env = parent.frame(),
                        choices = NULL,
                        aliases = NULL,
                        lower = TRUE) {
  normalize_one <- function(x) {
    if (is.character(x)) {
      out <- if (lower) tolower(x) else x
      if (!is.null(aliases)) {
        idx <- match(out, names(aliases))
        out[!is.na(idx)] <- unname(aliases[idx[!is.na(idx)]])
      }
      return(out)
    }

    if (is.symbol(x)) {
      name <- as.character(x)
      key <- if (lower) tolower(name) else name

      if (!is.null(aliases) && key %in% names(aliases)) {
        return(unname(aliases[[key]]))
      }

      choice_keys <- if (lower) tolower(choices) else choices
      if (!is.null(choices) && key %in% choice_keys) {
        return(if (lower) key else name)
      }

      if (exists(name, envir = env, inherits = TRUE)) {
        return(get(name, envir = env, inherits = TRUE))
      }

      return(if (lower) key else name)
    }

    if (is.call(x) && identical(x[[1L]], as.name("c"))) {
      return(unlist(lapply(as.list(x)[-1L], normalize_one), use.names = FALSE))
    }

    eval(x, envir = env)
  }

  out <- normalize_one(expr)
  if (is.character(out) && lower) {
    out <- tolower(out)
  }
  out
}

#' Normalize quoted or bare variable-name arguments
#'
#' @keywords internal
#' @noRd
.vars_arg <- function(expr,
                      env = parent.frame(),
                      allow_null = FALSE,
                      data_names = NULL) {
  if (identical(expr, quote(NULL))) {
    if (allow_null) {
      return(NULL)
    }
    stop("Variable names cannot be NULL.", call. = FALSE)
  }
  normalize_variable <- function(x) {
    if (is.symbol(x)) {
      name <- as.character(x)
      # A data-column name takes precedence over an unrelated object such as
      # the built-in `pressure` dataset.
      if (!is.null(data_names) && name %in% data_names) {
        return(name)
      }
    }

    if (is.call(x) && identical(x[[1L]], as.name("c"))) {
      return(unlist(
        lapply(as.list(x)[-1L], normalize_variable),
        use.names = FALSE
      ))
    }

    .choice_arg(x, env = env, lower = FALSE)
  }

  out <- normalize_variable(expr)
  if (is.null(out) && allow_null) {
    return(NULL)
  }
  out
}

#' Normalize quoted or bare interaction arguments
#'
#' @keywords internal
#' @noRd
.interaction_arg <- function(expr, env = parent.frame(), allow_null = TRUE) {
  if (identical(expr, quote(NULL))) {
    if (allow_null) {
      return(NULL)
    }
    stop("`interaction` cannot be NULL.", call. = FALSE)
  }

  if (is.character(expr)) {
    out <- expr
  } else if (is.symbol(expr)) {
    nm <- as.character(expr)
    if (exists(nm, envir = env, inherits = TRUE)) {
      out <- get(nm, envir = env, inherits = TRUE)
    } else {
      out <- nm
    }
  } else if (is.call(expr)) {
    if (as.character(expr[[1L]]) %in% c("*", ":")) {
      out <- paste(gsub("`", "", vapply(as.list(expr)[-1L], deparse, character(1)), fixed = TRUE),
                   collapse = as.character(expr[[1L]]))
    } else {
      out <- eval(expr, envir = env)
    }
  } else {
    out <- eval(expr, envir = env)
  }

  if (is.null(out) && allow_null) {
    return(NULL)
  }
  if (!is.character(out) || length(out) != 1L || is.na(out) || !nzchar(out)) {
    stop("`interaction` must be a single character string such as 'bmi*sex'.", call. = FALSE)
  }
  gsub("\\s+", "", out)
}

#' Normalize regression approach spellings
#'
#' @keywords internal
#' @noRd
.normalize_approach <- function(approach) {
  approach <- tolower(as.character(approach))
  approach[approach == "log-binomial"] <- "logbinomial"
  approach[approach %in% c("surv", "survival", "surv_reg")] <- "survreg"
  approach
}

#' Normalize parametric survival distribution arguments
#'
#' @keywords internal
#' @noRd
.surv_distribution_arg <- function(expr,
                                   env = parent.frame(),
                                   multiple = FALSE,
                                   arg = "distribution") {
  choices <- c("weibull", "exponential", "lognormal", "loglogistic")
  aliases <- c(
    "exp" = "exponential",
    "log-normal" = "lognormal",
    "log normal" = "lognormal",
    "log_normal" = "lognormal",
    "log-logistic" = "loglogistic",
    "log logistic" = "loglogistic",
    "log_logistic" = "loglogistic"
  )

  out <- .choice_arg(
    expr,
    env = env,
    choices = choices,
    aliases = aliases
  )

  if (!is.character(out) || anyNA(out) || any(!out %in% choices) ||
      length(out) < 1L || (!multiple && length(out) != 1L)) {
    if (identical(arg, "distributions")) {
      stop(
        "`distributions` must contain one or more of: ",
        paste(choices, collapse = ", "),
        call. = FALSE
      )
    }

    stop(
      "Invalid distribution. Choose one of: ",
      paste(choices, collapse = ", "),
      call. = FALSE
    )
  }

  out
}

# -------------------------------------------------------------------
# Saving and export utilities
# -------------------------------------------------------------------

#' Resolve a table object for saving
#' @keywords internal
#' @noRd
.resolve_table_object <- function(tbl) {
  if (inherits(tbl, c("gtregression", "merged_table")) && !is.null(tbl[["table"]])) {
    return(tbl[["table"]])
  }

  if (inherits(tbl, "gt_tbl") || inherits(tbl, "flextable")) {
    return(tbl)
  }

  stop(
    "`tbl` must be a gtregression object, merged_table object, gt_tbl, or flextable.",
    call. = FALSE
  )
}

#' Check whether an object can be exported as a table
#' @keywords internal
#' @noRd
.is_save_table_like <- function(x) {
  inherits(x, c("gtregression", "merged_table", "gt_tbl", "flextable"))
}

#' Normalize output filename
#' @keywords internal
#' @noRd
.normalize_save_path <- function(filename, ext) {
  if (!is.character(filename) || length(filename) != 1L ||
      is.na(filename) || !nzchar(filename)) {
    stop("`filename` must be a single non-empty character string.", call. = FALSE)
  }
  if (!is.character(ext) || length(ext) != 1L ||
      is.na(ext) || !nzchar(ext)) {
    stop("`ext` must be a single non-empty character string.", call. = FALSE)
  }

  if (!grepl(paste0("\\.", ext, "$"), filename, ignore.case = TRUE)) {
    filename <- paste0(filename, ".", ext)
  }

  has_path <- grepl("/", filename, fixed = TRUE) || grepl("\\\\", filename)
  if (!has_path) {
    filename <- file.path(tempdir(), filename)
  }

  filename
}

#' Validate DOCX table sizing options
#' @keywords internal
#' @noRd
.validate_docx_table_sizing <- function(fit_width = TRUE,
                                        font_size = 9,
                                        min_font_size = 8) {
  if (!is.logical(fit_width) || length(fit_width) != 1L || is.na(fit_width)) {
    stop("`fit_width` must be TRUE or FALSE.", call. = FALSE)
  }
  if (!is.numeric(font_size) || length(font_size) != 1L ||
      is.na(font_size) || font_size <= 0) {
    stop("`font_size` must be a single positive number.", call. = FALSE)
  }
  if (!is.numeric(min_font_size) || length(min_font_size) != 1L ||
      is.na(min_font_size) || min_font_size <= 0) {
    stop("`min_font_size` must be a single positive number.", call. = FALSE)
  }
  if (min_font_size > font_size) {
    stop("`min_font_size` must be less than or equal to `font_size`.", call. = FALSE)
  }

  invisible(TRUE)
}

#' Measure a flextable width in inches
#' @keywords internal
#' @noRd
.flextable_col_widths <- function(ft) {
  widths <- ft$body$colwidths
  if (is.null(widths)) {
    return(NULL)
  }
  widths
}

#' Measure a flextable width in inches
#' @keywords internal
#' @noRd
.flextable_width <- function(ft) {
  widths <- .flextable_col_widths(ft)
  if (is.null(widths)) {
    dims <- tryCatch(flextable::flextable_dim(ft), error = function(e) NULL)
    if (is.null(dims) || is.null(dims$widths)) {
      return(NA_real_)
    }
    width <- sum(dims$widths, na.rm = TRUE)
  } else {
    width <- sum(widths, na.rm = TRUE)
  }
  if (!is.finite(width) || width <= 0) NA_real_ else width
}

#' Scale flextable columns to a target width without reducing font size
#' @keywords internal
#' @noRd
.scale_flextable_width <- function(ft, table_width) {
  widths <- .flextable_col_widths(ft)
  if (is.null(widths)) {
    return(ft)
  }

  current_width <- sum(widths, na.rm = TRUE)
  if (!is.finite(current_width) || current_width <= 0 ||
      current_width <= table_width) {
    return(ft)
  }

  scaled <- widths * (table_width / current_width)
  col_keys <- names(scaled)
  if (is.null(col_keys) || any(!nzchar(col_keys))) {
    col_keys <- ft$col_keys
  }
  if (is.null(col_keys) || length(col_keys) != length(scaled)) {
    return(ft)
  }

  out <- ft
  for (i in seq_along(scaled)) {
    out <- tryCatch(
      flextable::width(out, j = col_keys[[i]], width = scaled[[i]]),
      error = function(e) out
    )
  }
  out
}

#' Fit flextable output to a Word page width without unsafe font shrinking
#' @keywords internal
#' @noRd
.fit_flextable_docx_width <- function(ft,
                                      table_width = 6.5,
                                      fit_width = TRUE,
                                      font_size = 9,
                                      min_font_size = 8,
                                      warn = TRUE) {
  .validate_docx_table_sizing(
    fit_width = fit_width,
    font_size = font_size,
    min_font_size = min_font_size
  )
  if (is.null(table_width)) {
    return(flextable::autofit(flextable::fontsize(ft, size = font_size, part = "all")))
  }
  if (!is.numeric(table_width) || length(table_width) != 1L ||
      is.na(table_width) || table_width <= 0) {
    stop("`table_width` must be NULL or a single positive number.", call. = FALSE)
  }

  ft_requested <- flextable::autofit(flextable::fontsize(ft, size = font_size, part = "all"))
  if (!isTRUE(fit_width)) {
    return(ft_requested)
  }

  fitted <- .scale_flextable_width(ft_requested, table_width)
  fitted_width <- .flextable_width(fitted)
  if (is.finite(fitted_width) && fitted_width <= table_width) {
    return(fitted)
  }

  if (font_size > min_font_size) {
    for (size in seq(font_size - 1, min_font_size, by = -1)) {
      candidate <- flextable::autofit(flextable::fontsize(ft, size = size, part = "all"))
      candidate <- .scale_flextable_width(candidate, table_width)
      candidate_width <- .flextable_width(candidate)
      if (is.finite(candidate_width) && candidate_width <= table_width) {
        return(candidate)
      }
    }
  }

  if (isTRUE(warn)) {
    warning(
      "The table is wider than the selected DOCX page width. ",
      "Saving with flextable::autofit() at the requested font size instead of ",
      "reducing below `min_font_size`. Consider landscape orientation, fewer columns, ",
      "a smaller `font_size`, or `fit_width = FALSE`.",
      call. = FALSE
    )
  }

  ft_requested
}

#' Resolve DOCX orientation from table width
#' @keywords internal
#' @noRd
.resolve_docx_orientation <- function(ft,
                                      orientation = "auto",
                                      portrait_width = 6.5,
                                      landscape_width = 9,
                                      wide_col_threshold = 6L) {
  orientation <- match.arg(orientation, c("auto", "portrait", "landscape"))

  if (!identical(orientation, "auto")) {
    return(orientation)
  }

  measured_width <- .flextable_width(flextable::autofit(ft))
  col_keys <- if (!is.null(ft$col_keys)) ft$col_keys else character(0)
  n_cols <- length(col_keys)

  if ((is.finite(measured_width) && measured_width > portrait_width) ||
      n_cols > wide_col_threshold) {
    return("landscape")
  }

  "portrait"
}

#' Prepare a flextable and Word section for DOCX export
#' @keywords internal
#' @noRd
.prepare_flextable_docx <- function(ft,
                                    orientation = c("auto", "portrait", "landscape"),
                                    fit_width = TRUE,
                                    font_size = 9,
                                    min_font_size = 8) {
  orientation <- match.arg(orientation)
  .validate_docx_table_sizing(
    fit_width = fit_width,
    font_size = font_size,
    min_font_size = min_font_size
  )

  resolved_orientation <- .resolve_docx_orientation(
    ft = flextable::fontsize(ft, size = font_size, part = "all"),
    orientation = orientation
  )
  page_width <- if (identical(resolved_orientation, "landscape")) 9 else 6.5
  section <- officer::prop_section(
    page_size = officer::page_size(orient = resolved_orientation)
  )

  list(
    table = .fit_flextable_docx_width(
      ft,
      table_width = page_width,
      fit_width = fit_width,
      font_size = font_size,
      min_font_size = min_font_size
    ),
    orientation = resolved_orientation,
    page_width = page_width,
    section = section
  )
}

#' Resolve a forest plot object for saving
#' @keywords internal
#' @noRd
.resolve_forest_object <- function(forest) {
  if (inherits(forest, "gtregression_forest") && !is.null(forest[["plot"]])) {
    return(forest[["plot"]])
  }

  if (inherits(forest, c("forestplot", "gtable", "gTree", "grob"))) {
    return(forest)
  }

  stop(
    "`forest` must be a forest_reg() object or a compatible forestploter/grid object.",
    call. = FALSE
  )
}

#' Estimate a practical export canvas for forest plots
#' @keywords internal
#' @noRd
.measure_forest_grob <- function(plot) {
  if (is.null(plot[["widths"]]) || is.null(plot[["heights"]])) {
    return(NULL)
  }

  tmp <- tempfile(fileext = ".pdf")
  grDevices::pdf(file = tmp, width = 20, height = 20)
  on.exit({
    grDevices::dev.off()
    unlink(tmp)
  }, add = TRUE)

  grid::grid.newpage()
  width <- grid::convertWidth(sum(plot$widths), unitTo = "in", valueOnly = TRUE)
  height <- grid::convertHeight(sum(plot$heights), unitTo = "in", valueOnly = TRUE)

  if (!is.finite(width) || !is.finite(height) || width <= 0 || height <= 0) {
    return(NULL)
  }

  list(width = width, height = height)
}

#' Estimate a practical export canvas for forest plots
#' @keywords internal
#' @noRd
.forest_export_dimensions <- function(forest, plot, width = NULL, height = NULL,
                                      scale = 1, padding = 0.25,
                                      auto_size = TRUE) {
  if (!is.numeric(scale) || length(scale) != 1L || is.na(scale) || scale <= 0) {
    stop("`scale` must be a single positive number.", call. = FALSE)
  }
  if (!is.logical(auto_size) || length(auto_size) != 1L || is.na(auto_size)) {
    stop("`auto_size` must be TRUE or FALSE.", call. = FALSE)
  }
  if (!is.null(width) &&
      (!is.numeric(width) || length(width) != 1L || is.na(width) || width <= 0)) {
    stop("`width` must be NULL or a single positive number.", call. = FALSE)
  }
  if (!is.null(height) &&
      (!is.numeric(height) || length(height) != 1L || is.na(height) || height <= 0)) {
    stop("`height` must be NULL or a single positive number.", call. = FALSE)
  }

  if (isTRUE(auto_size) && (is.null(width) || is.null(height))) {
    measured <- .measure_forest_grob(plot)
    if (!is.null(measured)) {
      if (is.null(width)) width <- measured$width + (2 * padding)
      if (is.null(height)) height <- measured$height + (2 * padding)
    }
  }

  if (is.null(width) || is.null(height)) {
    plot_data <- if (inherits(forest, "gtregression_forest")) forest[["data"]] else NULL

    if (is.data.frame(plot_data) && nrow(plot_data) > 0L) {
      n_rows <- nrow(plot_data)
      n_cols <- ncol(plot_data)
      n_ci_cols <- sum(grepl("^\\s+$", names(plot_data)))
      longest_label <- if ("Characteristic" %in% names(plot_data)) {
        max(nchar(as.character(plot_data$Characteristic)), na.rm = TRUE)
      } else {
        20
      }
      if (!is.finite(longest_label)) longest_label <- 20

      auto_width <- 3.4 + (0.95 * n_cols) + (1.4 * n_ci_cols) +
        (0.035 * min(longest_label, 60))
      auto_height <- 1.2 + (0.32 * n_rows)
    } else {
      auto_width <- 10
      auto_height <- 7
    }

    if (is.null(width)) width <- auto_width
    if (is.null(height)) height <- auto_height
  }

  list(width = width * scale, height = height * scale)
}

#' Draw a forest plot on the active graphics device
#' @keywords internal
#' @noRd
.draw_forest_export <- function(plot, padding = 0.25) {
  if (!is.numeric(padding) || length(padding) != 1L ||
      is.na(padding) || padding < 0) {
    stop("`padding` must be a single non-negative number.", call. = FALSE)
  }

  grid::grid.newpage()
  if (padding > 0) {
    grid::pushViewport(grid::viewport(
      width = grid::unit(1, "npc") - grid::unit(2 * padding, "in"),
      height = grid::unit(1, "npc") - grid::unit(2 * padding, "in")
    ))
    on.exit(grid::popViewport(), add = TRUE)
  }
  grid::grid.draw(plot)
  invisible(plot)
}

# -------------------------------------------------------------------
# save_table
# -------------------------------------------------------------------

#' Save a single regression or summary table
#'
#' Saves a \code{gtregression} table, merged table, \code{gt_tbl}, or
#' \code{flextable} as a Word, RTF, PDF, or HTML file.
#'
#' @param tbl A \code{gtregression} object, \code{merged_table} object,
#'   \code{gt_tbl}, or \code{flextable}.
#' @param filename File name for the output. Extension is optional. If no
#'   directory is supplied, the file is saved in \code{tempdir()}.
#' @param format Output format. One of \code{"docx"}, \code{"rtf"},
#'   \code{"pdf"}, or \code{"html"}. Flextable objects can be saved as
#'   \code{"docx"}, \code{"rtf"}, or \code{"html"}.
#' @param orientation Word page orientation for DOCX output. One of
#'   \code{"auto"}, \code{"portrait"}, or \code{"landscape"}. With
#'   \code{"auto"}, wide tables are saved in landscape orientation before any
#'   font-size reduction is attempted.
#' @param fit_width Logical. If \code{TRUE}, try to fit flextable DOCX output
#'   within the selected Word page width. If \code{FALSE}, keep the natural
#'   autofit table width.
#' @param font_size Requested font size for flextable DOCX output.
#' @param min_font_size Smallest font size allowed when fitting wide flextable
#'   DOCX output. The font size is never reduced below this value.
#'
#' @return Saves the file to disk. Invisibly returns the normalized file path.
#'
#' @examples
#' birthwt_data <- data_birthwt |>
#'   dplyr::mutate(
#'     smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
#'     low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
#'   )
#'
#' tbl <- uni_reg(
#'   data = birthwt_data,
#'   outcome = "low",
#'   exposures = c("age", "smoke"),
#'   approach = "logit"
#' )
#'
#' save_table(tbl, filename = tempfile("table"), format = "html")
#'
#' # Wide Word tables can be saved in landscape orientation.
#' \donttest{
#' save_table(
#'   tbl,
#'   filename = tempfile("table-wide"),
#'   format = "docx",
#'   orientation = "auto",
#'   fit_width = TRUE,
#'   font_size = 9,
#'   min_font_size = 8
#' )
#' }
#' @export
save_table <- function(tbl,
                       filename = "table",
                       format = c("docx", "rtf", "pdf", "html"),
                       orientation = c("auto", "portrait", "landscape"),
                       fit_width = TRUE,
                       font_size = 9,
                       min_font_size = 8) {
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("docx", "rtf", "pdf", "html"))
  format <- match.arg(format)
  orientation <- .choice_arg(
    substitute(orientation),
    env = parent.frame(),
    choices = c("auto", "portrait", "landscape")
  )
  orientation <- match.arg(orientation, c("auto", "portrait", "landscape"))
  filename <- .normalize_save_path(filename, format)

  obj <- .resolve_table_object(tbl)

  if (inherits(obj, "gt_tbl")) {
    if (!requireNamespace("gt", quietly = TRUE)) {
      stop("Package 'gt' is required to save gt tables.", call. = FALSE)
    }
    gt::gtsave(data = obj, filename = filename)

  } else if (inherits(obj, "flextable")) {
    if (!requireNamespace("flextable", quietly = TRUE)) {
      stop("Package 'flextable' is required to save flextable objects.", call. = FALSE)
    }

    if (identical(format, "docx")) {
      docx <- .prepare_flextable_docx(
        obj,
        orientation = orientation,
        fit_width = fit_width,
        font_size = font_size,
        min_font_size = min_font_size
      )
      flextable::save_as_docx(
        docx$table,
        path = filename,
        pr_section = docx$section
      )
    } else if (identical(format, "rtf")) {
      flextable::save_as_rtf(obj, path = filename)
    } else if (identical(format, "html")) {
      flextable::save_as_html(obj, path = filename)
    } else {
      stop(
        "Saving flextable objects as PDF is not directly supported. Save as DOCX, RTF, or HTML instead.",
        call. = FALSE
      )
    }

  } else {
    stop("Unsupported table type.", call. = FALSE)
  }

  message("Table saved at: ", normalizePath(filename))
  invisible(normalizePath(filename))
}

# -------------------------------------------------------------------
# save_plot
# -------------------------------------------------------------------

#' Save a single plot
#'
#' Saves a \code{ggplot2} plot to a file in PNG, PDF, or JPG format.
#'
#' @param plot A \code{ggplot2} object.
#' @param filename Name of the file to save, with or without extension. If no
#'   directory is supplied, the file is saved in \code{tempdir()}.
#' @param format Output format. One of \code{"png"}, \code{"pdf"}, or \code{"jpg"}.
#' @param width Width of the saved plot in inches.
#' @param height Height of the saved plot in inches.
#' @param dpi Resolution of the plot in dots per inch.
#'
#' @return Saves the file to disk. Invisibly returns the normalized file path.
#'
#' @examples
#' p <- ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
#'   ggplot2::geom_point()
#'
#' save_plot(p, filename = tempfile("plot"), format = "png")
#' @importFrom ggplot2 ggsave
#' @export
save_plot <- function(plot,
                      filename = "plot",
                      format = c("png", "pdf", "jpg"),
                      width = 8,
                      height = 6,
                      dpi = 300) {
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("png", "pdf", "jpg"))
  format <- match.arg(format)

  if (inherits(plot, "gtregression_forest")) {
    stop("Use `save_forest()` to save forest_reg() outputs.", call. = FALSE)
  }
  if (!inherits(plot, "ggplot")) {
    stop("`plot` must be a ggplot2 object.", call. = FALSE)
  }
  if (!is.numeric(width) || length(width) != 1L || is.na(width) || width <= 0) {
    stop("`width` must be a single positive number.", call. = FALSE)
  }
  if (!is.numeric(height) || length(height) != 1L || is.na(height) || height <= 0) {
    stop("`height` must be a single positive number.", call. = FALSE)
  }
  if (!is.numeric(dpi) || length(dpi) != 1L || is.na(dpi) || dpi <= 0) {
    stop("`dpi` must be a single positive number.", call. = FALSE)
  }

  filename <- .normalize_save_path(filename, format)

  ggplot2::ggsave(
    filename = filename,
    plot = plot,
    device = format,
    width = width,
    height = height,
    dpi = dpi
  )

  message("Plot saved at: ", normalizePath(filename))
  invisible(normalizePath(filename))
}

# -------------------------------------------------------------------
# save_forest
# -------------------------------------------------------------------

#' Save a forest_reg() output
#'
#' Saves a \code{forest_reg()} output, or a compatible
#' \pkg{forestploter}/grid object, to a fixed graphics device. This is useful
#' when the RStudio Viewer or operating-system graphics device crops wide forest
#' plots or compresses forest columns.
#'
#' @param forest A \code{gtregression_forest} object returned by
#'   \code{forest_reg()}, or a compatible \pkg{forestploter}/grid object.
#' @param filename File name for the output, with or without extension. If no
#'   directory is supplied, the file is saved in \code{tempdir()}.
#' @param format Output format. One of \code{"pdf"}, \code{"png"},
#'   \code{"tiff"}, or \code{"jpg"}.
#' @param width,height Optional export width and height in inches. If either is
#'   \code{NULL} and \code{auto_size = TRUE}, the size is measured from the
#'   intrinsic \pkg{forestploter} gtable dimensions. If measurement is not
#'   possible, a practical fallback is estimated from the number of rows and
#'   columns in the \code{forest_reg()} output.
#' @param scale Positive multiplier applied to the export width and height. This
#'   is a quick way to make a large forest plot roomier.
#' @param auto_size Logical. If \code{TRUE}, measure the intrinsic forest plot
#'   size when \code{width} or \code{height} is not supplied. If \code{FALSE},
#'   use the fallback row/column-based sizing.
#' @param padding White space around the forest plot in inches.
#' @param dpi Resolution for raster formats.
#'
#' @return Saves the file to disk. Invisibly returns the normalized file path.
#'
#' @examples
#' birthwt_data <- data_birthwt |>
#'   transform(
#'     smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
#'     ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
#'     low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
#'   )
#'
#' uni_or <- uni_reg(
#'   birthwt_data,
#'   outcome = "low",
#'   exposures = c("age", "smoke", "ht"),
#'   approach = "logit"
#' )
#'
#' forest <- forest_reg(uni = uni_or)
#' save_forest(forest, filename = tempfile("forest"), format = "pdf")
#'
#' # For large forest plots, increase width, height, scale, or padding.
#' save_forest(
#'   forest,
#'   filename = tempfile("forest-wide"),
#'   format = "png",
#'   scale = 1.2,
#'   padding = 0.35,
#'   dpi = 300
#' )
#' @export
#' @importFrom grid grid.newpage grid.draw pushViewport popViewport viewport unit convertWidth convertHeight
save_forest <- function(forest,
                        filename = "forest",
                        format = c("pdf", "png", "tiff", "jpg"),
                        width = NULL,
                        height = NULL,
                        scale = 1,
                        auto_size = TRUE,
                        padding = 0.25,
                        dpi = 300) {
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("pdf", "png", "tiff", "jpg"))
  format <- match.arg(format)

  if (!is.numeric(padding) || length(padding) != 1L ||
      is.na(padding) || padding < 0) {
    stop("`padding` must be a single non-negative number.", call. = FALSE)
  }
  if (!is.numeric(dpi) || length(dpi) != 1L || is.na(dpi) || dpi <= 0) {
    stop("`dpi` must be a single positive number.", call. = FALSE)
  }

  plot <- .resolve_forest_object(forest)
  dims <- .forest_export_dimensions(
    forest,
    plot = plot,
    width = width,
    height = height,
    scale = scale,
    padding = padding,
    auto_size = auto_size
  )
  width <- dims$width
  height <- dims$height

  if (padding * 2 >= min(width, height)) {
    stop("`padding` is too large for the requested export width/height.", call. = FALSE)
  }

  filename <- .normalize_save_path(filename, format)

  device_open <- FALSE
  if (identical(format, "pdf")) {
    grDevices::pdf(filename, width = width, height = height, onefile = FALSE)
  } else if (identical(format, "png")) {
    grDevices::png(filename, width = width, height = height, units = "in", res = dpi)
  } else if (identical(format, "jpg")) {
    grDevices::jpeg(filename, width = width, height = height, units = "in", res = dpi)
  } else {
    grDevices::tiff(filename, width = width, height = height, units = "in",
                    res = dpi, compression = "lzw")
  }
  device_open <- TRUE
  on.exit(if (device_open) grDevices::dev.off(), add = TRUE)

  .draw_forest_export(plot, padding = padding)
  grDevices::dev.off()
  device_open <- FALSE

  message("Forest plot saved at: ", normalizePath(filename))
  invisible(normalizePath(filename))
}

# -------------------------------------------------------------------
# save_docx
# -------------------------------------------------------------------

#' Save multiple tables and plots to a Word document
#'
#' Saves a collection of \code{gtregression} tables, merged tables,
#' \code{gt_tbl}/\code{flextable} objects, and \code{ggplot2} plots into a
#' single Word document.
#'
#' @param tables A list of tables. Each element may be a \code{gtregression}
#'   object, \code{merged_table} object, \code{gt_tbl}, or \code{flextable}.
#' @param plots A list of \code{ggplot2} plot objects.
#' @param filename File name for the output, with or without \code{.docx}. If no
#'   directory is supplied, the file is saved in \code{tempdir()}.
#' @param titles Optional character vector of titles for tables and plots in
#'   the order they are added.
#' @param table_width Maximum table width in inches for Word export. The default
#'   \code{6.5} fits a standard portrait Word page with common margins. Use
#'   \code{NULL} to keep the original flextable widths.
#' @param plot_width Width of inserted plots in inches.
#' @param plot_height Height of inserted plots in inches.
#'
#' @return Saves the Word document to disk. Invisibly returns the normalized
#'   file path.
#'
#' @examples
#' birthwt_data <- data_birthwt |>
#'   dplyr::mutate(
#'     smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
#'     low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
#'   )
#'
#' tbl <- uni_reg(
#'   data = birthwt_data,
#'   outcome = "low",
#'   exposures = c("age", "smoke"),
#'   approach = "logit",
#'   format = "flextable"
#' )
#'
#' save_docx(tables = tbl, filename = tempfile("report"))
#' @export
#' @importFrom officer read_docx body_add_par body_add_gg
#' @importFrom flextable body_add_flextable
save_docx <- function(tables = NULL,
                      plots = NULL,
                      filename = "report.docx",
                      titles = NULL,
                      table_width = 6.5,
                      plot_width = 6,
                      plot_height = 5) {
  if (!requireNamespace("officer", quietly = TRUE)) {
    stop("Package 'officer' is required.", call. = FALSE)
  }

  filename <- .normalize_save_path(filename, "docx")

  if (!is.null(tables) && .is_save_table_like(tables)) {
    tables <- list(tables)
  }
  if (!is.null(plots) && inherits(plots, "ggplot")) {
    plots <- list(plots)
  }
  if (!is.null(tables) && (is.data.frame(tables) || !is.list(tables))) {
    stop("`tables` must be NULL, a table object, or a list of table objects.", call. = FALSE)
  }
  if (!is.null(plots) && (is.data.frame(plots) || !is.list(plots))) {
    stop("`plots` must be NULL, a ggplot2 object, or a list of ggplot2 objects.", call. = FALSE)
  }
  if (!is.numeric(plot_width) || length(plot_width) != 1L ||
      is.na(plot_width) || plot_width <= 0) {
    stop("`plot_width` must be a single positive number.", call. = FALSE)
  }
  if (!is.null(table_width) &&
      (!is.numeric(table_width) || length(table_width) != 1L ||
       is.na(table_width) || table_width <= 0)) {
    stop("`table_width` must be NULL or a single positive number.", call. = FALSE)
  }
  if (!is.numeric(plot_height) || length(plot_height) != 1L ||
      is.na(plot_height) || plot_height <= 0) {
    stop("`plot_height` must be a single positive number.", call. = FALSE)
  }

  n_tables <- if (is.null(tables)) 0L else length(tables)
  n_plots  <- if (is.null(plots)) 0L else length(plots)
  n_items  <- n_tables + n_plots

  if (n_items == 0L) {
    stop("Provide at least one table or plot.", call. = FALSE)
  }

  if (!is.null(titles) && length(titles) != n_items) {
    warning(
      "Length of `titles` does not match the number of tables + plots. Titles will be ignored.",
      call. = FALSE
    )
    titles <- NULL
  }

  doc <- officer::read_docx()
  idx <- 1L

  if (!is.null(tables)) {
    if (!requireNamespace("flextable", quietly = TRUE)) {
      stop("Package 'flextable' is required to add tables to a DOCX file.", call. = FALSE)
    }

    for (tbl in tables) {
      obj <- .resolve_table_object(tbl)

      if (!is.null(titles)) {
        doc <- officer::body_add_par(doc, titles[[idx]], style = "heading 1")
        idx <- idx + 1L
      }

      if (inherits(obj, "flextable")) {
        ft <- .fit_flextable_docx_width(obj, table_width = table_width)
      } else if (inherits(obj, "gt_tbl")) {
        stop(
          paste0(
            "DOCX export currently supports flextable-based tables directly. For gt tables, ",
            "save as HTML/PDF with save_table(), or create the table with format = 'flextable'."
          ),
          call. = FALSE
        )
      } else {
        stop("Unsupported table type for DOCX export.", call. = FALSE)
      }

      doc <- flextable::body_add_flextable(doc, ft)
      doc <- officer::body_add_par(doc, "")
    }
  }

  if (!is.null(plots)) {
    for (p in plots) {
      if (!inherits(p, "ggplot")) {
        stop("All elements in `plots` must be ggplot2 objects.", call. = FALSE)
      }

      if (!is.null(titles)) {
        doc <- officer::body_add_par(doc, titles[[idx]], style = "heading 1")
        idx <- idx + 1L
      }

      doc <- officer::body_add_gg(
        doc,
        value = p,
        width = plot_width,
        height = plot_height
      )
      doc <- officer::body_add_par(doc, "")
    }
  }

  print(doc, target = filename)

  message("Word document saved at: ", normalizePath(filename))
  invisible(normalizePath(filename))
}

birthwt_save_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
    )
}

test_that("normalize save paths validates input and appends extensions", {
  path <- .normalize_save_path(file.path(tempdir(), "my-table"), "html")
  bare_path <- .normalize_save_path("my-table", "html")
  default_paths <- c(
    .normalize_save_path("table", "html"),
    .normalize_save_path("plot", "png"),
    .normalize_save_path("forest", "pdf"),
    .normalize_save_path("report.docx", "docx")
  )

  expect_match(path, "\\.html$")
  expect_equal(normalizePath(dirname(bare_path)), normalizePath(tempdir()))
  expect_equal(basename(bare_path), "my-table.html")
  expect_true(all(normalizePath(dirname(default_paths)) == normalizePath(tempdir())))
  expect_error(.normalize_save_path(character(0), "html"), "`filename` must be")
  expect_error(.normalize_save_path(NA_character_, "html"), "`filename` must be")
  expect_error(.normalize_save_path("x", ""), "`ext` must be")
})

test_that("resolve_table_object accepts package tables and raw rendered tables", {
  df <- birthwt_save_data()

  tbl <- uni_reg(df, outcome = "low", exposures = "smoke", approach = logit, format = gt)
  merged <- merge_tables(
    uni_reg(df, outcome = "low", exposures = "smoke", approach = logit, format = gt),
    multi_reg(df, outcome = "low", exposures = "smoke", approach = logit, format = gt)
  )
  merged_gt <- merge_tables(
    uni_reg(df, outcome = "low", exposures = "smoke", approach = logit, format = gt),
    multi_reg(df, outcome = "low", exposures = "smoke", approach = logit, format = gt),
    format = gt
  )

  expect_s3_class(.resolve_table_object(tbl), "gt_tbl")
  expect_s3_class(.resolve_table_object(merged), "flextable")
  expect_s3_class(.resolve_table_object(merged_gt), "gt_tbl")
  expect_s3_class(.resolve_table_object(tbl$table), "gt_tbl")
  expect_error(.resolve_table_object(data.frame(x = 1)), "`tbl` must be")
})

test_that("save_table writes gt html files and returns normalized paths", {
  skip_if_not_installed("gt")

  df <- birthwt_save_data()
  tbl <- uni_reg(df, outcome = "low", exposures = c("age", "smoke"), approach = logit, format = gt)
  filename <- file.path(tempdir(), paste0("gtregression-table-", Sys.getpid()))

  out <- suppressWarnings(save_table(tbl, filename = filename, format = html))

  expect_true(file.exists(out))
  expect_match(out, "\\.html$")
  unlink(out)
})

test_that("save_table writes flextable docx, rtf, and html files", {
  skip_if_not_installed("flextable")

  df <- birthwt_save_data()
  tbl <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = logit,
    format = flextable
  )

  file_docx <- file.path(tempdir(), paste0("gtregression-flex-", Sys.getpid(), ".docx"))
  file_rtf <- file.path(tempdir(), paste0("gtregression-flex-", Sys.getpid(), ".rtf"))
  file_html <- file.path(tempdir(), paste0("gtregression-flex-", Sys.getpid(), ".html"))

  out_docx <- save_table(
    tbl,
    filename = file_docx,
    format = docx,
    orientation = auto,
    fit_width = TRUE,
    font_size = 9,
    min_font_size = 8
  )
  out_rtf <- save_table(tbl, filename = file_rtf, format = rtf)
  out_html <- save_table(tbl, filename = file_html, format = html)

  expect_true(file.exists(out_docx))
  expect_true(file.exists(out_rtf))
  expect_true(file.exists(out_html))
  expect_error(save_table(tbl, filename = file.path(tempdir(), "bad-flex"), format = pdf),
               "PDF is not directly supported")

  unlink(c(out_docx, out_rtf, out_html))
})

test_that("docx flextables are fitted without unsafe font shrinking", {
  skip_if_not_installed("flextable")

  wide_df <- as.data.frame(matrix(seq_len(100), nrow = 5))
  ft <- flextable::autofit(flextable::flextable(wide_df))
  fitted <- .fit_flextable_docx_width(
    ft,
    table_width = 6.5,
    fit_width = TRUE,
    font_size = 9,
    min_font_size = 8
  )
  unfitted <- .fit_flextable_docx_width(ft, table_width = NULL, font_size = 9)
  natural <- .fit_flextable_docx_width(ft, table_width = 6.5, fit_width = FALSE)

  expect_lte(sum(flextable::flextable_dim(fitted)$widths), 6.5)
  expect_s3_class(unfitted, "flextable")
  expect_gt(sum(flextable::flextable_dim(natural)$widths),
            sum(flextable::flextable_dim(fitted)$widths))
  expect_error(.fit_flextable_docx_width(ft, table_width = 0), "`table_width`")
  expect_error(.fit_flextable_docx_width(ft, min_font_size = 10, font_size = 9),
               "`min_font_size`")
})

test_that("docx orientation auto prefers landscape for wide flextables", {
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")

  wide_df <- as.data.frame(matrix(seq_len(80), nrow = 4))
  names(wide_df) <- paste0("long_column_", seq_along(wide_df))
  ft <- flextable::autofit(flextable::flextable(wide_df))
  prepared <- .prepare_flextable_docx(
    ft,
    orientation = "auto",
    fit_width = TRUE,
    font_size = 9,
    min_font_size = 8
  )

  expect_equal(prepared$orientation, "landscape")
  expect_equal(prepared$page_width, 9)
  expect_s3_class(prepared$table, "flextable")
  expect_error(.prepare_flextable_docx(ft, fit_width = NA), "`fit_width`")
})

test_that("save_plot writes files and validates inputs", {
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
    ggplot2::geom_point()
  filename <- file.path(tempdir(), paste0("gtregression-plot-", Sys.getpid()))

  out <- save_plot(p, filename = filename, format = png, width = 4, height = 3, dpi = 72)

  expect_true(file.exists(out))
  expect_match(out, "\\.png$")
  expect_error(save_plot(mtcars), "`plot` must be")
  expect_error(save_plot(p, width = 0), "`width` must be")
  expect_error(save_plot(p, height = NA_real_), "`height` must be")
  expect_error(save_plot(p, dpi = -1), "`dpi` must be")

  unlink(out)
})

test_that("save_forest writes forest outputs and validates inputs", {
  skip_if_not_installed("forestploter")

  df <- birthwt_save_data()
  uni <- uni_reg(
    df,
    outcome = "low",
    exposures = c("age", "smoke", "ht"),
    approach = logit
  )
  multi <- multi_reg(
    df,
    outcome = "low",
    exposures = c("smoke", "ht"),
    adjust_for = "age",
    approach = logit
  )
  forest <- forest_reg(uni = uni, multi = multi, quiet = TRUE)

  file_pdf <- file.path(tempdir(), paste0("gtregression-forest-", Sys.getpid()))
  file_png <- file.path(tempdir(), paste0("gtregression-forest-", Sys.getpid(), "-png"))
  file_bare <- paste0("gtregression-forest-temp-", Sys.getpid())

  out_pdf <- save_forest(forest, filename = file_pdf, format = pdf,
                         width = 7, height = 5, padding = 0.1)
  out_png <- save_forest(forest, filename = file_png, format = png,
                         width = 7, height = 5, auto_size = FALSE, dpi = 72)
  out_bare <- save_forest(forest, filename = file_bare, format = pdf,
                          width = 7, height = 5)

  expect_true(file.exists(out_pdf))
  expect_true(file.exists(out_png))
  expect_true(file.exists(out_bare))
  expect_match(out_pdf, "\\.pdf$")
  expect_match(out_png, "\\.png$")
  expect_equal(normalizePath(dirname(out_bare)), normalizePath(tempdir()))

  expect_error(save_plot(forest), "Use `save_forest\\(\\)`")
  expect_error(save_forest(mtcars), "`forest` must be")
  expect_error(save_forest(forest, width = 0), "`width` must be")
  expect_error(save_forest(forest, height = NA_real_), "`height` must be")
  expect_error(save_forest(forest, scale = 0), "`scale` must be")
  expect_error(save_forest(forest, auto_size = NA), "`auto_size` must be")
  expect_error(save_forest(forest, padding = -1), "`padding` must be")
  expect_error(save_forest(forest, padding = 3, width = 4, height = 4), "`padding` is too large")
  expect_error(save_forest(forest, dpi = -1), "`dpi` must be")

  unlink(c(out_pdf, out_png, out_bare))
})

test_that("save_docx accepts single flextable table and single plot", {
  skip_if_not_installed("officer")
  skip_if_not_installed("flextable")

  df <- birthwt_save_data()
  tbl <- uni_reg(
    df,
    outcome = "low",
    exposures = "smoke",
    approach = logit,
    format = flextable
  )
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
    ggplot2::geom_point()
  filename <- file.path(tempdir(), paste0("gtregression-report-", Sys.getpid()))

  out <- save_docx(
    tables = tbl,
    plots = p,
    filename = filename,
    titles = c("Table 1", "Figure 1"),
    plot_width = 4,
    plot_height = 3
  )

  expect_true(file.exists(out))
  expect_match(out, "\\.docx$")

  unlink(out)
})

test_that("save_docx validates inputs and rejects gt tables for Word export", {
  skip_if_not_installed("officer")

  df <- birthwt_save_data()
  gt_tbl <- uni_reg(df, outcome = "low", exposures = "smoke", approach = logit, format = gt)

  expect_error(save_docx(), "at least one table or plot")
  expect_error(save_docx(tables = data.frame(x = 1)), "`tables` must be")
  expect_error(save_docx(plots = data.frame(x = 1)), "`plots` must be")
  expect_error(save_docx(plots = list(data.frame(x = 1))), "must be ggplot2 objects")
  expect_error(save_docx(plots = ggplot2::ggplot(), table_width = 0), "`table_width`")
  expect_error(save_docx(plots = ggplot2::ggplot(), plot_width = 0), "`plot_width` must be")
  expect_error(save_docx(plots = ggplot2::ggplot(), plot_height = NA_real_), "`plot_height` must be")
  expect_error(save_docx(tables = gt_tbl), "gt tables")

  skip_if_not_installed("flextable")
  ft_tbl <- uni_reg(
    df,
    outcome = "low",
    exposures = "smoke",
    approach = logit,
    format = flextable
  )
  expect_warning(
    out <- save_docx(
      tables = list(ft_tbl),
      filename = file.path(tempdir(), paste0("gtregression-warning-", Sys.getpid())),
      titles = c("One", "Too many")
    ),
    "Length of `titles`"
  )
  expect_true(file.exists(out))
  unlink(out)
})

# Internal helpers shared by the gtregression app and its tests.

gtx_reference_candidates <- function(data, predictors, outcome = NULL) {
  predictors <- setdiff(intersect(as.character(predictors), names(data)), outcome)
  keep <- vapply(data[predictors], function(x) {
    is_categorical <- is.factor(x) || is.character(x) || is.logical(x)
    is_categorical && length(unique(as.character(x[!is.na(x)]))) >= 2L
  }, logical(1))
  predictors <- predictors[keep]
  stats::setNames(lapply(predictors, function(variable) {
    x <- data[[variable]]
    values <- as.character(x[!is.na(x)])
    if (is.factor(x)) {
      levels(x)[levels(x) %in% values]
    } else {
      unique(values)
    }
  }), predictors)
}

gtx_relevel_predictors <- function(data, references) {
  if (!length(references)) return(data)
  for (variable in names(references)) {
    if (!variable %in% names(data)) stop("Unknown reference variable: ", variable, call. = FALSE)
    x <- data[[variable]]
    if (!(is.factor(x) || is.character(x) || is.logical(x))) {
      stop("Reference categories can only be set for categorical predictors: ", variable, call. = FALSE)
    }
    observed <- as.character(x[!is.na(x)])
    levels <- if (is.factor(x)) levels(x)[levels(x) %in% observed] else unique(observed)
    reference <- as.character(references[[variable]])
    if (!reference %in% levels) stop("Reference level is not observed for ", variable, ": ", reference, call. = FALSE)
    data[[variable]] <- stats::relevel(factor(as.character(x), levels = levels), ref = reference)
  }
  data
}

gtx_reference_code <- function(references, candidates, data_name = "analysis_data") {
  if (!length(references)) return(character(0))
  vapply(names(references), function(variable) {
    levels <- candidates[[variable]]
    sprintf(
      '%s[[%s]] <- stats::relevel(factor(as.character(%s[[%s]]), levels = %s), ref = %s)',
      data_name, shQuote(variable), data_name, shQuote(variable),
      paste0("c(", paste(shQuote(levels), collapse = ", "), ")"),
      shQuote(as.character(references[[variable]]))
    )
  }, character(1))
}

test_that("reference controls include only observed categorical predictors", {
  data <- data.frame(
    outcome = factor(c("No", "Yes", "No")),
    group = factor(c("A", "B", "A"), levels = c("A", "B", "Unused")),
    character_group = c("Control", "Treatment", "Control"),
    age = c(20, 30, 40),
    one_level = factor(c("Only", "Only", "Only"))
  )

  candidates <- gtx_reference_candidates(
    data,
    predictors = c("outcome", "group", "character_group", "age", "one_level"),
    outcome = "outcome"
  )

  expect_identical(names(candidates), c("group", "character_group"))
  expect_identical(candidates$group, c("A", "B"))
  expect_identical(candidates$character_group, c("Control", "Treatment"))
})

test_that("selected reference categories are applied and reproducible", {
  data <- data.frame(
    group = factor(c("A", "B", "A"), levels = c("A", "B")),
    age = c(20, 30, 40)
  )
  candidates <- gtx_reference_candidates(data, "group")
  changed <- gtx_relevel_predictors(data, list(group = "B"))
  code <- gtx_reference_code(list(group = "B"), candidates)

  expect_identical(levels(changed$group), c("B", "A"))
  expect_match(code, "stats::relevel", fixed = TRUE)
  expect_match(code, "ref = [\"']B[\"']")
  expect_error(gtx_relevel_predictors(data, list(group = "Missing")), "not observed")
  expect_error(gtx_relevel_predictors(data, list(age = "20")), "categorical predictors")
})

test_that("the app exposes Data Prep before modelling and reference guidance", {
  app_file <- system.file("shiny", "app.R", package = "gtregression")
  app_text <- paste(readLines(app_file, warn = FALSE), collapse = "\n")

  expect_lt(regexpr('"Data Prep"', app_text, fixed = TRUE)[[1]],
            regexpr('"Regression"', app_text, fixed = TRUE)[[1]])
  module_text <- paste(deparse(body(mod_data_prep_server)), collapse = " ")
  expect_match(module_text, "Original data is active for analysis", fixed = TRUE)
  expect_match(module_text, "use the prepared data", ignore.case = TRUE)
  expect_match(app_text, "The selected category is the baseline", fixed = TRUE)
  expect_match(app_text, "gtx_reference_code", fixed = TRUE)
})

#' Stepwise model selection with fit metrics
#'
#' Compare candidate models added or removed stepwise, then return the accepted
#' steps, model-fit metrics, and the lowest-AIC model.
#'
#' @param data A data frame containing the outcome and predictor variables.
#' @param outcome A single character string indicating the outcome variable.
#'   Quoted and bare names are accepted. Not used for survival approaches when
#'   \code{time} and \code{event} are supplied.
#' @param exposures Character vector of predictor variables to consider. Quoted
#'   names are recommended in scripts, and bare names are also accepted.
#' @param approach Regression method. One of:
#'   \code{"logit"}, \code{"logbinomial"}, \code{"poisson"},
#'   \code{"robpoisson"}, \code{"negbin"}, \code{"linear"}, \code{"cox"}, or
#'   \code{"survreg"}.
#' @param time,event Survival time and event indicator for \code{approach = "cox"}
#'   or \code{approach = "survreg"}. Quoted and bare names are accepted.
#' @param distribution Parametric survival distribution for
#'   \code{approach = "survreg"}. One of \code{"weibull"},
#'   \code{"exponential"}, \code{"lognormal"}, or \code{"loglogistic"}.
#'   Common spellings such as \code{"log-normal"} and \code{"log-logistic"} are
#'   also accepted.
#' @param direction Stepwise selection direction. One of:
#'   \code{"forward"} (default), \code{"backward"}, or \code{"both"}.
#' @param format Output format for the viewing table. One of
#'   \code{"flextable"} (default), \code{"gt"}, or \code{"tibble"}. Use
#'   \code{format = "tibble"} to keep only the original list structure.
#'
#' @return A list with the following components:
#' \itemize{
#'   \item \code{results_table}: A tibble summarising each accepted step's
#'   model metrics (AIC, BIC, deviance, log-likelihood, and adjusted R-squared
#'   for linear models).
#'   \item \code{best_model}: The best-fitting model object based on lowest AIC.
#'   \item \code{all_models}: A named list of the accepted stepwise models.
#'   \item \code{direction}: Stepwise selection direction used.
#'   \item \code{table}: A formatted \code{gt_tbl} or \code{flextable} when
#'   \code{format} is \code{"gt"} or \code{"flextable"}.
#' }
#'
#' @details
#' Treat stepwise selection as a screening tool. It is best used alongside the
#' study question, clinical judgement, and model diagnostics rather than as an
#' automatic final-model rule.
#'
#' @importFrom stats AIC BIC anova as.formula binomial coef cooks.distance
#' @importFrom stats deviance glm glm.control lm logLik na.omit nobs poisson
#' @importFrom stats predict reformulate residuals shapiro.test
#' @importFrom MASS glm.nb
#' @importFrom utils data
#' @importFrom tibble tibble
#' @importFrom dplyr bind_rows
#' @importFrom rlang .data
#' @examples
#' data <- data_birthwt
#' stepwise <- select_models(
#'   data = data,
#'   outcome = "bwt",
#'   exposures = c("age", "lwt", "smoke"),
#'   approach = "linear",
#'   direction = "forward"
#' )
#' stepwise$results_table
#' stepwise$best_model
#'
#' @export
select_models <- function(data, outcome, exposures, approach = "logit",
                          time = NULL,
                          event = NULL,
                          distribution = "weibull",
                          direction = "forward",
                          format = c("flextable", "gt", "tibble")) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }
  exposures <- .vars_arg(substitute(exposures), env = parent.frame())
  if (!is.character(exposures) || length(exposures) < 1L ||
      anyNA(exposures) || any(!nzchar(exposures))) {
    stop("`exposures` must be a non-empty character vector.", call. = FALSE)
  }

  approach <- .choice_arg(
    substitute(approach),
    env = parent.frame(),
    choices = c("logit", "logbinomial", "poisson", "robpoisson",
                "negbin", "linear", "cox", "survreg")
  )
  approach <- .normalize_approach(approach)
  .validate_approach(approach, context = "select_models")
  is_survival <- approach %in% c("cox", "survreg")

  if (is_survival) {
    time <- .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
    event <- .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
    outcome <- NULL
  } else {
    if (missing(outcome)) {
      stop("`outcome` must be supplied for non-survival model selection.", call. = FALSE)
    }
    outcome <- .vars_arg(substitute(outcome), env = parent.frame())
    if (!is.character(outcome) || length(outcome) != 1L ||
        is.na(outcome) || !nzchar(outcome)) {
      stop("`outcome` must be a single character variable name.", call. = FALSE)
    }
  }

  distribution <- if (approach == "survreg") {
    .surv_distribution_arg(
      substitute(distribution),
      env = parent.frame(),
      multiple = FALSE,
      arg = "distribution"
    )
  } else {
    .choice_arg(substitute(distribution), env = parent.frame())
  }

  missing_vars <- if (is_survival) {
    setdiff(c(time, event, exposures), names(data))
  } else {
    setdiff(c(outcome, exposures), names(data))
  }
  if (length(missing_vars)) {
    stop("Variables not found: ", paste(missing_vars, collapse = ", "),
         call. = FALSE)
  }
  direction <- .choice_arg(
    substitute(direction),
    env = parent.frame(),
    choices = c("forward", "backward", "both")
  )
  if (!is.character(direction) || length(direction) != 1L ||
      is.na(direction) || !direction %in% c("forward", "backward", "both")) {
    stop(
      "`direction` must be one of: forward, backward, both.",
      call. = FALSE
    )
  }
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  format <- match.arg(format, c("flextable", "gt", "tibble"))

  if (is_survival) {
    model_data <- .validate_cox_inputs(data, time, event, exposures, adjust_for = NULL)
  } else {
    .validate_outcome_by_approach(data[[outcome]], approach)
    model_data <- data
  }
  if (approach == "robpoisson" && is.factor(model_data[[outcome]])) {
    model_data[[outcome]] <- as.integer(model_data[[outcome]]) - 1L
  }

  fit_model <- function(vars) {
    if (is_survival) {
      bt <- function(x) paste0("`", gsub("`", "", x, fixed = TRUE), "`")
      rhs <- if (length(vars)) paste(bt(vars), collapse = " + ") else "1"
      fmla <- stats::as.formula(
        paste0("survival::Surv(", bt(time), ", ", bt(event), ") ~ ", rhs)
      )
    } else {
      fmla <- stats::reformulate(vars, response = outcome)
    }
    fmla_str <- paste(deparse(fmla), collapse = "")

    model <- if (approach == "cox") {
      survival::coxph(fmla, data = model_data, model = TRUE)
    } else if (approach == "survreg") {
      fit <- survival::survreg(fmla, data = model_data, dist = distribution, model = TRUE)
      attr(fit, "gtregression_events") <- sum(model_data[[event]] == 1, na.rm = TRUE)
      attr(fit, "gtregression_distribution") <- distribution
      fit
    } else if (approach == "negbin") {
      MASS::glm.nb(fmla, data = model_data)
    } else if (approach == "linear") {
      stats::lm(fmla, data = model_data)
    } else {
      family <- switch(
        approach,
        "logit" = stats::binomial(link = "logit"),
        "logbinomial" = stats::binomial(link = "log"),
        "poisson" = stats::poisson(link = "log"),
        "robpoisson" = stats::poisson(link = "log")
      )
      stats::glm(fmla, family = family, data = model_data)
    }

    attr(model, "formula_str") <- fmla_str
    attr(model, "selected_vars") <- vars
    model
  }

  model_metric <- function(model, model_id, vars) {
    out <- tibble::tibble(
      model_id = model_id,
      formula = attr(model, "formula_str"),
      model_terms = .gtregression_terms_label(vars),
      n_predictors = length(vars),
      AIC = stats::AIC(model),
      BIC = stats::BIC(model),
      logLik = as.numeric(stats::logLik(model)),
      deviance = .safe_numeric(stats::deviance(model))
    )

    if (approach == "linear") {
      dplyr::mutate(out, adj_r2 = summary(model)$adj.r.squared)
    } else if (approach == "cox") {
      smry <- tryCatch(summary(model), error = function(e) NULL)
      concordance <- if (!is.null(smry) && !is.null(smry$concordance)) {
        suppressWarnings(as.numeric(smry$concordance[1]))
      } else {
        NA_real_
      }
      dplyr::mutate(
        out,
        concordance = concordance,
        events = .safe_numeric(model$nevent),
        selected_vars = paste(vars, collapse = " + ")
      )
    } else if (approach == "survreg") {
      dplyr::mutate(
        out,
        distribution = distribution,
        scale = .safe_numeric(model$scale),
        events = .safe_numeric(attr(model, "gtregression_events", exact = TRUE)),
        selected_vars = paste(vars, collapse = " + ")
      )
    } else {
      dplyr::mutate(out, selected_vars = paste(vars, collapse = " + "))
    }
  }

  selected_vars <- if (direction == "backward") exposures else character(0)
  all_models <- list()
  model_metrics <- list()
  selected_vars_by_step <- list()
  step <- 1L

  repeat {
    current_vars <- selected_vars
    current_model <- fit_model(current_vars)
    model_name <- paste0("model_", step)
    all_models[[model_name]] <- current_model
    selected_vars_by_step[[model_name]] <- current_vars
    model_metrics[[model_name]] <- model_metric(current_model, step, current_vars)
    current_aic <- stats::AIC(current_model)

    add_candidates <- setdiff(exposures, current_vars)
    forward_models <- lapply(add_candidates, function(var) {
      fit_model(c(current_vars, var))
    })
    forward_aics <- vapply(forward_models, stats::AIC, FUN.VALUE = numeric(1))
    best_forward <- if (length(forward_aics)) min(forward_aics) else Inf
    best_forward_idx <- if (length(forward_aics)) which.min(forward_aics) else NA_integer_

    drop_candidates <- if (length(current_vars) > 1L) {
      lapply(current_vars, function(var) setdiff(current_vars, var))
    } else {
      list()
    }
    backward_models <- lapply(drop_candidates, fit_model)
    backward_aics <- vapply(backward_models, stats::AIC, FUN.VALUE = numeric(1))
    best_backward <- if (length(backward_aics)) min(backward_aics) else Inf
    best_backward_idx <- if (length(backward_aics)) which.min(backward_aics) else NA_integer_

    improved <- FALSE

    if (direction == "forward" && best_forward < current_aic - 1e-5) {
      selected_vars <- c(current_vars, add_candidates[[best_forward_idx]])
      improved <- TRUE
    } else if (direction == "backward" && best_backward < current_aic - 1e-5) {
      selected_vars <- drop_candidates[[best_backward_idx]]
      improved <- TRUE
    } else if (direction == "both") {
      if (best_forward < best_backward && best_forward < current_aic - 1e-5) {
        selected_vars <- c(current_vars, add_candidates[[best_forward_idx]])
        improved <- TRUE
      } else if (best_backward < current_aic - 1e-5) {
        selected_vars <- drop_candidates[[best_backward_idx]]
        improved <- TRUE
      }
    }

    step <- step + 1L
    if (!improved) break
  }

  metrics_tbl <- dplyr::bind_rows(model_metrics)
  best_row <- which.min(metrics_tbl$AIC)
  best_name <- names(model_metrics)[[best_row]]
  final_best_model <- all_models[[best_name]]
  attr(final_best_model, "selected_vars") <- selected_vars_by_step[[best_name]]

  out <- list(
    results_table = metrics_tbl,
    best_model = final_best_model,
    all_models = all_models,
    direction = direction
  )

  if (format != "tibble") {
    out$table <- .build_select_models_table(
      metrics_tbl,
      direction = direction,
      format = format
    )
  }

  out
}

#' Build formatted select_models table
#' @keywords internal
#' @noRd
.build_select_models_table <- function(metrics_tbl,
                                       direction = c("forward", "backward", "both"),
                                       format = c("flextable", "gt")) {
  direction <- match.arg(direction, c("forward", "backward", "both"))
  format <- match.arg(format, c("flextable", "gt"))
  direction_note <- paste0("Selection direction: ", direction, ".")
  caveat_note <- paste(
    "Screening aid only; compare candidate models with study design,",
    "clinical or subject-matter judgement, and model diagnostics."
  )

  display <- metrics_tbl |>
    dplyr::mutate(
      AIC = round(.data$AIC, 2),
      BIC = round(.data$BIC, 2),
      logLik = round(.data$logLik, 2),
      deviance = round(.data$deviance, 2),
      best = .data$AIC == min(.data$AIC, na.rm = TRUE)
    )
  display <- display[, setdiff(names(display), c("formula", "selected_vars")), drop = FALSE]

  if ("adj_r2" %in% names(display)) {
    display <- dplyr::mutate(display, adj_r2 = round(.data$adj_r2, 3))
  }

  if (format == "gt") {
    labels <- c(
      model_id = "Model",
      model_terms = "Selected variables",
      n_predictors = "Predictors",
      logLik = "Log-likelihood",
      adj_r2 = "Adjusted R-squared",
      best = "Best AIC"
    )
    labels <- labels[names(labels) %in% names(display)]
    left_cols <- intersect("model_terms", names(display))

    tbl <- gt::gt(display) |>
      gt::tab_header(title = "Stepwise model selection")
    tbl <- do.call(gt::cols_label, c(list(.data = tbl), as.list(labels)))
    tbl <- tbl |>
      gt::cols_align(align = "left", columns = dplyr::all_of(left_cols)) |>
      gt::cols_align(
        align = "center",
        columns = dplyr::all_of(setdiff(names(display), left_cols))
      ) |>
      gt::fmt(
        columns = "best",
        fns = function(x) ifelse(x, "Yes", "No")
      ) |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_column_labels()
      ) |>
      gt::tab_style(
        style = gt::cell_fill(color = "#e7f5ec"),
        locations = gt::cells_body(rows = .data$best)
      ) |>
      gt::tab_source_note(gt::md(direction_note)) |>
      gt::tab_source_note(gt::md(caveat_note)) |>
      .compact_gt_source_notes()

    return(tbl)
  }

  display$best <- ifelse(display$best, "Yes", "No")
  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Stepwise model selection")
  left_cols <- intersect("model_terms", names(display))
  ft <- flextable::align(ft, j = left_cols, align = "left", part = "all")
  ft <- flextable::align(
    ft,
    j = setdiff(names(display), "model_terms"),
    align = "center",
    part = "all"
  )
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  ft <- flextable::bg(ft, i = which(display$best == "Yes"), bg = "#e7f5ec", part = "body")
  ft <- flextable::add_footer_lines(ft, values = c(direction_note, caveat_note))
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

select_birthwt_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
    )
}

test_that("select_models works for logit approaches and all directions", {
  df <- select_birthwt_data()
  exposures <- c("age", "lwt", "race", "smoke", "ht")

  for (direction in c("forward", "backward", "both")) {
    result <- select_models(
      data = df,
      outcome = "low",
      exposures = exposures,
      approach = logit,
      direction = direction,
      format = tibble
    )

    expect_type(result, "list")
    expect_named(result, c("results_table", "best_model", "all_models", "direction"))
    expect_equal(result$direction, direction)
    expect_s3_class(result$results_table, "tbl_df")
    expect_s3_class(result$best_model, "glm")
    expect_true(all(c("model_id", "formula", "n_predictors", "AIC", "BIC",
                      "logLik", "deviance", "model_terms", "selected_vars") %in%
                      names(result$results_table)))
    expect_equal(length(result$all_models), nrow(result$results_table))
    expect_equal(
      attr(result$best_model, "formula_str"),
      result$results_table$formula[which.min(result$results_table$AIC)]
    )
  }
})

test_that("select_models supports logbinomial and robpoisson binary models", {
  df <- select_birthwt_data()

  logbin <- select_models(
    data = df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = logbinomial,
    direction = forward,
    format = tibble
  )
  rob <- select_models(
    data = df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = robpoisson,
    direction = forward,
    format = tibble
  )

  expect_s3_class(logbin$best_model, "glm")
  expect_s3_class(rob$best_model, "glm")
  expect_true(all(is.finite(logbin$results_table$AIC)))
  expect_true(all(is.finite(rob$results_table$AIC)))
})

test_that("select_models works for linear regression and returns adjusted R-squared", {
  result <- select_models(
    data = mtcars,
    outcome = "mpg",
    exposures = c("hp", "wt", "cyl"),
    approach = linear,
    direction = both,
    format = tibble
  )

  expect_s3_class(result$best_model, "lm")
  expect_true("adj_r2" %in% names(result$results_table))
  expect_true("model_terms" %in% names(result$results_table))
  expect_false("selected_vars" %in% names(result$results_table))
  expect_true(all(is.finite(result$results_table$adj_r2)))
})

test_that("select_models supports negative binomial regression", {
  data("quine", package = "MASS")
  quine_data <- quine |>
    dplyr::mutate(dplyr::across(c(Eth, Sex, Age, Lrn), as.factor))

  pois <- select_models(
    data = quine_data,
    outcome = "Days",
    exposures = c("Eth", "Sex", "Age", "Lrn"),
    approach = poisson,
    direction = forward,
    format = tibble
  )
  result <- select_models(
    data = quine_data,
    outcome = "Days",
    exposures = c("Eth", "Sex", "Age", "Lrn"),
    approach = negbin,
    direction = backward,
    format = tibble
  )

  expect_s3_class(pois$best_model, "glm")
  expect_true(all(is.finite(pois$results_table$AIC)))
  expect_true(inherits(result$best_model, "negbin"))
  expect_s3_class(result$results_table, "tbl_df")
  expect_true(all(is.finite(result$results_table$AIC)))
})

test_that("select_models supports Cox and parametric survival approaches", {
  skip_if_not_installed("survival")

  lung_data <- data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )

  cox_sel <- select_models(
    data = lung_data,
    time = time,
    event = status,
    exposures = c("trt", "celltype", "karno", "age", "prior"),
    approach = cox,
    direction = forward,
    format = tibble
  )

  expect_s3_class(cox_sel$best_model, "coxph")
  expect_true(all(c("concordance", "events", "selected_vars") %in%
                    names(cox_sel$results_table)))
  expect_true(all(is.finite(cox_sel$results_table$AIC)))
  expect_true(all(cox_sel$results_table$events > 0))

  surv_sel <- select_models(
    data = lung_data,
    time = "time",
    event = "status",
    exposures = c("trt", "celltype", "karno", "age", "prior"),
    approach = survreg,
    distribution = lognormal,
    direction = forward,
    format = gt
  )

  expect_s3_class(surv_sel$best_model, "survreg")
  expect_s3_class(surv_sel$table, "gt_tbl")
  expect_true(all(c("distribution", "scale", "events", "selected_vars") %in%
                    names(surv_sel$results_table)))
  expect_false("formula" %in% names(surv_sel$table$`_data`))
  expect_true("model_terms" %in% names(surv_sel$table$`_data`))
  expect_false(any(grepl("Surv", surv_sel$table$`_data`$model_terms, fixed = TRUE)))
  expect_true(all(surv_sel$results_table$distribution == "lognormal"))
  expect_true(all(is.finite(surv_sel$results_table$AIC)))

  surv_sel_alias <- select_models(
    data = lung_data,
    time = "time",
    event = "status",
    exposures = c("trt", "prior"),
    approach = survreg,
    distribution = "log-logistic",
    direction = forward,
    format = tibble
  )

  expect_true(all(surv_sel_alias$results_table$distribution == "loglogistic"))
})

test_that("select_models validates survival-specific inputs", {
  skip_if_not_installed("survival")

  lung_data <- data_lungcancer

  expect_error(
    select_models(
      data = lung_data,
      time = "time",
      event = "status",
      exposures = "missing",
      approach = cox
    ),
    "Variables not found: missing"
  )
  expect_error(
    select_models(
      data = lung_data,
      time = "time",
      event = "status",
      exposures = "age",
      approach = survreg,
      distribution = gaussian
    ),
    "Invalid distribution"
  )
})

test_that("select_models validates inputs clearly", {
  df <- select_birthwt_data()

  expect_error(
    select_models(list(y = 1), outcome = "y", exposures = "x"),
    "`data` must be a data frame"
  )
  expect_error(
    select_models(df, outcome = c("low", "bwt"), exposures = "age"),
    "`outcome` must be"
  )
  expect_error(
    select_models(df, outcome = "low", exposures = character(0)),
    "`exposures` must be"
  )
  expect_error(
    select_models(df, outcome = "low", exposures = "missing"),
    "Variables not found: missing"
  )
  expect_error(
    select_models(df, outcome = "missing", exposures = "age"),
    "Variables not found: missing"
  )
  expect_error(
    select_models(df, outcome = "low", exposures = "age", approach = "notreal"),
    "not a valid approach"
  )
  expect_error(
    select_models(df, outcome = "low", exposures = "age", direction = "sideways"),
    "must be one of"
  )
  expect_error(
    select_models(df, outcome = "bwt", exposures = "age", approach = logit),
    "This approach requires either a factor variable"
  )
})

test_that("select_models keeps best model selected variables consistent", {
  df <- mtcars
  result <- select_models(
    data = df,
    outcome = "mpg",
    exposures = c("hp", "wt", "qsec"),
    approach = linear,
    direction = forward,
    format = tibble
  )
  best_row <- which.min(result$results_table$AIC)

  expect_equal(attr(result$best_model, "formula_str"),
               result$results_table$formula[best_row])
  expect_equal(length(attr(result$best_model, "selected_vars")),
               result$results_table$n_predictors[best_row])
  expect_true(all(names(result$all_models) == paste0("model_", seq_along(result$all_models))))
})

test_that("select_models can add gt and flextable viewing tables", {
  gt_result <- select_models(
    data = mtcars,
    outcome = "mpg",
    exposures = c("hp", "wt"),
    approach = linear,
    direction = forward,
    format = gt
  )
  ft_result <- select_models(
    data = mtcars,
    outcome = "mpg",
    exposures = c("hp", "wt"),
    approach = linear,
    direction = forward,
    format = flextable
  )

  expect_s3_class(gt_result$results_table, "tbl_df")
  expect_s3_class(gt_result$table, "gt_tbl")
  expect_s3_class(ft_result$table, "flextable")
  expect_false("formula" %in% names(gt_result$table$`_data`))
  expect_true("model_terms" %in% names(gt_result$table$`_data`))
  expect_false("formula" %in% names(ft_result$table$body$dataset))
  expect_true("model_terms" %in% names(ft_result$table$body$dataset))
  expect_equal(
    gt_result$table$`_formats`[[1]]$func$default(c(FALSE, TRUE, FALSE)),
    c("No", "Yes", "No")
  )
  expect_equal(
    gt_result$results_table$model_id[which.min(gt_result$results_table$AIC)],
    max(gt_result$results_table$model_id)
  )
  expect_match(
    as.character(gt_result$table$`_source_notes`[[1]]),
    "Selection direction: forward"
  )
  expect_match(
    as.character(gt_result$table$`_source_notes`[[2]]),
    "Screening aid only"
  )
})

#' Compose footnotes for univariate strata
#' @keywords internal
.footnotes_uni_strata <- function(approach) {
  c(.abbrev_note(approach))
}

#' Describe the stratification variable once without repeating it in headings
#' @keywords internal
#' @noRd
.stratified_by_note <- function(stratifier) {
  paste0("Stratified by: ", stratifier, ".")
}

#' Compose footnotes for multivariable strata (abbrev + per-stratum N)
#' @keywords internal
.footnotes_multi_strata <- function(approach, fits_by_stratum, stratifier) {
  n_lines <- vapply(names(fits_by_stratum), function(k) {
    n_used <- tryCatch(stats::nobs(fits_by_stratum[[k]]), error = function(e) NA_integer_)
    if (!is.na(n_used)) {
      paste0(stratifier, " = ", k, ": N = ", n_used,
             " complete observations included in the multivariable model")
    } else {
      paste0(stratifier, " = ", k, ": complete observations included in the multivariable model")
    }
  }, character(1))
  c(.abbrev_note(approach), n_lines)
}
#' @keywords internal
#' @noRd
.ref_note <- function() {
  "Ref. = reference category."
}

#' @keywords internal
#' @noRd
.validate_show_ref <- function(show_ref) {
  if (!is.logical(show_ref) || length(show_ref) != 1L || is.na(show_ref)) {
    stop("`show_ref` must be TRUE or FALSE.", call. = FALSE)
  }

  invisible(show_ref)
}

#' @keywords internal
#' @noRd
.message_hidden_ref_rows <- function(function_name, table_body, show_ref) {
  if (isTRUE(show_ref) || is.null(table_body) || !"ref" %in% names(table_body)) {
    return(invisible(FALSE))
  }

  if (!any(table_body$ref %in% TRUE, na.rm = TRUE)) {
    return(invisible(FALSE))
  }

  message(
    function_name,
    "(): reference rows are hidden for dichotomous/categorical variables. ",
    "To display Ref., use `show_ref = TRUE`."
  )

  invisible(TRUE)
}
#' @keywords internal
#' @noRd
.adjustment_note <- function(adjust_for) {
  adjust_for <- unname(adjust_for)

  if (length(adjust_for) == 1) {
    return(paste0("Adjusted for ", adjust_for))
  }

  if (length(adjust_for) == 2) {
    return(paste0("Adjusted for ", adjust_for[1], " and ", adjust_for[2]))
  }

  paste0(
    "Adjusted for ",
    paste(adjust_for[-length(adjust_for)], collapse = ", "),
    ", and ",
    adjust_for[length(adjust_for)]
  )
}

#' Compose an adjustment note using displayed variable labels
#' @keywords internal
#' @noRd
.display_adjustment_note <- function(adjust_for, variable_labels = NULL) {
  display_vars <- vapply(
    adjust_for,
    .label_var,
    character(1),
    label_map = variable_labels
  )
  .adjustment_note(display_vars)
}

#' @keywords internal
#' @noRd
.interaction_note <- function(interaction) {
  paste0("Model includes interaction term: ", interaction)
}
#' @keywords internal
#' @noRd
.n_note_multi_strata <- function(stratifier, n_by_stratum) {
  if (!length(n_by_stratum)) {
    return("N reflects complete observations used in each stratum-specific model")
  }

  parts <- paste0(
    names(n_by_stratum),
    ": N = ",
    unlist(n_by_stratum, use.names = FALSE)
  )

  paste0(
    "Complete observations included by ",
    stratifier,
    " stratum: ",
    paste(parts, collapse = "; ")
  )
}

#' Kaplan-Meier risk table
#'
#' Tabulate the number at risk at selected follow-up times.
#'
#' @param data A \code{data.frame} containing survival time and event status.
#' @param time Survival follow-up time. Quoted and bare names are accepted.
#' @param event Event indicator. Quoted and bare names are accepted. Numeric
#'   \code{0/1}, numeric \code{1/2}, logical, character, and factor variables
#'   are accepted. For two-level character or factor variables, the second level
#'   is treated as the event.
#' @param by Optional grouping variable for separate Kaplan-Meier risk tables.
#'   Quoted and bare names are accepted.
#' @param times Numeric vector of follow-up times for the risk table.
#' @param digits Number of digits for displayed follow-up times.
#' @param extend Logical. If \code{TRUE}, requested times beyond the observed
#'   follow-up range are retained using the last available Kaplan-Meier risk set.
#' @param format Output format. One of \code{"flextable"} (default),
#'   \code{"gt"}, or \code{"tibble"}.
#' @param theme Table styling preset.
#'
#' @return A list of class \code{c("gtregression","km_risk_table", ...)}
#' with elements:
#' \describe{
#'   \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
#'   when \code{format = "tibble"}.}
#'   \item{\code{table_body}}{Tibble with Kaplan-Meier risk table counts.}
#'   \item{\code{table_display}}{Display data frame used to render the table.}
#'   \item{\code{fit}}{Fitted \code{survfit} object.}
#'   \item{\code{time,event,by,times,format,source}}{Metadata fields.}
#' }
#'
#' @details
#' The \code{At risk} column gives the number still under observation at each
#' requested time. The \code{Events} and \code{Censored} columns are interval
#' counts up to each requested time point as returned by
#' \code{summary.survfit()}.
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#'
#' km_risk_table(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   by = trt,
#'   times = c(0, 90, 180, 365)
#' )
#'
#' km_risk_table(
#'   data = lung_data,
#'   time = "time",
#'   event = "status",
#'   times = c(0, 90, 180),
#'   format = tibble
#' )
#'
#' @importFrom survival survfit
#' @importFrom stats complete.cases
#' @export
km_risk_table <- function(data,
                          time,
                          event,
                          by = NULL,
                          times,
                          digits = 1,
                          extend = TRUE,
                          format = c("flextable", "gt", "tibble"),
                          theme = c("minimal")) {

  time <- .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  event <- .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  by <- .vars_arg(substitute(by), env = parent.frame(), allow_null = TRUE)
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  theme <- .choice_arg(substitute(theme), env = parent.frame())

  format <- match.arg(format, c("flextable", "gt", "tibble"))
  theme <- .resolve_theme(theme)

  data_clean <- .validate_survival_prob_inputs(
    data = data,
    time = time,
    event = event,
    by = by,
    times = times,
    digits = digits,
    extend = extend
  )

  if (!is.null(by)) {
    data_clean[[by]] <- factor(data_clean[[by]])
  }

  fit <- survival::survfit(.km_formula(time, event, by), data = data_clean)
  sm <- summary(fit, times = times, extend = extend)
  table_body <- .km_risk_table_body(sm, by = by)

  if (format == "tibble") {
    return(tibble::as_tibble(table_body))
  }

  table_display <- .km_risk_table_display(table_body, digits = digits)
  tbl <- .build_km_risk_table(
    table_display,
    format = format,
    theme = theme
  )

  res <- list(
    table = tbl,
    table_body = tibble::as_tibble(table_body),
    table_display = table_display,
    fit = fit,
    time = time,
    event = event,
    by = by,
    times = times,
    extend = extend,
    format = format,
    source = "km_risk_table"
  )

  class(res) <- c("gtregression", "km_risk_table", class(res))
  res
}

#' @keywords internal
#' @noRd
.km_risk_table_body <- function(sm, by = NULL) {
  group <- if (is.null(by)) {
    rep("Overall", length(sm$time))
  } else {
    sub("^.*=", "", sm$strata)
  }

  data.frame(
    Group = group,
    Time = as.numeric(sm$time),
    N.risk = as.numeric(sm$n.risk),
    Events = as.numeric(sm$n.event),
    Censored = as.numeric(sm$n.censor),
    stringsAsFactors = FALSE
  )
}

#' @keywords internal
#' @noRd
.km_risk_table_display <- function(table_body, digits = 1) {
  fmt_time <- function(x) {
    formatC(x, digits = digits, format = "f", big.mark = ",")
  }

  data.frame(
    Group = table_body$Group,
    Time = fmt_time(table_body$Time),
    `At risk` = as.character(table_body$N.risk),
    Events = as.character(table_body$Events),
    Censored = as.character(table_body$Censored),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )
}

#' @keywords internal
#' @noRd
.build_km_risk_table <- function(display,
                                 format = c("flextable", "gt"),
                                 theme = c("minimal")) {
  format <- match.arg(format, c("flextable", "gt"))
  note <- paste(
    "At risk is the number still under observation at the requested time.",
    "Events and censored counts are interval counts up to each requested time point."
  )

  if (format == "gt") {
    tbl <- gt::gt(display) |>
      gt::tab_header(title = "Kaplan-Meier risk table") |>
      gt::cols_align(align = "left", columns = "Group") |>
      gt::cols_align(
        align = "center",
        columns = c("Time", "At risk", "Events", "Censored")
      ) |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_column_labels()
      ) |>
      gt::tab_source_note(gt::md(note)) |>
      .compact_gt_source_notes()

    if ("header_shaded" %in% theme) {
      tbl <- gt::tab_options(tbl, column_labels.background.color = "#f6f8fa")
    }
    if ("zebra" %in% theme) {
      tbl <- gt::opt_row_striping(tbl)
    }
    if ("compact" %in% theme) {
      tbl <- gt::tab_options(tbl, data_row.padding = gt::px(2))
    }
    return(tbl)
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Kaplan-Meier risk table")
  ft <- flextable::align(ft, j = "Group", align = "left", part = "all")
  ft <- flextable::align(
    ft,
    j = c("Time", "At risk", "Events", "Censored"),
    align = "center",
    part = "all"
  )
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  if ("header_shaded" %in% theme) {
    ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  }
  if ("zebra" %in% theme && nrow(display) > 1L) {
    ft <- flextable::bg(ft, i = seq(1, nrow(display), by = 2), bg = "#f6f8fa", part = "body")
  }
  if ("compact" %in% theme) {
    ft <- flextable::padding(ft, padding = 2, part = "body")
  }
  ft <- flextable::add_footer_lines(ft, values = note)
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

lung_km_risk_table_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment"))
    )
}

test_that("km_risk_table returns grouped Kaplan-Meier risk table", {
  skip_if_not_installed("survival")

  df <- lung_km_risk_table_data()

  res <- km_risk_table(
    data = df,
    time = time,
    event = status,
    by = trt,
    times = c(0, 90, 180, 365)
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "km_risk_table")
  expect_s3_class(res$table, "flextable")
  expect_s3_class(res$fit, "survfit")
  expect_equal(res$source, "km_risk_table")
  expect_equal(res$time, "time")
  expect_equal(res$event, "status")
  expect_equal(res$by, "trt")
  expect_equal(res$times, c(0, 90, 180, 365))
  expect_true(res$extend)
  expect_equal(
    res$table_body$Group,
    rep(c("Standard treatment", "Test treatment"), each = 4)
  )
  expect_equal(res$table_body$Time, rep(c(0, 90, 180, 365), times = 2))
  expect_equal(res$table_body$N.risk, c(69, 37, 13, 4, 68, 25, 14, 6))
  expect_equal(res$table_body$Events, c(0, 31, 21, 8, 0, 42, 9, 7))
  expect_equal(res$table_body$Censored, c(0, 1, 3, 1, 0, 2, 1, 1))
  expect_equal(
    names(res$table_display),
    c("Group", "Time", "At risk", "Events", "Censored")
  )
})

test_that("km_risk_table returns overall tibble with quoted names", {
  skip_if_not_installed("survival")

  df <- lung_km_risk_table_data()
  time_var <- "time"
  event_var <- "status"

  out <- km_risk_table(
    data = df,
    time = time_var,
    event = event_var,
    times = c(0, 90, 180),
    format = tibble
  )

  expect_s3_class(out, "tbl_df")
  expect_equal(out$Group, c("Overall", "Overall", "Overall"))
  expect_equal(out$Time, c(0, 90, 180))
  expect_equal(out$N.risk, c(137, 62, 27))
  expect_equal(out$Events, c(0, 73, 30))
  expect_equal(out$Censored, c(0, 3, 4))
})

test_that("km_risk_table returns gt table", {
  skip_if_not_installed("survival")
  skip_if_not_installed("gt")

  df <- lung_km_risk_table_data()

  res <- km_risk_table(
    data = df,
    time = time,
    event = status,
    by = trt,
    times = c(0, 90, 180),
    format = gt
  )

  expect_s3_class(res$table, "gt_tbl")
})

test_that("km_risk_table validates inputs", {
  skip_if_not_installed("survival")

  df <- lung_km_risk_table_data()
  df$bad_event <- 0
  df$one_group <- "A"

  expect_error(
    km_risk_table(data = "not data", time = time, event = status, times = 90),
    "`data` must be"
  )
  expect_error(
    km_risk_table(data = df, time = missing_time, event = status, times = 90),
    "`time` must be"
  )
  expect_error(
    km_risk_table(data = df, time = time, event = missing_event, times = 90),
    "`event` must be"
  )
  expect_error(
    km_risk_table(data = df, time = time, event = status),
    "`times` must be"
  )
  expect_error(
    km_risk_table(data = df, time = time, event = status, times = -1),
    "`times` must be"
  )
  expect_error(
    km_risk_table(data = df, time = time, event = bad_event, times = 90),
    "`event` must include at least one event"
  )
  expect_error(
    km_risk_table(data = df, time = time, event = status, by = one_group, times = 90),
    "`by` must contain at least two"
  )
  expect_error(
    km_risk_table(data = df, time = time, event = status, times = 90, digits = -1),
    "`digits` must be"
  )
  expect_error(
    km_risk_table(data = df, time = time, event = status, times = 90, extend = NA),
    "`extend` must be"
  )
})

#' Draw a forest table from regression outputs
#'
#' Wrapper around \code{forestploter::forest()} that works directly with
#' \code{forest_df()} output or with \code{gtregression} regression objects.
#' It can show descriptive columns and one or more model effect columns in a
#' table-style forest plot while leaving axis and interval drawing to
#' \pkg{forestploter}.
#'
#' @param df Output of \code{forest_df()}. If \code{NULL}, it is built from
#'   \code{uni}, \code{multi}, and \code{desc}.
#' @param uni,multi,desc Optional \code{gtregression} objects to pass through to
#'   \code{forest_df()}.
#' @param theme Optional \code{forestploter::forest_theme()}. If \code{NULL},
#'   \code{forestploter} defaults are used. You may pass colors and styling
#'   either here or through \code{...}.
#' @param ci_col_width Numeric value, or one value per effect column,
#'   controlling the blank spacer width used by \code{forestploter} for the
#'   confidence-interval plot column(s). Values greater than 1 are interpreted
#'   as approximate character counts. Values between 0 and 1 are accepted for
#'   backward compatibility and converted to character counts.
#' @param side Character. For each effect, position of the plot relative to the effect-size text:
#'   \code{"left"} = plot first then text; \code{"right"} = text first then plot.
#'   The \code{Characteristic} column and descriptive columns remain on the left.
#' @param quiet Logical. Suppress forestploter warnings. Default = `TRUE`.
#' @param effects Optional effect labels passed to \code{forestploter::forest()}.
#' @param ticks_at Optional numeric vector, or list of numeric vectors for
#'   multiple effect
#'   columns, specifying x-axis tick positions. If \code{NULL},
#'   \code{forestploter::forest()} chooses the default ticks.
#' @param ticks_digits Optional number of digits for x-axis tick labels.
#' @param xlim Optional numeric vector of length 2, or list of length-2 numeric
#'   vectors for multiple
#'   effect columns, specifying x-axis limits. If \code{NULL},
#'   \code{forestploter::forest()} chooses the default limits.
#' @param style_strata Logical. Retained for older vertically grouped
#'   stratified forest data. Current \code{forest_df()} stratified output places
#'   strata side by side, so no stratum header rows are styled.
#' @param strata_fill Character. Fill color used for older styled stratum header
#'   rows.
#' @param ... Passed to \code{forestploter::forest()}. Common options include
#'   \code{title} and \code{footnote}.
#'
#' @return A \code{gtregression_forest} object with elements:
#' \describe{
#'   \item{\code{plot}}{The forest plot object.}
#'   \item{\code{data}}{The plotting data sent to \code{forestploter::forest()}.}
#'   \item{\code{input_data}}{The original \code{forest_df()} data, including
#'   standard-error helper columns.}
#'   \item{\code{meta}}{Model metadata, including reference line and x-axis
#'   transformation.}
#' }
#' @importFrom forestploter forest
#' @examples
#' birthwt_data <- data_birthwt |>
#'   transform(
#'     smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
#'     ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
#'     low = factor(low, levels = c(0, 1),
#'                  labels = c("Normal BW", "Low BW"))
#'   )
#'
#' uni_or <- uni_reg(
#'   birthwt_data,
#'   outcome = "low",
#'   exposures = c("age", "lwt", "smoke", "ht"),
#'   approach = "logit"
#' )
#' multi_or <- multi_reg(
#'   birthwt_data,
#'   outcome = "low",
#'   exposures = c("smoke", "ht"),
#'   adjust_for = c("age", "lwt"),
#'   approach = "logit"
#' )
#'
#' forest_reg(uni = uni_or, multi = multi_or)
#'
#' # If axis labels overlap, set x-axis limits and tick marks.
#' forest_reg(
#'   uni = uni_or,
#'   multi = multi_or,
#'   xlim = list(c(0.25, 8), c(0.25, 12)),
#'   ticks_at = list(c(0.5, 1, 2, 4, 8), c(0.5, 1, 2, 4, 8))
#' )
#'
#' # If the forest plot panel is too narrow or too wide, tune ci_col_width.
#' forest_reg(
#'   uni = uni_or,
#'   multi = multi_or,
#'   ci_col_width = c(18, 22)
#' )

#' @export
forest_reg <- function(df = NULL, uni = NULL, multi = NULL, desc = NULL,
                       theme = NULL,
                       ci_col_width = 20,
                       side = c("right", "left"),
                       quiet = TRUE,
                       effects = NULL,
                       ticks_at = NULL,
                       ticks_digits = NULL,
                       xlim = NULL,
                       style_strata = TRUE,
                       strata_fill = "#EAF2F1",
                       ...) {

  side <- .choice_arg(substitute(side), env = parent.frame(), choices = c("right", "left"))
  side <- match.arg(side)
  if (!is.logical(quiet) || length(quiet) != 1L || is.na(quiet)) {
    stop("`quiet` must be TRUE or FALSE.", call. = FALSE)
  }
  if (!is.null(effects) && !is.character(effects)) {
    stop("`effects` must be NULL or a character vector.", call. = FALSE)
  }
  if (!is.numeric(ci_col_width) || anyNA(ci_col_width) ||
      !length(ci_col_width) || any(ci_col_width <= 0)) {
    stop("`ci_col_width` must be a positive numeric value or numeric vector.",
         call. = FALSE)
  }
  ci_col_width <- ifelse(ci_col_width <= 1, ci_col_width * 80, ci_col_width)
  ci_col_width <- pmax(4L, as.integer(round(ci_col_width)))
  validate_ticks <- function(x, name) {
    if (is.null(x)) return(invisible(NULL))
    ok <- is.numeric(x) && length(x) > 0L && !anyNA(x)
    if (!ok) {
      stop("`", name, "` must be NULL or a numeric vector.", call. = FALSE)
    }
    invisible(NULL)
  }
  if (is.list(ticks_at)) {
    if (!length(ticks_at)) {
      stop("`ticks_at` must be NULL, a numeric vector, or a non-empty list of numeric vectors.",
           call. = FALSE)
    }
    lapply(ticks_at, validate_ticks, name = "ticks_at")
  } else {
    validate_ticks(ticks_at, "ticks_at")
  }
  validate_xlim <- function(x) {
    if (is.null(x)) return(invisible(NULL))
    ok <- is.numeric(x) && length(x) == 2L && !anyNA(x) && x[1] < x[2]
    if (!ok) {
      stop("`xlim` must be NULL, a numeric vector of length 2, or a non-empty list of numeric length-2 vectors.",
           call. = FALSE)
    }
    invisible(NULL)
  }
  if (is.list(xlim)) {
    if (!length(xlim)) {
      stop("`xlim` must be NULL, a numeric vector of length 2, or a non-empty list of numeric length-2 vectors.",
           call. = FALSE)
    }
    lapply(xlim, validate_xlim)
  } else {
    validate_xlim(xlim)
  }
  if (!is.null(ticks_digits) &&
      (!is.numeric(ticks_digits) || length(ticks_digits) != 1L ||
       is.na(ticks_digits) || ticks_digits < 0)) {
    stop("`ticks_digits` must be NULL or a single non-negative number.", call. = FALSE)
  }
  if (!is.null(ticks_digits)) ticks_digits <- as.integer(ticks_digits)
  if (!is.logical(style_strata) || length(style_strata) != 1L || is.na(style_strata)) {
    stop("`style_strata` must be TRUE or FALSE.", call. = FALSE)
  }
  if (!is.character(strata_fill) || length(strata_fill) != 1L || is.na(strata_fill)) {
    stop("`strata_fill` must be a single character color.", call. = FALSE)
  }

  # build df if needed
  if (is.null(df)) {
    if (is.null(uni) && inherits(multi, "gtregression")) {
      df <- forest_df(uni = multi, desc = desc)
    } else if (!inherits(uni, "gtregression")) {
      stop(
        "Provide `df = forest_df(...)`, a `gtregression` object in `uni`, or a `multi_reg()` object in `multi`.",
        call. = FALSE
      )
    } else {
      df <- forest_df(uni = uni, multi = multi, desc = desc)
    }
  } else if (!is.data.frame(df)) {
    stop("`df` must be a data frame created by `forest_df()`.", call. = FALSE)
  }

  if (!is.null(df) && !is.null(uni) && is.data.frame(uni)) {
    stop(
      "When `df` is supplied, do not pass another data frame as the second argument. ",
      "Use `forest_df(uni, multi, desc = ...)` to combine inputs before calling `forest_reg()`.",
      call. = FALSE
    )
  }

  if (!("Characteristic" %in% names(df))) {
    stop("`df` must contain a `Characteristic` column.", call. = FALSE)
  }
  if (is.null(attr(df, "est")) || !("se_uni" %in% names(df)) ||
      !is.numeric(df$se_uni)) {
    stop(
      "`df` does not contain forest plot estimates. ",
        "Use `forest_df()` with a compatible regression object; ",
        "a descriptive-only `forest_df()` cannot be drawn by `forest_reg()`.",
        call. = FALSE
    )
  }

  # detect meta
  meta <- attr(df, "forest_meta")
  if (is.null(meta)) {
    meta <- list(x_trans = "none", ref_line = 0)
  }

  # choose estimate sets
  effect_sets <- .forest_effect_sets(df)
  est <- if (length(effect_sets$est) == 1L) effect_sets$est[[1]] else effect_sets$est
  lo <- if (length(effect_sets$lo) == 1L) effect_sets$lo[[1]] else effect_sets$lo
  hi <- if (length(effect_sets$hi) == 1L) effect_sets$hi[[1]] else effect_sets$hi

  df_plot <- df[, setdiff(names(df), c("se_uni","se_adj")), drop = FALSE]

  # --- position the plot columns per `side` and recompute ci_col ---
  # --- enforce layout rules: Characteristics/desc on the left; plot/text order by `side` ---

  nm <- names(df_plot)

  # 1) Identify CI text columns and anchor (plot) columns
  ci_cols_all <- grep("\\(95% CI\\)$", nm, value = TRUE)
  anchor_cols <- effect_sets$anchors
  text_cols <- effect_sets$text_cols
  if (!length(anchor_cols) || !length(text_cols) ||
      length(anchor_cols) != length(text_cols)) {
    stop("`df` does not contain matched forest plot columns.", call. = FALSE)
  }
  if (length(ci_col_width) == 1L) {
    ci_col_width <- rep(ci_col_width, length(anchor_cols))
  } else if (length(ci_col_width) != length(anchor_cols)) {
    stop(
      "`ci_col_width` must be length 1 or have one value per effect column.",
      call. = FALSE
    )
  }
  if (interactive() && is.null(ticks_at) && is.null(xlim)) {
    message(
      "forest_reg(): x-axis ticks use forestploter defaults. If labels overlap in your graphics device, ",
      "set `xlim` and `ticks_at` (use one vector per effect column when applicable) ",
      "or increase `ci_col_width`."
    )
  }
  for (i in seq_along(anchor_cols)) {
    df_plot[[anchor_cols[i]]] <- paste(rep(" ", ci_col_width[i]), collapse = "")
  }

  # 2) Characteristic + desc columns = everything that's not an anchor or CI text
  left_cols <- setdiff(nm, c(anchor_cols, ci_cols_all))   # preserve original order

  # 3) Build the effect-area blocks based on `side`
  blocks <- character(0)
  for (i in seq_along(anchor_cols)) {
    blocks <- c(
      blocks,
      if (side == "left") c(anchor_cols[i], text_cols[i]) else c(text_cols[i], anchor_cols[i])
    )
  }

  # 4) Reorder df_plot: left (Characteristic/desc) | effect blocks
  df_plot <- df_plot[, c(left_cols, blocks), drop = FALSE]

  # 5) Recompute ci_column indices (must be the plot-anchor columns in the same order as in `blocks`)
  anchor_in_blocks <- blocks[blocks %in% anchor_cols]
  ci_col <- match(anchor_in_blocks, names(df_plot))

  draw <- function() forestploter::forest(
    data       = df_plot,
    est        = est,
    lower      = lo,
    upper      = hi,
    ci_column  = ci_col,
    x_trans    = meta$x_trans,
    ref_line   = meta$ref_line,
    xlim       = xlim,
    ticks_at   = ticks_at,
    ticks_digits = ticks_digits,
    theme      = theme,
    effects    = effects,
    ...
  )

  plt <- if (quiet) suppressWarnings(draw()) else draw()

  strata_rows <- .forest_stratum_rows(df_plot, meta)
  if (isTRUE(style_strata) && length(strata_rows)) {
    plt <- .forest_style_stratum_rows(
      plot = plt,
      rows = strata_rows,
      cols = seq_along(df_plot),
      fill = strata_fill
    )
  }

  meta$ticks_at <- ticks_at
  meta$ticks_digits <- ticks_digits
  meta$xlim <- xlim
  meta$ci_col_width <- ci_col_width
  meta$strata_rows <- strata_rows
  meta$style_strata <- style_strata
  meta$strata_fill <- strata_fill

  structure(list(plot = plt, data = df_plot, input_data = df, meta = meta),
            class = c("gtregression_forest", "list"))
}

.forest_effect_sets <- function(df) {
  custom <- attr(df, "forest_estimates", exact = TRUE)
  if (!is.null(custom)) {
    out <- list(
      est = custom$est,
      lo = custom$lo,
      hi = custom$hi,
      anchors = custom$anchors,
      text_cols = custom$text_cols
    )
    if (length(out$est) && length(out$lo) && length(out$hi) &&
        length(out$anchors) && length(out$text_cols)) {
      return(out)
    }
  }

  ci_cols_all <- grep("\\(95% CI\\)$", names(df), value = TRUE)
  anchor_cols <- names(df)[grepl("^ +$", names(df))]

  if (!is.null(attr(df, "est2", exact = TRUE))) {
    text_cols <- character(0)
    ci_uni <- setdiff(ci_cols_all, ci_cols_all[grepl("^Adjusted\\s+", ci_cols_all)])
    ci_adj <- ci_cols_all[grepl("^Adjusted\\s+", ci_cols_all)]
    text_cols <- c(ci_uni[1], ci_adj[1])
    text_cols <- text_cols[!is.na(text_cols)]

    return(list(
      est = list(attr(df, "est", exact = TRUE), attr(df, "est2", exact = TRUE)),
      lo = list(attr(df, "lo", exact = TRUE), attr(df, "lo2", exact = TRUE)),
      hi = list(attr(df, "hi", exact = TRUE), attr(df, "hi2", exact = TRUE)),
      anchors = intersect(c(" ", "  "), anchor_cols),
      text_cols = text_cols
    ))
  }

  text_col <- ci_cols_all[1]
  list(
    est = list(attr(df, "est", exact = TRUE)),
    lo = list(attr(df, "lo", exact = TRUE)),
    hi = list(attr(df, "hi", exact = TRUE)),
    anchors = anchor_cols[1],
    text_cols = text_col
  )
}

.forest_stratum_rows <- function(df_plot, meta) {
  if (!isTRUE(meta$stratified) || is.null(meta$stratifier) || is.null(meta$strata)) {
    return(integer())
  }

  if (isTRUE(meta$side_by_side_strata)) {
    return(integer())
  }

  labels <- meta$strata
  which(df_plot$Characteristic %in% labels)
}

.forest_style_stratum_rows <- function(plot, rows, cols, fill) {
  for (row in rows) {
    plot <- forestploter::edit_plot(
      plot,
      row = row,
      col = cols,
      part = "body",
      which = "background",
      gp = grid::gpar(fill = fill, col = NA)
    )
    plot <- forestploter::edit_plot(
      plot,
      row = row,
      col = cols,
      part = "body",
      which = "text",
      gp = grid::gpar(fontface = "bold")
    )
  }
  plot
}

#' @export
print.gtregression_forest <- function(x, ..., autofit = FALSE) {
  if (!is.logical(autofit) || length(autofit) != 1L || is.na(autofit)) {
    stop("`autofit` must be TRUE or FALSE.", call. = FALSE)
  }
  if (isTRUE(autofit)) {
    warning(
      "`autofit = TRUE` is ignored for `forest_reg()` because it can compress ",
      "forest plot columns and cause overlap. Printing with `autofit = FALSE`. ",
      "Use a wider graphics device or export canvas for more side space.",
      call. = FALSE
    )
    autofit <- FALSE
  }
  print(x$plot, autofit = autofit)
  invisible(x)
}

# ---------- display-label relabelling ----------
#' @keywords internal
#' @noRd
.relabel_display <- function(display_df,
                             variable_labels = NULL,
                             level_labels    = NULL) {
  df <- as.data.frame(display_df, stringsAsFactors = FALSE)
  stopifnot(all(c("Characteristic","is_header") %in% names(df)))

  .must_be_named_character(variable_labels, "variable_labels", allow_null = TRUE)
  if (!is.null(level_labels) &&
      (!is.list(level_labels) || is.null(names(level_labels)) ||
       any(!nzchar(names(level_labels))))) {
    stop("`level_labels` must be a named list.", call. = FALSE)
  }
  if (!is.null(level_labels)) {
    for (nm in names(level_labels)) {
      .must_be_named_character(level_labels[[nm]], paste0("level_labels$", nm), allow_null = FALSE)
    }
  }

  # carry variable name down so we can match levels
  raw_exposure <- attr(display_df, "row_exposure", exact = TRUE)
  if (!is.null(raw_exposure) && length(raw_exposure) == nrow(df)) {
    var_id <- as.character(raw_exposure)
  } else {
    var_id <- character(nrow(df))
    cur <- NA_character_
    for (i in seq_len(nrow(df))) {
      if (isTRUE(df$is_header[i])) cur <- trimws(df$Characteristic[i])
      var_id[i] <- cur
    }
  }

  # variable (header) relabel
  if (!is.null(variable_labels)) {
    hdr_idx <- which(df$is_header %in% TRUE)
    if (length(hdr_idx)) {
      old_hdr <- var_id[hdr_idx]
      idx <- match(old_hdr, names(variable_labels))
      repl <- variable_labels[idx]
      df$Characteristic[hdr_idx] <- ifelse(is.na(idx), df$Characteristic[hdr_idx], unname(repl))
    }
  }

  # level relabel (keep indent)
  if (!is.null(level_labels)) {
    lvl_idx <- which(df$is_header %in% FALSE)
    if (length(lvl_idx)) {
      base_lvl <- sub("^\\s+", "", df$Characteristic[lvl_idx])
      df$Characteristic[lvl_idx] <- mapply(
        FUN = function(v, lv, original) {
          ll <- level_labels[[v]]
          if (is.null(ll)) return(original)
          hit <- match(lv, names(ll))
          if (is.na(hit)) return(original)
          paste0("  ", unname(ll[hit]))
        },
        v = var_id[lvl_idx], lv = base_lvl, original = df$Characteristic[lvl_idx],
        SIMPLIFY = TRUE, USE.NAMES = FALSE
      )
    }
  }

  .attach_display_metadata(
    df,
    row_exposure = var_id,
    variable_labels = attr(display_df, "variable_labels", exact = TRUE)
  )
}

#' Validate named character vector inputs
#' @keywords internal
#' @noRd
.must_be_named_character <- function(x, arg, allow_null = TRUE) {
  if (is.null(x) && isTRUE(allow_null)) {
    return(invisible(TRUE))
  }
  if (!is.character(x) || is.null(names(x)) || any(!nzchar(names(x)))) {
    stop("`", arg, "` must be a named character vector.", call. = FALSE)
  }
  invisible(TRUE)
}

#' Normalize common header aliases to visible table_display names
#' @keywords internal
#' @noRd
.normalize_header_labels <- function(header_labels, display_df, approach = NULL) {
  .must_be_named_character(header_labels, "header_labels", allow_null = TRUE)
  if (is.null(header_labels)) {
    return(NULL)
  }

  out <- header_labels
  effect_cols <- setdiff(names(display_df), c("Characteristic", "is_header", "N", "p-value"))
  effect_col <- if (length(effect_cols) == 1L) effect_cols else NA_character_
  aliases <- c(
    estimate = effect_col,
    p.value = "p-value",
    p_value = "p-value",
    pvalue = "p-value",
    N = "N"
  )

  for (nm in names(out)) {
    if (!nm %in% names(display_df) && nm %in% names(aliases) && !is.na(aliases[[nm]])) {
      names(out)[names(out) == nm] <- aliases[[nm]]
    }
  }

  bad <- setdiff(names(out), names(display_df))
  if (length(bad)) {
    stop(
      "`header_labels` contains columns not found in the table: ",
      paste(bad, collapse = ", "),
      call. = FALSE
    )
  }

  out
}

#' Drop display columns while preserving row metadata
#' @keywords internal
#' @noRd
.drop_display_columns <- function(display_df, cols) {
  cols <- intersect(cols, names(display_df))
  if (!length(cols)) {
    return(display_df)
  }

  out <- display_df[, setdiff(names(display_df), cols), drop = FALSE]
  attr(out, "row_exposure") <- attr(display_df, "row_exposure", exact = TRUE)
  attr(out, "variable_labels") <- attr(display_df, "variable_labels", exact = TRUE)
  out
}

#' Check for stratified regression output classes
#' @keywords internal
#' @noRd
.is_stratified_regression_output <- function(tbl) {
  inherits(tbl, "stratified_uni_reg") ||
    inherits(tbl, "stratified_multi_reg") ||
    inherits(tbl, "stratified_cox_reg") ||
    inherits(tbl, "stratified_surv_reg")
}

#' Pull a shared stratified effect label from header aliases
#' @keywords internal
#' @noRd
.stratified_effect_label <- function(header_labels, default) {
  if (is.null(header_labels)) {
    return(default)
  }

  allowed <- c("estimate", default)
  bad <- setdiff(names(header_labels), allowed)
  if (length(bad)) {
    stop(
      "`header_labels` contains columns not found in the stratified table: ",
      paste(bad, collapse = ", "),
      call. = FALSE
    )
  }

  if ("estimate" %in% names(header_labels)) {
    return(unname(header_labels[["estimate"]]))
  }
  if (default %in% names(header_labels)) {
    return(unname(header_labels[[default]]))
  }

  default
}

#' Rebuild a modified stratified table with the original strata builders
#' @keywords internal
#' @noRd
.rebuild_modified_stratified_table <- function(tbl, display_df, footnotes, header_labels = NULL) {
  if (!.is_stratified_regression_output(tbl)) {
    return(NULL)
  }

  spanners <- tbl$levels
  theme <- if (!is.null(tbl$theme)) tbl$theme else "minimal"
  format <- tolower(tbl$format)

  if (inherits(tbl, "stratified_uni_reg")) {
    effect_label <- .stratified_effect_label(header_labels, .get_effect_label(tbl$approach))
    if (identical(format, "gt")) {
      return(.build_gt_strata_wide_uni(display_df, spanners, effect_label, theme, footnotes))
    }
    return(.build_flex_strata_wide_uni(display_df, spanners, effect_label, theme, footnotes))
  }

  if (inherits(tbl, "stratified_multi_reg")) {
    effect_label <- .stratified_effect_label(
      header_labels,
      paste("Adjusted", .get_effect_label(tbl$approach))
    )
    if (identical(format, "gt")) {
      return(.build_gt_strata_wide_multi(display_df, spanners, effect_label, theme, footnotes))
    }
    return(.build_flex_strata_wide_multi(display_df, spanners, effect_label, theme, footnotes))
  }

  if (inherits(tbl, "stratified_cox_reg")) {
    effect_label <- .stratified_effect_label(
      header_labels,
      if (isTRUE(tbl$adjusted_mode) || isTRUE(tbl$multivariable)) {
      "Adjusted HR (95% CI)"
    } else {
      "HR (95% CI)"
      }
    )
    builder <- if (isTRUE(tbl$adjusted_mode) || isTRUE(tbl$multivariable)) {
      if (identical(format, "gt")) .build_gt_strata_wide_multi else .build_flex_strata_wide_multi
    } else {
      if (identical(format, "gt")) .build_gt_strata_wide_uni else .build_flex_strata_wide_uni
    }
    return(builder(display_df, spanners, effect_label, theme, footnotes))
  }

  if (inherits(tbl, "stratified_surv_reg")) {
    effect_label <- .stratified_effect_label(
      header_labels,
      if (isTRUE(tbl$adjusted_mode) || isTRUE(tbl$multivariable)) {
      "Adjusted Time Ratio (95% CI)"
    } else {
      "Time Ratio (95% CI)"
      }
    )
    builder <- if (isTRUE(tbl$adjusted_mode) || isTRUE(tbl$multivariable)) {
      if (identical(format, "gt")) .build_gt_strata_wide_multi else .build_flex_strata_wide_multi
    } else {
      if (identical(format, "gt")) .build_gt_strata_wide_uni else .build_flex_strata_wide_uni
    }
    return(builder(display_df, spanners, effect_label, theme, footnotes))
  }

  NULL
}

#' Validate logical scalar inputs
#' @keywords internal
#' @noRd
.must_be_flag <- function(x, arg) {
  if (!is.logical(x) || length(x) != 1L || is.na(x)) {
    stop("`", arg, "` must be TRUE or FALSE.", call. = FALSE)
  }
  invisible(TRUE)
}

# ---------- main: modify labels/headers/notes and rebuild (gtregression only) ----------
#' Modify Regression/Descriptive Tables (labels, headers, caption, notes)
#'
#' Works with objects created by this package (class \code{"gtregression"}):
#' \code{uni_reg()}, \code{multi_reg()}, \code{cox_reg()}, \code{surv_reg()},
#' stratified regression outputs, \code{descriptive_table()}, and
#' \code{merge_tables()}.
#' No \pkg{gtsummary} dependency or fallback.
#'
#' @param gt_table Table object produced by this package (must contain
#'   \code{$table_display}).
#' @param variable_labels Named character vector, for example
#'   \code{c(old_var = "New label", ...)}.
#' @param level_labels Named list for factor levels:
#'   \code{list(var1 = c(old = "New", ...), var2 = c(...))}.
#' @param header_labels Named character vector to rename visible headers, e.g.
#'   \code{c("OR (95\% CI)" = "Crude OR", "p-value" = "P")}. Common aliases
#'   such as \code{estimate}, \code{p.value}, and \code{N} are also accepted.
#' @param caption Optional caption/title.
#' @param bold_labels Logical; bold variable (header) rows in the body. Defaults
#'   to \code{TRUE} to preserve the package table hierarchy.
#' @param bold_levels Logical; bold factor level rows in the body.
#' @param italic_labels Logical; italicize variable (header) rows in the body.
#' @param italic_levels Logical; italicize factor level rows in the body.
#' @param remove_N Logical; if \code{TRUE}, drops displayed \code{N} columns
#'   from univariable and stratified package tables. For stratified survival
#'   outputs, event columns are retained unless the original table was created
#'   with \code{show_sample = "none"}.
#' @param remove_N_obs Logical; if \code{TRUE}, suppresses multivariable
#'   complete-case footnote.
#' @param remove_abbreviations Logical; if \code{TRUE}, removes the
#'   Abbreviations footnote line.
#' @param remove_adjustment_note Logical; if \code{TRUE}, removes the automatic
#'   \code{Adjusted for ...} footnote. Use \code{caveat} to add customised
#'   wording.
#' @param caveat Optional extra footnote.
#' @return The modified table object (same class as input).
#'
#' @examples
#' birthwt_data <- data_birthwt |>
#'   dplyr::mutate(
#'     smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
#'     ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
#'     low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
#'   )
#'
#' tbl <- uni_reg(
#'   data = birthwt_data,
#'   outcome = "low",
#'   exposures = c("age", "smoke", "ht"),
#'   approach = "logit",
#'   format = "gt"
#' )
#'
#' modify_table(
#'   tbl,
#'   variable_labels = c(age = "Maternal age", smoke = "Smoking"),
#'   level_labels = list(smoke = c(Yes = "Smoker")),
#'   header_labels = c(estimate = "Crude OR", p.value = "P"),
#'   caption = "Univariable regression for low birth weight"
#' )$table
#' @export
modify_table <- function(gt_table,
                         variable_labels = NULL,
                         level_labels = NULL,
                         header_labels = NULL,
                         caption = NULL,
                         bold_labels = TRUE,
                         bold_levels = FALSE,
                         italic_labels = FALSE,
                         italic_levels = FALSE,
                         remove_N = FALSE,
                         remove_N_obs = FALSE,
                         remove_abbreviations = FALSE,
                         remove_adjustment_note = FALSE,
                         caveat = NULL) {
  tbl <- gt_table

  # --------- accept only our objects ----------
  is_pkg_obj <- inherits(tbl, "gtregression") && !is.null(tbl$table_display)
  if (!is_pkg_obj) {
    stop("`modify_table()` expects a gtregression object from this package (with $table_display).",
         call. = FALSE)
  }

  .must_be_named_character(variable_labels, "variable_labels", allow_null = TRUE)
  .must_be_named_character(header_labels, "header_labels", allow_null = TRUE)
  .must_be_flag(bold_labels, "bold_labels")
  .must_be_flag(bold_levels, "bold_levels")
  .must_be_flag(italic_labels, "italic_labels")
  .must_be_flag(italic_levels, "italic_levels")
  .must_be_flag(remove_N, "remove_N")
  .must_be_flag(remove_N_obs, "remove_N_obs")
  .must_be_flag(remove_abbreviations, "remove_abbreviations")
  .must_be_flag(remove_adjustment_note, "remove_adjustment_note")
  if (!is.null(caption) && (!is.character(caption) || length(caption) != 1L || is.na(caption))) {
    stop("`caption` must be NULL or a single character string.", call. = FALSE)
  }
  if (!is.null(caveat) && (!is.character(caveat) || length(caveat) != 1L || is.na(caveat))) {
    stop("`caveat` must be NULL or a single character string.", call. = FALSE)
  }

  # --------- relabel display safely ----------
  tbl$table_display <- .relabel_display(
    tbl$table_display,
    variable_labels = variable_labels,
    level_labels    = level_labels
  )
  if (!is.null(variable_labels)) {
    current_labels <- tbl$variable_labels
    if (is.null(current_labels)) current_labels <- character(0)
    current_labels[names(variable_labels)] <- unname(variable_labels)
    tbl$variable_labels <- current_labels
    attr(tbl$table_display, "variable_labels") <- current_labels
  }

  # --------- optionally drop N columns ----------
  if (isTRUE(remove_N)) {
    n_cols <- names(tbl$table_display)[
      names(tbl$table_display) == "N" |
        startsWith(names(tbl$table_display), "..N__")
    ]
    tbl$table_display <- .drop_display_columns(tbl$table_display, n_cols)
  }

  if (!.is_stratified_regression_output(tbl)) {
    header_labels <- .normalize_header_labels(header_labels, tbl$table_display, tbl$approach)
  }

  # --------- collect/compose footnotes ----------
  footnotes <- character(0)
  # Existing package objects store their complete display notes in `$footnotes`.
  # Retain these first so a presentation-only modification never loses them.
  if (!is.null(tbl$footnotes)) footnotes <- c(footnotes, tbl$footnotes)
  # Some older objects attach notes to the object or rendered inner table.
  ft_attr1 <- attr(tbl, "footnotes", exact = TRUE)
  if (!is.null(ft_attr1)) footnotes <- c(footnotes, ft_attr1)
  ft_attr2 <- if (!is.null(tbl$table)) attr(tbl$table, "footnotes", exact = TRUE) else NULL
  if (!is.null(ft_attr2)) footnotes <- c(footnotes, ft_attr2)
  # abbreviations line (unless removed)
  if (!isTRUE(remove_abbreviations) && !is.null(tbl$approach) &&
      exists(".abbrev_note", mode = "function")) {
    footnotes <- c(footnotes, .abbrev_note(tbl$approach))
  }
  if (isTRUE(remove_abbreviations)) {
    footnotes <- footnotes[!grepl("^Abbreviations:", footnotes)]
  }
  # multivariable N note (unless removed)
  if (!isTRUE(remove_N_obs) && inherits(tbl, "multi_reg")) {
    if (!is.null(tbl$models) && !is.null(tbl$models$multivariable_model)) {
      n_used <- tryCatch(stats::nobs(tbl$models$multivariable_model),
                         error = function(e) NA_integer_)
      if (!is.na(n_used)) {
        footnotes <- c(footnotes,
                       paste0("N = ", n_used,
                              " complete observations included in the model."))
      }
    }
  }
  if (isTRUE(remove_N_obs)) {
    footnotes <- footnotes[!grepl("^N = .*complete observations", footnotes)]
  }
  # default descriptive notes (if none)
  if (!length(footnotes) && inherits(tbl, "descriptive_table")) {
    footnotes <- c(
      "Categorical variables shown as n (%).",
      "Continuous variables shown as Median (IQR)."
    )
  }
  # Keep the multivariable adjustment note aligned with the labels currently
  # visible in the table, including adjustment variables not shown as rows.
  if (inherits(tbl, c("multi_reg", "cox_reg", "surv_reg")) &&
      !is.null(tbl$adjust_for) &&
      length(tbl$adjust_for) && exists(".display_adjustment_note", mode = "function")) {
    footnotes <- footnotes[!grepl("^Adjusted for ", footnotes)]
    footnotes <- c(
      footnotes,
      .display_adjustment_note(tbl$adjust_for, tbl$variable_labels)
    )
  }
  if (isTRUE(remove_adjustment_note)) {
    footnotes <- footnotes[!grepl("^Adjusted for ", footnotes)]
  }
  # user caveat last
  if (!is.null(caveat) && nzchar(caveat)) footnotes <- c(footnotes, caveat)
  footnotes <- unique(footnotes[nzchar(footnotes)])
  tbl$footnotes <- footnotes
  if (!is.null(caption)) {
    tbl$caption <- caption
  }

  # --------- reorder visible columns for univariate: N → Effect → p-value ----------
  reorder_univariate_cols <- function(display_df) {
    stopifnot(all(c("Characteristic","is_header") %in% names(display_df)))
    cols <- names(display_df)
    body_cols <- setdiff(cols, c("Characteristic","is_header"))
    if (length(body_cols)) {
      # classic uni layout has exactly one effect col + optional N + p-value
      hasN   <- "N" %in% body_cols
      hasP   <- "p-value" %in% body_cols
      effect <- setdiff(body_cols, c("N","p-value"))
      if (hasP && length(effect) == 1L) {
        ord <- c("Characteristic", if (hasN) "N", effect, "p-value")
        return(display_df[, ord, drop = FALSE])
      }
    }
    # otherwise: drop is_header and keep as-is
    display_df[, setdiff(names(display_df), "is_header"), drop = FALSE]
  }

  # --------- builders (gt / flextable) ----------
  .build_pkg_gt <- function(display_df, header_labels, footnotes) {
    stopifnot(requireNamespace("gt", quietly = TRUE))
    out <- reorder_univariate_cols(display_df)

    merged_labels <- NULL
    if (inherits(tbl, "merged_table") && !is.null(tbl$column_labels)) {
      merged_labels <- c(
        Characteristic = "Characteristic",
        unlist(tbl$column_labels, use.names = TRUE)
      )
      merged_labels <- merged_labels[names(merged_labels) %in% names(out)]
    }

    lab_values <- if (!is.null(merged_labels)) merged_labels else header_labels
    if (!is.null(header_labels)) {
      lab_values[names(header_labels)] <- unname(header_labels)
    }
    lab_list <- if (!is.null(lab_values)) as.list(lab_values) else NULL
    names(lab_list) <- names(lab_values)

    tb <- gt::gt(out) |>
      gt::cols_label(.list = lab_list) |>
      gt::cols_align("left",   columns = "Characteristic") |>
      gt::cols_align("center", columns = setdiff(names(out), "Characteristic")) |>
      gt::sub_missing(columns = setdiff(names(out), "Characteristic"), missing_text = "") |>
      gt::tab_options(
        table.font.names         = "system-ui",
        data_row.padding         = gt::px(4),
        heading.background.color = "white",
        table.background.color   = "white",
        source_notes.padding     = gt::px(2)
      )

    # bold column labels
    tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                        locations = gt::cells_column_labels())
    # bold variable headers (on demand; note: many tables already bold by default)
    if (isTRUE(bold_labels)) {
      hdr_idx <- which(display_df$is_header %in% TRUE)
      if (length(hdr_idx)) {
        tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                            locations = gt::cells_body(rows = hdr_idx, columns = "Characteristic"))
      }
    }
    # optional bold levels
    if (isTRUE(bold_levels)) {
      lvl_idx <- which(display_df$is_header %in% FALSE)
      if (length(lvl_idx)) {
        tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                            locations = gt::cells_body(rows = lvl_idx, columns = "Characteristic"))
      }
    }
    # indent levels
    tb <- gt::tab_style(tb, gt::cell_text(indent = gt::px(12)),
                        locations = gt::cells_body(rows = display_df$is_header %in% FALSE,
                                                   columns = "Characteristic"))
    if (inherits(tbl, "merged_table") && !is.null(tbl$spanners) &&
        !is.null(tbl$column_labels)) {
      start <- 2L
      for (i in seq_along(tbl$spanners)) {
        cols_i <- names(tbl$column_labels[[i]])
        cols_i <- cols_i[cols_i %in% names(out)]
        k <- length(cols_i)
        if (k > 0) {
          tb <- gt::tab_spanner(
            tb,
            label = tbl$spanners[i],
            columns = names(out)[start:(start + k - 1L)]
          )
          start <- start + k
        }
      }
      tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                          locations = gt::cells_column_spanners())
    }
    # caption
    if (!is.null(caption)) tb <- gt::tab_caption(tb, caption = caption)
    # footnotes
    if (length(footnotes)) {
      tb <- gt::tab_source_note(tb, source_note = gt::md(paste(footnotes, collapse = "  \n")))
    }
    tb
  }

  .build_pkg_flex <- function(display_df, header_labels, footnotes) {
    stopifnot(requireNamespace("flextable", quietly = TRUE))
    out <- reorder_univariate_cols(display_df)

    # NA -> "" (older flextable compat)
    out[] <- lapply(out, function(x) { x[is.na(x)] <- ""; x })

    merged_labels <- NULL
    if (inherits(tbl, "merged_table") && !is.null(tbl$column_labels)) {
      merged_labels <- c(
        Characteristic = "Characteristic",
        unlist(tbl$column_labels, use.names = TRUE)
      )
      merged_labels <- merged_labels[names(merged_labels) %in% names(out)]
    }

    labels <- stats::setNames(names(out), names(out))
    if (!is.null(merged_labels)) {
      labels[names(merged_labels)] <- unname(merged_labels)
    }
    if (!is.null(header_labels)) {
      labels[names(header_labels)] <- unname(header_labels)
    }

    ft <- flextable::flextable(out)
    ft <- flextable::set_header_labels(ft, values = labels)
    if (inherits(tbl, "merged_table") && !is.null(tbl$spanners) &&
        !is.null(tbl$column_labels)) {
      widths <- c(
        1L,
        vapply(tbl$column_labels, function(x) {
          sum(names(x) %in% names(out))
        }, integer(1))
      )
      keep <- c(TRUE, widths[-1] > 0)
      ft <- flextable::add_header_row(
        ft,
        values = c("", tbl$spanners)[keep],
        colwidths = widths[keep]
      )
    }
    ft <- flextable::align(ft, j = "Characteristic", align = "left",  part = "all")
    ft <- flextable::align(ft, j = setdiff(names(out), "Characteristic"), align = "center", part = "all")
    ft <- flextable::bold(ft, part = "header", bold = TRUE)

    # bold variable headers (on demand)
    if (isTRUE(bold_labels)) {
      hdr_idx <- which(display_df$is_header %in% TRUE)
      if (length(hdr_idx)) ft <- flextable::bold(ft, i = hdr_idx, j = "Characteristic", bold = TRUE, part = "body")
    }
    # optional bold levels
    if (isTRUE(bold_levels)) {
      lvl_idx <- which(display_df$is_header %in% FALSE)
      if (length(lvl_idx)) ft <- flextable::bold(ft, i = lvl_idx, j = "Characteristic", bold = TRUE, part = "body")
    }
    # indent levels
    lvl_idx <- which(display_df$is_header %in% FALSE)
    if (length(lvl_idx)) ft <- flextable::padding(ft, i = lvl_idx, j = "Characteristic",
                                                  padding.left = 14, part = "body")
    # caption
    if (!is.null(caption)) ft <- flextable::set_caption(ft, caption = caption)
    # footnotes
    if (length(footnotes)) {
      ft <- flextable::add_footer_lines(ft, values = footnotes)
      ft <- flextable::align(ft, part = "footer", align = "left")
      ft <- flextable::padding(ft, part = "footer", padding.top = 0, padding.bottom = 0)
      ft <- flextable::line_spacing(ft, part = "footer", space = 0.9)
    }

    ft <- flextable::autofit(ft)
    ft
  }

  apply_body_text_style <- function(rendered, display_df) {
    header_rows <- which(display_df$is_header %in% TRUE)
    level_rows <- which(display_df$is_header %in% FALSE)

    if (inherits(rendered, "gt_tbl")) {
      if (length(header_rows)) {
        rendered <- gt::tab_style(
          rendered,
          style = gt::cell_text(
            weight = if (isTRUE(bold_labels)) "bold" else "normal",
            style = if (isTRUE(italic_labels)) "italic" else "normal"
          ),
          locations = gt::cells_body(rows = header_rows, columns = "Characteristic")
        )
      }
      if (length(level_rows)) {
        rendered <- gt::tab_style(
          rendered,
          style = gt::cell_text(
            weight = if (isTRUE(bold_levels)) "bold" else "normal",
            style = if (isTRUE(italic_levels)) "italic" else "normal"
          ),
          locations = gt::cells_body(rows = level_rows, columns = "Characteristic")
        )
      }
      return(rendered)
    }

    if (inherits(rendered, "flextable")) {
      if (length(header_rows)) {
        rendered <- flextable::bold(
          rendered, i = header_rows, j = "Characteristic",
          bold = isTRUE(bold_labels), part = "body"
        )
        rendered <- flextable::italic(
          rendered, i = header_rows, j = "Characteristic",
          italic = isTRUE(italic_labels), part = "body"
        )
      }
      if (length(level_rows)) {
        rendered <- flextable::bold(
          rendered, i = level_rows, j = "Characteristic",
          bold = isTRUE(bold_levels), part = "body"
        )
        rendered <- flextable::italic(
          rendered, i = level_rows, j = "Characteristic",
          italic = isTRUE(italic_levels), part = "body"
        )
      }
    }

    rendered
  }

  # --------- rebuild rendered table ----------
  strata_table <- .rebuild_modified_stratified_table(tbl, tbl$table_display, footnotes, header_labels)
  if (!is.null(strata_table)) {
    tbl$table <- strata_table
  } else if (identical(tolower(tbl$format), "gt") || inherits(tbl$table, "gt_tbl")) {
    tbl$table <- .build_pkg_gt(tbl$table_display, header_labels, footnotes)
  } else if (identical(tolower(tbl$format), "flextable") || inherits(tbl$table, "flextable")) {
    tbl$table <- .build_pkg_flex(tbl$table_display, header_labels, footnotes)
  }

  tbl$table <- apply_body_text_style(tbl$table, tbl$table_display)

  tbl
}

#' Check proportional hazards assumption for Cox models
#'
#' Test the proportional hazards assumption for fitted Cox proportional hazards
#' models using Schoenfeld residuals.
#'
#' @param model A fitted \code{coxph} model or a \code{cox_reg()} object.
#' @param transform Time transformation passed to \code{survival::cox.zph()}.
#'   One of \code{"km"}, \code{"rank"}, or \code{"identity"}.
#' @param alpha Significance level used for the simple interpretation column.
#' @param format Output format. One of \code{"flextable"} (default),
#'   \code{"gt"}, or \code{"tibble"}.
#'
#' @return A tibble, \code{gt_tbl}, or \code{flextable} with Schoenfeld
#' residual proportional hazards tests. The table contains:
#' \describe{
#'   \item{\code{Model}}{Model name. For a direct \code{coxph} object, this is
#'   \code{"cox_model"}.}
#'   \item{\code{Term}}{Model term or \code{"GLOBAL"}.}
#'   \item{\code{Chi.square}}{Chi-square statistic.}
#'   \item{\code{df}}{Degrees of freedom.}
#'   \item{\code{p.value}}{Test p-value.}
#'   \item{\code{Interpretation}}{Simple screening interpretation using
#'   \code{alpha}.}
#' }
#'
#' @details
#' \code{check_ph()} is a diagnostic aid for Cox models. A small p-value
#' suggests possible evidence against the proportional hazards assumption for a
#' term or for the global model test. This should be interpreted with plots,
#' clinical knowledge, follow-up patterns, and modelling purpose.
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#'
#' cox_fit <- cox_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = c(trt, celltype, age)
#' )
#'
#' check_ph(cox_fit)
#' check_ph(cox_fit, format = tibble)
#'
#' @importFrom survival cox.zph
#' @export
check_ph <- function(model,
                     transform = c("km", "rank", "identity"),
                     alpha = 0.05,
                     format = c("flextable", "gt", "tibble")) {

  transform <- .choice_arg(
    substitute(transform),
    env = parent.frame(),
    choices = c("km", "rank", "identity")
  )
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )

  transform <- match.arg(transform, c("km", "rank", "identity"))
  format <- match.arg(format, c("flextable", "gt", "tibble"))

  if (!is.numeric(alpha) || length(alpha) != 1L || is.na(alpha) ||
      alpha <= 0 || alpha >= 1) {
    stop("`alpha` must be a number between 0 and 1.", call. = FALSE)
  }

  model_list <- .check_ph_model_list(model)
  out <- do.call(
    rbind,
    lapply(names(model_list), function(nm) {
      .check_ph_one_model(model_list[[nm]], model_name = nm,
                          transform = transform, alpha = alpha)
    })
  )
  rownames(out) <- NULL

  if (format == "tibble") {
    return(tibble::as_tibble(out))
  }

  .build_check_ph_table(out, format = format, alpha = alpha,
                        transform = transform)
}

#' @keywords internal
#' @noRd
.check_ph_model_list <- function(model) {
  if (inherits(model, "coxph")) {
    return(list(cox_model = model))
  }

  if (!inherits(model, "gtregression") || !identical(model$source, "cox_reg")) {
    stop("`model` must be a fitted coxph model or cox_reg() object.", call. = FALSE)
  }

  if (is.null(model$models) || !length(model$models)) {
    stop("No Cox models found in `model`.", call. = FALSE)
  }

  if (!all(vapply(model$models, inherits, logical(1), what = "coxph"))) {
    stop("All models must be fitted coxph objects.", call. = FALSE)
  }

  model$models
}

#' @keywords internal
#' @noRd
.check_ph_one_model <- function(fit, model_name, transform, alpha) {
  zph <- tryCatch(
    survival::cox.zph(fit, transform = transform),
    error = function(e) {
      stop(
        "Could not compute proportional hazards test for model '",
        model_name,
        "': ",
        conditionMessage(e),
        call. = FALSE
      )
    }
  )

  tab <- as.data.frame(zph$table)
  tab$Term <- rownames(tab)
  rownames(tab) <- NULL

  chisq_col <- intersect(c("chisq", "Chisq"), names(tab))[1]
  df_col <- intersect(c("df", "Df"), names(tab))[1]
  p_col <- intersect(c("p", "P"), names(tab))[1]

  out <- data.frame(
    Model = model_name,
    Term = tab$Term,
    Chi.square = as.numeric(tab[[chisq_col]]),
    df = as.numeric(tab[[df_col]]),
    p.value = as.numeric(tab[[p_col]]),
    stringsAsFactors = FALSE
  )

  out$Test <- ifelse(out$Term == "GLOBAL", "Global", "Term")
  out$Interpretation <- ifelse(
    is.na(out$p.value),
    NA_character_,
    ifelse(
      out$p.value < alpha,
      "Possible PH violation",
      "No evidence of PH violation"
    )
  )

  out[, c("Model", "Term", "Test", "Chi.square", "df", "p.value", "Interpretation")]
}

#' @keywords internal
#' @noRd
.build_check_ph_table <- function(result,
                                  format = c("flextable", "gt"),
                                  alpha = 0.05,
                                  transform = "km") {
  format <- match.arg(format, c("flextable", "gt"))
  note <- paste0(
    "Screening aid only. Small p-values suggest possible non-proportional ",
    "hazards; interpret with Schoenfeld residual plots, follow-up pattern, ",
    "clinical context, and model purpose. alpha = ", alpha,
    "; transform = ", transform, "."
  )

  display <- result |>
    dplyr::mutate(
      Chi.square = dplyr::if_else(
        is.na(.data$Chi.square),
        "",
        formatC(.data$Chi.square, digits = 2, format = "f")
      ),
      df = dplyr::if_else(
        is.na(.data$df),
        "",
        formatC(.data$df, digits = 0, format = "f")
      ),
      p.value = dplyr::case_when(
        is.na(.data$p.value) ~ "",
        .data$p.value < 0.001 ~ "<0.001",
        TRUE ~ formatC(.data$p.value, digits = 3, format = "f")
      )
    )

  if (format == "gt") {
    return(
      gt::gt(display) |>
        gt::tab_header(title = "Proportional hazards check") |>
        gt::cols_label(
          Chi.square = "Chi-square",
          p.value = "p-value"
        ) |>
        gt::cols_align(align = "left", columns = c("Model", "Term", "Test", "Interpretation")) |>
        gt::cols_align(align = "center", columns = c("Chi.square", "df", "p.value")) |>
        gt::tab_style(
          style = gt::cell_text(weight = "bold"),
          locations = gt::cells_column_labels()
        ) |>
        gt::tab_style(
          style = gt::cell_fill(color = "#e7f5ec"),
          locations = gt::cells_body(rows = .data$Interpretation == "No evidence of PH violation")
        ) |>
        gt::tab_style(
          style = gt::cell_fill(color = "#fde2e2"),
          locations = gt::cells_body(rows = .data$Interpretation == "Possible PH violation")
        ) |>
        gt::tab_source_note(gt::md(note)) |>
        .compact_gt_source_notes()
    )
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Proportional hazards check")
  ft <- flextable::set_header_labels(
    ft,
    Chi.square = "Chi-square",
    p.value = "p-value"
  )
  ft <- flextable::align(
    ft,
    j = c("Model", "Term", "Test", "Interpretation"),
    align = "left",
    part = "all"
  )
  ft <- flextable::align(
    ft,
    j = c("Chi.square", "df", "p.value"),
    align = "center",
    part = "all"
  )
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  ft <- flextable::bg(
    ft,
    i = which(display$Interpretation == "No evidence of PH violation"),
    bg = "#e7f5ec",
    part = "body"
  )
  ft <- flextable::bg(
    ft,
    i = which(display$Interpretation == "Possible PH violation"),
    bg = "#fde2e2",
    part = "body"
  )
  ft <- flextable::add_footer_lines(ft, values = note)
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

# Internal Shiny module for safe, reversible data preparation.

mod_data_prep_ui <- function(id) {
  ns <- shiny::NS(id)
  shiny::tagList(
    shiny::tags$style(shiny::HTML(".gtx-prep-toolbar .nav { border-bottom: 1px solid #D9D9D4; margin-bottom: 0; display: flex; flex-wrap: wrap; gap: 2px; } .gtx-prep-toolbar .nav > li { float: none; } .gtx-prep-toolbar .nav > li > a { font-weight: 750; padding: 9px 13px; color:#3D3D39; border-radius: 7px 7px 0 0; } .gtx-prep-toolbar .nav > li > a:hover, .gtx-prep-toolbar .nav > li > a:focus { background:#ECECEA; color:#111111; } .gtx-prep-templates { margin-top: 12px; padding: 10px 12px; border-radius: 8px; background: #F7F7F5; border: 1px solid #E3E3DE; } .gtx-prep-templates strong { margin-right: 8px; } .gtx-prep-templates .btn { margin: 3px 5px 3px 0; } .gtx-prep-actions .btn { margin-right: 7px; margin-bottom: 8px; } .gtx-prep-apply { margin-top: 18px; padding-top: 14px; border-top: 1px solid #E3E3DE; } .gtx-value-guide { margin-top: 12px; padding: 11px 13px; border: 1px solid #D9D9D4; border-radius: 8px; background: #FAFAF8; } .gtx-value-guide h5 { margin: 0 0 6px; font-weight: 750; } .gtx-value-guide pre { white-space: pre-wrap; margin: 8px 0 0; padding: 8px; font-size: .9em; } .gtx-recode-map { margin-top: 14px; padding: 12px; border: 1px solid #D9D9D4; border-radius: 8px; background: #FDFDFC; } .gtx-recode-map h4 { margin: 0 0 4px; } .gtx-impact-table { width: 100%; margin-top: 8px; font-size: .92em; background: #fff; } .gtx-impact-table th, .gtx-impact-table td { padding: 7px 8px; border-bottom: 1px solid #E3E3DE; text-align: left; vertical-align: middle; } .gtx-impact-table th { font-weight: 750; color: #3D3D39; background: #F5F5F2; } .gtx-impact-table .form-group { margin-bottom: 0; } .gtx-impact-table .form-control { min-width: 170px; } @media (max-width: 767px) { .gtx-prep-toolbar .nav > li > a { padding: 8px 9px; font-size: .88em; } .gtx-impact-table { font-size: .85em; } .gtx-impact-table .form-control { min-width: 120px; } }")),
    shiny::div(class = "gtx-card",
      shiny::tags$h3("Data Prep"),
      shiny::tags$p(class = "help-copy", "Choose one clear task below. Each change is made to a working copy, can be undone, and becomes reproducible R code."),
      shiny::div(class = "gtx-prep-toolbar",
        shiny::tags$ul(class = "nav nav-pills",
          shiny::tags$li(shiny::actionLink(ns("menu_label"), "Set display label")),
          shiny::tags$li(shiny::actionLink(ns("menu_rename"), "Rename variable")),
          shiny::tags$li(shiny::actionLink(ns("menu_recode"), "Recode values")),
          shiny::tags$li(shiny::actionLink(ns("menu_type"), "Set type & order")),
          shiny::tags$li(shiny::actionLink(ns("menu_missing"), "Manage missing values")),
          shiny::tags$li(shiny::actionLink(ns("menu_calculation"), "Create calculation")),
          shiny::tags$li(shiny::actionLink(ns("menu_groups"), "Create groups")),
          shiny::tags$li(shiny::actionLink(ns("menu_filter"), "Filter observations")),
          shiny::tags$li(shiny::actionLink(ns("menu_keep"), "Keep variables"))
        )
      ),
      shiny::div(class = "gtx-prep-templates",
        shiny::tags$strong("Quick starts:"),
        shiny::actionButton(ns("template_age"), "Age groups", class = "btn-default btn-sm"),
        shiny::actionButton(ns("template_binary"), "0/1 to No/Yes", class = "btn-default btn-sm"),
        shiny::actionButton(ns("template_bmi"), "BMI-style calculation", class = "btn-default btn-sm"),
        shiny::actionButton(ns("template_missing"), "99/999 to missing", class = "btn-default btn-sm"),
        shiny::tags$span(class = "help-copy", "They fill a starting point only; review before applying.")
      )
    ),
    shiny::fluidRow(
      shiny::column(4, shiny::div(class = "cardish",
        shiny::tags$h4("Set up this change"), shiny::uiOutput(ns("operation_hint")), shiny::uiOutput(ns("controls")), shiny::uiOutput(ns("impact_preview")), shiny::uiOutput(ns("value_guide")),
        shiny::div(class = "gtx-prep-apply", shiny::tags$p(class = "help-copy", "Review the preview, then apply this one change."), shiny::actionButton(ns("apply"), "Apply this change", class = "btn-primary"))
      )),
      shiny::column(8, shiny::div(class = "gtx-card",
        shiny::tags$div(class = "card-heading", shiny::tags$h4("Working-data preview"), shiny::textOutput(ns("status"))),
        gt::gt_output(ns("preview"))
      ))
    ),
    shiny::div(class = "gtx-card gtx-prep-actions",
      shiny::tags$h4("Change controls"),
      shiny::actionButton(ns("undo"), "Undo last change"),
      shiny::actionButton(ns("redo"), "Redo"),
      shiny::actionButton(ns("reset"), "Reset all"),
      shiny::tags$hr(),
      shiny::tags$strong("Analysis dataset: "),
      shiny::actionButton(ns("continue_original"), "Use original data", class = "btn-default"),
      shiny::actionButton(ns("use_prepared"), "Use prepared data", class = "btn-success"),
      shiny::tags$p(
        class = "help-copy",
        "Original data is active automatically after loading. Choose prepared data only after applying changes."
      ),
      shiny::tags$hr(),
      shiny::tags$strong("Download the prepared data: "),
      shiny::selectInput(ns("prepared_format"), NULL, choices = c("CSV" = "csv", "Excel (.xlsx)" = "xlsx", "R data (.rds)" = "rds", "Stata (.dta)" = "dta"), selected = "csv", width = "160px"),
      shiny::downloadButton(ns("download_prepared"), "Download prepared data")
    ),
    shiny::tabsetPanel(type = "tabs",
      shiny::tabPanel("Change log", shiny::div(class = "gtx-card", shiny::verbatimTextOutput(ns("history")))),
      shiny::tabPanel("Reusable code", shiny::div(class = "gtx-card", shiny::tags$div(class = "card-heading", shiny::tags$h4("Copy this into RStudio"),
        shiny::tags$button(type = "button", class = "btn btn-default copy-code", `data-copy-target` = ns("code"), "Copy code"),
        shiny::downloadButton(ns("download_code"), "Download .R")), shiny::verbatimTextOutput(ns("code"))))
    )
  )
}

mod_data_prep_server <- function(id, source_data) {
  shiny::moduleServer(id, function(input, output, session) {
    ns <- session$ns
    state <- shiny::reactiveValues(raw_data = NULL, working_data = NULL, history = list(), redo_stack = list(), result = NULL, using_prepared = FALSE)
    shiny::observeEvent(source_data(), {
      state$raw_data <- source_data(); state$working_data <- source_data()
      state$history <- list(); state$redo_stack <- list(); state$result <- state$raw_data; state$using_prepared <- FALSE
    }, ignoreInit = FALSE)
    variables <- shiny::reactive(names(state$working_data %||% data.frame()))
    operation <- shiny::reactiveVal("label")
    transform_mode <- shiny::reactiveVal("arithmetic")
    choose_operation <- function(selected_operation, mode = NULL) {
      operation(selected_operation)
      if (!is.null(mode)) transform_mode(mode)
    }
    shiny::observeEvent(input$menu_rename, choose_operation("rename"), ignoreInit = TRUE)
    shiny::observeEvent(input$menu_label, choose_operation("label"), ignoreInit = TRUE)
    shiny::observeEvent(input$menu_recode, choose_operation("recode"), ignoreInit = TRUE)
    shiny::observeEvent(input$menu_type, choose_operation("type"), ignoreInit = TRUE)
    shiny::observeEvent(input$menu_missing, choose_operation("missing"), ignoreInit = TRUE)
    shiny::observeEvent(input$menu_filter, choose_operation("filter"), ignoreInit = TRUE)
    shiny::observeEvent(input$menu_keep, choose_operation("keep"), ignoreInit = TRUE)
    shiny::observeEvent(input$menu_calculation, choose_operation("transform", "arithmetic"), ignoreInit = TRUE)
    shiny::observeEvent(input$menu_groups, choose_operation("transform", "case_when"), ignoreInit = TRUE)
    shiny::observeEvent(list(operation(), input$variable), {
      if (!identical(operation(), "label")) return(NULL)
      variable <- input$variable %||% ""
      if (!nzchar(variable) || !variable %in% names(state$working_data)) return(NULL)
      label <- attr(state$working_data[[variable]], "label", exact = TRUE) %||% ""
      session$onFlushed(function() {
        shiny::updateTextInput(session, "variable_label", value = label)
      }, once = TRUE)
    }, ignoreInit = TRUE)
    numeric_variable <- function(preferred = NULL) {
      data <- state$working_data
      if (is.null(data)) return("")
      candidates <- names(data)[vapply(data, is.numeric, logical(1))]
      if (!length(candidates)) return("")
      if (!is.null(preferred) && preferred %in% candidates) preferred else candidates[[1L]]
    }
    after_controls_render <- function(code) session$onFlushed(code, once = TRUE)
    shiny::observeEvent(input$template_age, {
      variable <- numeric_variable("age")
      if (!nzchar(variable)) return(shiny::showNotification("Age groups needs a numeric variable. Choose Create groups after importing suitable data.", type = "warning"))
      choose_operation("transform", "case_when")
      after_controls_render(function() {
        shiny::updateSelectInput(session, "group_variable", selected = variable)
        shiny::updateRadioButtons(session, "group_count", selected = "2")
        shiny::updateSelectInput(session, "rule_op_1", selected = ">=")
        shiny::updateTextInput(session, "rule_value_1", value = "65")
        shiny::updateTextInput(session, "rule_result_1", value = "65 or older")
        shiny::updateTextInput(session, "rule_default", value = "Under 65")
        shiny::updateTextInput(session, "transform_new", value = paste0(variable, "_group"))
      })
      shiny::showNotification("Age-group starter loaded. Check the cut-off and labels before applying.", type = "message")
    }, ignoreInit = TRUE)
    shiny::observeEvent(input$template_binary, {
      data <- state$working_data
      candidates <- names(data)[vapply(data, function(x) {
        values <- sort(unique(as.character(x[!is.na(x)])))
        identical(values, c("0", "1"))
      }, logical(1))]
      if (!length(candidates)) return(shiny::showNotification("No variable coded exactly 0 and 1 was found. Choose Recode values and select a variable yourself.", type = "warning"))
      choose_operation("recode")
      after_controls_render(function() {
        shiny::updateSelectInput(session, "variable", selected = candidates[[1L]])
        after_controls_render(function() {
          shiny::updateTextInput(session, "recode_to_1", value = "No")
          shiny::updateTextInput(session, "recode_to_2", value = "Yes")
        })
      })
      shiny::showNotification("0/1 recoding starter loaded. Confirm that 0 means No and 1 means Yes before applying.", type = "message")
    }, ignoreInit = TRUE)
    shiny::observeEvent(input$template_bmi, {
      variable <- numeric_variable()
      if (!nzchar(variable)) return(shiny::showNotification("BMI-style calculation needs numeric variables.", type = "warning"))
      choose_operation("transform", "arithmetic")
      after_controls_render(function() {
        shiny::updateRadioButtons(session, "calculation_mode", selected = "ratio_power")
        after_controls_render(function() {
          shiny::updateSelectInput(session, "transform_source", selected = variable)
          shiny::updateNumericInput(session, "transform_number", value = 2)
          shiny::updateTextInput(session, "transform_new", value = "bmi")
        })
      })
      shiny::showNotification("BMI-style starter loaded. Choose weight in kg and height in metres before applying.", type = "message")
    }, ignoreInit = TRUE)
    shiny::observeEvent(input$template_missing, {
      variable <- numeric_variable()
      if (!nzchar(variable)) return(shiny::showNotification("This starter needs a variable that contains recorded missing-data codes.", type = "warning"))
      choose_operation("missing")
      after_controls_render(function() {
        shiny::updateSelectInput(session, "variable", selected = variable)
        shiny::updateRadioButtons(session, "missing_action", selected = "to_na")
        shiny::updateTextInput(session, "codes", value = "99, 999")
      })
      shiny::showNotification("Missing-code starter loaded. Replace 99/999 unless those are genuinely your missing-data codes.", type = "message")
    }, ignoreInit = TRUE)
    output$operation_hint <- shiny::renderUI({
      hint <- switch(operation(),
        rename = shiny::tagList(
          shiny::tags$strong("Give a variable a clearer name."),
          shiny::tags$p(class = "help-copy", "Example: rename `smoke` to `smoking_status`. This changes the column name only; values are unchanged.")
        ),
        label = shiny::tagList(
          shiny::tags$strong("Give a variable a publication-ready display label."),
          shiny::tags$p(class = "help-copy", "Example: keep the internal name `age`, but display Maternal age in tables and plots. This does not change the column name or values. Leave the label empty to remove it.")
        ),
        recode = shiny::tagList(
          shiny::tags$strong("Give each recorded value a clear label."),
          shiny::tags$p(class = "help-copy", "Each recorded value is shown below with its replacement label. Edit only the labels you want to change; values left unchanged stay as they are.")
        ),
        type = shiny::tagList(
          shiny::tags$strong("Set how a variable is treated in tables and analyses."),
          shiny::tags$p(class = "help-copy", "Use categorical for unordered values such as No/Yes; use ordered categorical for values with a real order, such as None, One, Two or more. The level order is used in tables and charts.")
        ),
        missing = shiny::tagList(
          shiny::tags$strong("Make missingness explicit and reproducible."),
          shiny::tags$p(class = "help-copy", "Convert recorded missing-data codes such as `999` or `Unknown` to R missing values (`NA`). Or, for a categorical variable, show existing NA values as a labelled Missing category for presentation.")
        ),
        transform = shiny::tagList(
          shiny::tags$strong("Create a new variable without changing the source variable."),
          shiny::tags$p(class = "help-copy", "Create calculations such as BMI or unit conversions, or use Create groups for age bands, BMI categories, and risk groups. The source variable is always retained.")
        ),
        filter = shiny::tagList(
          shiny::tags$strong("Keep only records meeting your eligibility criteria."),
          shiny::tags$p(class = "help-copy", "Example: age is at least 18 AND consent equals Yes. Always check the included, excluded, and unknown counts before continuing.")
        ),
        keep = shiny::tagList(
          shiny::tags$strong("Make a smaller analysis dataset without deleting the original import."),
          shiny::tags$p(class = "help-copy", "Tick the variables needed for your analysis. You can Undo this change at any time.")
        )
      )
      shiny::tags$div(class = "gtx-step", hint)
    })
    output$controls <- shiny::renderUI({
      ns <- session$ns; vars <- variables(); shiny::req(length(vars))
      if (identical(operation(), "rename")) shiny::tagList(
        shiny::selectInput(ns("variable"), "Existing variable", vars), shiny::textInput(ns("new_name"), "New variable name")
      ) else if (identical(operation(), "label")) shiny::tagList(
        shiny::selectInput(ns("variable"), "Variable", vars),
        shiny::textInput(ns("variable_label"), "Display label", placeholder = "Example: Maternal age"),
        shiny::tags$p(class = "help-copy", "Use the short internal name when selecting variables for analysis. This label is used in publication-ready output."),
        shiny::uiOutput(ns("label_level_mapping"))
      ) else if (identical(operation(), "recode")) shiny::tagList(
        shiny::selectInput(ns("variable"), "Variable", vars),
        shiny::uiOutput(ns("recode_mapping")),
        shiny::checkboxInput(ns("recode_ordered"), "Treat as ordered categorical, in the order shown", FALSE),
        shiny::tags$p(class = "help-copy", "The recoded variable is categorical. Tick this only when the displayed order has a genuine meaning, such as None, Mild, Moderate, Severe.")
      ) else if (identical(operation(), "type")) shiny::tagList(
        shiny::selectInput(ns("variable"), "Variable", vars),
        shiny::selectInput(ns("variable_type"), "Treat this variable as", c("Categorical" = "factor", "Ordered categorical" = "ordered", "Numeric" = "numeric", "Text" = "text")),
        shiny::uiOutput(ns("type_order_grid"))
      ) else if (identical(operation(), "missing")) shiny::tagList(
        shiny::selectInput(ns("variable"), "Variable", vars),
        shiny::radioButtons(ns("missing_action"), "What do you want to do?", c("Convert recorded codes to missing (NA)" = "to_na", "Show existing missing values as a category" = "show_category"), selected = "to_na"),
        shiny::uiOutput(ns("missing_controls"))
      ) else if (identical(operation(), "transform")) {
        mode <- transform_mode()
        numeric_vars <- vars[vapply(state$working_data[vars], is.numeric, logical(1))]
        operators <- c("equal to" = "==", "not equal to" = "!=", "greater than" = ">", "greater than or equal to" = ">=", "less than" = "<", "less than or equal to" = "<=")
        if (identical(mode, "arithmetic") && !length(numeric_vars)) shiny::tags$div(class = "gtx-note", "Create calculation needs at least one numeric variable. Use Create groups to make categories from recorded values.")
        else if (identical(mode, "arithmetic")) shiny::tagList(
          shiny::tags$strong("Create a new numeric variable."),
          shiny::tags$p(class = "help-copy", "Choose a common calculation pattern. Your original variables are always retained."),
          shiny::radioButtons(ns("calculation_mode"), "Calculation type", c("Adjust one variable" = "single", "Combine two variables" = "two_variables", "Divide by another variable raised to a power" = "ratio_power"), selected = "single"),
          shiny::uiOutput(ns("calculation_controls"))
        ) else shiny::tagList(
          shiny::tags$strong("Create a new grouped variable."),
          shiny::tags$p(class = "help-copy", "Choose one source variable, decide how many groups you need, then give each group a clear label. Groups are checked from top to bottom; the first matching condition wins."),
          shiny::selectInput(ns("group_variable"), "Variable to group", vars),
          shiny::radioButtons(ns("group_count"), "How many groups?", c("Two groups" = "2", "Three groups" = "3", "Four groups" = "4"), selected = "2", inline = TRUE),
          shiny::uiOutput(ns("group_rules")),
          shiny::textInput(ns("transform_new"), "Name for the new grouped variable", placeholder = "Example: age_group")
        )
      } else if (identical(operation(), "keep")) shiny::tagList(
        shiny::tags$strong("Choose the variables for your analysis dataset."),
        shiny::tags$p(class = "help-copy", "Start with all variables, remove those you do not need, and search by name when your dataset is large. This does not change the original imported data."),
        shiny::div(class = "gtx-prep-actions",
          shiny::actionButton(ns("keep_all"), "Keep all variables", class = "btn-default btn-sm"),
          shiny::actionButton(ns("keep_none"), "Clear selection", class = "btn-default btn-sm")
        ),
        shiny::selectizeInput(ns("keep_vars"), "Variables to keep", choices = vars, selected = vars, multiple = TRUE, options = list(placeholder = "Search and select variables"))
      ) else shiny::tagList(
        shiny::tags$strong("Keep observations that meet your inclusion criteria."),
        shiny::tags$p(class = "help-copy", "Start with one condition. Add a second only when needed. The preview shows exactly how many records will be kept, excluded, or have an unknown result because of missing data."),
        shiny::uiOutput(ns("filter_conditions"))
      )
    })
    recode_values <- shiny::reactive({
      variable <- input$variable %||% ""
      data <- state$working_data
      if (!nzchar(variable) || is.null(data) || !variable %in% names(data)) return(character())
      unique(as.character(data[[variable]][!is.na(data[[variable]])]))
    })
    output$recode_mapping <- shiny::renderUI({
      values <- recode_values()
      if (!length(values)) return(shiny::tags$p(class = "help-copy", "This variable has no observed values to recode."))
      rows <- lapply(seq_along(values), function(index) {
        shiny::tags$tr(
          shiny::tags$td(shiny::tags$code(values[[index]])),
          shiny::tags$td(shiny::textInput(ns(paste0("recode_to_", index)), NULL, value = values[[index]]))
        )
      })
      shiny::tags$div(class = "gtx-recode-map",
        shiny::tags$h4("Value mapping"),
        shiny::tags$p(class = "help-copy", "Change the label beside each recorded value. The left column is read-only."),
        shiny::tags$table(class = "gtx-impact-table",
          shiny::tags$thead(shiny::tags$tr(shiny::tags$th("Recorded value"), shiny::tags$th("New label"))),
          shiny::tags$tbody(rows)
        )
      )
    })
    recode_labels <- shiny::reactive({
      values <- recode_values()
      if (!length(values)) return(character())
      vapply(seq_along(values), function(index) {
        input[[paste0("recode_to_", index)]] %||% values[[index]]
      }, character(1))
    })
    label_level_values <- shiny::reactive({
      variable <- input$variable %||% ""
      data <- state$working_data
      if (!nzchar(variable) || is.null(data) || !variable %in% names(data)) return(character())
      x <- data[[variable]]
      if (!(is.factor(x) || is.character(x) || is.logical(x))) return(character())
      if (is.factor(x)) return(levels(x))
      unique(as.character(x[!is.na(x)]))
    })
    output$label_level_mapping <- shiny::renderUI({
      values <- label_level_values()
      if (!length(values)) return(NULL)
      rows <- lapply(seq_along(values), function(index) {
        shiny::tags$tr(
          shiny::tags$td(shiny::tags$code(values[[index]])),
          shiny::tags$td(shiny::textInput(ns(paste0("label_level_to_", index)), NULL, value = values[[index]]))
        )
      })
      shiny::tags$div(class = "gtx-recode-map",
        shiny::tags$h4("Category display labels"),
        shiny::tags$p(class = "help-copy", "Optional. Change these labels for neat table and plot output. The recorded categories are converted to a factor only when at least one label changes."),
        shiny::tags$table(class = "gtx-impact-table",
          shiny::tags$thead(shiny::tags$tr(shiny::tags$th("Current category"), shiny::tags$th("Display label"))),
          shiny::tags$tbody(rows)
        )
      )
    })
    label_level_labels <- shiny::reactive({
      values <- label_level_values()
      if (!length(values)) return(character())
      vapply(seq_along(values), function(index) {
        input[[paste0("label_level_to_", index)]] %||% values[[index]]
      }, character(1))
    })
    type_order_values <- shiny::reactive({
      variable <- input$variable %||% ""
      data <- state$working_data
      if (!nzchar(variable) || is.null(data) || !variable %in% names(data)) return(character())
      unique(as.character(data[[variable]][!is.na(data[[variable]])]))
    })
    output$type_order_grid <- shiny::renderUI({
      variable_type <- input$variable_type %||% "factor"
      if (identical(variable_type, "factor")) {
        return(shiny::tags$p(class = "help-copy", "Categorical treats values as separate groups with no meaningful order. Their current display order is retained."))
      }
      if (!identical(variable_type, "ordered")) {
        return(shiny::tags$p(class = "help-copy", "Numeric converts recorded values to numbers where possible. Text keeps labels as plain text."))
      }
      values <- type_order_values()
      if (!length(values)) return(shiny::tags$p(class = "help-copy", "This variable has no observed values to order."))
      ranks <- stats::setNames(seq_along(values), paste0(seq_along(values), ifelse(seq_along(values) == 1L, " (first)", "")))
      rows <- lapply(seq_along(values), function(index) {
        shiny::tags$tr(
          shiny::tags$td(shiny::tags$code(values[[index]])),
          shiny::tags$td(shiny::selectInput(ns(paste0("type_rank_", index)), NULL, choices = ranks, selected = index))
        )
      })
      shiny::tags$div(class = "gtx-recode-map",
        shiny::tags$h4("Display order"),
        shiny::tags$p(class = "help-copy", "Give each recorded value a different position. The default keeps the current order. This controls the order in tables and charts."),
        shiny::tags$table(class = "gtx-impact-table",
          shiny::tags$thead(shiny::tags$tr(shiny::tags$th("Recorded value"), shiny::tags$th("Position"))),
          shiny::tags$tbody(rows)
        )
      )
    })
    type_levels <- shiny::reactive({
      values <- type_order_values()
      if (!length(values)) return(character())
      ranks <- vapply(seq_along(values), function(index) {
        suppressWarnings(as.integer(input[[paste0("type_rank_", index)]] %||% index))
      }, integer(1))
      if (anyNA(ranks) || anyDuplicated(ranks) || !setequal(ranks, seq_along(values))) {
        stop("Give every recorded value a different display position.", call. = FALSE)
      }
      values[order(ranks)]
    })
    output$missing_controls <- shiny::renderUI({
      if (identical(input$missing_action %||% "to_na", "show_category")) {
        shiny::tagList(
          shiny::textInput(ns("missing_label"), "Label to show for missing values", value = "Missing"),
          shiny::tags$p(class = "help-copy", "This creates a categorical level for existing NA values. Use it only when showing missingness as a table category is appropriate; it changes how the variable is analysed.")
        )
      } else {
        shiny::tagList(
          shiny::textInput(ns("codes"), "Recorded codes to convert to missing", placeholder = "Example: 999, Unknown, N/A"),
          shiny::tags$p(class = "help-copy", "Use this only for codes that truly mean no value was recorded. R uses NA to represent missing data.")
        )
      }
    })
    output$group_rules <- shiny::renderUI({
      count <- suppressWarnings(as.integer(input$group_count %||% 2L))
      count <- if (count %in% 2:4) count else 2L
      numeric_group <- nzchar(input$group_variable %||% "") &&
        input$group_variable %in% names(state$working_data) &&
        is.numeric(state$working_data[[input$group_variable]])
      operators <- c("is equal to (=)" = "==", "is not equal to (!=)" = "!=")
      if (numeric_group) operators <- c(
        operators,
        "is greater than (>)" = ">", "is greater than or equal to (>=)" = ">=",
        "is less than (<)" = "<", "is less than or equal to (<=)" = "<=",
        "is between (inclusive)" = "between", "is outside the range" = "outside"
      )
      rule_rows <- lapply(seq_len(count - 1L), function(index) {
        shiny::tags$div(class = "gtx-step",
          shiny::tags$strong(paste("Group", index)),
          shiny::selectInput(ns(paste0("rule_op_", index)), "Include values that", operators),
          shiny::textInput(ns(paste0("rule_value_", index)), "Value or lower bound", placeholder = if (index == 1L) "Example: 35" else "Example: 65"),
          shiny::conditionalPanel(
            condition = sprintf("input['%s'] === 'between' || input['%s'] === 'outside'", ns(paste0("rule_op_", index)), ns(paste0("rule_op_", index))),
            shiny::textInput(ns(paste0("rule_value2_", index)), "Upper bound", placeholder = "Example: 64")
          ),
          shiny::textInput(ns(paste0("rule_result_", index)), "Group label", placeholder = if (index == 1L) "Example: Older adult" else "Example: Adult")
        )
      })
      shiny::tagList(
        rule_rows,
        shiny::tags$div(class = "gtx-step",
          shiny::tags$strong(paste("Group", count, "(everyone else)")),
          shiny::textInput(ns("rule_default"), "Group label", placeholder = "Example: Younger adult")
        )
      )
    })
    output$filter_conditions <- shiny::renderUI({
      operators <- c("is equal to" = "==", "is not equal to" = "!=", "is greater than" = ">", "is at least" = ">=", "is less than" = "<", "is at most" = "<=")
      vars <- variables()
      shiny::tagList(
        shiny::tags$div(class = "gtx-step",
          shiny::tags$strong("Condition 1"),
          shiny::selectInput(ns("variable1"), "Variable", vars),
          shiny::selectInput(ns("operator1"), "Keep when", operators),
          shiny::textInput(ns("value1"), "Value", placeholder = "Example: 18")
        ),
        shiny::checkboxInput(ns("filter_second"), "Add another condition", FALSE),
        shiny::uiOutput(ns("filter_second_condition"))
      )
    })
    shiny::observeEvent(input$keep_all, {
      shiny::updateSelectizeInput(session, "keep_vars", selected = variables(), server = TRUE)
    }, ignoreInit = TRUE)
    shiny::observeEvent(input$keep_none, {
      shiny::updateSelectizeInput(session, "keep_vars", selected = character(), server = TRUE)
    }, ignoreInit = TRUE)
    output$filter_second_condition <- shiny::renderUI({
      if (!isTRUE(input$filter_second)) return(NULL)
      operators <- c("is equal to" = "==", "is not equal to" = "!=", "is greater than" = ">", "is at least" = ">=", "is less than" = "<", "is at most" = "<=")
      shiny::tags$div(class = "gtx-step",
        shiny::tags$strong("Condition 2"),
        shiny::radioButtons(ns("connector"), "Keep records meeting", c("Both conditions" = "AND", "Either condition" = "OR"), selected = "AND", inline = TRUE),
        shiny::selectInput(ns("variable2"), "Variable", variables()),
        shiny::selectInput(ns("operator2"), "Keep when", operators),
        shiny::textInput(ns("value2"), "Value", placeholder = "Example: Yes")
      )
    })
    output$calculation_controls <- shiny::renderUI({
      numeric_vars <- variables()[vapply(state$working_data[variables()], is.numeric, logical(1))]
      mode <- input$calculation_mode %||% "single"
      if (!length(numeric_vars)) return(shiny::tags$p(class = "help-copy", "No numeric variables are available."))
      if (identical(mode, "single")) return(shiny::tagList(
        shiny::selectInput(ns("transform_source"), "Numeric variable", numeric_vars),
        shiny::selectInput(ns("transform_operator"), "Calculation", c("Add a number" = "+", "Subtract a number" = "-", "Multiply by a number" = "*", "Divide by a number" = "/", "Raise to a power" = "^")),
        shiny::numericInput(ns("transform_number"), "Number", 1),
        shiny::textInput(ns("transform_new"), "Name for the new variable", placeholder = "Example: weight_kg")
      ))
      if (identical(mode, "two_variables")) return(shiny::tagList(
        shiny::selectInput(ns("transform_source"), "First numeric variable", numeric_vars),
        shiny::selectInput(ns("transform_operator"), "Calculation", c("Add" = "+", "Subtract second from first" = "-", "Multiply" = "*", "Divide first by second" = "/")),
        shiny::selectInput(ns("transform_second"), "Second numeric variable", numeric_vars),
        shiny::textInput(ns("transform_new"), "Name for the new variable", placeholder = "Example: total_score")
      ))
      shiny::tagList(
        shiny::selectInput(ns("transform_source"), "Numerator variable", numeric_vars),
        shiny::selectInput(ns("transform_second"), "Denominator variable", numeric_vars),
        shiny::numericInput(ns("transform_number"), "Denominator power", 2),
        shiny::tags$p(class = "help-copy", "This calculates numerator / (denominator ^ power). For BMI, use weight in kg divided by height in metres with power 2."),
        shiny::textInput(ns("transform_new"), "Name for the new variable", placeholder = "Example: bmi")
      )
    })
    selected_variable <- shiny::reactive({
      if (operation() %in% c("recode", "type", "missing")) return(input$variable %||% "")
      if (identical(operation(), "transform")) {
        if (identical(transform_mode(), "arithmetic")) return(input$transform_source %||% "")
        return(input$group_variable %||% "")
      }
      if (identical(operation(), "filter")) return(input$variable1 %||% "")
      ""
    })
    output$value_guide <- shiny::renderUI({
      variable <- selected_variable()
      data <- state$working_data
      shiny::req(!is.null(data), nzchar(variable), variable %in% names(data))
      x <- data[[variable]]
      missing <- sum(is.na(x))
      is_categorical <- is.factor(x) || is.character(x) || is.logical(x)
      if (is_categorical) {
        values <- unique(as.character(x[!is.na(x)]))
        shown <- utils::head(values, 40L)
        suffix <- if (length(values) > length(shown)) paste0("\n... plus ", length(values) - length(shown), " more value(s)") else ""
        value_text <- paste0(paste(shown, collapse = ", "), suffix)
        shiny::tags$div(class = "gtx-value-guide",
          shiny::tags$h5("Recorded values"),
          shiny::tags$p(class = "help-copy", paste0("Exact non-missing values in `", variable, "` (", length(values), " unique; ", missing, " missing). Use these spellings when recoding or filtering.")),
          shiny::tags$button(type = "button", class = "btn btn-default btn-xs copy-code", `data-copy-target` = ns("values_to_copy"), "Copy values"),
          shiny::tags$pre(id = ns("values_to_copy"), value_text)
        )
      } else if (is.numeric(x)) {
        observed <- x[is.finite(x)]
        if (!length(observed)) return(shiny::tags$div(class = "gtx-value-guide", shiny::tags$h5("Numeric variable"), "No finite observed values are available."))
        unique_values <- sort(unique(observed))
        details <- paste0("Range: ", format(min(observed), trim = TRUE), " to ", format(max(observed), trim = TRUE), "; ", length(unique_values), " unique; ", missing, " missing.")
        if (length(unique_values) <= 12L) details <- paste0(details, " Recorded values: ", paste(unique_values, collapse = ", "), ".")
        shiny::tags$div(class = "gtx-value-guide", shiny::tags$h5("Numeric variable guide"), shiny::tags$p(class = "help-copy", details))
      } else {
        shiny::tags$div(class = "gtx-value-guide", shiny::tags$h5("Selected variable"), shiny::tags$p(class = "help-copy", paste0("Class: ", paste(class(x), collapse = ", "), "; ", missing, " missing value(s).")))
      }
    })
    impact_table <- function(headers, rows) {
      shiny::tags$table(class = "gtx-impact-table",
        shiny::tags$thead(shiny::tags$tr(lapply(headers, shiny::tags$th))),
        shiny::tags$tbody(lapply(rows, function(row) shiny::tags$tr(lapply(row, shiny::tags$td))))
      )
    }
    output$impact_preview <- shiny::renderUI({
      data <- state$working_data
      shiny::req(!is.null(data))
      content <- tryCatch({
        if (identical(operation(), "rename")) {
          if (!nzchar(input$new_name %||% "")) return(NULL)
          list(message = "Values will not change; only the column name will change.")
        } else if (identical(operation(), "recode")) {
          from <- recode_values()
          if (!length(from) || !nzchar(input$variable %||% "")) return(NULL)
          affected <- sum(as.character(data[[input$variable]]) %in% from, na.rm = TRUE)
          to <- recode_labels()
          if (any(!nzchar(to))) return(list(message = "Enter a new label for every displayed recorded value."))
          counts <- vapply(from, function(value) sum(as.character(data[[input$variable]]) %in% value, na.rm = TRUE), numeric(1))
          changed <- sum(counts[from != to])
          type_note <- if (isTRUE(input$recode_ordered)) " It will be an ordered categorical variable in the displayed order." else " It will be a categorical variable."
          list(message = paste0(changed, " cell(s) will receive a new label.", type_note), table = impact_table(c("Recorded value", "New label", "Affected rows"), Map(function(a, b, n) list(a, b, format(n, trim = TRUE)), from, to, counts)))
        } else if (identical(operation(), "type")) {
          if (!nzchar(input$variable %||% "")) return(NULL)
          list(message = paste0("All values will be retained. The variable will be treated as ", switch(input$variable_type %||% "factor", factor = "categorical", ordered = "ordered categorical", numeric = "numeric", text = "text"), "."))
        } else if (identical(operation(), "missing")) {
          if (identical(input$missing_action %||% "to_na", "show_category")) {
            if (!nzchar(input$variable %||% "")) return(NULL)
            affected <- sum(is.na(data[[input$variable]]))
            label <- trimws(input$missing_label %||% "")
            if (!nzchar(label)) return(list(message = "Enter a label to preview the missing category."))
            return(list(message = paste0(affected, " existing NA value(s) will be shown as the category ", encodeString(label, quote = '"'), ". This changes how the variable is analysed."), table = impact_table(c("Current value", "New category", "Affected rows"), list(list("NA (missing)", label, format(affected, trim = TRUE))))))
          }
          codes <- trimws(strsplit(input$codes %||% "", ",", fixed = TRUE)[[1L]])
          codes <- codes[nzchar(codes)]
          if (!length(codes) || !nzchar(input$variable %||% "")) return(NULL)
          affected <- sum(!is.na(data[[input$variable]]) & as.character(data[[input$variable]]) %in% codes)
          rows <- lapply(codes, function(code) {
            list(code, format(sum(!is.na(data[[input$variable]]) & as.character(data[[input$variable]]) %in% code), trim = TRUE))
          })
          list(message = paste0(affected, " value(s) will become missing (NA)."), table = impact_table(c("Code", "Values becoming missing"), rows))
        } else if (identical(operation(), "transform") && identical(transform_mode(), "arithmetic")) {
          if (!nzchar(input$transform_source %||% "")) return(NULL)
          mode <- input$calculation_mode %||% "single"
          second <- input$transform_second %||% ""
          count <- if (identical(mode, "single")) sum(!is.na(data[[input$transform_source]])) else if (second %in% names(data)) sum(!is.na(data[[input$transform_source]]) & !is.na(data[[second]])) else 0L
          description <- switch(mode, single = "The source variable will be retained.", two_variables = "Both source variables will be retained.", ratio_power = "This uses numerator / (denominator ^ power); both source variables will be retained.")
          list(message = paste0("A new variable will be created with ", count, " non-missing calculated value(s). ", description))
        } else if (identical(operation(), "filter")) {
          if (!nzchar(input$value1 %||% "") || !nzchar(input$variable1 %||% "")) return(NULL)
          variable2 <- if (isTRUE(input$filter_second)) input$variable2 %||% "" else ""
          value2 <- if (isTRUE(input$filter_second)) input$value2 %||% "" else ""
          filtered <- gt_dp_filter(data, input$variable1, input$operator1, input$value1, input$connector %||% "AND", variable2, input$operator2 %||% "==", value2)
          counts <- attr(filtered, "gt_dp_filter_counts")
          list(message = "Check the expected filter result before applying.", table = impact_table(c("Result", "Rows"), list(list("Kept", format(counts[["included"]], trim = TRUE)), list("Excluded", format(counts[["excluded"]], trim = TRUE)), list("Unknown", format(counts[["unknown"]], trim = TRUE)), list("Total", format(counts[["before"]], trim = TRUE)))))
        } else if (identical(operation(), "keep")) {
          selected <- input$keep_vars %||% character()
          if (!length(selected)) return(NULL)
          list(message = paste0(length(selected), " of ", ncol(data), " variable(s) will be retained; ", ncol(data) - length(selected), " will be removed from the working dataset."))
        } else if (identical(operation(), "transform")) {
          count <- suppressWarnings(as.integer(input$group_count %||% 2L))
          count <- if (count %in% 2:4) count else 2L
          indices <- seq_len(count - 1L)
          group_variable <- input$group_variable %||% ""
          if (!nzchar(group_variable) || !group_variable %in% names(data)) return(NULL)
          variables <- rep(group_variable, length(indices))
          operators <- vapply(indices, function(index) input[[paste0("rule_op_", index)]] %||% "==", character(1))
          values <- vapply(indices, function(index) input[[paste0("rule_value_", index)]] %||% "", character(1))
          values2 <- vapply(indices, function(index) input[[paste0("rule_value2_", index)]] %||% "", character(1))
          results <- vapply(indices, function(index) input[[paste0("rule_result_", index)]] %||% "", character(1))
          keep <- nzchar(values) & nzchar(results)
          range_complete <- !operators %in% c("between", "outside") | nzchar(values2)
          if (!all(keep) || !all(range_complete) || !nzchar(input$rule_default %||% "")) return(list(message = "Complete every group condition and label, including both bounds for ranges and the everyone-else group, to preview your groups."))
          grouped <- gt_dp_group_values(
            data, variables, operators, values, results,
            input$rule_default %||% "", values2
          )
          counts <- attr(grouped, "gt_dp_group_counts")
          empty_message <- gt_dp_empty_group_message(data, variables, counts)
          message <- if (is.null(empty_message)) {
            paste0("The new ", count, "-group variable uses the first matching condition.")
          } else {
            paste0("Cannot apply yet. ", empty_message)
          }
          list(
            message = message,
            table = impact_table(
              c("New group", "Rows"),
              lapply(seq_along(counts), function(index) {
                list(names(counts)[[index]], format(unname(counts[[index]]), trim = TRUE))
              })
            )
          )
        }
      }, error = function(e) NULL)
      if (is.null(content)) return(NULL)
      shiny::tags$div(class = "gtx-note", shiny::tags$strong("Before you apply: "), content$message, content$table)
    })
    apply_change <- function(label, after, code) {
      state$history <- c(state$history, list(list(label = label, before = state$working_data, after = after, code = code)))
      state$working_data <- after; state$redo_stack <- list()
    }
    shiny::observeEvent(input$apply, {
      shiny::req(state$working_data)
      tryCatch({
        if (identical(operation(), "rename")) {
          after <- gt_dp_rename(state$working_data, input$variable, input$new_name)
          apply_change(paste("Rename", input$variable, "to", input$new_name), after, gt_dp_code_line("rename", variable = input$variable, new_name = input$new_name))
        } else if (identical(operation(), "label")) {
          label <- trimws(input$variable_label %||% "")
          after <- gt_dp_set_label(state$working_data, input$variable, label)
          from <- label_level_values()
          to <- trimws(label_level_labels())
          level_change <- length(from) && !identical(from, to)
          if (level_change) {
            if (any(!nzchar(to))) stop("Enter a display label for every category.", call. = FALSE)
            after <- gt_dp_recode(after, input$variable, from, to, TRUE)
            after <- gt_dp_set_type(after, input$variable, "factor", to)
          }
          action_label <- if (nzchar(label)) {
            paste("Label", input$variable, "as", label)
          } else {
            paste("Remove display label from", input$variable)
          }
          if (level_change) action_label <- paste(action_label, "and update category labels")
          code <- gt_dp_code_line("label", variable = input$variable, label = label)
          if (level_change) {
            code <- paste(
              code,
              gt_dp_code_line("recode", variable = input$variable, from = from, to = to),
              gt_dp_code_line("type", variable = input$variable, type = "factor", levels = to),
              sep = "\n"
            )
          }
          apply_change(action_label, after, code)
        } else if (identical(operation(), "recode")) {
          from <- recode_values(); to <- recode_labels()
          if (!length(from) || any(!nzchar(to))) stop("Enter a new label for every displayed recorded value.", call. = FALSE)
          after <- gt_dp_recode(state$working_data, input$variable, from, to, TRUE)
          recoded <- attr(after, "gt_dp_affected")
          code <- gt_dp_code_line("recode", variable = input$variable, from = from, to = to)
          recode_type <- if (isTRUE(input$recode_ordered)) "ordered" else "factor"
          after <- gt_dp_set_type(after, input$variable, recode_type, to)
          final_levels <- base::levels(after[[input$variable]])
          code <- paste(code, gt_dp_code_line("type", variable = input$variable, type = recode_type, levels = final_levels), sep = "\n")
          type_label <- if (identical(recode_type, "ordered")) " as ordered categorical" else " as categorical"
          apply_change(paste0("Recode ", input$variable, " (", recoded, " cells)", type_label), after, code)
        } else if (identical(operation(), "type")) {
          level_values <- if (identical(input$variable_type, "ordered")) type_levels() else character()
          after <- gt_dp_set_type(state$working_data, input$variable, input$variable_type, level_values)
          final_levels <- if (input$variable_type %in% c("factor", "ordered")) base::levels(after[[input$variable]]) else character()
          apply_change(paste("Set", input$variable, "as", switch(input$variable_type, factor = "categorical", ordered = "ordered categorical", numeric = "numeric", text = "text")), after, gt_dp_code_line("type", variable = input$variable, type = input$variable_type, levels = final_levels))
        } else if (identical(operation(), "missing")) {
          if (identical(input$missing_action %||% "to_na", "show_category")) {
            after <- gt_dp_show_missing(state$working_data, input$variable, input$missing_label)
            apply_change(paste("Show missing values in", input$variable, "as", input$missing_label, "(", attr(after, "gt_dp_affected"), "cells)"), after, gt_dp_code_line("missing_category", variable = input$variable, label = input$missing_label))
          } else {
            codes <- trimws(strsplit(input$codes, ",", fixed = TRUE)[[1L]]); after <- gt_dp_define_missing(state$working_data, input$variable, codes)
            apply_change(paste("Convert missing-data codes in", input$variable, "to NA (", attr(after, "gt_dp_affected"), "cells)"), after, gt_dp_code_line("missing", variable = input$variable, codes = codes))
          }
        } else if (identical(operation(), "transform")) {
          if (identical(transform_mode(), "arithmetic")) {
            mode <- input$calculation_mode %||% "single"
            after <- gt_dp_calculate(state$working_data, input$transform_source, input$transform_new, mode, input$transform_operator %||% "+", input$transform_number %||% 1, input$transform_second %||% NULL)
            code <- switch(mode,
              single = paste0("data$", input$transform_new, " <- data$", input$transform_source, " ", input$transform_operator, " ", input$transform_number),
              two_variables = paste0("data$", input$transform_new, " <- data$", input$transform_source, " ", input$transform_operator, " data$", input$transform_second),
              ratio_power = paste0("data$", input$transform_new, " <- data$", input$transform_source, " / (data$", input$transform_second, " ^ ", input$transform_number, ")")
            )
          } else {
            count <- suppressWarnings(as.integer(input$group_count %||% 2L))
            count <- if (count %in% 2:4) count else 2L
            indices <- seq_len(count - 1L)
            group_variable <- input$group_variable %||% ""
            if (!nzchar(group_variable) || !group_variable %in% names(state$working_data)) stop("Choose the variable to group.", call. = FALSE)
            variables <- rep(group_variable, length(indices))
            operators <- vapply(indices, function(index) input[[paste0("rule_op_", index)]] %||% "==", character(1))
            values <- vapply(indices, function(index) input[[paste0("rule_value_", index)]] %||% "", character(1))
            values2 <- vapply(indices, function(index) input[[paste0("rule_value2_", index)]] %||% "", character(1))
            results <- vapply(indices, function(index) input[[paste0("rule_result_", index)]] %||% "", character(1))
            range_complete <- !operators %in% c("between", "outside") | nzchar(values2)
            if (any(!nzchar(values)) || any(!nzchar(results)) || !all(range_complete) || !nzchar(input$rule_default %||% "")) stop("Complete every group condition and label, including both bounds for ranges and the everyone-else group.", call. = FALSE)
            after <- gt_dp_transform_case_when(state$working_data, input$transform_new, variables, operators, values, results, input$rule_default, values2)
            rules <- which(nzchar(variables) & nzchar(values) & nzchar(results))
            value_code <- function(value) {
              number <- suppressWarnings(as.numeric(value))
              if (is.numeric(state$working_data[[group_variable]]) && is.finite(number)) format(number, scientific = FALSE, trim = TRUE) else encodeString(value, quote = '"')
            }
            condition_code <- function(index) {
              variable_code <- paste0("data$", variables[[index]])
              if (identical(operators[[index]], "between")) return(paste0("dplyr::between(", variable_code, ", ", value_code(values[[index]]), ", ", value_code(values2[[index]]), ")"))
              if (identical(operators[[index]], "outside")) return(paste0("(", variable_code, " < ", value_code(values[[index]]), " | ", variable_code, " > ", value_code(values2[[index]]), ")"))
              paste(variable_code, operators[[index]], value_code(values[[index]]))
            }
            rule_code <- vapply(rules, function(index) paste0("  ", condition_code(index), " ~ ", encodeString(results[[index]], quote = '"')), character(1))
            code <- if (length(indices) == 1L) paste0("data$", input$transform_new, " <- ifelse(", condition_code(1L), ", ", encodeString(results[[1L]], quote = '"'), ", ", encodeString(input$rule_default, quote = '"'), ")") else paste0("data$", input$transform_new, " <- dplyr::case_when(\n", paste(rule_code, collapse = ",\n"), ",\n  TRUE ~ ", encodeString(input$rule_default, quote = '"'), "\n)")
          }
          apply_change(paste("Create", input$transform_new, "(", attr(after, "gt_dp_affected"), "values matched)"), after, gt_dp_code_line("transform", code = code))
        } else if (identical(operation(), "keep")) {
          after <- gt_dp_keep_variables(state$working_data, input$keep_vars)
          apply_change(paste("Keep", ncol(after), "variable(s); remove", attr(after, "gt_dp_affected")), after, gt_dp_code_line("keep", variables = names(after)))
        } else {
          variable2 <- if (isTRUE(input$filter_second)) input$variable2 %||% "" else ""
          value2 <- if (isTRUE(input$filter_second)) input$value2 %||% "" else ""
          after <- gt_dp_filter(state$working_data, input$variable1, input$operator1, input$value1, input$connector %||% "AND", variable2, input$operator2 %||% "==", value2)
          counts <- attr(after, "gt_dp_filter_counts")
          expression <- paste(input$variable1, input$operator1, encodeString(input$value1, quote = '"'))
          if (nzchar(variable2) && nzchar(value2)) {
            connector <- if (identical(input$connector, "OR")) " | " else " & "
            expression <- paste0(expression, connector, variable2, " ", input$operator2, " ", encodeString(value2, quote = '"'))
          }
          apply_change(paste0("Filter: before ", counts[["before"]], "; included ", counts[["included"]], "; excluded ", counts[["excluded"]], "; unknown ", counts[["unknown"]]), after, gt_dp_code_line("filter", expression = expression))
        }
        shiny::showNotification("Change applied. Undo is available.", type = "message")
      }, error = function(e) shiny::showNotification(conditionMessage(e), type = "error", duration = NULL))
    })
    shiny::observeEvent(input$undo, { if (length(state$history)) { last <- state$history[[length(state$history)]]; state$history <- utils::head(state$history, -1L); state$redo_stack <- c(state$redo_stack, list(last)); state$working_data <- last$before } })
    shiny::observeEvent(input$redo, { if (length(state$redo_stack)) { last <- state$redo_stack[[length(state$redo_stack)]]; state$redo_stack <- utils::head(state$redo_stack, -1L); state$history <- c(state$history, list(last)); state$working_data <- last$after } })
    shiny::observeEvent(input$reset, {
      shiny::showModal(shiny::modalDialog(
        title = "Reset all data-preparation changes?",
        "This restores the original imported dataset and clears the change log. You cannot undo a reset.",
        footer = shiny::tagList(shiny::modalButton("Cancel"), shiny::actionButton(ns("confirm_reset"), "Reset all changes", class = "btn-danger"))
      ))
    }, ignoreInit = TRUE)
    shiny::observeEvent(input$confirm_reset, {
      state$working_data <- state$raw_data; state$history <- list(); state$redo_stack <- list(); state$result <- state$raw_data; state$using_prepared <- FALSE
      shiny::removeModal()
      shiny::showNotification("Data preparation reset. Original data is active for analysis.", type = "message")
    }, ignoreInit = TRUE)
    shiny::observeEvent(input$continue_original, { state$result <- state$raw_data; state$using_prepared <- FALSE; shiny::showNotification("Analyses will use the original data.", type = "message") })
    shiny::observeEvent(input$use_prepared, { state$result <- state$working_data; state$using_prepared <- TRUE; shiny::showNotification("Analyses will use the prepared data.", type = "message") })
    output$preview <- gt::render_gt({ gt::gt(utils::head(state$working_data, 10L)) })
    output$status <- shiny::renderText({
      analysis_state <- if (isTRUE(state$using_prepared)) {
        "analyses use prepared data"
      } else if (!is.null(state$result)) {
        "analyses use original data"
      } else {
        "original data will be active after loading"
      }
      paste(nrow(state$working_data), "prepared rows |", ncol(state$working_data), "variables |", length(state$history), "applied change(s) |", analysis_state)
    })
    output$history <- shiny::renderText({ if (!length(state$history)) "No changes applied." else paste(vapply(state$history, `[[`, character(1), "label"), collapse = "\n") })
    prep_code <- shiny::reactive({ if (!length(state$history)) "# No data-preparation changes applied." else paste(vapply(state$history, `[[`, character(1), "code"), collapse = "\n") })
    output$code <- shiny::renderText(prep_code())
    output$download_code <- shiny::downloadHandler(
      filename = function() "gtregression-data-prep.R",
      content = function(file) writeLines(prep_code(), file, useBytes = TRUE)
    )
    output$download_prepared <- shiny::downloadHandler(
      filename = function() paste0("gtregression-prepared-data.", input$prepared_format %||% "csv"),
      content = function(file) {
        data <- state$working_data
        format <- input$prepared_format %||% "csv"
        if (identical(format, "csv")) {
          utils::write.csv(data, file, row.names = FALSE, na = "")
        } else if (identical(format, "rds")) {
          saveRDS(data, file)
        } else {
          if (!requireNamespace("rio", quietly = TRUE)) {
            stop("Excel and Stata download require the optional 'rio' package. Install it with install.packages('rio') and restart the app.", call. = FALSE)
          }
          rio::export(data, file)
        }
      }
    )
    list(result = shiny::reactive(state$result), working_data = shiny::reactive(state$working_data), changed = shiny::reactive(length(state$history) > 0L), using_prepared = shiny::reactive(state$using_prepared), code = prep_code)
  })
}

#' Stratified univariable regression
#'
#' Fit univariable regression models within each level of a stratifier and
#' combine the results into a stratified table rendered with
#' \pkg{gt} or \pkg{flextable}.
#'
#' @param data A data frame containing the variables.
#' @param outcome Character scalar; name of the outcome variable. Quoted and
#'   bare names are accepted.
#' @param exposures Character vector of exposure variables to model. Quoted
#'   names are recommended in scripts, and bare names are also accepted.
#' @param stratifier Character scalar; name of the stratifying variable. Quoted
#'   and bare names are accepted.
#' @param approach Modeling approach. One of \code{"logit"}, \code{"firth"},
#'   \code{"logbinomial"}, \code{"poisson"}, \code{"robpoisson"},
#'   \code{"linear"}, or \code{"negbin"}.
#' @param format Output table format; one of \code{"flextable"} (default) or
#'   \code{"gt"}.
#' @param theme Table styling preset or theme primitives.
#' @param show_ref Logical; if \code{TRUE} (default), display reference-category
#'   rows as \code{"Ref."}. If \code{FALSE}, hide reference rows; a message
#'   reminds users to use \code{show_ref = TRUE} when reference rows are needed.
#'
#' @details
#' If exposure variables have a \code{"label"} attribute, for example from
#' \code{labelled::var_label()}, those labels are used automatically in the
#' displayed table. Internal matching still uses the original column names.
#'
#' @return A list of class
#'   \code{c("gtregression", "stratified_uni_reg", ...)} with elements:
#' \describe{
#'   \item{\code{table}}{A rendered \code{gt_tbl} or \code{flextable}.}
#'   \item{\code{table_display}}{Display-ready wide stratified results.}
#'   \item{\code{per_stratum}}{List of complete \code{uni_reg()} results by
#'   stratum.}
#'   \item{\code{models}}{List of fitted model objects by stratum.}
#'   \item{\code{model_summaries}}{List of model summaries by stratum.}
#'   \item{\code{variable_labels}}{Named character vector of display labels used
#'   for exposure variables.}
#'   \item{\code{reg_check}}{Regression diagnostics by stratum.}
#'   \item{\code{by}}{The stratifier variable.}
#'   \item{\code{levels}}{Strata included in the analysis.}
#'   \item{\code{approach}}{The regression approach used.}
#'   \item{\code{format}}{The output format used.}
#'   \item{\code{source}}{Function identifier
#'   (\code{"stratified_uni_reg"}).}
#' }
#'
#' @section Accessors:
#' \describe{
#'   \item{\code{$table}}{Rendered stratified regression table.}
#'   \item{\code{$table_display}}{Wide display data used to build the table.}
#'   \item{\code{$per_stratum}}{Full \code{uni_reg()} result objects by
#'   stratum.}
#'   \item{\code{$models}}{List of fitted model objects for each stratum.}
#'   \item{\code{$model_summaries}}{List of model summaries.}
#'   \item{\code{$reg_check}}{Diagnostic check results (when applicable).}
#' }
#'
#' @seealso [multi_reg()], [plot_reg()], [identify_confounder()]
#'
#' @examples
#' birthwt_data <- data_birthwt |>
#'   transform(
#'     race = factor(race, levels = c(1, 2, 3),
#'                   labels = c("White", "Black", "Other")),
#'     smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
#'     low = factor(low, levels = c(0, 1),
#'                  labels = c("Normal BW", "Low BW"))
#'   )
#'
#' stratified_uni <- stratified_uni_reg(
#'   data = birthwt_data,
#'   outcome = "low",
#'   exposures = c("age", "lwt", "smoke"),
#'   stratifier = "race",
#'   approach = "logit",
#'   format = "gt"
#' )
#'
#' stratified_uni$table
#'
#' @importFrom purrr map
#' @importFrom broom tidy


#' @export
stratified_uni_reg <- function(data, outcome, exposures, stratifier,
                               approach = "logit",
                               format   = c("flextable","gt"),
                               theme    = c("minimal"),
                               show_ref = TRUE) {
  outcome <- .vars_arg(substitute(outcome), env = parent.frame())
  exposures <- .vars_arg(substitute(exposures), env = parent.frame())
  stratifier <- .vars_arg(substitute(stratifier), env = parent.frame())
  approach <- .choice_arg(
    substitute(approach),
    env = parent.frame(),
    choices = c("logit","firth","logbinomial","poisson","robpoisson","linear","negbin")
  )
  approach <- .normalize_approach(approach)
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("flextable","gt"))
  theme <- .choice_arg(substitute(theme), env = parent.frame())
  .validate_show_ref(show_ref)

  format <- match.arg(format, c("flextable","gt"))
  theme  <- .resolve_theme(theme)
  variable_labels <- .var_label_map(data, unique(exposures))

  .validate_uni_inputs(data, outcome, exposures, approach)
  if (!stratifier %in% names(data)) stop("Stratifier not found in dataset.", call. = FALSE)

  data <- data[!is.na(data[[stratifier]]), , drop = FALSE]
  levs <- .strata_levels(data, stratifier)

  message("Running stratified univariate regression by: ", stratifier)

  per_stratum <- list()
  for (lv in levs) {
    message("  > Stratum: ", stratifier, " = ", lv)
    dlev <- data[data[[stratifier]] == lv, , drop = FALSE]
    res  <- tryCatch(
      suppressMessages(uni_reg(
        dlev,
        outcome,
        exposures,
        approach = approach,
        format = format,
        theme = theme,
        show_ref = show_ref
      )),
      error = function(e) { warning("Skipping stratum ", lv, ": ", e$message, call. = FALSE); NULL }
    )
    if (!is.null(res)) per_stratum[[as.character(lv)]] <- res
  }
  if (!length(per_stratum)) stop("No valid models across strata.", call. = FALSE)

  built <- .strata_build_wide_uni(
    data,
    outcome,
    exposures,
    stratifier,
    per_stratum,
    variable_labels = variable_labels,
    show_ref = show_ref
  )
  wide      <- built$wide
  spanners  <- built$spanners
  has_ref <- any(unlist(
    lapply(per_stratum, function(x) x$table_body$ref %in% TRUE),
    use.names = FALSE
  ))
  .message_hidden_ref_rows(
    "stratified_uni_reg",
    do.call(rbind, lapply(per_stratum, `[[`, "table_body")),
    show_ref
  )
  footnotes <- c(
    .stratified_by_note(stratifier),
    .abbrev_note(approach),
    if (isTRUE(show_ref) && has_ref) .ref_note() else NULL
  )
  eff_lab   <- .get_effect_label(approach)

  tbl <- if (format == "gt") {
    .build_gt_strata_wide_uni(wide, spanners, eff_lab, theme, footnotes)
  } else {
    .build_flex_strata_wide_uni(wide, spanners, eff_lab, theme, footnotes)
  }

  # collect metadata for users
  models_list <- lapply(per_stratum, `[[`, "models")
  summaries   <- lapply(per_stratum, `[[`, "model_summaries")
  diags       <- lapply(per_stratum, `[[`, "reg_check")

  fmt_class <- if (format == "gt") "gt_strata_uni" else "ft_strata_uni"
  out <- list(
    table         = tbl,
    table_display = wide,
    per_stratum   = per_stratum,
    models        = models_list,
    model_summaries = summaries,
    variable_labels = variable_labels,
    reg_check     = .as_reg_check_result(diags, format = format),
    by            = stratifier,
    levels        = levs,
    approach      = approach,
    format        = format,
    source        = "stratified_uni_reg",
    show_ref      = isTRUE(show_ref)
  )
  class(out) <- c("gtregression","stratified_uni_reg", fmt_class, class(out))
  out
}

birthwt_strata_uni_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
      ptl_cat = ifelse(ptl > 0, "Yes", "No"),
      ftv_cat = dplyr::case_when(
        ftv == 0 ~ "None",
        ftv == 1 ~ "One",
        ftv >= 2 ~ "Two or more"
      )
    ) |>
    dplyr::mutate(
      ptl_cat = factor(ptl_cat, levels = c("No", "Yes")),
      ftv_cat = factor(ftv_cat, levels = c("None", "One", "Two or more"))
    )
}

test_that("stratified_uni_reg returns a gtregression object for logit models", {
  df <- birthwt_strata_uni_data()

  res <- suppressMessages(
    stratified_uni_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "lwt", "smoke", "ht"),
      stratifier = "race",
      approach = logit,
      format = gt
    )
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "stratified_uni_reg")
  expect_s3_class(res, "gt_strata_uni")
  expect_s3_class(res$table, "gt_tbl")
  expect_equal(res$approach, "logit")
  expect_equal(res$format, "gt")
  expect_equal(res$source, "stratified_uni_reg")
  expect_equal(res$by, "race")
  expect_equal(res$levels, c("White", "Black", "Other"))
  expect_named(
    res,
    c("table", "table_display", "per_stratum", "models",
      "model_summaries", "variable_labels", "reg_check", "by", "levels", "approach",
      "format", "source", "show_ref")
  )
  expect_named(res$per_stratum, c("White", "Black", "Other"))
  expect_true(all(vapply(res$per_stratum, inherits, logical(1), what = "uni_reg")))
  expect_true(all(c("Characteristic", "is_header", "..N__White",
                    "..eff__White", "..p__White") %in% names(res$table_display)))
  eff_cols <- startsWith(names(res$table_display), "..eff__")
  expect_true("Ref." %in% unlist(res$table_display[eff_cols]))
  expect_true(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                        fixed = TRUE)))
})

test_that("stratified_uni_reg preserves factor level order and excludes missing strata", {
  df <- birthwt_strata_uni_data()
  df$race <- factor(df$race, levels = c("Other", "White", "Black"))
  df$race[1:3] <- NA

  res <- suppressMessages(
    stratified_uni_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "smoke"),
      stratifier = "race",
      approach = "logit"
    )
  )

  expect_equal(res$levels, c("Other", "White", "Black"))
  expect_named(res$per_stratum, c("Other", "White", "Black"))
  expect_true(all(!grepl("NA", names(res$table_display), fixed = TRUE)))
})

test_that("stratified_uni_reg explains how to show reference rows when hidden", {
  df <- birthwt_strata_uni_data()

  expect_message(
    res <- stratified_uni_reg(
      data = df,
      outcome = low,
      exposures = smoke,
      stratifier = ht,
      approach = logit,
      show_ref = FALSE
    ),
    "To display Ref., use `show_ref = TRUE`.",
    fixed = TRUE
  )

  expect_true(any(vapply(res$per_stratum, function(x) any(x$table_body$ref), logical(1))))
  eff_cols <- startsWith(names(res$table_display), "..eff__")
  expect_false("Ref." %in% unlist(res$table_display[eff_cols]))
  expect_false(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                         fixed = TRUE)))
})

test_that("stratified_uni_reg accepts bare and quoted options", {
  df <- birthwt_strata_uni_data()

  bare <- suppressMessages(
    stratified_uni_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "smoke"),
      stratifier = "race",
      approach = logit,
      format = gt,
      theme = clinical
    )
  )
  quoted <- suppressMessages(
    stratified_uni_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "smoke"),
      stratifier = "race",
      approach = "logit",
      format = "gt",
      theme = "clinical"
    )
  )

  expect_equal(bare$approach, quoted$approach)
  expect_equal(bare$format, quoted$format)
  expect_equal(bare$table_display, quoted$table_display)
})

test_that("stratified_uni_reg supports flextable output", {
  skip_if_not_installed("flextable")

  df <- birthwt_strata_uni_data()

  res <- suppressMessages(
    stratified_uni_reg(
      data = df,
      outcome = "low",
      exposures = c("age", "smoke"),
      stratifier = "race",
      approach = logit,
      format = flextable
    )
  )

  expect_s3_class(res, "ft_strata_uni")
  expect_s3_class(res$table, "flextable")
  expect_equal(res$format, "flextable")
})

test_that("stratified_uni_reg returns diagnostics for linear models", {
  df <- birthwt_strata_uni_data()

  res <- suppressMessages(
    stratified_uni_reg(
      data = df,
      outcome = "bwt",
      exposures = c("age", "lwt", "smoke"),
      stratifier = "race",
      approach = linear
    )
  )

  expect_s3_class(res, "stratified_uni_reg")
  expect_true(any(grepl("..eff__White", names(res$table_display), fixed = TRUE)))
  expect_equal(res$approach, "linear")
  expect_true(all(vapply(res$reg_check, is.list, logical(1))))
  expect_true("age" %in% names(res$reg_check$White))
  expect_true("Test" %in% names(res$reg_check$White$age))
})

test_that("stratified_uni_reg validates missing variables and outcome types", {
  df <- birthwt_strata_uni_data()

  expect_error(
    stratified_uni_reg(
      df,
      outcome = "low",
      exposures = "not_here",
      stratifier = "race",
      approach = logit
    ),
    "exposure variables were not found"
  )
  expect_error(
    stratified_uni_reg(
      df,
      outcome = "low",
      exposures = "age",
      stratifier = "not_here",
      approach = logit
    ),
    "Stratifier not found"
  )
  expect_error(
    stratified_uni_reg(
      df,
      outcome = "bwt",
      exposures = "age",
      stratifier = "race",
      approach = logit
    ),
    "requires either a factor variable"
  )
})

test_that("stratified_uni_reg errors when no valid strata remain", {
  df <- birthwt_strata_uni_data()
  df$race <- NA_character_

  expect_error(
    suppressWarnings(
      suppressMessages(
        stratified_uni_reg(
          df,
          outcome = "low",
          exposures = c("age", "lwt"),
          stratifier = "race",
          approach = logit
        )
      )
    ),
    "No valid models across strata"
  )
})

#' Launch the gtregression app
#'
#' Open a menu-driven Shiny app for common \pkg{gtregression} workflows:
#' data import, descriptive tables, regression tables, survival analysis,
#' diagnostics, causal mediation, plots, and exports.
#'
#' @param ... Additional arguments passed to \code{shiny::runApp()}.
#' @param launch.browser Logical; passed to \code{shiny::runApp()}. The default
#'   uses the RStudio Viewer when available, otherwise opens a browser only in
#'   interactive sessions.
#'
#' @return Invisibly returns the result of \code{shiny::runApp()}.
#'
#' @details
#' The app is intentionally kept out of the core package startup path. Shiny and
#' other interface packages are suggested dependencies and are loaded only when
#' \code{gtregression_app()} is called.
#'
#' The Advanced tab includes a guided candidate-model builder. Users can name
#' two to six models, choose model-specific exposures and adjustment variables,
#' optionally add an interaction, and track a primary exposure. The app fits
#' compatible \code{multi_reg()}, \code{cox_reg()}, or \code{surv_reg()} objects
#' before passing them to \code{compare_models()}. Generated code records every
#' fitting call and the final comparison for reproducible use outside the app.
#'
#' @examples
#' if (interactive()) {
#'   gtregression_app()
#' }
#'
#' @seealso \code{compare_models()}, \code{multi_reg()}, \code{cox_reg()},
#'   \code{surv_reg()}
#'
#' @export
gtregression_app <- function(..., launch.browser = NULL) {
  if (!requireNamespace("shiny", quietly = TRUE)) {
    stop(
      "The gtregression app requires the 'shiny' package.\n",
      "Install it with install.packages('shiny') and run gtregression_app() again.",
      call. = FALSE
    )
  }

  app_dir <- system.file("shiny", package = "gtregression")
  if (!nzchar(app_dir) || !dir.exists(app_dir)) {
    stop(
      "The gtregression Shiny app was not found in the installed package.",
      call. = FALSE
    )
  }

  if (is.null(launch.browser)) {
    launch.browser <- if (
      interactive() &&
        requireNamespace("rstudioapi", quietly = TRUE) &&
        rstudioapi::isAvailable()
    ) {
      rstudioapi::viewer
    } else {
      interactive()
    }
  }

  shiny::runApp(app_dir, launch.browser = launch.browser, ...)
}

#' Kaplan-Meier survival probability table
#'
#' Estimate Kaplan-Meier survival probabilities at user-specified follow-up
#' times, such as 30-day, 6-month, or 1-year survival.
#'
#' @param data A \code{data.frame} containing survival time and event status.
#' @param time Survival follow-up time. Quoted and bare names are accepted.
#' @param event Event indicator. Quoted and bare names are accepted. Numeric
#'   \code{0/1}, numeric \code{1/2}, logical, character, and factor variables
#'   are accepted. For two-level character or factor variables, the second level
#'   is treated as the event.
#' @param by Optional grouping variable for separate Kaplan-Meier survival
#'   probabilities. Quoted and bare names are accepted.
#' @param times Numeric vector of follow-up times at which survival probability
#'   should be estimated.
#' @param digits Number of digits for percentages and survival probabilities.
#' @param extend Logical. If \code{TRUE}, requested times beyond the observed
#'   follow-up range are retained using the last available Kaplan-Meier estimate.
#' @param format Output format. One of \code{"flextable"} (default),
#'   \code{"gt"}, or \code{"tibble"}.
#' @param theme Table styling preset.
#'
#' @return A list of class \code{c("gtregression","survival_prob", ...)}
#' with elements:
#' \describe{
#'   \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
#'   when \code{format = "tibble"}.}
#'   \item{\code{table_body}}{Tibble with Kaplan-Meier survival probabilities.}
#'   \item{\code{table_display}}{Display data frame used to render the table.}
#'   \item{\code{fit}}{Fitted \code{survfit} object.}
#'   \item{\code{time,event,by,times,format,source}}{Metadata fields.}
#' }
#'
#' @details
#' Survival probabilities are estimated from \code{survival::survfit()} at the
#' requested follow-up times. Events and censored counts are interval counts up
#' to each requested time point as returned by \code{summary.survfit()}.
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#'
#' survival_prob(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   by = trt,
#'   times = c(90, 180, 365)
#' )
#'
#' survival_prob(
#'   data = lung_data,
#'   time = "time",
#'   event = "status",
#'   times = c(90, 180),
#'   format = tibble
#' )
#'
#' @importFrom survival survfit
#' @importFrom stats complete.cases
#' @export
survival_prob <- function(data,
                          time,
                          event,
                          by = NULL,
                          times,
                          digits = 1,
                          extend = TRUE,
                          format = c("flextable", "gt", "tibble"),
                          theme = c("minimal")) {

  time <- .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  event <- .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  by <- .vars_arg(substitute(by), env = parent.frame(), allow_null = TRUE)
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  theme <- .choice_arg(substitute(theme), env = parent.frame())

  format <- match.arg(format, c("flextable", "gt", "tibble"))
  theme <- .resolve_theme(theme)

  data_clean <- .validate_survival_prob_inputs(
    data = data,
    time = time,
    event = event,
    by = by,
    times = times,
    digits = digits,
    extend = extend
  )

  if (!is.null(by)) {
    data_clean[[by]] <- factor(data_clean[[by]])
  }

  fit <- survival::survfit(.km_formula(time, event, by), data = data_clean)
  sm <- summary(fit, times = times, extend = extend)
  table_body <- .survival_prob_table_body(sm, by = by)

  if (format == "tibble") {
    return(tibble::as_tibble(table_body))
  }

  table_display <- .survival_prob_display(table_body, digits = digits)
  tbl <- .build_survival_prob_table(
    table_display,
    format = format,
    theme = theme
  )

  res <- list(
    table = tbl,
    table_body = tibble::as_tibble(table_body),
    table_display = table_display,
    fit = fit,
    time = time,
    event = event,
    by = by,
    times = times,
    extend = extend,
    format = format,
    source = "survival_prob"
  )

  class(res) <- c("gtregression", "survival_prob", class(res))
  res
}

#' @keywords internal
#' @noRd
.validate_survival_prob_inputs <- function(data, time, event, by, times, digits, extend) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }
  if (!is.character(time) || length(time) != 1L || !time %in% names(data)) {
    stop("`time` must be a single survival time variable in `data`.", call. = FALSE)
  }
  if (!is.character(event) || length(event) != 1L || !event %in% names(data)) {
    stop("`event` must be a single event indicator variable in `data`.", call. = FALSE)
  }
  if (!is.null(by) && (!is.character(by) || length(by) != 1L || !by %in% names(data))) {
    stop("`by` must be NULL or a single grouping variable in `data`.", call. = FALSE)
  }
  if (!is.numeric(data[[time]])) {
    stop("`time` must be numeric.", call. = FALSE)
  }
  if (any(data[[time]] < 0, na.rm = TRUE)) {
    stop("`time` must contain non-negative follow-up times.", call. = FALSE)
  }
  if (missing(times) || !is.numeric(times) || length(times) < 1L ||
      anyNA(times) || any(times < 0)) {
    stop("`times` must be a numeric vector of non-negative follow-up times.", call. = FALSE)
  }
  if (!is.numeric(digits) || length(digits) != 1L || is.na(digits) ||
      digits < 0 || digits != floor(digits)) {
    stop("`digits` must be a non-negative whole number.", call. = FALSE)
  }
  if (!is.logical(extend) || length(extend) != 1L || is.na(extend)) {
    stop("`extend` must be TRUE or FALSE.", call. = FALSE)
  }

  vars_needed <- unique(c(time, event, by))
  data_clean <- data[stats::complete.cases(data[, vars_needed, drop = FALSE]), , drop = FALSE]
  if (nrow(data_clean) == 0) {
    stop("No complete cases available for survival probabilities.", call. = FALSE)
  }

  data_clean[[event]] <- .cox_event01(data_clean[[event]])
  if (sum(data_clean[[event]] == 1) == 0) {
    stop("`event` must include at least one event.", call. = FALSE)
  }
  if (!is.null(by) && length(unique(data_clean[[by]])) < 2L) {
    stop("`by` must contain at least two non-missing groups.", call. = FALSE)
  }

  data_clean
}

#' @keywords internal
#' @noRd
.survival_prob_table_body <- function(sm, by = NULL) {
  group <- if (is.null(by)) {
    rep("Overall", length(sm$time))
  } else {
    sub("^.*=", "", sm$strata)
  }

  data.frame(
    Group = group,
    Time = as.numeric(sm$time),
    N.risk = as.numeric(sm$n.risk),
    Events = as.numeric(sm$n.event),
    Censored = as.numeric(sm$n.censor),
    Survival.probability = as.numeric(sm$surv),
    CI.lower = as.numeric(sm$lower),
    CI.upper = as.numeric(sm$upper),
    stringsAsFactors = FALSE
  )
}

#' @keywords internal
#' @noRd
.survival_prob_display <- function(table_body, digits = 1) {
  fmt_time <- function(x) {
    formatC(x, digits = digits, format = "f", big.mark = ",")
  }
  fmt_pct <- function(x) {
    ifelse(
      is.na(x),
      "NA",
      paste0(formatC(100 * x, digits = digits, format = "f"), "%")
    )
  }

  surv_txt <- paste0(
    fmt_pct(table_body$Survival.probability),
    " (",
    fmt_pct(table_body$CI.lower),
    "-",
    fmt_pct(table_body$CI.upper),
    ")"
  )

  data.frame(
    Group = table_body$Group,
    Time = fmt_time(table_body$Time),
    `At risk` = as.character(table_body$N.risk),
    Events = as.character(table_body$Events),
    Censored = as.character(table_body$Censored),
    `Survival probability (95% CI)` = surv_txt,
    check.names = FALSE,
    stringsAsFactors = FALSE
  )
}

#' @keywords internal
#' @noRd
.build_survival_prob_table <- function(display,
                                       format = c("flextable", "gt"),
                                       theme = c("minimal")) {
  format <- match.arg(format, c("flextable", "gt"))
  note <- paste(
    "Survival probabilities are estimated using Kaplan-Meier methods.",
    "Events and censored counts are interval counts up to each requested time point."
  )

  if (format == "gt") {
    tbl <- gt::gt(display) |>
      gt::tab_header(title = "Kaplan-Meier survival probabilities") |>
      gt::cols_align(align = "left", columns = "Group") |>
      gt::cols_align(
        align = "center",
        columns = c("Time", "At risk", "Events", "Censored", "Survival probability (95% CI)")
      ) |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_column_labels()
      ) |>
      gt::tab_source_note(gt::md(note)) |>
      .compact_gt_source_notes()

    if ("header_shaded" %in% theme) {
      tbl <- gt::tab_options(tbl, column_labels.background.color = "#f6f8fa")
    }
    if ("zebra" %in% theme) {
      tbl <- gt::opt_row_striping(tbl)
    }
    if ("compact" %in% theme) {
      tbl <- gt::tab_options(tbl, data_row.padding = gt::px(2))
    }
    return(tbl)
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Kaplan-Meier survival probabilities")
  ft <- flextable::align(ft, j = "Group", align = "left", part = "all")
  ft <- flextable::align(
    ft,
    j = c("Time", "At risk", "Events", "Censored", "Survival probability (95% CI)"),
    align = "center",
    part = "all"
  )
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  if ("header_shaded" %in% theme) {
    ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  }
  if ("zebra" %in% theme && nrow(display) > 1L) {
    ft <- flextable::bg(ft, i = seq(1, nrow(display), by = 2), bg = "#f6f8fa", part = "body")
  }
  if ("compact" %in% theme) {
    ft <- flextable::padding(ft, padding = 2, part = "body")
  }
  ft <- flextable::add_footer_lines(ft, values = note)
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

lung_survival_prob_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment"))
    )
}

test_that("survival_prob returns grouped Kaplan-Meier survival probabilities", {
  skip_if_not_installed("survival")

  df <- lung_survival_prob_data()

  res <- survival_prob(
    data = df,
    time = time,
    event = status,
    by = trt,
    times = c(90, 180, 365)
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "survival_prob")
  expect_s3_class(res$table, "flextable")
  expect_s3_class(res$fit, "survfit")
  expect_equal(res$source, "survival_prob")
  expect_equal(res$time, "time")
  expect_equal(res$event, "status")
  expect_equal(res$by, "trt")
  expect_equal(res$times, c(90, 180, 365))
  expect_true(res$extend)
  expect_equal(
    res$table_body$Group,
    rep(c("Standard treatment", "Test treatment"), each = 3)
  )
  expect_equal(res$table_body$Time, rep(c(90, 180, 365), times = 2))
  expect_equal(res$table_body$N.risk, c(37, 13, 4, 25, 14, 6))
  expect_equal(res$table_body$Events, c(31, 21, 8, 42, 9, 7))
  expect_equal(res$table_body$Censored, c(1, 3, 1, 2, 1, 1))
  expect_equal(
    round(res$table_body$Survival.probability, 3),
    c(0.547, 0.212, 0.071, 0.380, 0.233, 0.110)
  )
  expect_true("Survival probability (95% CI)" %in% names(res$table_display))
})

test_that("survival_prob returns overall tibble with quoted names", {
  skip_if_not_installed("survival")

  df <- lung_survival_prob_data()
  time_var <- "time"
  event_var <- "status"

  out <- survival_prob(
    data = df,
    time = time_var,
    event = event_var,
    times = c(90, 180),
    format = tibble
  )

  expect_s3_class(out, "tbl_df")
  expect_equal(out$Group, c("Overall", "Overall"))
  expect_equal(out$Time, c(90, 180))
  expect_equal(out$N.risk, c(62, 27))
  expect_equal(out$Events, c(73, 30))
  expect_equal(out$Censored, c(3, 4))
  expect_equal(round(out$Survival.probability, 3), c(0.464, 0.222))
})

test_that("survival_prob returns gt table", {
  skip_if_not_installed("survival")
  skip_if_not_installed("gt")

  df <- lung_survival_prob_data()

  res <- survival_prob(
    data = df,
    time = time,
    event = status,
    by = trt,
    times = c(90, 180),
    format = gt
  )

  expect_s3_class(res$table, "gt_tbl")
})

test_that("survival_prob validates inputs", {
  skip_if_not_installed("survival")

  df <- lung_survival_prob_data()
  df$bad_event <- 0
  df$one_group <- "A"

  expect_error(
    survival_prob(data = "not data", time = time, event = status, times = 90),
    "`data` must be"
  )
  expect_error(
    survival_prob(data = df, time = missing_time, event = status, times = 90),
    "`time` must be"
  )
  expect_error(
    survival_prob(data = df, time = time, event = missing_event, times = 90),
    "`event` must be"
  )
  expect_error(
    survival_prob(data = df, time = time, event = status),
    "`times` must be"
  )
  expect_error(
    survival_prob(data = df, time = time, event = status, times = -1),
    "`times` must be"
  )
  expect_error(
    survival_prob(data = df, time = time, event = bad_event, times = 90),
    "`event` must include at least one event"
  )
  expect_error(
    survival_prob(data = df, time = time, event = status, by = one_group, times = 90),
    "`by` must contain at least two"
  )
  expect_error(
    survival_prob(data = df, time = time, event = status, times = 90, digits = -1),
    "`digits` must be"
  )
  expect_error(
    survival_prob(data = df, time = time, event = status, times = 90, extend = NA),
    "`extend` must be"
  )
})

#' Linear regression diagnostic checks (internal)
#'
#' Runs diagnostic checks for fitted linear regression models.
#'
#' The returned rows cover heteroskedasticity, residual normality, model
#' specification, and influential observations. If a diagnostic cannot be
#' computed for a given model, the function returns a clear "not available"
#' result for that row instead of failing.
#'
#' @param model A fitted linear model (`lm` object).
#' @param exposure Character string giving the name of the exposure variable
#'   for labeling.
#'
#' @return A data frame with one row per diagnostic test, including:
#' \describe{
#'   \item{Exposure}{Name of the exposure variable.}
#'   \item{Test}{Diagnostic test name.}
#'   \item{Statistic}{Test statistic or summary (e.g., p-values).}
#'   \item{Interpretation}{Plain-language result interpretation.}
#' }
#'
#' @keywords internal
.reg_check_linear <- function(model, exposure) {
  if (!inherits(model, "lm")) {
    stop("`model` must be a fitted linear model created by lm().", call. = FALSE)
  }
  if (!is.character(exposure) || length(exposure) != 1L ||
      is.na(exposure) || !nzchar(exposure)) {
    stop("`exposure` must be a single non-empty character string.",
         call. = FALSE)
  }

  safe_p_value <- function(expr) {
    result <- tryCatch(
      suppressWarnings(expr),
      error = function(e) NULL
    )
    if (is.null(result) || is.null(result$p.value) ||
        length(result$p.value) != 1L || is.na(result$p.value)) {
      return(NA_real_)
    }
    as.numeric(result$p.value)
  }

  format_p <- function(p) {
    if (is.na(p)) {
      return("not available")
    }
    paste0("p = ", signif(p, 4))
  }

  bp_p <- safe_p_value(lmtest::bptest(model))

  resid <- stats::residuals(model)
  resid <- resid[is.finite(resid)]
  sw_p <- if (length(resid) < 3L) {
    NA_real_
  } else {
    safe_p_value(stats::shapiro.test(utils::head(resid, 5000L)))
  }

  reset_p <- safe_p_value(lmtest::resettest(model, power = 2:3, type = "fitted"))

  n <- tryCatch(stats::nobs(model), error = function(e) NA_integer_)
  cooks <- tryCatch(
    suppressWarnings(stats::cooks.distance(model)),
    error = function(e) numeric(0)
  )
  cooks <- cooks[is.finite(cooks)]
  threshold <- if (!is.na(n) && n > 0) 4 / n else NA_real_
  high_infl <- if (length(cooks) && !is.na(threshold)) {
    sum(cooks > threshold, na.rm = TRUE)
  } else {
    NA_integer_
  }

  interp <- c(
    if (is.na(bp_p)) {
      "Heteroskedasticity test could not be calculated for this model."
    } else if (bp_p < 0.05) {
      "Heteroskedasticity detected: residual variance may not be constant."
    } else {
      "No evidence of heteroskedasticity."
    },
    if (is.na(sw_p)) {
      "Residual normality test could not be calculated for this model."
    } else if (sw_p < 0.05) {
      "Residuals may not be normally distributed; use caution with small samples."
    } else {
      "Residuals appear normally distributed."
    },
    if (is.na(reset_p)) {
      "Model specification test could not be calculated for this model."
    } else if (reset_p < 0.05) {
      "Model may be mis-specified; consider non-linear terms or interactions."
    } else {
      "Functional form appears adequate."
    },
    if (is.na(high_infl)) {
      "Cook's distance could not be calculated for this model."
    } else if (high_infl > 0) {
      "Influential points detected; review outliers or high-leverage observations."
    } else {
      "No strong influential observations detected."
    }
  )

  cook_stat <- if (is.na(high_infl) || is.na(threshold)) {
    "not available"
  } else {
    paste0(high_infl, " obs > 4/n (", round(threshold, 4), ")")
  }

  data.frame(
    Exposure = rep(exposure, 4),
    Test = c("Breusch-Pagan", "Shapiro-Wilk", "RESET", "Cook's Distance"),
    Statistic = c(
      format_p(bp_p),
      format_p(sw_p),
      format_p(reset_p),
      cook_stat
    ),
    Interpretation = interp,
    stringsAsFactors = FALSE
  )
}

test_that(".reg_check_linear works as expected", {
  data(mtcars)
  model <- lm(mpg ~ wt + hp, data = mtcars)
  result <- gtregression:::.reg_check_linear(model, exposure = "hp")

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 4)
  expect_named(result, c("Exposure", "Test", "Statistic", "Interpretation"))
  expect_equal(unique(result$Exposure), "hp")
  expect_equal(
    result$Test,
    c("Breusch-Pagan", "Shapiro-Wilk", "RESET", "Cook's Distance")
  )
  expect_true(all(nzchar(result$Statistic)))
  expect_true(all(nzchar(result$Interpretation)))
})

test_that(".reg_check_linear validates inputs", {
  data(mtcars)
  model <- lm(mpg ~ wt, data = mtcars)

  expect_error(
    gtregression:::.reg_check_linear(model = mtcars, exposure = "wt"),
    "`model` must be a fitted linear model"
  )
  expect_error(
    gtregression:::.reg_check_linear(model = model, exposure = c("wt", "hp")),
    "`exposure` must be a single non-empty character string"
  )
  expect_error(
    gtregression:::.reg_check_linear(model = model, exposure = ""),
    "`exposure` must be a single non-empty character string"
  )
  expect_error(
    gtregression:::.reg_check_linear(model = model, exposure = NA_character_),
    "`exposure` must be a single non-empty character string"
  )
})

test_that(".reg_check_linear returns stable output when diagnostics are unavailable", {
  small_data <- data.frame(y = c(1, 2), x = c(0, 1))
  small_model <- lm(y ~ x, data = small_data)

  result <- gtregression:::.reg_check_linear(small_model, exposure = "x")

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 4)
  expect_true("not available" %in% result$Statistic)
  expect_true(any(grepl("could not be calculated", result$Interpretation)))
})

test_that(".reg_check_linear handles influential observations", {
  influential_data <- data.frame(
    y = c(1, 2, 3, 4, 5, 60),
    x = c(1, 2, 3, 4, 5, 6)
  )
  model <- lm(y ~ x, data = influential_data)

  result <- gtregression:::.reg_check_linear(model, exposure = "x")
  cook_row <- result[result$Test == "Cook's Distance", ]

  expect_match(cook_row$Statistic, "obs > 4/n", fixed = TRUE)
  expect_match(cook_row$Interpretation, "Influential points detected")
})

#' Run core multivariable regression workflow
#'
#' @keywords internal
#' @noRd
.model_summary <- function(model) {
  out <- NULL
  utils::capture.output(out <- summary(model))
  out
}

#' @keywords internal
#' @noRd
.run_multi_core <- function(data,
                            outcome,
                            exposures,
                            approach,
                            adjust_for = NULL,
                            interaction = NULL) {

  adjusted_mode <- !is.null(adjust_for) && length(adjust_for) > 0

  if (!adjusted_mode) {
    data_clean <- .validate_multi_inputs(
      data = data,
      outcome = outcome,
      exposures = exposures,
      approach = approach,
      adjust_for = adjust_for,
      interaction = interaction
    )

    fit <- .fit_multi_model(
      data = data_clean,
      outcome = outcome,
      exposures = exposures,
      approach = approach,
      adjust_for = NULL,
      interaction = interaction
    )

    if (is.null(fit)) {
      stop("Model fitting failed.", call. = FALSE)
    }

    td <- .tidy_multi(fit, exposures, approach)

    if (is.null(td) || !nrow(td)) {
      stop("No estimable coefficients for supplied exposures.", call. = FALSE)
    }

    model_list <- list(multivariable_model = fit)
    model_summaries <- list(multivariable_model = .model_summary(fit))
    reg_diagnostics <- list(
      multivariable_model = if (approach == "linear") {
        .reg_check_linear(fit, exposure = "multivariable_model")
      } else {
        "Regression diagnostics available only for 'linear' models."
      }
    )

    n_used <- tryCatch(stats::nobs(fit), error = function(e) NA_integer_)

  } else {
    fits <- vector("list", length(exposures))
    names(fits) <- exposures

    tds <- vector("list", length(exposures))
    names(tds) <- exposures
    n_used <- stats::setNames(rep(NA_integer_, length(exposures)), exposures)

    for (i in seq_along(exposures)) {
      exp_i <- exposures[i]
      data_i <- .validate_multi_inputs(
        data = data,
        outcome = outcome,
        exposures = exp_i,
        approach = approach,
        adjust_for = adjust_for,
        interaction = interaction
      )

      fit_i <- .fit_multi_model(
        data = data_i,
        outcome = outcome,
        exposures = exp_i,
        approach = approach,
        adjust_for = adjust_for,
        interaction = interaction
      )

      if (is.null(fit_i)) {
        stop(
          paste0("Model fitting failed for exposure '", exp_i, "'."),
          call. = FALSE
        )
      }

      td_i <- .tidy_multi(fit_i, exp_i, approach)

      if (is.null(td_i) || !nrow(td_i)) {
        stop(
          paste0("No estimable coefficients for exposure '", exp_i, "'."),
          call. = FALSE
        )
      }

      fits[[i]] <- fit_i
      tds[[i]] <- td_i
      n_used[[i]] <- tryCatch(stats::nobs(fit_i), error = function(e) nrow(data_i))
    }

    td <- do.call(rbind, tds)

    model_list <- fits
    model_summaries <- lapply(fits, .model_summary)
    reg_diagnostics <- lapply(
      seq_along(fits),
      function(i) {
        fit_i <- fits[[i]]
        exp_i <- names(fits)[i]
        if (approach == "linear") {
          .reg_check_linear(fit_i, exposure = exp_i)
        } else {
          "Regression diagnostics available only for 'linear' models."
        }
      }
    )
    names(reg_diagnostics) <- names(fits)

    data_clean <- data
  }

  list(
    data_clean = data_clean,
    table_body = td,
    models = model_list,
    model_summaries = model_summaries,
    reg_check = reg_diagnostics,
    n_used = n_used,
    adjusted_mode = adjusted_mode
  )
}

#' @keywords internal
#' @noRd
.compact_flex_footer <- function(ft) {
  ft <- flextable::fontsize(ft, size = 8, part = "footer")
  ft <- flextable::padding(ft, part = "footer", padding.top = 0, padding.bottom = 0)
  ft <- flextable::line_spacing(ft, part = "footer", space = 0.9)
  ft <- flextable::align(ft, part = "footer", align = "left")
  ft
}

#' @keywords internal
#' @noRd
.apply_theme_flex <- function(ft, df, theme) {
  ft <- flextable::theme_vanilla(ft)
  hdr_idx <- which(df$is_header %in% TRUE)
  lvl_idx <- which(df$is_header %in% FALSE)

  # indent level rows (flextable ignores leading spaces)
  if (length(lvl_idx)) ft <- flextable::padding(ft, i = lvl_idx, j = "Characteristic", padding.left = 14)

  if ("header_shaded" %in% theme) {
    ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  }
  if ("zebra" %in% theme && length(lvl_idx)) {
    ft <- flextable::bg(ft, i = lvl_idx[seq(1, length(lvl_idx), by = 2)], bg = "#f6f8fa")
  }
  # lines — use hline
  if ("lines" %in% theme) {
    n_body <- nrow(ft$body$dataset)
    if (!is.null(n_body) && n_body > 0) {
      ft <- flextable::hline(ft, i = 1, part = "body")
      ft <- flextable::hline(ft, i = n_body, part = "body")
    }
  }
  # bold exposure labels by default
  if (length(hdr_idx)) ft <- flextable::bold(ft, i = hdr_idx, j = "Characteristic", bold = TRUE)

  if ("compact" %in% theme) ft <- flextable::padding(ft, padding = 2)
  ft
}

#' @keywords internal
#' @noRd

.build_flextable <- function(df, effect_label, source_note, theme) {
  if (!requireNamespace("flextable", quietly = TRUE))
    stop("Install 'flextable' or use format='gt'.", call. = FALSE)
  .must_be_display_df(df)

  df$N <- ifelse(df$is_header, df$N, NA_integer_)
  out <- df[, c("Characteristic", "N", effect_label, "p-value"), drop = FALSE]
  out$N <- ifelse(is.na(out$N), "", as.character(out$N))

  ft <- flextable::flextable(out)
  ft <- flextable::align(ft, j = "Characteristic", align = "left")
  ft <- flextable::align(ft, j = c("N", effect_label, "p-value"), align = "center")
  ft <- flextable::bold(ft, part = "header", bold = TRUE)

  ft <- .apply_theme_flex(ft, df, theme)
  ft <- flextable::autofit(ft)

  ft <- flextable::add_footer_lines(ft, values = source_note)
  .compact_flex_footer(ft)
}
#' @keywords internal
#' @noRd

.build_flextable_multi <- function(df, effect_label, footnotes, theme) {
  if (!requireNamespace("flextable", quietly = TRUE))
    stop("Install 'flextable' or use format='gt'.", call. = FALSE)

  out <- df[, c("Characteristic", effect_label, "p-value"), drop = FALSE]
  ft <- flextable::flextable(out)
  ft <- flextable::align(ft, j = "Characteristic", align = "left")
  ft <- flextable::align(ft, j = c(effect_label, "p-value"), align = "center")
  ft <- flextable::bold(ft, part = "header", bold = TRUE)

  hdr_idx <- which(df$is_header %in% TRUE)
  lvl_idx <- which(df$is_header %in% FALSE)
  if (length(lvl_idx)) ft <- flextable::padding(ft, i = lvl_idx, j = "Characteristic", padding.left = 14)
  ft <- .apply_theme_flex(ft, df, theme)

  ft <- flextable::autofit(ft)

  # two separate lines in the footer
  ft <- flextable::add_footer_lines(ft, values = footnotes)
  .compact_flex_footer(ft)
}
#' Flextable builder for stratified wide tables (3 cols per stratum)
#' @keywords internal
.build_flex_strata_wide_uni <- function(df, spanners, effect_label, theme, footnotes) {
  if (!requireNamespace("flextable", quietly = TRUE))
    stop("Install 'flextable' or use format='gt'.", call. = FALSE)

  spec <- .strata_display_spec(df, spanners, effect_label)
  out <- df[, c("Characteristic", spec$ids), drop = FALSE]

  ft <- flextable::flextable(out)
  ft <- flextable::set_header_labels(ft, values = setNames(spec$labels, names(out)))
  ft <- flextable::add_header_row(ft, values = c("", spanners), colwidths = c(1, spec$widths))
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  ft <- flextable::align(ft, j = "Characteristic", align = "left",  part = "all")
  ft <- flextable::align(ft, j = setdiff(names(out), "Characteristic"), align = "center", part = "all")

  # body styling (NO row-index offset)
  lvl_idx <- which(df$is_header %in% FALSE)
  hdr_idx <- which(df$is_header %in% TRUE)
  if (length(lvl_idx)) ft <- flextable::padding(ft, i = lvl_idx, j = "Characteristic",
                                                padding.left = 14, part = "body")
  if (length(hdr_idx)) ft <- flextable::bold(ft, i = hdr_idx, j = "Characteristic",
                                             bold = TRUE, part = "body")

  # themes
  if ("header_shaded" %in% theme) ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  if ("zebra" %in% theme && length(lvl_idx)) {
    ft <- flextable::bg(ft, i = lvl_idx[seq(1, length(lvl_idx), by = 2)], bg = "#f6f8fa", part = "body")
  }
  if ("lines" %in% theme) {
    n_body <- nrow(df)
    if (!is.null(n_body) && n_body > 0) {
      ft <- flextable::hline(ft, i = 1,      part = "body")
      ft <- flextable::hline(ft, i = n_body, part = "body")
    }
  }
  if ("compact" %in% theme) ft <- flextable::padding(ft, padding = 2, part = "body")

  ft <- flextable::autofit(ft)
  ft <- flextable::add_footer_lines(ft, values = footnotes)
  .compact_flex_footer(ft)
}

#' Flextable builder for stratified MULTIVARIABLE wide tables (2 cols/stratum)
#' @keywords internal
.build_flex_strata_wide_multi <- function(df, spanners, effect_label_adj, theme, footnotes) {
  if (!requireNamespace("flextable", quietly = TRUE))
    stop("Install 'flextable' or use format='gt'.", call. = FALSE)

  spec <- .strata_display_spec(df, spanners, effect_label_adj)
  out <- df[, c("Characteristic", spec$ids), drop = FALSE]

  ft <- flextable::flextable(out)
  ft <- flextable::set_header_labels(ft, values = setNames(spec$labels, names(out)))
  ft <- flextable::add_header_row(ft, values = c("", spanners), colwidths = c(1, spec$widths))
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  ft <- flextable::align(ft, j = "Characteristic", align = "left",  part = "all")
  ft <- flextable::align(ft, j = setdiff(names(out), "Characteristic"), align = "center", part = "all")

  lvl_idx <- which(df$is_header %in% FALSE)
  hdr_idx <- which(df$is_header %in% TRUE)
  if (length(lvl_idx)) ft <- flextable::padding(ft, i = lvl_idx, j = "Characteristic", padding.left = 14, part = "body")
  if (length(hdr_idx)) ft <- flextable::bold(ft, i = hdr_idx, j = "Characteristic", bold = TRUE, part = "body")

  if ("header_shaded" %in% theme) ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  if ("zebra" %in% theme && length(lvl_idx)) {
    ft <- flextable::bg(ft, i = lvl_idx[seq(1, length(lvl_idx), by = 2)], bg = "#f6f8fa", part = "body")
  }
  if ("lines" %in% theme) {
    n_body <- nrow(df)
    if (!is.null(n_body) && n_body > 0) {
      ft <- flextable::hline(ft, i = 1,      part = "body")
      ft <- flextable::hline(ft, i = n_body, part = "body")
    }
  }
  if ("compact" %in% theme) ft <- flextable::padding(ft, padding = 2, part = "body")

  ft <- flextable::autofit(ft)
  ft <- flextable::add_footer_lines(ft, values = footnotes)
  .compact_flex_footer(ft)
}

#' Parametric survival regression
#'
#' Fit parametric survival models and report time ratios.
#'
#' @param data A \code{data.frame} containing survival time, event status, and
#'   exposure variables.
#' @param time Survival follow-up time. Quoted and bare names are accepted.
#' @param event Event indicator. Quoted and bare names are accepted. Numeric
#'   \code{0/1}, numeric \code{1/2}, logical, character, and factor variables
#'   are accepted. For two-level character or factor variables, the second level
#'   is treated as the event.
#' @param exposures Character vector of exposure variable names. Quoted names
#'   are recommended in scripts, and bare names are also accepted.
#' @param adjust_for Optional character vector of adjustment variables. When
#'   supplied, one adjusted model is fitted per exposure.
#' @param stratifier Optional single stratifying variable. When supplied,
#'   stratum-specific parametric survival tables are produced using the same
#'   crude, adjusted, or multivariable workflow requested by the other
#'   arguments. The stratifier cannot also be used as the time, event,
#'   exposure, adjustment, or interaction variable.
#' @param interaction Optional character scalar specifying one interaction term
#'   using standard formula syntax, e.g. \code{"trt*prior"}. Quoted and bare
#'   interaction syntax are accepted. In exposure-by-exposure mode, supply a
#'   single exposure; in \code{multivariable = TRUE} mode, the interaction is
#'   added to the single multivariable model.
#' @param multivariable Logical; if \code{FALSE} (default), the current
#'   exposure-by-exposure workflow is used. If \code{TRUE}, one multivariable
#'   parametric survival model is fitted using all variables in
#'   \code{exposures}, and all exposure coefficients are reported.
#' @param multivariate Optional logical alias for \code{multivariable}. This is
#'   accepted for convenience; \code{multivariable} is used internally.
#' @param distribution Parametric survival distribution. One of
#'   \code{"weibull"}, \code{"exponential"}, \code{"lognormal"}, or
#'   \code{"loglogistic"}. Quoted and bare values are accepted. Common
#'   spellings such as \code{"log-normal"} and \code{"log-logistic"} are also
#'   accepted.
#' @param format Output table format; one of \code{"flextable"} (default) or
#'   \code{"gt"}.
#' @param theme Table styling preset.
#' @param show_sample For stratified parametric survival tables, controls which
#'   sample-size columns are shown in the publication table. One of
#'   \code{"events"} (default), \code{"n"}, \code{"both"}, or \code{"none"}.
#'   Model statistics, when requested, still retain both N and event counts.
#' @param model_stats Logical; if \code{TRUE}, extract model-fit statistics
#'   including AIC, BIC, log-likelihood, scale, number of events, and N.
#' @param show_ref Logical; if \code{TRUE} (default), display reference-category
#'   rows as \code{"Ref."}. If \code{FALSE}, hide reference rows; a message
#'   reminds users to use \code{show_ref = TRUE} when reference rows are needed.
#'
#' @details
#' \code{surv_reg()} fits accelerated failure time style parametric survival
#' models using \code{survival::survreg()}. The exponentiated coefficient is
#' displayed as a time ratio. A time ratio above 1 suggests longer survival time;
#' a time ratio below 1 suggests shorter survival time, conditional on the chosen
#' distribution.
#'
#' By default, \code{surv_reg()} keeps the exposure-by-exposure workflow:
#' without \code{adjust_for}, one crude model is fitted per exposure; with
#' \code{adjust_for}, one adjusted model is fitted per exposure and only the
#' exposure estimate is reported.
#'
#' With \code{multivariable = TRUE}, all variables in \code{exposures} are
#' included in one parametric survival model and all coefficients are reported.
#' Since these estimates are adjusted for the other variables in the same model,
#' the table reports \code{Adjusted Time Ratio (95\% CI)}. The
#' \code{adjust_for} argument is not used in this mode; include every variable
#' that belongs in the model inside \code{exposures}.
#'
#' Interaction terms specified via \code{interaction} are included using
#' standard formula expansion (for example, \code{trt*prior}). Interaction
#' effects are displayed as additional rows beneath the corresponding exposure.
#'
#' Stratified parametric survival tables show event counts by default. Use
#' \code{show_sample = "n"}, \code{show_sample = "both"}, or
#' \code{show_sample = "none"} to control the displayed sample columns. Crude
#' stratified tables calculate these counts for each exposure-specific model;
#' adjusted and multivariable stratified tables use the corresponding fitted
#' model within each stratum.
#'
#' If exposure variables have a \code{"label"} attribute, for example from
#' \code{labelled::var_label()}, those labels are used automatically in the
#' displayed table.
#'
#' @return A list of class \code{c("gtregression","surv_reg", ...)} with elements:
#' \describe{
#'   \item{table}{A \code{flextable} or \code{gt_tbl}.}
#'   \item{table_body}{Data frame of time ratios and confidence intervals.}
#'   \item{table_display}{Data frame used to render the publication table.}
#'   \item{models}{List of fitted \code{survreg} models.}
#'   \item{model_summaries}{Summary output for the fitted models.}
#'   \item{model_stats}{Model-fit statistics when \code{model_stats = TRUE};
#'   otherwise \code{NULL}.}
#'   \item{variable_labels}{Named character vector of display labels.}
#'   \item{footnotes}{Character vector used by the rendered table. Adjustment
#'   notes use the same display labels as the table.}
#'   \item{time,event,distribution,approach,format,source,adjust_for,exposures,interaction}{Metadata fields.}
#' }
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#' lung_data$prior <- factor(lung_data$prior, levels = c(0, 10),
#'                           labels = c("No", "Yes"))
#'
#' surv_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = c("trt", "celltype", "karno", "age")
#' )
#'
#' surv_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = c(trt, celltype, prior),
#'   adjust_for = c(age, karno),
#'   distribution = lognormal
#' )
#'
#' # Interaction in an adjusted exposure model
#' surv_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = trt,
#'   adjust_for = c(age, karno),
#'   interaction = trt*prior,
#'   distribution = weibull
#' )
#'
#' surv_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = c(trt, celltype, prior, age, karno),
#'   distribution = weibull,
#'   multivariable = TRUE
#' )
#'
#' # multivariate is accepted as an alias
#' surv_reg(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = c(trt, age, karno),
#'   multivariate = TRUE
#' )
#'
#' @importFrom survival Surv survreg
#' @export
surv_reg <- function(data,
                     time,
                     event,
                     exposures,
                     adjust_for = NULL,
                     stratifier = NULL,
                     interaction = NULL,
                     multivariable = FALSE,
                     multivariate = NULL,
                     distribution = "weibull",
                     format = c("flextable", "gt"),
                     theme = c("minimal"),
                     show_sample = "events",
                     model_stats = FALSE,
                     show_ref = TRUE) {

  time <- .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  event <- .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  exposures <- .vars_arg(substitute(exposures), env = parent.frame())
  adjust_for <- .vars_arg(substitute(adjust_for), env = parent.frame(), allow_null = TRUE)
  stratifier <- .survival_optional_single_var_arg(
    substitute(stratifier),
    data = data,
    env = parent.frame()
  )
  interaction <- .interaction_arg(substitute(interaction), env = parent.frame(), allow_null = TRUE)
  distribution <- .surv_distribution_arg(
    substitute(distribution),
    env = parent.frame(),
    multiple = FALSE,
    arg = "distribution"
  )
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("flextable", "gt"))
  theme <- .choice_arg(substitute(theme), env = parent.frame())
  show_sample <- .choice_arg(
    substitute(show_sample),
    env = parent.frame(),
    choices = c("events", "n", "both", "none")
  )

  if (!is.logical(model_stats) || length(model_stats) != 1L || is.na(model_stats)) {
    stop("`model_stats` must be TRUE or FALSE.", call. = FALSE)
  }
  .validate_show_ref(show_ref)
  if (!is.null(multivariate)) {
    if (!is.logical(multivariate) || length(multivariate) != 1L || is.na(multivariate)) {
      stop("`multivariate` must be TRUE or FALSE.", call. = FALSE)
    }
    if (!identical(multivariable, FALSE) && !identical(multivariable, multivariate)) {
      stop("Use only one of `multivariable` or `multivariate`, or give them the same value.", call. = FALSE)
    }
    multivariable <- multivariate
  }
  if (!is.logical(multivariable) || length(multivariable) != 1L || is.na(multivariable)) {
    stop("`multivariable` must be TRUE or FALSE.", call. = FALSE)
  }
  if (isTRUE(multivariable) && !is.null(adjust_for) && length(adjust_for) > 0) {
    stop(
      "`adjust_for` is not used when `multivariable = TRUE`; include all model variables in `exposures`.",
      call. = FALSE
    )
  }

  format <- match.arg(format, c("flextable", "gt"))
  show_sample_choices <- c("events", "n", "both", "none")
  if (!is.character(show_sample) || length(show_sample) != 1L || !show_sample %in% show_sample_choices) {
    stop("`show_sample` must be one of 'events', 'n', 'both', or 'none'.", call. = FALSE)
  }
  show_sample <- match.arg(show_sample, show_sample_choices)
  theme <- .resolve_theme(theme)
  adjusted_mode <- !is.null(adjust_for) && length(adjust_for) > 0
  fmt_class <- if (format == "gt") "gt_surv" else "ft_surv"

  if (!is.null(stratifier)) {
    .validate_survival_stratifier(
      data = data,
      time = time,
      event = event,
      exposures = exposures,
      adjust_for = adjust_for,
      interaction = interaction,
      stratifier = stratifier
    )

    return(.run_stratified_surv_reg(
      data = data,
      time = time,
      event = event,
      exposures = exposures,
      adjust_for = adjust_for,
      stratifier = stratifier,
      interaction = interaction,
      multivariable = multivariable,
      distribution = distribution,
      format = format,
      theme = theme,
      show_sample = show_sample,
      model_stats = model_stats,
      show_ref = show_ref,
      fmt_class = fmt_class
    ))
  }

  core <- .run_surv_core(
    data = data,
    time = time,
    event = event,
    exposures = exposures,
    adjust_for = adjust_for,
    interaction = interaction,
    multivariable = multivariable,
    distribution = distribution
  )

  effect_label <- if (adjusted_mode || isTRUE(multivariable)) {
    "Adjusted Time Ratio (95% CI)"
  } else {
    "Time Ratio (95% CI)"
  }

  variable_labels <- .var_label_map(
    data,
    unique(c(exposures, adjust_for, .interaction_vars(interaction)))
  )
  core$table_body <- .format_interaction_levels(
    core$table_body, core$data_clean, variable_labels
  )
  crude_mode <- !adjusted_mode && !isTRUE(multivariable)

  if (crude_mode) {
    display_df <- .make_display_survival_uni(
      core$table_body,
      core$data_clean,
      time = time,
      event = event,
      effect_label = effect_label,
      variable_labels = variable_labels,
      show_ref = show_ref
    )
    .must_be_display_df(display_df)
  } else {
    display_df <- .make_display_multi(
      core$table_body,
      core$data_clean,
      outcome = event,
      effect_label = effect_label,
      variable_labels = variable_labels,
      show_ref = show_ref
    )
    .must_be_display_df_multi(display_df)
  }

  footnotes <- c(
    .abbrev_note("survreg"),
    paste0("Distribution: ", distribution, "."),
    if (isTRUE(show_ref) && any(core$table_body$ref %in% TRUE)) .ref_note() else NULL,
    if (adjusted_mode) .display_adjustment_note(adjust_for, variable_labels) else NULL,
    if (isTRUE(multivariable)) "Adjusted for the other variables in the model." else NULL,
    if (!is.null(interaction)) .interaction_note(interaction) else NULL,
    NULL
  )

  .message_hidden_ref_rows("surv_reg", core$table_body, show_ref)

  tbl <- if (format == "gt") {
    if (crude_mode) {
      .build_gt(display_df, effect_label, footnotes, theme)
    } else {
      .build_gt_multi(display_df, effect_label, footnotes, theme)
    }
  } else {
    if (crude_mode) {
      .build_flextable(display_df, effect_label, footnotes, theme)
    } else {
      .build_flextable_multi(display_df, effect_label, footnotes, theme)
    }
  }

  res <- list(
    table = tbl,
    table_body = core$table_body,
    table_display = display_df,
    models = core$models,
    model_summaries = core$model_summaries,
    model_stats = if (isTRUE(model_stats)) .surv_model_stats_table(core$models) else NULL,
    variable_labels = variable_labels,
    footnotes = footnotes,
    time = time,
    event = event,
    distribution = distribution,
    approach = "survreg",
    format = format,
    source = "surv_reg",
    adjusted_mode = adjusted_mode,
    multivariable = isTRUE(multivariable),
    adjust_for = if (adjusted_mode) unique(adjust_for) else NULL,
    exposures = unique(exposures),
    interaction = interaction,
    show_ref = isTRUE(show_ref)
  )

  class(res) <- c("gtregression", "surv_reg", fmt_class, class(res))
  res
}

#' @keywords internal
#' @noRd
.run_surv_core <- function(data,
                           time,
                           event,
                           exposures,
                           adjust_for = NULL,
                           interaction = NULL,
                           multivariable = FALSE,
                           distribution = "weibull") {
  data_valid <- .validate_cox_inputs(
    data,
    time,
    event,
    exposures,
    adjust_for,
    interaction = interaction,
    multivariable = multivariable
  )
  adjusted_mode <- !is.null(adjust_for) && length(adjust_for) > 0
  interaction_vars <- .interaction_vars(interaction)

  if (isTRUE(multivariable)) {
    predictors <- unique(c(exposures, interaction_vars))
    data_model <- .surv_model_data(data_valid, time, event, predictors)
    fit <- .fit_surv_model(data_model, time, event, predictors, distribution, interaction = interaction)

    if (is.null(fit)) {
      stop("Parametric survival model fitting failed for the multivariable model.", call. = FALSE)
    }

    td_list <- lapply(unique(exposures), function(exposure) .tidy_survreg(fit, exposure, interaction = interaction))
    td_list <- Filter(Negate(is.null), td_list)
    if (!length(td_list)) {
      stop("No estimable survival coefficients for the multivariable model.", call. = FALSE)
    }

    fits <- list(multivariable_model = fit)
    return(list(
      data_clean = data_valid,
      table_body = do.call(rbind, td_list),
      models = fits,
      model_summaries = lapply(fits, summary)
    ))
  }

  fits <- vector("list", length(exposures))
  names(fits) <- exposures
  tds <- vector("list", length(exposures))
  names(tds) <- exposures

  for (i in seq_along(exposures)) {
    exposure <- exposures[i]
    predictors <- unique(c(if (adjusted_mode) c(exposure, adjust_for) else exposure, interaction_vars))
    data_model <- .surv_model_data(data_valid, time, event, predictors)
    fit <- .fit_surv_model(data_model, time, event, predictors, distribution, interaction = interaction)

    if (is.null(fit)) {
      stop("Parametric survival model fitting failed for exposure '", exposure, "'.", call. = FALSE)
    }

    td <- .tidy_survreg(fit, exposure, interaction = interaction)
    if (is.null(td) || !nrow(td)) {
      stop("No estimable survival coefficients for exposure '", exposure, "'.", call. = FALSE)
    }

    fits[[i]] <- fit
    tds[[i]] <- td
  }

  list(
    data_clean = data_valid,
    table_body = do.call(rbind, tds),
    models = fits,
    model_summaries = lapply(fits, summary)
  )
}

#' @keywords internal
#' @noRd
.surv_model_data <- function(data, time, event, predictors) {
  vars_needed <- unique(c(time, event, predictors))
  cc_idx <- stats::complete.cases(data[, vars_needed, drop = FALSE])
  data_model <- data[cc_idx, , drop = FALSE]

  if (nrow(data_model) == 0) {
    stop("No complete cases available for this parametric survival model.", call. = FALSE)
  }

  if (sum(data_model[[event]] == 1, na.rm = TRUE) == 0) {
    stop("`event` must include at least one event for this parametric survival model.", call. = FALSE)
  }
  if (sum(data_model[[event]] == 0, na.rm = TRUE) == 0) {
    stop("`event` must include at least one censored observation for this parametric survival model.", call. = FALSE)
  }

  .validate_exposures(data_model, predictors)
  data_model
}

#' @keywords internal
#' @noRd
.fit_surv_model <- function(data, time, event, predictors, distribution, interaction = NULL) {
  bt <- .surv_bt
  rhs <- .survival_rhs(predictors, interaction)
  fml <- stats::as.formula(paste0("survival::Surv(", bt(time), ", ", bt(event), ") ~ ", rhs))

  tryCatch(
    {
      fit <- survival::survreg(fml, data = data, dist = distribution, model = TRUE)
      attr(fit, "gtregression_events") <- sum(data[[event]] == 1, na.rm = TRUE)
      attr(fit, "gtregression_distribution") <- distribution
      fit
    },
    error = function(e) {
      warning("Parametric survival model failed: ", e$message, call. = FALSE)
      NULL
    }
  )
}

#' @keywords internal
#' @noRd
.tidy_survreg <- function(fit, exposure, interaction = NULL) {
  smry <- summary(fit)
  coefs <- smry$table
  if (is.null(coefs) || nrow(coefs) == 0) {
    return(NULL)
  }

  rn <- rownames(coefs)
  rn_clean <- gsub("`", "", rn, fixed = TRUE)
  exposure_clean <- gsub("`", "", exposure, fixed = TRUE)

  keep <- rn_clean != "(Intercept)" &
    rn_clean != "Log(scale)" &
    (rn_clean == exposure_clean |
       (startsWith(rn_clean, exposure_clean) & !grepl(":", rn_clean, fixed = TRUE)) |
       (!is.null(interaction) &
          grepl(":", rn_clean, fixed = TRUE) &
          vapply(strsplit(rn_clean, ":", fixed = TRUE),
                 function(parts) any(startsWith(parts, exposure_clean) | parts == exposure_clean),
                 logical(1))))
  idx <- which(keep)

  df_nonref <- NULL
  if (length(idx)) {
    est_log <- coefs[idx, "Value"]
    se <- coefs[idx, "Std. Error"]
    p_col <- grep("^p$|^Pr\\(", colnames(coefs), value = TRUE)
    p <- if (length(p_col)) coefs[idx, p_col[1]] else NA_real_
    z <- stats::qnorm(0.975)

    levels_out <- vapply(rn_clean[idx], function(term) {
      if (grepl(":", term, fixed = TRUE)) {
        return(paste(strsplit(term, ":", fixed = TRUE)[[1]], collapse = " x "))
      }
      lvl <- sub(paste0("^", exposure_clean), "", term)
      if (identical(lvl, "")) exposure else lvl
    }, character(1))

    df_nonref <- data.frame(
      exposure = exposure,
      level = levels_out,
      estimate = exp(est_log),
      conf.low = exp(est_log - z * se),
      conf.high = exp(est_log + z * se),
      p.value = p,
      ref = FALSE,
      stringsAsFactors = FALSE
    )
  }

  ref_row <- NULL
  exposure_data <- .model_data_col(fit, exposure)
  if (!is.null(exposure_data) && is.factor(exposure_data)) {
    levs <- levels(exposure_data)
    ref_row <- data.frame(
      exposure = exposure,
      level = levs[1],
      estimate = 1,
      conf.low = NA_real_,
      conf.high = NA_real_,
      p.value = NA_real_,
      ref = TRUE,
      stringsAsFactors = FALSE
    )

    if (!is.null(df_nonref)) {
      df_nonref$..ord <- match(df_nonref$level, levs)
      df_nonref <- df_nonref[order(df_nonref$..ord), , drop = FALSE]
      df_nonref$..ord <- NULL
    }
  }

  if (is.null(ref_row) && is.null(df_nonref)) {
    return(NULL)
  }
  if (!is.null(ref_row) && !is.null(df_nonref)) {
    return(rbind(ref_row, df_nonref))
  }
  if (!is.null(ref_row)) ref_row else df_nonref
}

#' @keywords internal
#' @noRd
.surv_model_stats_table <- function(models) {
  out <- lapply(names(models), function(model_name) {
    fit <- models[[model_name]]
    data.frame(
      model = model_name,
      distribution = attr(fit, "gtregression_distribution", exact = TRUE),
      AIC = .safe_numeric(stats::AIC(fit)),
      BIC = .safe_numeric(stats::BIC(fit)),
      logLik = .safe_numeric(as.numeric(stats::logLik(fit))),
      scale = .safe_numeric(fit$scale),
      events = .safe_numeric(attr(fit, "gtregression_events", exact = TRUE)),
      n = .safe_numeric(stats::nobs(fit)),
      stringsAsFactors = FALSE
    )
  })

  out <- Filter(Negate(is.null), out)
  if (!length(out)) {
    return(NULL)
  }
  do.call(rbind, out)
}

#' @keywords internal
#' @noRd
.compact_gt_source_notes <- function(tb) {
  gt::tab_options(tb, source_notes.padding = gt::px(2))
}

#' @keywords internal
#' @noRd
.apply_theme_gt <- function(tb, df, theme) {
  if ("header_shaded" %in% theme) {
    tb <- gt::tab_options(tb, column_labels.background.color = "#f6f8fa")
  }
  if ("zebra" %in% theme) tb <- gt::opt_row_striping(tb)
  if ("lines" %in% theme) {
    tb <- gt::tab_style(
      tb,
      style = list(gt::cell_borders(sides = "top", color = "#DADADA", weight = gt::px(1))),
      locations = gt::cells_body(rows = TRUE)
    )
    tb <- gt::tab_options(tb,
                          table.border.top.style = "solid",   table.border.top.color = "#DADADA",
                          table.border.bottom.style = "solid",table.border.bottom.color = "#DADADA")
  }
  # bold exposure labels by default
  hdr_idx <- which(df$is_header %in% TRUE)
  if (length(hdr_idx)) {
    tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                        gt::cells_body(rows = hdr_idx, columns = "Characteristic"))
  }
  if ("compact" %in% theme) tb <- gt::tab_options(tb, data_row.padding = gt::px(2))
  tb
}
#' @keywords internal
#' @noRd
.build_gt <- function(df, effect_label, source_note, theme) {
  if (!requireNamespace("gt", quietly = TRUE))
    stop("Install 'gt' or use format='flextable'.", call. = FALSE)
  .must_be_display_df(df)

  df$N <- ifelse(df$is_header, df$N, NA_integer_)
  out <- df[, c("Characteristic", "N", effect_label, "p-value"), drop = FALSE]

  tb <- gt::gt(out) |>
    gt::cols_align("left",   columns = "Characteristic") |>
    gt::cols_align("center", columns = c("N", effect_label, "p-value")) |>
    gt::sub_missing(columns = "N", missing_text = "") |>
    gt::tab_options(
      table.font.names         = "system-ui",
      data_row.padding         = gt::px(4),
      heading.background.color = "white",
      table.background.color   = "white"
    )

  # indent levels
  tb <- gt::tab_style(tb, gt::cell_text(indent = gt::px(12)),
                      gt::cells_body(rows = df$is_header %in% FALSE, columns = "Characteristic"))
  tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                      locations = gt::cells_column_labels())
  # try to bold spanners if present
  tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                  locations = gt::cells_column_spanners())

  tb <- .apply_theme_gt(tb, df, theme)
  tb <- gt::tab_source_note(tb, source_note = source_note)
  .compact_gt_source_notes(tb)
}
#' @keywords internal
#' @noRd
.build_gt_multi <- function(df, effect_label, footnotes, theme) {
  if (!requireNamespace("gt", quietly = TRUE))
    stop("Install 'gt' or use format='flextable'.", call. = FALSE)

  out <- df[, c("Characteristic", effect_label, "p-value"), drop = FALSE]

  tb <- gt::gt(out) |>
    gt::cols_align("left",   columns = "Characteristic") |>
    gt::cols_align("center", columns = c(effect_label, "p-value")) |>
    gt::tab_options(
      table.font.names         = "system-ui",
      data_row.padding         = gt::px(4),
      heading.background.color = "white",
      table.background.color   = "white"
    )

  tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                      gt::cells_body(rows = df$is_header %in% TRUE, columns = "Characteristic"))
  tb <- gt::tab_style(tb, gt::cell_text(indent = gt::px(12)),
                      gt::cells_body(rows = df$is_header %in% FALSE, columns = "Characteristic"))
  tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                      locations = gt::cells_column_labels())
  # try to bold spanners if present
  tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                      locations = gt::cells_column_spanners())

  tb <- .apply_theme_gt(tb, df, theme)

  # two separate lines in the footer
  tb <- gt::tab_source_note(tb, source_note = footnotes)
  .compact_gt_source_notes(tb)
}
#' GT builder for stratified wide tables (3 cols per stratum)
#' @keywords internal
.build_gt_strata_wide_uni <- function(df, spanners, effect_label, theme, footnotes) {
  if (!requireNamespace("gt", quietly = TRUE))
    stop("Install 'gt' or use format='flextable'.", call. = FALSE)

  spec <- .strata_display_spec(df, spanners, effect_label)
  out <- df[, c("Characteristic", spec$ids), drop = FALSE]

  tb <- gt::gt(out) |>
    gt::cols_label(.list = setNames(spec$labels, names(out))) |>
    gt::cols_align("left",   columns = "Characteristic") |>
    gt::cols_align("center", columns = setdiff(names(out), "Characteristic")) |>
    gt::tab_options(
      table.font.names         = "system-ui",
      data_row.padding         = gt::px(4),
      heading.background.color = "white",
      table.background.color   = "white"
    )

  # bold column labels
  tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                      locations = gt::cells_column_labels())

  start <- 2L
  for (i in seq_along(spanners)) {
    cols <- names(out)[start:(start + spec$widths[i] - 1L)]
    tb <- gt::tab_spanner(tb, label = spanners[i], columns = cols)
    start <- start + spec$widths[i]
  }
  # bold spanners after creating them
  tb <- tryCatch(
    gt::tab_style(tb, gt::cell_text(weight = "bold"),
                  locations = gt::cells_column_spanners(spanners = spanners)),
    error = function(e) gt::tab_style(tb, gt::cell_text(weight = "bold"),
                                      locations = gt::cells_column_spanners())
  )

  # exposure headers bold; levels indented
  tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                      gt::cells_body(rows = df$is_header %in% TRUE, columns = "Characteristic"))
  tb <- gt::tab_style(tb, gt::cell_text(indent = gt::px(12)),
                      gt::cells_body(rows = df$is_header %in% FALSE, columns = "Characteristic"))

  # themes
  if ("header_shaded" %in% theme) tb <- gt::tab_options(tb, column_labels.background.color = "#f6f8fa")
  if ("zebra" %in% theme) tb <- gt::opt_row_striping(tb)
  if ("lines" %in% theme) {
    tb <- gt::tab_style(tb, gt::cell_borders(sides = "top", color = "#DADADA", weight = gt::px(1)),
                        gt::cells_body(rows = TRUE))
    tb <- gt::tab_options(tb,
                          table.border.top.style = "solid",   table.border.top.color = "#DADADA",
                          table.border.bottom.style = "solid",table.border.bottom.color = "#DADADA")
  }
  if ("compact" %in% theme) tb <- gt::tab_options(tb, data_row.padding = gt::px(2))

  tb <- gt::tab_source_note(tb, source_note = footnotes)
  .compact_gt_source_notes(tb)
}
#' @keywords internal
.build_gt_strata_wide_multi <- function(df, spanners, effect_label_adj, theme, footnotes) {
  if (!requireNamespace("gt", quietly = TRUE))
    stop("Install 'gt' or use format='flextable'.", call. = FALSE)

  spec <- .strata_display_spec(df, spanners, effect_label_adj)
  out <- df[, c("Characteristic", spec$ids), drop = FALSE]

  tb <- gt::gt(out) |>
    gt::cols_label(.list = setNames(spec$labels, names(out))) |>
    gt::cols_align("left",   columns = "Characteristic") |>
    gt::cols_align("center", columns = setdiff(names(out), "Characteristic")) |>
    gt::tab_options(
      table.font.names         = "system-ui",
      data_row.padding         = gt::px(4),
      heading.background.color = "white",
      table.background.color   = "white"
    )

  tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                      locations = gt::cells_column_labels())

  start <- 2L
  for (i in seq_along(spanners)) {
    cols <- names(out)[start:(start + spec$widths[i] - 1L)]
    tb <- gt::tab_spanner(tb, label = spanners[i], columns = cols)
    start <- start + spec$widths[i]
  }
  tb <- tryCatch(
    gt::tab_style(tb, gt::cell_text(weight = "bold"),
                  locations = gt::cells_column_spanners(spanners = spanners)),
    error = function(e) gt::tab_style(tb, gt::cell_text(weight = "bold"),
                                      locations = gt::cells_column_spanners())
  )

  tb <- gt::tab_style(tb, gt::cell_text(weight = "bold"),
                      gt::cells_body(rows = df$is_header %in% TRUE, columns = "Characteristic"))
  tb <- gt::tab_style(tb, gt::cell_text(indent = gt::px(12)),
                      gt::cells_body(rows = df$is_header %in% FALSE, columns = "Characteristic"))

  if ("header_shaded" %in% theme) tb <- gt::tab_options(tb, column_labels.background.color = "#f6f8fa")
  if ("zebra" %in% theme) tb <- gt::opt_row_striping(tb)
  if ("lines" %in% theme) {
    tb <- gt::tab_style(tb, gt::cell_borders(sides = "top", color = "#DADADA", weight = gt::px(1)),
                        gt::cells_body(rows = TRUE))
    tb <- gt::tab_options(tb,
                          table.border.top.style = "solid",   table.border.top.color = "#DADADA",
                          table.border.bottom.style = "solid",table.border.bottom.color = "#DADADA")
  }
  if ("compact" %in% theme) tb <- gt::tab_options(tb, data_row.padding = gt::px(2))

  tb <- gt::tab_source_note(tb, source_note = footnotes)
  .compact_gt_source_notes(tb)
}

#' @keywords internal
#' @noRd
.abbrev_note <- function(approach) {
  approach <- .normalize_approach(approach)
  note <- switch(
    approach,
    "linear"       = "Beta = Linear regression coefficient; CI = Confidence Interval.",
    "logit"        = "OR = Odds Ratio; CI = Confidence Interval.",
    "firth"        = "OR = Odds Ratio from Firth penalized logistic regression; CI = Confidence Interval.",
    "logbinomial" = "RR = Risk Ratio; CI = Confidence Interval.",
    "poisson"      = "IRR = Incidence Rate Ratio; CI = Confidence Interval.",
    "robpoisson"   = "RR = Relative Risk; CI = Confidence Interval.",
    "negbin"       = "IRR = Incidence Rate Ratio; CI = Confidence Interval.",
    "cox"          = "HR = Hazard Ratio; CI = Confidence Interval.",
    "survreg"      = "Time Ratio = exponentiated accelerated failure time coefficient; CI = Confidence Interval.",
    "CI = Confidence Interval."
  )
  paste0("Abbreviations: ", note)
}
#' @keywords internal
#' @noRd
.is_ratio <- function(a) {
  .normalize_approach(a) %in% c(
    "logit", "firth", "logbinomial", "poisson", "robpoisson", "negbin", "cox", "survreg"
  )
}
#' @keywords internal
#' @noRd
.get_effect_label <- function(a) switch(.normalize_approach(a),
                                        "linear"       = "Beta (95% CI)",
                                        "logit"        = "OR (95% CI)",
                                        "firth"        = "OR (95% CI)",
                                        "logbinomial" = "RR (95% CI)",
                                        "poisson"      = "IRR (95% CI)",
                                        "robpoisson"   = "RR (95% CI)",
                                        "negbin"       = "IRR (95% CI)",
                                        "cox"          = "HR (95% CI)",
                                        "survreg"      = "Time Ratio (95% CI)"
)
#' @keywords internal
#' @noRd
.fmt_p <- function(p) ifelse(is.na(p), "", ifelse(p < 0.001, "<0.001", formatC(p, format="f", digits=3)))

test_that(".get_effect_label returns correct labels", {
  expect_equal(.get_effect_label("logit"), "OR (95% CI)")
  expect_equal(.get_effect_label("poisson"), "IRR (95% CI)")
  expect_equal(.get_effect_label("linear"), "Beta (95% CI)")
  expect_equal(.get_effect_label("firth"), "OR (95% CI)")
  expect_equal(.get_effect_label("logbinomial"), "RR (95% CI)")
  expect_equal(.get_effect_label("robpoisson"), "RR (95% CI)")
  expect_equal(.get_effect_label("negbin"), "IRR (95% CI)")
})

test_that(".abbrev_note returns expected descriptions", {
  expect_equal(.abbrev_note("logit"), "Abbreviations: OR = Odds Ratio; CI = Confidence Interval.")
  expect_equal(
    .abbrev_note("firth"),
    "Abbreviations: OR = Odds Ratio from Firth penalized logistic regression; CI = Confidence Interval."
  )
  expect_equal(.abbrev_note("logbinomial"), "Abbreviations: RR = Risk Ratio; CI = Confidence Interval.")
  expect_equal(.abbrev_note("poisson"), "Abbreviations: IRR = Incidence Rate Ratio; CI = Confidence Interval.")
  expect_equal(.abbrev_note("linear"), "Abbreviations: Beta = Linear regression coefficient; CI = Confidence Interval.")
  expect_equal(.abbrev_note("unknown"), "Abbreviations: CI = Confidence Interval.")
})

#' Get ordered levels for a stratifier
#' @keywords internal
.strata_levels <- function(data, stratifier) {
  v <- data[[stratifier]]
  if (is.factor(v)) {
    levels(stats::na.omit(v))
  } else {
    unique(stats::na.omit(v))
  }
}
#' Build canonical row skeleton (exposure headers + factor levels)
#' @keywords internal
#' @noRd
.strata_build_skeleton <- function(data, exposures, variable_labels = NULL, show_ref = TRUE) {
  is_factor <- vapply(exposures, function(x) is.factor(data[[x]]), logical(1))
  label_map <- variable_labels
  if (is.null(label_map)) {
    label_map <- .var_label_map(data, exposures)
  }

  levels_map <- lapply(
    exposures,
    function(x) if (is.factor(data[[x]])) levels(data[[x]]) else NULL
  )
  names(levels_map) <- exposures

  rows <- list()

  for (x in exposures) {
    rows[[length(rows) + 1]] <- data.frame(
      exposure = x,
      level = NA_character_,
      Characteristic = .label_var(x, label_map),
      is_header = TRUE,
      stringsAsFactors = FALSE
    )

    if (is_factor[[x]]) {
      levels_to_show <- levels_map[[x]]
      if (!isTRUE(show_ref) && length(levels_to_show) > 0L) {
        levels_to_show <- levels_to_show[-1]
      }

      for (lv in levels_to_show) {
        rows[[length(rows) + 1]] <- data.frame(
          exposure = x,
          level = lv,
          Characteristic = paste0("  ", lv),
          is_header = FALSE,
          stringsAsFactors = FALSE
        )
      }
    }
  }

  skeleton <- do.call(rbind, rows)
  skeleton <- .attach_display_metadata(
    skeleton,
    row_exposure = skeleton$exposure,
    variable_labels = label_map
  )

  list(
    skeleton = skeleton,
    is_factor = is_factor
  )
}
#' Build canonical row skeleton for stratified multivariable tables
#'
#' Uses the union of rows observed across per-stratum multivariable outputs,
#' preserving exposure headers, factor levels, and interaction rows.
#'
#' @keywords internal
#' @noRd
.strata_build_skeleton_multi <- function(exposures,
                                         td_by_stratum,
                                         variable_labels = NULL,
                                         show_ref = TRUE) {
  rows <- list()

  for (x in exposures) {
    rows[[length(rows) + 1]] <- data.frame(
      exposure = x,
      level = NA_character_,
      Characteristic = .label_var(x, variable_labels),
      is_header = TRUE,
      stringsAsFactors = FALSE
    )

    td_exp_list <- lapply(td_by_stratum, function(td) {
      td[td$exposure == x, , drop = FALSE]
    })
    td_exp_list <- Filter(function(z) nrow(z) > 0, td_exp_list)

    if (!length(td_exp_list)) {
      next
    }

    td_exp <- do.call(rbind, td_exp_list)

    # factor reference/main rows first, then non-reference rows
    seen_levels <- character(0)

    # factor levels if present
    if (any(td_exp$ref, na.rm = TRUE)) {
      factor_levels <- unique(td_exp$level)
      factor_levels <- factor_levels[!is.na(factor_levels)]
      factor_levels <- factor_levels[factor_levels != x]
      if (!isTRUE(show_ref)) {
        ref_levels <- unique(td_exp$level[td_exp$ref %in% TRUE])
        factor_levels <- setdiff(factor_levels, ref_levels)
      }

      for (lv in factor_levels) {
        if (!(lv %in% seen_levels)) {
          rows[[length(rows) + 1]] <- data.frame(
            exposure = x,
            level = lv,
            Characteristic = paste0("  ", lv),
            is_header = FALSE,
            stringsAsFactors = FALSE
          )
          seen_levels <- c(seen_levels, lv)
        }
      }
    } else {
      # continuous exposure: include non-main rows such as interactions
      extra_levels <- unique(td_exp$level)
      extra_levels <- extra_levels[!is.na(extra_levels)]
      extra_levels <- extra_levels[extra_levels != x]

      for (lv in extra_levels) {
        if (!(lv %in% seen_levels)) {
          rows[[length(rows) + 1]] <- data.frame(
            exposure = x,
            level = lv,
            Characteristic = paste0("  ", lv),
            is_header = FALSE,
            stringsAsFactors = FALSE
          )
          seen_levels <- c(seen_levels, lv)
        }
      }
    }
  }

  skeleton <- do.call(rbind, rows)
  .attach_display_metadata(
    skeleton,
    row_exposure = skeleton$exposure,
    variable_labels = variable_labels
  )
}
# ----- UNIVARIATE: pull N/effect/p per stratum aligned to skeleton -----------

#' Pull (N, Effect, p) for a univariate stratum, aligned to a skeleton
#' @keywords internal
.strata_pull_cols_uni <- function(dlev, uni_res, skeleton, is_factor, exposures, outcome) {
  td <- uni_res$table_body

  # pairwise complete N for (outcome + exposure) within this stratum
  N_map <- vapply(exposures, function(x) {
    sum(stats::complete.cases(dlev[, c(outcome, x), drop = FALSE]))
  }, integer(1))
  names(N_map) <- exposures

  fmt_est_ci <- function(est, lo, hi, digits = 2) {
    f <- function(z) formatC(z, digits = digits, format = "f", big.mark = ",")
    paste0(f(est), " (", f(lo), "\u2013", f(hi), ")")
  }

  N_vec   <- character(nrow(skeleton))
  eff_vec <- character(nrow(skeleton))
  p_vec   <- character(nrow(skeleton))

  for (i in seq_len(nrow(skeleton))) {
    ex  <- skeleton$exposure[i]
    lv  <- skeleton$level[i]
    hdr <- skeleton$is_header[i]

    if (hdr) {
      N_vec[i] <- as.character(N_map[[ex]])
      if (!is_factor[[ex]]) {
        row <- td[td$exposure == ex & td$level == ex, , drop = FALSE]
        if (nrow(row)) {
          eff_vec[i] <- fmt_est_ci(row$estimate, row$conf.low, row$conf.high)
          p_vec[i]   <- .fmt_p(row$p.value)
        }
      }
    } else {
      row <- td[td$exposure == ex & td$level == lv, , drop = FALSE]
      if (nrow(row)) {
        if (isTRUE(row$ref[1])) {
          eff_vec[i] <- "Ref."; p_vec[i] <- ""
        } else {
          eff_vec[i] <- fmt_est_ci(row$estimate, row$conf.low, row$conf.high)
          p_vec[i]   <- .fmt_p(row$p.value)
        }
      }
    }
  }

  list(N = N_vec, effect = eff_vec, pval = p_vec)
}

#' Assemble wide display DF for all strata (univariate)
#' @keywords internal
.strata_build_wide_uni <- function(data,
                                   outcome,
                                   exposures,
                                   stratifier,
                                   per_stratum,
                                   variable_labels = NULL,
                                   show_ref = TRUE) {
  sk <- .strata_build_skeleton(
    data,
    exposures,
    variable_labels = variable_labels,
    show_ref = show_ref
  )
  skeleton  <- sk$skeleton
  is_factor <- sk$is_factor

  wide <- skeleton[, c("Characteristic","is_header"), drop = FALSE]
  wide <- .attach_display_metadata(
    wide,
    row_exposure = skeleton$exposure,
    variable_labels = attr(skeleton, "variable_labels", exact = TRUE)
  )

  for (lev in names(per_stratum)) {
    dlev <- data[data[[stratifier]] == lev, , drop = FALSE]
    cols <- .strata_pull_cols_uni(dlev, per_stratum[[lev]], skeleton, is_factor,
                                  exposures, outcome)
    wide[[paste0("..N__",   lev)]] <- cols$N
    wide[[paste0("..eff__", lev)]] <- cols$effect
    wide[[paste0("..p__",   lev)]] <- cols$pval
  }

  list(wide = wide, spanners = names(per_stratum))
}

# ----- MULTIVARIABLE: pull Adjusted effect/p per stratum ---------------------
#' Pull (Adjusted Effect, p) for a multivariable stratum, aligned to skeleton
#' @keywords internal
#' @noRd
.strata_pull_cols_multi <- function(td, skeleton) {
  fmt_est_ci <- function(est, lo, hi, digits = 2) {
    f <- function(z) formatC(z, digits = digits, format = "f", big.mark = ",")
    paste0(f(est), " (", f(lo), "\u2013", f(hi), ")")
  }

  eff <- character(nrow(skeleton))
  pv  <- character(nrow(skeleton))

  for (i in seq_len(nrow(skeleton))) {
    ex  <- skeleton$exposure[i]
    lv  <- skeleton$level[i]
    hdr <- skeleton$is_header[i]

    if (hdr) {
      row <- td[td$exposure == ex & td$level == ex, , drop = FALSE]

      if (nrow(row)) {
        eff[i] <- fmt_est_ci(row$estimate[1], row$conf.low[1], row$conf.high[1])
        pv[i]  <- .fmt_p(row$p.value[1])
      } else {
        # factor headers or missing main row
        eff[i] <- ""
        pv[i]  <- ""
      }
    } else {
      row <- td[td$exposure == ex & td$level == lv, , drop = FALSE]

      if (nrow(row)) {
        if (isTRUE(row$ref[1])) {
          eff[i] <- "Ref."
          pv[i]  <- ""
        } else {
          eff[i] <- fmt_est_ci(row$estimate[1], row$conf.low[1], row$conf.high[1])
          pv[i]  <- .fmt_p(row$p.value[1])
        }
      }
    }
  }

  list(effect = eff, pval = pv)
}
#' Assemble wide display DF for all strata (multivariable)
#' @keywords internal
#' @noRd
.strata_build_wide_multi <- function(data,
                                     exposures,
                                     stratifier,
                                     td_by_stratum,
                                     variable_labels = NULL,
                                     show_ref = TRUE) {
  if (is.null(variable_labels)) {
    variable_labels <- .var_label_map(data, exposures)
  }
  skeleton <- .strata_build_skeleton_multi(
    exposures = exposures,
    td_by_stratum = td_by_stratum,
    variable_labels = variable_labels,
    show_ref = show_ref
  )

  wide <- skeleton[, c("Characteristic", "is_header"), drop = FALSE]
  wide <- .attach_display_metadata(
    wide,
    row_exposure = skeleton$exposure,
    variable_labels = variable_labels
  )
  strata_names <- names(td_by_stratum)

  for (lev in strata_names) {
    td <- td_by_stratum[[lev]]
    cols <- .strata_pull_cols_multi(td, skeleton)
    wide[[paste0("..eff__", lev)]] <- cols$effect
    wide[[paste0("..p__", lev)]] <- cols$pval
  }

  list(
    wide = wide,
    spanners = strata_names
  )
}

#' Add model N columns to a stratified multivariable display table
#' @keywords internal
#' @noRd
.strata_add_model_n <- function(wide, models_by_stratum) {
  if (!length(models_by_stratum) || !"is_header" %in% names(wide)) {
    return(wide)
  }

  row_exposure <- attr(wide, "row_exposure", exact = TRUE)
  if (is.null(row_exposure)) {
    return(wide)
  }

  is_header <- wide$is_header %in% TRUE

  for (lev in names(models_by_stratum)) {
    models <- models_by_stratum[[lev]]
    if (!length(models)) {
      next
    }

    n_vec <- character(nrow(wide))
    model_names <- names(models)
    has_single_model <- identical(model_names, "multivariable_model")

    for (i in seq_len(nrow(wide))) {
      if (!isTRUE(is_header[i])) {
        next
      }

      fit <- if (has_single_model) {
        models[["multivariable_model"]]
      } else {
        models[[row_exposure[i]]]
      }

      if (!is.null(fit)) {
        n_vec[i] <- tryCatch(
          as.character(as.integer(stats::nobs(fit))),
          error = function(e) ""
        )
      }
    }

    wide[[paste0("..N__", lev)]] <- n_vec
  }

  wide
}

#' Describe visible columns for stratified table builders
#' @keywords internal
#' @noRd
.strata_display_spec <- function(df, spanners, effect_label) {
  block_ids <- character(0)
  block_labels <- character(0)
  widths <- integer(length(spanners))

  for (i in seq_along(spanners)) {
    nm <- sub("^.*=\\s*", "", spanners[i])
    ids <- c(
      paste0("..N__", nm),
      paste0("..Events__", nm),
      paste0("..eff__", nm),
      paste0("..p__", nm)
    )
    labels <- c("N", "Events", effect_label, "p-value")
    keep <- ids %in% names(df)

    ids <- ids[keep]
    labels <- labels[keep]
    widths[i] <- length(ids)

    block_ids <- c(block_ids, ids)
    block_labels <- c(block_labels, labels)
  }

  list(
    ids = block_ids,
    labels = c("Characteristic", block_labels),
    widths = widths
  )
}

#' Multivariable regression
#'
#' Fit adjusted or multivariable regression models and return a manuscript-style
#' table in \pkg{flextable} or \pkg{gt} format.
#'
#' @param data A \code{data.frame} containing the variables of interest.
#' @param outcome Character scalar; name of the outcome variable. Quoted and
#'   bare names are accepted.
#' @param exposures Character vector; exposure variable(s) to report.
#'   Quoted names are recommended in scripts, and bare names are also accepted.
#'   If \code{adjust_for = NULL}, all exposures are included in a single
#'   multivariable model. If \code{adjust_for} is supplied, one adjusted
#'   model is fitted per exposure and only exposure-specific adjusted
#'   estimate(s) are displayed.
#' @param adjust_for Optional character vector of adjustment variables. Quoted
#'   and bare names are accepted. Must not overlap with \code{exposures}.
#' @param interaction Optional character scalar specifying one interaction term
#'   using standard formula syntax, e.g. \code{"bmi*sex"}.
#'   When used with \code{adjust_for}, only a single exposure should be supplied.
#' @param approach Character scalar specifying the regression approach.
#'   One of \code{"logit"}, \code{"firth"}, \code{"logbinomial"}, \code{"poisson"},
#'   \code{"linear"}, \code{"robpoisson"}, or \code{"negbin"}.
#'   Use \code{"firth"} for Firth penalized logistic regression, especially
#'   with sparse cells or separation.
#' @param format Output table format; one of \code{"flextable"} (default) or
#'   \code{"gt"}.
#' @param theme Table styling preset (e.g. \code{"minimal"}, \code{"striped"},
#'   \code{"clinical"}, \code{"shaded"}, \code{"jama"}) or a character vector of
#'   primitives such as \code{c("plain","zebra","lines","labels_bold","compact","header_shaded")}.
#' @param model_stats Logical; if \code{TRUE}, extract model-fit statistics
#'   such as AIC, BIC, log-likelihood, deviance, pseudo R-squared for
#'   non-linear models, and R-squared for linear models. Statistics are stored
#'   in the returned object's \code{model_stats} element and are not added to
#'   the publication table.
#' @param show_ref Logical; if \code{TRUE} (default), display reference-category
#'   rows as \code{"Ref."}. If \code{FALSE}, hide reference rows; a message
#'   reminds users to use \code{show_ref = TRUE} when reference rows are needed.
#'
#' @details
#' Use the default mode when you want all exposures in one model. Use
#' \code{adjust_for} when you want one adjusted estimate per reported exposure,
#' with the same adjustment set used repeatedly.
#'
#' If exposure variables have a \code{"label"} attribute, for example from
#' \code{labelled::var_label()}, those labels are used automatically in the
#' displayed table and plots. Internal matching still uses the original column
#' names.
#'
#' @return A list of class \code{c("gtregression","multi_reg", ...)} with elements:
#' \describe{
#'   \item{table}{A \code{flextable} (if \code{format="flextable"}) or
#'   \code{gt_tbl} (if \code{format="gt"}).}
#'   \item{table_body}{A data frame of adjusted estimates and confidence intervals
#'   for each exposure and level.}
#'   \item{table_display}{A data frame used for rendering the final table,
#'   including header and level rows.}
#'   \item{models}{A list of fitted model(s).}
#'   \item{model_summaries}{\code{summary()} output for the fitted model(s).}
#'   \item{model_stats}{Model-fit statistics when \code{model_stats = TRUE};
#'   otherwise \code{NULL}.}
#'   \item{variable_labels}{Named character vector of display labels used for
#'   exposure variables.}
#'   \item{reg_check}{Regression diagnostics for linear models. Printing this
#'   element renders a publication-ready table; individual diagnostic rows remain
#'   available for programmatic use. Other approaches return a message.}
#'   \item{approach}{The regression approach used.}
#'   \item{format}{The output format used.}
#'   \item{source}{Function identifier (\code{"multi_reg"}).}
#'   \item{adjusted_mode}{Whether one adjusted model per exposure was fitted.}
#'   \item{adjust_for}{Adjustment variables supplied by the user, if any.}
#'   \item{exposures}{Exposure variables supplied by the user.}
#'   \item{interaction}{Interaction term supplied by the user, if any.}
#' }
#'
#' @details
#' In default mode (\code{adjust_for = NULL}), all exposures are included in a
#' single multivariable model. In adjusted mode, one model is fitted per exposure,
#' adjusting for the variables specified in \code{adjust_for}.
#'
#' Interaction terms specified via \code{interaction} are included in the model
#' using standard formula expansion (e.g. \code{bmi*sex}). Interaction effects are
#' displayed as additional rows beneath the corresponding exposure.
#'
#' @examples
#' endometrial_data <- data_endometrial
#' endometrial_data$HG <- factor(endometrial_data$HG, levels = c(0, 1))
#' endometrial_data$NV <- factor(endometrial_data$NV, levels = c(0, 1))
#' multi_reg(endometrial_data, HG, c(NV, PI, EH), approach = firth, format = gt)$table
#'
#' @importFrom stats qnorm residuals nobs
#' @export
multi_reg <- function(data,
                      outcome,
                      exposures,
                      adjust_for = NULL,
                      interaction = NULL,
                      approach = "logit",
                      format = c("flextable", "gt"),
                      theme = c("minimal"),
                      model_stats = FALSE,
                      show_ref = TRUE) {

  outcome <- .vars_arg(substitute(outcome), env = parent.frame(), data_names = names(data))
  exposures <- .vars_arg(substitute(exposures), env = parent.frame(), data_names = names(data))
  adjust_for <- .vars_arg(
    substitute(adjust_for),
    env = parent.frame(),
    allow_null = TRUE,
    data_names = names(data)
  )
  interaction <- .interaction_arg(substitute(interaction), env = parent.frame(), allow_null = TRUE)
  approach <- .choice_arg(
    substitute(approach),
    env = parent.frame(),
    choices = c("logit","firth","logbinomial","poisson","robpoisson","linear","negbin")
  )
  approach <- .normalize_approach(approach)
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("flextable","gt"))
  theme <- .choice_arg(substitute(theme), env = parent.frame())
  if (!is.logical(model_stats) || length(model_stats) != 1L || is.na(model_stats)) {
    stop("`model_stats` must be TRUE or FALSE.", call. = FALSE)
  }
  .validate_show_ref(show_ref)

  format <- match.arg(format, c("flextable","gt"))
  theme <- .resolve_theme(theme)

  fmt_class <- if (format == "gt") "gt_multi" else "ft_multi"
  effect_label_adj <- paste("Adjusted", .get_effect_label(approach))
  variable_labels <- .var_label_map(
    data,
    unique(c(exposures, adjust_for, .interaction_vars(interaction)))
  )

  core <- .run_multi_core(
    data = data,
    outcome = outcome,
    exposures = exposures,
    approach = approach,
    adjust_for = adjust_for,
    interaction = interaction
  )
  core$table_body <- .format_interaction_levels(
    core$table_body, core$data_clean, variable_labels
  )

  footnotes <- if (!core$adjusted_mode) {
    c(
      .abbrev_note(approach),
      if (isTRUE(show_ref) && any(core$table_body$ref %in% TRUE)) .ref_note() else NULL,
      if (!is.null(interaction)) .interaction_note(interaction) else NULL,
      if (!is.na(core$n_used)) {
        paste0(
          "N = ", core$n_used,
          " complete observations included in the model."
        )
      } else {
        "N reflects complete observations included in the model."
      }
    )
  } else {
    n_note <- if (length(unique(stats::na.omit(core$n_used))) == 1L) {
      paste0(
        "N = ", unique(stats::na.omit(core$n_used)),
        " complete observations included in each adjusted model."
      )
    } else {
      paste0(
        "N varies across adjusted models according to complete observations for the outcome, ",
        "current exposure, and adjustment variables."
      )
    }

    c(
      .abbrev_note(approach),
      if (isTRUE(show_ref) && any(core$table_body$ref %in% TRUE)) .ref_note() else NULL,
      .display_adjustment_note(adjust_for, variable_labels),
      if (!is.null(interaction)) .interaction_note(interaction) else NULL,
      n_note
    )
  }

  .message_hidden_ref_rows("multi_reg", core$table_body, show_ref)
  display_df <- .make_display_multi(
    core$table_body,
    core$data_clean,
    outcome,
    effect_label_adj,
    variable_labels = variable_labels,
    show_ref = show_ref
  )
  .must_be_display_df_multi(display_df)

  tbl <- if (format == "gt") {
    .build_gt_multi(display_df, effect_label_adj, footnotes, theme)
  } else {
    .build_flextable_multi(display_df, effect_label_adj, footnotes, theme)
  }

  res <- list(
    table = tbl,
    table_body = core$table_body,
    table_display = display_df,
    models = core$models,
    model_summaries = core$model_summaries,
    model_stats = if (isTRUE(model_stats)) .model_stats_table(core$models, approach) else NULL,
    variable_labels = variable_labels,
    footnotes = footnotes,
    reg_check = .as_reg_check_result(core$reg_check, format = format),
    approach = approach,
    format = format,
    source = "multi_reg",
    adjusted_mode = core$adjusted_mode,
    adjust_for = if (isTRUE(core$adjusted_mode)) unique(adjust_for) else NULL,
    exposures = unique(exposures),
    interaction = interaction,
    show_ref = isTRUE(show_ref)
  )

  class(res) <- c("gtregression", "multi_reg", fmt_class, class(res))
  res
}

birthwt_multi_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
      ptl_cat = ifelse(ptl > 0, "Yes", "No"),
      ftv_cat = dplyr::case_when(
        ftv == 0 ~ "None",
        ftv == 1 ~ "One",
        ftv >= 2 ~ "Two or more"
      )
    ) |>
    dplyr::mutate(
      ptl_cat = factor(ptl_cat, levels = c("No", "Yes")),
      ftv_cat = factor(ftv_cat, levels = c("None", "One", "Two or more"))
    )
}

test_that("multi_reg returns a gtregression object for default logit models", {
  df <- birthwt_multi_data()

  res <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "lwt", "race", "smoke"),
    approach = logit,
    format = gt
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "multi_reg")
  expect_s3_class(res, "gt_multi")
  expect_s3_class(res$table, "gt_tbl")
  expect_equal(res$approach, "logit")
  expect_equal(res$format, "gt")
  expect_equal(res$source, "multi_reg")
  expect_named(
    res,
    c("table", "table_body", "table_display", "models",
      "model_summaries", "model_stats", "variable_labels", "footnotes", "reg_check",
      "approach", "format", "source",
      "adjusted_mode", "adjust_for", "exposures", "interaction", "show_ref")
  )
  expect_null(res$model_stats)
  expect_true(any(grepl("OR = Odds Ratio", res$footnotes, fixed = TRUE)))
  expect_true(any(grepl("^N = .* complete observations", res$footnotes)))
  expect_false(res$adjusted_mode)
  expect_null(res$adjust_for)
  expect_equal(res$exposures, c("age", "lwt", "race", "smoke"))
  expect_null(res$interaction)
  expect_named(res$models, "multivariable_model")
  expect_s3_class(res$models$multivariable_model, "glm")
  expect_s3_class(res$model_summaries$multivariable_model, "summary.glm")
  expect_true(all(c("exposure", "level", "estimate", "conf.low",
                    "conf.high", "p.value", "ref") %in% names(res$table_body)))
  expect_true(all(c("Characteristic", "Adjusted OR (95% CI)", "p-value",
                    "is_header") %in% names(res$table_display)))
  expect_true(any(res$table_body$ref))
  expect_true("Ref." %in% res$table_display[["Adjusted OR (95% CI)"]])
  expect_true(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                        fixed = TRUE)))
  expect_true("Regression diagnostics available only for 'linear' models." %in% res$reg_check)
})

test_that("multi_reg uses the same complete-case note for linear and logistic models", {
  df <- birthwt_multi_data()

  logistic <- multi_reg(
    df, outcome = low, exposures = c(age, lwt), approach = logit
  )
  linear <- multi_reg(
    df, outcome = bwt, exposures = c(age, lwt), approach = linear
  )

  note_pattern <- "^N = [0-9]+ complete observations included in the model\\.$"
  expect_true(any(grepl(note_pattern, logistic$footnotes)))
  expect_true(any(grepl(note_pattern, linear$footnotes)))
})

test_that("multi_reg publication table preserves user exposure order", {
  df <- birthwt_multi_data()
  exposures <- c("smoke", "age", "ht", "race", "lwt")

  res <- multi_reg(
    data = df,
    outcome = "low",
    exposures = exposures,
    approach = logit
  )

  headers <- res$table_display$Characteristic[res$table_display$is_header]
  expect_equal(headers, exposures)
  expect_equal(unique(res$table_body$exposure), exposures)

  adjusted <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("ht", "smoke", "ui"),
    adjust_for = c("age", "lwt"),
    approach = logit
  )
  expect_equal(unique(adjusted$table_body$exposure), c("ht", "smoke", "ui"))
})

test_that("multi_reg explains how to show reference rows when hidden", {
  df <- birthwt_multi_data()

  expect_message(
    res <- multi_reg(
      data = df,
      outcome = low,
      exposures = smoke,
      approach = logit,
      show_ref = FALSE
    ),
    "To display Ref., use `show_ref = TRUE`.",
    fixed = TRUE
  )

  expect_true(any(res$table_body$ref))
  expect_equal(nrow(res$table_display), 1L)
  expect_false("Ref." %in% res$table_display[["Adjusted OR (95% CI)"]])
  expect_false(any(grepl("Ref. = reference category", res$table$`_source_notes`,
                         fixed = TRUE)))
})

test_that("multi_reg optionally returns model-fit statistics", {
  df <- birthwt_multi_data()

  res <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "lwt", "race", "smoke"),
    approach = logit,
    model_stats = TRUE
  )

  expect_s3_class(res$model_stats, "data.frame")
  expect_equal(res$model_stats$model, "multivariable_model")
  expect_true(all(c("AIC", "BIC", "logLik", "deviance", "null_deviance",
                    "pseudo_r2", "r_squared", "adj_r_squared", "n") %in%
                    names(res$model_stats)))
  expect_true(is.finite(res$model_stats$AIC))
  expect_true(is.finite(res$model_stats$BIC))
  expect_true(is.finite(res$model_stats$pseudo_r2))

  adjusted <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("smoke", "ht", "ui"),
    adjust_for = c("age", "lwt", "race"),
    approach = logit,
    model_stats = TRUE
  )
  expect_equal(adjusted$model_stats$model, c("smoke", "ht", "ui"))
  expect_true(all(is.finite(adjusted$model_stats$AIC)))
  expect_error(
    multi_reg(df, outcome = "low", exposures = "age", model_stats = NA),
    "`model_stats` must be TRUE or FALSE"
  )
})

test_that("multi_reg default mode matches the direct combined model", {
  df <- data.frame(
    y = c(0, 1, 0, 1, 0, 1, 1, 0),
    age = c(40, 44, 50, 52, 60, 62, 70, 72),
    marker = c(1, 0, NA, 1, NA, 0, 1, 0)
  )

  res <- multi_reg(
    data = df,
    outcome = y,
    exposures = c(age, marker),
    approach = logit,
    model_stats = TRUE
  )
  direct <- stats::glm(y ~ age + marker, data = df, family = stats::binomial("logit"))

  expect_equal(
    stats::coef(res$models$multivariable_model),
    stats::coef(direct),
    tolerance = 1e-8
  )
  expect_equal(res$model_stats$n, stats::nobs(direct))
})

test_that("multi_reg adjusted mode fits one model per exposure", {
  df <- birthwt_multi_data()

  res <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("smoke", "ht", "ui"),
    adjust_for = c("age", "lwt", "race"),
    approach = "logit",
    theme = clinical
  )

  expect_s3_class(res, "multi_reg")
  expect_named(res$models, c("smoke", "ht", "ui"))
  expect_named(res$model_summaries, c("smoke", "ht", "ui"))
  expect_named(res$reg_check, c("smoke", "ht", "ui"))
  expect_true(res$adjusted_mode)
  expect_equal(res$adjust_for, c("age", "lwt", "race"))
  expect_equal(res$exposures, c("smoke", "ht", "ui"))
  expect_equal(unique(res$table_body$exposure), c("smoke", "ht", "ui"))
  expect_true(all(vapply(res$models, inherits, logical(1), what = "glm")))
})

test_that("multi_reg adjusted mode uses exposure-specific complete cases", {
  df <- data.frame(
    y = c(1.2, 1.5, 1.8, 2.3, 2.8, 3.1, 3.4, 3.9),
    age = c(40, 44, 50, 52, 60, 62, 70, 72),
    marker = c(1, 0, NA, 1, NA, 0, 1, 0),
    adjust = c(2, 1, 2, 3, 3, 4, 5, 4)
  )

  res <- multi_reg(
    data = df,
    outcome = y,
    exposures = c(age, marker),
    adjust_for = adjust,
    approach = linear,
    model_stats = TRUE
  )

  direct_age <- stats::lm(y ~ age + adjust, data = df)
  direct_marker <- stats::lm(y ~ marker + adjust, data = df)

  expect_equal(stats::nobs(res$models$age), stats::nobs(direct_age))
  expect_equal(stats::nobs(res$models$marker), stats::nobs(direct_marker))
  expect_gt(stats::nobs(res$models$age), stats::nobs(res$models$marker))
  expect_equal(res$model_stats$n, c(stats::nobs(direct_age), stats::nobs(direct_marker)))
  expect_equal(stats::coef(res$models$age), stats::coef(direct_age), tolerance = 1e-8)
  expect_equal(stats::coef(res$models$marker), stats::coef(direct_marker), tolerance = 1e-8)
})

test_that("multi_reg accepts bare and quoted output options", {
  df <- birthwt_multi_data()

  bare <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = logit,
    format = gt,
    theme = minimal
  )
  quoted <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = "logit",
    format = "gt",
    theme = "minimal"
  )

  expect_equal(bare$approach, quoted$approach)
  expect_equal(bare$format, quoted$format)
  expect_equal(bare$table_display, quoted$table_display)
})

test_that("multi_reg supports flextable output", {
  skip_if_not_installed("flextable")

  df <- birthwt_multi_data()

  res <- multi_reg(
    data = df,
    outcome = "low",
    exposures = c("age", "smoke"),
    approach = logit,
    format = flextable
  )

  expect_s3_class(res, "ft_multi")
  expect_s3_class(res$table, "flextable")
  expect_equal(res$format, "flextable")
})

test_that("multi_reg supports Firth logistic regression", {
  skip_if_not_installed("logistf")

  df <- data_endometrial
  df$HG <- factor(df$HG, levels = c(0, 1),
                  labels = c("Low grade", "High grade"))
  df$NV <- factor(df$NV, levels = c(0, 1),
                  labels = c("Absent", "Present"))

  res <- multi_reg(df, outcome = HG, exposures = c(NV, PI, EH), approach = firth)

  expect_s3_class(res, "multi_reg")
  expect_equal(res$approach, "firth")
  expect_s3_class(res$models$multivariable_model, "logistf")
  expect_true("Adjusted OR (95% CI)" %in% names(res$table_display))
  expect_true(any(res$table_body$ref))
  expect_true(any(is.finite(res$table_body$estimate[!res$table_body$ref])))
  expect_equal(unname(table(df$HG, df$NV)["Low grade", "Present"]), 0)
})

test_that("multi_reg supports interaction terms in logit models", {
  df <- birthwt_multi_data()

  res <- multi_reg(
    data = df,
    outcome = "low",
    exposures = "smoke",
    adjust_for = c("age", "lwt"),
    interaction = "smoke*ht",
    approach = logit
  )

  expect_s3_class(res, "multi_reg")
  expect_named(res$models, "smoke")
  expect_match(
    paste(deparse(stats::formula(res$models$smoke)), collapse = " "),
    "smoke \\* ht"
  )
  expect_equal(res$table_body$level, c("No", "Yes"))
})

test_that("multi_reg returns diagnostics for linear models", {
  df <- birthwt_multi_data()

  res <- multi_reg(
    data = df,
    outcome = "bwt",
    exposures = c("age", "lwt", "race"),
    approach = linear
  )

  expect_s3_class(res, "multi_reg")
  expect_s3_class(res$models$multivariable_model, "lm")
  expect_s3_class(res$reg_check, "gtregression_reg_check")
  expect_true("Adjusted Beta (95% CI)" %in% names(res$table_display))
  expect_type(res$reg_check, "list")
  expect_named(res$reg_check, "multivariable_model")
  expect_true("Test" %in% names(res$reg_check$multivariable_model))

  stats_res <- multi_reg(
    data = df,
    outcome = "bwt",
    exposures = c("age", "lwt", "race"),
    approach = linear,
    model_stats = TRUE
  )
  expect_true(is.na(stats_res$model_stats$pseudo_r2))
  expect_true(is.finite(stats_res$model_stats$r_squared))
  expect_true(is.finite(stats_res$model_stats$adj_r_squared))
})

test_that("multi_reg gives data columns precedence for bare-name vectors", {
  data("data_SynthDiabetes", package = "gtregression")

  result <- multi_reg(
    data = data_SynthDiabetes,
    outcome = glucose,
    exposures = c(mass, insulin, pedigree),
    adjust_for = c(age, pregnant, pressure),
    approach = linear
  )

  expect_named(result$models, c("mass", "insulin", "pedigree"))
  expect_equal(result$adjust_for, c("age", "pregnant", "pressure"))
})

test_that("multi_reg validates required variables and outcome types", {
  df <- birthwt_multi_data()

  expect_error(
    multi_reg(df, outcome = "not_here", exposures = "age", approach = logit),
    "Outcome variable not found"
  )
  expect_error(
    multi_reg(df, outcome = "low", exposures = "not_here", approach = logit),
    "exposure variables were not found"
  )
  expect_error(
    multi_reg(df, outcome = "bwt", exposures = "age", approach = logit),
    "requires either a factor variable"
  )
  expect_error(
    multi_reg(df, outcome = "low", exposures = "age", approach = linear),
    "Linear regression requires a continuous outcome"
  )
})

test_that("multi_reg validates adjustment and interaction inputs", {
  df <- birthwt_multi_data()

  expect_error(
    multi_reg(
      df,
      outcome = "low",
      exposures = "smoke",
      adjust_for = "smoke",
      approach = logit
    ),
    "must not overlap"
  )
  expect_error(
    multi_reg(
      df,
      outcome = "low",
      exposures = "smoke",
      adjust_for = "low",
      approach = logit
    ),
    "Outcome variable cannot be included"
  )
  expect_error(
    multi_reg(
      df,
      outcome = "low",
      exposures = "smoke",
      interaction = "smoke:ht",
      approach = logit
    ),
    "not ':'"
  )
  expect_error(
    multi_reg(
      df,
      outcome = "low",
      exposures = "smoke",
      interaction = "age*lwt",
      approach = logit
    ),
    "exposure must be part"
  )
})
test_that("multi_reg displays every level of character predictors", {
  set.seed(411)
  d <- data.frame(
    outcome = stats::rbinom(180, 1, 0.4),
    classification = rep(c("Associated", "Elementary", "Combined"), each = 60),
    age = stats::rnorm(180, 50, 10)
  )

  res <- multi_reg(
    d, outcome, c(classification, age), approach = "logit", format = "gt"
  )
  rows <- res$table_body[res$table_body$exposure == "classification", ]

  expect_equal(rows$level, c("Associated", "Combined", "Elementary"))
  expect_equal(sum(rows$ref), 1L)
  expect_equal(nrow(rows), 3L)
})

#' Causal mediation analysis
#'
#' Estimate direct, indirect, total, and proportion mediated effects from
#' regression models.
#'
#' @param data A data frame.
#' @param exposure Exposure/treatment variable. Quoted and bare names are
#'   accepted.
#' @param mediator Mediator variable. Quoted and bare names are accepted.
#' @param outcome Outcome variable. Quoted and bare names are accepted.
#' @param covariates Optional character vector of covariate names. Quoted names
#'   are recommended in scripts, and bare names are also accepted.
#' @param mediator_approach Mediator model. Currently \code{"linear"}.
#' @param outcome_approach Outcome model. One of \code{"linear"} or
#'   \code{"logit"}. Logistic mediation effects are reported on the predicted
#'   probability-difference scale.
#' @param exposure_value Exposure value used as the treatment level. If
#'   \code{NULL}, the second factor level, value 1 for 0/1 variables, or the
#'   75th percentile for continuous exposures is used.
#' @param reference_value Exposure value used as the reference level. If
#'   \code{NULL}, the first factor level, value 0 for 0/1 variables, or the
#'   25th percentile for continuous exposures is used.
#' @param sims Number of non-parametric bootstrap replicates used for confidence
#'   intervals.
#' @param conf_level Confidence level for intervals.
#' @param seed Optional random seed for reproducible bootstrap intervals.
#' @param format One of \code{"flextable"} (default) or \code{"gt"}.
#' @param theme Table theme.
#'
#' @details
#' This function is for planned mediation questions, not automatic causal
#' discovery. Causal interpretation requires the usual mediation assumptions,
#' including no unmeasured exposure-outcome, exposure-mediator, or
#' mediator-outcome confounding, correct temporal order, and suitable model
#' specification. Use a directed acyclic graph (DAG) and subject-matter
#' knowledge before interpreting the estimates causally.
#'
#' For \code{outcome_approach = "linear"}, effects are mean differences. For
#' \code{outcome_approach = "logit"}, effects are predicted probability
#' differences, not odds ratios.
#'
#' @return A list of class \code{c("gtregression", "mediation_analysis", ...)}
#' with elements:
#' \describe{
#'   \item{table}{A formatted \code{flextable} or \code{gt_tbl}.}
#'   \item{table_body}{Data frame of mediation effect estimates.}
#'   \item{table_display}{Formatted data frame used to build the table.}
#'   \item{models}{Fitted mediator and outcome models.}
#'   \item{boot}{Bootstrap replicate estimates.}
#'   \item{values}{Reference and exposure values used.}
#'   \item{variable_labels}{Named character vector of display labels.}
#'   \item{call}{Matched function call.}
#' }
#'
#' @examples
#' med <- mediation_analysis(
#'   data = data_diabetes_mediation,
#'   exposure = obesity,
#'   mediator = glucose,
#'   outcome = diabetes,
#'   covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
#'   outcome_approach = logit,
#'   sims = 50,
#'   seed = 123
#' )
#'
#' med$table
#' med$table_body
#'
#' # HTML-first output
#' mediation_analysis(
#'   data = data_diabetes_mediation,
#'   exposure = obesity,
#'   mediator = glucose,
#'   outcome = diabetes,
#'   covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
#'   outcome_approach = logit,
#'   format = gt,
#'   sims = 50,
#'   seed = 123
#' )$table
#'
#' @export
mediation_analysis <- function(data,
                               exposure,
                               mediator,
                               outcome,
                               covariates = NULL,
                               mediator_approach = "linear",
                               outcome_approach = "linear",
                               exposure_value = NULL,
                               reference_value = NULL,
                               sims = 1000,
                               conf_level = 0.95,
                               seed = NULL,
                               format = c("flextable", "gt"),
                               theme = c("minimal")) {
  exposure <- .vars_arg(substitute(exposure), env = parent.frame())
  mediator <- .vars_arg(substitute(mediator), env = parent.frame())
  outcome <- .vars_arg(substitute(outcome), env = parent.frame())
  covariates <- .vars_arg(substitute(covariates), env = parent.frame(), allow_null = TRUE)

  mediator_approach <- .choice_arg(
    substitute(mediator_approach),
    env = parent.frame(),
    choices = c("linear")
  )
  outcome_approach <- .choice_arg(
    substitute(outcome_approach),
    env = parent.frame(),
    choices = c("linear", "logit")
  )
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("flextable", "gt"))
  theme <- .choice_arg(substitute(theme), env = parent.frame())
  theme <- .resolve_theme(theme)

  .validate_mediation_inputs(
    data = data,
    exposure = exposure,
    mediator = mediator,
    outcome = outcome,
    covariates = covariates,
    mediator_approach = mediator_approach,
    outcome_approach = outcome_approach,
    sims = sims,
    conf_level = conf_level
  )

  vars <- unique(c(exposure, mediator, outcome, covariates))
  df <- data[, vars, drop = FALSE]
  df <- df[stats::complete.cases(df), , drop = FALSE]
  if (nrow(df) < 10L) {
    stop("At least 10 complete observations are required.", call. = FALSE)
  }

  values <- .mediation_values(
    df[[exposure]],
    exposure_value = exposure_value,
    reference_value = reference_value
  )

  model_m <- .mediation_fit_mediator(df, exposure, mediator, covariates)
  model_y <- .mediation_fit_outcome(df, exposure, mediator, outcome, covariates, outcome_approach)
  point <- .mediation_effects(
    df = df,
    exposure = exposure,
    mediator = mediator,
    model_m = model_m,
    model_y = model_y,
    reference_value = values$reference_value,
    exposure_value = values$exposure_value,
    outcome_approach = outcome_approach
  )

  if (!is.null(seed)) {
    old_seed <- if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
      get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
    } else {
      NULL
    }
    on.exit({
      if (is.null(old_seed)) {
        if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
          rm(".Random.seed", envir = .GlobalEnv)
        }
      } else {
        assign(".Random.seed", old_seed, envir = .GlobalEnv)
      }
    }, add = TRUE)
    set.seed(seed)
  }

  boot <- .mediation_bootstrap(
    df = df,
    exposure = exposure,
    mediator = mediator,
    outcome = outcome,
    covariates = covariates,
    outcome_approach = outcome_approach,
    reference_value = values$reference_value,
    exposure_value = values$exposure_value,
    sims = as.integer(sims)
  )

  table_body <- .mediation_table_body(point, boot, conf_level)
  table_display <- .mediation_table_display(table_body)
  variable_labels <- .var_label_map(data, vars)
  attr(table_display, "variable_labels") <- variable_labels

  footnotes <- .mediation_footnotes(
    outcome_approach = outcome_approach,
    exposure = .label_var(exposure, variable_labels),
    mediator = .label_var(mediator, variable_labels),
    outcome = .label_var(outcome, variable_labels),
    covariates = .mediation_covariate_labels(covariates, variable_labels),
    values = values,
    sims = sims
  )

  tbl <- if (identical(format, "gt")) {
    .build_gt_mediation(table_display, footnotes, theme)
  } else {
    .build_flex_mediation(table_display, footnotes, theme)
  }

  res <- list(
    table = tbl,
    table_body = table_body,
    table_display = table_display,
    models = list(mediator = model_m, outcome = model_y),
    boot = boot,
    values = values,
    complete_data = df,
    variable_labels = variable_labels,
    exposure = exposure,
    mediator = mediator,
    outcome = outcome,
    covariates = covariates,
    mediator_approach = mediator_approach,
    outcome_approach = outcome_approach,
    format = format,
    source = "mediation_analysis",
    call = match.call()
  )
  class(res) <- c(
    "gtregression",
    "mediation_analysis",
    if (identical(format, "gt")) "gt_mediation" else "ft_mediation",
    class(res)
  )
  res
}

.validate_mediation_inputs <- function(data,
                                       exposure,
                                       mediator,
                                       outcome,
                                       covariates,
                                       mediator_approach,
                                       outcome_approach,
                                       sims,
                                       conf_level) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }
  vars <- unique(c(exposure, mediator, outcome, covariates))
  missing <- setdiff(vars, names(data))
  if (length(missing)) {
    stop("Variables not found in `data`: ", paste(missing, collapse = ", "), call. = FALSE)
  }
  if (length(unique(c(exposure, mediator, outcome))) != 3L) {
    stop("`exposure`, `mediator`, and `outcome` must be different variables.", call. = FALSE)
  }
  if (!identical(mediator_approach, "linear")) {
    stop("`mediator_approach` currently supports only `linear`.", call. = FALSE)
  }
  if (!outcome_approach %in% c("linear", "logit")) {
    stop("`outcome_approach` must be `linear` or `logit`.", call. = FALSE)
  }
  if (!is.numeric(data[[mediator]])) {
    stop("`mediator` must be numeric for `mediator_approach = linear`.", call. = FALSE)
  }
  if (identical(outcome_approach, "linear") && !is.numeric(data[[outcome]])) {
    stop("`outcome` must be numeric for `outcome_approach = linear`.", call. = FALSE)
  }
  if (identical(outcome_approach, "logit") && length(stats::na.omit(unique(data[[outcome]]))) != 2L) {
    stop("`outcome` must have exactly two non-missing values for `outcome_approach = logit`.", call. = FALSE)
  }
  if (!is.numeric(sims) || length(sims) != 1L || is.na(sims) || sims < 20) {
    stop("`sims` must be a single number greater than or equal to 20.", call. = FALSE)
  }
  if (!is.numeric(conf_level) || length(conf_level) != 1L ||
      is.na(conf_level) || conf_level <= 0 || conf_level >= 1) {
    stop("`conf_level` must be a single number between 0 and 1.", call. = FALSE)
  }
  invisible(TRUE)
}

.mediation_values <- function(x, exposure_value = NULL, reference_value = NULL) {
  `%||%` <- function(a, b) if (!is.null(a)) a else b
  observed <- stats::na.omit(x)
  if (is.factor(observed)) {
    lev <- levels(observed)
    if (length(lev) < 2L) {
      stop("`exposure` must have at least two observed levels.", call. = FALSE)
    }
    ref <- reference_value %||% lev[1]
    exp <- exposure_value %||% lev[2]
    return(list(reference_value = ref, exposure_value = exp))
  }

  ux <- sort(unique(observed))
  if (length(ux) < 2L) {
    stop("`exposure` must contain at least two observed values.", call. = FALSE)
  }

  if (is.numeric(observed) && all(ux %in% c(0, 1))) {
    ref <- reference_value %||% 0
    exp <- exposure_value %||% 1
  } else if (is.numeric(observed)) {
    ref <- reference_value %||% unname(stats::quantile(observed, 0.25, na.rm = TRUE, names = FALSE))
    exp <- exposure_value %||% unname(stats::quantile(observed, 0.75, na.rm = TRUE, names = FALSE))
  } else {
    ref <- reference_value %||% ux[1]
    exp <- exposure_value %||% ux[2]
  }

  list(reference_value = ref, exposure_value = exp)
}

.mediation_formula <- function(lhs, rhs) {
  stats::as.formula(paste(lhs, "~", paste(rhs, collapse = " + ")))
}

.mediation_fit_mediator <- function(df, exposure, mediator, covariates) {
  stats::lm(.mediation_formula(mediator, c(exposure, covariates)), data = df)
}

.mediation_fit_outcome <- function(df, exposure, mediator, outcome, covariates, outcome_approach) {
  f <- .mediation_formula(outcome, c(exposure, mediator, covariates))
  if (identical(outcome_approach, "logit")) {
    y <- df[[outcome]]
    if (is.factor(y)) {
      df[[outcome]] <- stats::relevel(y, ref = levels(y)[1])
    }
    return(stats::glm(f, data = df, family = stats::binomial()))
  }
  stats::lm(f, data = df)
}

.mediation_newdata <- function(df, exposure, mediator, exposure_value, mediator_value) {
  nd <- df
  nd[[exposure]] <- .mediation_cast_value(exposure_value, df[[exposure]])
  nd[[mediator]] <- mediator_value
  nd
}

.mediation_cast_value <- function(value, template) {
  if (is.factor(template)) {
    return(factor(value, levels = levels(template)))
  }
  value
}

.mediation_predict_mediator <- function(model_m, df, exposure, exposure_value) {
  nd <- df
  nd[[exposure]] <- .mediation_cast_value(exposure_value, df[[exposure]])
  stats::predict(model_m, newdata = nd, type = "response")
}

.mediation_predict_outcome <- function(model_y, nd, outcome_approach) {
  if (identical(outcome_approach, "logit")) {
    return(stats::predict(model_y, newdata = nd, type = "response"))
  }
  stats::predict(model_y, newdata = nd, type = "response")
}

.mediation_effects <- function(df,
                               exposure,
                               mediator,
                               model_m,
                               model_y,
                               reference_value,
                               exposure_value,
                               outcome_approach) {
  m_ref <- .mediation_predict_mediator(model_m, df, exposure, reference_value)
  m_exp <- .mediation_predict_mediator(model_m, df, exposure, exposure_value)

  y_exp_m_exp <- .mediation_predict_outcome(
    model_y,
    .mediation_newdata(df, exposure, mediator, exposure_value, m_exp),
    outcome_approach
  )
  y_ref_m_ref <- .mediation_predict_outcome(
    model_y,
    .mediation_newdata(df, exposure, mediator, reference_value, m_ref),
    outcome_approach
  )
  y_exp_m_ref <- .mediation_predict_outcome(
    model_y,
    .mediation_newdata(df, exposure, mediator, exposure_value, m_ref),
    outcome_approach
  )

  total <- mean(y_exp_m_exp - y_ref_m_ref, na.rm = TRUE)
  direct <- mean(y_exp_m_ref - y_ref_m_ref, na.rm = TRUE)
  indirect <- mean(y_exp_m_exp - y_exp_m_ref, na.rm = TRUE)
  prop <- if (isTRUE(all.equal(total, 0))) NA_real_ else indirect / total

  c(total = total, direct = direct, indirect = indirect, proportion = prop)
}

.mediation_bootstrap <- function(df,
                                 exposure,
                                 mediator,
                                 outcome,
                                 covariates,
                                 outcome_approach,
                                 reference_value,
                                 exposure_value,
                                 sims) {
  out <- matrix(NA_real_, nrow = sims, ncol = 4L)
  colnames(out) <- c("total", "direct", "indirect", "proportion")
  n <- nrow(df)

  for (i in seq_len(sims)) {
    idx <- sample.int(n, size = n, replace = TRUE)
    b <- df[idx, , drop = FALSE]
    out[i, ] <- tryCatch({
      model_m <- .mediation_fit_mediator(b, exposure, mediator, covariates)
      model_y <- .mediation_fit_outcome(b, exposure, mediator, outcome, covariates, outcome_approach)
      .mediation_effects(
        df = b,
        exposure = exposure,
        mediator = mediator,
        model_m = model_m,
        model_y = model_y,
        reference_value = reference_value,
        exposure_value = exposure_value,
        outcome_approach = outcome_approach
      )
    }, error = function(e) rep(NA_real_, 4L))
  }

  as.data.frame(out)
}

.mediation_table_body <- function(point, boot, conf_level) {
  alpha <- 1 - conf_level
  q <- c(alpha / 2, 1 - alpha / 2)
  effects <- c("total", "direct", "indirect", "proportion")
  labels <- c("Total effect", "Direct effect", "Indirect effect", "Proportion mediated")
  interpretation <- c(
    "Overall exposure-outcome association",
    "Association not through the mediator",
    "Association through the mediator",
    "Share of total effect through the mediator"
  )

  ci <- t(vapply(effects, function(effect) {
    vals <- boot[[effect]]
    vals <- vals[is.finite(vals)]
    if (!length(vals)) {
      return(c(NA_real_, NA_real_))
    }
    stats::quantile(vals, probs = q, na.rm = TRUE, names = FALSE)
  }, numeric(2)))

  p <- vapply(effects, function(effect) {
    vals <- boot[[effect]]
    vals <- vals[is.finite(vals)]
    if (!length(vals)) {
      return(NA_real_)
    }
    min(1, 2 * min(mean(vals <= 0), mean(vals >= 0)))
  }, numeric(1))

  data.frame(
    effect = effects,
    Effect = labels,
    estimate = unname(point[effects]),
    conf.low = ci[, 1],
    conf.high = ci[, 2],
    p.value = p,
    Interpretation = interpretation,
    stringsAsFactors = FALSE
  )
}

.mediation_table_display <- function(table_body) {
  fmt_num <- function(x, digits = 3) {
    ifelse(is.na(x), "", formatC(x, digits = digits, format = "f"))
  }
  fmt_p <- .fmt_p
  out <- data.frame(
    Effect = table_body$Effect,
    Estimate = fmt_num(table_body$estimate),
    `95% CI` = paste0(fmt_num(table_body$conf.low), " to ", fmt_num(table_body$conf.high)),
    `p-value` = fmt_p(table_body$p.value),
    Interpretation = table_body$Interpretation,
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  out
}

.mediation_covariate_labels <- function(covariates, label_map) {
  if (is.null(covariates) || !length(covariates)) {
    return(NULL)
  }
  vapply(covariates, .label_var, character(1), label_map = label_map)
}

.mediation_footnotes <- function(outcome_approach,
                                 exposure,
                                 mediator,
                                 outcome,
                                 covariates,
                                 values,
                                 sims) {
  scale_note <- if (identical(outcome_approach, "logit")) {
    "Effects are predicted probability differences from logistic outcome models."
  } else {
    "Effects are mean differences from linear outcome models."
  }
  value_note <- paste0(
    "Comparison: ",
    exposure,
    " = ",
    values$exposure_value,
    " vs ",
    values$reference_value,
    "; mediator = ",
    mediator,
    "; outcome = ",
    outcome,
    "."
  )
  adj_note <- if (!is.null(covariates) && length(covariates)) {
    paste0("Adjusted for ", paste(covariates, collapse = ", "), ".")
  } else {
    "No covariates included."
  }
  c(
    scale_note,
    paste0(value_note, " Bootstrap replicates = ", sims, "."),
    adj_note,
    "Causal interpretation requires DAG-supported no-unmeasured-confounding and correct temporal-order assumptions."
  )
}

.build_flex_mediation <- function(df, footnotes, theme) {
  if (!requireNamespace("flextable", quietly = TRUE)) {
    stop("Install 'flextable' or use format='gt'.", call. = FALSE)
  }
  ft <- flextable::flextable(df)
  ft <- flextable::theme_vanilla(ft)
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  ft <- flextable::align(ft, j = "Effect", align = "left", part = "all")
  ft <- flextable::align(ft, j = setdiff(names(df), c("Effect", "Interpretation")), align = "center", part = "all")
  ft <- flextable::align(ft, j = "Interpretation", align = "left", part = "all")
  if ("header_shaded" %in% theme) {
    ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  }
  if ("compact" %in% theme) {
    ft <- flextable::padding(ft, padding = 2)
  }
  ft <- flextable::autofit(ft)
  ft <- flextable::add_footer_lines(ft, values = footnotes)
  .compact_flex_footer(ft)
}

.build_gt_mediation <- function(df, footnotes, theme) {
  if (!requireNamespace("gt", quietly = TRUE)) {
    stop("Install 'gt' or use format='flextable'.", call. = FALSE)
  }
  tb <- gt::gt(df) |>
    gt::cols_align("left", columns = c("Effect", "Interpretation")) |>
    gt::cols_align("center", columns = c("Estimate", "95% CI", "p-value")) |>
    gt::tab_options(
      table.font.names = "system-ui",
      data_row.padding = gt::px(4),
      table.background.color = "white"
    ) |>
    gt::tab_style(gt::cell_text(weight = "bold"), gt::cells_column_labels())
  if ("header_shaded" %in% theme) {
    tb <- gt::tab_options(tb, column_labels.background.color = "#f6f8fa")
  }
  if ("zebra" %in% theme) {
    tb <- gt::opt_row_striping(tb)
  }
  if ("compact" %in% theme) {
    tb <- gt::tab_options(tb, data_row.padding = gt::px(2))
  }
  tb <- gt::tab_source_note(tb, source_note = footnotes)
  .compact_gt_source_notes(tb)
}

test_that("mediation_analysis returns formatted logit mediation output", {
  med <- mediation_analysis(
    data = data_diabetes_mediation,
    exposure = obesity,
    mediator = glucose,
    outcome = diabetes,
    covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
    outcome_approach = logit,
    sims = 30,
    seed = 123
  )

  expect_s3_class(med, "gtregression")
  expect_s3_class(med, "mediation_analysis")
  expect_s3_class(med$table, "flextable")
  expect_named(med$table_body, c(
    "effect", "Effect", "estimate", "conf.low", "conf.high", "p.value",
    "Interpretation"
  ))
  expect_equal(med$table_body$effect, c("total", "direct", "indirect", "proportion"))
  expect_true(all(is.finite(med$table_body$estimate)))
  expect_s3_class(med$models$mediator, "lm")
  expect_s3_class(med$models$outcome, "glm")
  expect_equal(med$values$reference_value, "No")
  expect_equal(med$values$exposure_value, "Yes")
  expect_equal(med$variable_labels[["obesity"]], "Obesity")
  expect_equal(med$variable_labels[["glucose"]], "Plasma glucose")
})

test_that("mediation_analysis accepts quoted names and gt output", {
  med <- mediation_analysis(
    data = data_diabetes_mediation,
    exposure = "obesity",
    mediator = "glucose",
    outcome = "diabetes",
    covariates = c("age", "blood_pressure"),
    outcome_approach = "logit",
    format = "gt",
    sims = 25,
    seed = 321
  )

  expect_s3_class(med$table, "gt_tbl")
  expect_equal(med$covariates, c("age", "blood_pressure"))
})

test_that("mediation_analysis supports a linear outcome", {
  med <- mediation_analysis(
    data = data_diabetes_mediation,
    exposure = obesity,
    mediator = glucose,
    outcome = bmi,
    covariates = c(age, blood_pressure),
    outcome_approach = linear,
    sims = 25,
    seed = 456
  )

  expect_s3_class(med$models$outcome, "lm")
  expect_equal(med$outcome_approach, "linear")
  expect_true(all(is.finite(med$table_body$estimate[1:3])))
})

test_that("mediation_analysis validates inputs", {
  expect_error(
    mediation_analysis(
      data = data_diabetes_mediation,
      exposure = obesity,
      mediator = diabetes,
      outcome = glucose,
      sims = 20
    ),
    "`mediator` must be numeric",
    fixed = TRUE
  )

  expect_error(
    mediation_analysis(
      data = data_diabetes_mediation,
      exposure = obesity,
      mediator = glucose,
      outcome = bmi,
      outcome_approach = logit,
      sims = 20
    ),
    "`outcome` must have exactly two non-missing values",
    fixed = TRUE
  )

  expect_error(
    mediation_analysis(
      data = data_diabetes_mediation,
      exposure = obesity,
      mediator = glucose,
      outcome = diabetes,
      outcome_approach = logit,
      sims = 10
    ),
    "`sims` must be a single number greater than or equal to 20",
    fixed = TRUE
  )
})

#' Internal validation helpers for model setup
#' Validate regression approach by context
#'
#' Ensures that only valid modeling approaches are used in each function context
#'
#' @param approach A character string specifying the regression method.
#' @param context The name of the function using the approach (e.g., "uni_reg").
#'
#' @return Stops with an informative error if the approach is not allowed.
#' @keywords internal
#' @noRd
.validate_approach <- function(approach, context = NULL) {
  approach_input <- approach
  approach <- .normalize_approach(approach)

  # Define valid approaches per context
  valid <- switch(context,
                  "uni_reg" = c("logit", "logbinomial", "poisson",
                                "robpoisson", "linear", "negbin", "firth"),
                  "multi_reg" = c("logit", "logbinomial", "poisson",
                                  "robpoisson", "linear", "negbin", "firth"),
                  "interaction_models" = c("logit", "logbinomial", "poisson",
                                           "robpoisson", "negbin", "linear",
                                           "cox", "survreg"),
                  "check_convergence" = c("logit", "logbinomial", "poisson",
                                          "robpoisson", "negbin"),
                  "identify_confounder" = c("logit", "logbinomial", "poisson",
                                            "robpoisson", "negbin", "linear",
                                            "cox", "survreg"),
                  "select_models" = c("logit", "logbinomial", "poisson",
                                      "robpoisson", "negbin", "linear",
                                      "cox", "survreg"),
                  stop("The function '", context, "' is not recognized.\n",
                       "Please use a valid function")
  )

  # Validate the chosen approach
  if (!approach %in% valid) {
    stop(approach_input, "  is not a valid approach for ", context, ".\n",
         "Valid options: ", paste(valid, collapse = ", "), call. = FALSE)
  }

  invisible(TRUE)
}

#' Validate Exposure Variable(s) for Regression
#'
#' Ensures that the exposure variable has at least two non-missing levels or
#' sufficient numeric variation to support regression modelling.
#' @param exposures Character vector of column names to validate.
#' @param data A data frame containing the exposure variables.
#' @return Returns TRUE if valid; otherwise throws an error.
#' @keywords internal
.validate_exposures <- function(data, exposures) {
  # Initialize lists to hold variable names with different issues
  vars_all_na <- c()
  vars_invalid_cat <- c()
  vars_invalid_num <- c()
  vars_invalid_type <- c()

  for (var in exposures) {
    x <- data[[var]]

    # Check for all NAs
    if (all(is.na(x))) {
      vars_all_na <- c(vars_all_na, var)
      next
    }

    # Drop NA values for observed-level checks.
    x_non_na <- x[!is.na(x)]

    # Type checks should use the original extracted vector. Some classes can
    # lose useful metadata after subsetting, while the observed-level count
    # should use the non-missing values.
    is_categorical <- is.factor(x) || is.character(x)
    is_numeric <- is.numeric(x)

    if (is_categorical) {
      if (length(unique(as.character(x_non_na))) < 2) {
        vars_invalid_cat <- c(vars_invalid_cat, var)
      }
    } else if (is_numeric) {
      if (length(unique(x_non_na)) <= 1) {
        vars_invalid_num <- c(vars_invalid_num, var)
      }
    } else {
      vars_invalid_type <- c(vars_invalid_type, var)
    }
  }

  # Combine all errors into one message
  error_msgs <- c()

  if (length(vars_all_na) > 0) {
    error_msgs <- c(
      error_msgs,
      paste0("Exposure with all missing values: ",
             paste(vars_all_na, collapse = ", "),
             "Please fix them to proceed." ),
      call. = FALSE
    )
  }

  if (length(vars_invalid_cat) > 0) {
    error_msgs <- c(
      error_msgs,
      paste0("Exposure with <2 levels: ",
             paste(vars_invalid_cat, collapse = ", "),
             "Please fix them to proceed."),
      call. = FALSE
    )
  }

  if (length(vars_invalid_num) > 0) {
    error_msgs <- c(
      error_msgs,
      paste0("Exposure with no variation: ",
             paste(vars_invalid_num, collapse = ", "),
             "Please fix them to proceed."),
      call. = FALSE
    )
  }

  if (length(vars_invalid_type) > 0) {
    error_msgs <- c(
      error_msgs,
      paste0("Exposure with unsupported types: ",
             paste(vars_invalid_type, collapse = ", "),
             "Please fix them to proceed."),
      call. = FALSE
    )
  }

  # If any errors exist, stop with full message
  if (length(error_msgs) > 0) {
    stop(paste(error_msgs, collapse = "\n"), call. = FALSE)
  }

  invisible(TRUE)
}
# validate outcome against the approaches used
#' @keywords internal
#' @noRd
.validate_outcome_by_approach <- function(outcome, approach) {
  approach <- .normalize_approach(approach)

  # Missing outcome values in cols
  if (all(is.na(outcome))) {
    stop("All values in the outcome variable are missing.",
         call. = FALSE)
  }
  # Conditional check binary outcomes
  is_binary <- function(outcome) {
    # Check for factor or character with exactly 2 levels
    if (is.factor(outcome) || is.character(outcome)) {
      return(length(unique(na.omit(outcome))) == 2)
    }

    # Check for numeric with values in 0/1 or 1/2
    if (is.numeric(outcome)) {
      vals <- unique(na.omit(outcome))
      return(all(vals %in% c(0, 1)) || all(vals %in% c(1, 2)))
    }
    return(FALSE)
  }

  # check for Non-negative integers
  is_count <- function(outcome) {
    is.numeric(outcome) &&
      all(outcome >= 0 &
            outcome == floor(outcome), na.rm = TRUE) &&
      length(unique(outcome[!is.na(outcome)])) >= 1
  }

  # check for numeric- includes whole numbers and decimals
  is_continuous <- function(outcome) {
    is.numeric(outcome) && length(unique(na.omit(outcome))) > 2
  }

  # apply logic
  if (approach %in% c("logit", "firth", "logbinomial", "robpoisson") &&
      !is_binary(outcome)) {
    stop("This approach requires either a factor variable ",
         "or numeric variable coded as 0 and 1 (or 1 and 2).", call. = FALSE)
  }
  if (approach == "poisson" && !is_count(outcome)) {
    stop("Poisson regression requires a count outcome.", call. = FALSE)
  }
  if (approach == "negbin" && !is_count(outcome)) {
    stop("Negative binomial requires a count outcome.", call. = FALSE)
  }
  if (approach == "linear" && !is_continuous(outcome)) {
    stop("Linear regression requires a continuous outcome.", call. = FALSE)
  }
}

# validate inputs for uni reg
#' @keywords internal
#' @noRd
.validate_uni_inputs <- function(data,
                                 outcome,
                                 exposures,
                                 approach) {
  # validate approach in the function
  .validate_approach(approach, context = "uni_reg")

  # check variable presence
  if (!outcome %in% names(data))
    stop("Outcome variable not found in the dataset.", call. = FALSE)

  # check variable presence >1
  if (!all(exposures %in% names(data)))
    stop("One or more exposure variables were not found in the dataset.",
         call. = FALSE)

  # outcome variable validation
  .validate_outcome_by_approach(data[[outcome]], approach)

  # Validate exposures
  .validate_exposures(data, exposures)
}

#' Validate inputs for multi_reg
#'
#' @keywords internal
#' @noRd
.validate_multi_inputs <- function(data,
                                   outcome,
                                   exposures,
                                   approach,
                                   adjust_for = NULL,
                                   interaction = NULL) {

  .validate_approach(approach, context = "multi_reg")

  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }

  if (!is.character(outcome) || length(outcome) != 1) {
    stop("`outcome` must be a single character string.", call. = FALSE)
  }

  exposures <- unique(exposures)

  if (!outcome %in% names(data)) {
    stop("Outcome variable not found in the dataset.", call. = FALSE)
  }

  if (!is.character(exposures) || length(exposures) < 1) {
    stop("`exposures` must be a character vector with at least one variable.",
         call. = FALSE)
  }

  if (!all(exposures %in% names(data))) {
    stop(
      "One or more exposure variables were not found in the dataset.",
      call. = FALSE
    )
  }

  .validate_outcome_by_approach(data[[outcome]], approach)
  .validate_exposures(data, exposures)

  adjusted_mode <- !is.null(adjust_for) && length(adjust_for) > 0

  if (adjusted_mode) {
    adjust_for <- unique(adjust_for)

    if (!is.character(adjust_for)) {
      stop("`adjust_for` must be a character vector.", call. = FALSE)
    }

    if (!all(adjust_for %in% names(data))) {
      stop(
        "One or more adjustment variables were not found in the dataset.",
        call. = FALSE
      )
    }

    if (outcome %in% adjust_for) {
      stop("Outcome variable cannot be included in `adjust_for`.", call. = FALSE)
    }

    if (any(exposures %in% adjust_for)) {
      stop(
        "In adjusted mode, `exposures` and `adjust_for` must not overlap.",
        call. = FALSE
      )
    }

    .validate_exposures(data, adjust_for)
  } else {
    adjust_for <- NULL
  }

  interaction_vars <- character(0)

  if (!is.null(interaction) && length(interaction) > 0) {
    if (!is.character(interaction) || length(interaction) != 1) {
      stop(
        "`interaction` must be a single character string such as 'bmi*sex'.",
        call. = FALSE
      )
    }

    if (grepl(":", interaction, fixed = TRUE)) {
      stop(
        "Use standard interaction syntax with '*', for example 'bmi*sex', not ':'.",
        call. = FALSE
      )
    }

    if (!grepl("\\*", interaction)) {
      stop(
        "`interaction` must contain '*', for example 'bmi*sex'.",
        call. = FALSE
      )
    }

    interaction_vars <- trimws(unlist(strsplit(interaction, "\\*")))
    interaction_vars <- interaction_vars[nzchar(interaction_vars)]

    if (length(interaction_vars) != 2) {
      stop(
        "`interaction` must contain exactly two variables, e.g. 'bmi*sex'.",
        call. = FALSE
      )
    }

    if (!all(interaction_vars %in% names(data))) {
      stop(
        "One or more variables in `interaction` were not found in the dataset.",
        call. = FALSE
      )
    }

    if (adjusted_mode && length(exposures) != 1) {
      stop(
        "When `interaction` is supplied with `adjust_for`, please provide a single exposure.",
        call. = FALSE
      )
    }

    if (!any(exposures %in% interaction_vars)) {
      stop(
        "The exposure must be part of the interaction term.",
        call. = FALSE
      )
    }
  } else {
    interaction <- NULL
  }

  vars_needed <- unique(c(outcome, exposures, adjust_for, interaction_vars))

  cc_idx <- stats::complete.cases(data[, vars_needed, drop = FALSE])
  data_clean <- data[cc_idx, , drop = FALSE]

  if (nrow(data_clean) == 0) {
    stop("No complete cases available for analysis.", call. = FALSE)
  }

  vars_to_check <- unique(c(exposures, adjust_for, interaction_vars))
  vars_to_check <- vars_to_check[vars_to_check %in% names(data_clean)]

  insufficient_vars <- vars_to_check[
    vapply(
      data_clean[, vars_to_check, drop = FALSE],
      function(x) length(unique(stats::na.omit(x))) < 2,
      logical(1)
    )
  ]

  if (length(insufficient_vars) > 0) {
    stop(
      "Variable(s) with less than 2 unique values after complete-case filtering: ",
      paste(insufficient_vars, collapse = ", "),
      call. = FALSE
    )
  }

  data_clean
}

#' Validate and split an interaction term
#'
#' @keywords internal
#' @noRd
.validate_interaction_term <- function(data,
                                       exposures,
                                       interaction = NULL,
                                       adjusted_mode = FALSE,
                                       exposure_by_exposure = FALSE) {
  if (is.null(interaction) || !length(interaction)) {
    return(character(0))
  }

  if (!is.character(interaction) || length(interaction) != 1L) {
    stop(
      "`interaction` must be a single character string such as 'bmi*sex'.",
      call. = FALSE
    )
  }

  if (grepl(":", interaction, fixed = TRUE)) {
    stop(
      "Use standard interaction syntax with '*', for example 'bmi*sex', not ':'.",
      call. = FALSE
    )
  }

  if (!grepl("\\*", interaction)) {
    stop(
      "`interaction` must contain '*', for example 'bmi*sex'.",
      call. = FALSE
    )
  }

  interaction_vars <- trimws(unlist(strsplit(interaction, "\\*")))
  interaction_vars <- interaction_vars[nzchar(interaction_vars)]

  if (length(interaction_vars) != 2L) {
    stop(
      "`interaction` must contain exactly two variables, e.g. 'bmi*sex'.",
      call. = FALSE
    )
  }

  if (!all(interaction_vars %in% names(data))) {
    stop(
      "One or more variables in `interaction` were not found in the dataset.",
      call. = FALSE
    )
  }

  if ((adjusted_mode || exposure_by_exposure) && length(exposures) != 1L) {
    stop(
      paste0(
        "When `interaction` is supplied in exposure-by-exposure mode, please provide a single ",
        "exposure or set `multivariable = TRUE`."
      ),
      call. = FALSE
    )
  }

  if (!any(exposures %in% interaction_vars)) {
    stop("The exposure must be part of the interaction term.", call. = FALSE)
  }

  interaction_vars
}

test_that(".validate_approach accepts valid inputs for each context", {
  expect_invisible(.validate_approach("logit",         "uni_reg"))
  expect_invisible(.validate_approach("firth",         "uni_reg"))
  expect_invisible(.validate_approach("firth",         "multi_reg"))
  expect_invisible(.validate_approach("negbin",        "multi_reg"))
  expect_invisible(.validate_approach("robpoisson",    "interaction_models"))
  expect_invisible(.validate_approach("logbinomial",  "check_convergence"))
  expect_invisible(.validate_approach("linear",        "identify_confounder"))
  expect_invisible(.validate_approach("poisson",       "select_models"))
})

test_that(".validate_approach errors for invalid approach and unknown context", {
  # Allow variable whitespace in the error ("R  is ...")
  expect_error(
    .validate_approach("R", "uni_reg"),
    regexp = "^R\\s+is not a valid approach for\\s+uni_reg\\.",
    perl   = TRUE
  )
  expect_error(
    .validate_approach("logit", "unknown_context"),
    regexp = "The function 'unknown_context' is not recognized",
    fixed  = TRUE
  )
})

test_that(".validate_exposures passes with good inputs", {
  df <- data.frame(
    num = c(1, 2, 3, 4),
    fac = factor(c("a", "b", "a", "b")),
    chr = c("x", "y", "x", "y")
  )
  expect_invisible(.validate_exposures(df, c("num", "fac", "chr")))
})

test_that(".validate_exposures counts observed factor levels", {
  df <- data.frame(
    fac = factor(c("No", "Yes", "No", "Yes"), levels = c("No", "Yes"))
  )

  expect_invisible(.validate_exposures(df, "fac"))
})

test_that(".validate_exposures aggregates all error messages", {
  df <- data.frame(
    allNA  = c(NA, NA, NA),
    fac1   = factor(c("a", "a", NA)),  # < 2 levels after NA drop
    const  = c(5, 5, 5),               # no numeric variation
    when   = as.Date(c("2020-01-01", NA, "2020-01-03"))  # unsupported type branch
  )

  msg <- tryCatch(
    { .validate_exposures(df, names(df)); "" },
    error = function(e) e$message
  )

  expect_true(nzchar(msg))
  needles <- c("all missing", "<2 levels", "no variation", "unsupported types")
  expect_true(all(vapply(needles, function(p) grepl(p, msg, ignore.case = TRUE), logical(1))))
})

test_that(".validate_outcome_by_approach: success cases", {
  bin_01     <- c(0,1,1,0,1)
  bin_12     <- c(1,2,2,1,1)        # also allowed
  counts     <- c(0,1,2,3,5)
  continuous <- c(1.2, 2.4, 3.1, 4.0, 5.6)

  expect_silent(.validate_outcome_by_approach(bin_01, "logit"))
  expect_silent(.validate_outcome_by_approach(bin_01, "firth"))
  expect_silent(.validate_outcome_by_approach(bin_12, "logit"))
  expect_silent(.validate_outcome_by_approach(bin_01, "robpoisson"))
  expect_silent(.validate_outcome_by_approach(bin_12, "logbinomial"))
  expect_silent(.validate_outcome_by_approach(counts, "poisson"))
  expect_silent(.validate_outcome_by_approach(counts, "negbin"))
  expect_silent(.validate_outcome_by_approach(continuous, "linear"))
})

test_that(".validate_outcome_by_approach: failure cases", {
  all_na   <- c(NA, NA, NA)
  nonbin   <- c(0, 1, 2)      # not binary
  noncount <- c(1.5, 2.3, 3)  # not all integers / may include decimals
  notcont  <- c(0, 1, 1, 0)   # not continuous (>2 uniques required)

  expect_error(.validate_outcome_by_approach(all_na, "logit"),
               "All values in the outcome variable are missing", fixed = TRUE)

  expect_error(.validate_outcome_by_approach(nonbin, "logit"),
               regexp = "requires either a factor variable|numeric variable coded as 0 and 1",
               ignore.case = TRUE)
  expect_error(.validate_outcome_by_approach(nonbin, "firth"),
               regexp = "requires either a factor variable|numeric variable coded as 0 and 1",
               ignore.case = TRUE)

  expect_error(.validate_outcome_by_approach(noncount, "poisson"),
               "Poisson regression requires a count outcome", fixed = TRUE)
  expect_error(.validate_outcome_by_approach(noncount, "negbin"),
               "Negative binomial requires a count outcome", fixed = TRUE)

  expect_error(.validate_outcome_by_approach(notcont, "linear"),
               "Linear regression requires a continuous outcome", fixed = TRUE)
})

test_that(".validate_uni_inputs wires validations correctly", {
  df <- data.frame(y = c(0,1,1,0), x = c(1,2,3,4))
  expect_silent(.validate_uni_inputs(df, outcome = "y", exposures = "x", approach = "logit"))

  expect_error(.validate_uni_inputs(df, outcome = "missing", exposures = "x", approach = "logit"),
               "Outcome variable not found", fixed = TRUE)
  expect_error(.validate_uni_inputs(df, outcome = "y", exposures = "bad", approach = "logit"),
               "One or more exposure variables were not found", fixed = TRUE)

  # Exposure invalid (<2 levels) → bubbles up from .validate_exposures()
  df_bad <- data.frame(y = c(0,1,1,0), x = factor(c("a","a","a","a")))
  expect_error(.validate_uni_inputs(df_bad, outcome = "y", exposures = "x", approach = "logit"),
               "<2 levels", fixed = FALSE)
})

test_that(".validate_multi_inputs wires validations correctly", {
  df <- data.frame(y = c(0,1,1,0), x = c(1,2,3,4), z = factor(c("a","b","a","b")))
  expect_silent(.validate_multi_inputs(df, outcome = "y", exposures = c("x","z"), approach = "logit"))

  expect_error(.validate_multi_inputs(df, outcome = "missing", exposures = c("x","z"), approach = "logit"),
               "Outcome variable not found", fixed = TRUE)
  expect_error(.validate_multi_inputs(df, outcome = "y", exposures = c("x","bad"), approach = "logit"),
               "One or more exposure variables were not found", fixed = TRUE)

  # Exposure invalid (<2 levels) → bubbles up from .validate_exposures()
  df_bad <- data.frame(y = c(0,1,1,0), x = factor(c("a","a","a","a")), z = factor(c("a","b","a","b")))
  expect_error(.validate_multi_inputs(df_bad, outcome = "y", exposures = c("x","z"), approach = "logit"),
               "<2 levels", fixed = FALSE)
})
# End of file

#' Check regression model convergence
#'
#' Check whether requested models converge, either one exposure at a time or as
#' a single multivariable model.
#'
#' @param data A data frame containing the dataset.
#' @param exposures A character vector of predictor variable names. Quoted names
#'   are recommended in scripts, and bare names are also accepted.
#'   If \code{multivariate = FALSE}, each exposure is assessed separately.
#'   If \code{multivariate = TRUE}, exposures are included together.
#' @param outcome A character string specifying the outcome variable. Quoted and
#'   bare names are accepted.
#' @param approach A character string specifying the regression approach.
#' One of:
#'   \code{"logit"}, \code{"logbinomial"}, \code{"poisson"},
#'   \code{"robpoisson"}, or \code{"negbin"}.
#' @param multivariate Logical. If \code{TRUE},
#' checks convergence for a multivariable model;
#'   otherwise, performs checks for each univariate model.
#' @param format Output format. One of \code{"flextable"} (default),
#'   \code{"gt"}, or \code{"tibble"}. Use \code{format = "tibble"} for the
#'   original data-frame style output.
#'
#' @return A data frame, \code{gt_tbl}, or \code{flextable} summarizing
#' convergence diagnostics, including:
#' \describe{
#'   \item{\code{Exposure}}{Name of the exposure variable.}
#'   \item{\code{Model}}{The regression approach used.}
#'   \item{\code{Converged}}{\code{TRUE} if the model converged successfully;
#'    \code{FALSE} otherwise.}
#'   \item{\code{Max.prob.}}{Maximum predicted probability or
#'   fitted value in the dataset.}
#' }
#'
#' @details
#' For \code{robpoisson}, predicted probabilities (fitted values) may exceed 1,
#' which is acceptable when estimating risk ratios but should not be interpreted
#' as actual probabilities.
#'
#' This is especially useful for \code{"logbinomial"} models, where convergence
#' problems are common and robust Poisson may be a practical alternative.
#'
#' @seealso [identify_confounder()],  [interaction_models()]
#'
#' @examples
#' if (requireNamespace("gtregression", quietly = TRUE)) {
#'   data(data_SynthDiabetes, package = "gtregression")
#'
#'   check_convergence(
#'     data = data_SynthDiabetes,
#'     exposures = c("age", "mass"),
#'     outcome = "diabetes",
#'     approach = "logit",
#'     format = "tibble"
#'   )
#'
#'   check_convergence(
#'     data = data_SynthDiabetes,
#'     exposures = c("age", "mass"),
#'     outcome = "diabetes",
#'     approach = "logit",
#'     multivariate = TRUE,
#'     format = "gt"
#'   )
#' }
#' @export
check_convergence <- function(data,
                              exposures,
                              outcome,
                              approach = "logit",
                              multivariate = FALSE,
                              format = c("flextable", "gt", "tibble")) {

  if (!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }
  exposures <- .vars_arg(substitute(exposures), env = parent.frame())
  outcome <- .vars_arg(substitute(outcome), env = parent.frame())
  if (!is.character(exposures) || length(exposures) < 1L ||
      anyNA(exposures) || any(!nzchar(exposures))) {
    stop("`exposures` must be a non-empty character vector.", call. = FALSE)
  }
  if (!is.character(outcome) || length(outcome) != 1L ||
      is.na(outcome) || !nzchar(outcome)) {
    stop("`outcome` must be a single character variable name.", call. = FALSE)
  }
  if (!is.logical(multivariate) || length(multivariate) != 1L ||
      is.na(multivariate)) {
    stop("`multivariate` must be TRUE or FALSE.", call. = FALSE)
  }
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  format <- match.arg(format, c("flextable", "gt", "tibble"))

  missing_vars <- setdiff(c(outcome, exposures), names(data))
  if (length(missing_vars)) {
    stop("Variables not found: ", paste(missing_vars, collapse = ", "),
         call. = FALSE)
  }

  approach <- .choice_arg(
    substitute(approach),
    env = parent.frame(),
    choices = c("logit","logbinomial","poisson","robpoisson","negbin")
  )
  approach <- .normalize_approach(approach)

  .validate_approach(approach, context = "check_convergence")

  # Return early if dataset is empty

  if (nrow(data) == 0) {
    empty <- data.frame(
      Exposure = character(0),
      Model = character(0),
      Converged = logical(0),
      Max.prob. = numeric(0)
    )
    if (format == "tibble") {
      return(empty)
    }
    return(.build_check_convergence_table(empty, format = format))
  }

  .validate_outcome_by_approach(data[[outcome]], approach)

  fit_model <- function(fmla) {
    switch(approach,
      "logit" = stats::glm(fmla, data = data, family = stats::binomial("logit")),
      "logbinomial" = stats::glm(fmla, data = data, family = stats::binomial("log")),
      "poisson" = stats::glm(fmla, data = data, family = stats::poisson("log")),
      "negbin" = MASS::glm.nb(fmla, data = data),
      risks::riskratio(formula = fmla, data = data, approach = approach)
    )
  }

  extract_convergence <- function(fit) {
    if ("converged" %in% names(fit)) {
      return(isTRUE(fit$converged))
    }
    if (!is.null(fit$conv)) {
      return(isTRUE(fit$conv == 0))
    }
    NA
  }

  extract_max_prob <- function(fit) {
    max_prob <- if ("maxprob" %in% names(fit)) {
      fit$maxprob
    } else {
      max(stats::predict(fit, type = "response"), na.rm = TRUE)
    }
    as.numeric(max_prob)
  }

  warn_if_large_robpoisson <- function(max_prob) {
    if (approach == "robpoisson" && is.finite(max_prob) && max_prob > 1) {
      warning(
        "robpoisson: predicted fitted value exceeds 1; interpret estimates with caution.",
        call. = FALSE
      )
    }
  }

  results_list <- list()

  if (!multivariate) {
    # Loop through exposures and fit separate models
    for (exposure in exposures) {
      fmla <- stats::reformulate(exposure, response = outcome)
      result <- tryCatch({
          fit <- fit_model(fmla)
          # Extract convergence status and predicted max
          converged <- extract_convergence(fit)
          max_prob <- extract_max_prob(fit)
          # Warning if robpoisson max_prob > 1
          warn_if_large_robpoisson(max_prob)
          data.frame(Exposure = exposure, Model = approach,
                     Converged = converged, Max.prob. = max_prob)
        },
        error = function(e) {
          data.frame(Exposure = exposure, Model = approach,
                     Converged = FALSE, Max.prob. = NA_real_)
        }
      )
      results_list[[exposure]] <- result
    }
  } else {
    # Build multivariable model
    fmla <- stats::reformulate(exposures, response = outcome)
    result <- tryCatch({
        fit <- fit_model(fmla)
        converged <- extract_convergence(fit)
        max_prob <- extract_max_prob(fit)

        # Custom warning if predicted prob > 1
        warn_if_large_robpoisson(max_prob)
        data.frame(Exposure = paste(exposures, collapse = " + "),
                   Model = approach, Converged = converged,
                   Max.prob. = max_prob)
      },
      error = function(e) {
        warning("Model fitting failed for the selected approach", call. = FALSE)
        data.frame(Exposure = paste(exposures, collapse = " + "),
                   Model = approach,
                   Converged = FALSE,
                   Max.prob. = NA_real_)
      }
    )


    results_list[["multivariable"]] <- result
  }

  out <- do.call(rbind, results_list)
  if (format == "tibble") {
    return(out)
  }
  .build_check_convergence_table(out, format = format)
}

#' Build formatted convergence table
#' @keywords internal
#' @noRd
.build_check_convergence_table <- function(result,
                                           format = c("flextable", "gt")) {
  format <- match.arg(format, c("flextable", "gt"))
  note <- paste(
    "Screening aid only; inspect non-convergence, impossible fitted values,",
    "and model specification before interpreting estimates."
  )

  result$Max.prob. <- suppressWarnings(as.numeric(result$Max.prob.))

  display <- result |>
    dplyr::mutate(
      Converged = dplyr::case_when(
        is.na(.data$Converged) ~ "",
        .data$Converged ~ "Yes",
        TRUE ~ "No"
      ),
      Max.prob. = ifelse(is.na(.data$Max.prob.), "",
                          formatC(.data$Max.prob., digits = 3, format = "f"))
    )

  if (format == "gt") {
    return(
      gt::gt(display) |>
        gt::tab_header(title = "Convergence check") |>
        gt::cols_label(Max.prob. = "Max fitted value") |>
        gt::cols_align(align = "left", columns = c("Exposure", "Model")) |>
        gt::cols_align(align = "center", columns = c("Converged", "Max.prob.")) |>
        gt::tab_style(
          style = gt::cell_text(weight = "bold"),
          locations = gt::cells_column_labels()
        ) |>
        gt::tab_style(
          style = gt::cell_fill(color = "#e7f5ec"),
          locations = gt::cells_body(rows = .data$Converged == "Yes")
        ) |>
        gt::tab_style(
          style = gt::cell_fill(color = "#fde2e2"),
          locations = gt::cells_body(rows = .data$Converged == "No")
        ) |>
        gt::tab_source_note(gt::md(note)) |>
        .compact_gt_source_notes()
    )
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Convergence check")
  ft <- flextable::set_header_labels(ft, Max.prob. = "Max fitted value")
  ft <- flextable::align(ft, j = c("Exposure", "Model"), align = "left", part = "all")
  ft <- flextable::align(ft, j = c("Converged", "Max.prob."), align = "center", part = "all")
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  ft <- flextable::bg(ft, i = which(display$Converged == "Yes"), bg = "#e7f5ec", part = "body")
  ft <- flextable::bg(ft, i = which(display$Converged == "No"), bg = "#fde2e2", part = "body")
  ft <- flextable::add_footer_lines(ft, values = note)
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

birthwt_convergence_data <- function() {
  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
    )
}

test_that("check_convergence runs univariable and multivariable logit models", {
  df <- birthwt_convergence_data()
  exposures <- c("age", "lwt", "race", "smoke")

  result_uni <- check_convergence(
    data = df,
    exposures = exposures,
    outcome = "low",
    approach = logit,
    format = tibble
  )
  result_multi <- check_convergence(
    data = df,
    exposures = exposures,
    outcome = "low",
    approach = "logit",
    multivariate = TRUE,
    format = tibble
  )

  expect_s3_class(result_uni, "data.frame")
  expect_named(result_uni, c("Exposure", "Model", "Converged", "Max.prob."))
  expect_equal(result_uni$Exposure, exposures)
  expect_true(all(result_uni$Model == "logit"))
  expect_true(all(result_uni$Converged))
  expect_true(all(is.finite(result_uni$Max.prob.)))

  expect_s3_class(result_multi, "data.frame")
  expect_equal(nrow(result_multi), 1L)
  expect_equal(result_multi$Exposure, paste(exposures, collapse = " + "))
  expect_true(result_multi$Converged)
})

test_that("check_convergence supports logbinomial and robpoisson approaches", {
  df <- birthwt_convergence_data()

  logbin <- check_convergence(
    data = df,
    exposures = c("age", "smoke"),
    outcome = "low",
    approach = logbinomial,
    format = tibble
  )
  rob <- suppressWarnings(check_convergence(
    data = df,
    exposures = c("age", "smoke"),
    outcome = "low",
    approach = robpoisson,
    format = tibble
  ))

  expect_s3_class(logbin, "data.frame")
  expect_true(all(logbin$Model == "logbinomial"))
  expect_true(all(logbin$Exposure == c("age", "smoke")))
  expect_s3_class(rob, "data.frame")
  expect_true(all(rob$Model == "robpoisson"))
  expect_true(all(is.na(rob$Converged) | rob$Converged %in% c(TRUE, FALSE)))
})

test_that("check_convergence supports poisson and negative binomial count models", {
  data("quine", package = "MASS")
  quine_data <- quine |>
    dplyr::mutate(Days = as.numeric(Days))
  exposures <- c("Eth", "Sex", "Age", "Lrn")

  pois <- check_convergence(
    data = quine_data,
    exposures = exposures,
    outcome = "Days",
    approach = poisson,
    format = tibble
  )
  nb <- check_convergence(
    data = quine_data,
    exposures = exposures,
    outcome = "Days",
    approach = negbin,
    multivariate = TRUE,
    format = tibble
  )

  expect_s3_class(pois, "data.frame")
  expect_equal(nrow(pois), length(exposures))
  expect_true(all(pois$Converged))
  expect_true(all(is.finite(pois$Max.prob.)))

  expect_s3_class(nb, "data.frame")
  expect_equal(nrow(nb), 1L)
  expect_true(nb$Converged)
  expect_true(is.finite(nb$Max.prob.))
})

test_that("check_convergence validates inputs clearly", {
  df <- birthwt_convergence_data()

  expect_error(
    check_convergence(list(y = 1), exposures = "x", outcome = "y"),
    "`data` must be a data frame"
  )
  expect_error(
    check_convergence(df, exposures = character(0), outcome = "low"),
    "`exposures` must be"
  )
  expect_error(
    check_convergence(df, exposures = "age", outcome = c("low", "bwt")),
    "`outcome` must be"
  )
  expect_error(
    check_convergence(df, exposures = "missing", outcome = "low"),
    "Variables not found: missing"
  )
  expect_error(
    check_convergence(df, exposures = "age", outcome = "missing"),
    "Variables not found: missing"
  )
  expect_error(
    check_convergence(df, exposures = "age", outcome = "low", multivariate = NA),
    "`multivariate` must be"
  )
  expect_error(
    check_convergence(df, exposures = "age", outcome = "low", approach = linear),
    "linear  is not a valid approach"
  )
  expect_error(
    check_convergence(df, exposures = "age", outcome = "bwt", approach = logit),
    "This approach requires either a factor variable"
  )
})

test_that("check_convergence handles empty data and fitting failures", {
  df <- birthwt_convergence_data()

  empty <- check_convergence(
    data = df[0, ],
    exposures = c("age", "smoke"),
    outcome = "low",
    approach = logit,
    format = tibble
  )
  expect_s3_class(empty, "data.frame")
  expect_equal(nrow(empty), 0L)
  expect_named(empty, c("Exposure", "Model", "Converged", "Max.prob."))

  broken_data <- data.frame(
    diabetes = c(0, 1, 0, 1),
    all_one = factor("yes", levels = "yes")
  )

  result <- check_convergence(
    broken_data,
    exposures = "all_one",
    outcome = "diabetes",
    approach = "logit",
    format = tibble
  )

  expect_false(result$Converged)
  expect_true(is.na(result$Max.prob.))

  result_multi <- suppressWarnings(check_convergence(
    broken_data,
    exposures = "all_one",
    outcome = "diabetes",
    approach = "logbinomial",
    multivariate = TRUE,
    format = tibble
  ))

  expect_false(result_multi$Converged)
  expect_equal(result_multi$Exposure, "all_one")
  expect_true(is.na(result_multi$Max.prob.))

  expect_s3_class(
    suppressWarnings(check_convergence(
      broken_data,
      exposures = "all_one",
      outcome = "diabetes",
      approach = "logbinomial",
      multivariate = TRUE
    )),
    "flextable"
  )
})

test_that("check_convergence warns for robpoisson fitted values above one", {
  df <- data.frame(
    outcome = c(0, 1, 1, 1, 1, 1),
    x = c(0, 1, 2, 3, 4, 5)
  )

  expect_warning(
    suppressMessages(check_convergence(
      data = df,
      exposures = "x",
      outcome = "outcome",
      approach = robpoisson,
      format = tibble
    )),
    "predicted fitted value exceeds 1"
  )
})

test_that("check_convergence can return gt and flextable outputs", {
  df <- birthwt_convergence_data()

  gt_out <- check_convergence(
    data = df,
    exposures = c("age", "smoke"),
    outcome = "low",
    approach = logit,
    format = gt
  )
  ft_out <- check_convergence(
    data = df,
    exposures = c("age", "smoke"),
    outcome = "low",
    approach = logit,
    format = flextable
  )

  expect_s3_class(gt_out, "gt_tbl")
  expect_s3_class(ft_out, "flextable")
  expect_equal(names(gt_out$`_data`), c("Exposure", "Model", "Converged", "Max.prob."))
  expect_match(
    as.character(gt_out$`_source_notes`[[1]]),
    "Screening aid only"
  )
  expect_error(
    check_convergence(df, "age", "low", format = "bad"),
    "should be one of"
  )
})

#' Validate a survival stratifier argument
#' @keywords internal
#' @noRd
.survival_optional_single_var_arg <- function(expr, data, env = parent.frame()) {
  if (is.null(expr) || identical(expr, quote(NULL))) {
    return(NULL)
  }

  .cox_single_var_arg(expr, data = data, env = env)
}

#' Validate a survival stratifier argument
#' @keywords internal
#' @noRd
.validate_survival_stratifier <- function(data,
                                          time,
                                          event,
                                          exposures,
                                          adjust_for = NULL,
                                          interaction = NULL,
                                          stratifier = NULL) {
  if (is.null(stratifier)) {
    return(invisible(NULL))
  }

  if (!is.character(stratifier) || length(stratifier) != 1L || !nzchar(stratifier)) {
    stop("`stratifier` must be a single variable.", call. = FALSE)
  }

  if (!stratifier %in% names(data)) {
    stop("`stratifier` was not found in `data`.", call. = FALSE)
  }

  interaction_vars <- .interaction_vars(interaction)
  blocked <- unique(c(time, event, exposures, adjust_for, interaction_vars))
  if (stratifier %in% blocked) {
    stop(
      "`stratifier` cannot also be used as `time`, `event`, an exposure, ",
      "an adjustment variable, or an interaction variable.",
      call. = FALSE
    )
  }

  invisible(NULL)
}

#' Run stratified Cox models and build a wide stratified table
#' @keywords internal
#' @noRd
.run_stratified_cox_reg <- function(data,
                                    time,
                                    event,
                                    exposures,
                                    adjust_for = NULL,
                                    stratifier,
                                    interaction = NULL,
                                    multivariable = FALSE,
                                    format = "flextable",
                                    theme = "minimal",
                                    show_sample = "events",
                                    model_stats = FALSE,
                                    show_ref = TRUE,
                                    fmt_class = "ft_cox") {
  .run_stratified_survival_reg(
    data = data,
    time = time,
    event = event,
    exposures = exposures,
    adjust_for = adjust_for,
    stratifier = stratifier,
    interaction = interaction,
    multivariable = multivariable,
    format = format,
    theme = theme,
    show_sample = show_sample,
    model_stats = model_stats,
    show_ref = show_ref,
    fmt_class = fmt_class,
    approach = "cox",
    source = "stratified_cox_reg",
    object_class = "cox_reg",
    run_core = .run_cox_core,
    stats_fun = .cox_model_stats_table,
    effect_label = if (!is.null(adjust_for) && length(adjust_for) > 0 || isTRUE(multivariable)) {
      "Adjusted HR (95% CI)"
    } else {
      "HR (95% CI)"
    },
    extra_footnotes = NULL
  )
}

#' Run stratified parametric survival models and build a wide stratified table
#' @keywords internal
#' @noRd
.run_stratified_surv_reg <- function(data,
                                     time,
                                     event,
                                     exposures,
                                     adjust_for = NULL,
                                     stratifier,
                                     interaction = NULL,
                                     multivariable = FALSE,
                                     distribution = "weibull",
                                     format = "flextable",
                                     theme = "minimal",
                                     show_sample = "events",
                                     model_stats = FALSE,
                                     show_ref = TRUE,
                                     fmt_class = "ft_surv") {
  run_core <- function(data, time, event, exposures, adjust_for, interaction, multivariable) {
    .run_surv_core(
      data = data,
      time = time,
      event = event,
      exposures = exposures,
      adjust_for = adjust_for,
      interaction = interaction,
      multivariable = multivariable,
      distribution = distribution
    )
  }

  .run_stratified_survival_reg(
    data = data,
    time = time,
    event = event,
    exposures = exposures,
    adjust_for = adjust_for,
    stratifier = stratifier,
    interaction = interaction,
    multivariable = multivariable,
    format = format,
    theme = theme,
    show_sample = show_sample,
    model_stats = model_stats,
    show_ref = show_ref,
    fmt_class = fmt_class,
    approach = "survreg",
    source = "stratified_surv_reg",
    object_class = "surv_reg",
    run_core = run_core,
    stats_fun = .surv_model_stats_table,
    effect_label = if (!is.null(adjust_for) && length(adjust_for) > 0 || isTRUE(multivariable)) {
      "Adjusted Time Ratio (95% CI)"
    } else {
      "Time Ratio (95% CI)"
    },
    extra_footnotes = paste0("Distribution: ", distribution, "."),
    distribution = distribution
  )
}

#' Shared stratified survival model runner
#' @keywords internal
#' @noRd
.run_stratified_survival_reg <- function(data,
                                         time,
                                         event,
                                         exposures,
                                         adjust_for = NULL,
                                         stratifier,
                                         interaction = NULL,
                                         multivariable = FALSE,
                                         format = "flextable",
                                         theme = "minimal",
                                         show_sample = "events",
                                         model_stats = FALSE,
                                         show_ref = TRUE,
                                         fmt_class,
                                         approach,
                                         source,
                                         object_class,
                                         run_core,
                                         stats_fun,
                                         effect_label,
                                         extra_footnotes = NULL,
                                         distribution = NULL) {
  message("Running stratified survival regression by: ", stratifier)

  levs <- .strata_levels(data, stratifier)
  if (!length(levs)) {
    stop("`stratifier` has no non-missing strata.", call. = FALSE)
  }

  adjusted_mode <- !is.null(adjust_for) && length(adjust_for) > 0
  variable_labels <- .var_label_map(
    data,
    unique(c(exposures, adjust_for, .interaction_vars(interaction)))
  )
  per_stratum <- list()
  tds <- list()
  models <- list()
  sums <- list()
  stats_by_stratum <- list()
  n_by_stratum <- list()

  for (lv in levs) {
    message("  > Stratum: ", stratifier, " = ", lv)

    dlev <- data[data[[stratifier]] == lv, , drop = FALSE]
    key <- as.character(lv)

    res_i <- tryCatch(
      run_core(
        data = dlev,
        time = time,
        event = event,
        exposures = exposures,
        adjust_for = adjust_for,
        interaction = interaction,
        multivariable = multivariable
      ),
      error = function(e) {
        warning("Skipping stratum ", lv, ": ", e$message, call. = FALSE)
        NULL
      }
    )

    if (is.null(res_i)) {
      next
    }

    res_i$table_body <- .format_interaction_levels(
      res_i$table_body, dlev, variable_labels
    )

    tds[[key]] <- res_i$table_body
    models[[key]] <- res_i$models
    sums[[key]] <- res_i$model_summaries
    stats_by_stratum[[key]] <- if (isTRUE(model_stats)) stats_fun(res_i$models) else NULL
    n_by_stratum[[key]] <- .stratified_survival_n_used(res_i$models)

    per_stratum[[key]] <- list(
      table_body = res_i$table_body,
      models = res_i$models,
      model_summaries = res_i$model_summaries,
      model_stats = stats_by_stratum[[key]],
      adjusted_mode = adjusted_mode,
      multivariable = isTRUE(multivariable)
    )
  }

  if (!length(tds)) {
    stop("No valid models across strata.", call. = FALSE)
  }

  if (adjusted_mode || isTRUE(multivariable)) {
    built <- .strata_build_wide_multi(
      data = data,
      exposures = exposures,
      stratifier = stratifier,
      td_by_stratum = tds,
      variable_labels = variable_labels,
      show_ref = show_ref
    )
    wide <- .strata_filter_survival_sample_cols(
      .strata_add_survival_counts(built$wide, models),
      show_sample = show_sample
    )
    spanners <- built$spanners
    tbl <- if (format == "gt") {
      .build_gt_strata_wide_multi(wide, spanners, effect_label, theme, .stratified_survival_footnotes(
        approach = approach,
        tds = tds,
        event = event,
        adjusted_mode = adjusted_mode,
        adjust_for = adjust_for,
        multivariable = multivariable,
        interaction = interaction,
        stratifier = stratifier,
        n_by_stratum = n_by_stratum,
        show_sample = show_sample,
        show_ref = show_ref,
        extra_footnotes = extra_footnotes
      ))
    } else {
      .build_flex_strata_wide_multi(wide, spanners, effect_label, theme, .stratified_survival_footnotes(
        approach = approach,
        tds = tds,
        event = event,
        adjusted_mode = adjusted_mode,
        adjust_for = adjust_for,
        multivariable = multivariable,
        interaction = interaction,
        stratifier = stratifier,
        n_by_stratum = n_by_stratum,
        show_sample = show_sample,
        show_ref = show_ref,
        extra_footnotes = extra_footnotes
      ))
    }
  } else {
    per_for_uni <- lapply(per_stratum, function(x) list(table_body = x$table_body))
    built <- .strata_build_wide_survival_uni(
      data = data,
      time = time,
      event = event,
      exposures = exposures,
      stratifier = stratifier,
      per_stratum = per_for_uni,
      variable_labels = variable_labels,
      show_ref = show_ref
    )
    wide <- .strata_filter_survival_sample_cols(built$wide, show_sample = show_sample)
    spanners <- built$spanners
    tbl <- if (format == "gt") {
      .build_gt_strata_wide_uni(wide, spanners, effect_label, theme, .stratified_survival_footnotes(
        approach = approach,
        tds = tds,
        event = event,
        adjusted_mode = adjusted_mode,
        adjust_for = adjust_for,
        multivariable = multivariable,
        interaction = interaction,
        stratifier = stratifier,
        n_by_stratum = n_by_stratum,
        show_sample = show_sample,
        show_ref = show_ref,
        extra_footnotes = extra_footnotes
      ))
    } else {
      .build_flex_strata_wide_uni(wide, spanners, effect_label, theme, .stratified_survival_footnotes(
        approach = approach,
        tds = tds,
        event = event,
        adjusted_mode = adjusted_mode,
        adjust_for = adjust_for,
        multivariable = multivariable,
        interaction = interaction,
        stratifier = stratifier,
        n_by_stratum = n_by_stratum,
        show_sample = show_sample,
        show_ref = show_ref,
        extra_footnotes = extra_footnotes
      ))
    }
  }

  .message_hidden_ref_rows(
    source,
    do.call(rbind, tds),
    show_ref
  )

  model_stats_out <- if (isTRUE(model_stats)) {
    .bind_stratified_model_stats(stats_by_stratum)
  } else {
    NULL
  }

  out <- list(
    table = tbl,
    table_display = wide,
    per_stratum = per_stratum,
    models = models,
    model_summaries = sums,
    model_stats = model_stats_out,
    variable_labels = variable_labels,
    time = time,
    event = event,
    distribution = distribution,
    by = stratifier,
    levels = names(tds),
    approach = approach,
    format = format,
    source = source,
    stratified = TRUE,
    show_sample = show_sample,
    adjusted_mode = adjusted_mode,
    multivariable = isTRUE(multivariable),
    adjust_for = if (adjusted_mode) unique(adjust_for) else NULL,
    exposures = unique(exposures),
    interaction = interaction,
    show_ref = isTRUE(show_ref)
  )

  class(out) <- c("gtregression", source, object_class, fmt_class, class(out))
  out
}

#' Pull crude stratified survival columns aligned to a skeleton
#' @keywords internal
#' @noRd
.strata_pull_cols_survival_uni <- function(dlev,
                                           uni_res,
                                           skeleton,
                                           is_factor,
                                           exposures,
                                           time,
                                           event) {
  td <- uni_res$table_body

  N_map <- vapply(exposures, function(x) {
    sum(stats::complete.cases(dlev[, c(time, event, x), drop = FALSE]))
  }, integer(1))
  names(N_map) <- exposures

  Events_map <- vapply(exposures, function(x) {
    complete <- stats::complete.cases(dlev[, c(time, event, x), drop = FALSE])
    if (!any(complete)) {
      return(0L)
    }
    sum(.cox_event01(dlev[[event]][complete]) == 1L, na.rm = TRUE)
  }, integer(1))
  names(Events_map) <- exposures

  fmt_est_ci <- function(est, lo, hi, digits = 2) {
    f <- function(z) formatC(z, digits = digits, format = "f", big.mark = ",")
    paste0(f(est), " (", f(lo), "\u2013", f(hi), ")")
  }

  N_vec <- character(nrow(skeleton))
  events_vec <- character(nrow(skeleton))
  eff_vec <- character(nrow(skeleton))
  p_vec <- character(nrow(skeleton))

  for (i in seq_len(nrow(skeleton))) {
    ex <- skeleton$exposure[i]
    lv <- skeleton$level[i]
    hdr <- skeleton$is_header[i]

    if (hdr) {
      N_vec[i] <- as.character(N_map[[ex]])
      events_vec[i] <- as.character(Events_map[[ex]])
      if (!is_factor[[ex]]) {
        row <- td[td$exposure == ex & td$level == ex, , drop = FALSE]
        if (nrow(row)) {
          eff_vec[i] <- fmt_est_ci(row$estimate, row$conf.low, row$conf.high)
          p_vec[i] <- .fmt_p(row$p.value)
        }
      }
    } else {
      row <- td[td$exposure == ex & td$level == lv, , drop = FALSE]
      if (nrow(row)) {
        if (isTRUE(row$ref[1])) {
          eff_vec[i] <- "Ref."
          p_vec[i] <- ""
        } else {
          eff_vec[i] <- fmt_est_ci(row$estimate, row$conf.low, row$conf.high)
          p_vec[i] <- .fmt_p(row$p.value)
        }
      }
    }
  }

  list(N = N_vec, Events = events_vec, effect = eff_vec, pval = p_vec)
}

#' Assemble wide display data for crude stratified survival models
#' @keywords internal
#' @noRd
.strata_build_wide_survival_uni <- function(data,
                                            time,
                                            event,
                                            exposures,
                                            stratifier,
                                            per_stratum,
                                            variable_labels = NULL,
                                            show_ref = TRUE) {
  sk <- .strata_build_skeleton(
    data,
    exposures,
    variable_labels = variable_labels,
    show_ref = show_ref
  )
  skeleton <- sk$skeleton
  is_factor <- sk$is_factor

  wide <- skeleton[, c("Characteristic", "is_header"), drop = FALSE]
  wide <- .attach_display_metadata(
    wide,
    row_exposure = skeleton$exposure,
    variable_labels = attr(skeleton, "variable_labels", exact = TRUE)
  )

  for (lev in names(per_stratum)) {
    dlev <- data[data[[stratifier]] == lev, , drop = FALSE]
    cols <- .strata_pull_cols_survival_uni(
      dlev = dlev,
      uni_res = per_stratum[[lev]],
      skeleton = skeleton,
      is_factor = is_factor,
      exposures = exposures,
      time = time,
      event = event
    )
    wide[[paste0("..N__", lev)]] <- cols$N
    wide[[paste0("..Events__", lev)]] <- cols$Events
    wide[[paste0("..eff__", lev)]] <- cols$effect
    wide[[paste0("..p__", lev)]] <- cols$pval
  }

  list(wide = wide, spanners = names(per_stratum))
}

#' Survival footnotes for stratified Cox and survreg output
#' @keywords internal
#' @noRd
.stratified_survival_footnotes <- function(approach,
                                           tds,
                                           event,
                                           adjusted_mode,
                                           adjust_for = NULL,
                                           multivariable = FALSE,
                                           interaction = NULL,
                                           stratifier,
                                           n_by_stratum,
                                           show_sample = "events",
                                           show_ref = TRUE,
                                           extra_footnotes = NULL) {
  c(
    .stratified_by_note(stratifier),
    .abbrev_note(approach),
    extra_footnotes,
    if (
      isTRUE(show_ref) &&
        any(unlist(lapply(tds, function(x) x$ref %in% TRUE), use.names = FALSE))
    ) .ref_note() else NULL,
    if (isTRUE(adjusted_mode)) .adjustment_note(adjust_for) else NULL,
    if (isTRUE(multivariable)) "Adjusted for the other variables in the model." else NULL,
    if (!is.null(interaction)) .interaction_note(interaction) else NULL,
    .stratified_survival_sample_note(stratifier, n_by_stratum, show_sample)
  )
}

#' Keep only requested sample columns in stratified survival display tables
#' @keywords internal
#' @noRd
.strata_filter_survival_sample_cols <- function(wide, show_sample = "events") {
  if (!show_sample %in% c("events", "n", "both", "none")) {
    stop("`show_sample` must be one of 'events', 'n', 'both', or 'none'.", call. = FALSE)
  }

  drop_prefix <- switch(
    show_sample,
    events = "..N__",
    n = "..Events__",
    both = character(0),
    none = c("..N__", "..Events__")
  )

  if (!length(drop_prefix)) {
    return(wide)
  }

  keep <- !Reduce(`|`, lapply(drop_prefix, function(prefix) startsWith(names(wide), prefix)))
  out <- wide[, keep, drop = FALSE]
  attr(out, "row_exposure") <- attr(wide, "row_exposure", exact = TRUE)
  attr(out, "variable_labels") <- attr(wide, "variable_labels", exact = TRUE)
  out
}

#' Footnote matching the displayed sample columns in stratified survival tables
#' @keywords internal
#' @noRd
.stratified_survival_sample_note <- function(stratifier, n_by_stratum, show_sample) {
  if (identical(show_sample, "none")) {
    return(NULL)
  }

  event_note <- "Events are event counts in each stratum-specific model."
  n_note <- .n_note_multi_strata(stratifier, n_by_stratum)

  switch(
    show_sample,
    events = event_note,
    n = n_note,
    both = c(n_note, event_note),
    NULL
  )
}

#' Extract a compact model N summary for stratified survival footnotes
#' @keywords internal
#' @noRd
.stratified_survival_n_used <- function(models) {
  n_vals <- vapply(models, function(fit) {
    tryCatch(as.numeric(stats::nobs(fit)), error = function(e) NA_real_)
  }, numeric(1))

  n_vals <- stats::na.omit(n_vals)
  if (!length(n_vals)) {
    return(NA_integer_)
  }

  if (length(unique(n_vals)) == 1L) {
    return(as.integer(n_vals[1]))
  }

  paste0(min(n_vals), "-", max(n_vals))
}

#' Add model N and event count columns to a stratified survival display table
#' @keywords internal
#' @noRd
.strata_add_survival_counts <- function(wide, models_by_stratum) {
  wide <- .strata_add_model_n(wide, models_by_stratum)

  if (!length(models_by_stratum) || !"is_header" %in% names(wide)) {
    return(wide)
  }

  row_exposure <- attr(wide, "row_exposure", exact = TRUE)
  if (is.null(row_exposure)) {
    return(wide)
  }

  is_header <- wide$is_header %in% TRUE

  for (lev in names(models_by_stratum)) {
    models <- models_by_stratum[[lev]]
    if (!length(models)) {
      next
    }

    events_vec <- character(nrow(wide))
    model_names <- names(models)
    has_single_model <- identical(model_names, "multivariable_model")

    for (i in seq_len(nrow(wide))) {
      if (!isTRUE(is_header[i])) {
        next
      }

      fit <- if (has_single_model) {
        models[["multivariable_model"]]
      } else {
        models[[row_exposure[i]]]
      }

      if (!is.null(fit)) {
        events_vec[i] <- .strata_survival_event_count(fit)
      }
    }

    wide[[paste0("..Events__", lev)]] <- events_vec
  }

  wide
}

#' Extract event counts from Cox or parametric survival fits
#' @keywords internal
#' @noRd
.strata_survival_event_count <- function(fit) {
  events <- NULL

  if (inherits(fit, "coxph") && !is.null(fit$nevent)) {
    events <- fit$nevent
  }

  if (is.null(events)) {
    events <- attr(fit, "gtregression_events", exact = TRUE)
  }

  if (is.null(events)) {
    y <- tryCatch(stats::model.response(stats::model.frame(fit)), error = function(e) NULL)
    if (inherits(y, "Surv")) {
      events <- sum(y[, ncol(y)] == 1L, na.rm = TRUE)
    }
  }

  if (is.null(events) || !is.finite(as.numeric(events))) {
    return("")
  }

  as.character(as.integer(events))
}

#' Bind model statistics with a stratum column
#' @keywords internal
#' @noRd
.bind_stratified_model_stats <- function(stats_by_stratum) {
  stats_by_stratum <- Filter(Negate(is.null), stats_by_stratum)
  if (!length(stats_by_stratum)) {
    return(NULL)
  }

  out <- Map(function(x, nm) {
    cbind(stratum = nm, x, stringsAsFactors = FALSE)
  }, stats_by_stratum, names(stats_by_stratum))

  do.call(rbind, out)
}

#' Plot model-fit diagnostics
#'
#' Visualise model fit for fitted regression models and models stored inside
#' \code{uni_reg()} or \code{multi_reg()} results.
#'
#' @param model A fitted \code{lm} or \code{glm} model, or a
#'   \code{uni_reg()} / \code{multi_reg()} result.
#' @param model_name Optional model name to select when \code{model} contains
#'   multiple fitted models. Quoted and bare names are accepted.
#' @param type Plot type. One of \code{"auto"}, \code{"all"},
#'   \code{"residual"}, \code{"qq"}, \code{"scale_location"},
#'   \code{"cooks"}, \code{"observed_predicted"}, or \code{"calibration"}.
#'   Quoted and bare values are accepted.
#' @param bins Number of groups used for binomial calibration plots.
#' @param base_size Base font size for the plot theme.
#'
#' @return A \code{ggplot2} object for a single plot, or a patchwork object when
#'   multiple diagnostics are requested.
#'
#' @details
#' \code{plot_model_fit()} is a visual check of how a fitted model behaves, not
#' a formal model-selection rule. For survival models, use \code{check_ph()} for Cox
#' proportional hazards diagnostics and \code{plot_surv_fit()} for parametric
#' survival model fit.
#'
#' For binomial models, \code{type = "calibration"} compares grouped predicted
#' probabilities with observed event proportions. This is most informative for
#' multivariable models, where predictions vary across many patients. A
#' univariable binary predictor may produce only two calibration points; that is
#' expected and simply reflects the two fitted probabilities in the model.
#' Logistic residual plots often show two bands because the outcome is coded as
#' event/non-event.
#'
#' @examples
#' fit_lm <- lm(mpg ~ wt + hp, data = mtcars)
#' plot_model_fit(fit_lm)
#'
#' fit_glm <- glm(am ~ mpg + wt, data = mtcars, family = binomial())
#' plot_model_fit(fit_glm, type = calibration, bins = 4)
#'
#' uni_fit <- uni_reg(mtcars, am, c(mpg, wt), approach = logit)
#' plot_model_fit(uni_fit, model_name = mpg, type = residual)
#'
#' @export
plot_model_fit <- function(model,
                           model_name = NULL,
                           type = c(
                             "auto",
                             "all",
                             "residual",
                             "qq",
                             "scale_location",
                             "cooks",
                             "observed_predicted",
                             "calibration"
                           ),
                           bins = 10,
                           base_size = 13) {
  choices <- c(
    "auto",
    "all",
    "residual",
    "qq",
    "scale_location",
    "cooks",
    "observed_predicted",
    "calibration"
  )
  type <- .choice_arg(substitute(type), env = parent.frame(), choices = choices)
  type <- match.arg(type, choices)
  model_name <- .plot_model_fit_model_name_arg(
    substitute(model_name),
    env = parent.frame()
  )

  if (!is.numeric(bins) || length(bins) != 1L || is.na(bins) || bins < 2) {
    stop("`bins` must be a single number greater than or equal to 2.", call. = FALSE)
  }
  bins <- as.integer(bins)

  if (!is.numeric(base_size) || length(base_size) != 1L || is.na(base_size) ||
      base_size <= 0) {
    stop("`base_size` must be a single positive number.", call. = FALSE)
  }

  selected <- .plot_model_fit_extract_model(model, model_name)
  fit <- selected$fit

  if (.plot_model_fit_is_survival_result(model) ||
      inherits(fit, c("coxph", "survreg"))) {
    stop(
      "Survival model fit is handled by `check_ph()` and `plot_surv_fit()`.",
      call. = FALSE
    )
  }

  if (!.plot_model_fit_supported(fit)) {
    stop(
      "`plot_model_fit()` currently supports `lm` and `glm` compatible models.",
      call. = FALSE
    )
  }

  data <- .plot_model_fit_data(fit)
  model_label <- selected$name

  plot_type <- type
  if (identical(plot_type, "auto")) {
    plot_type <- "all"
  }

  if (identical(plot_type, "all")) {
    plot_types <- .plot_model_fit_default_types(fit)
    plots <- lapply(
      plot_types,
      .plot_model_fit_single,
      data = data,
      fit = fit,
      model_label = model_label,
      bins = bins,
      base_size = base_size
    )
    out <- patchwork::wrap_plots(plots, ncol = 2)
  } else {
    out <- .plot_model_fit_single(
      plot_type,
      data = data,
      fit = fit,
      model_label = model_label,
      bins = bins,
      base_size = base_size
    )
  }

  attr(out, "model_name") <- model_label
  attr(out, "source") <- "plot_model_fit"
  out
}

.plot_model_fit_model_name_arg <- function(expr, env = parent.frame()) {
  if (identical(expr, quote(NULL))) {
    return(NULL)
  }

  if (is.symbol(expr)) {
    name <- as.character(expr)
    if (exists(name, envir = env, inherits = TRUE)) {
      value <- get(name, envir = env, inherits = TRUE)
      if (is.character(value) && length(value) == 1L) {
        return(value)
      }
    }
    return(name)
  }

  value <- eval(expr, envir = env)
  if (is.null(value)) {
    return(NULL)
  }
  if (!is.character(value) || length(value) != 1L || is.na(value)) {
    stop("`model_name` must be a single model name.", call. = FALSE)
  }
  value
}

.plot_model_fit_extract_model <- function(model, model_name = NULL) {
  if (inherits(model, c("glm", "lm"))) {
    return(list(
      fit = model,
      name = .plot_model_fit_formula_label(model)
    ))
  }

  if (!inherits(model, "gtregression")) {
    stop(
      "`model` must be a fitted model or a gtregression result containing fitted models.",
      call. = FALSE
    )
  }

  models <- model[["models"]]
  if (!is.list(models) || !length(models)) {
    stop("No fitted models were found in `model$models`.", call. = FALSE)
  }

  model_names <- names(models)
  if (is.null(model_names)) {
    model_names <- paste0("model_", seq_along(models))
  }

  if (is.null(model_name)) {
    selected <- model_names[[1L]]
    if (length(models) > 1L) {
      message(
        "Multiple fitted models found; plotting `", selected,
        "`. Use `model_name = ...` to choose another model."
      )
    }
  } else {
    selected <- model_name
  }

  idx <- match(selected, model_names)
  if (is.na(idx)) {
    stop(
      "`model_name` must match one of: ",
      paste(model_names, collapse = ", "),
      call. = FALSE
    )
  }

  list(fit = models[[idx]], name = selected)
}

.plot_model_fit_formula_label <- function(fit) {
  f <- stats::formula(fit)
  paste(deparse(f), collapse = " ")
}

.plot_model_fit_is_survival_result <- function(model) {
  inherits(model, c("cox_reg", "surv_reg", "surv_model_compare"))
}

.plot_model_fit_supported <- function(fit) {
  inherits(fit, c("glm", "lm"))
}

.plot_model_fit_default_types <- function(fit) {
  if (inherits(fit, "glm") && .plot_model_fit_is_binomial(fit)) {
    return(c("calibration", "residual", "cooks"))
  }
  if (inherits(fit, "glm")) {
    return(c("observed_predicted", "residual", "cooks"))
  }
  c("residual", "qq", "scale_location", "cooks")
}

.plot_model_fit_data <- function(fit) {
  mf <- stats::model.frame(fit)
  observed <- stats::model.response(mf)
  if (is.factor(observed)) {
    observed <- as.numeric(observed) - 1
  }
  if (is.logical(observed)) {
    observed <- as.numeric(observed)
  }

  residual_type <- if (inherits(fit, "glm")) "deviance" else "response"
  std_resid <- tryCatch(stats::rstandard(fit), error = function(e) NA_real_)

  data.frame(
    observed = as.numeric(observed),
    fitted = as.numeric(stats::fitted(fit)),
    residual = as.numeric(stats::residuals(fit, type = residual_type)),
    std_resid = as.numeric(std_resid),
    cooks = as.numeric(stats::cooks.distance(fit)),
    index = seq_along(stats::fitted(fit))
  )
}

.plot_model_fit_is_binomial <- function(fit) {
  inherits(fit, "glm") &&
    !is.null(fit$family$family) &&
    identical(fit$family$family, "binomial")
}

.plot_model_fit_single <- function(type,
                                   data,
                                   fit,
                                   model_label,
                                   bins,
                                   base_size) {
  switch(
    type,
    residual = .plot_model_fit_residual(data, model_label, base_size),
    qq = .plot_model_fit_qq(data, model_label, base_size),
    scale_location = .plot_model_fit_scale_location(data, model_label, base_size),
    cooks = .plot_model_fit_cooks(data, model_label, base_size),
    observed_predicted = .plot_model_fit_observed_predicted(data, model_label, base_size),
    calibration = .plot_model_fit_calibration(data, fit, model_label, bins, base_size),
    stop("Unsupported plot type.", call. = FALSE)
  )
}

.plot_model_fit_theme <- function(base_size) {
  ggplot2::theme_minimal(base_size = base_size) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold"),
      panel.grid.minor = ggplot2::element_blank()
    )
}

.plot_model_fit_residual <- function(data, model_label, base_size) {
  ggplot2::ggplot(data, ggplot2::aes(x = .data$fitted, y = .data$residual)) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", colour = "grey45") +
    ggplot2::geom_point(alpha = 0.75, colour = "#1F77B4") +
    ggplot2::labs(
      title = "Residuals vs fitted",
      subtitle = model_label,
      x = "Fitted values",
      y = "Residuals"
    ) +
    .plot_model_fit_theme(base_size)
}

.plot_model_fit_qq <- function(data, model_label, base_size) {
  ggplot2::ggplot(data, ggplot2::aes(sample = .data$residual)) +
    ggplot2::stat_qq(alpha = 0.75, colour = "#1F77B4") +
    ggplot2::stat_qq_line(colour = "grey35") +
    ggplot2::labs(
      title = "Normal Q-Q plot",
      subtitle = model_label,
      x = "Theoretical quantiles",
      y = "Sample quantiles"
    ) +
    .plot_model_fit_theme(base_size)
}

.plot_model_fit_scale_location <- function(data, model_label, base_size) {
  data$sqrt_abs_std_resid <- sqrt(abs(data$std_resid))
  ggplot2::ggplot(data, ggplot2::aes(x = .data$fitted, y = .data$sqrt_abs_std_resid)) +
    ggplot2::geom_point(alpha = 0.75, colour = "#1F77B4") +
    ggplot2::labs(
      title = "Scale-location",
      subtitle = model_label,
      x = "Fitted values",
      y = "sqrt(|standardised residuals|)"
    ) +
    .plot_model_fit_theme(base_size)
}

.plot_model_fit_cooks <- function(data, model_label, base_size) {
  ggplot2::ggplot(data, ggplot2::aes(x = .data$index, y = .data$cooks)) +
    ggplot2::geom_col(fill = "#4C78A8", alpha = 0.85) +
    ggplot2::labs(
      title = "Cook's distance",
      subtitle = model_label,
      x = "Observation",
      y = "Cook's distance"
    ) +
    .plot_model_fit_theme(base_size)
}

.plot_model_fit_observed_predicted <- function(data, model_label, base_size) {
  ggplot2::ggplot(data, ggplot2::aes(x = .data$fitted, y = .data$observed)) +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", colour = "grey45") +
    ggplot2::geom_point(alpha = 0.75, colour = "#1F77B4") +
    ggplot2::labs(
      title = "Observed vs predicted",
      subtitle = model_label,
      x = "Predicted values",
      y = "Observed values"
    ) +
    .plot_model_fit_theme(base_size)
}

.plot_model_fit_calibration <- function(data, fit, model_label, bins, base_size) {
  if (!.plot_model_fit_is_binomial(fit)) {
    stop("`type = \"calibration\"` is available only for binomial `glm` models.", call. = FALSE)
  }

  unique_fitted <- sort(unique(data$fitted[is.finite(data$fitted)]))
  if (length(unique_fitted) < 2L) {
    stop("Predicted probabilities do not vary enough for a calibration plot.", call. = FALSE)
  }

  if (length(unique_fitted) <= bins) {
    data$bin <- factor(
      match(data$fitted, unique_fitted),
      levels = seq_along(unique_fitted)
    )
  } else {
    probs <- stats::quantile(
      data$fitted,
      probs = seq(0, 1, length.out = bins + 1L),
      na.rm = TRUE,
      names = FALSE
    )
    probs <- unique(probs)
    if (length(probs) < 3L) {
      data$bin <- factor(match(data$fitted, unique_fitted))
    } else {
      data$bin <- cut(data$fitted, breaks = probs, include.lowest = TRUE)
    }
  }
  cal <- stats::aggregate(
    cbind(predicted = data$fitted, observed = data$observed),
    by = list(bin = data$bin),
    FUN = mean,
    na.rm = TRUE
  )

  ggplot2::ggplot(cal, ggplot2::aes(x = .data$predicted, y = .data$observed)) +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", colour = "grey45") +
    ggplot2::geom_line(colour = "#1F77B4") +
    ggplot2::geom_point(size = 2.4, colour = "#1F77B4") +
    ggplot2::coord_cartesian(xlim = c(0, 1), ylim = c(0, 1)) +
    ggplot2::labs(
      title = "Calibration plot",
      subtitle = model_label,
      x = "Mean predicted probability",
      y = "Observed event proportion"
    ) +
    .plot_model_fit_theme(base_size)
}

#' Extract model-fit statistics from fitted models
#'
#' @keywords internal
#' @noRd
.model_stats_table <- function(models, approach) {
  if (is.null(models) || !length(models)) {
    return(NULL)
  }

  out <- lapply(names(models), function(model_name) {
    .model_stats_one(models[[model_name]], model_name = model_name, approach = approach)
  })

  out <- Filter(Negate(is.null), out)
  if (!length(out)) {
    return(NULL)
  }

  do.call(rbind, out)
}

#' Extract model-fit statistics from one fitted model
#'
#' @keywords internal
#' @noRd
.model_stats_one <- function(model, model_name, approach) {
  data.frame(
    model = model_name,
    AIC = .safe_numeric(stats::AIC(model)),
    BIC = .safe_numeric(stats::BIC(model)),
    logLik = .safe_numeric(as.numeric(stats::logLik(model))),
    deviance = .safe_numeric(stats::deviance(model)),
    null_deviance = .safe_null_deviance(model),
    pseudo_r2 = .safe_pseudo_r2(model, approach),
    r_squared = .safe_r_squared(model),
    adj_r_squared = .safe_adj_r_squared(model),
    n = .safe_numeric(stats::nobs(model)),
    stringsAsFactors = FALSE
  )
}

#' Safely coerce a scalar statistic to numeric
#'
#' @keywords internal
#' @noRd
.safe_numeric <- function(expr) {
  out <- tryCatch(expr, error = function(e) NA_real_)
  out <- suppressWarnings(as.numeric(out))
  if (!length(out) || is.na(out[1])) {
    return(NA_real_)
  }
  out[1]
}

#' Safely extract null deviance
#'
#' @keywords internal
#' @noRd
.safe_null_deviance <- function(model) {
  out <- tryCatch(model$null.deviance, error = function(e) NA_real_)
  out <- suppressWarnings(as.numeric(out))
  if (!length(out) || is.na(out[1])) {
    return(NA_real_)
  }
  out[1]
}

#' Safely calculate McFadden pseudo R-squared for non-linear models
#'
#' @keywords internal
#' @noRd
.safe_pseudo_r2 <- function(model, approach) {
  if (identical(approach, "linear")) {
    return(NA_real_)
  }

  ll_model <- .safe_numeric(as.numeric(stats::logLik(model)))
  if (!is.finite(ll_model)) {
    return(NA_real_)
  }

  ll_null <- tryCatch({
    null_model <- NULL
    utils::capture.output(
      null_model <- stats::update(model, stats::as.formula(". ~ 1"))
    )
    as.numeric(stats::logLik(null_model))
  }, error = function(e) NA_real_)
  ll_null <- .safe_numeric(ll_null)

  if (!is.finite(ll_null) || isTRUE(all.equal(ll_null, 0))) {
    return(.safe_deviance_pseudo_r2(model))
  }

  out <- 1 - (ll_model / ll_null)
  if (!is.finite(out)) {
    return(.safe_deviance_pseudo_r2(model))
  }
  out
}

#' Safely calculate deviance-based pseudo R-squared
#'
#' @keywords internal
#' @noRd
.safe_deviance_pseudo_r2 <- function(model) {
  dev <- .safe_numeric(stats::deviance(model))
  null_dev <- .safe_null_deviance(model)

  if (!is.finite(dev) || !is.finite(null_dev) || isTRUE(all.equal(null_dev, 0))) {
    return(NA_real_)
  }

  out <- 1 - (dev / null_dev)
  if (!is.finite(out)) {
    return(NA_real_)
  }
  out
}

#' Safely extract R-squared for linear models
#'
#' @keywords internal
#' @noRd
.safe_r_squared <- function(model) {
  if (!inherits(model, "lm")) {
    return(NA_real_)
  }
  out <- tryCatch(summary(model)$r.squared, error = function(e) NA_real_)
  out <- suppressWarnings(as.numeric(out))
  if (!length(out) || is.na(out[1])) {
    return(NA_real_)
  }
  out[1]
}

#' Safely extract adjusted R-squared for linear models
#'
#' @keywords internal
#' @noRd
.safe_adj_r_squared <- function(model) {
  if (!inherits(model, "lm")) {
    return(NA_real_)
  }
  out <- tryCatch(summary(model)$adj.r.squared, error = function(e) NA_real_)
  out <- suppressWarnings(as.numeric(out))
  if (!length(out) || is.na(out[1])) {
    return(NA_real_)
  }
  out[1]
}

#' Prepare linear-regression diagnostics for console display
#'
#' @keywords internal
#' @noRd
.as_reg_check_result <- function(x, format = c("flextable", "gt")) {
  has_diagnostics <- function(item) {
    is.data.frame(item) || (
      is.list(item) && any(vapply(item, has_diagnostics, logical(1), USE.NAMES = FALSE))
    )
  }

  if (!is.list(x) || !length(x) || !has_diagnostics(x)) {
    return(x)
  }

  attr(x, "reg_check_format") <- match.arg(format)
  class(x) <- unique(c("gtregression_reg_check", class(x)))
  x
}

#' Combine nested linear-regression diagnostics into a display data frame
#'
#' @keywords internal
#' @noRd
.reg_check_display_data <- function(x) {
  rows <- list()

  collect <- function(item, path = character(0)) {
    if (is.data.frame(item)) {
      out <- item
      if (length(path) >= 2L) {
        out$Stratum <- path[[1L]]
      }
      rows[[length(rows) + 1L]] <<- out
      return(invisible(NULL))
    }

    if (is.list(item)) {
      nms <- names(item)
      if (is.null(nms)) nms <- rep("", length(item))
      for (i in seq_along(item)) {
        collect(item[[i]], c(path, nms[[i]]))
      }
    }
    invisible(NULL)
  }

  collect(unclass(x))
  if (!length(rows)) return(NULL)

  out <- do.call(rbind, rows)
  rownames(out) <- NULL
  preferred <- c("Stratum", "Exposure", "Test", "Statistic", "Interpretation")
  out[, c(intersect(preferred, names(out)), setdiff(names(out), preferred)), drop = FALSE]
}

#' Print linear-regression diagnostics as a publication-ready table
#'
#' @param x A diagnostic list stored in a gtregression regression result.
#' @param ... Unused.
#'
#' @return The diagnostic object, invisibly.
#' @keywords internal
#' @export
print.gtregression_reg_check <- function(x, ...) {
  display <- .reg_check_display_data(x)
  if (is.null(display) || !nrow(display)) {
    print(unclass(x))
    return(invisible(x))
  }

  format <- attr(x, "reg_check_format", exact = TRUE)
  if (identical(format, "gt")) {
    out <- gt::gt(display) |>
      gt::tab_header(title = "Linear regression diagnostics") |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_column_labels()
      ) |>
      gt::tab_source_note(
        paste0(
          "Diagnostics are screening checks and should be interpreted alongside ",
          "residual plots and study context."
        )
      ) |>
      .compact_gt_source_notes()
    out <- gt::cols_align(
      out,
      align = "left",
      columns = intersect(c("Stratum", "Exposure", "Test", "Interpretation"), names(display))
    )
    if ("Statistic" %in% names(display)) {
      out <- gt::cols_align(out, align = "center", columns = "Statistic")
    }
  } else {
    out <- flextable::flextable(display)
    out <- flextable::set_caption(out, caption = "Linear regression diagnostics")
    out <- flextable::bold(out, part = "header", bold = TRUE)
    out <- flextable::align(
      out,
      j = intersect(c("Stratum", "Exposure", "Test", "Interpretation"), names(display)),
      align = "left",
      part = "all"
    )
    if ("Statistic" %in% names(display)) {
      out <- flextable::align(out, j = "Statistic", align = "center", part = "all")
    }
    out <- flextable::add_footer_lines(
      out,
      values = "Diagnostics are screening checks and should be interpreted alongside residual plots and study context."
    )
    out <- .compact_flex_footer(out)
    out <- flextable::italic(out, italic = TRUE, part = "footer")
    out <- flextable::autofit(out)
  }

  print(out)
  invisible(x)
}

#' Fit multivariable regression model (Internal)
#'
#' Fits a regression model based on the selected approach.
#'
#' @param data A data.frame containing the analysis variables.
#' @param outcome A character scalar giving the outcome variable name.
#' @param exposures A character vector of exposure variable(s).
#' @param approach A regression approach. One of "logit", "firth", "logbinomial",
#'   "poisson", "linear", "robpoisson", or "negbin".
#' @param adjust_for Optional character vector of adjustment variables.
#' @param interaction Optional character scalar specifying an interaction term,
#'   e.g. "bmi*sex".
#'
#' @return A fitted model object or `NULL` if fitting fails.
#' @keywords internal
#' @noRd
.fit_multi_model <- function(data,
                             outcome,
                             exposures,
                             approach,
                             adjust_for = NULL,
                             interaction = NULL) {
  approach <- .normalize_approach(approach)

  rhs_terms <- unique(c(exposures, adjust_for))

  if (!is.null(interaction) && length(interaction) > 0) {
    rhs_terms <- unique(c(rhs_terms, interaction))
  }

  formula_str <- paste(outcome, "~", paste(rhs_terms, collapse = " + "))
  formula <- stats::as.formula(formula_str)

  tryCatch(
    {
      fit <- switch(
        approach,
        "logit" = stats::glm(
          formula = formula,
          data = data,
          family = stats::binomial("logit")
        ),
        "firth" = logistf::logistf(
          formula = formula,
          data = data
        ),
        "logbinomial" = stats::glm(
          formula = formula,
          data = data,
          family = stats::binomial("log")
        ),
        "poisson" = stats::glm(
          formula = formula,
          data = data,
          family = stats::poisson("log")
        ),
        "linear" = stats::lm(
          formula = formula,
          data = data
        ),
        "robpoisson" = risks::riskratio(
          formula = formula,
          data = data,
          approach = "robpoisson"
        ),
        "negbin" = MASS::glm.nb(
          formula = formula,
          data = data,
          control = stats::glm.control(maxit = 200)
        ),
        stop(
          paste0(
            "Invalid approach: '", approach,
            "'. Choose from: logit, firth, logbinomial, poisson, ",
            "robpoisson, linear, negbin."
          ),
          call. = FALSE
        )
      )
      attr(fit, "gtregression_model_frame") <- stats::model.frame(formula, data = data)
      fit
    },
    error = function(e) {
      warning(
        paste0(
          "Model fitting failed for exposure(s) ",
          paste(exposures, collapse = ", "),
          " using approach '", approach, "': ",
          e$message
        ),
        call. = FALSE
      )
      NULL
    }
  )
}

test_that(".fit_multi_model returns correct model class for each approach", {
  skip_if_not_installed("risks")

  df <- data_SynthDiabetes |>
    dplyr::mutate(
      diabetes = ifelse(diabetes == "pos", 1, 0),
      age_cat = dplyr::case_when(
        age < 30 ~ "Young",
        age >= 30 & age < 50 ~ "Middle-aged",
        age >= 50 ~ "Older"
      ),
      age_cat = factor(age_cat, levels = c("Young", "Middle-aged", "Older")),
      bmi_cat = dplyr::case_when(
        mass < 25 ~ "Normal",
        mass >= 25 & mass < 30 ~ "Overweight",
        mass >= 30 ~ "Obese"
      ),
      bmi_cat = factor(bmi_cat, levels = c("Normal", "Overweight", "Obese"))
    ) |>
    dplyr::filter(!is.na(diabetes), !is.na(age_cat), !is.na(bmi_cat), !is.na(mass), !is.na(glucose))

  exposures <- c("age_cat", "bmi_cat")

  # logit
  m_logit <- .fit_multi_model(df, outcome = "diabetes", exposures = exposures, approach = "logit")
  expect_s3_class(m_logit, "glm")
  expect_equal(family(m_logit)$family, "binomial")

  # logbinomial
  m_logbin <- .fit_multi_model(df, outcome = "diabetes", exposures = exposures, approach = "logbinomial")
  expect_s3_class(m_logbin, "glm")
  expect_equal(family(m_logbin)$link, "log")

  # poisson (using count outcome)
  df$count_outcome <- round(df$glucose / 10)
  m_pois <- .fit_multi_model(df, outcome = "count_outcome", exposures = exposures, approach = "poisson")
  expect_s3_class(m_pois, "glm")
  expect_equal(family(m_pois)$family, "poisson")

  # linear
  m_lm <- .fit_multi_model(df, outcome = "mass", exposures = exposures, approach = "linear")
  expect_s3_class(m_lm, "lm")

  # robpoisson
  m_rob <- .fit_multi_model(df, outcome = "diabetes", exposures = exposures, approach = "robpoisson")
  expect_s3_class(m_rob, "risks")
  expect_true("robpoisson" %in% class(m_rob))

  # negbin
  df <- df[complete.cases(df[, c("glucose", exposures)]), ]

  m_negbin <- .fit_multi_model(df, outcome = "glucose",
                               exposures = exposures, approach = "negbin")
  expect_s3_class(m_negbin, "glm")
})

test_that(".fit_multi_model returns NULL and warns on failure", {
  df_fail <- data.frame(
    outcome = c(1, 0, 1, 0),
    x1 = factor("only", levels = "only"),  # one-level factor
    x2 = factor("only", levels = "only")
  )

  expect_warning({
    model <- .fit_multi_model(df_fail, outcome = "outcome", exposures = c("x1", "x2"), approach = "logit")
    expect_null(model)
  }, regexp = "Model fitting failed")
})

test_that(".fit_multi_model supports Firth logistic regression when logistf is available", {
  skip_if_not_installed("logistf")

  df <- data_endometrial
  df$HG <- factor(df$HG, levels = c(0, 1))
  df$NV <- factor(df$NV, levels = c(0, 1))

  model <- .fit_multi_model(df, outcome = "HG", exposures = c("NV", "PI", "EH"), approach = "firth")

  expect_s3_class(model, "logistf")
  expect_true("gtregression_model_frame" %in% names(attributes(model)))
})

#' Compare parametric survival model distributions
#'
#' Fit the same parametric survival regression model using multiple
#' distributions and compare model-fit statistics such as AIC and BIC.
#'
#' @param data A \code{data.frame} containing survival time, event status, and
#'   predictor variables.
#' @param time Survival follow-up time. Quoted and bare names are accepted.
#' @param event Event indicator. Quoted and bare names are accepted. Numeric
#'   \code{0/1}, numeric \code{1/2}, logical, character, and factor variables
#'   are accepted. For two-level character or factor variables, the second level
#'   is treated as the event.
#' @param exposures Character vector of main exposure variable names. Quoted
#'   names are recommended in scripts, and bare names are also accepted.
#' @param adjust_for Optional character vector of adjustment variables. These
#'   variables are included in every candidate model.
#' @param distributions Parametric survival distributions to compare. Defaults
#'   to \code{c("weibull", "exponential", "lognormal", "loglogistic")}.
#'   Quoted and bare values are accepted. Common spellings such as
#'   \code{"log-normal"} and \code{"log-logistic"} are also accepted.
#' @param digits Number of digits for displayed model statistics.
#' @param format Output format. One of \code{"flextable"} (default),
#'   \code{"gt"}, or \code{"tibble"}.
#' @param theme Table styling preset.
#'
#' @return A list of class \code{c("gtregression","surv_model_compare", ...)}
#' with elements:
#' \describe{
#'   \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
#'   when \code{format = "tibble"}.}
#'   \item{\code{table_body}}{Tibble with model-fit statistics.}
#'   \item{\code{table_display}}{Display data frame used to render the table.}
#'   \item{\code{models}}{Named list of fitted \code{survreg} models.}
#'   \item{\code{time,event,exposures,adjust_for,distributions,format,source}}{Metadata fields.}
#' }
#'
#' @details
#' The same model formula is fitted for each candidate distribution using
#' \code{survival::survreg()}. Lower AIC or BIC values indicate better relative
#' model fit among the compared distributions. These statistics should be used
#' with clinical judgment and visual checks; they do not prove that a
#' distribution is scientifically correct.
#'
#' @examples
#' lung_data <- data_lungcancer
#' lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
#'                         labels = c("Standard", "Test"))
#' lung_data$prior <- factor(lung_data$prior, levels = c(0, 10),
#'                           labels = c("No", "Yes"))
#'
#' surv_model_compare(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposures = c("trt", "celltype", "prior"),
#'   adjust_for = c("age", "karno")
#' )
#'
#' surv_model_compare(
#'   data = lung_data,
#'   time = "time",
#'   event = "status",
#'   exposures = c(trt, prior),
#'   distributions = c(weibull, "log-logistic"),
#'   format = tibble
#' )
#'
#' @importFrom survival survreg
#' @export
surv_model_compare <- function(data,
                               time,
                               event,
                               exposures,
                               adjust_for = NULL,
                               distributions = c("weibull", "exponential", "lognormal", "loglogistic"),
                               digits = 2,
                               format = c("flextable", "gt", "tibble"),
                               theme = c("minimal")) {

  time <- .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  event <- .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  exposures <- .vars_arg(substitute(exposures), env = parent.frame())
  adjust_for <- .vars_arg(substitute(adjust_for), env = parent.frame(), allow_null = TRUE)
  distributions <- .surv_distribution_arg(
    substitute(distributions),
    env = parent.frame(),
    multiple = TRUE,
    arg = "distributions"
  )
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  theme <- .choice_arg(substitute(theme), env = parent.frame())

  format <- match.arg(format, c("flextable", "gt", "tibble"))
  theme <- .resolve_theme(theme)

  data_clean <- .validate_surv_model_compare_inputs(
    data = data,
    time = time,
    event = event,
    exposures = exposures,
    adjust_for = adjust_for,
    distributions = distributions,
    digits = digits
  )

  predictors <- unique(c(exposures, adjust_for))
  models <- vector("list", length(distributions))
  names(models) <- distributions

  for (dist in distributions) {
    fit <- .fit_surv_model(data_clean, time, event, predictors, dist)
    if (is.null(fit)) {
      stop("Parametric survival model fitting failed for distribution '", dist, "'.", call. = FALSE)
    }
    models[[dist]] <- fit
  }

  table_body <- .surv_model_compare_body(models)

  if (format == "tibble") {
    return(tibble::as_tibble(table_body))
  }

  table_display <- .surv_model_compare_display(table_body, digits = digits)
  tbl <- .build_surv_model_compare_table(
    table_display,
    format = format,
    theme = theme
  )

  res <- list(
    table = tbl,
    table_body = tibble::as_tibble(table_body),
    table_display = table_display,
    models = models,
    time = time,
    event = event,
    exposures = unique(exposures),
    adjust_for = if (!is.null(adjust_for) && length(adjust_for)) unique(adjust_for) else NULL,
    distributions = distributions,
    format = format,
    source = "surv_model_compare"
  )

  class(res) <- c("gtregression", "surv_model_compare", class(res))
  res
}

#' @keywords internal
#' @noRd
.validate_surv_model_compare_inputs <- function(data,
                                                time,
                                                event,
                                                exposures,
                                                adjust_for,
                                                distributions,
                                                digits) {
  choices <- c("weibull", "exponential", "lognormal", "loglogistic")
  if (!is.character(distributions) || length(distributions) < 1L ||
      anyNA(distributions) || any(!distributions %in% choices)) {
    stop(
      "`distributions` must contain one or more of: ",
      paste(choices, collapse = ", "),
      call. = FALSE
    )
  }
  if (!is.numeric(digits) || length(digits) != 1L || is.na(digits) ||
      digits < 0 || digits != floor(digits)) {
    stop("`digits` must be a non-negative whole number.", call. = FALSE)
  }

  .validate_cox_inputs(
    data = data,
    time = time,
    event = event,
    exposures = unique(exposures),
    adjust_for = adjust_for
  )
}

#' @keywords internal
#' @noRd
.surv_model_compare_body <- function(models) {
  out <- lapply(names(models), function(dist) {
    fit <- models[[dist]]
    data.frame(
      Distribution = dist,
      AIC = .safe_numeric(stats::AIC(fit)),
      BIC = .safe_numeric(stats::BIC(fit)),
      logLik = .safe_numeric(as.numeric(stats::logLik(fit))),
      Scale = .safe_numeric(fit$scale),
      N = .safe_numeric(stats::nobs(fit)),
      Events = .safe_numeric(attr(fit, "gtregression_events", exact = TRUE)),
      stringsAsFactors = FALSE
    )
  })

  out <- do.call(rbind, out)
  out$Best.AIC <- out$AIC == min(out$AIC, na.rm = TRUE)
  out$Best.BIC <- out$BIC == min(out$BIC, na.rm = TRUE)
  out <- out[order(out$AIC), , drop = FALSE]
  rownames(out) <- NULL
  out
}

#' @keywords internal
#' @noRd
.surv_model_compare_display <- function(table_body, digits = 2) {
  fmt_num <- function(x) {
    ifelse(
      is.na(x),
      "NA",
      formatC(x, digits = digits, format = "f", big.mark = ",")
    )
  }

  data.frame(
    Distribution = table_body$Distribution,
    AIC = fmt_num(table_body$AIC),
    BIC = fmt_num(table_body$BIC),
    `Log-likelihood` = fmt_num(table_body$logLik),
    Scale = fmt_num(table_body$Scale),
    N = as.character(table_body$N),
    Events = as.character(table_body$Events),
    `Best AIC` = ifelse(table_body$Best.AIC, "Yes", "No"),
    `Best BIC` = ifelse(table_body$Best.BIC, "Yes", "No"),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )
}

#' @keywords internal
#' @noRd
.build_surv_model_compare_table <- function(display,
                                            format = c("flextable", "gt"),
                                            theme = c("minimal")) {
  format <- match.arg(format, c("flextable", "gt"))
  note <- paste(
    "Lower AIC or BIC indicates better relative fit among the compared distributions.",
    "Use model fit statistics with clinical judgment and visual checks."
  )

  if (format == "gt") {
    tbl <- gt::gt(display) |>
      gt::tab_header(title = "Parametric survival model comparison") |>
      gt::cols_align(align = "left", columns = "Distribution") |>
      gt::cols_align(
        align = "center",
        columns = c("AIC", "BIC", "Log-likelihood", "Scale", "N", "Events", "Best AIC", "Best BIC")
      ) |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_column_labels()
      ) |>
      gt::tab_source_note(gt::md(note)) |>
      .compact_gt_source_notes()

    if ("header_shaded" %in% theme) {
      tbl <- gt::tab_options(tbl, column_labels.background.color = "#f6f8fa")
    }
    if ("zebra" %in% theme) {
      tbl <- gt::opt_row_striping(tbl)
    }
    if ("compact" %in% theme) {
      tbl <- gt::tab_options(tbl, data_row.padding = gt::px(2))
    }
    return(tbl)
  }

  ft <- flextable::flextable(display)
  ft <- flextable::set_caption(ft, caption = "Parametric survival model comparison")
  ft <- flextable::align(ft, j = "Distribution", align = "left", part = "all")
  ft <- flextable::align(
    ft,
    j = c("AIC", "BIC", "Log-likelihood", "Scale", "N", "Events", "Best AIC", "Best BIC"),
    align = "center",
    part = "all"
  )
  ft <- flextable::bold(ft, part = "header", bold = TRUE)
  if ("header_shaded" %in% theme) {
    ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  }
  if ("zebra" %in% theme && nrow(display) > 1L) {
    ft <- flextable::bg(ft, i = seq(1, nrow(display), by = 2), bg = "#f6f8fa", part = "body")
  }
  if ("compact" %in% theme) {
    ft <- flextable::padding(ft, padding = 2, part = "body")
  }
  ft <- flextable::add_footer_lines(ft, values = note)
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

lung_surv_model_compare_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )
}

test_that("surv_model_compare compares candidate distributions", {
  skip_if_not_installed("survival")

  df <- lung_surv_model_compare_data()

  res <- surv_model_compare(
    data = df,
    time = time,
    event = status,
    exposures = c("trt", "celltype", "prior"),
    adjust_for = c("age", "karno")
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "surv_model_compare")
  expect_s3_class(res$table, "flextable")
  expect_equal(res$source, "surv_model_compare")
  expect_equal(res$time, "time")
  expect_equal(res$event, "status")
  expect_equal(res$exposures, c("trt", "celltype", "prior"))
  expect_equal(res$adjust_for, c("age", "karno"))
  expect_equal(res$distributions, c("weibull", "exponential", "lognormal", "loglogistic"))
  expect_true(all(vapply(res$models, inherits, logical(1), what = "survreg")))

  expect_equal(
    res$table_body$Distribution,
    c("loglogistic", "lognormal", "exponential", "weibull")
  )
  expect_equal(round(res$table_body$AIC, 3), c(1441.926, 1447.286, 1448.319, 1449.106))
  expect_equal(round(res$table_body$BIC, 3), c(1468.206, 1473.566, 1471.679, 1475.386))
  expect_equal(round(res$table_body$logLik, 3), c(-711.963, -714.643, -716.160, -715.553))
  expect_equal(round(res$table_body$Scale, 3), c(0.579, 1.060, 1.000, 0.928))
  expect_equal(res$table_body$N, rep(137, 4))
  expect_equal(res$table_body$Events, rep(128, 4))
  expect_equal(res$table_body$Best.AIC, c(TRUE, FALSE, FALSE, FALSE))
  expect_equal(res$table_body$Best.BIC, c(TRUE, FALSE, FALSE, FALSE))
  expect_true("Best AIC" %in% names(res$table_display))
})

test_that("surv_model_compare supports quoted names and unquoted distribution vector", {
  skip_if_not_installed("survival")

  df <- lung_surv_model_compare_data()
  time_var <- "time"
  event_var <- "status"

  out <- surv_model_compare(
    data = df,
    time = time_var,
    event = event_var,
    exposures = c(trt, prior),
    distributions = c(weibull, lognormal),
    format = tibble
  )

  expect_s3_class(out, "tbl_df")
  expect_equal(out$Distribution, c("weibull", "lognormal"))
  expect_equal(round(out$AIC, 1), c(1503.1, 1506.2))
  expect_equal(round(out$BIC, 1), c(1514.8, 1517.9))
  expect_true(all(is.finite(out$logLik)))
  expect_equal(out$Best.AIC, c(TRUE, FALSE))
})

test_that("surv_model_compare normalizes common distribution spellings", {
  skip_if_not_installed("survival")

  df <- lung_surv_model_compare_data()

  out <- surv_model_compare(
    data = df,
    time = time,
    event = status,
    exposures = c(trt, prior),
    distributions = c("Weibull", "log-logistic", "log_normal", exp),
    format = tibble
  )

  expect_setequal(
    out$Distribution,
    c("weibull", "loglogistic", "lognormal", "exponential")
  )
})

test_that("surv_model_compare returns gt table", {
  skip_if_not_installed("survival")
  skip_if_not_installed("gt")

  df <- lung_surv_model_compare_data()

  res <- surv_model_compare(
    data = df,
    time = time,
    event = status,
    exposures = c("trt", "prior"),
    distributions = c("weibull", "lognormal"),
    format = gt
  )

  expect_s3_class(res$table, "gt_tbl")
})

test_that("surv_model_compare validates inputs", {
  skip_if_not_installed("survival")

  df <- lung_surv_model_compare_data()
  df$bad_event <- 0

  expect_error(
    surv_model_compare(data = "not data", time = time, event = status, exposures = "trt"),
    "`data` must be"
  )
  expect_error(
    surv_model_compare(data = df, time = missing_time, event = status, exposures = "trt"),
    "`time` must be"
  )
  expect_error(
    surv_model_compare(data = df, time = time, event = missing_event, exposures = "trt"),
    "`event` must be"
  )
  expect_error(
    surv_model_compare(data = df, time = time, event = status, exposures = "missing_exposure"),
    "exposure variables were not found"
  )
  expect_error(
    surv_model_compare(data = df, time = time, event = bad_event, exposures = "trt"),
    "`event` must include at least one event"
  )
  expect_error(
    surv_model_compare(
      data = df,
      time = time,
      event = status,
      exposures = "trt",
      distributions = c(weibull, gaussian)
    ),
    "`distributions` must contain"
  )
  expect_error(
    surv_model_compare(data = df, time = time, event = status, exposures = "trt", digits = -1),
    "`digits` must be"
  )
})

#' Dissect a dataset before regression
#'
#' Returns a tidy summary of each variable's structure, missingness, uniqueness,
#' and suitability for use in regression models.
#'
#' @param data A data frame.
#' @param verbose Logical; if \code{TRUE}, print the summary and interpretation
#'   notes. The tibble is returned invisibly only when printed by the console.
#' @param format Output format. One of \code{"flextable"} (default),
#'   \code{"gt"}, or \code{"tibble"}. Use \code{format = "tibble"} for
#'   pipeline-friendly raw output.
#'
#' @return A tibble, \code{gt_tbl}, or \code{flextable}, depending on
#'   \code{format}. The tibble has columns: Variable, Type, Missing (%), Unique,
#'   Levels, Compatibility, and Hint.
#' @export
#'
#' @examples
#' dissect(data_birthwt, format = "tibble")
#' dissect(data_birthwt, format = "gt")
#'
#' # Print notes that help interpret the data structure
#' dissect(data_birthwt, verbose = TRUE, format = "tibble")
dissect <- function(data, verbose = FALSE, format = c("flextable", "gt", "tibble")) {
  if (!is.data.frame(data)) {
    stop("The input to `dissect()` must be a data frame. ",
         "You provided an object of class: ", class(data)[1], call. = FALSE)
  }
  if (!is.logical(verbose) || length(verbose) != 1L || is.na(verbose)) {
    stop("`verbose` must be TRUE or FALSE.", call. = FALSE)
  }
  format <- .choice_arg(
    substitute(format),
    env = parent.frame(),
    choices = c("flextable", "gt", "tibble")
  )
  format <- match.arg(format, c("flextable", "gt", "tibble"))


  summarize_column <- function(x, name) {
    x_non_na <- x[!is.na(x)]
    type <- class(x)[1]
    unique_vals <- length(unique(x_non_na))
    pct_missing <- round(mean(is.na(x)) * 100, 1)

    # Factor levels or sample values
    if (is.factor(x)) {
      lvls <- levels(x)
      levels_out <- paste(head(lvls, 5), collapse = ", ")
      if (length(lvls) > 5) levels_out <- paste0(levels_out, ", ...")
    } else if (is.character(x) || is.logical(x)) {
      lvls <- unique(x_non_na)
      levels_out <- paste(head(lvls, 5), collapse = ", ")
      if (length(lvls) > 5) levels_out <- paste0(levels_out, ", ...")
    } else {
      levels_out <- "-"
    }

    # Regression-readiness tag and practical hint
    if (all(is.na(x))) {
      comp <- "incompatible"
      hint <- "All values are missing; do not use as-is."
    } else if (unique_vals == 1) {
      comp <- "incompatible"
      hint <- "Only one observed value; no variation for regression."
    } else if (is.numeric(x)) {
      if (unique_vals > 2) {
        comp <- "compatible"
        hint <- "Numeric variable can be used as continuous."
      } else {
        comp <- "maybe"
        hint <- "Two numeric values; confirm binary coding or convert to factor."
      }
    } else if (is.factor(x)) {
      if (unique_vals > 5) {
        comp <- "maybe"
        hint <- "Many factor levels; check sparse categories before modeling."
      } else {
        comp <- "compatible"
        hint <- "Factor variable can be used as categorical."
      }
    } else if (is.character(x)) {
      if (unique_vals > 5) {
        comp <- "maybe"
        hint <- "Character variable has many values; consider factor grouping."
      } else {
        comp <- "compatible"
        hint <- "Consider converting character values to factor before modeling."
      }
    } else if (inherits(x, "Date")) {
      comp <- "maybe"
      hint <- "Date variable usually needs transformation before modeling."
    } else if (is.logical(x)) {
      comp <- "compatible"
      hint <- "Logical variable can be used as binary."
    } else {
      comp <- "incompatible"
      hint <- "Unsupported variable type; transform before modeling."
    }

    tibble::tibble(
      Variable = name,
      Type = type,
      `Missing (%)` = paste0(pct_missing, "%"),
      Unique = unique_vals,
      Levels = levels_out,
      Compatibility = comp,
      Hint = hint
    )
  }

  result <- purrr::map_dfr(names(data), ~ summarize_column(data[[.x]], .x))

  if (isTRUE(verbose)) {
    print(result)

    message("\nInterpretation notes:\n",
        "- compatible: ready to use in regression or with minimal preparation\n",
        "- maybe: check coding, sparse levels, or whether transformation is needed\n",
        "- incompatible: not usable as-is (e.g., all NA or no variation)\n")
  }

  if (format == "tibble") {
    return(result)
  }

  .build_dissect_table(result, format = format)
}

#' Build formatted dissect table
#' @keywords internal
#' @noRd
.build_dissect_table <- function(result,
                                 format = c("flextable", "gt")) {
  format <- match.arg(format, c("flextable", "gt"))
  note <- paste(
    "Screening aid only; review coding, missingness, sparse levels,",
    "and study context before modeling."
  )

  if (format == "gt") {
    return(
      gt::gt(result) |>
        gt::tab_header(title = "Dataset dissection before regression") |>
        gt::cols_align(
          align = "left",
          columns = c("Variable", "Type", "Levels", "Compatibility", "Hint")
        ) |>
        gt::cols_align(
          align = "center",
          columns = c("Missing (%)", "Unique")
        ) |>
        gt::tab_style(
          style = gt::cell_text(weight = "bold"),
          locations = gt::cells_column_labels()
        ) |>
        gt::tab_style(
          style = gt::cell_fill(color = "#e7f5ec"),
          locations = gt::cells_body(
            columns = "Compatibility",
            rows = .data$Compatibility == "compatible"
          )
        ) |>
        gt::tab_style(
          style = gt::cell_fill(color = "#fff4d6"),
          locations = gt::cells_body(
            columns = "Compatibility",
            rows = .data$Compatibility == "maybe"
          )
        ) |>
        gt::tab_style(
          style = gt::cell_fill(color = "#fde2e2"),
          locations = gt::cells_body(
            columns = "Compatibility",
            rows = .data$Compatibility == "incompatible"
          )
        ) |>
        gt::tab_source_note(gt::md(note)) |>
        .compact_gt_source_notes()
    )
  }

  ft <- flextable::flextable(result)
  ft <- flextable::set_caption(ft, caption = "Dataset dissection before regression")
  ft <- flextable::align(
    ft,
    j = c("Variable", "Type", "Levels", "Compatibility", "Hint"),
    align = "left",
    part = "all"
  )
  ft <- flextable::align(
    ft,
    j = c("Missing (%)", "Unique"),
    align = "center",
    part = "all"
  )
  ft <- flextable::bold(ft, part = "header", bold = TRUE)

  ft <- flextable::bg(
    ft,
    i = which(result$Compatibility == "compatible"),
    j = "Compatibility",
    bg = "#e7f5ec",
    part = "body"
  )
  ft <- flextable::bg(
    ft,
    i = which(result$Compatibility == "maybe"),
    j = "Compatibility",
    bg = "#fff4d6",
    part = "body"
  )
  ft <- flextable::bg(
    ft,
    i = which(result$Compatibility == "incompatible"),
    j = "Compatibility",
    bg = "#fde2e2",
    part = "body"
  )
  ft <- flextable::add_footer_lines(ft, values = note)
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  flextable::autofit(ft)
}

test_that("dissect() covers all branches and returns a tibble when requested", {
  # Build a 6-row data frame to exercise all code paths
  df <- data.frame(
    num3   = c(1, 2, 3, NA, 4, 5),                         # numeric >2 uniques  -> compatible
    num2   = c(0, 1, 1, 0, 1, 0),                          # numeric 2 uniques   -> maybe
    fac3   = factor(c("a","b","a", NA, "b","a")),          # factor ≤5 levels    -> compatible
    fac10  = factor(c("L1","L2","L3","L4","L5","L6"),
                    levels = paste0("L", 1:10)),           # factor >5 levels    -> maybe + "..."
    chr3   = c("x","y","x", NA, "y","x"),                  # character ≤5 uniq   -> compatible
    chr10  = c("a","b","c","d","e","f"),                   # character >5 uniq   -> maybe + "..."
    logi   = c(TRUE, FALSE, NA, TRUE, FALSE, TRUE),        # logical            -> compatible
    allNA  = rep(NA, 6),                                    # all NA             -> incompatible
    const  = rep(5, 6),                                     # 1 unique           -> incompatible
    date   = as.Date(c("2020-01-01","2020-01-02", NA,
                       "2020-01-04","2020-01-05","2020-01-06"))  # Date -> maybe
  )

  expect_silent(res <- dissect(df, format = tibble))
  expect_s3_class(res, "tbl_df")
  expect_true(all(c("Variable", "Type", "Missing (%)", "Unique", "Levels",
                    "Compatibility", "Hint") %in% names(res)))
  expect_equal(nrow(res), ncol(df))

  get_row <- function(v) res[res$Variable == v, , drop = FALSE]

  expect_equal(get_row("num3")$Compatibility,  "compatible")
  expect_equal(get_row("num2")$Compatibility,  "maybe")
  expect_equal(get_row("fac3")$Compatibility,  "compatible")
  expect_equal(get_row("fac10")$Compatibility, "maybe")
  expect_equal(get_row("chr3")$Compatibility,  "compatible")
  expect_equal(get_row("chr10")$Compatibility, "maybe")
  expect_equal(get_row("logi")$Compatibility,  "compatible")
  expect_equal(get_row("allNA")$Compatibility, "incompatible")
  expect_equal(get_row("const")$Compatibility, "incompatible")
  expect_equal(get_row("date")$Compatibility,  "maybe")

  # Ellipsis for long level lists
  expect_true(grepl(", \\.\\.\\.$", get_row("fac10")$Levels))
  expect_true(grepl(", \\.\\.\\.$", get_row("chr10")$Levels))

  # Missing (%) formatting like "0%", "16.7%", "100%"
  expect_true(all(grepl("^\\d+(\\.\\d)?%$", res$`Missing (%)`)))

  expect_equal(get_row("const")$Unique, 1)
  expect_equal(get_row("num2")$Unique, 2)
  expect_match(get_row("num2")$Hint, "binary coding")
  expect_match(get_row("allNA")$Hint, "missing")
})

test_that("dissect() can print beginner interpretation notes", {
  df <- data.frame(x = c(1, 2, 3), y = factor(c("a", "b", "a")))

  expect_message(
    printed <- dissect(df, verbose = TRUE, format = tibble),
    "Interpretation notes"
  )

  expect_s3_class(printed, "tbl_df")
})

test_that("dissect() can return gt and flextable outputs", {
  df <- data.frame(
    x = c(1, 2, 3),
    y = factor(c("a", "b", "a")),
    z = rep(NA, 3)
  )

  gt_out <- dissect(df, format = gt)
  ft_out <- dissect(df, format = "flextable")

  expect_s3_class(gt_out, "gt_tbl")
  expect_s3_class(ft_out, "flextable")
  expect_equal(names(gt_out$`_data`), c(
    "Variable", "Type", "Missing (%)", "Unique", "Levels",
    "Compatibility", "Hint"
  ))
  expect_match(
    as.character(gt_out$`_source_notes`[[1]]),
    "Screening aid only"
  )
})

test_that("dissect() errors on non-data.frame", {
  expect_error(dissect(1:3), "must be a data frame", fixed = TRUE)
  expect_error(dissect(data.frame(x = 1), verbose = NA), "`verbose` must be")
  expect_error(dissect(data.frame(x = 1), format = "bad"), "should be one of")
})

#' Fit Regression Model with One or More Predictors (Internal)
#'
#' Fits a regression model based on the selected approach. Can handle a single exposure or a vector of exposures.
#'
#' @param data A `data.frame` with complete observations for outcome and exposures.
#' @param outcome A string. Name of the outcome variable.
#' @param exposures A string or character vector of predictor(s).
#' @param approach A string specifying the regression approach. One of
#' `"logit"`, `"firth"`, `"logbinomial"`, `"poisson"`, `"linear"`, `"robpoisson"`, or `"negbin"`.
#'
#' @return A fitted model object (`glm`, `logistf`, `lm`, `riskratio`, or `negbin`) or `NULL` if fitting fails.
#' @keywords internal
.fit_uni_model <- function(data, outcome, exposures, approach) {
  approach <- .normalize_approach(approach)
  formula <- as.formula(paste(outcome, "~", exposures))

  tryCatch({
    # Fit model by approach
    fit <- switch(approach,
                  "logit" = glm(formula,
                                data = data,
                                family = binomial("logit")),
                  "firth" = logistf::logistf(formula = formula,
                                             data = data),
                  "logbinomial" = glm(formula,
                                       data = data,
                                       family = binomial("log")),
                  "poisson" = glm(formula,
                                  data = data,
                                  family = poisson("log")),
                  "linear" = lm(formula,
                                data = data),
                  "robpoisson" = risks::riskratio(formula,
                                                  data = data,
                                                  approach = "robpoisson"),
                  "negbin" = MASS::glm.nb(formula,
                                          data = data,
                                          control = glm.control(maxit = 200)),
                  stop(
                    paste0("Invalid approach: '", approach,
                           "'. Choose from: logit, firth, logbinomial, poisson, ",
                           "robpoisson, linear, negbin."),
                    call. = FALSE
                  )
    )
    attr(fit, "gtregression_model_frame") <- stats::model.frame(formula, data = data)
    return(fit)
  }, error = function(e) {
    warning("Model failed for exposure(s) '", paste(exposures, collapse = ", "),
            "' using '", approach, "': ", e$message, call. = FALSE)
    return(NULL)
  })
}

test_that(".fit_uni_model returns correct model class", {
  skip_if_not_installed("risks")

  df <- data_SynthDiabetes |>
    dplyr::mutate(
      diabetes = ifelse(diabetes == "pos", 1, 0),
      age_cat = dplyr::case_when(
        age < 30 ~ "Young",
        age >= 30 & age < 50 ~ "Middle-aged",
        age >= 50 ~ "Older"
      ),
      age_cat = factor(age_cat, levels = c("Young", "Middle-aged", "Older"))
    ) |>
    dplyr::filter(!is.na(diabetes), !is.na(age_cat), !is.na(mass))

  # logit
  m1 <- .fit_uni_model(df, outcome = "diabetes", exposure = "age_cat", approach = "logit")
  expect_s3_class(m1, "glm")
  expect_equal(family(m1)$family, "binomial")

  # logbinomial
  m2 <- .fit_uni_model(df, outcome = "diabetes", exposure = "age_cat", approach = "logbinomial")
  expect_s3_class(m2, "glm")
  expect_equal(family(m2)$link, "log")

  # poisson
  df$count_outcome <- round(df$glucose / 10)
  m3 <- .fit_uni_model(df, outcome = "count_outcome", exposure = "age_cat", approach = "poisson")
  expect_s3_class(m3, "glm")
  expect_equal(family(m3)$family, "poisson")

  # linear
  m4 <- .fit_uni_model(df, outcome = "mass", exposure = "age_cat", approach = "linear")
  expect_s3_class(m4, "lm")

  # robpoisson
  m5 <- .fit_uni_model(df, outcome = "diabetes", exposure = "age_cat", approach = "robpoisson")
  expect_true(any(class(m5) %in% c("riskratio", "risks")))

  # negbin

  m6 <- .fit_uni_model(df, outcome = "glucose", exposures = "age_cat", approach = "negbin")
  expect_s3_class(m6, "negbin")
  expect_s3_class(m6, "glm")


})

test_that(".fit_uni_model handles model fitting failure gracefully", {
  df <- data_SynthDiabetes |>
    dplyr::mutate(
      diabetes = ifelse(diabetes == "pos", 1, 0),
      constant = factor("only", levels = "only")
    )

  expect_warning(
    model <- .fit_uni_model(df, outcome = "diabetes", exposure = "constant", approach = "logit"),
    regexp = "Model failed for"
  )
  expect_null(model)
})

test_that(".fit_uni_model supports Firth logistic regression when logistf is available", {
  skip_if_not_installed("logistf")

  df <- data_endometrial
  df$HG <- factor(df$HG, levels = c(0, 1))
  df$NV <- factor(df$NV, levels = c(0, 1))

  model <- .fit_uni_model(df, outcome = "HG", exposures = "NV", approach = "firth")

  expect_s3_class(model, "logistf")
  expect_true("gtregression_model_frame" %in% names(attributes(model)))
})

#' Internal helper: assess one exposure-candidate pair
#' @keywords internal
#' @noRd
.identify_one_confounder <- function(data,
                                     outcome = NULL,
                                     exposure,
                                     candidate,
                                     approach = "logit",
                                     time = NULL,
                                     event = NULL,
                                     distribution = "weibull",
                                     method = "change",
                                     threshold = 10,
                                     emm_threshold = 10,
                                     emm_test = c("both", "estimate", "interaction"),
                                     interaction_alpha = 0.05) {
  approach <- .choice_arg(
    substitute(approach),
    env = parent.frame(),
    choices = c("logit","logbinomial","poisson","robpoisson","negbin","linear",
                "cox","surv","survreg")
  )
  approach <- .normalize_approach(approach)
  is_survival <- approach %in% c("cox", "survreg")
  method <- .choice_arg(substitute(method), env = parent.frame(), choices = c("change", "mh", "both"))
  emm_test <- .choice_arg(substitute(emm_test), env = parent.frame(), choices = c("both", "estimate", "interaction"))
  method <- match.arg(method, c("change", "mh", "both"))
  emm_test <- match.arg(emm_test, c("both", "estimate", "interaction"))

  if (is_survival) {
    if (is.null(time) || is.null(event)) {
      stop("`time` and `event` are required when `approach` is cox or survreg.", call. = FALSE)
    }
    .validate_cox_inputs(
      data = data,
      time = time,
      event = event,
      exposures = unique(c(exposure, candidate)),
      adjust_for = NULL
    )
    vars_needed <- unique(c(time, event, exposure, candidate))
    dat <- stats::na.omit(data[, vars_needed, drop = FALSE])
    dat[[event]] <- .cox_event01(dat[[event]])
    response <- paste0("survival::Surv(", .surv_bt(time), ", ", .surv_bt(event), ")")
  } else {
    .validate_outcome_by_approach(data[[outcome]], approach)
    vars_needed <- unique(c(outcome, exposure, candidate))
    dat <- stats::na.omit(data[, vars_needed, drop = FALSE])
    response <- .surv_bt(outcome)
  }

  if (nrow(dat) == 0) {
    stop("No complete cases available for analysis.", call. = FALSE)
  }

  get_model <- function(data, formula, approach) {
    if (approach == "robpoisson") {
      response <- all.vars(formula)[[1]]
      if (is.factor(data[[response]]) || is.character(data[[response]])) {
        lev <- .binary_levels(data[[response]])
        if (length(lev) == 2L) {
          data[[response]] <- .as_binary01(data[[response]], lev)
        }
      }
    }

    switch(
      approach,
      "logit" = glm(formula, data = data, family = binomial("logit")),
      "logbinomial" = glm(formula, data = data, family = binomial("log")),
      "poisson" = glm(formula, data = data, family = poisson("log")),
      "negbin" = MASS::glm.nb(formula, data = data),
      "linear" = lm(formula, data = data),
      "cox" = survival::coxph(formula, data = data, model = TRUE),
      "survreg" = survival::survreg(formula, data = data, dist = distribution, model = TRUE),
      "robpoisson" = risks::riskratio(
        formula = formula,
        data = data,
        approach = "robpoisson"
      ),
      stop("Unsupported approach.", call. = FALSE)
    )
  }

  extract_estimate <- function(model, exposure, approach) {
    if (inherits(model, c("glm", "lm"))) {
      coefs <- stats::coef(model)
      idx <- grep(paste0("^", exposure), names(coefs))
      if (!length(idx)) return(NA_real_)
      est <- unname(coefs[idx[1]])
      return(if (approach == "linear") est else exp(est))
    }

    if (inherits(model, c("coxph", "survreg"))) {
      coefs <- stats::coef(model)
      idx <- grep(paste0("^", exposure), names(coefs))
      if (!length(idx)) return(NA_real_)
      return(exp(unname(coefs[idx[1]])))
    }

    if (inherits(model, "riskratio")) {
      sm <- model$summary
      if (!is.null(sm) && "term" %in% names(sm)) {
        idx <- grep(paste0("^", exposure), sm$term)
        if (!length(idx)) return(NA_real_)
        return(sm$RR[idx[1]])
      }
    }

    NA_real_
  }

  safe_fit <- function(formula, data, approach) {
    tryCatch(
      get_model(data, formula, approach),
      error = function(e) NULL
    )
  }

  crude_formula <- stats::as.formula(
    paste(response, "~", .survival_rhs(exposure))
  )
  crude_model <- safe_fit(crude_formula, dat, approach)
  crude_est <- if (!is.null(crude_model)) {
    extract_estimate(crude_model, exposure, approach)
  } else {
    NA_real_
  }

  # adjusted model
  adj_formula <- stats::as.formula(
    paste(response, "~", .survival_rhs(c(exposure, candidate)))
  )
  adj_model <- safe_fit(adj_formula, dat, approach)
  adj_est <- if (!is.null(adj_model)) {
    extract_estimate(adj_model, exposure, approach)
  } else {
    NA_real_
  }

  pct_change <- if (anyNA(c(crude_est, adj_est)) || crude_est == 0) {
    NA_real_
  } else {
    abs((adj_est - crude_est) / crude_est) * 100
  }

  mh <- .confounder_mh_estimate(
    data = dat,
    outcome = outcome,
    exposure = exposure,
    candidate = candidate,
    approach = approach
  )

  pct_change_mh <- if (anyNA(c(crude_est, mh$estimate)) || crude_est == 0) {
    NA_real_
  } else {
    abs((mh$estimate - crude_est) / crude_est) * 100
  }

  is_confounder_model <- if (!is.na(pct_change)) pct_change >= threshold else NA
  is_confounder_mh <- if (!is.na(pct_change_mh)) pct_change_mh >= threshold else NA
  is_confounder <- switch(
    method,
    "change" = is_confounder_model,
    "mh" = is_confounder_mh,
    "both" = if (isTRUE(is_confounder_model) || isTRUE(is_confounder_mh)) {
      TRUE
    } else if (isFALSE(is_confounder_model) && isFALSE(is_confounder_mh)) {
      FALSE
    } else {
      NA
    }
  )

  # stratified estimates
  strata_levels <- unique(stats::na.omit(dat[[candidate]]))

  stratum_estimates <- lapply(strata_levels, function(lev) {
    dsub <- dat[dat[[candidate]] == lev, , drop = FALSE]
    mod <- safe_fit(crude_formula, dsub, approach)
    est <- if (!is.null(mod)) extract_estimate(mod, exposure, approach) else NA_real_

    data.frame(
      candidate = candidate,
      stratum = as.character(lev),
      n = nrow(dsub),
      estimate = est,
      stringsAsFactors = FALSE
    )
  })

  stratum_estimates <- do.call(rbind, stratum_estimates)

  n_strata_total <- length(strata_levels)
  n_strata_estimable <- sum(!is.na(stratum_estimates$estimate))
  min_stratum_n <- if (nrow(stratum_estimates) > 0) {
    min(stratum_estimates$n, na.rm = TRUE)
  } else {
    NA_integer_
  }

  # estimate-based EMM screen
  valid_strata <- stats::na.omit(stratum_estimates$estimate)

  emm_statistic <- NA_real_
  emm_by_estimate <- NA

  if (length(valid_strata) >= 2) {
    denom <- min(abs(valid_strata))
    if (!is.na(denom) && denom > 0) {
      emm_statistic <- (max(valid_strata) - min(valid_strata)) / denom * 100
      emm_by_estimate <- emm_statistic >= emm_threshold
    }
  }

  # interaction-based EMM screen
  interaction_p <- NA_real_
  emm_by_interaction <- NA

  int_formula <- stats::as.formula(
    paste(response, "~", .survival_rhs(c(exposure, candidate), paste0(exposure, "*", candidate)))
  )

  int_model <- safe_fit(int_formula, dat, approach)

  if (!is.null(adj_model) &&
      !is.null(int_model) &&
      inherits(adj_model, c("glm", "lm", "coxph", "survreg")) &&
      inherits(int_model, c("glm", "lm", "coxph", "survreg"))) {
    lr <- tryCatch(
      suppressWarnings(stats::anova(adj_model, int_model, test = "LRT")),
      error = function(e) NULL
    )

    if (!is.null(lr) && nrow(lr) >= 2 && "Pr(>Chi)" %in% names(lr)) {
      interaction_p <- suppressWarnings(lr$`Pr(>Chi)`[2])
      emm_by_interaction <- !is.na(interaction_p) && interaction_p < interaction_alpha
    }
  }

  # final EMM basis
  emm_basis <- "none"

  if (isTRUE(emm_by_estimate) && isTRUE(emm_by_interaction)) {
    emm_basis <- "both"
  } else if (isTRUE(emm_by_estimate)) {
    emm_basis <- "estimate"
  } else if (isTRUE(emm_by_interaction)) {
    emm_basis <- "interaction"
  }

  is_effect_modifier <- switch(
    emm_test,
    "estimate" = isTRUE(emm_by_estimate),
    "interaction" = isTRUE(emm_by_interaction),
    "both" = isTRUE(emm_by_estimate) || isTRUE(emm_by_interaction)
  )

  # decision strength
  # EMM signal strength
  emm_signal_strength <- "none"
  if (isTRUE(emm_by_estimate) && isTRUE(emm_by_interaction)) {
    emm_signal_strength <- "strong"
  } else if (isTRUE(emm_by_estimate) || isTRUE(emm_by_interaction)) {
    emm_signal_strength <- "possible"
  }

  # final overall decision
  # final overall decision
  if (isTRUE(is_effect_modifier)) {
    decision <- "effect_modification"
    decision_strength <- emm_signal_strength

    reason <- switch(
      emm_basis,
      "both" = "Stratum-specific estimates differ and the interaction test also supports effect modification.",
      "estimate" = paste0(
        "Stratum-specific estimates differ meaningfully, although the interaction test is not ",
        "strongly significant."
      ),
      "interaction" = "Interaction test supports effect modification, although estimate spread is less marked.",
      "Effect modification detected."
    )

    recommendation <- paste0(
      "Report stratum-specific effects for ", exposure,
      " across levels of ", candidate, "."
    )

    model_build_recommendation <- "include_interaction"

  } else if (isTRUE(is_confounder)) {
    decision <- "confounder"

    decision_strength <- if (emm_signal_strength == "none") {
      "strong"
    } else {
      "possible"
    }

    reason <- .confounder_reason(
      method = method,
      pct_change = pct_change,
      pct_change_mh = pct_change_mh,
      mh_status = mh$status
    )

    recommendation <- paste0("Adjust for ", candidate, ".")
    model_build_recommendation <- "adjust_only"

  } else {
    decision <- if ((method == "mh" && is.na(pct_change_mh)) ||
                    (all(is.na(c(pct_change, pct_change_mh))) &&
                     !isTRUE(is_effect_modifier))) {
      "not_estimable"
    } else {
      "no_evidence"
    }
    decision_strength <- "none"

    reason <- .no_confounder_reason(
      method = method,
      pct_change = pct_change,
      pct_change_mh = pct_change_mh,
      mh_status = mh$status
    )

    recommendation <- paste0(
      "No clear statistical evidence to include ", candidate,
      " as a confounder."
    )

    model_build_recommendation <- "exclude"
  }

  list(
    exposure = exposure,
    candidate = candidate,
    crude = crude_est,
    adjusted = adj_est,
    mh_estimate = mh$estimate,
    mh_method = mh$method,
    mh_status = mh$status,
    percent_change = pct_change,
    percent_change_model = pct_change,
    percent_change_mh = pct_change_mh,
    is_confounder = is_confounder,
    is_confounder_model = is_confounder_model,
    is_confounder_mh = is_confounder_mh,
    stratum_estimates = stratum_estimates,
    n_strata_total = n_strata_total,
    n_strata_estimable = n_strata_estimable,
    min_stratum_n = min_stratum_n,
    emm_basis = emm_basis,
    emm_signal_strength = emm_signal_strength,
    emm_statistic = emm_statistic,
    is_effect_modifier = is_effect_modifier,
    interaction_p = interaction_p,
    decision = decision,
    decision_strength = decision_strength,
    reason = reason,
    recommendation = recommendation,
    model_build_recommendation = model_build_recommendation
  )
}

.binary_levels <- function(x) {
  x <- x[!is.na(x)]
  if (is.factor(x)) {
    levels(droplevels(x))
  } else {
    sort(unique(x))
  }
}

.as_binary01 <- function(x, levels) {
  ifelse(x == levels[[2]], 1L, 0L)
}

.confounder_mh_estimate <- function(data,
                                    outcome,
                                    exposure,
                                    candidate,
                                    approach) {
  if (!approach %in% c("logit", "logbinomial", "robpoisson")) {
    return(list(
      estimate = NA_real_,
      method = NA_character_,
      status = "MH is available only for binary outcome models."
    ))
  }

  y_levels <- .binary_levels(data[[outcome]])
  x_levels <- .binary_levels(data[[exposure]])
  z_levels <- .binary_levels(data[[candidate]])

  if (length(y_levels) != 2L || length(x_levels) != 2L || length(z_levels) < 2L) {
    return(list(
      estimate = NA_real_,
      method = NA_character_,
      status = "MH requires binary outcome, binary exposure, and categorical strata."
    ))
  }

  y <- .as_binary01(data[[outcome]], y_levels)
  x <- .as_binary01(data[[exposure]], x_levels)
  z <- data[[candidate]]
  keep <- !is.na(y) & !is.na(x) & !is.na(z)
  y <- y[keep]
  x <- x[keep]
  z <- droplevels(factor(z[keep]))

  strata <- levels(z)
  if (length(strata) < 2L) {
    return(list(
      estimate = NA_real_,
      method = NA_character_,
      status = "MH requires at least two non-empty strata."
    ))
  }

  counts <- lapply(strata, function(s) {
    idx <- z == s
    a <- sum(y[idx] == 1L & x[idx] == 1L)
    b <- sum(y[idx] == 0L & x[idx] == 1L)
    c <- sum(y[idx] == 1L & x[idx] == 0L)
    d <- sum(y[idx] == 0L & x[idx] == 0L)
    c(a = a, b = b, c = c, d = d, n = a + b + c + d)
  })

  count_mat <- do.call(rbind, counts)
  count_mat <- count_mat[count_mat[, "n"] > 0, , drop = FALSE]
  if (nrow(count_mat) < 2L) {
    return(list(
      estimate = NA_real_,
      method = NA_character_,
      status = "MH requires at least two non-empty strata."
    ))
  }

  if (approach == "logit") {
    numerator <- sum(count_mat[, "a"] * count_mat[, "d"] / count_mat[, "n"])
    denominator <- sum(count_mat[, "b"] * count_mat[, "c"] / count_mat[, "n"])
    method <- "mh_or"
  } else {
    numerator <- sum(count_mat[, "a"] * (count_mat[, "c"] + count_mat[, "d"]) /
                       count_mat[, "n"])
    denominator <- sum(count_mat[, "c"] * (count_mat[, "a"] + count_mat[, "b"]) /
                         count_mat[, "n"])
    method <- "mh_rr"
  }

  if (!is.finite(numerator) || !is.finite(denominator) || denominator <= 0) {
    return(list(
      estimate = NA_real_,
      method = method,
      status = "MH estimate could not be calculated because of sparse strata."
    ))
  }

  list(
    estimate = unname(numerator / denominator),
    method = method,
    status = "calculated"
  )
}

.confounder_reason <- function(method, pct_change, pct_change_mh, mh_status) {
  if (method == "change") {
    return(paste0(
      "Crude vs adjusted estimate changed by ",
      round(pct_change, 1), "%."
    ))
  }
  if (method == "mh") {
    return(paste0(
      "Crude vs Mantel-Haenszel estimate changed by ",
      round(pct_change_mh, 1), "%."
    ))
  }
  paste0(
    "Confounding detected by change-in-estimate and/or Mantel-Haenszel method",
    if (!is.na(pct_change)) paste0("; model change = ", round(pct_change, 1), "%") else "",
    if (!is.na(pct_change_mh)) paste0("; MH change = ", round(pct_change_mh, 1), "%") else "",
    if (!identical(mh_status, "calculated")) paste0("; MH status: ", mh_status) else "",
    "."
  )
}

.no_confounder_reason <- function(method, pct_change, pct_change_mh, mh_status) {
  if (method == "change") {
    if (is.na(pct_change)) return("Model-based change-in-estimate could not be calculated.")
    return("No strong effect modification and little change between crude and adjusted estimates.")
  }
  if (method == "mh") {
    if (is.na(pct_change_mh)) return(mh_status)
    return("No strong effect modification and little change between crude and Mantel-Haenszel estimates.")
  }
  paste0(
    "No strong effect modification and no important change by the selected methods",
    if (!identical(mh_status, "calculated")) paste0("; MH status: ", mh_status) else "",
    "."
  )
}

#' Identify confounders and effect modifiers
#'
#' Review whether one or more candidate variables may act as confounders or
#' effect modifiers for one or more exposures.
#'
#' The function first assesses possible effect modification using
#' stratum-specific estimates and/or an interaction test. If no important
#' effect modification is detected, it then assesses confounding using the
#' selected method.
#'
#' This is a screening aid for viewing and organising results. Confounding and
#' effect modification should be interpreted using subject-matter knowledge,
#' study design, and causal diagrams such as DAGs. Automated change-in-estimate
#' and interaction checks should not be used as the sole basis for model
#' adjustment.
#'
#' For each exposure-candidate pair, the interaction screen fits
#' \code{outcome ~ exposure + potential_confounder +
#' exposure:potential_confounder}. It does not adjust for other covariates.
#' Consequently, the reported interaction p-value is a candidate-only screening
#' result, not a final adjusted interaction analysis. Use
#' \code{interaction_models(covariates = ...)} when testing a planned,
#' covariate-adjusted interaction.
#'
#' Use this function when you want to screen one or more candidate variables and
#' organise crude, adjusted, Mantel-Haenszel, and effect-modification signals in
#' one place. For a focused comparison of models with and without a planned
#' exposure-by-modifier interaction term, use \code{\link{interaction_models}()}.
#'
#' @param data A data frame.
#' @param outcome Outcome variable name. Quoted and bare names are accepted.
#'   Required for ordinary regression approaches. Leave unset for Cox and
#'   parametric survival approaches and supply \code{time} and \code{event}
#'   instead.
#' @param exposure Exposure variable name(s). Can be a character scalar or
#'   vector. Quoted names are recommended in scripts, and bare names are also
#'   accepted.
#' @param potential_confounder Candidate confounder/effect-modifier variable
#'   name(s). Can be a character scalar or vector. Quoted names are recommended
#'   in scripts, and bare names are also accepted.
#' @param approach Regression approach. One of \code{"logit"},
#'   \code{"logbinomial"}, \code{"poisson"}, \code{"robpoisson"},
#'   \code{"linear"}, \code{"negbin"}, \code{"cox"}, or
#'   \code{"survreg"}.
#' @param time Survival time variable name for \code{approach = "cox"} or
#'   \code{approach = "survreg"}. Quoted and bare names are accepted.
#' @param event Event indicator variable name for survival approaches. Values
#'   may be coded as 0/1, 1/2, logical, or a two-level factor/character
#'   variable.
#' @param distribution Parametric survival distribution for
#'   \code{approach = "survreg"}. One of \code{"weibull"},
#'   \code{"exponential"}, \code{"lognormal"}, or \code{"loglogistic"}.
#' @param method Confounding assessment method. One of \code{"change"},
#'   \code{"mh"}, or \code{"both"}. \code{"change"} compares crude and
#'   adjusted model estimates. \code{"mh"} compares crude and
#'   Mantel-Haenszel pooled estimates and is available for binary outcome,
#'   binary exposure, and categorical strata. \code{"both"} uses either method.
#' @param threshold Percent change threshold for confounding assessment.
#' @param emm_threshold Threshold for relative spread in stratum-specific
#'   estimates when using estimate-based effect-modification screening.
#' @param emm_test One of \code{"interaction"}, \code{"both"}, or
#'   \code{"estimate"}.
#' @param interaction_alpha Alpha threshold for interaction p-values.
#' @param format Output table format. One of \code{"flextable"} (default) or
#'   \code{"gt"}.
#' @param theme Table theme preset or primitives.
#'
#' @return
#' If a single exposure-candidate pair is supplied, returns a detailed list.
#'
#' If multiple combinations are supplied, returns a list with:
#' \describe{
#'   \item{summary}{A tibble with one row per exposure-candidate combination.}
#'   \item{details}{A named list of detailed results for each combination.}
#' }
#'
#' @seealso \code{\link{interaction_models}()} for focused model comparison of
#'   a planned interaction term.
#'
#' @examples
#' birthwt_data <- data_birthwt |>
#'   dplyr::mutate(
#'     low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
#'     smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
#'     race = factor(race, levels = c(1, 2, 3),
#'                   labels = c("White", "Black", "Other"))
#'   )
#'
#' identify_confounder(
#'   data = birthwt_data,
#'   outcome = low,
#'   exposure = smoke,
#'   potential_confounder = race,
#'   approach = logit
#' )
#'
#' lung_data <- data_lungcancer |>
#'   dplyr::mutate(
#'     trt = factor(trt, levels = c(1, 2),
#'                  labels = c("Standard treatment", "Test treatment")),
#'     prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
#'   )
#'
#' identify_confounder(
#'   data = lung_data,
#'   time = time,
#'   event = status,
#'   exposure = trt,
#'   potential_confounder = prior,
#'   approach = cox
#' )
#'
#' @importFrom survival coxph survreg Surv
#' @export
identify_confounder <- function(data,
                                outcome = NULL,
                                exposure,
                                potential_confounder,
                                approach = "logit",
                                time = NULL,
                                event = NULL,
                                distribution = "weibull",
                                method = "change",
                                threshold = 10,
                                emm_threshold = 10,
                                emm_test = c("interaction", "both", "estimate"),
                                interaction_alpha = 0.05,
                                format = c("flextable", "gt"),
                                theme = c("minimal")) {

  outcome <- if (missing(outcome) || identical(substitute(outcome), quote(NULL))) {
    NULL
  } else {
    .vars_arg(substitute(outcome), env = parent.frame())
  }
  exposure <- .vars_arg(substitute(exposure), env = parent.frame())
  potential_confounder <- .vars_arg(substitute(potential_confounder), env = parent.frame())
  approach <- .choice_arg(
    substitute(approach),
    env = parent.frame(),
    choices = c("logit","logbinomial","poisson","robpoisson","negbin","linear",
                "cox","surv","survreg")
  )
  approach <- .normalize_approach(approach)
  .validate_approach(approach, context = "identify_confounder")
  is_survival <- approach %in% c("cox", "survreg")
  time <- if (missing(time) || identical(substitute(time), quote(NULL))) {
    NULL
  } else {
    .cox_single_var_arg(substitute(time), data = data, env = parent.frame())
  }
  event <- if (missing(event) || identical(substitute(event), quote(NULL))) {
    NULL
  } else {
    .cox_single_var_arg(substitute(event), data = data, env = parent.frame())
  }
  distribution <- if (identical(approach, "survreg")) {
    .surv_distribution_arg(
      substitute(distribution),
      env = parent.frame(),
      multiple = FALSE,
      arg = "distribution"
    )
  } else {
    "weibull"
  }
  method <- .choice_arg(substitute(method), env = parent.frame(), choices = c("change", "mh", "both"))
  emm_test <- .choice_arg(substitute(emm_test), env = parent.frame(), choices = c("interaction", "both", "estimate"))
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("flextable", "gt"))
  theme <- .choice_arg(substitute(theme), env = parent.frame())

  method <- match.arg(method, c("change", "mh", "both"))
  emm_test <- match.arg(emm_test, c("interaction", "both", "estimate"))
  format <- match.arg(format, c("flextable", "gt"))
  theme <- .resolve_theme(theme)

  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }

  if (!is_survival && (!is.character(outcome) || length(outcome) != 1L)) {
    stop("`outcome` must be a single character string.", call. = FALSE)
  }

  if (!is_survival && !outcome %in% names(data)) {
    stop("Outcome variable not found in the dataset.", call. = FALSE)
  }

  if (is_survival && (is.null(time) || is.null(event))) {
    stop("`time` and `event` are required when `approach` is cox or survreg.", call. = FALSE)
  }

  if (!is.character(exposure) || length(exposure) < 1L) {
    stop("`exposure` must be a character vector with at least one variable.", call. = FALSE)
  }

  if (!is.character(potential_confounder) || length(potential_confounder) < 1L) {
    stop(
      "`potential_confounder` must be a character vector with at least one variable.",
      call. = FALSE
    )
  }

  if (!all(exposure %in% names(data))) {
    stop("One or more exposure variables were not found in the dataset.", call. = FALSE)
  }

  if (!all(potential_confounder %in% names(data))) {
    stop(
      "One or more potential confounder variables were not found in the dataset.",
      call. = FALSE
    )
  }

  combos <- expand.grid(
    exposure = exposure,
    candidate = potential_confounder,
    stringsAsFactors = FALSE
  )

  details <- vector("list", nrow(combos))
  names(details) <- paste0(combos$exposure, "__", combos$candidate)

  for (i in seq_len(nrow(combos))) {
    exp_i <- combos$exposure[i]
    cand_i <- combos$candidate[i]

    if (identical(exp_i, cand_i)) {
      details[[i]] <- list(
        exposure = exp_i,
        candidate = cand_i,
        crude = NA_real_,
        adjusted = NA_real_,
        mh_estimate = NA_real_,
        mh_method = NA_character_,
        mh_status = "Exposure and candidate variable are the same.",
        percent_change = NA_real_,
        percent_change_model = NA_real_,
        percent_change_mh = NA_real_,
        is_confounder = NA,
        is_confounder_model = NA,
        is_confounder_mh = NA,
        stratum_estimates = NULL,
        n_strata_total = NA_integer_,
        n_strata_estimable = NA_integer_,
        min_stratum_n = NA_integer_,
        emm_statistic = NA_real_,
        emm_basis = "none",
        emm_signal_strength = "none",
        is_effect_modifier = NA,
        interaction_p = NA_real_,
        decision = "invalid",
        decision_strength = "none",
        reason = "Exposure and candidate variable are the same.",
        recommendation = "Choose different variables for exposure and candidate.",
        model_build_recommendation = "exclude"
      )
      next
    }

    details[[i]] <- .identify_one_confounder(
      data = data,
      outcome = outcome,
      exposure = exp_i,
      candidate = cand_i,
      approach = approach,
      time = time,
      event = event,
      distribution = distribution,
      method = method,
      threshold = threshold,
      emm_threshold = emm_threshold,
      emm_test = emm_test,
      interaction_alpha = interaction_alpha
    )
  }

  summary_tbl <- dplyr::bind_rows(lapply(details, function(x) {
    tibble::tibble(
      exposure = x$exposure,
      candidate = x$candidate,
      crude_est = round(x$crude, 3),
      adjusted_est = round(x$adjusted, 3),
      mh_est = round(x$mh_estimate, 3),
      percent_change = round(x$percent_change_model, 2),
      percent_change_model = round(x$percent_change_model, 2),
      percent_change_mh = round(x$percent_change_mh, 2),
      is_confounder = x$is_confounder,
      interaction_p = round(x$interaction_p, 3),
      is_effect_modifier = x$is_effect_modifier,
      decision = x$decision,
      recommendation = x$recommendation
    )
  }))

  summary_table <- .build_identify_confounder_table(
    summary_tbl = summary_tbl,
    outcome = if (is_survival) paste0("Surv(", time, ", ", event, ")") else outcome,
    approach = approach,
    format = format,
    theme = theme
  )

  if (nrow(combos) == 1L) {
    res <- details[[1]]
    res$summary <- summary_tbl
    res$table <- summary_table
    res$format <- format
    res$source <- "identify_confounder"
    class(res) <- c("identify_confounder_result", class(res))

    return(res)
  }

  out <- list(
    summary = summary_tbl,
    table = summary_table,
    details = details,
    format = format,
    source = "identify_confounder"
  )

  class(out) <- c("identify_confounder_result", class(out))
  out
}

#' Build identify_confounder summary table
#' @keywords internal
#' @noRd
.build_identify_confounder_table <- function(summary_tbl,
                                             outcome,
                                             approach,
                                             format = c("flextable", "gt"),
                                             theme = c("minimal")) {
  format <- .choice_arg(substitute(format), env = parent.frame(), choices = c("flextable", "gt"))
  theme <- .choice_arg(substitute(theme), env = parent.frame())

  format <- match.arg(format, c("flextable", "gt"))
  theme <- .resolve_theme(theme)

  df <- summary_tbl
  caveat <- paste(
    "Screening aid only; use DAGs, subject-matter knowledge, and study design",
    "to decide confounding and effect modification."
  )
  model_note <- paste0(
    "Outcome: ", outcome, ". For each exposure E and potential confounder C, ",
    "models tested: ", outcome, " ~ E; ", outcome, " ~ E + C; and interaction screen: ",
    outcome, " ~ E + C + E:C (", .get_effect_label(approach), " scale)."
  )

  df_display <- df |>
    dplyr::mutate(
      crude_est = dplyr::if_else(
        is.na(.data$crude_est),
        "",
        formatC(.data$crude_est, digits = 3, format = "f")
      ),
      adjusted_est = dplyr::if_else(
        is.na(.data$adjusted_est),
        "",
        formatC(.data$adjusted_est, digits = 3, format = "f")
      ),
      mh_est = dplyr::if_else(
        is.na(.data$mh_est),
        "",
        formatC(.data$mh_est, digits = 3, format = "f")
      ),
      interaction_p = dplyr::if_else(
        is.na(.data$interaction_p),
        "",
        formatC(.data$interaction_p, digits = 3, format = "f")
      ),
      percent_change = dplyr::if_else(
        is.na(.data$percent_change),
        "",
        formatC(.data$percent_change, digits = 2, format = "f")
      ),
      percent_change_model = dplyr::if_else(
        is.na(.data$percent_change_model),
        "",
        formatC(.data$percent_change_model, digits = 2, format = "f")
      ),
      percent_change_mh = dplyr::if_else(
        is.na(.data$percent_change_mh),
        "",
        formatC(.data$percent_change_mh, digits = 2, format = "f")
      ),
      is_confounder = dplyr::case_when(
        is.na(.data$is_confounder) ~ "",
        .data$is_confounder ~ "Yes",
        TRUE ~ "No"
      ),
      is_effect_modifier = dplyr::case_when(
        is.na(.data$is_effect_modifier) ~ "",
        .data$is_effect_modifier ~ "Yes",
        TRUE ~ "No"
      ),
      decision = dplyr::recode(
        .data$decision,
        effect_modification = "Effect modifier",
        confounder = "Confounder",
        no_evidence = "No evidence",
        not_estimable = "Not estimable",
        invalid = "Invalid",
        .default = .data$decision
      )
    ) |>
    dplyr::transmute(
      "Exposure" = .data$exposure,
      "Potential confounder" = .data$candidate,
      "Crude estimate" = .data$crude_est,
      "Adjusted estimate" = .data$adjusted_est,
      "MH estimate" = .data$mh_est,
      "% change model" = .data$percent_change_model,
      "% change MH" = .data$percent_change_mh,
      "Confounder?" = .data$is_confounder,
      "Interaction p" = .data$interaction_p,
      "Effect modifier?" = .data$is_effect_modifier,
      "Decision" = .data$decision,
      "Recommendation" = .data$recommendation
    )

  if (format == "gt") {
    tbl <- gt::gt(df_display) |>
      gt::cols_align(align = "left", columns = c("Exposure", "Potential confounder",
                                                 "Decision", "Recommendation")) |>
      gt::cols_align(align = "center",
                     columns = setdiff(names(df_display),
                                       c("Exposure", "Potential confounder", "Decision",
                                         "Recommendation"))) |>
      gt::tab_style(
        style = gt::cell_text(weight = "bold"),
        locations = gt::cells_column_labels()
      ) |>
      gt::tab_source_note(gt::md(model_note)) |>
      gt::tab_source_note(gt::md(caveat)) |>
      .compact_gt_source_notes()

    if ("zebra" %in% theme) {
      tbl <- gt::opt_row_striping(tbl)
    }

    if ("compact" %in% theme) {
      tbl <- gt::tab_options(tbl, data_row.padding = gt::px(2))
    }

    if ("header_shaded" %in% theme) {
      tbl <- gt::tab_options(tbl, column_labels.background.color = "#f6f8fa")
    }

    if ("lines" %in% theme) {
      tbl <- gt::tab_options(
        tbl,
        table.border.top.style = "solid",
        table.border.top.color = "#DADADA",
        table.border.bottom.style = "solid",
        table.border.bottom.color = "#DADADA"
      )
    }

    return(tbl)
  }

  ft <- flextable::flextable(df_display)
  ft <- flextable::align(
    ft,
    j = c("Exposure", "Potential confounder", "Decision", "Recommendation"),
    align = "left",
    part = "all"
  )
  ft <- flextable::align(
    ft,
    j = setdiff(names(df_display),
                c("Exposure", "Potential confounder", "Decision", "Recommendation")),
    align = "center",
    part = "all"
  )
  ft <- flextable::bold(ft, part = "header", bold = TRUE)

  if ("zebra" %in% theme) {
    ft <- flextable::bg(ft, i = seq(1, nrow(df_display), by = 2),
                        bg = "#f6f8fa", part = "body")
  }

  if ("compact" %in% theme) {
    ft <- flextable::padding(ft, padding = 2, part = "body")
  }

  if ("header_shaded" %in% theme) {
    ft <- flextable::bg(ft, part = "header", bg = "#f6f8fa")
  }

  ft <- flextable::add_footer_lines(ft, values = c(model_note, caveat))
  ft <- .compact_flex_footer(ft)
  ft <- flextable::italic(ft, italic = TRUE, part = "footer")
  ft <- flextable::autofit(ft)
  ft
}

birthwt_confounder_data <- function() {
  data("data_birthwt", package = "gtregression")

  data_birthwt |>
    dplyr::mutate(
      race = factor(race, levels = c(1, 2, 3),
                    labels = c("White", "Black", "Other")),
      smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
      ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
      ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
      low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
    )
}

lung_confounder_data <- function() {
  data("data_lungcancer", package = "gtregression")

  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )
}

test_that("identify_confounder returns a focused summary for multiple pairs", {
  df <- birthwt_confounder_data()

  result <- identify_confounder(
    data = df,
    outcome = "low",
    exposure = "smoke",
    potential_confounder = c("race", "ht"),
    approach = logit,
    method = change
  )

  expect_s3_class(result, "identify_confounder_result")
  expect_s3_class(result$summary, "tbl_df")
  expect_named(
    result$summary,
    c(
      "exposure", "candidate", "crude_est", "adjusted_est", "mh_est",
      "percent_change", "percent_change_model", "percent_change_mh",
      "is_confounder", "interaction_p", "is_effect_modifier",
      "decision", "recommendation"
    )
  )
  expect_equal(nrow(result$summary), 2)
  expect_true(all(result$summary$decision %in%
                    c("confounder", "effect_modification", "no_evidence",
                      "not_estimable", "invalid")))
  expect_type(result$details, "list")
  expect_true("smoke__race" %in% names(result$details))
})

test_that("identify_confounder keeps detailed fields for single-pair calls", {
  df <- birthwt_confounder_data()

  result <- identify_confounder(
    data = df,
    outcome = "low",
    exposure = "smoke",
    potential_confounder = "race",
    approach = "logit"
  )

  expect_type(result, "list")
  expect_s3_class(result, "identify_confounder_result")
  expect_true(all(c("crude", "adjusted", "percent_change", "is_confounder",
                    "summary", "table") %in% names(result)))
  expect_true(is.numeric(result$crude))
  expect_s3_class(result$summary, "tbl_df")
  expect_equal(nrow(result$summary), 1)
})

test_that("identify_confounder stores its formatted table by default", {
  df <- birthwt_confounder_data()

  result <- identify_confounder(
    data = df,
    outcome = "low",
    exposure = "smoke",
    potential_confounder = "race",
    approach = "logit"
  )

  expect_invisible(print(result))
  expect_s3_class(result$table, "flextable")
  expect_true("Potential confounder" %in% names(result$table$header$dataset))
  expect_true(any(grepl(
    "Outcome: low. For each exposure E",
    unlist(result$table$footer$dataset, use.names = FALSE),
    fixed = TRUE
  )))
})

test_that("identify_confounder supports Mantel-Haenszel method", {
  df <- birthwt_confounder_data()

  result <- identify_confounder(
    data = df,
    outcome = "low",
    exposure = "smoke",
    potential_confounder = "race",
    approach = logit,
    method = mh
  )

  expect_equal(result$mh_status, "calculated")
  expect_equal(result$mh_method, "mh_or")
  expect_true(is.numeric(result$mh_estimate))
  expect_true(is.numeric(result$percent_change_mh))
  expect_equal(result$summary$mh_est, round(result$mh_estimate, 3))
})

test_that("identify_confounder reports MH ineligibility clearly", {
  df <- birthwt_confounder_data()

  result <- identify_confounder(
    data = df,
    outcome = "low",
    exposure = "age",
    potential_confounder = "smoke",
    approach = logit,
    method = mh
  )

  expect_true(is.na(result$mh_estimate))
  expect_match(result$mh_status, "MH requires binary outcome")
  expect_true(is.na(result$summary$mh_est))
  expect_equal(result$decision, "not_estimable")
})

test_that("identify_confounder supports both method and risk-ratio MH", {
  df <- birthwt_confounder_data()

  result <- identify_confounder(
    data = df,
    outcome = "low",
    exposure = "smoke",
    potential_confounder = "race",
    approach = logbinomial,
    method = both
  )

  expect_equal(result$mh_method, "mh_rr")
  expect_true(is.numeric(result$mh_estimate))
  expect_true(is.logical(result$is_confounder) || is.na(result$is_confounder))
})

test_that("identify_confounder supports count outcomes with change method", {
  data("quine", package = "MASS")
  quine_data <- quine |>
    dplyr::mutate(dplyr::across(c(Eth, Sex, Age, Lrn), as.factor))

  result <- identify_confounder(
    data = quine_data,
    outcome = "Days",
    exposure = "Sex",
    potential_confounder = c("Eth", "Age"),
    approach = negbin
  )

  expect_s3_class(result$summary, "tbl_df")
  expect_equal(nrow(result$summary), 2)
  expect_true(all(c("crude_est", "adjusted_est", "percent_change") %in%
                    names(result$summary)))
})

test_that("identify_confounder supports additional model approaches", {
  df <- birthwt_confounder_data()

  linear_result <- identify_confounder(
    data = df,
    outcome = "bwt",
    exposure = "age",
    potential_confounder = "smoke",
    approach = linear
  )
  expect_true(is.numeric(linear_result$crude))
  expect_equal(linear_result$mh_status, "MH is available only for binary outcome models.")

  rob_result <- identify_confounder(
    data = df,
    outcome = "low",
    exposure = "smoke",
    potential_confounder = "race",
    approach = robpoisson
  )
  expect_true(is.numeric(rob_result$crude))

  data("quine", package = "MASS")
  quine_data <- quine |>
    dplyr::mutate(dplyr::across(c(Eth, Sex, Age, Lrn), as.factor))
  poisson_result <- identify_confounder(
    data = quine_data,
    outcome = "Days",
    exposure = "Sex",
    potential_confounder = "Eth",
    approach = poisson
  )
  expect_true(is.numeric(poisson_result$crude))
})

test_that("identify_confounder supports Cox and parametric survival approaches", {
  skip_if_not_installed("survival")

  df <- lung_confounder_data()

  cox_result <- identify_confounder(
    data = df,
    time = time,
    event = status,
    exposure = trt,
    potential_confounder = prior,
    approach = cox,
    method = change
  )

  direct_cox <- survival::coxph(
    survival::Surv(time, status) ~ trt,
    data = stats::na.omit(df[, c("time", "status", "trt")])
  )

  expect_s3_class(cox_result, "identify_confounder_result")
  expect_equal(cox_result$summary$crude_est, unname(round(exp(stats::coef(direct_cox)[1]), 3)))
  expect_equal(cox_result$mh_status, "MH is available only for binary outcome models.")
  expect_true(is.numeric(cox_result$adjusted))
  expect_true(is.numeric(cox_result$interaction_p) || is.na(cox_result$interaction_p))

  surv_result <- identify_confounder(
    data = df,
    time = "time",
    event = "status",
    exposure = "trt",
    potential_confounder = "prior",
    approach = surv,
    distribution = weibull
  )

  expect_s3_class(surv_result, "identify_confounder_result")
  expect_true(is.numeric(surv_result$crude))
  expect_true(is.numeric(surv_result$adjusted))
  expect_equal(surv_result$mh_status, "MH is available only for binary outcome models.")
})

test_that("identify_confounder reports effect modification when estimate spread is large", {
  df <- birthwt_confounder_data()

  result <- identify_confounder(
    data = df,
    outcome = "low",
    exposure = "smoke",
    potential_confounder = "race",
    approach = logit,
    emm_test = estimate,
    emm_threshold = 0
  )

  expect_equal(result$decision, "effect_modification")
  expect_true(result$is_effect_modifier)
  expect_match(result$recommendation, "Report stratum-specific effects")
})

test_that("identify_confounder handles invalid and not-estimable combinations", {
  df <- birthwt_confounder_data()

  invalid <- identify_confounder(
    data = df,
    outcome = "low",
    exposure = "smoke",
    potential_confounder = "smoke",
    approach = logit
  )
  expect_equal(invalid$decision, "invalid")
  expect_match(invalid$recommendation, "Choose different variables")

  missing_df <- df
  missing_df$smoke <- NA
  expect_error(
    identify_confounder(
      data = missing_df,
      outcome = "low",
      exposure = "smoke",
      potential_confounder = "race",
      approach = logit
    ),
    "No complete cases available"
  )
})

test_that("identify_confounder builds flextable output", {
  df <- birthwt_confounder_data()

  result <- identify_confounder(
    data = df,
    outcome = "low",
    exposure = "smoke",
    potential_confounder = "race",
    approach = logit,
    format = flextable,
    theme = clinical
  )

  expect_s3_class(result$table, "flextable")
})

test_that("identify_confounder display table includes caveat", {
  df <- birthwt_confounder_data()

  result <- identify_confounder(
    data = df,
    outcome = "low",
    exposure = "smoke",
    potential_confounder = c("race", "ht"),
    approach = logit,
    method = both,
    format = gt
  )

  display_cols <- names(result$table$`_data`)

  expect_true("Interaction p" %in% display_cols)
  expect_true("Confounder?" %in% display_cols)
  expect_true("Effect modifier?" %in% display_cols)
  expect_true("is_confounder" %in% names(result$summary))
  expect_true("is_effect_modifier" %in% names(result$summary))
  source_notes <- vapply(result$table$`_source_notes`, as.character, character(1))
  expect_true(any(grepl("Outcome: low", source_notes, fixed = TRUE)))
  expect_true(any(grepl("Screening aid only", source_notes, fixed = TRUE)))
})

test_that("identify_confounder validates inputs clearly", {
  df <- birthwt_confounder_data()

  expect_error(
    identify_confounder(list(), "low", "smoke", "race"),
    "`data` must be a data.frame"
  )
  expect_error(
    identify_confounder(df, c("low", "bwt"), "smoke", "race"),
    "`outcome` must be a single character string"
  )
  expect_error(
    identify_confounder(df, "missing", "smoke", "race"),
    "Outcome variable not found"
  )
  expect_error(
    identify_confounder(df, "low", character(0), "race"),
    "`exposure` must be a character vector"
  )
  expect_error(
    identify_confounder(df, "low", "missing", "race"),
    "exposure variables were not found"
  )
  expect_error(
    identify_confounder(df, "low", "smoke", "missing"),
    "potential confounder variables were not found"
  )
  expect_error(
    identify_confounder(df, "low", "smoke", "race", approach = "bad"),
    "not a valid approach"
  )
})

#' @keywords internal
#' @noRd
is_gt <- function(x) inherits(x, "gt_uni") || inherits(x$table, "gt_tbl")

#' @keywords internal
#' @noRd
is_flextable <- function(x) inherits(x, "ft_uni") || inherits(x$table, "flextable")

#' @keywords internal
#' @noRd
is_uni <- function(x) inherits(x, "uni_reg")

# Internal data-preparation operations used by the gtregression Shiny app.

gt_dp_rename <- function(data, variable, new_name) {
  if (!variable %in% names(data)) stop("Selected variable was not found.", call. = FALSE)
  new_name <- trimws(new_name)
  if (!nzchar(new_name) || !make.names(new_name) == new_name) {
    stop("New variable names must be non-empty valid R names.", call. = FALSE)
  }
  if (!identical(variable, new_name) && new_name %in% names(data)) {
    stop("A variable with that name already exists.", call. = FALSE)
  }
  names(data)[match(variable, names(data))] <- new_name
  data
}

gt_dp_set_label <- function(data, variable, label = "") {
  if (!variable %in% names(data)) stop("Selected variable was not found.", call. = FALSE)
  label <- trimws(as.character(label))
  if (length(label) != 1L || is.na(label)) {
    stop("Display labels must be a single non-missing value.", call. = FALSE)
  }
  attr(data[[variable]], "label") <- if (nzchar(label)) label else NULL
  attr(data, "gt_dp_affected") <- sum(!is.na(data[[variable]]))
  data
}

gt_dp_recode <- function(data, variable, from, to, preserve = TRUE) {
  if (!variable %in% names(data)) stop("Selected variable was not found.", call. = FALSE)
  from <- as.character(from)
  to <- as.character(to)
  keep <- nzchar(from)
  from <- from[keep]; to <- to[keep]
  if (!length(from)) stop("Add at least one value to recode.", call. = FALSE)
  if (anyDuplicated(from)) stop("Each source value can be mapped only once.", call. = FALSE)
  original <- as.character(data[[variable]])
  changed <- original %in% from
  replacement <- to[match(original, from)]
  if (isTRUE(preserve)) replacement[!changed] <- original[!changed]
  data[[variable]] <- replacement
  attr(data, "gt_dp_affected") <- sum(changed & !is.na(replacement) & original != replacement, na.rm = TRUE)
  data
}

gt_dp_define_missing <- function(data, variable, codes) {
  if (!variable %in% names(data)) stop("Selected variable was not found.", call. = FALSE)
  codes <- trimws(as.character(codes))
  codes <- codes[nzchar(codes)]
  if (!length(codes)) stop("Enter at least one missing-value code.", call. = FALSE)
  values <- as.character(data[[variable]])
  affected <- !is.na(values) & values %in% codes
  data[[variable]][affected] <- NA
  attr(data, "gt_dp_affected") <- sum(affected)
  data
}

gt_dp_show_missing <- function(data, variable, label = "Missing") {
  if (!variable %in% names(data)) stop("Selected variable was not found.", call. = FALSE)
  label <- trimws(as.character(label))
  if (!nzchar(label)) stop("Enter a label for missing values.", call. = FALSE)
  values <- data[[variable]]
  affected <- is.na(values)
  if (!any(affected)) stop("This variable has no missing values to show as a category.", call. = FALSE)
  existing <- unique(as.character(values[!is.na(values)]))
  if (label %in% existing) {
    stop(
      "The missing-value label is already a recorded value. Choose a different label.",
      call. = FALSE
    )
  }
  output <- as.character(values)
  output[affected] <- label
  data[[variable]] <- factor(output, levels = c(existing, label))
  attr(data, "gt_dp_affected") <- sum(affected)
  data
}

gt_dp_set_type <- function(data, variable, type, levels = character()) {
  if (!variable %in% names(data)) stop("Selected variable was not found.", call. = FALSE)
  type <- match.arg(type, c("factor", "ordered", "numeric", "text"))
  values <- data[[variable]]
  if (identical(type, "numeric")) {
    converted <- suppressWarnings(as.numeric(as.character(values)))
    invalid <- !is.na(values) & is.na(converted)
    if (any(invalid)) {
      stop(
        "This variable contains non-numeric values and cannot safely be converted to numeric.",
        call. = FALSE
      )
    }
    data[[variable]] <- converted
  } else if (identical(type, "text")) {
    data[[variable]] <- as.character(values)
  } else {
    levels <- trimws(as.character(levels))
    levels <- levels[nzchar(levels)]
    observed <- unique(as.character(values[!is.na(values)]))
    if (!length(levels)) levels <- observed
    omitted <- setdiff(observed, levels)
    if (length(omitted)) {
      stop(paste0("Level order is missing observed value(s): ", paste(omitted, collapse = ", "), "."), call. = FALSE)
    }
    data[[variable]] <- factor(as.character(values), levels = levels, ordered = identical(type, "ordered"))
  }
  attr(data, "gt_dp_affected") <- sum(!is.na(values))
  attr(data, "gt_dp_type") <- type
  data
}

gt_dp_keep_variables <- function(data, variables) {
  variables <- unique(as.character(variables))
  variables <- variables[nzchar(variables)]
  if (!length(variables)) stop("Select at least one variable to keep.", call. = FALSE)
  missing <- setdiff(variables, names(data))
  if (length(missing)) stop("One or more selected variables were not found.", call. = FALSE)
  result <- data[, variables, drop = FALSE]
  attr(result, "gt_dp_affected") <- ncol(data) - ncol(result)
  result
}

gt_dp_condition <- function(data, variable, operator, value, value2 = NULL) {
  if (!variable %in% names(data)) stop("Selected variable was not found.", call. = FALSE)
  x <- data[[variable]]
  value_num <- suppressWarnings(as.numeric(value))
  compare_value <- if (is.numeric(x) && !is.na(value_num)) value_num else as.character(value)
  x_compare <- if (is.numeric(x) && !is.na(value_num)) x else as.character(x)
  if (operator %in% c("between", "outside")) {
    if (!is.numeric(x)) stop("Between and outside-range conditions require a numeric variable.", call. = FALSE)
    lower <- suppressWarnings(as.numeric(value))
    upper <- suppressWarnings(as.numeric(value2))
    if (!is.finite(lower) || !is.finite(upper)) stop("Enter numeric lower and upper bounds.", call. = FALSE)
    if (lower > upper) stop("The lower bound must not exceed the upper bound.", call. = FALSE)
    if (identical(operator, "between")) return(x >= lower & x <= upper)
    return(x < lower | x > upper)
  }
  switch(operator,
    "==" = x_compare == compare_value, "!=" = x_compare != compare_value,
    ">" = x_compare > compare_value, ">=" = x_compare >= compare_value,
    "<" = x_compare < compare_value, "<=" = x_compare <= compare_value,
    stop("Choose a valid filter operator.", call. = FALSE)
  )
}

gt_dp_transform_arithmetic <- function(data, source, new_name, operator, number) {
  new_name <- trimws(new_name)
  if (!nzchar(new_name) || !make.names(new_name) == new_name || new_name %in% names(data)) {
    stop("New variable name must be a unique valid R name.", call. = FALSE)
  }
  if (!source %in% names(data) || !is.numeric(data[[source]])) {
    stop("Arithmetic and comparison transforms require a numeric source variable.", call. = FALSE)
  }
  number <- suppressWarnings(as.numeric(number))
  if (!is.finite(number)) stop("Enter a finite numeric value.", call. = FALSE)
  data[[new_name]] <- switch(operator,
    "+" = data[[source]] + number, "-" = data[[source]] - number,
    "*" = data[[source]] * number, "/" = data[[source]] / number,
    "^" = data[[source]] ^ number,
    ">" = data[[source]] > number, ">=" = data[[source]] >= number,
    "<" = data[[source]] < number, "<=" = data[[source]] <= number,
    "==" = data[[source]] == number, "!=" = data[[source]] != number,
    stop("Choose a valid arithmetic operator.", call. = FALSE)
  )
  attr(data, "gt_dp_affected") <- sum(!is.na(data[[source]]))
  data
}

gt_dp_calculate <- function(data, source, new_name, mode = c("single", "two_variables", "ratio_power"),
                            operator = "+", number = 1, second = NULL) {
  mode <- match.arg(mode)
  if (identical(mode, "single")) {
    return(gt_dp_transform_arithmetic(data, source, new_name, operator, number))
  }
  new_name <- trimws(new_name)
  if (!nzchar(new_name) || !make.names(new_name) == new_name || new_name %in% names(data)) {
    stop("New variable name must be a unique valid R name.", call. = FALSE)
  }
  if (
    !source %in% names(data) || !second %in% names(data) ||
      !is.numeric(data[[source]]) || !is.numeric(data[[second]])
  ) {
    stop("Choose two numeric source variables.", call. = FALSE)
  }
  if (identical(mode, "two_variables")) {
    data[[new_name]] <- switch(operator,
      "+" = data[[source]] + data[[second]], "-" = data[[source]] - data[[second]],
      "*" = data[[source]] * data[[second]], "/" = data[[source]] / data[[second]],
      stop("Choose a valid calculation.", call. = FALSE)
    )
  } else {
    power <- suppressWarnings(as.numeric(number))
    if (!is.finite(power)) stop("Enter a finite denominator power.", call. = FALSE)
    data[[new_name]] <- data[[source]] / (data[[second]] ^ power)
  }
  attr(data, "gt_dp_affected") <- sum(!is.na(data[[source]]) & !is.na(data[[second]]))
  data
}

gt_dp_group_values <- function(data, variables, operators, values, results, default,
                               values2 = rep("", length(values))) {
  keep <- nzchar(variables) & nzchar(values) & nzchar(results)
  if (!any(keep)) stop("Add at least one condition and result.", call. = FALSE)
  labels <- trimws(c(results[keep], default))
  if (any(!nzchar(labels))) stop("Enter a label for every group.", call. = FALSE)
  if (anyDuplicated(labels)) stop("Each group needs a unique label.", call. = FALSE)
  output <- rep(as.character(default), nrow(data))
  unmatched <- rep(TRUE, nrow(data))
  matched_count <- 0L
  for (index in which(keep)) {
    matched <- gt_dp_condition(
      data, variables[[index]], operators[[index]], values[[index]], values2[[index]]
    ) %in% TRUE
    assign <- unmatched & matched
    output[assign] <- results[[index]]
    unmatched[assign] <- FALSE
    matched_count <- matched_count + sum(assign)
  }
  counts <- stats::setNames(
    vapply(labels, function(label) sum(output == label, na.rm = TRUE), integer(1)),
    labels
  )
  attr(output, "gt_dp_group_counts") <- counts
  attr(output, "gt_dp_affected") <- matched_count
  output
}

gt_dp_empty_group_message <- function(data, variables, counts) {
  empty <- names(counts)[counts == 0L]
  if (!length(empty)) return(NULL)
  source <- unique(variables[nzchar(variables)])
  range_note <- ""
  if (length(source) == 1L && source %in% names(data) && is.numeric(data[[source]])) {
    observed <- data[[source]][is.finite(data[[source]])]
    if (length(observed)) {
      range_note <- paste0(
        " Observed range of `", source, "`: ",
        format(min(observed), trim = TRUE), " to ", format(max(observed), trim = TRUE), "."
      )
    }
  }
  paste0(
    "Grouping would create empty group(s): ", paste(empty, collapse = ", "), ".",
    range_note, " Revise the conditions before applying."
  )
}

gt_dp_transform_case_when <- function(data, new_name, variables, operators, values, results, default,
                                      values2 = rep("", length(values))) {
  new_name <- trimws(new_name)
  if (!nzchar(new_name) || !make.names(new_name) == new_name || new_name %in% names(data)) {
    stop("New variable name must be a unique valid R name.", call. = FALSE)
  }
  output <- gt_dp_group_values(data, variables, operators, values, results, default, values2)
  empty_message <- gt_dp_empty_group_message(
    data, variables, attr(output, "gt_dp_group_counts")
  )
  if (!is.null(empty_message)) stop(empty_message, call. = FALSE)
  affected <- attr(output, "gt_dp_affected")
  attributes(output) <- NULL
  data[[new_name]] <- output
  attr(data, "gt_dp_affected") <- affected
  data
}

gt_dp_filter <- function(data, variable1, operator1, value1,
                         connector = "", variable2 = "", operator2 = "", value2 = "") {
  first <- gt_dp_condition(data, variable1, operator1, value1)
  condition <- first
  if (nzchar(variable2) && nzchar(value2)) {
    second <- gt_dp_condition(data, variable2, operator2, value2)
    condition <- if (identical(connector, "OR")) first | second else first & second
  }
  attr(data, "gt_dp_filter_counts") <- c(
    before = nrow(data), included = sum(condition %in% TRUE),
    excluded = sum(condition %in% FALSE), unknown = sum(is.na(condition))
  )
  data[which(condition %in% TRUE), , drop = FALSE]
}

gt_dp_code_line <- function(operation, ...) {
  args <- list(...)
  quote_value <- function(x) encodeString(as.character(x), quote = '"')
  switch(operation,
    rename = paste0("data <- dplyr::rename(data, ", args$new_name, " = ", args$variable, ")"),
    label = if (nzchar(args$label)) {
      paste0("attr(data$", args$variable, ", \"label\") <- ", quote_value(args$label))
    } else {
      paste0("attr(data$", args$variable, ", \"label\") <- NULL")
    },
    recode = paste0("data$", args$variable, " <- dplyr::recode(as.character(data$", args$variable,
      "), ", paste(paste0(quote_value(args$from), " = ", quote_value(args$to)), collapse = ", "), ")"),
    missing = paste0("data$", args$variable, "[as.character(data$", args$variable, ") %in% ",
      "c(", paste(quote_value(args$codes), collapse = ", "), ")] <- NA"),
    missing_category = paste0("data$", args$variable, " <- factor(ifelse(is.na(data$", args$variable, "), ",
      quote_value(args$label), ", as.character(data$", args$variable, ")))") ,
    type = switch(args$type,
      factor = paste0(
        "data$", args$variable, " <- factor(data$", args$variable,
        ", levels = c(", paste(quote_value(args$levels), collapse = ", "), "))"
      ),
      ordered = paste0(
        "data$", args$variable, " <- ordered(data$", args$variable,
        ", levels = c(", paste(quote_value(args$levels), collapse = ", "), "))"
      ),
      numeric = paste0("data$", args$variable, " <- as.numeric(as.character(data$", args$variable, "))"),
      text = paste0("data$", args$variable, " <- as.character(data$", args$variable, ")")
    ),
    keep = paste0("data <- dplyr::select(data, ", paste(args$variables, collapse = ", "), ")"),
    filter = paste0("data <- dplyr::filter(data, ", args$expression, ")"),
    transform = args$code,
    ""
  )
}

test_that("internal data-preparation helpers apply safe, traceable changes", {
  data <- data.frame(
    age = c(20, 30, 999),
    sex = c("M", "F", "Unknown"),
    stringsAsFactors = FALSE
  )

  renamed <- gt_dp_rename(data, "age", "age_years")
  expect_identical(names(renamed), c("age_years", "sex"))
  expect_error(gt_dp_rename(data, "age", "sex"), "already exists")

  labelled <- gt_dp_set_label(data, "age", "Maternal age")
  expect_identical(attr(labelled$age, "label"), "Maternal age")
  expect_identical(attr(gt_dp_set_label(labelled, "age", "")$age, "label"), NULL)

  recoded <- gt_dp_recode(data, "sex", c("M", "F"), c("Male", "Female"))
  expect_equal(recoded$sex[1:2], c("Male", "Female"))
  expect_equal(attr(recoded, "gt_dp_affected"), 2)

  missing <- gt_dp_define_missing(data, "age", "999")
  expect_true(is.na(missing$age[[3L]]))
  expect_equal(attr(missing, "gt_dp_affected"), 1)

  ordered <- gt_dp_set_type(data, "sex", "ordered", c("F", "M", "Unknown"))
  expect_true(is.ordered(ordered$sex))
  expect_identical(levels(ordered$sex), c("F", "M", "Unknown"))

  filtered <- gt_dp_filter(data, "age", ">=", "25")
  expect_equal(nrow(filtered), 2L)
  expect_equal(unname(attr(filtered, "gt_dp_filter_counts")[["before"]]), 3L)

  transformed <- gt_dp_transform_arithmetic(data, "age", "age_plus_one", "+", 1)
  expect_equal(transformed$age_plus_one[[1L]], 21)

  grouped <- gt_dp_transform_case_when(
    data, "age_group", "age", ">=", "25", "Older", "Younger"
  )
  expect_equal(grouped$age_group[1:2], c("Younger", "Older"))
})

test_that("group creation respects first-match order and numeric ranges", {
  data <- data.frame(age = c(20, 34, 35, 64, 65, 80))

  grouped <- gt_dp_transform_case_when(
    data,
    "age_group",
    variables = c("age", "age"),
    operators = c("<", "<"),
    values = c("35", "65"),
    results = c("Young", "Older"),
    default = "Elder"
  )
  expect_identical(grouped$age_group, c("Young", "Young", "Older", "Older", "Elder", "Elder"))

  ranged <- gt_dp_transform_case_when(
    data,
    "age_band",
    variables = c("age", "age"),
    operators = c("between", ">="),
    values = c("35", "65"),
    values2 = c("64", ""),
    results = c("35 to 64", "65 or older"),
    default = "Under 35"
  )
  expect_identical(ranged$age_band, c("Under 35", "Under 35", "35 to 64", "35 to 64", "65 or older", "65 or older"))
  expect_error(gt_dp_condition(data, "age", "between", "65", "35"), "lower bound")

  birth_age <- data.frame(age = 14:45)
  preview <- gt_dp_group_values(
    birth_age,
    variables = c("age", "age"),
    operators = c("<=", ">="),
    values = c("45", "66"),
    results = c("<=45", ">65"),
    default = "46-65"
  )
  expect_identical(
    attr(preview, "gt_dp_group_counts"),
    c("<=45" = 32L, ">65" = 0L, "46-65" = 0L)
  )
  expect_match(
    gt_dp_empty_group_message(
      birth_age, c("age", "age"), attr(preview, "gt_dp_group_counts")
    ),
    "Observed range of `age`: 14 to 45"
  )
  expect_error(
    gt_dp_transform_case_when(
      birth_age, "age_group", c("age", "age"), c("<=", ">="),
      c("45", "66"), c("<=45", ">65"), "46-65"
    ),
    "empty group\\(s\\): >65, 46-65"
  )
  expect_error(
    gt_dp_transform_case_when(
      data, "duplicate_group", "age", "<", "35", "Age band", "Age band"
    ),
    "unique label"
  )
})

test_that("Data Prep activates original data by default and supports prepared-data choice", {
  skip_if_not_installed("shiny")
  source_data <- shiny::reactiveVal(data.frame(
    age = c(20, 30),
    group = c("A", "B"),
    stringsAsFactors = FALSE
  ))

  suppressWarnings(shiny::testServer(
    mod_data_prep_server,
    args = list(source_data = source_data),
    {
    session$flushReact()
    expect_identical(session$returned$result(), source_data())
    expect_false(session$returned$using_prepared())

    session$setInputs(continue_original = 1)
    session$flushReact()
    expect_identical(session$returned$result(), source_data())
    expect_false(session$returned$using_prepared())

    session$setInputs(use_prepared = 1)
    session$flushReact()
    expect_identical(session$returned$result(), session$returned$working_data())
    expect_true(session$returned$using_prepared())
    }
  ))
})

test_that("Set display label can also polish categorical levels", {
  skip_if_not_installed("shiny")
  source_data <- shiny::reactiveVal(data.frame(
    smoking = factor(c("N", "Y"), levels = c("N", "Y")),
    stringsAsFactors = FALSE
  ))

  suppressWarnings(shiny::testServer(
    mod_data_prep_server,
    args = list(source_data = source_data),
    {
      session$flushReact()
      session$setInputs(variable = "smoking")
      session$flushReact()
      session$setInputs(
        variable_label = "Smoking during pregnancy",
        label_level_to_1 = "No",
        label_level_to_2 = "Yes"
      )
      session$flushReact()
      session$setInputs(apply = 0)
      session$flushReact()
      session$setInputs(apply = 1)
      session$flushReact()

      prepared <- session$returned$working_data()
      expect_identical(attr(prepared$smoking, "label"), "Smoking during pregnancy")
      expect_identical(levels(prepared$smoking), c("No", "Yes"))
      expect_false(session$returned$using_prepared())
    }
  ))
})

test_that("Data Prep module retains its core recovery and reuse controls", {
  ui_text <- paste(deparse(body(mod_data_prep_ui)), collapse = " ")
  server_text <- paste(deparse(body(mod_data_prep_server)), collapse = " ")

  expect_match(ui_text, "Quick starts", fixed = TRUE)
  expect_match(ui_text, "Set display label", fixed = TRUE)
  expect_match(ui_text, "Undo", fixed = TRUE)
  expect_match(ui_text, "Redo", fixed = TRUE)
  expect_match(ui_text, "Reset", fixed = TRUE)
  expect_match(server_text, "confirm_reset", fixed = TRUE)
  expect_match(server_text, "download_prepared", fixed = TRUE)
  expect_match(server_text, "download_code", fixed = TRUE)
  expect_match(server_text, "gt_dp_set_label", fixed = TRUE)
  expect_match(server_text, "Category display labels", fixed = TRUE)
  expect_match(server_text, "label_level_mapping", fixed = TRUE)
  expect_match(server_text, 'reactiveVal("label")', fixed = TRUE)
})

#' Side-by-side forest plots: univariate vs multivariable
#'
#' Creates two aligned forest plots from compatible \code{gtregression}
#' regression objects such as crude and adjusted outputs from \code{uni_reg()},
#' \code{multi_reg()}, \code{cox_reg()}, or \code{surv_reg()}.
#'
#' @param tbl_uni A univariate \code{gtregression} object.
#' @param tbl_multi A multivariable \code{gtregression} object.
#' @param title_uni,title_multi Optional panel titles.
#' @param caption Optional combined plot caption. If \code{NULL}, notes are
#'   added automatically for adjusted \code{multi_reg()} objects and displayed
#'   reference categories.
#' @param ref_line Optional numeric reference line. If \code{NULL}, uses 0 for
#'   linear models and 1 otherwise.
#' @param order_y Optional character vector to customize exposure ordering.
#' @param log_x Logical. If \code{TRUE}, use a log-scaled x-axis for ratio
#'   measures. The default \code{FALSE} uses a linear x-axis.
#' @param point_color,errorbar_color Base colors for non-significant rows.
#' @param point_size Diameter of estimate points in millimetres.
#' @param point_stroke Outline width of estimate points.
#' @param ci_linewidth Line width of confidence intervals.
#' @param base_size Base font size for the plot theme.
#' @param show_ref Logical or \code{NULL}. The default \code{NULL} inherits the
#'   setting used to create the supplied tables. If \code{TRUE}, include
#'   reference levels as \code{(Ref.)}. If \code{FALSE}, binary exposures are
#'   shown as compact rows for the estimated non-reference category;
#'   affirmative levels such as \code{Yes} or \code{1} are displayed using the
#'   variable name.
#' @param sig_color,sig_errorbar_color Optional colors for significant rows.
#'   If \code{NULL}, base colors are reused.
#' @param xlim_uni,breaks_uni Optional x-axis limits and breaks for the
#'   univariate panel.
#' @param xlim_multi,breaks_multi Optional x-axis limits and breaks for the
#'   multivariable panel.
#' @param alpha Significance level for linear models when \code{p.value} is
#'   available.
#' @param show_adjustment_note Logical; if \code{TRUE}, add a default caption
#'   describing \code{adjust_for} variables from \code{tbl_multi} when available.
#'
#' @return A \code{patchwork} object with two \code{ggplot2} panels.
#' @importFrom rlang .data
#' @importFrom stats setNames
#' @export
plot_reg_combine <- function(tbl_uni,
                             tbl_multi,
                             title_uni = NULL,
                             title_multi = NULL,
                             caption = NULL,
                             ref_line = NULL,
                             order_y = NULL,
                             log_x = FALSE,
                             point_color = "#6B7280",
                             errorbar_color = "#6B7280",
                             point_size = 2.8,
                             point_stroke = 0.55,
                             ci_linewidth = 0.55,
                             base_size = 12,
                             show_ref = NULL,
                             sig_color = "#0072B2",
                             sig_errorbar_color = "#0072B2",
                             xlim_uni = NULL,
                             breaks_uni = NULL,
                             xlim_multi = NULL,
                             breaks_multi = NULL,
                             alpha = 0.05,
                             show_adjustment_note = TRUE) {

  if (!inherits(tbl_uni, "gtregression")) {
    stop("`tbl_uni` must be a gtregression object.", call. = FALSE)
  }
  if (!inherits(tbl_multi, "gtregression")) {
    stop("`tbl_multi` must be a gtregression object.", call. = FALSE)
  }
  if (.plot_is_stratified(tbl_uni) || .plot_is_stratified(tbl_multi)) {
    stop("plot_reg_combine() does not support stratified objects.", call. = FALSE)
  }

  show_ref <- .plot_resolve_show_ref(show_ref, tbl_uni, tbl_multi)

  valid_standard_pair <- identical(tbl_uni$source, "uni_reg") &&
    identical(tbl_multi$source, "multi_reg")
  valid_survival_pair <- tbl_uni$source %in% c("cox_reg", "surv_reg") &&
    identical(tbl_uni$source, tbl_multi$source) &&
    !.is_adjusted_reg_output(tbl_uni) &&
    .is_adjusted_reg_output(tbl_multi)

  if (!valid_standard_pair && !valid_survival_pair) {
    if (!identical(tbl_uni$source, "uni_reg") &&
        !tbl_uni$source %in% c("cox_reg", "surv_reg")) {
      stop("`tbl_uni` must come from uni_reg() or a crude cox_reg()/surv_reg() object.", call. = FALSE)
    }
    if (!identical(tbl_multi$source, "multi_reg") &&
        !tbl_multi$source %in% c("cox_reg", "surv_reg")) {
      stop("`tbl_multi` must come from multi_reg() or an adjusted cox_reg()/surv_reg() object.", call. = FALSE)
    }
    if (tbl_uni$source %in% c("cox_reg", "surv_reg") ||
        tbl_multi$source %in% c("cox_reg", "surv_reg")) {
      stop(
        paste0(
          "For survival objects, use a crude cox_reg()/surv_reg() object first and its adjusted ",
          "counterpart second."
        ),
        call. = FALSE
      )
    }
    stop("`tbl_uni` must come from uni_reg() and `tbl_multi` must come from multi_reg().", call. = FALSE)
  }

  req_body <- c("exposure", "level", "estimate", "conf.low", "conf.high", "p.value", "ref")
  req_disp <- c("Characteristic", "is_header")

  if (!all(req_body %in% names(tbl_uni$table_body))) {
    stop("`tbl_uni$table_body` does not have the expected columns.", call. = FALSE)
  }
  if (!all(req_body %in% names(tbl_multi$table_body))) {
    stop("`tbl_multi$table_body` does not have the expected columns.", call. = FALSE)
  }
  if (!all(req_disp %in% names(tbl_uni$table_display))) {
    stop("`tbl_uni$table_display` does not have the expected columns.", call. = FALSE)
  }
  if (!all(req_disp %in% names(tbl_multi$table_display))) {
    stop("`tbl_multi$table_display` does not have the expected columns.", call. = FALSE)
  }

  get_axis_label <- function(approach, adjusted = FALSE) {
    approach <- .normalize_approach(approach)
    base <- dplyr::case_when(
      approach %in% c("logit", "firth") ~ "Odds Ratio",
      approach == "logbinomial" ~ "Risk Ratio",
      approach %in% c("poisson", "negbin") ~ "Incidence Rate Ratio",
      approach == "robpoisson" ~ "Risk Ratio",
      approach == "cox" ~ "Hazard Ratio",
      approach == "survreg" ~ "Time Ratio",
      approach == "linear" ~ "Beta Coefficient",
      TRUE ~ "Effect Size"
    )
    if (adjusted) paste("Adjusted", base) else base
  }

  build_plot_df <- function(tbl, show_ref = TRUE, order_y = NULL) {
    df_body <- tbl$table_body
    df_disp <- tbl$table_display

    df_body$estimate <- suppressWarnings(as.numeric(df_body$estimate))
    df_body$conf.low <- suppressWarnings(as.numeric(df_body$conf.low))
    df_body$conf.high <- suppressWarnings(as.numeric(df_body$conf.high))
    df_body$p.value <- suppressWarnings(as.numeric(df_body$p.value))

    exposure_order <- unique(df_body$exposure)

    if (!is.null(order_y)) {
      exposure_order <- c(
        intersect(order_y, exposure_order),
        setdiff(exposure_order, order_y)
      )
    }

    plot_rows <- list()

    for (ex in exposure_order) {
      dfx <- df_body[df_body$exposure == ex, , drop = FALSE]
      if (!nrow(dfx)) next

      ex_label <- .label_var(ex, tbl$variable_labels)
      is_factor_exp <- any(dfx$ref)
      is_binary_exp <- is_factor_exp && nrow(dfx) == 2L && sum(dfx$ref %in% TRUE) == 1L

      disp_idx <- which(df_disp$is_header & trimws(df_disp$Characteristic) == ex_label)
      if (!length(disp_idx)) next

      start_i <- disp_idx[1]
      end_i <- if (start_i < nrow(df_disp)) {
        next_headers <- which(df_disp$is_header & seq_len(nrow(df_disp)) > start_i)
        if (length(next_headers)) min(next_headers) - 1 else nrow(df_disp)
      } else {
        nrow(df_disp)
      }

      disp_block <- df_disp[start_i:end_i, , drop = FALSE]

      main_row <- dfx[!dfx$ref & dfx$level == ex, , drop = FALSE]

      header_plot <- data.frame(
        exposure = ex,
        label = ex_label,
        estimate = NA_real_,
        conf.low = NA_real_,
        conf.high = NA_real_,
        p.value = NA_real_,
        ref = FALSE,
        is_header = TRUE,
        is_data = FALSE,
        stringsAsFactors = FALSE
      )

      if (!is_factor_exp && nrow(main_row)) {
        header_plot$estimate <- main_row$estimate[1]
        header_plot$conf.low <- main_row$conf.low[1]
        header_plot$conf.high <- main_row$conf.high[1]
        header_plot$p.value <- main_row$p.value[1]
        header_plot$is_data <- TRUE
      }

      compact_binary <- is_binary_exp && !isTRUE(show_ref)
      if (compact_binary) {
        binary_row <- dfx[!dfx$ref, , drop = FALSE]
        if (nrow(binary_row)) {
          header_plot$label <- .plot_binary_label(ex_label, binary_row$level[1])
          header_plot$estimate <- binary_row$estimate[1]
          header_plot$conf.low <- binary_row$conf.low[1]
          header_plot$conf.high <- binary_row$conf.high[1]
          header_plot$p.value <- binary_row$p.value[1]
          header_plot$is_header <- TRUE
          header_plot$is_data <- TRUE
          plot_rows[[length(plot_rows) + 1]] <- header_plot
          next
        }
      }

      plot_rows[[length(plot_rows) + 1]] <- header_plot

      if (nrow(disp_block) > 1) {
        sub_block <- disp_block[-1, , drop = FALSE]

        for (i in seq_len(nrow(sub_block))) {
          char_i <- sub_block$Characteristic[i]
          lab_i <- trimws(char_i)

          row_match <- dfx[dfx$level == lab_i, , drop = FALSE]
          if (!nrow(row_match)) next

          if (isTRUE(row_match$ref[1]) && !show_ref) next

          plot_rows[[length(plot_rows) + 1]] <- data.frame(
            exposure = ex,
            label = lab_i,
            estimate = row_match$estimate[1],
            conf.low = row_match$conf.low[1],
            conf.high = row_match$conf.high[1],
            p.value = row_match$p.value[1],
            ref = isTRUE(row_match$ref[1]),
            is_header = FALSE,
            is_data = !isTRUE(row_match$ref[1]),
            stringsAsFactors = FALSE
          )
        }
      } else {
        extra_rows <- dfx[!dfx$ref & dfx$level != ex, , drop = FALSE]
        if (nrow(extra_rows)) {
          for (i in seq_len(nrow(extra_rows))) {
            plot_rows[[length(plot_rows) + 1]] <- data.frame(
              exposure = ex,
              label = extra_rows$level[i],
              estimate = extra_rows$estimate[i],
              conf.low = extra_rows$conf.low[i],
              conf.high = extra_rows$conf.high[i],
              p.value = extra_rows$p.value[i],
              ref = FALSE,
              is_header = FALSE,
              is_data = TRUE,
              stringsAsFactors = FALSE
            )
          }
        }
      }
    }

    out <- do.call(rbind, plot_rows)

    indent_html <- "<span style='color:white;'>..</span>"

    out$label_clean <- dplyr::case_when(
      out$is_header ~ paste0("<b>", out$label, "</b>"),
      out$ref & show_ref ~ paste0(
        indent_html,
        out$label,
        " <span style='color:gray;'>(Ref.)</span>"
      ),
      !out$is_header ~ paste0(indent_html, out$label),
      TRUE ~ NA_character_
    )

    out$row_key <- paste0(out$exposure, "::", out$label)
    out
  }

  approach_uni <- tbl_uni$approach
  approach_multi <- tbl_multi$approach

  xlab_uni <- get_axis_label(approach_uni, adjusted = FALSE)
  xlab_multi <- get_axis_label(approach_multi, adjusted = TRUE)

  log_x_uni <- .plot_resolve_log_x(log_x, .normalize_approach(approach_uni))
  log_x_multi <- .plot_resolve_log_x(log_x, .normalize_approach(approach_multi))

  if (log_x_uni) xlab_uni <- paste0(xlab_uni, " (log scale)")
  if (log_x_multi) xlab_multi <- paste0(xlab_multi, " (log scale)")

  ref_uni <- if (is.null(ref_line)) {
    if (identical(approach_uni, "linear")) 0 else 1
  } else {
    ref_line
  }

  ref_multi <- if (is.null(ref_line)) {
    if (identical(approach_multi, "linear")) 0 else 1
  } else {
    ref_line
  }

  df_uni <- build_plot_df(tbl_uni, show_ref = show_ref, order_y = order_y)
  df_multi <- build_plot_df(tbl_multi, show_ref = show_ref, order_y = order_y)

  row_keys <- unique(c(df_uni$row_key, df_multi$row_key))

  skeleton <- data.frame(
    row_key = row_keys,
    stringsAsFactors = FALSE
  )

  label_from_key <- function(x) sub("^[^:]+::", "", x)
  skeleton$label <- label_from_key(skeleton$row_key)

  df_uni <- merge(skeleton, df_uni, by = c("row_key", "label"), all.x = TRUE, sort = FALSE)
  df_multi <- merge(skeleton, df_multi, by = c("row_key", "label"), all.x = TRUE, sort = FALSE)

  df_uni$is_header[is.na(df_uni$is_header)] <- FALSE
  df_uni$is_data[is.na(df_uni$is_data)] <- FALSE
  df_multi$is_header[is.na(df_multi$is_header)] <- FALSE
  df_multi$is_data[is.na(df_multi$is_data)] <- FALSE
  df_uni$ref[is.na(df_uni$ref)] <- FALSE
  df_multi$ref[is.na(df_multi$ref)] <- FALSE

  # preserve label markdown from uni panel
  label_map_raw <- df_uni$label_clean
  label_map_raw[is.na(label_map_raw)] <- paste0(
    "<span style='color:white;'>..</span>",
    df_uni$label[is.na(label_map_raw)]
  )

  compute_is_sig <- function(df, approach, alpha, ref_line_val) {
    if (identical(approach, "linear")) {
      df$is_sig <- dplyr::case_when(
        !df$is_data ~ FALSE,
        !is.na(df$p.value) & df$p.value < alpha ~ TRUE,
        TRUE ~ FALSE
      )
    } else {
      df$is_sig <- dplyr::case_when(
        !df$is_data ~ FALSE,
        is.finite(df$conf.low) & is.finite(df$conf.high) &
          ((df$conf.low > ref_line_val) | (df$conf.high < ref_line_val)) ~ TRUE,
        TRUE ~ FALSE
      )
    }
    df
  }

  df_uni <- compute_is_sig(df_uni, approach_uni, alpha, ref_uni)
  df_multi <- compute_is_sig(df_multi, approach_multi, alpha, ref_multi)

  df_uni$row_id <- factor(seq_len(nrow(df_uni)), levels = rev(seq_len(nrow(df_uni))))
  df_multi$row_id <- factor(seq_len(nrow(df_multi)), levels = rev(seq_len(nrow(df_multi))))

  label_map <- stats::setNames(label_map_raw, as.character(df_uni$row_id))

  fill_vals <- c(
    "FALSE" = point_color,
    "TRUE" = if (!is.null(sig_color)) sig_color else point_color
  )
  line_vals <- c(
    "FALSE" = errorbar_color,
    "TRUE" = if (!is.null(sig_errorbar_color)) sig_errorbar_color else errorbar_color
  )

  build_panel <- function(df, panel_title, x_label, ref_line_val, use_log,
                          xlim = NULL, breaks = NULL, show_y = TRUE) {

    if (is.null(xlim)) {
      vals <- c(df$conf.low, df$conf.high, df$estimate, ref_line_val)
      vals <- vals[is.finite(vals)]
      if (length(vals) >= 2L) {
        rng <- range(vals, na.rm = TRUE)
        if (use_log) {
          lower <- max(min(rng[1], ref_line_val, na.rm = TRUE), .Machine$double.eps * 10)
          upper <- max(rng[2], ref_line_val, na.rm = TRUE)
        } else {
          span <- diff(rng)
          pad <- if (is.finite(span) && span > 0) 0.05 * span else 0.1
          lower <- min(rng[1], ref_line_val, na.rm = TRUE) - pad
          upper <- max(rng[2], ref_line_val, na.rm = TRUE) + pad
        }
        xlim <- c(lower, upper)
      }
    }

    p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$estimate, y = .data$row_id)) +
      .add_h_ci(df[df$is_data, , drop = FALSE], linewidth = ci_linewidth) +
      ggplot2::geom_point(
        data = df[df$is_data, , drop = FALSE],
        ggplot2::aes(fill = .data$is_sig),
        shape = 21,
        size = point_size,
        stroke = point_stroke,
        show.legend = FALSE,
        na.rm = TRUE
      ) +
      ggplot2::geom_vline(
        xintercept = ref_line_val,
        linetype = "dashed",
        colour = "gray60"
      ) +
      ggplot2::scale_fill_manual(values = fill_vals, guide = "none") +
      ggplot2::scale_color_manual(values = line_vals, guide = "none") +
      ggplot2::scale_y_discrete(
        labels = label_map,
        limits = levels(df$row_id),
        drop = FALSE
      ) +
      ggplot2::labs(title = panel_title, x = x_label, y = NULL) +
      .plot_reg_theme(base_size = base_size) +
      ggplot2::theme(
        axis.text.y = if (show_y) ggtext::element_markdown(hjust = 0,
                                                           colour = "#1F2937")
        else ggplot2::element_blank(),
        axis.text.y.left = if (show_y) ggtext::element_markdown(hjust = 0,
                                                                colour = "#1F2937")
        else ggplot2::element_blank(),
        axis.title.y = ggplot2::element_blank()
      )

    if (use_log) {
      if (is.null(breaks)) {
        breaks <- .plot_log_breaks(xlim)
      }
      p <- p + ggplot2::scale_x_log10(breaks = breaks)
    } else if (!is.null(breaks)) {
      p <- p + ggplot2::scale_x_continuous(breaks = breaks)
    }
    if (!is.null(xlim)) {
      p <- p + ggplot2::coord_cartesian(xlim = xlim)
    }

    p
  }

  p1 <- build_panel(
    df = df_uni,
    panel_title = title_uni,
    x_label = xlab_uni,
    ref_line_val = ref_uni,
    use_log = log_x_uni,
    xlim = xlim_uni,
    breaks = breaks_uni,
    show_y = TRUE
  )

  p2 <- build_panel(
    df = df_multi,
    panel_title = title_multi,
    x_label = xlab_multi,
    ref_line_val = ref_multi,
    use_log = log_x_multi,
    xlim = xlim_multi,
    breaks = breaks_multi,
    show_y = FALSE
  )

  out <- patchwork::wrap_plots(p1, p2, ncol = 2, widths = c(1.2, 1))

  caption <- .plot_caption(
    tbl = tbl_multi,
    caption = caption,
    show_adjustment_note = show_adjustment_note,
    show_ref = show_ref,
    has_ref = any(df_uni$ref %in% TRUE) || any(df_multi$ref %in% TRUE)
  )
  if (!is.null(caption)) {
    out <- out + patchwork::plot_annotation(caption = caption)
  }

  out
}

lung_survival_summary_data <- function() {
  data_lungcancer |>
    dplyr::mutate(
      trt = factor(trt, levels = c(1, 2),
                   labels = c("Standard treatment", "Test treatment")),
      prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
    )
}

test_that("survival_summary returns grouped Kaplan-Meier summary table", {
  skip_if_not_installed("survival")

  df <- lung_survival_summary_data()

  res <- survival_summary(
    data = df,
    time = time,
    event = status,
    by = trt
  )

  expect_s3_class(res, "gtregression")
  expect_s3_class(res, "survival_summary")
  expect_s3_class(res$table, "flextable")
  expect_s3_class(res$fit, "survfit")
  expect_equal(res$source, "survival_summary")
  expect_equal(res$time, "time")
  expect_equal(res$event, "status")
  expect_equal(res$by, "trt")
  expect_equal(res$table_body$Group, c("Standard treatment", "Test treatment"))
  expect_equal(res$table_body$N, c(69, 68))
  expect_equal(res$table_body$Events, c(64, 64))
  expect_equal(res$table_body$Censored, c(5, 4))
  expect_equal(res$table_body$Median, c(103, 52.5))
  expect_true("Median survival (95% CI)" %in% names(res$table_display))
})

test_that("survival_summary returns overall tibble with quoted names", {
  skip_if_not_installed("survival")

  df <- lung_survival_summary_data()
  time_var <- "time"
  event_var <- "status"

  out <- survival_summary(
    data = df,
    time = time_var,
    event = event_var,
    format = tibble
  )

  expect_s3_class(out, "tbl_df")
  expect_equal(out$Group, "Overall")
  expect_equal(out$N, 137)
  expect_equal(out$Events, 128)
  expect_equal(out$Censored, 9)
  expect_equal(out$Median, 80)
})

test_that("survival_summary returns gt table", {
  skip_if_not_installed("survival")
  skip_if_not_installed("gt")

  df <- lung_survival_summary_data()

  res <- survival_summary(
    data = df,
    time = time,
    event = status,
    by = trt,
    format = gt
  )

  expect_s3_class(res$table, "gt_tbl")
})

test_that("survival_summary validates inputs", {
  skip_if_not_installed("survival")

  df <- lung_survival_summary_data()
  df$bad_event <- 0
  df$one_group <- "A"

  expect_error(
    survival_summary(data = "not data", time = time, event = status),
    "`data` must be"
  )
  expect_error(
    survival_summary(data = df, time = missing_time, event = status),
    "`time` must be"
  )
  expect_error(
    survival_summary(data = df, time = time, event = missing_event),
    "`event` must be"
  )
  expect_error(
    survival_summary(data = df, time = time, event = bad_event),
    "`event` must include at least one event"
  )
  expect_error(
    survival_summary(data = df, time = time, event = status, by = one_group),
    "`by` must contain at least two"
  )
  expect_error(
    survival_summary(data = df, time = time, event = status, digits = -1),
    "`digits` must be"
  )
})

test_that("data_endometrial is available for Firth logistic examples", {
  expect_s3_class(data_endometrial, "data.frame")
  expect_equal(nrow(data_endometrial), 79)
  expect_equal(names(data_endometrial), c("NV", "PI", "EH", "HG"))
  expect_equal(unname(table(data_endometrial$HG, data_endometrial$NV)["0", "1"]), 0)
})

test_that("data_diabetes_mediation is available and suitable for mediation examples", {
  expect_s3_class(data_diabetes_mediation, "data.frame")
  expect_named(data_diabetes_mediation, c(
    "diabetes", "obesity", "glucose", "bmi", "age", "blood_pressure",
    "pregnancies", "diabetes_pedigree"
  ))
  expect_gt(nrow(data_diabetes_mediation), 500)
  expect_equal(levels(data_diabetes_mediation$diabetes), c("No", "Yes"))
  expect_equal(levels(data_diabetes_mediation$obesity), c("No", "Yes"))
  expect_true(is.numeric(data_diabetes_mediation$glucose))
  expect_true(is.numeric(data_diabetes_mediation$bmi))
  expect_false(anyNA(data_diabetes_mediation))
  expect_equal(attr(data_diabetes_mediation$glucose, "label"), "Plasma glucose")
})

---
title: "Start Here: Model to Manuscript"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Start Here: Model to Manuscript}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo=FALSE, out.width='160px'}
logo_candidates <- c(
  "man/figures/gtregression_hex.png",
  "../man/figures/gtregression_hex.png"
)
logo_path <- logo_candidates[file.exists(logo_candidates)][1]

if (!is.na(logo_path)) {
  knitr::include_graphics(logo_path)
}
```

# gtregression

**Publication-ready regression tables and plots for real-world health data.**

`gtregression` helps you fit, adjust, stratify, visualise, and export
regression results with approachable R syntax. It supports logistic,
log-binomial, Poisson, robust Poisson, negative binomial, Cox survival,
parametric survival, and linear regression. `flextable` is the default table
engine, so outputs are Word-friendly from the start; `format = gt` remains
available for HTML-first workflows.

## What You Can Make

- Clean descriptive tables.
- Univariable and multivariable regression tables.
- Adjusted models with clear footnotes.
- Stratified regression outputs.
- Kaplan-Meier curves, survival summaries, Cox models, and parametric survival
  models.
- Forest plots and publication-style forest tables.
- Model diagnostics, model selection, confounding, and interaction checks.
- HTML, PDF, PNG, and Word outputs.

## What Powers the Package

`gtregression` is a readable interface over standard R modelling and reporting
packages. The fitted models remain available inside the returned objects, so
users can inspect the analysis behind the displayed table.

| Area | Core packages used |
|------------------------------------|------------------------------------|
| Data handling | `dplyr`, `purrr`, `tibble`, `rlang` |
| Regression and survival models | `stats`, `MASS`, `survival`, `risks`, `logistf` |
| Robust inference and model tidying | `sandwich`, `lmtest`, `broom`, `broom.helpers` |
| Tables and Word output | `flextable`, `officer`, `gt` |
| Plots and forest plots | `ggplot2`, `patchwork`, `forestploter`, `scales` |

## Install

```r
install.packages("gtregression")

# Development version
devtools::install_github("ThinkDenominator/gtregression")
```

## Prepare Example Data

The articles use `data_birthwt`, a small built-in dataset that is easy to
learn with.

```{r setup-birthwt, message=FALSE, warning=FALSE}
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = ifelse(ptl > 0, "Yes", "No"),
    ftv_cat = case_when(
      ftv == 0 ~ "None",
      ftv == 1 ~ "One",
      ftv >= 2 ~ "Two or more"
    )
  ) |>
  mutate(
    ptl_cat = factor(ptl_cat, levels = c("No", "Yes")),
    ftv_cat = factor(ftv_cat, levels = c("None", "One", "Two or more"))
  )

birthwt_exposures <- c(
  "age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat", "ftv_cat"
)

attr(birthwt_data$age, "label") <- "Maternal age"
attr(birthwt_data$lwt, "label") <- "Maternal weight"
attr(birthwt_data$race, "label") <- "Maternal race"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"
attr(birthwt_data$ui, "label") <- "Uterine irritability"
attr(birthwt_data$ptl_cat, "label") <- "Previous preterm labour"
attr(birthwt_data$ftv_cat, "label") <- "First trimester visits"
```

## Five-Minute Workflow

### Describe

```{r quick-describe, message=FALSE, warning=FALSE}
birthwt_summary <- descriptive_table(
  data = birthwt_data,
  exposures = birthwt_exposures,
  by = low,
  percent = column,
  show_overall = last,
  theme = clinical
)

birthwt_summary$table
```

### Model

```{r quick-model, message=FALSE, warning=FALSE}
birthwt_uni <- uni_reg(
  data = birthwt_data,
  outcome = low,
  exposures = birthwt_exposures,
  approach = logit,
  theme = clinical
)

birthwt_multi <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat", "ftv_cat"),
  adjust_for = c("age", "lwt", "race"),
  approach = logit,
  theme = striped
)

birthwt_multi$table
```

### Visualise

```{r quick-plot, fig.width=12, fig.height=8,message=FALSE, warning=FALSE}
plot_reg(
  birthwt_multi,
  title = "Adjusted Regression for Low Birth Weight"
)

forest_reg(forest_df(birthwt_uni, birthwt_multi))
```

### Merge and Polish

```{r quick-merge, message=FALSE, warning=FALSE}
birthwt_final <- merge_tables(
  birthwt_summary,
  birthwt_uni,
  birthwt_multi,
  spanners = c("Clinical profile", "Crude OR", "Adjusted OR")
)

birthwt_final <- modify_table(
  birthwt_final,
  caption = "Clinical profile and regression estimates for low birth weight",
  caveat = "Adjusted estimates are adjusted for maternal age, maternal weight, and maternal race."
)

birthwt_final$table
```

Save helpers return file paths and use `tempdir()` when no directory is supplied,
which keeps examples CRAN-safe.

```{r quick-save, eval=FALSE}
save_table(birthwt_final, filename = "birthwt-table", format = html)
save_docx(tables = birthwt_final, filename = "birthwt-report")
```

## Where To Go Next

- [**Descriptive Tables**](descriptive-tables.html): build baseline tables users
  can read.
- [**Regression Tables**](regression-tables.html): create crude and adjusted
  publication-ready outputs.
- [**Survival Analysis**](survival-analysis.html): Kaplan-Meier curves, survival
  summaries, Cox regression, parametric survival models, and survival
  predictions.
- [**Causal Mediation**](causal-mediation.html): estimate direct, indirect,
  total, and proportion mediated effects with clear causal caveats.
- [**Visualise Results**](visualise-results.html): plot regression estimates and
  forest tables.
- [**Stratified Analysis**](stratified-analysis.html): repeat models across
  subgroups.
- [**Diagnostics**](diagnostics-selection.html): check convergence,
  collinearity, and model selection.
- [**Confounding & Interaction**](confounding-interaction.html): support
  interpretation and model decisions.
- [**Customize & Export**](customize-export.html): polish and save tables,
  plots, and reports.

---
title: "Descriptive Tables"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Descriptive Tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Start with a table that people can actually read. `descriptive_table()` creates
publication-ready summaries for continuous, categorical, and mixed exposure
sets.

```{r desc-setup, message=FALSE, warning=FALSE}
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes")),
    ftv_cat = factor(case_when(
      ftv == 0 ~ "None",
      ftv == 1 ~ "One",
      ftv >= 2 ~ "Two or more"
    ), levels = c("None", "One", "Two or more"))
  )

birthwt_exposures <- c(
  "age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat", "ftv_cat"
)

attr(birthwt_data$age, "label") <- "Maternal age"
attr(birthwt_data$lwt, "label") <- "Maternal weight"
attr(birthwt_data$race, "label") <- "Maternal race"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"
attr(birthwt_data$ui, "label") <- "Uterine irritability"
attr(birthwt_data$ptl_cat, "label") <- "Previous preterm labour"
attr(birthwt_data$ftv_cat, "label") <- "First trimester visits"
```

## Column Percentages

Use column percentages when the table is grouped by outcome or another column.
Continuous variables are shown as median (IQR) by default, and variable labels
are picked up automatically when variables have a `"label"` attribute.

```{r desc-column, message=FALSE, warning=FALSE}
desc_column <- descriptive_table(
  data = birthwt_data,
  exposures = birthwt_exposures,
  by = "low",
  percent = "column",
  show_overall = "last",
  theme = clinical
)

desc_column
```

## Row Percentages

Use row percentages when the question is how each exposure level is distributed
across groups. Common option values can be written with or without quotes.

```{r desc-row, message=FALSE, warning=FALSE}
descriptive_table(
  data = birthwt_data,
  exposures = birthwt_exposures,
  by = "low",
  percent = "row",
  show_overall = "first",
  show_missing = no,
  theme = striped
)$table
```

## Summary Choices

Use `statistic` when continuous variables need a different summary. A single
value applies to all numeric variables. A named vector lets you mix summaries,
including treating numeric ordinal variables as categorical.

```{r desc-statistic, message=FALSE, warning=FALSE}
descriptive_table(
  data = birthwt_data,
  exposures = c("age", "lwt", "ftv", "smoke"),
  by = low,
  statistic = c(
    age = mean,
    lwt = median,
    ftv = categorical
  ),
  percent = column,
  show_missing = no
)
```

The quoted form is equivalent and often clearer in saved scripts:

```{r desc-statistic-quoted, eval=FALSE}
descriptive_table(
  data = birthwt_data,
  exposures = c("age", "lwt", "ftv", "smoke"),
  by = "low",
  statistic = c(
    age = "mean",
    lwt = "median",
    ftv = "categorical"
  )
)
```

## Output Format

`flextable` is the default because it behaves well in Word workflows. Use
`format = gt` when the output is mainly for HTML or pkgdown.

```{r desc-flex, message=FALSE, warning=FALSE}
desc_gt <- descriptive_table(
  data = birthwt_data,
  exposures = birthwt_exposures,
  by = "low",
  percent = "column",
  format = gt
)

desc_gt$table
```

## What To Inspect

- `$table`: rendered `gt` or `flextable` output.
- `$table_body`: clean data behind the table.
- `$variable_labels`: labels used for display; raw variable names remain in
  `$table_body` for reliable merging and modification.
- `$format`: output format used by the table builder.

---
title: "Confounding and Interaction"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Confounding and Interaction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Confounding asks whether adjustment changes the exposure effect. Interaction
asks whether the exposure effect differs across another variable.

These checks are screening aids for viewing and organising results. Use DAGs,
subject-matter knowledge, and study design to decide which variables are
confounders or effect modifiers. Automated change-in-estimate and interaction
checks should not be used as the sole basis for model adjustment.

## Which Function Should I Use?

| Question | Use | Why |
|---|---|---|
| Could these candidate variables be confounders or effect modifiers? | `identify_confounder()` | Screens crude, adjusted, Mantel-Haenszel, and interaction signals together. |
| Does this planned interaction term improve the model? | `interaction_models()` | Compares models with and without `exposure:effect_modifier` using LRT or Wald tests. |
| Which automatically generated candidate model has the strongest fit? | `select_models()` | Fits and ranks a defined set of candidate models using model-fit statistics. |
| How do my named, fitted gtregression models differ? | `compare_models()` | Compares supplied fitted models, checks analysis-sample consistency, and reports fit statistics transparently. |
| Could part of an exposure-outcome association operate through a mediator? | `mediation_analysis()` | Estimates direct, indirect, total, and proportion mediated effects with explicit causal caveats. |
| Do I need a Mantel-Haenszel estimate? | `identify_confounder(method = "mh")` or `identify_confounder(method = "both")` | MH is a stratified pooled estimate for eligible binary/categorical settings, not a formal interaction-term test. |

A practical workflow is:

1. Use DAGs, prior literature, and study design to list important variables.
2. Use `identify_confounder()` to organise screening evidence for candidate
   confounders or effect modifiers.
3. Use `interaction_models()` when you have a planned interaction hypothesis.
4. Use `mediation_analysis()` when the mediator is part of a planned causal
   question and the temporal order is defensible.
5. If interaction is important, consider stratified reporting with
   `stratified_uni_reg()` or `stratified_multi_reg()`.

## How the Interaction Tests Differ

`identify_confounder()` is a candidate-by-candidate screening aid. For each
exposure and potential confounder, it compares `outcome ~ exposure` with
`outcome ~ exposure + potential_confounder`, examines stratum-specific
estimates, and screens the candidate interaction:

```r
outcome ~ exposure + potential_confounder + exposure:potential_confounder
```

No other covariates are included in this interaction screen. Its interaction
p-value helps organise early evidence; it is not a final covariate-adjusted
interaction analysis.

Use `interaction_models()` for a planned interaction analysis. Its `covariates`
are included in both nested models, preserving a common adjusted analysis
sample. For example, `covariates = c(age, lwt)` fits:

```r
outcome ~ exposure + effect_modifier + age + lwt
outcome ~ exposure + effect_modifier + age + lwt + exposure:effect_modifier
```

Choose covariates using the study design, a DAG, and subject-matter knowledge.
Do not automatically adjust for mediators or colliders.

The formatted interaction table labels the **effect modifier tested**, displays
the outcome, and records the two fitted model formulas in its table notes.

```{r ci-setup, message=FALSE, warning=FALSE}
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
  )

attr(birthwt_data$race, "label") <- "Maternal race"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"
```

## Identify Confounders

Use `method = "change"` for the model-based change-in-estimate method. Use
`method = "mh"` or `method = "both"` when Mantel-Haenszel is appropriate.
The output is intentionally tidy and intended for viewing, not as a final
publication table. Calling the object displays the formatted table; use
`$summary` when you want the underlying tibble. Its table notes identify the
outcome and the three candidate-level models: crude, adjusted for the potential
confounder, and the interaction screen.

```{r confounder, message=FALSE, warning=FALSE}
confounder_check <- identify_confounder(
  data = birthwt_data,
  outcome = low,
  exposure = smoke,
  potential_confounder = c("race", "ht"),
  approach = logit,
  method = both,
  format = gt
)

confounder_check$table
```

The underlying summary remains available for inspection or further filtering.

```{r confounder-summary, message=FALSE, warning=FALSE}
confounder_check$summary
```

## Mantel-Haenszel Estimate

Mantel-Haenszel is useful when the question is whether a stratified pooled
estimate differs meaningfully from the crude estimate. It is available for
eligible binary/categorical settings. It is not the same as fitting an
interaction term.

```{r confounder-mh, message=FALSE, warning=FALSE}
identify_confounder(
  data = birthwt_data,
  outcome = low,
  exposure = smoke,
  potential_confounder = race,
  approach = logit,
  method = mh,
  format = flextable
)$table
```

## Test Interaction

`interaction_models()` compares models with and without the interaction term.
It is deliberately model-based and uses `LRT` or `Wald`, not
Mantel-Haenszel. Use it when the interaction term is planned or supported by
clinical, causal, or subject-matter reasoning. Covariates are included in both
models, so this is the appropriate function for an adjusted interaction test.

```{r interaction, message=FALSE, warning=FALSE}
interaction_check <- interaction_models(
  data = birthwt_data,
  outcome = low,
  exposure = smoke,
  effect_modifier = race,
  covariates = c("age", "lwt"),
  approach = logit,
  test = LRT,
  format = gt
)

interaction_check$table
```

## Survival Confounding and Interaction

The same grammar works for survival models. For `cox` and `surv`/`survreg`,
use `time` and `event` instead of `outcome`. Cox models report hazard-ratio
style estimates; parametric survival models report time-ratio style estimates.
Mantel-Haenszel screening is for binary outcome settings, so survival examples
use model-based change-in-estimate and interaction checks.

```{r survival-confounder, message=FALSE, warning=FALSE}
lung_data <- data_lungcancer |>
  dplyr::mutate(
    trt = factor(trt, levels = c(1, 2),
                 labels = c("Standard treatment", "Test treatment")),
    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
  )

survival_confounder <- identify_confounder(
  data = lung_data,
  time = time,
  event = status,
  exposure = trt,
  potential_confounder = prior,
  approach = cox,
  method = change,
  format = gt
)

survival_confounder$table
```

```{r survival-interaction, message=FALSE, warning=FALSE}
survival_interaction <- interaction_models(
  data = lung_data,
  time = time,
  event = status,
  exposure = trt,
  effect_modifier = prior,
  covariates = c(age, karno),
  approach = cox,
  test = LRT,
  format = gt
)

survival_interaction$table
```

## Causal Mediation

`mediation_analysis()` asks whether part of an exposure-outcome association may
operate through a mediator. In this example, obesity is the exposure, plasma
glucose is the mediator, and diabetes is the outcome. The question is not only
"is obesity associated with diabetes?", but also "how much of that association
may operate through plasma glucose?".

The default output is a publication-ready flextable. The table reports:

- **Total effect**: the overall exposure-outcome association.
- **Direct effect**: the part not operating through the mediator.
- **Indirect effect**: the part operating through the mediator.
- **Proportion mediated**: the share of the total effect attributed to the
  indirect pathway.

For logistic outcomes, effects are reported as predicted probability
differences, not odds ratios. For example, an estimate of `0.068` means about a
6.8 percentage-point difference on the predicted probability scale.

Treat mediation output as causal only when the usual mediation assumptions are
supported by study design, DAGs, temporality, and subject-matter knowledge. In
particular, there should be no unmeasured exposure-outcome,
exposure-mediator, or mediator-outcome confounding, and the mediator should
occur before the outcome.

```{r mediation, message=FALSE, warning=FALSE}
data("data_diabetes_mediation", package = "gtregression")

diabetes_med <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = obesity,
  mediator = glucose,
  outcome = diabetes,
  covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
  outcome_approach = logit,
  sims = 100,
  seed = 123
)

diabetes_med
```

The underlying values remain available for audit, reporting, or custom
formatting.

```{r mediation-body}
diabetes_med$table_body
```

Use `format = gt` when preparing HTML-first outputs such as websites or
teaching pages.

```{r mediation-gt, message=FALSE, warning=FALSE}
med_gt <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = obesity,
  mediator = glucose,
  outcome = diabetes,
  covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
  outcome_approach = logit,
  format = gt,
  sims = 100,
  seed = 123
)

med_gt$table
```

`plot_mediation()` draws the same planned causal structure as a path diagram.
It is useful for teaching, presentations, or checking that the exposure,
mediator, and outcome have been specified as intended.

```{r mediation-plot, message=FALSE, warning=FALSE, fig.width=7, fig.height=4}
plot_mediation(diabetes_med)
```

If the estimates make the plot too busy, hide them and use the diagram only to
show the assumed path structure.

```{r mediation-plot-simple, message=FALSE, warning=FALSE, fig.width=7, fig.height=4}
plot_mediation(diabetes_med, show_estimates = FALSE)
```

## Overlap and Difference

| Topic | `identify_confounder()` | `interaction_models()` |
|---|---|---|
| Main purpose | Organises candidate confounder and effect-modifier screening signals. | Tests a planned exposure-by-modifier term. |
| Typical input | Exposure plus one or more candidate variables. | One exposure and one effect modifier. |
| Confounding | Crude vs adjusted change-in-estimate; optional Mantel-Haenszel comparison. | Not designed for confounder selection. |
| Effect modification | Screening signal from stratum-specific estimates and interaction p-value. | Model comparison using LRT or Wald test. |
| Best use | Early review of candidate variables, with DAGs and judgement. | Focused test of a clinically or biologically plausible interaction. |
| Output status | Viewing aid, not publication-ready evidence by itself. | Viewing aid; report with stratum-specific estimates when relevant. |

`mediation_analysis()` is different from both functions. It decomposes one
planned exposure-outcome relationship into direct and mediator-related
components; it does not select confounders or test whether effects differ across
strata.

## What To Inspect

- `identify_confounder()`: `$summary`, `$table`, `$details`, `$mh_estimate`,
  `$mh_status`, `$decision`, and `$recommendation`.
- `interaction_models()`: `$summary`, `$table`, `$p_value`, `$decision`, and
  fitted model objects.
- `mediation_analysis()`: `$table`, `$table_body`, `$models`, `$boot`,
  `$values`, `$variable_labels`, and `$complete_data`.
- Use subject-matter knowledge with these outputs. The functions support
  interpretation; they do not replace the study design.

---
title: "Function Options"
description: "A quick reference for gtregression arguments, defaults, available choices, and what each option does."
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Function Options}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

# Function Options at a Glance

This page is a practical companion to the function reference. It lists the
important arguments, their defaults, accepted choices, and what each option
does.

Most user-facing arguments accept quoted names and bare names. Quoted names are
recommended inside scripts, functions, and Shiny apps because they are easier to
program with.

## Common Choices

| Option family | Available choices | Meaning |
|---|---|---|
| Table `format` | `"flextable"`, `"gt"`, `"tibble"` where supported | `flextable` is Word-friendly and the package default for publication tables; `gt` is useful for HTML/pkgdown; `tibble` is useful for inspection and downstream work. |
| Table `theme` | `"minimal"`, `"clinical"`, `"striped"`, `"shaded"`, `"jama"` | Preset table styling. `theme = "minimal"` is the default in most table functions. |
| Regression `approach` | `"logit"`, `"firth"`, `"logbinomial"`, `"poisson"`, `"robpoisson"`, `"negbin"`, `"linear"` | Chooses the model family for standard regression tables. |
| Survival `approach` | `"cox"`, `"survreg"` where supported | Enables Cox or parametric survival workflows in selected helper functions. |
| Parametric survival `distribution` | `"weibull"`, `"exponential"`, `"lognormal"`, `"loglogistic"` | Distribution used by parametric survival regression. |
| Reference rows | `show_ref = TRUE` or `FALSE` | `TRUE` displays reference categories as `Ref.`; `FALSE` makes compact tables and plots. |
| Survival sample display | `"events"`, `"n"`, `"both"`, `"none"` | Controls whether stratified Cox/survival tables show event counts, N, both, or neither. |

## Describe and Inspect Data

### `descriptive_table()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Analysis dataset. |
| `exposures` | required | column names | Variables to summarise. |
| `by` | `NULL` | one column name or `NULL` | Creates grouped summary columns. |
| `percent` | `"column"` | `"column"`, `"row"` | Chooses denominator for categorical percentages. |
| `digits` | `1` | whole number | Number of decimal places. |
| `show_missing` | `"ifany"` | `"ifany"`, `"no"` | Shows missing rows only when present, or hides them. |
| `show_dichotomous` | `"all_levels"` | `"all_levels"`, `"single_row"` | Shows both binary levels by default; `single_row` shows one selected level. |
| `show_overall` | `"no"` | `"no"`, `"first"`, `"last"` | Adds an overall column before or after grouped columns. |
| `statistic` | `NULL` | `mean`, `median`, `mode`, `count`, `categorical`; can be named vector | Chooses numeric summary style or forces a numeric variable to display as categorical. |
| `value` | `NULL` | named vector of displayed levels | Selects the displayed level when `show_dichotomous = "single_row"`. |
| `format` | `"flextable"` | `"flextable"`, `"gt"` | Output table engine. |
| `theme` | `"minimal"` | table theme preset | Table styling. |

```r
descriptive_table(
  data = birthwt_data,
  exposures = c(age, lwt, smoke, race),
  by = low,
  statistic = c(age = mean, lwt = median),
  show_overall = last
)
```

### `dissect()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Dataset to inspect. |
| `verbose` | `FALSE` | `TRUE`, `FALSE` | Prints extra console guidance when `TRUE`. |
| `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Publication table or inspectable tibble. |

## Regression Tables

### `uni_reg()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Analysis dataset. |
| `outcome` | required | one column name | Outcome variable. |
| `exposures` | required | column names | Fits one univariable model per exposure. |
| `approach` | `"logit"` | `"logit"`, `"firth"`, `"logbinomial"`, `"poisson"`, `"robpoisson"`, `"negbin"`, `"linear"` | Model family and effect measure. |
| `format` | `"flextable"` | `"flextable"`, `"gt"` | Output table engine. |
| `theme` | `"minimal"` | table theme preset | Table styling. |
| `model_stats` | `FALSE` | `TRUE`, `FALSE` | Attaches AIC, BIC, log-likelihood, pseudo R2 or R2 where relevant, and N. |
| `show_ref` | `TRUE` | `TRUE`, `FALSE` | Displays or hides reference category rows. |

### `multi_reg()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Analysis dataset. |
| `outcome` | required | one column name | Outcome variable. |
| `exposures` | required | column names | Variables to report. |
| `adjust_for` | `NULL` | column names or `NULL` | Adds adjustment covariates while reporting only `exposures`. |
| `interaction` | `NULL` | interaction specification or `NULL` | Fits interaction terms, for example `smoke*race`. |
| `approach` | `"logit"` | `"logit"`, `"firth"`, `"logbinomial"`, `"poisson"`, `"robpoisson"`, `"negbin"`, `"linear"` | Model family and effect measure. |
| `format` | `"flextable"` | `"flextable"`, `"gt"` | Output table engine. |
| `theme` | `"minimal"` | table theme preset | Table styling. |
| `model_stats` | `FALSE` | `TRUE`, `FALSE` | Attaches model-fit statistics. |
| `show_ref` | `TRUE` | `TRUE`, `FALSE` | Displays or hides reference category rows. |

### `stratified_uni_reg()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Analysis dataset. |
| `outcome` | required | one column name | Outcome variable. |
| `exposures` | required | column names | Fits one univariable model per exposure within each stratum. |
| `stratifier` | required | one column name | Splits the analysis into strata. |
| `approach` | `"logit"` | standard regression approaches | Model family and effect measure. |
| `format` | `"flextable"` | `"flextable"`, `"gt"` | Output table engine. |
| `theme` | `"minimal"` | table theme preset | Table styling. |
| `show_ref` | `TRUE` | `TRUE`, `FALSE` | Displays or hides reference category rows. |

### `stratified_multi_reg()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Analysis dataset. |
| `outcome` | required | one column name | Outcome variable. |
| `exposures` | required | column names | Variables to report within each stratum. |
| `stratifier` | required | one column name | Splits the analysis into strata. |
| `adjust_for` | `NULL` | column names or `NULL` | Adds adjustment covariates within each stratum. |
| `interaction` | `NULL` | interaction specification or `NULL` | Fits interaction terms within each stratum. |
| `approach` | `"logit"` | standard regression approaches | Model family and effect measure. |
| `format` | `"flextable"` | `"flextable"`, `"gt"` | Output table engine. |
| `theme` | `"minimal"` | table theme preset | Table styling. |
| `show_ref` | `TRUE` | `TRUE`, `FALSE` | Displays or hides reference category rows. |

## Cox and Parametric Survival Regression

### `cox_reg()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Survival dataset. |
| `time` | required | one column name | Follow-up time. Zero follow-up is allowed; negative time is not. |
| `event` | required | one column name | Event indicator. |
| `exposures` | required | column names | Variables to report or use in a full multivariable Cox model. |
| `adjust_for` | `NULL` | column names or `NULL` | Adds adjustment covariates while reporting only `exposures`. |
| `stratifier` | `NULL` | one column name or `NULL` | Repeats Cox analyses within strata. |
| `interaction` | `NULL` | interaction specification or `NULL` | Fits interaction terms. |
| `multivariable` | `FALSE` | `TRUE`, `FALSE` | If `TRUE`, fits one Cox model using all variables in `exposures`. |
| `multivariate` | `NULL` | `TRUE`, `FALSE`, `NULL` | Backward-compatible alias for `multivariable`. |
| `format` | `"flextable"` | `"flextable"`, `"gt"` | Output table engine. |
| `theme` | `"minimal"` | table theme preset | Table styling. |
| `show_sample` | `"events"` | `"events"`, `"n"`, `"both"`, `"none"` | Controls displayed sample columns in stratified Cox tables. |
| `model_stats` | `FALSE` | `TRUE`, `FALSE` | Attaches AIC, BIC, log-likelihood, concordance, N, and events. |
| `show_ref` | `TRUE` | `TRUE`, `FALSE` | Displays or hides reference category rows. |

### `surv_reg()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Survival dataset. |
| `time` | required | one column name | Follow-up time. Zero follow-up is allowed; negative time is not. |
| `event` | required | one column name | Event indicator. |
| `exposures` | required | column names | Variables to report or use in a full multivariable model. |
| `adjust_for` | `NULL` | column names or `NULL` | Adds adjustment covariates while reporting only `exposures`. |
| `stratifier` | `NULL` | one column name or `NULL` | Repeats parametric survival analyses within strata. |
| `interaction` | `NULL` | interaction specification or `NULL` | Fits interaction terms. |
| `multivariable` | `FALSE` | `TRUE`, `FALSE` | If `TRUE`, fits one model using all variables in `exposures`. |
| `multivariate` | `NULL` | `TRUE`, `FALSE`, `NULL` | Backward-compatible alias for `multivariable`. |
| `distribution` | `"weibull"` | `"weibull"`, `"exponential"`, `"lognormal"`, `"loglogistic"` | Parametric survival distribution. |
| `format` | `"flextable"` | `"flextable"`, `"gt"` | Output table engine. |
| `theme` | `"minimal"` | table theme preset | Table styling. |
| `show_sample` | `"events"` | `"events"`, `"n"`, `"both"`, `"none"` | Controls displayed sample columns in stratified survival tables. |
| `model_stats` | `FALSE` | `TRUE`, `FALSE` | Attaches model-fit statistics. |
| `show_ref` | `TRUE` | `TRUE`, `FALSE` | Displays or hides reference category rows. |

## Kaplan-Meier and Survival Summaries

### `km_plot()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Survival dataset. |
| `time` | required | one column name | Follow-up time. |
| `event` | required | one column name | Event indicator. |
| `by` | `NULL` | one column name or `NULL` | Draws grouped curves when supplied. |
| `conf.int` | `TRUE` | `TRUE`, `FALSE` | Shows shaded confidence intervals. |
| `risk_table` | `TRUE` | `TRUE`, `FALSE` | Adds a number-at-risk table. |
| `p_value` | `TRUE` | `TRUE`, `FALSE` | Adds log-rank p-value for grouped curves. |
| `p_value_position` | `NULL` | numeric `c(x, y)` or `NULL` | Manually positions the p-value label. |
| `censor` | `TRUE` | `TRUE`, `FALSE` | Shows censor marks. |
| `break_time_by` | `NULL` | number or `NULL` | Controls x-axis and risk-table time breaks. |
| `xlim` | `NULL` | numeric range or `NULL` | Zooms the follow-up time axis. |
| `ylim` | `NULL` | numeric range or `NULL` | Zooms the survival probability axis, for example `c(0.5, 1)`. |
| `xlab` | `"Time"` | character | X-axis label. |
| `ylab` | `"Survival probability"` | character | Y-axis label. |
| `title` | `NULL` | character or `NULL` | Main title. |
| `subtitle` | `NULL` | character or `NULL` | Subtitle. |
| `caption` | `NULL` | character or `NULL` | Figure caption. |
| `title_size` | `NULL` | number or `NULL` | Custom title size for publication panels. |
| `title_face` | `"bold"` | `"plain"`, `"bold"`, `"italic"`, `"bold.italic"` | Title font face. |
| `legend_title` | `NULL` | character or `NULL` | Legend heading. |
| `legend_position` | `NULL` | `"bottom"`, `"top"`, `"right"`, `"left"`, `"none"`, `NULL` | Legend location. |
| `palette` | `NULL` | colour vector or `NULL` | Curve and confidence-band colours. |
| `y_percent` | `TRUE` | `TRUE`, `FALSE` | Displays survival probability as percentages. |
| `theme` | `"classic"` | `"classic"`, `"minimal"`, `"bw"`, `"light"`, `"none"` | ggplot theme style. |
| `grid` | `FALSE` | `TRUE`, `FALSE` | Adds grid lines when `TRUE`. |
| `base_size` | `13` | number | Base font size. |

### `survival_summary()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data`, `time`, `event` | required | data frame and column names | Defines the survival object. |
| `by` | `NULL` | one column name or `NULL` | Produces grouped summaries. |
| `digits` | `1` | whole number | Number of decimal places. |
| `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Output format. |
| `theme` | `"minimal"` | table theme preset | Table styling. |

### `survival_quantiles()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data`, `time`, `event` | required | data frame and column names | Defines the survival object. |
| `by` | `NULL` | one column name or `NULL` | Produces grouped quantiles. |
| `probs` | `c(0.25, 0.5, 0.75)` | probabilities between 0 and 1 | Survival-time quantiles to display. |
| `digits` | `1` | whole number | Number of decimal places. |
| `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Output format. |
| `theme` | `"minimal"` | table theme preset | Table styling. |

### `survival_prob()` and `km_risk_table()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data`, `time`, `event` | required | data frame and column names | Defines the survival object. |
| `by` | `NULL` | one column name or `NULL` | Produces grouped outputs. |
| `times` | required | numeric vector | Time points for survival probability or number at risk. |
| `digits` | `1` | whole number | Number of decimal places. |
| `extend` | `TRUE` | `TRUE`, `FALSE` | Allows reporting beyond the last event time where appropriate. |
| `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Output format. |
| `theme` | `"minimal"` | table theme preset | Table styling. |

### `logrank_test()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data`, `time`, `event`, `by` | required | data frame and column names | Defines grouped survival comparison. |
| `digits` | `2` | whole number | Number of decimal places. |
| `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Output format. |
| `theme` | `"minimal"` | table theme preset | Table styling. |

### `rmst_table()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data`, `time`, `event` | required | data frame and column names | Defines the survival object. |
| `by` | `NULL` | one column name or `NULL` | Produces grouped RMST estimates. |
| `tau` | required | number | Restriction time for RMST. |
| `digits` | `1` | whole number | Number of decimal places. |
| `conf.level` | `0.95` | number between 0 and 1 | Confidence level. |
| `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Output format. |
| `theme` | `"minimal"` | table theme preset | Table styling. |

### Parametric survival helpers

| Function | Argument | Default | Available options or input | What it does |
|---|---|---:|---|---|
| `surv_model_compare()` | `distributions` | four common distributions | `"weibull"`, `"exponential"`, `"lognormal"`, `"loglogistic"` | Compares candidate parametric survival distributions. |
| `surv_model_compare()` | `exposures`, `adjust_for` | required, `NULL` | column names | Defines predictors and adjustment covariates. |
| `surv_predict()` | `model`, `newdata`, `times` | required, `NULL`, required | fitted model, data frame, numeric times | Predicts survival probabilities. |
| `plot_surv_fit()` | `distributions` | four common distributions | distribution names | Draws fitted parametric survival curves. |
| `plot_surv_fit()` | `by`, `adjust_for` | `NULL`, `NULL` | column names or `NULL` | Groups curves or adjusts predicted curves. |
| `plot_surv_fit()` | `break_time_by`, `xlim`, `n_points` | `NULL`, `NULL`, `200` | number, range, number | Controls curve resolution and x-axis. |
| `check_ph()` | `transform` | `"km"` | `"km"`, `"rank"`, `"identity"` | Chooses time transform for proportional hazards test. |
| `check_ph()` | `alpha` | `0.05` | number | Threshold used to flag possible PH violation. |
| `check_ph()` | `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Output format. |

## Regression Plots and Model Fit

### `plot_reg()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `tbl` | required | gtregression regression object | Plots regression estimates. |
| `title`, `caption` | `NULL` | character or `NULL` | Plot title and caption. |
| `ref_line` | automatic | number or `NULL` | Null reference line; usually 1 for ratios and 0 for linear estimates. |
| `order_y` | `NULL` | character vector or `NULL` | Manually controls vertical order. |
| `log_x` | `FALSE` | `TRUE`, `FALSE` | Uses a linear x-axis by default. Set `TRUE` for the conventional log-scale display of odds, risk, hazard, or time ratios. |
| `xlim`, `breaks` | `NULL` | numeric vectors or `NULL` | Controls axis range and tick marks. |
| `point_color`, `errorbar_color` | `"#6B7280"` | colour values | Styles non-significant estimates and confidence intervals with neutral grey. |
| `sig_color`, `sig_errorbar_color` | `"#0072B2"` | colour values or `NULL` | Highlights estimates whose confidence interval excludes the null with accessible blue; use `NULL` to reuse the base colours. |
| `point_size`, `point_stroke`, `ci_linewidth` | `2.8`, `0.55`, `0.55` | positive numbers | Controls estimate marker size, marker outline, and confidence-interval thickness. |
| `base_size` | `12` | number | Base font size. Increase for a single large figure. |
| `show_ref` | `TRUE` | `TRUE`, `FALSE` | Displays or hides reference rows. |
| `alpha` | `0.05` | number | Significance threshold. |
| `show_adjustment_note` | `TRUE` | `TRUE`, `FALSE` | Adds adjustment note when available. |

### `plot_reg_combine()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `tbl_uni`, `tbl_multi` | required | two compatible regression objects | Plots crude and adjusted estimates side by side. |
| `title_uni`, `title_multi`, `caption` | `NULL` | character or `NULL` | Plot labels. |
| `ref_line`, `order_y`, `log_x` | automatic/`NULL`/`FALSE` | number, order vector, logical | Controls the null line, order, and scale; set `log_x = TRUE` for a ratio-scale log axis. |
| `xlim_uni`, `breaks_uni` | `NULL` | numeric vectors or `NULL` | Axis control for the crude plot. |
| `xlim_multi`, `breaks_multi` | `NULL` | numeric vectors or `NULL` | Axis control for the adjusted plot. |
| `point_color`, `errorbar_color` | `"#6B7280"` | colour values | Styles non-significant estimates and confidence intervals with neutral grey. |
| `sig_color`, `sig_errorbar_color` | `"#0072B2"` | colour values or `NULL` | Highlights estimates whose confidence interval excludes the null with accessible blue. |
| `point_size`, `point_stroke`, `ci_linewidth` | `2.8`, `0.55`, `0.55` | positive numbers | Controls estimate marker size, marker outline, and confidence-interval thickness. |
| `base_size`, `show_ref`, `alpha` | `12`, `TRUE`, `0.05` | number, logical, number | Controls readability, reference rows, and highlighting. |
| `show_adjustment_note` | `TRUE` | `TRUE`, `FALSE` | Adds adjustment note when available. |

### `plot_model_fit()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `model` | required | `uni_reg()`, `multi_reg()`, `cox_reg()`, `surv_reg()`, or fitted model | Object to diagnose. |
| `model_name` | `NULL` | model name or `NULL` | Selects one named model when the object stores several models. |
| `type` | `"auto"` | `"auto"`, `"all"`, `"residual"`, `"qq"`, `"scale_location"`, `"cooks"`, `"observed_predicted"`, `"calibration"` | Chooses diagnostic plot type. |
| `bins` | `10` | whole number | Number of bins for calibration plots. |
| `base_size` | `13` | number | Base font size. |

## Forest Tables and Forest Plots

### `forest_df()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `uni` | required | regression object, descriptive table, or stratified regression object | Main object used to create the forest-ready data. |
| `multi` | `NULL` | adjusted regression object or `NULL` | Adds adjusted estimates. |
| `desc` | `NULL` | descriptive table or `NULL` | Adds descriptive columns. |
| `digits` | `2` | whole number | Number of decimals for forest table estimates. |

### `forest_reg()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `df` | `NULL` | `forest_df()` output or `NULL` | Plots prepared forest data. |
| `uni`, `multi`, `desc` | `NULL` | gtregression objects | Builds and plots in one call. |
| `theme` | `NULL` | forestploter theme or `NULL` | Plot theme passed to forestploter. |
| `ci_col_width` | `20` | number | Width of CI drawing columns; increase when forest columns feel cramped. |
| `side` | `"right"` | `"right"`, `"left"` | Places forest columns to the right or left of text columns. |
| `quiet` | `TRUE` | `TRUE`, `FALSE` | Suppresses helpful messages. |
| `effects` | `NULL` | effect column names or `NULL` | Selects which estimate columns to draw. |
| `ticks_at` | `NULL` | numeric vector or list | Manually sets axis tick marks; useful when x-axis labels overlap. |
| `ticks_digits` | `NULL` | whole number or `NULL` | Controls tick-label precision. |
| `xlim` | `NULL` | numeric range or list | Manually sets forest x-axis limits. |
| `style_strata` | `TRUE` | `TRUE`, `FALSE` | Highlights stratum header rows. |
| `strata_fill` | `"#EAF2F1"` | colour value | Background colour for stratum rows. |
| `...` | passed on | forestploter options | Extra arguments forwarded to `forestploter::forest()`. |

### `save_forest()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `forest` | required | forest plot object | Object returned by `forest_reg()`. |
| `filename` | `"forest"` | file path or file stem | Output file. Uses a temporary location when no path is supplied. |
| `format` | `"pdf"` | `"pdf"`, `"png"`, `"tiff"`, `"jpg"` | Export file type. |
| `width`, `height` | `NULL` | inches or `NULL` | Manual canvas size. |
| `scale` | `1` | positive number | Scales automatic width and height. |
| `auto_size` | `TRUE` | `TRUE`, `FALSE` | Estimates a sensible canvas size from table dimensions. |
| `padding` | `0.25` | number | Extra white space around the saved forest plot. |
| `dpi` | `300` | number | Image resolution for raster formats. |

## Diagnostics, Selection, and Model Comparison

`select_models()` and `compare_models()` answer different questions.
`select_models()` generates and ranks models as part of a selection workflow.
`compare_models()` compares specific fitted models that you have named and
chosen, checks whether their analysis samples are comparable, and preserves all
fit statistics for transparent reporting. Use `interaction_models()` for one
planned interaction comparison and `identify_confounder()` for candidate-level
confounding/effect-modification screening rather than final causal decisions.

For linear models, typing `result$reg_check` displays a publication-ready
diagnostic table. The underlying rows remain available for scripts, for example
`result$reg_check$multivariable_model`. These checks complement, rather than
replace, residual and influence plots.

### `check_convergence()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data`, `exposures`, `outcome` | required | data frame and column names | Defines candidate models. |
| `approach` | `"logit"` | `"logit"`, `"logbinomial"`, `"poisson"`, `"robpoisson"`, `"negbin"` | Model family to test. |
| `multivariate` | `FALSE` | `TRUE`, `FALSE` | Checks one multivariable model instead of separate univariable models. |
| `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Output format. |

### `check_collinearity()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `model` | required | fitted model | Model to assess for collinearity. |
| `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Output format. |

### `select_models()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data`, `outcome`, `exposures` | required | data frame and column names | Defines candidate predictors. |
| `approach` | `"logit"` | standard regression approaches plus `"cox"` and `"survreg"` | Model family used for selection. |
| `time`, `event` | `NULL` | column names or `NULL` | Required for Cox and parametric survival selection. |
| `distribution` | `"weibull"` | survival distributions | Parametric survival distribution when `approach = "survreg"`. |
| `direction` | `"forward"` | `"forward"`, `"backward"`, `"both"` when supported | Stepwise selection direction. |
| `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Output format. |

### `compare_models()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `...` | required | gtregression model objects or fitted models | Candidate models to compare. |
| `model_names` | `NULL` | character vector or `NULL` | Overrides default model labels; otherwise object names are used when available. |
| `nested` | `TRUE` | `TRUE`, `FALSE` | Requests sequential likelihood-ratio comparison where appropriate. |
| `primary_exposure` | `NULL` | one term name or `NULL` | Displays primary estimate and percentage change. |
| `exponentiate` | `NULL` | `TRUE`, `FALSE`, `NULL` | Controls whether primary estimate is exponentiated. |
| `digits` | `2` | whole number | Number of decimals for estimates and fit statistics. |
| `p_digits` | `3` | whole number | Number of decimals for p-values. |
| `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Output format. |
| `theme` | `"minimal"` | table theme preset | Table styling. |

#### Compare Models in the gtregression App

The R function above compares already fitted gtregression objects. The app adds
a candidate-model builder in **Advanced > Compare models**, fits the requested
models, and then calls the same function.

| App control | Default | Available options or input | What it does |
|---|---:|---|---|
| Regression approach | `logit` | logistic, linear, Firth, log-binomial, Poisson, robust Poisson, negative binomial, Cox, parametric survival | Selects the gtregression fitting function used for every candidate. |
| Outcome | first eligible variable | one column | Common outcome for ordinary regression candidates. |
| Follow-up time, event | first eligible variables | numeric time and event columns | Replace outcome for Cox and parametric survival candidates. |
| Distribution | `weibull` | distributions supported by `surv_reg()` | Appears only for parametric survival regression. |
| Primary exposure to track | none | one predictor or none | Adds the estimate and percentage change from the first candidate. |
| Candidate name | generated model name | unique text | Becomes the visible model label in the comparison table and generated code. |
| Reported exposures | none | one or more predictors | Defines the candidate's focal predictors. These are included in the fitted model. |
| Adjustment variables | none | zero or more predictors | Adds planned covariates to that candidate. |
| Interaction | none | two predictors already in the candidate | Adds one interaction while retaining its main effects. |
| Add candidate | two initially | two to six candidates | Adds another independently specified model. |
| Remove candidate | not applicable | candidates above the minimum | Removes the last candidate while retaining at least two. |

Candidates are compared in their displayed order. Use meaningful unique names,
place the baseline model first, and include the tracked primary exposure in
every candidate. The app's **Code** panel contains the complete `multi_reg()`,
`cox_reg()`, or `surv_reg()` calls followed by `compare_models()`.

## Confounding, Interaction, and Mediation

### `identify_confounder()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Analysis dataset. |
| `outcome` | `NULL` | column name or `NULL` | Outcome for non-survival models. |
| `exposure` | required | one column name | Main exposure. |
| `potential_confounder` | required | one column name | Candidate screened as a potential confounder and effect modifier. |
| `approach` | `"logit"` | standard approaches plus `"cox"` and `"survreg"` | Model family. |
| `time`, `event` | `NULL` | column names or `NULL` | Required for survival approaches. |
| `distribution` | `"weibull"` | survival distributions | Parametric distribution for `approach = "survreg"`. |
| `method` | `"change"` | `"change"`, `"mh"`, `"both"` | Confounder assessment method. |
| `threshold` | `10` | number | Percent-change threshold for confounding. |
| `emm_threshold` | `10` | number | Percent-change threshold for effect-measure modification. |
| `emm_test` | `"interaction"` | `"interaction"`, `"both"`, `"estimate"` | How to assess effect modification. |
| `interaction_alpha` | `0.05` | number | P-value threshold for the candidate-only interaction screen; no other covariates are adjusted for. |
| `format` | `"flextable"` | `"flextable"`, `"gt"` | Output format. |
| `theme` | `"minimal"` | table theme preset | Table styling. |

### `interaction_models()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Analysis dataset. |
| `outcome` | `NULL` | column name or `NULL` | Outcome for non-survival models. |
| `exposure` | required | one column name | Main exposure. |
| `covariates` | `NULL` | column names or `NULL` | Adjustment variables included in both models being compared. |
| `effect_modifier` | required | one column name | Candidate effect modifier. |
| `approach` | `"logit"` | standard approaches plus `"cox"` and `"survreg"` | Model family. |
| `time`, `event` | `NULL` | column names or `NULL` | Required for survival approaches. |
| `distribution` | `"weibull"` | survival distributions | Parametric distribution for `approach = "survreg"`. |
| `test` | `"LRT"` | `"LRT"`, `"Wald"` | Interaction test type. |
| `alpha` | `0.05` | number | Threshold used to flag evidence of interaction. |
| `verbose` | `FALSE` | `TRUE`, `FALSE` | Prints extra details when `TRUE`. |
| `format` | `"flextable"` | `"flextable"`, `"gt"`, `"tibble"` | Output format. |

### `mediation_analysis()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `data` | required | data frame | Analysis dataset. |
| `exposure`, `mediator`, `outcome` | required | column names | Defines mediation pathway. |
| `covariates` | `NULL` | column names or `NULL` | Adjustment variables. |
| `mediator_approach` | `"linear"` | currently linear mediator model | Fits mediator model. |
| `outcome_approach` | `"linear"` | `"linear"`, `"logit"` | Fits continuous or binary outcome model. |
| `exposure_value`, `reference_value` | `NULL` | values or `NULL` | Defines exposure contrast. |
| `sims` | `1000` | whole number | Bootstrap replicates. |
| `conf_level` | `0.95` | number between 0 and 1 | Confidence level. |
| `seed` | `NULL` | number or `NULL` | Reproducibility seed. |
| `format` | `"flextable"` | `"flextable"`, `"gt"` | Output table engine. |
| `theme` | `"minimal"` | table theme preset | Table styling. |

### `plot_mediation()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `mediation_object` | required | output from `mediation_analysis()` | Draws mediation path diagram. |
| `show_estimates` | `TRUE` | `TRUE`, `FALSE` | Displays effect estimates on the diagram. |
| `base_size` | `13` | number | Base font size. |

## Merge, Modify, and Export

### `merge_tables()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `...` | required | gtregression table objects | Merges descriptive, crude, adjusted, Cox, survival, or other compatible tables. |
| `spanners` | `NULL` | character vector or `NULL` | Adds group headers above merged table blocks. |
| `theme` | `"minimal"` | table theme preset | Table styling. |
| `format` | `"flextable"` | `"flextable"`, `"gt"` | Output engine for the merged table, independent of the input-table formats. |

For descriptive + crude + adjusted tables, keep binary-variable rows consistent.
The recommended publication layout is `show_dichotomous = "all_levels"` in
`descriptive_table()` and `show_ref = TRUE` in every regression table. Mixing
these settings can add unexpected rows, and `merge_tables()` issues a warning
that explains how to correct it.

The merged result carries the footnotes already present in each input table.
Exact duplicate notes are shown once; table-specific notes, including an
adjustment note from `multi_reg()`, are retained unchanged. Multivariable
adjustment notes use the same display labels as the table body.

### `modify_table()`

| Argument | Default | Available options or input | What it does |
|---|---:|---|---|
| `gt_table` | required | gtregression table object | Table to modify. |
| `variable_labels` | `NULL` | named character vector or `NULL` | Relabels variables. |
| `level_labels` | `NULL` | named list of named character vectors or `NULL` | Relabels factor levels within each variable. |
| `header_labels` | `NULL` | named character vector or `NULL` | Relabels column headers. |
| `caption` | `NULL` | character or `NULL` | Adds table caption. |
| `bold_labels` | `TRUE` | `TRUE`, `FALSE` | Displays variable label rows in bold. This preserves the default gtregression hierarchy. |
| `bold_levels` | `FALSE` | `TRUE`, `FALSE` | Displays level rows in bold. |
| `italic_labels` | `FALSE` | `TRUE`, `FALSE` | Italicizes variable label rows. |
| `italic_levels` | `FALSE` | `TRUE`, `FALSE` | Italicizes factor level rows. |
| `remove_N` | `FALSE` | `TRUE`, `FALSE` | Removes displayed sample-size columns when present. |
| `remove_N_obs` | `FALSE` | `TRUE`, `FALSE` | Removes the complete-case `N = ... observations` footnote. |
| `remove_abbreviations` | `FALSE` | `TRUE`, `FALSE` | Removes abbreviation footnotes. |
| `remove_adjustment_note` | `FALSE` | `TRUE`, `FALSE` | Removes the automatic adjustment note; use `caveat` for customised wording. |
| `caveat` | `NULL` | character or `NULL` | Adds custom caveat or interpretation note. |

### Save helpers

| Function | Argument | Default | Available options or input | What it does |
|---|---|---:|---|---|
| `save_table()` | `filename` | `"table"` | path or file stem | Saves to a temporary location when no path is supplied. |
| `save_table()` | `format` | `"docx"` | `"docx"`, `"pdf"`, `"html"` | Table export type. |
| `save_table()` | `orientation` | `"auto"` | `"auto"`, `"portrait"`, `"landscape"` | Chooses page orientation; auto prefers landscape for wide tables. |
| `save_table()` | `fit_width` | `TRUE` | `TRUE`, `FALSE` | Attempts to fit table to page width. |
| `save_table()` | `font_size`, `min_font_size` | `9`, `8` | numbers | Font-size control with lower bound. |
| `save_docx()` | `tables`, `plots` | `NULL`, `NULL` | lists or objects | Builds a Word report from tables and plots. |
| `save_docx()` | `filename` | `"report.docx"` | path or file stem | Word output file. |
| `save_docx()` | `titles` | `NULL` | character vector or `NULL` | Section titles in the Word report. |
| `save_docx()` | `table_width`, `plot_width`, `plot_height` | `6.5`, `6`, `5` | numbers | Output sizing in inches. |
| `save_plot()` | `filename` | `"plot"` | path or file stem | Saves to a temporary location when no path is supplied. |
| `save_plot()` | `format` | `"png"` | `"png"`, `"pdf"`, `"jpg"` | Plot export type. |
| `save_plot()` | `width`, `height`, `dpi` | `8`, `6`, `300` | numbers | Plot canvas and resolution. |

## Practical Defaults

| Goal | Recommended option |
|---|---|
| Word manuscript table | Use the default `format = "flextable"`. |
| HTML/pkgdown output | Use `format = "gt"`. |
| Data checking or pipes | Use `format = "tibble"` when supported. |
| Compact binary regression tables | Use `show_ref = FALSE`; use `show_ref = TRUE` when reference rows are needed. |
| Compact binary descriptive tables | Use `show_dichotomous = "single_row"`; use `"all_levels"` for full level display. |
| KM plot with publication panels | Use `title_face = "plain"`, smaller `title_size`, `risk_table = FALSE`, and patchwork. |
| KM plot when survival remains high | Use `ylim = c(0.5, 1)` or another clinically meaningful range. |
| Forest plot x-axis overlap | Set `xlim` and `ticks_at` manually; increase `ci_col_width` if the CI drawing column is cramped. |
| Wide forest plot export | Use `save_forest()` with `auto_size = TRUE`, or manually increase `width`. |

---
title: "Survival Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Survival Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

`gtregression` supports a complete survival workflow: describe
the cohort, draw Kaplan-Meier curves, summarise observed survival, compare
groups, fit Cox or parametric survival models, check assumptions, predict
survival probabilities, visualise estimates, and export publication-ready
tables.

```{r surv-setup, message=FALSE, warning=FALSE}
library(gtregression)
library(dplyr)

data("data_lungcancer", package = "gtregression")

lung_data <- data_lungcancer |>
  mutate(
    trt = factor(trt, levels = c(1, 2),
                 labels = c("Standard treatment", "Test treatment")),
    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes")),
    celltype = factor(
      celltype,
      levels = c("squamous", "smallcell", "adeno", "large"),
      labels = c("Squamous", "Small cell", "Adenocarcinoma", "Large cell")
    )
  )

attr(lung_data$time, "label") <- "Survival time"
attr(lung_data$status, "label") <- "Death status"
attr(lung_data$trt, "label") <- "Treatment group"
attr(lung_data$celltype, "label") <- "Cancer cell type"
attr(lung_data$karno, "label") <- "Karnofsky performance score"
attr(lung_data$age, "label") <- "Age"
attr(lung_data$prior, "label") <- "Prior therapy"

surv_exposures <- c("trt", "celltype", "karno", "age", "prior")
```

## 1. Describe The Cohort

Start with a baseline table. This helps readers understand the treatment groups
before looking at survival curves or models.

```{r surv-describe, message=FALSE, warning=FALSE}
lung_summary <- descriptive_table(
  data = lung_data,
  exposures = c("time", "status", "celltype", "karno", "age", "prior"),
  by = trt,
  statistic = c(time = "median", karno = "mean", age = "mean"),
  percent = column,
  show_overall = last
)

lung_summary$table
```

## 2. Show Observed Survival

Use `km_plot()` for the Kaplan-Meier curve. Add `risk_table = TRUE` when the
number at risk should appear under the curve. When survival remains high, use
`ylim` to focus the y-axis, for example `ylim = c(50, 100)` with the default
percentage scale. Confidence intervals are shown as shaded bands when
`conf.int = TRUE`.

```{r surv-km-plot, message=FALSE, warning=FALSE}
km_curve <- km_plot(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  break_time_by = 200,
  ylim = c(50, 100),
  title = "Kaplan-Meier Survival by Treatment"
)

km_curve
```

For multi-panel publication figures, make each Kaplan-Meier plot lighter and
let `patchwork` arrange the panels. A common pattern is to remove the risk table,
use short panel titles, reduce `title_size`, and collect legends across panels.

```{r surv-km-panel, fig.width=7, fig.height=5, message=FALSE, warning=FALSE}
km_trt_panel <- km_plot(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  risk_table = FALSE,
  break_time_by = 200,
  ylim = c(50, 100),
  title = "A. Treatment group",
  title_size = 10,
  title_face = plain,
  legend_position = bottom,
  base_size = 10
)

km_prior_panel <- km_plot(
  data = lung_data,
  time = time,
  event = status,
  by = prior,
  risk_table = FALSE,
  break_time_by = 200,
  ylim = c(50, 100),
  title = "B. Prior therapy",
  title_size = 10,
  title_face = plain,
  legend_position = bottom,
  base_size = 10
)

patchwork::wrap_plots(km_trt_panel, km_prior_panel, ncol = 2) +
  patchwork::plot_layout(guides = "collect") &
  ggplot2::theme(legend.position = "bottom")
```

Use table summaries when readers need exact survival values.

```{r surv-observed-tables, message=FALSE, warning=FALSE}
survival_summary(
  data = lung_data,
  time = time,
  event = status,
  by = trt
)$table

survival_prob(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  times = c(90, 180, 365)
)$table
```

`rmst_table()` reports restricted mean survival time up to a chosen follow-up
time. This is useful when an absolute survival-time summary is easier to explain
than a ratio measure.

```{r surv-rmst, message=FALSE, warning=FALSE}
rmst_table(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  tau = 365
)$table
```

## 3. Compare Survival Curves

`logrank_test()` compares Kaplan-Meier curves. It is a group comparison test,
not an effect-size model.

```{r surv-logrank, message=FALSE, warning=FALSE}
logrank_test(
  data = lung_data,
  time = time,
  event = status,
  by = trt
)$table
```

## 4. Fit Cox Regression

`cox_reg()` reports hazard ratios. Use `adjust_for` to produce adjusted hazard
ratios while keeping the syntax aligned with `multi_reg()`.

```{r surv-cox, message=FALSE, warning=FALSE}
cox_crude <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = surv_exposures
)

cox_adjusted <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno)
)

cox_adjusted$table
```

Planned interactions use the same `interaction = exposure*modifier` grammar as
`multi_reg()`. In the default exposure-by-exposure workflow, supply the single
exposure you want to interpret.

```{r surv-cox-interaction, message=FALSE, warning=FALSE}
cox_interaction <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno),
  interaction = trt*prior
)

cox_interaction$table
```

Check the proportional hazards assumption before treating Cox hazard ratios as
final.

```{r surv-check-ph, message=FALSE, warning=FALSE}
check_ph(cox_adjusted)$table
```

## 5. Fit Parametric Survival Regression

`surv_reg()` fits accelerated failure time style parametric survival models and
reports time ratios. A time ratio above 1 suggests longer survival time; below 1
suggests shorter survival time, conditional on the selected distribution.

Before choosing the final distribution, compare candidate parametric models
numerically and visually. Lower AIC/BIC is useful for screening; the fitted
curve should also look reasonable against the Kaplan-Meier curve.

```{r surv-parametric-checks, message=FALSE, warning=FALSE}
surv_model_compare(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno),
  distributions = c(weibull, exponential, "log-normal", "log-logistic")
)$table

plot_surv_fit(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  adjust_for = c(age, karno),
  distributions = c(weibull, "log-logistic"),
  break_time_by = 200
)
```

After selecting a distribution, fit crude and adjusted publication-ready tables.

```{r surv-parametric, message=FALSE, warning=FALSE}
surv_crude <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = surv_exposures,
  distribution = loglogistic
)

surv_adjusted <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno),
  distribution = loglogistic,
  model_stats = TRUE
)

surv_adjusted$table
surv_adjusted$model_stats
```

Parametric survival models use the same interaction grammar and display
Adjusted Time Ratio (95% CI) when adjustment or multivariable modelling is used.

```{r surv-parametric-interaction, message=FALSE, warning=FALSE}
surv_interaction <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno),
  interaction = trt*prior,
  distribution = loglogistic
)

surv_interaction$table
```

## 6. Predict Survival Probabilities

`surv_predict()` turns a fitted parametric survival model into predicted
survival probabilities at selected follow-up times for a profile.

```{r surv-predict, message=FALSE, warning=FALSE}
surv_predict(
  model = surv_adjusted$models$trt,
  newdata = data.frame(
    trt = factor("Test treatment", levels = levels(lung_data$trt)),
    age = 60,
    karno = 70
  ),
  times = c(90, 180, 365)
)$table
```

## 7. Visualise And Export Model Results

The survival model outputs work with the same downstream tools used for other
regression tables.

```{r surv-visualise-export, message=FALSE, warning=FALSE}
plot_reg_combine(
  cox_crude,
  cox_adjusted,
  show_ref = FALSE,
  title_uni = "Crude HR",
  title_multi = "Adjusted HR"
)

surv_forest_data <- forest_df(cox_crude, cox_adjusted, desc = lung_summary)

forest_reg(
  surv_forest_data,
  xlim = list(c(0.25, 8), c(0.25, 8)),
  ticks_at = list(c(0.5, 1, 2, 4, 8), c(0.5, 1, 2, 4, 8)),
  quiet = TRUE
)
```

If forest plot x-axis labels overlap, set `xlim` and `ticks_at`. If the
confidence-interval plot panel is too narrow or too wide, tune `ci_col_width`.
For very wide descriptive-plus-crude-plus-adjusted tables, export using a wider
graphics device or Word canvas.

## Survival Workflow Map

| Task | Function |
|---|---|
| Kaplan-Meier curve | `km_plot()` |
| Number at risk | `km_risk_table()` |
| Median survival | `survival_summary()` |
| Survival quantiles | `survival_quantiles()` |
| Fixed-time survival probability | `survival_prob()` |
| Restricted mean survival time | `rmst_table()` |
| Compare KM curves | `logrank_test()` |
| Cox hazard ratios | `cox_reg()` |
| Cox PH check | `check_ph()` |
| Parametric time ratios | `surv_reg()` |
| Compare parametric distributions | `surv_model_compare()` |
| Plot fitted parametric curves | `plot_surv_fit()` |
| Predict survival probabilities | `surv_predict()` |
| Model plots and forest tables | `plot_reg()`, `plot_reg_combine()`, `forest_df()`, `forest_reg()` |

---
title: "Diagnostics and Model Selection"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Diagnostics and Model Selection}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Before the final table goes into a manuscript, check the model. These helpers
keep diagnostics close to the regression workflow. They are screening aids:
interpret them with the study design, clinical or subject-matter judgement, and
the diagnostics from the fitted model.

```{r diag-setup, message=FALSE, warning=FALSE}
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes"))
  )

exposures <- c("age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat")
```

## Convergence Screening

Use `check_convergence()` before interpreting model estimates, especially for
log-binomial and small or sparse binary-outcome models. A non-converged model is
a fitting warning, not a finding.

```{r diag-convergence, message=FALSE, warning=FALSE}
check_convergence(
  data = birthwt_data,
  exposures = exposures,
  outcome = low,
  approach = logit,
  multivariate = TRUE
)
```

For risk-ratio workflows, this same check helps users decide whether a
log-binomial model fitted cleanly or whether a robust Poisson approach may be a
more practical sensitivity analysis.

```{r diag-convergence-logbin, message=FALSE, warning=FALSE}
check_convergence(
  data = birthwt_data,
  exposures = c("smoke", "ht", "ui", "ptl_cat"),
  outcome = low,
  approach = logbinomial,
  multivariate = TRUE
)
```

## Collinearity Screening

`check_collinearity()` reports VIF-style diagnostics for multivariable models.
High VIF values are prompts to inspect coding, overlap between predictors, and
the scientific purpose of the model.

```{r diag-collinearity, message=FALSE, warning=FALSE}
birthwt_multi <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = exposures,
  approach = logit
)

check_collinearity(birthwt_multi, format = gt)
```

Adjusted-mode `multi_reg()` objects contain one model per exposure. The
collinearity output keeps that list structure so each model can be inspected
separately.

```{r diag-collinearity-adjusted, message=FALSE, warning=FALSE}
birthwt_adjusted <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat"),
  adjust_for = c("age", "lwt", "race"),
  approach = logit
)

check_collinearity(birthwt_adjusted, format = tibble)
```

## Model Fit Plots

`plot_model_fit()` turns fitted models into quick diagnostic plots. It accepts
raw `lm()` and `glm()` objects, and it also works with models saved inside
`uni_reg()` and `multi_reg()` results.

For logistic regression, the calibration plot compares predicted probabilities
with observed event proportions. Points close to the diagonal line suggest that
the model predictions are reasonably aligned with the observed data. Calibration
is usually most useful for multivariable models because predicted probabilities
vary across many people.

```{r diag-fit-logistic, message=FALSE, warning=FALSE}
plot_model_fit(
  birthwt_multi,
  type = calibration,
  bins = 6
)
```

When a `uni_reg()` object contains several models, use `model_name` to choose
the exposure you want to inspect. For a simple binary exposure, calibration may
only show two points because the model has only two fitted probabilities; in
that situation, residual and influence plots are usually more useful.

```{r diag-fit-uni, message=FALSE, warning=FALSE}
birthwt_uni <- uni_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("age", "lwt", "smoke"),
  approach = logit
)

plot_model_fit(
  birthwt_uni,
  model_name = smoke,
  type = residual
)
```

For logistic models, residual plots often form two visible bands. That is a
normal consequence of a 0/1 outcome and should be interpreted as a screening
plot rather than a linear-model residual plot.

For linear regression, `type = all` shows the classic residual, Q-Q,
scale-location, and Cook's distance views.

```{r diag-fit-linear, message=FALSE, warning=FALSE}
fit_lm <- lm(bwt ~ age + lwt, data = birthwt_data)
plot_model_fit(fit_lm)
```

## Proportional Hazards Screening

For Cox models, use `check_ph()` before treating hazard ratios as final. It
reports Schoenfeld residual tests from `survival::cox.zph()`, including a global
test. Small p-values suggest possible non-proportional hazards and should be
reviewed with plots, follow-up pattern, and clinical judgement.

```{r diag-ph-setup, message=FALSE, warning=FALSE}
data("data_lungcancer", package = "gtregression")

lung_data <- data_lungcancer |>
  mutate(
    trt = factor(trt, levels = c(1, 2),
                 labels = c("Standard treatment", "Test treatment")),
    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes")),
    celltype = factor(
      celltype,
      levels = c("squamous", "smallcell", "adeno", "large"),
      labels = c("Squamous", "Small cell", "Adenocarcinoma", "Large cell")
    )
  )

cox_fit <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno)
)
```

```{r diag-ph-table, message=FALSE, warning=FALSE}
check_ph(cox_fit, format = gt)
```

Use `format = tibble` when you want to inspect or filter the diagnostic results.

```{r diag-ph-tibble, message=FALSE, warning=FALSE}
check_ph(cox_fit, transform = rank, format = tibble)
```

## Stepwise Model Selection

`compare_models()` is for prespecified candidate models that have already been
fitted with gtregression. It answers a different question from stepwise
selection: "How do these planned models compare?" The inputs should be
`multi_reg()`, `cox_reg()`, or `surv_reg()` outputs, not raw `lm()`, `glm()`,
`coxph()`, or `survreg()` objects. This keeps the workflow consistent with the
publication-ready tables created by the package.

```{r diag-compare-logit, message=FALSE, warning=FALSE}
logit_m0 <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = smoke,
  approach = logit
)

logit_m1 <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c(smoke, age, lwt),
  approach = logit
)

logit_m2 <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c(smoke, age, lwt, race, ht, ui),
  approach = logit
)

logit_model_comparison <- compare_models(
  logit_m0,
  logit_m1,
  logit_m2,
  model_names = c(
    "Smoking only",
    "Add age and weight",
    "Full clinical model"
  ),
  primary_exposure = smoke,
  format = gt
)

logit_model_comparison$table
```

The table reports N, number of parameters, AIC, BIC, log-likelihood, and
likelihood-ratio comparisons when `nested = TRUE`. Lower AIC or BIC identifies
better relative fit among the compared models. When `primary_exposure` is
supplied, the table also tracks that effect estimate and the percentage change
across models.

`compare_models()` automatically checks whether the candidate models appear to
use the same analysis sample. It uses retained row identifiers when the fitted
model stores them; otherwise it compares N and event counts. If the models use
different complete-case samples, the table still displays AIC, BIC,
log-likelihood, and likelihood-ratio statistics for transparency, but the footer
warns that these values should not be interpreted as formal model-selection
criteria across different datasets. In that situation, use the primary exposure
estimate, percentage change, confidence intervals, and clinical or
epidemiological reasoning to judge robustness.

### Build Candidates in the App

The function API intentionally accepts fitted gtregression objects. In the
gtregression app, **Advanced > Compare models** provides a guided layer before
that API: choose the shared outcome and approach, name two to six candidates,
and select each candidate's exposures, adjustment variables, and optional
interaction. The app fits `multi_reg()`, `cox_reg()`, or `surv_reg()` objects and
then calls `compare_models()` with those objects.

For the birth weight models above, enter *Smoking only*, *Add age and weight*,
and *Full clinical model* as the candidate names, place `smoke` in the exposure
control, and add the corresponding variables under adjustment. Select `smoke`
as the primary exposure to reproduce the estimate-change columns. The generated
code records every fitted model and is suitable for direct use in an R script.

Candidate order is deliberate. Put the simplest or primary model first because
percentage change is calculated from that model and likelihood-ratio statistics
compare sequential pairs. If the fitted samples differ, or a sequential pair is
not nested, the rendered table displays only the warning relevant to that
comparison. See the **gtregression App** article for the complete ordinary,
Cox, and parametric survival walkthrough.

For Cox and parametric survival models, fit the candidate models with `cox_reg()`
or `surv_reg()` first. `compare_models()` then keeps survival-specific columns
such as events and Cox concordance.

```{r diag-compare-cox, message=FALSE, warning=FALSE}
cox_m0 <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt
)

cox_m1 <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno)
)

cox_m2 <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, age, karno, celltype, prior),
  multivariable = TRUE
)

cox_model_comparison <- compare_models(
  list(
    "Treatment only" = cox_m0,
    "Add age and performance" = cox_m1,
    "Full clinical model" = cox_m2
  ),
  primary_exposure = trt,
  format = gt
)

cox_model_comparison$table
```

`select_models()` compares candidate models step by step. It is useful for
exploration, teaching, and sensitivity checks. It should not replace a planned
model based on study design or a causal framework.

```{r diag-select, message=FALSE, warning=FALSE}
selected <- select_models(
  data = birthwt_data,
  outcome = low,
  exposures = exposures,
  approach = logit,
  direction = forward
)

selected
```

The selected direction is recorded in the formatted table footer. Backward and
both-direction searches are available using the same interface.

```{r diag-select-other-directions, message=FALSE, warning=FALSE}
select_models(
  data = birthwt_data,
  outcome = low,
  exposures = exposures,
  approach = logit,
  direction = backward,
  format = tibble
)$results_table

select_models(
  data = birthwt_data,
  outcome = low,
  exposures = exposures,
  approach = logit,
  direction = both,
  format = tibble
)$results_table
```

## What To Inspect

- `check_convergence()`: convergence status and maximum fitted probabilities.
  Use `format = gt` or `format = flextable` for viewing tables.
- `check_collinearity()`: VIF and interpretation. Nested model outputs keep
  their list structure when formatted.
- `plot_model_fit()`: residual, calibration, observed-versus-predicted, and
  influence plots for `lm`/`glm` models and stored `uni_reg()` / `multi_reg()`
  fitted models.
- `check_ph()`: Schoenfeld residual proportional hazards tests for Cox models,
  including term-level and global tests.
- `compare_models()`: AIC, BIC, log-likelihood, likelihood-ratio tests, sample
  size, events for survival models, and optional primary-exposure tracking for
  gtregression candidate models.
- `select_models()`: `$results_table`, `$best_model`, `$all_models`, and
  `$direction`; `$table` is added when `format = gt` or `format = flextable`.

---
title: "Stratified Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Stratified Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Stratified regression repeats the analysis inside each subgroup and places the
results side by side. It is useful when the same association may look different
across groups.

```{r strata-setup, message=FALSE, warning=FALSE}
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes"))
  )

attr(birthwt_data$age, "label") <- "Maternal age"
attr(birthwt_data$lwt, "label") <- "Maternal weight"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"
attr(birthwt_data$ui, "label") <- "Uterine irritability"
attr(birthwt_data$ptl_cat, "label") <- "Previous preterm labour"
```

## Describe by Stratum

Start with a descriptive table by the stratifying variable. This is the companion
table for the stratified regression: it helps users see the size and clinical
profile of each subgroup before fitting stratum-specific models.

```{r strata-desc, message=FALSE, warning=FALSE}
strata_desc <- descriptive_table(
  data = birthwt_data,
  exposures = c("age", "lwt", "smoke", "ht", "ui", "ptl_cat"),
  by = race,
  percent = column,
  show_overall = last,
  theme = clinical
)

strata_desc$table
```

## Univariable by Stratum

`stratified_uni_reg()` fits one model per exposure inside each stratum. The
result is a single wide table, with one spanner per stratum.

```{r strata-uni, message=FALSE, warning=FALSE}
strata_uni <- stratified_uni_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("age", "lwt", "smoke", "ht", "ui", "ptl_cat"),
  stratifier = race,
  approach = logit,
  theme = clinical
)

strata_uni$table
```

## Full Multivariable Model by Stratum

With `adjust_for = NULL`, `stratified_multi_reg()` fits one multivariable model
inside each stratum using all supplied exposures.

```{r strata-multi-full, message=FALSE, warning=FALSE}
strata_full <- stratified_multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("age", "lwt", "smoke", "ht", "ui", "ptl_cat"),
  stratifier = race,
  approach = logit,
  theme = clinical
)

strata_full$table
```

## Exposure-Specific Adjusted Models by Stratum

Use `adjust_for` when each exposure should be adjusted for the same variables
within each stratum. This mirrors `multi_reg(adjust_for = ...)`, but repeats the
same workflow separately inside each stratum.

```{r strata-multi, message=FALSE, warning=FALSE}
strata_multi <- stratified_multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat"),
  stratifier = race,
  adjust_for = c("age", "lwt"),
  approach = logit,
  theme = striped
)

strata_multi$table
```

If a stratum cannot fit a model, the function skips that stratum with a warning
and continues. This is intentional: sparse strata are common in real data, and
one small subgroup should not erase the whole analysis.

## Forest Plot by Stratum

`forest_df()` can also prepare one stratified regression object for
`forest_reg()`. The variable rows are kept once and each stratum is placed in a
side-by-side effect column, which is easier to compare than repeating the full
variable list for every subgroup.

```{r strata-forest, message=FALSE, warning=FALSE, fig.width=8, fig.height=7}
strata_forest_data <- forest_df(strata_multi)

forest_reg(
  strata_forest_data,
  ci_col_width = 18
)
```

## What To Inspect

- `$table`: rendered side-by-side table.
- `$table_display`: wide data used to build the table.
- `$per_stratum`: full per-stratum result objects.
- `$models`: fitted models by stratum.
- `$model_summaries`: summaries for the fitted models.
- `$variable_labels`: display labels used in the wide table.
- `$reg_check`: diagnostics for linear models.

---
title: "Using the gtregression App"
description: "A beginner-friendly manual for the menu-driven gtregression Shiny app."
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using the gtregression App}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

# gtregression App Manual

The `gtregression` app is a menu-driven companion to the R package. It is
designed for users who want to explore data, create publication-ready tables,
visualise model results, and export outputs without remembering every function
argument.

The app is not a replacement for a reproducible R script. Its best use is:

1. Load or upload a dataset.
2. Build the analysis step by step.
3. Review the table or plot.
4. Copy the generated R code.
5. Paste the code into RStudio and save the final manuscript output.

Each analysis tab includes a **Code** panel with a copy button. This is important:
the app helps users learn the package while creating reproducible code they can
edit, save, and share.

> **Guide update:** annotated screenshots are being added to this manual. The
> written workflow and generated-code examples are current; visual callouts will
> follow as the screenshots are uploaded.

Suggested screenshot files are `man/figures/app-data.png`,
`man/figures/app-regression.png`, `man/figures/app-survival.png`, and
`man/figures/app-visualise-export.png`.

## Launch the App

Install and load `gtregression`, then run:

```r
library(gtregression)

gtregression_app()
```

When the app is launched from RStudio, it opens in the RStudio Viewer when
available. Outside RStudio, it opens in a browser during an interactive session.

You can pass arguments to `shiny::runApp()` through `gtregression_app()`:

```r
gtregression_app(launch.browser = TRUE)
```

To run the app without automatically opening a browser:

```r
gtregression_app(launch.browser = FALSE)
```

## Close the App Safely

The app keeps the R session busy while it is running. This is normal for Shiny.

Use the **Close app** button in the bottom-right corner of the app to stop the
session cleanly. This avoids the confusing console behaviour that can happen
when users stop the app manually from RStudio.

If R ever enters a `Browse[1]>` prompt after an interrupted app session, type:

```r
Q
```

and press Enter.

## App Dependencies

The app is launched only when `gtregression_app()` is called. This keeps the
main package lightweight for users who only want command-line functions.

The app uses standard, trusted R packages:

| Purpose | Packages |
|---|---|
| App interface | `shiny` |
| Table display and Word/RTF export | `flextable` |
| HTML table fallback | `gt` |
| Plot display | `ggplot2`, `patchwork`, `forestploter` |
| Data upload and preview | `readxl`, `DT` where available |
| RStudio Viewer support | `rstudioapi` where available |

If a dependency is missing, the app gives a message explaining what to install.

## Recommended Workflow

The app is easiest to use when users follow a deliberate order:

1. **Data**: load a built-in dataset or import CSV, Excel, RDS, Stata, SPSS, or SAS data.
2. **Data Prep**: review the data only when changes are needed. The original
   data are active automatically; choose **Use prepared data** after applying
   changes.
3. **Descriptive**: inspect variables and create a baseline table.
4. **Regression**: run univariable models first.
5. **Regression**: reselect the clinically important variables and run the
   multivariable model.
6. **Visualise & Export**: merge descriptive, crude, and adjusted outputs.
7. **Visualise & Export**: create `plot_reg()` or `forest_reg()` previews.
8. **Advanced**: check model fit, compare models, assess confounding, and assess
   interaction where appropriate.
9. **Code panels**: copy the generated code into RStudio for reproducible
   saving and editing.

The app intentionally encourages users to run analyses one step at a time. For
example, run univariable regression first, then decide which variables to keep,
then run the multivariable model.

## Core and Additional Options

The app does not use a global Simple/Advanced switch. Each tab presents its
core workflow first and keeps uncommon controls in a local **Additional
options** section. This keeps the interface calm without hiding package
features behind a separate experience-level choice.

For example, the Descriptive tab keeps grouping, percentage, missing-value,
binary-display, and numeric-statistic choices together. The Survival tab keeps
the less-common time-break interval under **Additional KM options**.

## Quick-Start Templates

The **Data** tab includes quick-start templates. These load labelled teaching
datasets and preselect sensible variables across the app:

| Template | Best for |
|---|---|
| Birthweight regression | Descriptive tables, logistic regression, merged tables, forest plots |
| Lung survival | Cox regression, parametric survival regression, Kaplan-Meier plots |
| Diabetes mediation | Causal mediation workflow and path diagrams |
| Firth logistic | Sparse-data logistic regression |

Templates are meant to help users learn the flow. After selecting a template,
users can still change any variable, model, or display option.

## Workflow Guide

The app shows a compact workflow guide near the top of the screen. It marks
which steps have already been completed and gives one suggested action.

For example:

- after loading data, it suggests reviewing the preview and choosing an analysis;
- after a descriptive table, it suggests regression or survival analysis;
- after fitted models, it suggests visualisation, model checks, merge, or export;
- after plots or merged tables, it suggests code review or download/export.

This is deliberately beginner-friendly. Users do not need to know the whole
package before starting; the app nudges them through a reproducible analysis.

## Live Analysis Status and Preflight Checks

A status strip remains visible near the top of the app. It identifies the
loaded dataset, whether modelling uses the original or prepared data, the
current sample size, and the selected regression outcome and approach. This
helps prevent an analysis being run on the wrong version of the data.

The Regression and Survival tabs also show a short preflight summary immediately
above their run buttons. Review it before fitting a model. It confirms the
outcome or survival variables, the number of exposures and adjustment variables,
the modelling mode, and any selected categorical reference levels. If a required
choice is missing, the summary states what must be selected next.

## Select and Clear Variable Buttons

Several tabs include **Select all** and **Clear all** buttons beside long
variable lists. These are available where users commonly need to choose multiple
variables, including:

- descriptive table variables;
- regression exposures and adjustment variables;
- survival exposures and adjustment variables;
- advanced model-building variables;
- mediation covariates.

The buttons are role-aware. For example, after selecting an outcome, **Select
all** for regression exposures avoids selecting the outcome as an exposure. In
the survival tab, **Select all** avoids the selected time, event, and stratifier
variables. This reduces accidental modelling errors while keeping variable
selection fast for larger uploaded datasets.

## Data Tab

Use the **Data** tab to start the analysis.

Users can:

- load a built-in `gtregression` teaching dataset;
- upload a CSV or Excel file;
- import an RDS file containing a data frame or tibble;
- import Stata (`.dta`), SPSS (`.sav`, `.zsav`, `.por`), or SAS (`.sas7bdat`, `.xpt`) data when `haven` is installed;
- preview the dataset;
- inspect a quick data summary;
- copy reusable data-loading code.

For uploaded CSV files, check:

- whether the file has a header;
- the separator: comma, semicolon, or tab.

For Excel files, optionally provide the sheet name or sheet number.

CSV controls are ignored for other formats, and the Excel sheet field is
ignored unless an Excel file is selected. An RDS file must contain a data frame
or tibble. Generated code records the appropriate `readRDS()`,
`readxl::read_excel()`, or `haven::read_*()` call.

After loading data, check the preview before moving to the next tab. This helps
catch common issues such as numeric variables imported as character, missing
factor labels, or incorrect event coding.

## Data Prep Tab

The **Data Prep** tab is an internal part of the gtregression app. It does not
load or depend on the `gtstats` package. It provides a safe, visible sequence of
common preparation steps while retaining the original imported dataset.

Users can rename and retain variables, set variable types and factor order,
recode values, define or display missing values, filter rows, calculate new
variables, and create grouped variables. **Quick starts** provide common
recipes. Every applied change is added to a readable history and reusable R
code.

When creating groups from a numeric variable, conditions include `=`, `!=`,
`>`, `>=`, `<`, `<=`, an inclusive **between** range, and **outside the
range**. Rules are checked from top to bottom and each row is assigned to the
first rule it matches. The final group contains everyone not matched earlier.
For example, `< 35`, then `< 65`, then everyone else produces groups for
under 35, 35--64, and 65 or older. The preview shows the row count in every
group before the change is applied.

The recovery controls are deliberately prominent:

- **Undo** reverses the latest change;
- **Redo** reapplies an undone change;
- **Reset** asks for confirmation before restoring the original data;
- **Download prepared data** saves the current prepared dataset;
- **Download code** saves the preparation recipe.

The original dataset is active immediately after loading, so users can run a
table or model without visiting Data Prep. Select a different analysis dataset
only when preparation is wanted:

- **Use original data**: subsequent analyses use the untouched imported data;
- **Use prepared data**: subsequent analyses use the current prepared version.

Data Prep is optional. Use **Set display label** when an internal variable name such as
`age` should remain unchanged but publication output should show a clearer name
such as *Maternal age*. Labels are reversible, are retained in prepared-data
downloads, and are included in the reusable code.

For a categorical variable, the same page also shows its observed categories.
Edit a category display label such as `0` to `No` or `1` to `Yes` there. This
creates a labelled factor only when a category label changes, retaining a clean
and reproducible analysis dataset from the outset.

If preparation is changed later, choose **Use prepared data** when those latest
changes should be used for subsequent analyses. The generated workflow records
that choice and includes the preparation operations when prepared data are
selected.

## Descriptive Tab

Use the **Descriptive** tab to create baseline tables with
`descriptive_table()`.

Choose **Group by** first: leave it as `None` for one overall table, or select
the comparison variable before choosing the variables to summarise. Then choose
column or row percentages, overall-column placement, missing-value display, and
binary-category display to suit the reporting question.

Typical steps:

1. Select variables in **Exposures**.
2. Optionally select a grouping variable in **By**.
3. Choose the percentage denominator.
4. Decide whether to show an overall column.
5. Decide how missing values should be displayed.
6. Run the table.

Common settings:

```r
desc_result <- descriptive_table(
  data = df,
  exposures = c("age", "lwt", "race", "smoke"),
  by = "low",
  percent = "column",
  show_overall = "last",
  show_missing = "ifany"
)
```

### Binary Variables

For baseline tables, the default binary display is usually helpful because it
shows both levels. This makes descriptive outputs easier to align with
regression and forest tables later.

If a user wants a compact one-row binary summary, they can change the binary
display option. For merged tables and forest plots, showing both levels is often
clearer.

### Numeric Statistic Overrides

The app allows numeric summaries to be changed by typing named overrides such as:

```r
age = mean, lwt = median
```

This creates code similar to:

```r
descriptive_table(
  data = df,
  exposures = c("age", "lwt", "race", "smoke"),
  by = "low",
  statistic = c(age = "mean", lwt = "median")
)
```

## Regression Tab

Use the **Regression** tab for standard regression tables:

- logistic regression;

Choose the outcome before the exposures. Run univariable models first, then
move clinically justified variables into the multivariable model. The app does
not select adjustment variables automatically: use the study question, a DAG,
and clinical knowledge.
- Firth logistic regression;
- log-binomial regression;
- Poisson regression;
- robust Poisson regression;
- negative binomial regression;
- linear regression.

The recommended order is:

1. Choose the outcome.
2. Choose all candidate exposures.
3. Run **Univariate** analysis.
4. Review the crude results.
5. Reselect variables for the adjusted analysis.
6. Add adjustment variables where needed.
7. Run **Multivariable** analysis.
8. Copy the generated code into an R script.

Example:

```r
uni_result <- uni_reg(
  data = df,
  outcome = "low",
  exposures = c("age", "lwt", "race", "smoke"),
  approach = "logit",
  show_ref = TRUE,
  format = "flextable"
)

multi_result <- multi_reg(
  data = df,
  outcome = "low",
  exposures = c("smoke", "ht", "ui"),
  adjust_for = c("age", "lwt", "race"),
  approach = "logit",
  show_ref = TRUE,
  format = "flextable"
)
```

### Reference Categories

After categorical predictors are selected, the app displays a **Reference
categories** section. Each control lists the levels actually observed in the
selected analysis dataset. The selected level is the model baseline. It is not
shown for continuous predictors or for the outcome.

For example, choosing `No` as the baseline for `smoke` means every displayed
smoking estimate is interpreted relative to non-smokers. The app applies the
choice with `factor()` and `stats::relevel()` and writes the same operation into
the generated and downloaded R scripts:

```r
analysis_data[["smoke"]] <- stats::relevel(
  factor(
    as.character(analysis_data[["smoke"]]),
    levels = c("No", "Yes")
  ),
  ref = "No"
)
```

Use **Show reference categories** when planning to merge tables or create forest
plots. This displays the chosen baseline row as `Ref.` and keeps categorical
variables easier to interpret.

If compact tables are preferred, reference rows can be hidden. When reference
rows are hidden, binary variables may appear as single rows. That is acceptable,
but merged descriptive-regression outputs may be clearer when `show_ref = TRUE`.

### Model Statistics

Model statistics are stored by default. The **Model Stats** tab then shows fit
statistics such as AIC, BIC, log-likelihood, pseudo R-squared where
appropriate, and N. They are not added to the publication-ready regression
table. Clear **Store model statistics** before fitting when they are not needed.

For logistic, Poisson, Cox, and parametric survival models, ordinary linear
`R2` is not usually available. In those settings, pseudo R-squared or survival
model statistics are more appropriate.

## Stratified Regression

The **Stratified** output repeats regression analyses within levels of a
stratifier.

Use it when the clinical or epidemiological question is naturally stratified,
for example:

- sex-specific associations;
- treatment subgroup analyses;
- hospital or site-level exploratory analyses;
- diabetes status subgroups.

Example:

```r
stratified_result <- stratified_multi_reg(
  data = df,
  outcome = "low",
  exposures = c("smoke", "ht"),
  adjust_for = c("age", "lwt"),
  stratifier = "race",
  approach = "logit",
  show_ref = TRUE,
  format = "flextable"
)
```

Stratified outputs are useful for tables. For plots, `plot_reg()` can create
quick presentation-style views. For publication forest outputs, use
`forest_df()` and `forest_reg()` after checking that the display remains readable.

## Survival Tab

Use the **Survival** tab for:

- Cox regression with `cox_reg()`;

Choose Cox regression for hazard ratios when no baseline-hazard distribution is
specified. Choose parametric survival regression when a distributional model
and time-ratio interpretation are appropriate. Kaplan-Meier plots describe the
observed curves and should not be used as a substitute for either fitted model.
- parametric survival regression with `surv_reg()`;
- Kaplan-Meier curves with `km_plot()`;
- log-rank tests and survival summaries.

At the top of the workspace, choose **Cox regression** or **Parametric
survival**. The app shows one model workflow at a time: Cox mode hides the
distribution control, while Parametric survival mode asks for the Weibull,
exponential, log-normal, or log-logistic distribution. The active mode also
controls the run button, model result, and reusable code. Each method retains
two independent result slots: **Exposure models** and **Multivariable model**.
Running one does not replace the other. Both completed tables remain available
as separate choices in **Visualise & Export**, including the merge-table menu,
and in compatible **Advanced** tools.

For a complete GUI workflow, first leave **Single multivariable model** clear
and run the exposure models. Then select the predictors for the joint model,
check **Single multivariable model**, and run again. Review or download either
result from its named subtab. This mirrors the two API calls below and makes it
possible to merge the resulting crude/exposure and multivariable tables.

Typical Cox regression workflow:

```r
cox_result <- cox_reg(
  data = df,
  time = "time",
  event = "status",
  exposures = c("trt", "celltype", "prior"),
  adjust_for = c("age", "karno"),
  show_ref = TRUE,
  format = "flextable"
)
```

For one full multivariable Cox model using all selected exposures:

```r
cox_result <- cox_reg(
  data = df,
  time = "time",
  event = "status",
  exposures = c("trt", "celltype", "prior", "age", "karno"),
  multivariable = TRUE,
  show_ref = TRUE,
  format = "flextable"
)
```

Parametric survival regression follows the same app flow, with an additional
distribution option:

```r
survreg_result <- surv_reg(
  data = df,
  time = "time",
  event = "status",
  exposures = c("trt", "celltype", "prior"),
  adjust_for = c("age", "karno"),
  distribution = "weibull",
  show_ref = TRUE,
  format = "flextable"
)
```

## Kaplan-Meier Plots

The app supports publication-friendly Kaplan-Meier plots:

- grouped or overall curves;
- shaded confidence intervals;
- risk tables;
- log-rank p-values;
- percent or probability y-axis;
- x-axis limits;
- y-axis limits;
- classic, minimal, black-and-white, light, or no theme;
- optional grid lines.

Example:

```r
km_result <- km_plot(
  data = df,
  time = "time",
  event = "status",
  by = "trt",
  conf.int = TRUE,
  risk_table = TRUE,
  p_value = TRUE,
  y_percent = TRUE,
  ylim = c(50, 100),
  xlim = c(0, 800),
  break_time_by = 200,
  theme = "classic",
  grid = FALSE
)
```

Use y-axis limits when curves sit between high survival probabilities. For
example, if survival stays between 80% and 100%, setting `ylim = c(50, 100)` can
make group differences easier to see.

For multi-panel figures created with `patchwork`, use smaller titles or remove
titles in the app-generated code and add labels later:

```r
p1 <- km_plot(df, time = "time", event = "status", by = "trt",
              title = NULL, base_size = 10)
p2 <- km_plot(df, time = "time", event = "status", by = "celltype",
              title = NULL, base_size = 10)

p1 + p2
```

## Visualise & Export Tab

The **Visualise & Export** tab turns fitted results into tables and figures.
The top-level **Session Code** tab provides **Download complete R script**, which saves a single
`gtregression-session.R` file for the current session. The script records the
selected data source, any Data Prep changes, categorical reference levels,
completed descriptive and modelling steps, visualisations, and export calls.
Only analyses that have actually been run are included.

Open **Session Code** to review the same script before downloading it.
The downloaded script begins with `library(gtregression)` and can be edited or
rerun in RStudio as the reproducible record of the app workflow.

Choose the task from the menu across the top. The left panel then displays only
the controls needed for that task, while the corresponding preview opens on the
right. This keeps table merging, regression plots, forest plots, and model-fit
diagnostics separate.

It can preview:

- merged tables;
- single-result `plot_reg()` outputs;
- combined crude and adjusted `plot_reg_combine()` outputs;
- single-result or combined crude and adjusted `forest_reg()` outputs;
- model fit plots.

It can also generate code for:

- `plot_reg()`;
- `plot_reg_combine()`;
- `merge_tables()`;
- `forest_df()`;
- `forest_reg()`;
- `save_table()`;
- `save_plot()`;
- `save_forest()`.

### Model Fit

The **Model Fit** menu exposes the full `plot_model_fit()` choice set:
**Auto**, **All diagnostics**, residuals versus fitted, normal Q-Q,
scale-location, Cook's distance, observed versus predicted, and calibration.
Use **Auto** for the appropriate diagnostic panel. Calibration uses the chosen
number of bins; the base font size applies to every model-fit view. The generated
code and Session Code preserve the selected type, model name, bins, and font
size.

Browser plots should be treated as previews. For final output, copy the code
into RStudio and export with explicit width and height.

### Export Size Presets

The app provides export presets for plot downloads:

| Preset | When to use |
|---|---|
| Standard | Routine regression plots and moderate forest plots |
| Wide | Long labels, combined crude/adjusted plots, or manuscript figures |
| Many rows | Forest plots with many variables or strata |
| Compact | Quick previews or slides with few variables |

These presets change the downloaded PNG/PDF size. The browser preview may still
look cramped on a small screen, so final checks should be done on the exported
file.

### Forest Plot Controls

These controls are available directly from the Forest Plot menu:

| Control | What it does | When to change it |
|---|---|---|
| Forest plot side | Places the confidence-interval plot before or after the text | Match a journal or slide layout |
| CI column width | Allocates room to each forest panel | Increase it when intervals or tick labels overlap |
| Forest x limits | Sets the effect range using two comma-separated values | Widen it when intervals approach an edge |
| Forest tick marks | Sets comma-separated axis labels | Use fewer ticks when labels overlap |
| Export size preset | Controls PNG/PDF dimensions | Use **Wide** for long labels and **Many rows** for large tables |

Leave x limits and tick marks blank to use `forestploter`'s automatic choices.
Every custom tick must lie inside the selected limits.

### Download Formats

Table previews offer **DOCX**, **RTF**, and **HTML**. HTML is the browser-ready
copy; DOCX and RTF are editable manuscript formats. Plot previews offer **PNG**
and **PDF**, because plots are graphics objects rather than HTML tables.
Each download is generated in Shiny's temporary download location and passed to
the browser. The app does not choose, overwrite, or write directly into a user
folder; the browser handles the final save location.

### Session Code

The **Session Code** panel collects the analysis steps completed in the
app into one script-style block. This is useful after a user has loaded data,
run descriptive tables, fitted models, and created plots.

The recommended behaviour is:

1. Run the app steps interactively.
2. Open **Session Code**.
3. Click the copy button.
4. Paste the code into RStudio.
5. Save and edit the script as the final reproducible analysis.

This is the easiest way for app users to move from menu-driven analysis to a
proper script.

### Merge Tables

Choose **Merge tables** from the top menu. The app lists only completed tables.
Select at least two tables, keep them in the intended descriptive/crude/adjusted
order, and click **Merge selected tables**. This also supports two-table merges,
such as descriptive plus crude or crude plus adjusted.

The app creates code similar to:

```r
merged_table <- merge_tables(
  desc_result,
  uni_result,
  multi_result,
  spanners = c("Descriptive", "Crude", "Adjusted")
)

save_table(merged_table, filename = "merged_table.docx")
```

If binary variables do not align as expected, rerun regression tables with
`show_ref = TRUE`. This displays reference rows and usually gives the cleanest
merged output.

### Regression Plots

Choose **Regression plot** from the top menu. Choose **Selected result** to
preview the object named in **Result** with `plot_reg()`. When a matching pair
has been fitted, the app also offers **Regression crude + adjusted**, **Cox
exposure + multivariable**, or **Parametric exposure + multivariable**. These
choices call `plot_reg_combine()` with the two corresponding stored results;
an option appears only after both members of that pair exist.

Use `plot_reg()` for quick presentation-style plots:

```r
reg_plot <- plot_reg(
  multi_result,
  log_x = TRUE
)

save_plot(reg_plot, filename = "plot_reg.png", width = 9, height = 6)
```

Use `plot_reg_combine()` when both crude and adjusted models are available:

```r
combined_plot <- plot_reg_combine(
  uni_result,
  multi_result,
  log_x = TRUE
)

save_plot(
  combined_plot,
  filename = "plot_reg_combined.png",
  width = 10,
  height = 7
)
```

### Modify Table

Choose **Modify table** to polish any completed descriptive, regression, or
merged table without changing the original analysis result. The app creates a
separate modified table that can be downloaded as DOCX, RTF, or HTML.

All `modify_table()` options are available: variable, level, and header labels;
a caption; bold or italic characteristic and level rows; removal of displayed
N columns, complete-case notes, abbreviations, or the automatic adjustment
note; and a final custom note. Label mappings are entered one per line:

```text
age = Maternal age
smoke: Yes = Smoker
estimate = Adjusted OR
```

The first line changes a variable label, the second changes one factor level,
and the third changes a visible header. The generated code remains available in
the Visualise & Export **Code** tab, while the whole reproducible workflow is
available from the top-level **Session Code** tab.

For a level label, the text before the colon must be the internal variable name,
and the text between `:` and `=` must match the current level exactly. For
example, use `smoke: Yes = Smoker`, not `Smoking during pregnancy: Yes =
Smoker` after changing the variable's display label.

`plot_reg_combine()` is intended for standard univariable and multivariable
outputs. Stratified outputs are better handled carefully with separate
`plot_reg()` calls or forest tables, depending on the publication need.

### Forest Tables

Choose **Forest plot** from the top menu. **Selected result** creates
`forest_df()` from one result. **Crude + adjusted** combines the stored
univariable and multivariable objects. **Descriptive + crude + adjusted** is a
separate option and appears only after all three results exist; descriptive
columns are therefore never added unexpectedly.

Cox and parametric survival results follow the same pattern. After fitting
both the exposure-model and multivariable modes, choose their named combined
option. If a descriptive table is also available, choose the corresponding
**Descriptive + ...** option to place those columns beside the survival effect
estimates. This prevents an older regression result from being combined with
the currently selected survival analysis by accident.

Use forest plots for manuscript-style effect summaries.

For one regression object:

```r
forest_data <- forest_df(multi_result)

forest_plot <- forest_reg(
  forest_data,
  ci_col_width = 20
)
```

For descriptive plus crude plus adjusted results:

```r
forest_both <- forest_df(
  uni_result,
  multi_result,
  desc = desc_result
)

forest_both_plot <- forest_reg(
  forest_both,
  ci_col_width = 24,
  xlim = c(0.25, 12),
  ticks_at = c(0.5, 1, 2, 4, 8)
)

save_forest(
  forest_both_plot,
  filename = "forest_reg_combined.pdf",
  width = 13,
  height = 9
)
```

If the forest plot looks crowded:

- increase `save_forest()` `width`;
- increase `save_forest()` `height` for many rows;
- increase `ci_col_width` when the confidence interval drawing area is narrow;
- set `xlim` to control the x-axis range;
- set fewer or clearer `ticks_at` values if x-axis labels overlap.

Example for a crowded odds-ratio forest plot:

```r
forest_reg(
  forest_both,
  ci_col_width = 28,
  xlim = c(0.25, 20),
  ticks_at = c(0.5, 1, 2, 5, 10, 20)
)
```

For a very long table, export instead of judging the small browser preview:

```r
save_forest(
  forest_both_plot,
  filename = "large_forest_plot.pdf",
  width = 16,
  height = 12
)
```

## Advanced Tab

The **Advanced** tab supports model building, diagnostics, confounding, and
interaction assessment without presenting every setting at once.

Choose one tool across the top of the page:

| Tool | Question it helps answer | Inputs shown |
|---|---|---|
| Select models | Which candidate path has the preferred AIC? | Model roles, candidate predictors, direction |
| Compare models | How do named candidate models differ? | Shared outcome/approach, model-specific exposures, adjustment variables, interactions, and optional primary exposure |
| Confounder | How much does a candidate change the exposure estimate? | Model roles, exposure, potential confounder |
| Interaction | Does the exposure association differ between groups? | Model roles, exposure, modifier, adjustment variables |
| Convergence | Did the requested models fit reliably? | Outcome, predictors, supported approach |
| Collinearity | Do fitted predictors contain overlapping information? | One fitted multivariable model |

The left panel then shows only the controls needed by that tool. For Cox and
parametric survival tools, **Follow-up time** and **Event status** replace the
ordinary outcome selector. Survival distribution appears only for parametric
survival regression.

After running a tool, the app opens the **Output** panel and provides:

- a publication-style table when the function returns one;
- concise guidance specific to that result;
- console messages and warnings;
- DOCX, RTF, and HTML downloads;
- a complete copyable command under **Code**.

In **Compare models**, the app fits each candidate model for you. Choose the
outcome and regression approach once, then define two to six candidates. Each
candidate has a meaningful name, reported exposures, adjustment variables, and
an optional interaction. The app creates compatible `multi_reg()`, `cox_reg()`,
or `surv_reg()` objects and passes them to `compare_models()`. The generated code
shows every fitted model, so the comparison is fully reproducible in RStudio.

Use **Primary exposure to track** when the same exposure appears in every
candidate. The comparison then shows how its estimate changes as variables are
added. Variables selected as adjustment variables are included in the fitted
model even though the app distinguishes them from the exposures being reported.

Collinearity requires an explicitly selected fitted multivariable model,
because it is a property of predictors fitted together.

### Build and Compare Candidate Models

The app version of **Compare models** is designed for users who have a set of
planned models but have not fitted them elsewhere. It does two jobs in a visible
sequence:

1. it fits each named candidate with a gtregression modelling function; and
2. it sends those fitted gtregression objects to `compare_models()`.

To use it:

1. Open **Advanced**, then choose **Compare models**.
2. Select the common regression approach and outcome. For Cox or parametric
   survival regression, select follow-up time and event status instead.
3. Optionally select a **Primary exposure to track**. Use the same exposure in
   every candidate when you want to inspect estimate stability.
4. Give each candidate a short, meaningful name such as *Clinical core*,
   *Expanded model*, or *Sensitivity model*.
5. Select the exposures and adjustment variables for that candidate.
6. Optionally add one interaction. Both interaction variables must already be
   included in that candidate as an exposure or adjustment variable.
7. Add further candidates as needed, then choose **Fit and compare models**.

The app requires at least two candidates, permits up to six, and requires unique
non-empty model names. Candidate order matters: the first model is the reference
for percentage change in the primary estimate, and sequential likelihood-ratio
comparisons follow the displayed order.

The **reported exposures** and **adjustment variables** controls make the model
building intention easy to read. Both sets are included in the fitted model.
The generated code shows their combined predictor set explicitly, so there is
no hidden formula construction.

#### Birth Weight Example

With the prepared `data_birthwt` example, choose:

| App control | Selection |
|---|---|
| Approach | Logistic regression |
| Outcome | `low` |
| Primary exposure | `smoke` |
| Candidate 1 name | Clinical core |
| Candidate 1 exposures | `smoke` |
| Candidate 1 adjustment variables | `age`, `lwt` |
| Candidate 2 name | Expanded model |
| Candidate 2 exposures | `smoke` |
| Candidate 2 adjustment variables | `age`, `lwt`, `race`, `ht` |

This is equivalent to the R code shown in the next section. To assess a planned
interaction, add a third candidate containing both `smoke` and `ht`, then select
`smoke` and `ht` as the interaction pair. Keep the corresponding main effects
in the model when interpreting the interaction.

#### Cox and Parametric Survival Examples

For the lung cancer example, the same builder can compare survival models:

| App control | Cox example |
|---|---|
| Approach | Cox regression |
| Follow-up time | `time` |
| Event status | `status` |
| Primary exposure | `trt` |
| Candidate 1 | Treatment only: exposure `trt` |
| Candidate 2 | Clinical adjustment: exposure `trt`; adjust for `age`, `karno` |

Choose parametric survival regression instead to build `surv_reg()` candidates.
The app then reveals the distribution control; all candidates in that comparison
use the selected distribution. Cox comparisons report hazard ratios and events,
while parametric survival comparisons report the model's time-ratio estimates.

#### Reading the Comparison

The output combines fit statistics with interpretation checks:

- **Model** uses the names entered in the candidate cards.
- **N** and **Events** show the fitted analysis sample; events appear for
  survival models.
- **AIC**, **BIC**, and **Log-likelihood** describe relative model fit.
- **LR statistic**, **df**, and **p-value** compare sequential models when
  likelihood-ratio comparison is requested.
- **Concordance** appears where available for Cox models.
- **Primary estimate** and **Change from first model** appear when a primary
  exposure is selected.

The status immediately below the table is context-aware. **Same analysis
sample** means likelihood-based statistics can be compared, provided the models
are also nested. **Different analysis sample** means missingness or inclusion
criteria changed the fitted observations; the statistics remain visible for
transparency, but estimate stability and clinical reasoning deserve greater
weight. A separate status identifies non-nested sequential comparisons.

Open **Code** to copy every model call and the final `compare_models()` call.
This is the recommended route for revising a candidate, auditing the formulas,
or retaining the full analysis in a script. The table can also be downloaded as
DOCX, RTF, or HTML.

### Model Selection and Comparison

Use `select_models()` when exploring candidate model paths:

```r
selected_models <- select_models(
  data = df,
  outcome = "low",
  exposures = c("age", "lwt", "smoke", "race"),
  approach = "logit",
  direction = "both",
  format = "flextable"
)
```

The app builds the fitted objects before calling `compare_models()`. The
equivalent R workflow is:

```r
clinical <- multi_reg(
  data = df,
  outcome = "low",
  exposures = c("smoke", "age", "lwt"),
  approach = "logit"
)

expanded <- multi_reg(
  data = df,
  outcome = "low",
  exposures = c("smoke", "age", "lwt", "race", "ht"),
  approach = "logit"
)

comparison_table <- compare_models(
  clinical = clinical,
  expanded = expanded,
  primary_exposure = "smoke",
  format = "flextable"
)
```

The comparison output warns users when models are fitted on different analysis
samples or when likelihood-ratio comparisons need caution.

For Cox models, the equivalent generated workflow is:

```r
treatment_only <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt
)

clinical_adjustment <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno)
)

compare_models(
  `Treatment only` = treatment_only,
  `Clinical adjustment` = clinical_adjustment,
  primary_exposure = trt
)
```

`select_models()` and **Compare models** answer different questions.
`select_models()` explores a stepwise path from one predictor pool. **Compare
models** fits and compares the distinct, prespecified candidates entered by the
user, including candidates with different adjustment sets or interactions.

### Role Checks

The app checks for common role conflicts before fitting models. For example, the
same variable should not be selected as:

- the outcome and an exposure;
- the survival time and an exposure;
- the survival event and an adjustment variable;
- the stratifier and an exposure;
- both an exposure and an adjustment variable.

When this happens, the app shows a direct message explaining what to change.
These checks prevent silent mistakes and make the generated code cleaner.

### Confounding and Interaction

Use `identify_confounder()` and `interaction_models()` as screening aids, not
as replacements for study design and subject-matter judgement.

Example confounder check:

```r
confounder_check <- identify_confounder(
  data = df,
  outcome = "low",
  exposure = "smoke",
  potential_confounder = "race",
  approach = "logit",
  format = "flextable"
)
```

Example interaction check:

```r
interaction_check <- interaction_models(
  data = df,
  outcome = "low",
  exposure = "smoke",
  effect_modifier = "race",
  covariates = c("age", "lwt"),
  approach = "logit",
  format = "flextable"
)
```

Interpret confounding and effect modification with DAGs, temporal order,
clinical reasoning, and model diagnostics.

## Mediation Tab

The **Mediation** tab runs `mediation_analysis()` and draws a mediation path
diagram.

Typical workflow:

1. Choose exposure.
2. Choose mediator.
3. Choose outcome.
4. Add covariates.
5. Choose `logit` or `linear` outcome model.
6. Choose bootstrap replicates.
7. Run mediation analysis.
8. Review the table and diagram.
9. Copy the code.

Example:

```r
mediation_result <- mediation_analysis(
  data = df,
  exposure = "obesity",
  mediator = "glucose",
  outcome = "diabetes",
  covariates = c("age", "blood_pressure", "pregnancies", "diabetes_pedigree"),
  outcome_approach = "logit",
  sims = 300,
  seed = 123,
  format = "flextable"
)

plot_mediation(mediation_result)
```

Mediation results require strong assumptions. Interpret them causally only when
the design, DAG, temporal order, and no-unmeasured-confounding assumptions are
defensible.

## Saving Outputs

The app provides download buttons for tables and plots. For final publication
work, copied code gives more control.

Tables:

```r
save_table(merged_table, filename = "table_1.docx")
save_table(merged_table, filename = "table_1.rtf")
save_table(merged_table, filename = "table_1.html")
```

The app displays publication-ready tables with the same flextable engine used
for DOCX and RTF export. Use HTML when you want a browser-friendly copy of the
same table. Direct table PDF export is intentionally not shown in the app
because flextable output is more reliable through DOCX, RTF, or HTML.

Plots:

```r
save_plot(reg_plot, filename = "regression_plot.png", width = 9, height = 6)
save_plot(km_result, filename = "km_plot.png", width = 8, height = 6)
```

Forest plots:

```r
save_forest(
  forest_both_plot,
  filename = "forest_plot.pdf",
  width = 13,
  height = 9
)
```

When a save function is called without a user-supplied destination, the package
uses a temporary location. This is helpful for examples, testing, and CRAN-safe
documentation.

## Troubleshooting

### The App Does Not Open

Check that `shiny` is installed:

```r
install.packages("shiny")
```

Then run:

```r
gtregression_app()
```

### The Browser Preview Looks Cramped

Browser preview size depends on the device, browser, zoom level, and RStudio
Viewer size. A cramped preview does not always mean the exported figure will be
cramped.

For final outputs, copy the code and export with a wider canvas:

```r
save_forest(forest_plot, "forest_plot.pdf", width = 16, height = 10)
```

### Forest Plot X-Axis Labels Overlap

Control the axis explicitly:

```r
forest_reg(
  forest_data,
  xlim = c(0.25, 20),
  ticks_at = c(0.5, 1, 2, 5, 10, 20)
)
```

Use fewer tick marks when labels are too close together.

### Forest Plot Columns Feel Too Narrow

Increase the confidence interval column width:

```r
forest_reg(
  forest_data,
  ci_col_width = 28
)
```

Also increase the saved output width:

```r
save_forest(forest_plot, "forest_plot.pdf", width = 16, height = 10)
```

### Kaplan-Meier Curves Look Almost Flat

If survival probabilities stay high, use y-axis limits:

```r
km_plot(
  data = df,
  time = "time",
  event = "status",
  by = "trt",
  y_percent = TRUE,
  ylim = c(50, 100)
)
```

If `y_percent = FALSE`, use probability limits instead:

```r
km_plot(
  data = df,
  time = "time",
  event = "status",
  by = "trt",
  y_percent = FALSE,
  ylim = c(0.5, 1)
)
```

### Merged Tables Do Not Align as Expected

Use reference rows in regression tables:

```r
uni_result <- uni_reg(
  data = df,
  outcome = "low",
  exposures = exposures,
  approach = "logit",
  show_ref = TRUE
)
```

Then rerun the merged table:

```r
merge_tables(desc_result, uni_result, multi_result)
```

### Chrome or Browser Crashes

Use the RStudio Viewer where available by running:

```r
gtregression_app()
```

If the browser keeps causing problems, run:

```r
gtregression_app(launch.browser = FALSE)
```

and open the printed local Shiny URL manually in another browser.

### Nothing Appears After Clicking a Button

Check that:

- a dataset has been loaded;
- required variables are selected;
- the outcome or time/event variables are selected;
- at least two uniquely named candidate cards are complete before running
  **Compare models**;
- descriptive and regression outputs exist before running `merge_tables()`.

## Beginner Case Study: Birth Weight

This example mirrors a common app workflow.

```r
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = ifelse(ptl > 0, "Yes", "No"),
    ftv_cat = case_when(
      ftv == 0 ~ "None",
      ftv == 1 ~ "One",
      ftv >= 2 ~ "Two or more"
    ),
    ptl_cat = factor(ptl_cat, levels = c("No", "Yes")),
    ftv_cat = factor(ftv_cat, levels = c("None", "One", "Two or more"))
  )

exposures <- c("age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat", "ftv_cat")

desc_result <- descriptive_table(
  data = birthwt_data,
  exposures = exposures,
  by = "low",
  percent = "column",
  show_overall = "last"
)

uni_result <- uni_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = exposures,
  approach = "logit",
  show_ref = TRUE
)

multi_result <- multi_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = c("smoke", "ht", "ui"),
  adjust_for = c("age", "lwt", "race", "ptl_cat", "ftv_cat"),
  approach = "logit",
  show_ref = TRUE
)

merged_table <- merge_tables(
  desc_result,
  uni_result,
  multi_result,
  spanners = c("Descriptive", "Crude", "Adjusted")
)

forest_data <- forest_df(uni_result, multi_result, desc = desc_result)

forest_plot <- forest_reg(
  forest_data,
  ci_col_width = 24,
  xlim = c(0.25, 12),
  ticks_at = c(0.5, 1, 2, 4, 8)
)
```

The app helps users build this workflow by clicking through tabs. The copied
code gives them the same analysis in script form.

## Best Practice

Use the app to learn, explore, and generate reproducible code. Use the copied
code in RStudio for final analyses, manuscript tables, and final figure exports.

---
title: "Causal Mediation Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Causal Mediation Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Mediation analysis asks whether part of an exposure-outcome association may pass
through an intermediate variable. In health research, that question is usually
interesting only after the clinical story, temporal order, and likely
confounding structure have already been considered.

`gtregression` provides a compact workflow for:

- fitting the mediator and outcome models;
- estimating total, direct, indirect, and proportion mediated effects;
- displaying a publication-style table;
- drawing a simple mediation path diagram.

The output is deliberately transparent. It is a model-based aid to
interpretation, not proof of causality by itself.

## Example Question

This article uses `data_diabetes_mediation`, a teaching dataset based on a
diabetes risk profile. The example question is:

> Does plasma glucose explain part of the association between obesity and
> diabetes?

```{r setup, message=FALSE, warning=FALSE}
library(gtregression)
library(dplyr)

data("data_diabetes_mediation", package = "gtregression")

dissect(data_diabetes_mediation)
```

## Logistic Outcome

For a binary outcome, use `outcome_approach = logit`. The effects are reported as
predicted probability differences, which are often easier to explain than odds
ratios in a mediation table.

For final analyses, use a larger number of bootstrap simulations such as
`sims = 500` or `sims = 1000`. The article uses a smaller value to keep the
example quick to run.

```{r mediation-logit, message=FALSE, warning=FALSE}
diabetes_med <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = obesity,
  mediator = glucose,
  outcome = diabetes,
  covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
  outcome_approach = logit,
  sims = 100,
  seed = 123
)

diabetes_med
```

The returned object keeps the table body, fitted models, bootstrap draws, and
exposure comparison values available for checking.

```{r inspect-object}
diabetes_med$table_body
diabetes_med$values
diabetes_med$models$mediator
diabetes_med$models$outcome
head(diabetes_med$boot)
```

## Path Diagram

`plot_mediation()` draws the exposure, mediator, outcome, and the direct and
indirect paths.

```{r mediation-plot, fig.width=7, fig.height=5}
plot_mediation(diabetes_med)
```

If the figure is being used only to explain the causal structure, hide the
estimates.

```{r mediation-plot-no-estimates, fig.width=7, fig.height=5}
plot_mediation(diabetes_med, show_estimates = FALSE)
```

## Quoted Names

Quoted column names and stored character vectors work too. This is useful inside
scripts, functions, and Shiny-style workflows.

```{r mediation-quoted, message=FALSE, warning=FALSE}
exposure_var <- "obesity"
mediator_var <- "glucose"
outcome_var <- "diabetes"
covariate_vars <- c(
  "age", "blood_pressure", "pregnancies", "diabetes_pedigree"
)

med_quoted <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = exposure_var,
  mediator = mediator_var,
  outcome = outcome_var,
  covariates = covariate_vars,
  outcome_approach = "logit",
  sims = 100,
  seed = 456
)

med_quoted
```

## Linear Outcome

For a continuous outcome, use `outcome_approach = linear`. In this example, the
outcome is body mass index, so the effects are reported as mean differences.

```{r mediation-linear, message=FALSE, warning=FALSE}
med_linear <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = obesity,
  mediator = glucose,
  outcome = bmi,
  covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
  outcome_approach = linear,
  sims = 100,
  seed = 789
)

med_linear
```

```{r mediation-linear-plot, fig.width=7, fig.height=5}
plot_mediation(med_linear)
```

## How To Report

A compact reporting sentence might look like this:

> In this teaching analysis, plasma glucose explained part of the model-based
> obesity-diabetes association. Effects were estimated on the predicted
> probability difference scale using logistic outcome models and bootstrap
> confidence intervals.

The table footnote records the exposure comparison, mediator, outcome, bootstrap
replicates, and adjustment variables so readers can see what was estimated.

## What Not To Claim

Mediation estimates should not be treated as automatic causal proof. A cautious
analysis should consider:

- whether the exposure clearly precedes the mediator;
- whether the mediator clearly precedes the outcome;
- whether exposure-mediator, mediator-outcome, and exposure-outcome confounding
  have been handled;
- whether post-exposure confounders are present;
- whether the model forms are plausible;
- whether a DAG or subject-matter argument supports the causal interpretation.

Use the table and plot to support interpretation after that thinking has been
done.

---
title: "Regression Tables"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Regression Tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Go from fitted models to publication-ready tables without hand-formatting
effect estimates. `gtregression` supports logistic, log-binomial, Poisson,
robust Poisson, negative binomial, Cox survival, parametric survival, and
linear regression.

```{r reg-setup, message=FALSE, warning=FALSE}
library(gtregression)
library(dplyr)

data("data_birthwt", package = "gtregression")

birthwt_data <- data_birthwt |>
  mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    ptl_cat = factor(ifelse(ptl > 0, "Yes", "No"), levels = c("No", "Yes")),
    ftv_cat = factor(case_when(
      ftv == 0 ~ "None",
      ftv == 1 ~ "One",
      ftv >= 2 ~ "Two or more"
    ), levels = c("None", "One", "Two or more"))
  )

birthwt_exposures <- c(
  "age", "lwt", "race", "smoke", "ht", "ui", "ptl_cat", "ftv_cat"
)

attr(birthwt_data$age, "label") <- "Maternal age"
attr(birthwt_data$lwt, "label") <- "Maternal weight"
attr(birthwt_data$race, "label") <- "Maternal race"
attr(birthwt_data$smoke, "label") <- "Smoking during pregnancy"
attr(birthwt_data$ht, "label") <- "Hypertension"
attr(birthwt_data$ui, "label") <- "Uterine irritability"
attr(birthwt_data$ptl_cat, "label") <- "Previous preterm labour"
attr(birthwt_data$ftv_cat, "label") <- "First trimester visits"
```

## Univariable Models

`uni_reg()` fits one model per exposure and returns a table ready for reports.
Variable labels set with `attr(x, "label")` or `labelled::var_label()` are used
automatically in the displayed table, while raw column names remain in
`$table_body`.

```{r uni-logit, message=FALSE, warning=FALSE}
birthwt_uni <- uni_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = birthwt_exposures,
  approach = "logit",
  theme = clinical
)

birthwt_uni
```

## Multivariable Models

With `adjust_for = NULL`, `multi_reg()` fits all supplied exposures in one
multivariable model. This is the usual fully adjusted model when every exposure
listed should appear in the same formula.

```{r multi-full, message=FALSE, warning=FALSE}
birthwt_full <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("age", "lwt", "race", "smoke", "ht", "ui"),
  approach = logit,
  theme = clinical
)

birthwt_full
```

## Exposure-Specific Adjusted Models

Use `adjust_for` when you want one adjusted model per exposure, each adjusted for
the same core covariate set. This is useful for screening several clinically
important exposures while keeping the adjustment strategy explicit.

```{r multi-adjusted, message=FALSE, warning=FALSE}
birthwt_multi <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat", "ftv_cat"),
  adjust_for = c("age", "lwt", "race"),
  approach = logit,
  theme = striped
)

birthwt_multi
```

The adjustment variables are recorded in a compact table footnote, so the result
is ready for manuscript-style reporting without making the table unnecessarily
tall.

## Optional Model Statistics

Publication tables should stay readable. When you need model-fit information,
set `model_stats = TRUE` and inspect the returned object's `$model_stats`
element. This keeps AIC, BIC, log-likelihood, deviance, pseudo R-squared, and
linear-model R-squared values available without adding clutter to the main
table.

```{r model-stats, message=FALSE, warning=FALSE}
birthwt_uni_stats <- uni_reg(
  data = birthwt_data,
  outcome = low,
  exposures = birthwt_exposures,
  approach = logit,
  model_stats = TRUE
)

birthwt_uni_stats$model_stats
```

For adjusted mode, `multi_reg()` returns one row of statistics per adjusted
exposure-specific model.

```{r adjusted-model-stats, message=FALSE, warning=FALSE}
birthwt_multi_stats <- multi_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat"),
  adjust_for = c("age", "lwt", "race"),
  approach = logit,
  model_stats = TRUE
)

birthwt_multi_stats$model_stats
```

## Other Effect Measures

Switch the `approach` to change the estimand.

```{r reg-risk-ratio, message=FALSE, warning=FALSE}
uni_reg(
  data = birthwt_data,
  outcome = low,
  exposures = c("smoke", "ht", "ui", "ptl_cat"),
  approach = logbinomial
)
```

Use `approach = firth` when a binary-outcome logistic model has sparse cells,
very wide intervals, or separation concerns. The output remains an odds-ratio
table, but the model is fitted with Firth penalized logistic regression.

The built-in `data_endometrial` dataset is a useful teaching example because
neovascularization is completely absent among low-grade cases, a pattern that
can make ordinary logistic regression unstable.

```{r reg-firth, message=FALSE, warning=FALSE}
data("data_endometrial", package = "gtregression")

endometrial_data <- data_endometrial |>
  mutate(
    HG = factor(HG, levels = c(0, 1),
                labels = c("Low grade", "High grade")),
    NV = factor(NV, levels = c(0, 1), labels = c("Absent", "Present"))
  )

multi_reg(
  data = endometrial_data,
  outcome = HG,
  exposures = c(NV, PI, EH),
  approach = firth
)
```

## Cox Survival Models

`cox_reg()` uses direct `time` and `event` arguments and returns
hazard ratios. Without `adjust_for`, the table shows crude HRs. With
`adjust_for`, the table shows adjusted HRs.

```{r cox-setup, message=FALSE, warning=FALSE}
data("data_lungcancer", package = "gtregression")

lung_data <- data_lungcancer |>
  mutate(
    trt = factor(trt, levels = c(1, 2),
                 labels = c("Standard treatment", "Test treatment")),
    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
  )

attr(lung_data$trt, "label") <- "Treatment group"
attr(lung_data$celltype, "label") <- "Cancer cell type"
attr(lung_data$karno, "label") <- "Karnofsky performance score"
attr(lung_data$age, "label") <- "Age"
attr(lung_data$prior, "label") <- "Prior therapy"
```

```{r cox-crude, message=FALSE, warning=FALSE}
lung_hr <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c("trt", "celltype", "karno", "age"),
  theme = clinical
)

lung_hr
```

```{r cox-adjusted, message=FALSE, warning=FALSE}
lung_adj_hr <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno),
  model_stats = TRUE,
  theme = striped
)

lung_adj_hr
lung_adj_hr$model_stats
```

Use `interaction = exposure*modifier` for planned effect modification. In the
default exposure-by-exposure workflow, provide one exposure.

```{r cox-interaction, message=FALSE, warning=FALSE}
cox_interaction <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno),
  interaction = trt*prior
)

cox_interaction
```

## Parametric Survival Models

`surv_reg()` uses the same `time`, `event`, `exposures`, and `adjust_for`
grammar as `cox_reg()`, but fits parametric survival models with
`survival::survreg()`. The table reports time ratios rather than hazard ratios.
A time ratio above 1 suggests longer survival time; below 1 suggests shorter
survival time, conditional on the chosen distribution.

```{r surv-crude, message=FALSE, warning=FALSE}
lung_time_ratio <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c("trt", "celltype", "karno", "age"),
  distribution = weibull,
  theme = clinical
)

lung_time_ratio
```

```{r surv-adjusted, message=FALSE, warning=FALSE}
lung_adj_time_ratio <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno),
  distribution = lognormal,
  model_stats = TRUE,
  theme = striped
)

lung_adj_time_ratio
lung_adj_time_ratio$model_stats
```

```{r surv-interaction, message=FALSE, warning=FALSE}
surv_interaction <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno),
  interaction = trt*prior,
  distribution = weibull
)

surv_interaction
```

## Continuous Outcomes

Linear regression outputs beta coefficients and keeps diagnostics under
`$reg_check`.

```{r reg-linear, message=FALSE, warning=FALSE}
birthwt_linear <- multi_reg(
  data = birthwt_data,
  outcome = bwt,
  exposures = c("age", "lwt", "race", "smoke", "ht", "ui"),
  approach = linear
)

birthwt_linear
```

## What To Inspect

- `$table`: publication-ready table.
- `$table_body`: numeric estimates behind the display.
- `$models`: fitted model objects.
- `$model_summaries`: model-level summaries.
- `$model_stats`: optional model-fit statistics when `model_stats = TRUE`.
- `$variable_labels`: labels used in publication output.
- `$reg_check`: diagnostics for linear models.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uni_reg.R
\name{uni_reg}
\alias{uni_reg}
\title{Univariate regression}
\usage{
uni_reg(
  data,
  outcome,
  exposures,
  approach = "logit",
  format = c("flextable", "gt"),
  theme = c("minimal"),
  model_stats = FALSE,
  show_ref = TRUE
)
}
\arguments{
\item{data}{A data frame containing the outcome and exposure variables.}

\item{outcome}{Character scalar; outcome column name. Quoted and bare names
are accepted.}

\item{exposures}{Character vector; exposure column names. Quoted names are
recommended in scripts, and bare names are also accepted.}

\item{approach}{Regression approach. One of \code{"logit"}, \code{"firth"},
\code{"logbinomial"}, \code{"poisson"}, \code{"robpoisson"},
\code{"linear"}, or \code{"negbin"}. Use \code{"firth"} for Firth
penalized logistic regression, especially with sparse cells or separation.}

\item{format}{One of \code{"flextable"} (default) or \code{"gt"}.}

\item{theme}{Preset name (e.g. \code{"minimal"}, \code{"striped"}, \code{"clinical"},
\code{"shaded"}, \code{"jama"}) or primitives
\code{c("plain","zebra","lines","labels_bold","compact","header_shaded")}}

\item{model_stats}{Logical; if \code{TRUE}, extract model-fit statistics
such as AIC, BIC, log-likelihood, deviance, pseudo R-squared for
non-linear models, and R-squared for linear models. Statistics are stored
in the returned object's \code{model_stats} element and are not added to
the publication table.}

\item{show_ref}{Logical; if \code{TRUE} (default), display reference-category
rows as \code{"Ref."}. If \code{FALSE}, hide reference rows; a message
reminds users to use \code{show_ref = TRUE} when reference rows are needed.}
}
\value{
A list of class \code{c("gtregression","uni_reg", ...)} with elements:
\describe{
  \item{table}{A \code{flextable} (when \code{format="flextable"}) or
  \code{gt_tbl} (when \code{format="gt"}).}
  \item{table_body}{Data frame of numeric estimates and CIs.}
  \item{table_display}{Data frame for display (headers + levels).}
  \item{models}{List of fitted univariate models.}
  \item{model_summaries}{Per-model \code{summary()} results.}
  \item{model_stats}{Model-fit statistics when \code{model_stats = TRUE};
  otherwise \code{NULL}.}
  \item{variable_labels}{Named character vector of display labels used for
  exposure variables.}
  \item{reg_check}{Diagnostics for linear models. Printing this element
  renders a publication-ready table; individual diagnostic rows remain
  available for programmatic use. Other approaches return a message.}
  \item{approach, format, source}{Metadata fields.}
}
}
\description{
Fit one model per exposure and return a clean regression table in
\pkg{flextable} or \pkg{gt} format.
}
\details{
Use this when you want a quick crude association table before building an
adjusted model. The fitted models are kept in the returned object, so the
formatted table does not hide the underlying analysis.

If exposure variables have a \code{"label"} attribute, for example from
\code{labelled::var_label()}, those labels are used automatically in the
displayed table and plots. Internal matching still uses the original column
names.
}
\examples{
d <- mtcars
if (requireNamespace("gt", quietly = TRUE)) {
  uni_reg(d, "am", c("mpg","cyl"), approach = "logit", format = "gt")$table
}
if (requireNamespace("flextable", quietly = TRUE)) {
  ft <- uni_reg(d, "am", c("mpg","cyl"), approach = "logit", format = "flextable")
  class(ft$table)
}

endometrial_data <- data_endometrial
endometrial_data$HG <- factor(endometrial_data$HG, levels = c(0, 1))
endometrial_data$NV <- factor(endometrial_data$NV, levels = c(0, 1))
uni_reg(endometrial_data, HG, c(NV, PI, EH), approach = firth, format = gt)$table

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_model_compare.R
\name{surv_model_compare}
\alias{surv_model_compare}
\title{Compare parametric survival model distributions}
\usage{
surv_model_compare(
  data,
  time,
  event,
  exposures,
  adjust_for = NULL,
  distributions = c("weibull", "exponential", "lognormal", "loglogistic"),
  digits = 2,
  format = c("flextable", "gt", "tibble"),
  theme = c("minimal")
)
}
\arguments{
\item{data}{A \code{data.frame} containing survival time, event status, and
predictor variables.}

\item{time}{Survival follow-up time. Quoted and bare names are accepted.}

\item{event}{Event indicator. Quoted and bare names are accepted. Numeric
\code{0/1}, numeric \code{1/2}, logical, character, and factor variables
are accepted. For two-level character or factor variables, the second level
is treated as the event.}

\item{exposures}{Character vector of main exposure variable names. Quoted
names are recommended in scripts, and bare names are also accepted.}

\item{adjust_for}{Optional character vector of adjustment variables. These
variables are included in every candidate model.}

\item{distributions}{Parametric survival distributions to compare. Defaults
to \code{c("weibull", "exponential", "lognormal", "loglogistic")}.
Quoted and bare values are accepted. Common spellings such as
\code{"log-normal"} and \code{"log-logistic"} are also accepted.}

\item{digits}{Number of digits for displayed model statistics.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}.}

\item{theme}{Table styling preset.}
}
\value{
A list of class \code{c("gtregression","surv_model_compare", ...)}
with elements:
\describe{
  \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
  when \code{format = "tibble"}.}
  \item{\code{table_body}}{Tibble with model-fit statistics.}
  \item{\code{table_display}}{Display data frame used to render the table.}
  \item{\code{models}}{Named list of fitted \code{survreg} models.}
  \item{\code{time,event,exposures,adjust_for,distributions,format,source}}{Metadata fields.}
}
}
\description{
Fit the same parametric survival regression model using multiple
distributions and compare model-fit statistics such as AIC and BIC.
}
\details{
The same model formula is fitted for each candidate distribution using
\code{survival::survreg()}. Lower AIC or BIC values indicate better relative
model fit among the compared distributions. These statistics should be used
with clinical judgment and visual checks; they do not prove that a
distribution is scientifically correct.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))
lung_data$prior <- factor(lung_data$prior, levels = c(0, 10),
                          labels = c("No", "Yes"))

surv_model_compare(
  data = lung_data,
  time = time,
  event = status,
  exposures = c("trt", "celltype", "prior"),
  adjust_for = c("age", "karno")
)

surv_model_compare(
  data = lung_data,
  time = "time",
  event = "status",
  exposures = c(trt, prior),
  distributions = c(weibull, "log-logistic"),
  format = tibble
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{data_diabetes_mediation}
\alias{data_diabetes_mediation}
\title{Diabetes Mediation Teaching Dataset}
\format{
A data frame with variables:
\describe{
  \item{diabetes}{Diabetes status (\code{No}, \code{Yes})}
  \item{obesity}{Obesity status based on BMI >= 30 (\code{No}, \code{Yes})}
  \item{glucose}{Plasma glucose concentration}
  \item{bmi}{Body mass index}
  \item{age}{Age in years}
  \item{blood_pressure}{Diastolic blood pressure}
  \item{pregnancies}{Number of pregnancies}
  \item{diabetes_pedigree}{Diabetes pedigree function}
}
}
\source{
Derived from \code{data_SynthDiabetes}, which is based on
\code{mlbench::SynthDiabetes2} (\pkg{mlbench} version 2.1-11 or later).
}
\usage{
data_diabetes_mediation
}
\description{
A health-related teaching dataset prepared from synthetic diabetes data.
It is designed for the practical question: could part of the obesity
and diabetes relationship operate through plasma glucose, after adjustment
for basic clinical covariates?
}
\details{
This dataset is intended for practice and teaching. Mediation estimates from
observational data should be interpreted causally only when the temporal
ordering and no-unmeasured-confounding assumptions are justified by study
design, DAGs, and subject-matter knowledge.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_collinearity.R
\name{check_collinearity}
\alias{check_collinearity}
\title{Check collinearity using VIF for fitted models}
\usage{
check_collinearity(model, format = c("flextable", "gt", "tibble"))
}
\arguments{
\item{model}{A fitted model object returned by \code{multi_reg()},
\code{stratified_multi_reg()}, or compatible \code{gtregression}
functions. Univariable model objects are rejected because VIF is not
applicable.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}. Use \code{format = "tibble"} to preserve
the original tibble or nested-list output.}
}
\value{
For multivariable models, a tibble if a single fitted model is
present, or a named list of tibbles if multiple fitted models are present.
With \code{format = "gt"} or \code{format = "flextable"}, leaf tibbles are
converted to formatted tables while preserving any list nesting. The tibble
contains:
\describe{
  \item{\code{Variable}}{Model term.}
  \item{\code{VIF}}{Variance inflation factor. For multi-degree-of-freedom
  terms, this is the adjusted GVIF: \code{GVIF^(1 / (2 * Df))}.}
  \item{\code{Interpretation}}{Simple interpretation based on common cut
  points: no collinearity, moderate, or high.}
}

For univariate models, an error is raised indicating that VIF is not applicable.
}
\description{
Computes Variance Inflation Factors (VIF) for fitted multivariable models
returned by \code{multi_reg()} or related functions.
}
\details{
If the \pkg{car} package is installed, \code{check_collinearity()}
uses \code{car::vif()}. Otherwise, it computes VIF/GVIF from the fitted model
matrix so that diagnostics remain available without an additional dependency.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/footnotes.R
\name{.footnotes_uni_strata}
\alias{.footnotes_uni_strata}
\title{Compose footnotes for univariate strata}
\usage{
.footnotes_uni_strata(approach)
}
\description{
Compose footnotes for univariate strata
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/descriptive_table.R
\name{descriptive_table}
\alias{descriptive_table}
\title{Descriptive Summary Table (no gtsummary) using gt/flextable}
\usage{
descriptive_table(
  data,
  exposures,
  by = NULL,
  percent = c("column", "row"),
  digits = 1,
  show_missing = c("ifany", "no"),
  show_dichotomous = c("all_levels", "single_row"),
  show_overall = c("no", "first", "last"),
  statistic = NULL,
  value = NULL,
  format = c("flextable", "gt"),
  theme = c("minimal")
)
}
\arguments{
\item{data}{data.frame}

\item{exposures}{Character vector of variables to summarise. Quoted names are
recommended in scripts, and bare names are also accepted.}

\item{by}{Optional single grouping variable. Quoted and bare names are
accepted.}

\item{percent}{"column" (default) or "row"; aliases like "col"/"rows" accepted}

\item{digits}{integer; decimals for \% and continuous stats (default 1)}

\item{show_missing}{"ifany" (default) or "no"}

\item{show_dichotomous}{"all_levels" (default) or "single_row"}

\item{show_overall}{"no" (default), "first", or "last"}

\item{statistic}{Optional summary type for numeric variables. Use a single
value such as \code{"mean"} or \code{mean} for all numeric variables, or a
named vector such as \code{c(age = mean, ftv = categorical)}. Supported
values are \code{"mean"}, \code{"median"}, \code{"mode"},
\code{"count"}, and \code{"categorical"}. Use \code{"categorical"} for
numeric ordinal variables that should be shown as n (\%).}

\item{value}{optional named list for single-row binaries (e.g., list(sex="Female"));
formula entries like list(sex ~ "Female") are also accepted}

\item{format}{"flextable" (default) or "gt"}

\item{theme}{preset or primitives}
}
\value{
A list with class \code{c("gtregression", "descriptive_table", ...)}
  containing:
  \describe{
    \item{\code{table}}{A \code{gt_tbl} or \code{flextable}.}
    \item{\code{table_display}}{Display-ready data.}
    \item{\code{table_body}}{Long audit data with variable, level, and type.}
    \item{\code{variable_labels}}{Named character vector of display labels
    used for variables.}
    \item{metadata}{Additional metadata fields.}
  }
}
\description{
Publication-ready summary of categorical and continuous variables
(optionally stratified). Mimics the OG gtsummary style:
* column headers include N, e.g. "Overall, N=200"
* categorical rows shown as n (\%)
* continuous rows default to Median (IQR) (footnote reflects summary)
}
\details{
If variables have a \code{"label"} attribute, for example from
\code{labelled::var_label()}, those labels are used automatically in the
displayed table. Internal matching still uses the original column names.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save_functions.R
\name{save_plot}
\alias{save_plot}
\title{Save a single plot}
\usage{
save_plot(
  plot,
  filename = "plot",
  format = c("png", "pdf", "jpg"),
  width = 8,
  height = 6,
  dpi = 300
)
}
\arguments{
\item{plot}{A \code{ggplot2} object.}

\item{filename}{Name of the file to save, with or without extension. If no
directory is supplied, the file is saved in \code{tempdir()}.}

\item{format}{Output format. One of \code{"png"}, \code{"pdf"}, or \code{"jpg"}.}

\item{width}{Width of the saved plot in inches.}

\item{height}{Height of the saved plot in inches.}

\item{dpi}{Resolution of the plot in dots per inch.}
}
\value{
Saves the file to disk. Invisibly returns the normalized file path.
}
\description{
Saves a \code{ggplot2} plot to a file in PNG, PDF, or JPG format.
}
\examples{
p <- ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
  ggplot2::geom_point()

save_plot(p, filename = tempfile("plot"), format = "png")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mediation_analysis.R
\name{mediation_analysis}
\alias{mediation_analysis}
\title{Causal mediation analysis}
\usage{
mediation_analysis(
  data,
  exposure,
  mediator,
  outcome,
  covariates = NULL,
  mediator_approach = "linear",
  outcome_approach = "linear",
  exposure_value = NULL,
  reference_value = NULL,
  sims = 1000,
  conf_level = 0.95,
  seed = NULL,
  format = c("flextable", "gt"),
  theme = c("minimal")
)
}
\arguments{
\item{data}{A data frame.}

\item{exposure}{Exposure/treatment variable. Quoted and bare names are
accepted.}

\item{mediator}{Mediator variable. Quoted and bare names are accepted.}

\item{outcome}{Outcome variable. Quoted and bare names are accepted.}

\item{covariates}{Optional character vector of covariate names. Quoted names
are recommended in scripts, and bare names are also accepted.}

\item{mediator_approach}{Mediator model. Currently \code{"linear"}.}

\item{outcome_approach}{Outcome model. One of \code{"linear"} or
\code{"logit"}. Logistic mediation effects are reported on the predicted
probability-difference scale.}

\item{exposure_value}{Exposure value used as the treatment level. If
\code{NULL}, the second factor level, value 1 for 0/1 variables, or the
75th percentile for continuous exposures is used.}

\item{reference_value}{Exposure value used as the reference level. If
\code{NULL}, the first factor level, value 0 for 0/1 variables, or the
25th percentile for continuous exposures is used.}

\item{sims}{Number of non-parametric bootstrap replicates used for confidence
intervals.}

\item{conf_level}{Confidence level for intervals.}

\item{seed}{Optional random seed for reproducible bootstrap intervals.}

\item{format}{One of \code{"flextable"} (default) or \code{"gt"}.}

\item{theme}{Table theme.}
}
\value{
A list of class \code{c("gtregression", "mediation_analysis", ...)}
with elements:
\describe{
  \item{table}{A formatted \code{flextable} or \code{gt_tbl}.}
  \item{table_body}{Data frame of mediation effect estimates.}
  \item{table_display}{Formatted data frame used to build the table.}
  \item{models}{Fitted mediator and outcome models.}
  \item{boot}{Bootstrap replicate estimates.}
  \item{values}{Reference and exposure values used.}
  \item{variable_labels}{Named character vector of display labels.}
  \item{call}{Matched function call.}
}
}
\description{
Estimate direct, indirect, total, and proportion mediated effects from
regression models.
}
\details{
This function is for planned mediation questions, not automatic causal
discovery. Causal interpretation requires the usual mediation assumptions,
including no unmeasured exposure-outcome, exposure-mediator, or
mediator-outcome confounding, correct temporal order, and suitable model
specification. Use a directed acyclic graph (DAG) and subject-matter
knowledge before interpreting the estimates causally.

For \code{outcome_approach = "linear"}, effects are mean differences. For
\code{outcome_approach = "logit"}, effects are predicted probability
differences, not odds ratios.
}
\examples{
med <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = obesity,
  mediator = glucose,
  outcome = diabetes,
  covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
  outcome_approach = logit,
  sims = 50,
  seed = 123
)

med$table
med$table_body

# HTML-first output
mediation_analysis(
  data = data_diabetes_mediation,
  exposure = obesity,
  mediator = glucose,
  outcome = diabetes,
  covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
  outcome_approach = logit,
  format = gt,
  sims = 50,
  seed = 123
)$table

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{$.gtregression}
\alias{$.gtregression}
\title{Access fields on gtregression objects with `$`}
\usage{
\method{$}{gtregression}(x, name)
}
\arguments{
\item{x}{A \code{gtregression} object.}

\item{name}{Field name to access.}
}
\description{
Works for any object from this package, since they all carry class
`"gtregression"`. Returns NULL (quietly) if the field is not present.
}
\details{
Common fields:
- table, table_display, table_body
- models, model_summaries, reg_check
- approach, format (or engine), source
- parts, spanners (for merged tables)
- by, levels (for descriptive tables)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/footnotes.R
\name{.footnotes_multi_strata}
\alias{.footnotes_multi_strata}
\title{Compose footnotes for multivariable strata (abbrev + per-stratum N)}
\usage{
.footnotes_multi_strata(approach, fits_by_stratum, stratifier)
}
\description{
Compose footnotes for multivariable strata (abbrev + per-stratum N)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/builders-flex.R
\name{.build_flex_strata_wide_multi}
\alias{.build_flex_strata_wide_multi}
\title{Flextable builder for stratified MULTIVARIABLE wide tables (2 cols/stratum)}
\usage{
.build_flex_strata_wide_multi(df, spanners, effect_label_adj, theme, footnotes)
}
\description{
Flextable builder for stratified MULTIVARIABLE wide tables (2 cols/stratum)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dissect.R
\name{dissect}
\alias{dissect}
\title{Dissect a dataset before regression}
\usage{
dissect(data, verbose = FALSE, format = c("flextable", "gt", "tibble"))
}
\arguments{
\item{data}{A data frame.}

\item{verbose}{Logical; if \code{TRUE}, print the summary and interpretation
notes. The tibble is returned invisibly only when printed by the console.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}. Use \code{format = "tibble"} for
pipeline-friendly raw output.}
}
\value{
A tibble, \code{gt_tbl}, or \code{flextable}, depending on
  \code{format}. The tibble has columns: Variable, Type, Missing (%), Unique,
  Levels, Compatibility, and Hint.
}
\description{
Returns a tidy summary of each variable's structure, missingness, uniqueness,
and suitability for use in regression models.
}
\examples{
dissect(data_birthwt, format = "tibble")
dissect(data_birthwt, format = "gt")

# Print notes that help interpret the data structure
dissect(data_birthwt, verbose = TRUE, format = "tibble")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interaction_models.R
\name{interaction_models}
\alias{interaction_models}
\title{Compare Models With and Without an Interaction Term}
\usage{
interaction_models(
  data,
  outcome = NULL,
  exposure,
  covariates = NULL,
  effect_modifier,
  approach = "logit",
  time = NULL,
  event = NULL,
  distribution = "weibull",
  test = c("LRT", "Wald"),
  alpha = 0.05,
  verbose = FALSE,
  format = c("flextable", "gt", "tibble")
)
}
\arguments{
\item{data}{A data frame containing all required variables.}

\item{outcome}{Outcome variable name. Quoted and bare names are accepted.
Required for ordinary regression approaches. Leave unset for Cox and
parametric survival approaches and supply \code{time} and \code{event}
instead.}

\item{exposure}{Main exposure variable name. Quoted and bare names are
accepted.}

\item{covariates}{Optional character vector of adjustment covariates. They
are included in both the model without and the model with the interaction
term. Quoted names are recommended in scripts, and bare names are also
accepted.}

\item{effect_modifier}{Variable name for the potential effect modifier.
Quoted and bare names are accepted.}

\item{approach}{Regression approach. One of \code{"logit"},
\code{"logbinomial"}, \code{"poisson"}, \code{"robpoisson"},
\code{"negbin"}, \code{"linear"}, \code{"cox"}, or
\code{"survreg"}.}

\item{time}{Survival time variable name for \code{approach = "cox"} or
\code{approach = "survreg"}. Quoted and bare names are accepted.}

\item{event}{Event indicator variable name for survival approaches.}

\item{distribution}{Parametric survival distribution for
\code{approach = "survreg"}. One of \code{"weibull"},
\code{"exponential"}, \code{"lognormal"}, or \code{"loglogistic"}.}

\item{test}{Statistical test for model comparison. One of \code{"LRT"} or
\code{"Wald"}.}

\item{alpha}{Significance threshold used to classify the interaction result.}

\item{verbose}{Logical; if \code{TRUE}, prints a short interpretation.}

\item{format}{Output format for the viewing table. One of
\code{"flextable"} (default), \code{"gt"}, or \code{"tibble"}. Use
\code{format = "tibble"} to keep only the original list structure.}
}
\value{
A list with model objects, formulas, p-value, decision, and a
  one-row summary tibble. When \code{format} is \code{"gt"} or
  \code{"flextable"}, the list also includes \code{table}.
}
\description{
Fits two models, one with and one without an interaction term between an
exposure and a potential effect modifier. The models are compared using a
likelihood ratio test or Wald test to assess statistical evidence of
interaction.
}
\details{
Use this function when the interaction is planned or clinically/causally
motivated and you want a focused model comparison. Mantel-Haenszel estimation
is not used here because this function tests an explicit interaction term in
a regression model. For broader screening of candidate confounders or effect
modifiers, including Mantel-Haenszel-supported checks when appropriate, use
\code{\link{identify_confounder}()}.

With \code{covariates = c(age, sex)}, the two fitted models are
\code{outcome ~ exposure + effect_modifier + age + sex} and
\code{outcome ~ exposure + effect_modifier + age + sex +
exposure:effect_modifier}. Both models use the same complete-case analysis
data, so covariate adjustment is applied consistently to the interaction
comparison.
}
\examples{
birthwt_data <- data_birthwt |>
  dplyr::mutate(
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other"))
  )

interaction_models(
  data = birthwt_data,
  outcome = low,
  exposure = smoke,
  effect_modifier = race,
  covariates = c(age, lwt),
  approach = logit
)

lung_data <- data_lungcancer |>
  dplyr::mutate(
    trt = factor(trt, levels = c(1, 2),
                 labels = c("Standard treatment", "Test treatment")),
    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
  )

interaction_models(
  data = lung_data,
  time = time,
  event = status,
  exposure = trt,
  effect_modifier = prior,
  covariates = c(age, karno),
  approach = cox
)

}
\seealso{
\code{\link{identify_confounder}()} for broader confounding and
  effect-modification screening.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_quantiles.R
\name{survival_quantiles}
\alias{survival_quantiles}
\title{Kaplan-Meier survival quantile table}
\usage{
survival_quantiles(
  data,
  time,
  event,
  by = NULL,
  probs = c(0.25, 0.5, 0.75),
  digits = 1,
  format = c("flextable", "gt", "tibble"),
  theme = c("minimal")
)
}
\arguments{
\item{data}{A \code{data.frame} containing survival time and event status.}

\item{time}{Survival follow-up time. Quoted and bare names are accepted.}

\item{event}{Event indicator. Quoted and bare names are accepted. Numeric
\code{0/1}, numeric \code{1/2}, logical, character, and factor variables
are accepted. For two-level character or factor variables, the second level
is treated as the event.}

\item{by}{Optional grouping variable for separate Kaplan-Meier quantiles.
Quoted and bare names are accepted.}

\item{probs}{Numeric vector of event-time quantiles to estimate. The default
\code{c(0.25, 0.5, 0.75)} reports the 25th percentile, median, and 75th
percentile event times.}

\item{digits}{Number of digits for survival time summaries.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}.}

\item{theme}{Table styling preset.}
}
\value{
A list of class \code{c("gtregression","survival_quantiles", ...)}
with elements:
\describe{
  \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
  when \code{format = "tibble"}.}
  \item{\code{table_body}}{Tibble with Kaplan-Meier quantiles.}
  \item{\code{table_display}}{Display data frame used to render the table.}
  \item{\code{fit}}{Fitted \code{survfit} object.}
  \item{\code{time,event,by,probs,format,source}}{Metadata fields.}
}
}
\description{
Estimate Kaplan-Meier survival time quantiles, such as the 25th percentile,
median, and 75th percentile survival times.
}
\details{
A probability of \code{0.50} is the median event time: the estimated time by
which 50% of participants have had the event, corresponding to 50% survival.
A probability of \code{0.25} is the time by which 25% have had the event,
corresponding to 75% survival. If a quantile is not reached during observed
follow-up, the display table shows \code{"Not reached"}.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))

survival_quantiles(
  data = lung_data,
  time = time,
  event = status,
  by = trt
)

survival_quantiles(
  data = lung_data,
  time = "time",
  event = "status",
  probs = c(0.25, 0.5),
  format = tibble
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reg_check_print.R
\name{print.gtregression_reg_check}
\alias{print.gtregression_reg_check}
\title{Print linear-regression diagnostics as a publication-ready table}
\usage{
\method{print}{gtregression_reg_check}(x, ...)
}
\arguments{
\item{x}{A diagnostic list stored in a gtregression regression result.}

\item{...}{Unused.}
}
\value{
The diagnostic object, invisibly.
}
\description{
Print linear-regression diagnostics as a publication-ready table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_prob.R
\name{survival_prob}
\alias{survival_prob}
\title{Kaplan-Meier survival probability table}
\usage{
survival_prob(
  data,
  time,
  event,
  by = NULL,
  times,
  digits = 1,
  extend = TRUE,
  format = c("flextable", "gt", "tibble"),
  theme = c("minimal")
)
}
\arguments{
\item{data}{A \code{data.frame} containing survival time and event status.}

\item{time}{Survival follow-up time. Quoted and bare names are accepted.}

\item{event}{Event indicator. Quoted and bare names are accepted. Numeric
\code{0/1}, numeric \code{1/2}, logical, character, and factor variables
are accepted. For two-level character or factor variables, the second level
is treated as the event.}

\item{by}{Optional grouping variable for separate Kaplan-Meier survival
probabilities. Quoted and bare names are accepted.}

\item{times}{Numeric vector of follow-up times at which survival probability
should be estimated.}

\item{digits}{Number of digits for percentages and survival probabilities.}

\item{extend}{Logical. If \code{TRUE}, requested times beyond the observed
follow-up range are retained using the last available Kaplan-Meier estimate.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}.}

\item{theme}{Table styling preset.}
}
\value{
A list of class \code{c("gtregression","survival_prob", ...)}
with elements:
\describe{
  \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
  when \code{format = "tibble"}.}
  \item{\code{table_body}}{Tibble with Kaplan-Meier survival probabilities.}
  \item{\code{table_display}}{Display data frame used to render the table.}
  \item{\code{fit}}{Fitted \code{survfit} object.}
  \item{\code{time,event,by,times,format,source}}{Metadata fields.}
}
}
\description{
Estimate Kaplan-Meier survival probabilities at user-specified follow-up
times, such as 30-day, 6-month, or 1-year survival.
}
\details{
Survival probabilities are estimated from \code{survival::survfit()} at the
requested follow-up times. Events and censored counts are interval counts up
to each requested time point as returned by \code{summary.survfit()}.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))

survival_prob(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  times = c(90, 180, 365)
)

survival_prob(
  data = lung_data,
  time = "time",
  event = "status",
  times = c(90, 180),
  format = tibble
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/builders-flex.R
\name{.build_flex_strata_wide_uni}
\alias{.build_flex_strata_wide_uni}
\title{Flextable builder for stratified wide tables (3 cols per stratum)}
\usage{
.build_flex_strata_wide_uni(df, spanners, effect_label, theme, footnotes)
}
\description{
Flextable builder for stratified wide tables (3 cols per stratum)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{data_infertility}
\alias{data_infertility}
\title{Infertility Matched Case-Control Study}
\format{
A data frame with 248 observations and 8 variables:
\describe{
  \item{education}{Education level (0 = 0–5 years, 1 = 6–11 years,
  2 = 12+ years)}
  \item{age}{Age in years}
  \item{parity}{Number of prior pregnancies}
  \item{induced}{Number of induced abortions}
  \item{case}{Infertility case status (1 = case, 0 = control)}
  \item{spontaneous}{Number of spontaneous abortions}
  \item{stratum}{Matched set ID}
  \item{pooled.stratum}{Pooled stratum ID used for conditional regression}
}
}
\source{
\url{https://stat.ethz.ch/R-manual/R-devel/library/datasets/html/infert.html}
}
\usage{
data_infertility
}
\description{
investigating the relationship between infertility and abortions.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/builders-gt.R
\name{.build_gt_strata_wide_uni}
\alias{.build_gt_strata_wide_uni}
\title{GT builder for stratified wide tables (3 cols per stratum)}
\usage{
.build_gt_strata_wide_uni(df, spanners, effect_label, theme, footnotes)
}
\description{
GT builder for stratified wide tables (3 cols per stratum)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_ph.R
\name{check_ph}
\alias{check_ph}
\title{Check proportional hazards assumption for Cox models}
\usage{
check_ph(
  model,
  transform = c("km", "rank", "identity"),
  alpha = 0.05,
  format = c("flextable", "gt", "tibble")
)
}
\arguments{
\item{model}{A fitted \code{coxph} model or a \code{cox_reg()} object.}

\item{transform}{Time transformation passed to \code{survival::cox.zph()}.
One of \code{"km"}, \code{"rank"}, or \code{"identity"}.}

\item{alpha}{Significance level used for the simple interpretation column.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}.}
}
\value{
A tibble, \code{gt_tbl}, or \code{flextable} with Schoenfeld
residual proportional hazards tests. The table contains:
\describe{
  \item{\code{Model}}{Model name. For a direct \code{coxph} object, this is
  \code{"cox_model"}.}
  \item{\code{Term}}{Model term or \code{"GLOBAL"}.}
  \item{\code{Chi.square}}{Chi-square statistic.}
  \item{\code{df}}{Degrees of freedom.}
  \item{\code{p.value}}{Test p-value.}
  \item{\code{Interpretation}}{Simple screening interpretation using
  \code{alpha}.}
}
}
\description{
Test the proportional hazards assumption for fitted Cox proportional hazards
models using Schoenfeld residuals.
}
\details{
\code{check_ph()} is a diagnostic aid for Cox models. A small p-value
suggests possible evidence against the proportional hazards assumption for a
term or for the global model test. This should be interpreted with plots,
clinical knowledge, follow-up patterns, and modelling purpose.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))

cox_fit <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, age)
)

check_ph(cox_fit)
check_ph(cox_fit, format = tibble)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_model_fit.R
\name{plot_model_fit}
\alias{plot_model_fit}
\title{Plot model-fit diagnostics}
\usage{
plot_model_fit(
  model,
  model_name = NULL,
  type = c("auto", "all", "residual", "qq", "scale_location", "cooks",
    "observed_predicted", "calibration"),
  bins = 10,
  base_size = 13
)
}
\arguments{
\item{model}{A fitted \code{lm} or \code{glm} model, or a
\code{uni_reg()} / \code{multi_reg()} result.}

\item{model_name}{Optional model name to select when \code{model} contains
multiple fitted models. Quoted and bare names are accepted.}

\item{type}{Plot type. One of \code{"auto"}, \code{"all"},
\code{"residual"}, \code{"qq"}, \code{"scale_location"},
\code{"cooks"}, \code{"observed_predicted"}, or \code{"calibration"}.
Quoted and bare values are accepted.}

\item{bins}{Number of groups used for binomial calibration plots.}

\item{base_size}{Base font size for the plot theme.}
}
\value{
A \code{ggplot2} object for a single plot, or a patchwork object when
  multiple diagnostics are requested.
}
\description{
Visualise model fit for fitted regression models and models stored inside
\code{uni_reg()} or \code{multi_reg()} results.
}
\details{
\code{plot_model_fit()} is a visual check of how a fitted model behaves, not
a formal model-selection rule. For survival models, use \code{check_ph()} for Cox
proportional hazards diagnostics and \code{plot_surv_fit()} for parametric
survival model fit.

For binomial models, \code{type = "calibration"} compares grouped predicted
probabilities with observed event proportions. This is most informative for
multivariable models, where predictions vary across many patients. A
univariable binary predictor may produce only two calibration points; that is
expected and simply reflects the two fitted probabilities in the model.
Logistic residual plots often show two bands because the outcome is coded as
event/non-event.
}
\examples{
fit_lm <- lm(mpg ~ wt + hp, data = mtcars)
plot_model_fit(fit_lm)

fit_glm <- glm(am ~ mpg + wt, data = mtcars, family = binomial())
plot_model_fit(fit_glm, type = calibration, bins = 4)

uni_fit <- uni_reg(mtcars, am, c(mpg, wt), approach = logit)
plot_model_fit(uni_fit, model_name = mpg, type = residual)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_df.R
\name{forest_df}
\alias{forest_df}
\title{Build a compatible data frame for forest plots}
\usage{
forest_df(uni, multi = NULL, desc = NULL, digits = 2)
}
\arguments{
\item{uni}{A \code{gtregression} object from \code{uni_reg()}. If
\code{multi} is supplied without \code{uni}, the multivariable object is
plotted as a single effect column.}

\item{multi}{Optional \code{gtregression} object from \code{multi_reg()}.}

\item{desc}{Optional descriptive table object from \code{descriptive_table()}.}

\item{digits}{Number of digits used when formatting confidence intervals.}
}
\value{
A data frame with display columns, formatted effect-size columns,
  standard-error columns, and plotting attributes used by \code{forest_reg()}
  to draw confidence intervals.
}
\description{
Creates the tabular input used by \code{forest_reg()} from \code{uni_reg()},
\code{multi_reg()}, and optionally \code{descriptive_table()} objects. This
function is useful when users want to inspect, edit, or reuse the exact data
that will be passed to the forest plot. Stratified regression objects are
also supported when supplied one at a time; the returned data frame keeps the
characteristic rows once and places strata side by side as separate effect
columns for \code{forest_reg()}.
}
\examples{
birthwt_data <- data_birthwt |>
  dplyr::mutate(
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
  )

uni_or <- uni_reg(
  birthwt_data,
  outcome = "low",
  exposures = c("age", "lwt", "smoke", "ht"),
  approach = "logit"
)
multi_or <- multi_reg(
  birthwt_data,
  outcome = "low",
  exposures = c("smoke", "ht"),
  adjust_for = c("age", "lwt"),
  approach = "logit"
)

forest_data <- forest_df(uni_or, multi_or)
head(forest_data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save_functions.R
\name{save_forest}
\alias{save_forest}
\title{Save a forest_reg() output}
\usage{
save_forest(
  forest,
  filename = "forest",
  format = c("pdf", "png", "tiff", "jpg"),
  width = NULL,
  height = NULL,
  scale = 1,
  auto_size = TRUE,
  padding = 0.25,
  dpi = 300
)
}
\arguments{
\item{forest}{A \code{gtregression_forest} object returned by
\code{forest_reg()}, or a compatible \pkg{forestploter}/grid object.}

\item{filename}{File name for the output, with or without extension. If no
directory is supplied, the file is saved in \code{tempdir()}.}

\item{format}{Output format. One of \code{"pdf"}, \code{"png"},
\code{"tiff"}, or \code{"jpg"}.}

\item{width, height}{Optional export width and height in inches. If either is
\code{NULL} and \code{auto_size = TRUE}, the size is measured from the
intrinsic \pkg{forestploter} gtable dimensions. If measurement is not
possible, a practical fallback is estimated from the number of rows and
columns in the \code{forest_reg()} output.}

\item{scale}{Positive multiplier applied to the export width and height. This
is a quick way to make a large forest plot roomier.}

\item{auto_size}{Logical. If \code{TRUE}, measure the intrinsic forest plot
size when \code{width} or \code{height} is not supplied. If \code{FALSE},
use the fallback row/column-based sizing.}

\item{padding}{White space around the forest plot in inches.}

\item{dpi}{Resolution for raster formats.}
}
\value{
Saves the file to disk. Invisibly returns the normalized file path.
}
\description{
Saves a \code{forest_reg()} output, or a compatible
\pkg{forestploter}/grid object, to a fixed graphics device. This is useful
when the RStudio Viewer or operating-system graphics device crops wide forest
plots or compresses forest columns.
}
\examples{
birthwt_data <- data_birthwt |>
  transform(
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
  )

uni_or <- uni_reg(
  birthwt_data,
  outcome = "low",
  exposures = c("age", "smoke", "ht"),
  approach = "logit"
)

forest <- forest_reg(uni = uni_or)
save_forest(forest, filename = tempfile("forest"), format = "pdf")

# For large forest plots, increase width, height, scale, or padding.
save_forest(
  forest,
  filename = tempfile("forest-wide"),
  format = "png",
  scale = 1.2,
  padding = 0.35,
  dpi = 300
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{data_endometrial}
\alias{data_endometrial}
\title{Endometrial Cancer Histology Grade Data}
\format{
A data frame with 79 observations and 4 variables:
\describe{
  \item{NV}{Neovascularization status (0 = absent, 1 = present)}
  \item{PI}{Pulsatility index of the uterine artery}
  \item{EH}{Endometrium height}
  \item{HG}{Histology grade (0 = low grade, 1 = high grade)}
}
}
\source{
\pkg{brglm2} package. The packaged dataset was sourced from
\url{https://users.stat.ufl.edu/~aa/glm/data/}, the data repository used in
Agresti (2015). Originally analyzed in Heinze and Schemper (2002).
}
\usage{
data_endometrial
}
\description{
A classic endometrial cancer dataset used to demonstrate separation in
logistic regression. The outcome is high histology grade. Neovascularization
is completely absent among low-grade cases in this dataset, making it useful
for teaching Firth penalized logistic regression.
}
\references{
Agresti A (2015). \emph{Foundations of Linear and Generalized Linear Models}.
Wiley.

Heinze G, Schemper M (2002). A solution to the problem of separation in
logistic regression. \emph{Statistics in Medicine}, 21, 2409-2419.
\doi{10.1002/sim.1047}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_models.R
\name{select_models}
\alias{select_models}
\title{Stepwise model selection with fit metrics}
\usage{
select_models(
  data,
  outcome,
  exposures,
  approach = "logit",
  time = NULL,
  event = NULL,
  distribution = "weibull",
  direction = "forward",
  format = c("flextable", "gt", "tibble")
)
}
\arguments{
\item{data}{A data frame containing the outcome and predictor variables.}

\item{outcome}{A single character string indicating the outcome variable.
Quoted and bare names are accepted. Not used for survival approaches when
\code{time} and \code{event} are supplied.}

\item{exposures}{Character vector of predictor variables to consider. Quoted
names are recommended in scripts, and bare names are also accepted.}

\item{approach}{Regression method. One of:
\code{"logit"}, \code{"logbinomial"}, \code{"poisson"},
\code{"robpoisson"}, \code{"negbin"}, \code{"linear"}, \code{"cox"}, or
\code{"survreg"}.}

\item{time, event}{Survival time and event indicator for \code{approach = "cox"}
or \code{approach = "survreg"}. Quoted and bare names are accepted.}

\item{distribution}{Parametric survival distribution for
\code{approach = "survreg"}. One of \code{"weibull"},
\code{"exponential"}, \code{"lognormal"}, or \code{"loglogistic"}.
Common spellings such as \code{"log-normal"} and \code{"log-logistic"} are
also accepted.}

\item{direction}{Stepwise selection direction. One of:
\code{"forward"} (default), \code{"backward"}, or \code{"both"}.}

\item{format}{Output format for the viewing table. One of
\code{"flextable"} (default), \code{"gt"}, or \code{"tibble"}. Use
\code{format = "tibble"} to keep only the original list structure.}
}
\value{
A list with the following components:
\itemize{
  \item \code{results_table}: A tibble summarising each accepted step's
  model metrics (AIC, BIC, deviance, log-likelihood, and adjusted R-squared
  for linear models).
  \item \code{best_model}: The best-fitting model object based on lowest AIC.
  \item \code{all_models}: A named list of the accepted stepwise models.
  \item \code{direction}: Stepwise selection direction used.
  \item \code{table}: A formatted \code{gt_tbl} or \code{flextable} when
  \code{format} is \code{"gt"} or \code{"flextable"}.
}
}
\description{
Compare candidate models added or removed stepwise, then return the accepted
steps, model-fit metrics, and the lowest-AIC model.
}
\details{
Treat stepwise selection as a screening tool. It is best used alongside the
study question, clinical judgement, and model diagnostics rather than as an
automatic final-model rule.
}
\examples{
data <- data_birthwt
stepwise <- select_models(
  data = data,
  outcome = "bwt",
  exposures = c("age", "lwt", "smoke"),
  approach = "linear",
  direction = "forward"
)
stepwise$results_table
stepwise$best_model

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_reg.R
\name{surv_reg}
\alias{surv_reg}
\title{Parametric survival regression}
\usage{
surv_reg(
  data,
  time,
  event,
  exposures,
  adjust_for = NULL,
  stratifier = NULL,
  interaction = NULL,
  multivariable = FALSE,
  multivariate = NULL,
  distribution = "weibull",
  format = c("flextable", "gt"),
  theme = c("minimal"),
  show_sample = "events",
  model_stats = FALSE,
  show_ref = TRUE
)
}
\arguments{
\item{data}{A \code{data.frame} containing survival time, event status, and
exposure variables.}

\item{time}{Survival follow-up time. Quoted and bare names are accepted.}

\item{event}{Event indicator. Quoted and bare names are accepted. Numeric
\code{0/1}, numeric \code{1/2}, logical, character, and factor variables
are accepted. For two-level character or factor variables, the second level
is treated as the event.}

\item{exposures}{Character vector of exposure variable names. Quoted names
are recommended in scripts, and bare names are also accepted.}

\item{adjust_for}{Optional character vector of adjustment variables. When
supplied, one adjusted model is fitted per exposure.}

\item{stratifier}{Optional single stratifying variable. When supplied,
stratum-specific parametric survival tables are produced using the same
crude, adjusted, or multivariable workflow requested by the other
arguments. The stratifier cannot also be used as the time, event,
exposure, adjustment, or interaction variable.}

\item{interaction}{Optional character scalar specifying one interaction term
using standard formula syntax, e.g. \code{"trt*prior"}. Quoted and bare
interaction syntax are accepted. In exposure-by-exposure mode, supply a
single exposure; in \code{multivariable = TRUE} mode, the interaction is
added to the single multivariable model.}

\item{multivariable}{Logical; if \code{FALSE} (default), the current
exposure-by-exposure workflow is used. If \code{TRUE}, one multivariable
parametric survival model is fitted using all variables in
\code{exposures}, and all exposure coefficients are reported.}

\item{multivariate}{Optional logical alias for \code{multivariable}. This is
accepted for convenience; \code{multivariable} is used internally.}

\item{distribution}{Parametric survival distribution. One of
\code{"weibull"}, \code{"exponential"}, \code{"lognormal"}, or
\code{"loglogistic"}. Quoted and bare values are accepted. Common
spellings such as \code{"log-normal"} and \code{"log-logistic"} are also
accepted.}

\item{format}{Output table format; one of \code{"flextable"} (default) or
\code{"gt"}.}

\item{theme}{Table styling preset.}

\item{show_sample}{For stratified parametric survival tables, controls which
sample-size columns are shown in the publication table. One of
\code{"events"} (default), \code{"n"}, \code{"both"}, or \code{"none"}.
Model statistics, when requested, still retain both N and event counts.}

\item{model_stats}{Logical; if \code{TRUE}, extract model-fit statistics
including AIC, BIC, log-likelihood, scale, number of events, and N.}

\item{show_ref}{Logical; if \code{TRUE} (default), display reference-category
rows as \code{"Ref."}. If \code{FALSE}, hide reference rows; a message
reminds users to use \code{show_ref = TRUE} when reference rows are needed.}
}
\value{
A list of class \code{c("gtregression","surv_reg", ...)} with elements:
\describe{
  \item{table}{A \code{flextable} or \code{gt_tbl}.}
  \item{table_body}{Data frame of time ratios and confidence intervals.}
  \item{table_display}{Data frame used to render the publication table.}
  \item{models}{List of fitted \code{survreg} models.}
  \item{model_summaries}{Summary output for the fitted models.}
  \item{model_stats}{Model-fit statistics when \code{model_stats = TRUE};
  otherwise \code{NULL}.}
  \item{variable_labels}{Named character vector of display labels.}
  \item{footnotes}{Character vector used by the rendered table. Adjustment
  notes use the same display labels as the table.}
  \item{time,event,distribution,approach,format,source,adjust_for,exposures,interaction}{Metadata fields.}
}
}
\description{
Fit parametric survival models and report time ratios.
}
\details{
\code{surv_reg()} fits accelerated failure time style parametric survival
models using \code{survival::survreg()}. The exponentiated coefficient is
displayed as a time ratio. A time ratio above 1 suggests longer survival time;
a time ratio below 1 suggests shorter survival time, conditional on the chosen
distribution.

By default, \code{surv_reg()} keeps the exposure-by-exposure workflow:
without \code{adjust_for}, one crude model is fitted per exposure; with
\code{adjust_for}, one adjusted model is fitted per exposure and only the
exposure estimate is reported.

With \code{multivariable = TRUE}, all variables in \code{exposures} are
included in one parametric survival model and all coefficients are reported.
Since these estimates are adjusted for the other variables in the same model,
the table reports \code{Adjusted Time Ratio (95\% CI)}. The
\code{adjust_for} argument is not used in this mode; include every variable
that belongs in the model inside \code{exposures}.

Interaction terms specified via \code{interaction} are included using
standard formula expansion (for example, \code{trt*prior}). Interaction
effects are displayed as additional rows beneath the corresponding exposure.

Stratified parametric survival tables show event counts by default. Use
\code{show_sample = "n"}, \code{show_sample = "both"}, or
\code{show_sample = "none"} to control the displayed sample columns. Crude
stratified tables calculate these counts for each exposure-specific model;
adjusted and multivariable stratified tables use the corresponding fitted
model within each stratum.

If exposure variables have a \code{"label"} attribute, for example from
\code{labelled::var_label()}, those labels are used automatically in the
displayed table.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))
lung_data$prior <- factor(lung_data$prior, levels = c(0, 10),
                          labels = c("No", "Yes"))

surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c("trt", "celltype", "karno", "age")
)

surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior),
  adjust_for = c(age, karno),
  distribution = lognormal
)

# Interaction in an adjusted exposure model
surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno),
  interaction = trt*prior,
  distribution = weibull
)

surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, celltype, prior, age, karno),
  distribution = weibull,
  multivariable = TRUE
)

# multivariate is accepted as an alias
surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = c(trt, age, karno),
  multivariate = TRUE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_models.R
\name{compare_models}
\alias{compare_models}
\title{Compare Prespecified Regression Models}
\usage{
compare_models(
  ...,
  model_names = NULL,
  nested = TRUE,
  primary_exposure = NULL,
  exponentiate = NULL,
  digits = 2,
  p_digits = 3,
  format = c("flextable", "gt", "tibble"),
  theme = c("minimal")
)
}
\arguments{
\item{...}{Two or more gtregression model objects, or one list containing
them. Inputs should be outputs from \code{multi_reg()}, \code{cox_reg()},
or \code{surv_reg()}.}

\item{model_names}{Optional character vector of names to display. If omitted,
names supplied in \code{...} are used; otherwise models are labelled
\code{Model 1}, \code{Model 2}, etc.}

\item{nested}{Logical. If \code{TRUE}, likelihood-ratio statistics are
calculated sequentially by comparing each model with the previous model.
Use this only when models are nested and supplied in the intended order.}

\item{primary_exposure}{Optional exposure or exact coefficient name to track
across models. For Cox models this can be used to show the hazard ratio
and percentage change in the log-effect estimate across candidate models.}

\item{exponentiate}{Logical. If \code{NULL}, Cox, logistic, Poisson,
negative-binomial, and parametric survival models are exponentiated by
default, while linear models are not.}

\item{digits}{Number of digits for model statistics and estimates.}

\item{p_digits}{Number of digits for p-values.}

\item{format}{Output format. Defaults to \code{"flextable"}.}

\item{theme}{Table theme preset.}
}
\value{
A \code{gtregression} object with:
\itemize{
  \item \code{table}: publication-ready table
  \item \code{table_body}: raw comparison statistics
  \item \code{table_display}: formatted display data
  \item \code{models}: fitted models compared
  \item \code{comparison_status}: whether models appear to use the same
    analysis sample
  \item \code{comparison_warnings}: caution messages that are highlighted
    in rendered tables when interpretation needs extra care
}
}
\description{
Compare gtregression candidate models side by side using model-fit statistics.
This is intended for transparent model comparison after you have already
fitted the candidate models with functions such as \code{multi_reg()},
\code{cox_reg()}, or \code{surv_reg()}.
}
\details{
\code{compare_models()} does not refit models and does not perform hidden
complete-case filtering. It compares models already fitted by gtregression
and extracts the single fitted model stored in each object's \code{models}
element. The reported N, event counts, and fit statistics therefore come from
the model already fitted by \code{multi_reg()}, \code{cox_reg()}, or
\code{surv_reg()}. This keeps model comparison separate from model
selection: compare candidate models first, then choose the final model using
clinical, epidemiological, and statistical judgement.
The guided builder in \code{gtregression_app()} can create the fitted
gtregression objects before calling this function; the generated app code
exposes every model formula and comparison call.

Likelihood-ratio p-values are meaningful only for nested models fitted to
the same analysis sample. \code{compare_models()} checks whether the fitted
models appear to use the same analysis sample using retained model row
identifiers when available; otherwise it compares N and event counts. It
also checks whether sequential model pairs appear to be nested when
\code{nested = TRUE}. Rendered warnings are context-aware: no warning about
different analysis samples is shown when the compared models use the same
observations, and no nested-model warning is shown when sequential models
appear nested. AIC, BIC, log-likelihood, and likelihood-ratio statistics
remain visible when warnings are needed, but should then be interpreted with
the displayed caution.
}
\examples{
data("data_lungcancer", package = "gtregression")

lung_data <- data_lungcancer

cox_1 <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt
)

cox_2 <- cox_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = trt,
  adjust_for = c(age, karno)
)

compare_models(
  cox_1,
  cox_2,
  primary_exposure = trt
)

compare_models(
  cox_1,
  cox_2,
  model_names = c("Treatment only", "Treatment + age + performance"),
  primary_exposure = trt
)

}
\seealso{
\code{gtregression_app()}, \code{multi_reg()}, \code{cox_reg()},
  \code{surv_reg()}, \code{select_models()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_reg.R
\name{forest_reg}
\alias{forest_reg}
\title{Draw a forest table from regression outputs}
\usage{
forest_reg(
  df = NULL,
  uni = NULL,
  multi = NULL,
  desc = NULL,
  theme = NULL,
  ci_col_width = 20,
  side = c("right", "left"),
  quiet = TRUE,
  effects = NULL,
  ticks_at = NULL,
  ticks_digits = NULL,
  xlim = NULL,
  style_strata = TRUE,
  strata_fill = "#EAF2F1",
  ...
)
}
\arguments{
\item{df}{Output of \code{forest_df()}. If \code{NULL}, it is built from
\code{uni}, \code{multi}, and \code{desc}.}

\item{uni, multi, desc}{Optional \code{gtregression} objects to pass through to
\code{forest_df()}.}

\item{theme}{Optional \code{forestploter::forest_theme()}. If \code{NULL},
\code{forestploter} defaults are used. You may pass colors and styling
either here or through \code{...}.}

\item{ci_col_width}{Numeric value, or one value per effect column,
controlling the blank spacer width used by \code{forestploter} for the
confidence-interval plot column(s). Values greater than 1 are interpreted
as approximate character counts. Values between 0 and 1 are accepted for
backward compatibility and converted to character counts.}

\item{side}{Character. For each effect, position of the plot relative to the effect-size text:
\code{"left"} = plot first then text; \code{"right"} = text first then plot.
The \code{Characteristic} column and descriptive columns remain on the left.}

\item{quiet}{Logical. Suppress forestploter warnings. Default = `TRUE`.}

\item{effects}{Optional effect labels passed to \code{forestploter::forest()}.}

\item{ticks_at}{Optional numeric vector, or list of numeric vectors for
multiple effect
columns, specifying x-axis tick positions. If \code{NULL},
\code{forestploter::forest()} chooses the default ticks.}

\item{ticks_digits}{Optional number of digits for x-axis tick labels.}

\item{xlim}{Optional numeric vector of length 2, or list of length-2 numeric
vectors for multiple
effect columns, specifying x-axis limits. If \code{NULL},
\code{forestploter::forest()} chooses the default limits.}

\item{style_strata}{Logical. Retained for older vertically grouped
stratified forest data. Current \code{forest_df()} stratified output places
strata side by side, so no stratum header rows are styled.}

\item{strata_fill}{Character. Fill color used for older styled stratum header
rows.}

\item{...}{Passed to \code{forestploter::forest()}. Common options include
\code{title} and \code{footnote}.}
}
\value{
A \code{gtregression_forest} object with elements:
\describe{
  \item{\code{plot}}{The forest plot object.}
  \item{\code{data}}{The plotting data sent to \code{forestploter::forest()}.}
  \item{\code{input_data}}{The original \code{forest_df()} data, including
  standard-error helper columns.}
  \item{\code{meta}}{Model metadata, including reference line and x-axis
  transformation.}
}
}
\description{
Wrapper around \code{forestploter::forest()} that works directly with
\code{forest_df()} output or with \code{gtregression} regression objects.
It can show descriptive columns and one or more model effect columns in a
table-style forest plot while leaving axis and interval drawing to
\pkg{forestploter}.
}
\examples{
birthwt_data <- data_birthwt |>
  transform(
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1),
                 labels = c("Normal BW", "Low BW"))
  )

uni_or <- uni_reg(
  birthwt_data,
  outcome = "low",
  exposures = c("age", "lwt", "smoke", "ht"),
  approach = "logit"
)
multi_or <- multi_reg(
  birthwt_data,
  outcome = "low",
  exposures = c("smoke", "ht"),
  adjust_for = c("age", "lwt"),
  approach = "logit"
)

forest_reg(uni = uni_or, multi = multi_or)

# If axis labels overlap, set x-axis limits and tick marks.
forest_reg(
  uni = uni_or,
  multi = multi_or,
  xlim = list(c(0.25, 8), c(0.25, 12)),
  ticks_at = list(c(0.5, 1, 2, 4, 8), c(0.5, 1, 2, 4, 8))
)

# If the forest plot panel is too narrow or too wide, tune ci_col_width.
forest_reg(
  uni = uni_or,
  multi = multi_or,
  ci_col_width = c(18, 22)
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_reg_combine.R
\name{plot_reg_combine}
\alias{plot_reg_combine}
\title{Side-by-side forest plots: univariate vs multivariable}
\usage{
plot_reg_combine(
  tbl_uni,
  tbl_multi,
  title_uni = NULL,
  title_multi = NULL,
  caption = NULL,
  ref_line = NULL,
  order_y = NULL,
  log_x = FALSE,
  point_color = "#6B7280",
  errorbar_color = "#6B7280",
  point_size = 2.8,
  point_stroke = 0.55,
  ci_linewidth = 0.55,
  base_size = 12,
  show_ref = NULL,
  sig_color = "#0072B2",
  sig_errorbar_color = "#0072B2",
  xlim_uni = NULL,
  breaks_uni = NULL,
  xlim_multi = NULL,
  breaks_multi = NULL,
  alpha = 0.05,
  show_adjustment_note = TRUE
)
}
\arguments{
\item{tbl_uni}{A univariate \code{gtregression} object.}

\item{tbl_multi}{A multivariable \code{gtregression} object.}

\item{title_uni, title_multi}{Optional panel titles.}

\item{caption}{Optional combined plot caption. If \code{NULL}, notes are
added automatically for adjusted \code{multi_reg()} objects and displayed
reference categories.}

\item{ref_line}{Optional numeric reference line. If \code{NULL}, uses 0 for
linear models and 1 otherwise.}

\item{order_y}{Optional character vector to customize exposure ordering.}

\item{log_x}{Logical. If \code{TRUE}, use a log-scaled x-axis for ratio
measures. The default \code{FALSE} uses a linear x-axis.}

\item{point_color, errorbar_color}{Base colors for non-significant rows.}

\item{point_size}{Diameter of estimate points in millimetres.}

\item{point_stroke}{Outline width of estimate points.}

\item{ci_linewidth}{Line width of confidence intervals.}

\item{base_size}{Base font size for the plot theme.}

\item{show_ref}{Logical or \code{NULL}. The default \code{NULL} inherits the
setting used to create the supplied tables. If \code{TRUE}, include
reference levels as \code{(Ref.)}. If \code{FALSE}, binary exposures are
shown as compact rows for the estimated non-reference category;
affirmative levels such as \code{Yes} or \code{1} are displayed using the
variable name.}

\item{sig_color, sig_errorbar_color}{Optional colors for significant rows.
If \code{NULL}, base colors are reused.}

\item{xlim_uni, breaks_uni}{Optional x-axis limits and breaks for the
univariate panel.}

\item{xlim_multi, breaks_multi}{Optional x-axis limits and breaks for the
multivariable panel.}

\item{alpha}{Significance level for linear models when \code{p.value} is
available.}

\item{show_adjustment_note}{Logical; if \code{TRUE}, add a default caption
describing \code{adjust_for} variables from \code{tbl_multi} when available.}
}
\value{
A \code{patchwork} object with two \code{ggplot2} panels.
}
\description{
Creates two aligned forest plots from compatible \code{gtregression}
regression objects such as crude and adjusted outputs from \code{uni_reg()},
\code{multi_reg()}, \code{cox_reg()}, or \code{surv_reg()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_tables.R
\name{merge_tables}
\alias{merge_tables}
\title{Merge gtregression tables and preserve structure and notes}
\usage{
merge_tables(
  ...,
  spanners = NULL,
  theme = "minimal",
  format = c("flextable", "gt")
)
}
\arguments{
\item{...}{Two or more \code{gtregression} objects containing
\code{$table_display}.}

\item{spanners}{Character vector of spanner labels, one per table.
If \code{NULL}, defaults to \code{"Table 1"}, \code{"Table 2"}, etc.}

\item{theme}{Merge theme preset or vector of primitives.}

\item{format}{Output table format. One of \code{"flextable"} (default) or
\code{"gt"}. The merged display is rebuilt in this format, independently
of the formats used by the input tables.}
}
\value{
A merged table object of class
  \code{c("gtregression", "merged_table", ...)}.
}
\description{
Merge gtregression tables and preserve structure and notes
}
\details{
Binary variables should use the same row display across all input tables.
For a descriptive, univariable, and multivariable merge, the clearest
publication layout is usually \code{show_dichotomous = "all_levels"} in
\code{descriptive_table()} and \code{show_ref = TRUE} in each regression
function. Mixing these settings can create additional binary rows; in that
situation \code{merge_tables()} issues a warning with the compatible
settings.

Footnotes created by each input table are retained unchanged. Exact duplicate
notes are shown once in the merged table.
}
\examples{
birthwt_data <- data_birthwt |>
  dplyr::mutate(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
  )

uni_tbl <- uni_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = c("age", "lwt", "smoke", "ht"),
  approach = "logit"
)

multi_tbl <- multi_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = c("smoke", "ht"),
  adjust_for = c("age", "lwt"),
  approach = "logit"
)

merge_tables(
  uni_tbl,
  multi_tbl,
  spanners = c("Univariable", "Adjusted")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_summary.R
\name{survival_summary}
\alias{survival_summary}
\title{Kaplan-Meier survival summary table}
\usage{
survival_summary(
  data,
  time,
  event,
  by = NULL,
  digits = 1,
  format = c("flextable", "gt", "tibble"),
  theme = c("minimal")
)
}
\arguments{
\item{data}{A \code{data.frame} containing survival time and event status.}

\item{time}{Survival follow-up time. Quoted and bare names are accepted.}

\item{event}{Event indicator. Quoted and bare names are accepted. Numeric
\code{0/1}, numeric \code{1/2}, logical, character, and factor variables
are accepted. For two-level character or factor variables, the second level
is treated as the event.}

\item{by}{Optional grouping variable for separate Kaplan-Meier summaries.
Quoted and bare names are accepted.}

\item{digits}{Number of digits for survival time summaries.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}.}

\item{theme}{Table styling preset.}
}
\value{
A list of class \code{c("gtregression","survival_summary", ...)}
with elements:
\describe{
  \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
  when \code{format = "tibble"}.}
  \item{\code{table_body}}{Tibble with numeric Kaplan-Meier summaries.}
  \item{\code{table_display}}{Display data frame used to render the table.}
  \item{\code{fit}}{Fitted \code{survfit} object.}
  \item{\code{time,event,by,format,source}}{Metadata fields.}
}
}
\description{
Summarise Kaplan-Meier follow-up with total N, events, censoring, and median
survival with a 95% confidence interval.
}
\details{
Median survival is estimated from \code{survival::survfit()}. If the median
survival time is not reached during follow-up, the display table shows
\code{"Not reached"}.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))

survival_summary(
  data = lung_data,
  time = time,
  event = status,
  by = trt
)

survival_summary(
  data = lung_data,
  time = "time",
  event = "status",
  format = tibble
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_uni_model.R
\name{.fit_uni_model}
\alias{.fit_uni_model}
\title{Fit Regression Model with One or More Predictors (Internal)}
\usage{
.fit_uni_model(data, outcome, exposures, approach)
}
\arguments{
\item{data}{A `data.frame` with complete observations for outcome and exposures.}

\item{outcome}{A string. Name of the outcome variable.}

\item{exposures}{A string or character vector of predictor(s).}

\item{approach}{A string specifying the regression approach. One of
`"logit"`, `"firth"`, `"logbinomial"`, `"poisson"`, `"linear"`, `"robpoisson"`, or `"negbin"`.}
}
\value{
A fitted model object (`glm`, `logistf`, `lm`, `riskratio`, or `negbin`) or `NULL` if fitting fails.
}
\description{
Fits a regression model based on the selected approach. Can handle a single exposure or a vector of exposures.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_predict.R
\name{surv_predict}
\alias{surv_predict}
\title{Predict survival probabilities from a parametric survival model}
\usage{
surv_predict(
  model,
  newdata = NULL,
  times,
  digits = 1,
  format = c("flextable", "gt", "tibble"),
  theme = c("minimal")
)
}
\arguments{
\item{model}{A fitted \code{survreg} model, or a \code{surv_reg()} object
containing exactly one fitted model.}

\item{newdata}{Optional \code{data.frame} of profiles for prediction. If
\code{NULL}, one typical profile is built from the model data using medians
for numeric variables and the most common level for categorical variables.}

\item{times}{Numeric vector of follow-up times at which survival probability
should be predicted.}

\item{digits}{Number of digits for displayed follow-up times and
probabilities.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}.}

\item{theme}{Table styling preset.}
}
\value{
A list of class \code{c("gtregression","surv_predict", ...)}
with elements:
\describe{
  \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
  when \code{format = "tibble"}.}
  \item{\code{table_body}}{Tibble with predicted survival probabilities.}
  \item{\code{table_display}}{Display data frame used to render the table.}
  \item{\code{model}}{The fitted \code{survreg} model used for prediction.}
  \item{\code{newdata,times,distribution,format,source}}{Metadata fields.}
}
}
\description{
Estimate model-based survival probabilities at user-specified follow-up
times from a fitted \code{survival::survreg()} model.
}
\details{
\code{surv_predict()} is for parametric survival regression models fitted by
\code{surv_reg()} or \code{survival::survreg()}. It is not a Kaplan-Meier
estimate and it is not a Cox prediction helper.

Supported distributions are \code{"weibull"}, \code{"exponential"},
\code{"lognormal"}, and \code{"loglogistic"}, matching
\code{surv_reg()}. Predictions are conditional on the supplied profile and
the chosen parametric distribution.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))

fit <- surv_reg(
  data = lung_data,
  time = time,
  event = status,
  exposures = "trt",
  adjust_for = c("age", "karno"),
  distribution = weibull
)

surv_predict(
  model = fit$models$trt,
  newdata = data.frame(
    trt = factor("Test", levels = levels(lung_data$trt)),
    age = 60,
    karno = 70
  ),
  times = c(90, 180, 365)
)

surv_predict(
  model = fit,
  times = c(90, 180),
  format = tibble
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logrank_test.R
\name{logrank_test}
\alias{logrank_test}
\title{Log-rank test for Kaplan-Meier survival curves}
\usage{
logrank_test(
  data,
  time,
  event,
  by,
  digits = 2,
  format = c("flextable", "gt", "tibble"),
  theme = c("minimal")
)
}
\arguments{
\item{data}{A \code{data.frame} containing survival time, event status, and
grouping variable.}

\item{time}{Survival follow-up time. Quoted and bare names are accepted.}

\item{event}{Event indicator. Quoted and bare names are accepted. Numeric
\code{0/1}, numeric \code{1/2}, logical, character, and factor variables
are accepted. For two-level character or factor variables, the second level
is treated as the event.}

\item{by}{Grouping variable used to compare survival curves. Quoted and bare
names are accepted.}

\item{digits}{Number of digits for the chi-square statistic and expected
events.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}.}

\item{theme}{Table styling preset.}
}
\value{
A list of class \code{c("gtregression","logrank_test", ...)} with
elements:
\describe{
  \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
  when \code{format = "tibble"}.}
  \item{\code{table_body}}{Tibble with group-level log-rank components and
  overall test statistics.}
  \item{\code{test}}{One-row tibble with chi-square statistic, degrees of
  freedom, and p-value.}
  \item{\code{fit}}{The \code{survdiff} object.}
  \item{\code{time,event,by,format,source}}{Metadata fields.}
}
}
\description{
Compare survival curves between groups using the log-rank test.
}
\details{
The log-rank test compares the observed number of events with the expected
number of events in each group under the null hypothesis that the survival
curves are the same. It is a group comparison, not an effect-size estimate;
use \code{cox_reg()} when a hazard ratio is needed.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))

logrank_test(
  data = lung_data,
  time = time,
  event = status,
  by = trt
)

logrank_test(
  data = lung_data,
  time = "time",
  event = "status",
  by = "trt",
  format = tibble
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers_validation.R
\name{.validate_exposures}
\alias{.validate_exposures}
\title{Validate Exposure Variable(s) for Regression}
\usage{
.validate_exposures(data, exposures)
}
\arguments{
\item{data}{A data frame containing the exposure variables.}

\item{exposures}{Character vector of column names to validate.}
}
\value{
Returns TRUE if valid; otherwise throws an error.
}
\description{
Ensures that the exposure variable has at least two non-missing levels or
sufficient numeric variation to support regression modelling.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmst_table.R
\name{rmst_table}
\alias{rmst_table}
\title{Restricted mean survival time table}
\usage{
rmst_table(
  data,
  time,
  event,
  by = NULL,
  tau,
  digits = 1,
  conf.level = 0.95,
  format = c("flextable", "gt", "tibble"),
  theme = c("minimal")
)
}
\arguments{
\item{data}{A \code{data.frame} containing survival time and event status.}

\item{time}{Survival follow-up time. Quoted and bare names are accepted.}

\item{event}{Event indicator. Quoted and bare names are accepted. Numeric
\code{0/1}, numeric \code{1/2}, logical, character, and factor variables
are accepted. For two-level character or factor variables, the second level
is treated as the event.}

\item{by}{Optional grouping variable for separate RMST estimates. Quoted and
bare names are accepted.}

\item{tau}{Restriction time for RMST. For example, \code{tau = 365} reports
mean survival time restricted to the first 365 days of follow-up.}

\item{digits}{Number of digits for displayed survival time summaries.}

\item{conf.level}{Confidence level for RMST confidence intervals.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}.}

\item{theme}{Table styling preset.}
}
\value{
A list of class \code{c("gtregression","rmst_table", ...)} with
elements:
\describe{
  \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
  when \code{format = "tibble"}.}
  \item{\code{table_body}}{Tibble with numeric RMST summaries.}
  \item{\code{table_display}}{Display data frame used to render the table.}
  \item{\code{fit}}{Fitted \code{survfit} object.}
  \item{\code{time,event,by,tau,format,source}}{Metadata fields.}
}
}
\description{
Estimate restricted mean survival time (RMST) up to a user-specified follow-up
time. RMST is the average survival time observed within a fixed time window,
such as 365 days.
}
\details{
RMST is estimated from \code{survival::survfit()} using Kaplan-Meier methods
and \code{summary.survfit(rmean = tau)}. When \code{by} has exactly two
groups, the table includes the RMST difference as the second group minus the
first group, with a Wald confidence interval and p-value based on the
reported RMST standard errors.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))

rmst_table(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  tau = 365
)

rmst_table(
  data = lung_data,
  time = "time",
  event = "status",
  tau = 180,
  format = tibble
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_surv_fit.R
\name{plot_surv_fit}
\alias{plot_surv_fit}
\title{Plot observed and fitted parametric survival curves}
\usage{
plot_surv_fit(
  data,
  time,
  event,
  by = NULL,
  adjust_for = NULL,
  distributions = c("weibull", "exponential", "lognormal", "loglogistic"),
  break_time_by = NULL,
  xlim = NULL,
  xlab = "Time",
  ylab = "Survival probability",
  title = NULL,
  legend_title = NULL,
  palette = NULL,
  base_size = 13,
  n_points = 200
)
}
\arguments{
\item{data}{A \code{data.frame} containing survival time, event status, and
optional grouping or adjustment variables.}

\item{time}{Survival follow-up time. Quoted and bare names are accepted.}

\item{event}{Event indicator. Quoted and bare names are accepted. Numeric
\code{0/1}, numeric \code{1/2}, logical, character, and factor variables
are accepted. For two-level character or factor variables, the second level
is treated as the event.}

\item{by}{Optional grouping variable for observed and fitted curves. Quoted
and bare names are accepted.}

\item{adjust_for}{Optional character vector of adjustment variables included
in the fitted parametric model. Fitted curves are predicted at typical
adjustment values.}

\item{distributions}{Parametric survival distributions to overlay. One or
more of \code{"weibull"}, \code{"exponential"}, \code{"lognormal"}, or
\code{"loglogistic"}. Quoted and bare values are accepted. Common
spellings such as \code{"log-normal"} and \code{"log-logistic"} are also
accepted.}

\item{break_time_by}{Optional numeric interval for x-axis breaks. If
\code{NULL}, breaks are chosen automatically.}

\item{xlim}{Optional numeric vector of length 2 specifying x-axis limits.}

\item{xlab, ylab}{Axis labels.}

\item{title}{Optional plot title.}

\item{legend_title}{Optional legend title. If \code{NULL}, the labelled
\code{by} variable name is used.}

\item{palette}{Optional character vector of colors for observed groups.}

\item{base_size}{Base font size.}

\item{n_points}{Number of points used to draw each fitted curve.}
}
\value{
A \code{ggplot2} object with attributes \code{km_fit},
  \code{model_fits}, \code{observed_data}, \code{fitted_data}, and
  \code{prediction_data}.
}
\description{
Compare the observed Kaplan-Meier survival curve with fitted parametric
survival curves from \code{survival::survreg()}.
}
\details{
\code{plot_surv_fit()} is a visual diagnostic for parametric survival
modelling. It is useful after \code{surv_model_compare()} and before treating
a final \code{surv_reg()} model as the preferred model. It is not a Cox-model
diagnostic; use \code{check_ph()} for Cox proportional hazards assumptions.

When \code{adjust_for} is supplied, fitted curves are predicted at typical
adjustment values: medians for numeric variables and the most common level for
categorical variables. Use this as a model-fit screen, not as a replacement
for clinical or subject-matter judgement.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))

plot_surv_fit(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  distributions = c(weibull, lognormal),
  break_time_by = 200
)

plot_surv_fit(
  data = lung_data,
  time = "time",
  event = "status",
  by = "trt",
  adjust_for = c(age, karno),
  distributions = "log-logistic"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtregression_app.R
\name{gtregression_app}
\alias{gtregression_app}
\title{Launch the gtregression app}
\usage{
gtregression_app(..., launch.browser = NULL)
}
\arguments{
\item{...}{Additional arguments passed to \code{shiny::runApp()}.}

\item{launch.browser}{Logical; passed to \code{shiny::runApp()}. The default
uses the RStudio Viewer when available, otherwise opens a browser only in
interactive sessions.}
}
\value{
Invisibly returns the result of \code{shiny::runApp()}.
}
\description{
Open a menu-driven Shiny app for common \pkg{gtregression} workflows:
data import, descriptive tables, regression tables, survival analysis,
diagnostics, causal mediation, plots, and exports.
}
\details{
The app is intentionally kept out of the core package startup path. Shiny and
other interface packages are suggested dependencies and are loaded only when
\code{gtregression_app()} is called.

The Advanced tab includes a guided candidate-model builder. Users can name
two to six models, choose model-specific exposures and adjustment variables,
optionally add an interaction, and track a primary exposure. The app fits
compatible \code{multi_reg()}, \code{cox_reg()}, or \code{surv_reg()} objects
before passing them to \code{compare_models()}. Generated code records every
fitting call and the final comparison for reproducible use outside the app.
}
\examples{
if (interactive()) {
  gtregression_app()
}

}
\seealso{
\code{compare_models()}, \code{multi_reg()}, \code{cox_reg()},
  \code{surv_reg()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_mediation.R
\name{plot_mediation}
\alias{plot_mediation}
\title{Plot mediation paths}
\usage{
plot_mediation(mediation_object, show_estimates = TRUE, base_size = 13)
}
\arguments{
\item{mediation_object}{Object returned by \code{mediation_analysis()}.}

\item{show_estimates}{Logical; if \code{TRUE}, show direct and indirect
effect estimates on the plot.}

\item{base_size}{Base font size.}
}
\value{
A \code{ggplot2} object.
}
\description{
Draw the exposure, mediator, and outcome path from a
\code{mediation_analysis()} result.
}
\examples{
med <- mediation_analysis(
  data = data_diabetes_mediation,
  exposure = obesity,
  mediator = glucose,
  outcome = diabetes,
  covariates = c(age, blood_pressure, pregnancies, diabetes_pedigree),
  outcome_approach = logit,
  sims = 50,
  seed = 123
)
plot_mediation(med)
plot_mediation(med, show_estimates = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_convergence.R
\name{check_convergence}
\alias{check_convergence}
\title{Check regression model convergence}
\usage{
check_convergence(
  data,
  exposures,
  outcome,
  approach = "logit",
  multivariate = FALSE,
  format = c("flextable", "gt", "tibble")
)
}
\arguments{
\item{data}{A data frame containing the dataset.}

\item{exposures}{A character vector of predictor variable names. Quoted names
are recommended in scripts, and bare names are also accepted.
If \code{multivariate = FALSE}, each exposure is assessed separately.
If \code{multivariate = TRUE}, exposures are included together.}

\item{outcome}{A character string specifying the outcome variable. Quoted and
bare names are accepted.}

\item{approach}{A character string specifying the regression approach.
One of:
  \code{"logit"}, \code{"logbinomial"}, \code{"poisson"},
  \code{"robpoisson"}, or \code{"negbin"}.}

\item{multivariate}{Logical. If \code{TRUE},
checks convergence for a multivariable model;
  otherwise, performs checks for each univariate model.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}. Use \code{format = "tibble"} for the
original data-frame style output.}
}
\value{
A data frame, \code{gt_tbl}, or \code{flextable} summarizing
convergence diagnostics, including:
\describe{
  \item{\code{Exposure}}{Name of the exposure variable.}
  \item{\code{Model}}{The regression approach used.}
  \item{\code{Converged}}{\code{TRUE} if the model converged successfully;
   \code{FALSE} otherwise.}
  \item{\code{Max.prob.}}{Maximum predicted probability or
  fitted value in the dataset.}
}
}
\description{
Check whether requested models converge, either one exposure at a time or as
a single multivariable model.
}
\details{
For \code{robpoisson}, predicted probabilities (fitted values) may exceed 1,
which is acceptable when estimating risk ratios but should not be interpreted
as actual probabilities.

This is especially useful for \code{"logbinomial"} models, where convergence
problems are common and robust Poisson may be a practical alternative.
}
\examples{
if (requireNamespace("gtregression", quietly = TRUE)) {
  data(data_SynthDiabetes, package = "gtregression")

  check_convergence(
    data = data_SynthDiabetes,
    exposures = c("age", "mass"),
    outcome = "diabetes",
    approach = "logit",
    format = "tibble"
  )

  check_convergence(
    data = data_SynthDiabetes,
    exposures = c("age", "mass"),
    outcome = "diabetes",
    approach = "logit",
    multivariate = TRUE,
    format = "gt"
  )
}
}
\seealso{
[identify_confounder()],  [interaction_models()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modify_table.R
\name{modify_table}
\alias{modify_table}
\title{Modify Regression/Descriptive Tables (labels, headers, caption, notes)}
\usage{
modify_table(
  gt_table,
  variable_labels = NULL,
  level_labels = NULL,
  header_labels = NULL,
  caption = NULL,
  bold_labels = TRUE,
  bold_levels = FALSE,
  italic_labels = FALSE,
  italic_levels = FALSE,
  remove_N = FALSE,
  remove_N_obs = FALSE,
  remove_abbreviations = FALSE,
  remove_adjustment_note = FALSE,
  caveat = NULL
)
}
\arguments{
\item{gt_table}{Table object produced by this package (must contain
\code{$table_display}).}

\item{variable_labels}{Named character vector, for example
\code{c(old_var = "New label", ...)}.}

\item{level_labels}{Named list for factor levels:
\code{list(var1 = c(old = "New", ...), var2 = c(...))}.}

\item{header_labels}{Named character vector to rename visible headers, e.g.
\code{c("OR (95\% CI)" = "Crude OR", "p-value" = "P")}. Common aliases
such as \code{estimate}, \code{p.value}, and \code{N} are also accepted.}

\item{caption}{Optional caption/title.}

\item{bold_labels}{Logical; bold variable (header) rows in the body. Defaults
to \code{TRUE} to preserve the package table hierarchy.}

\item{bold_levels}{Logical; bold factor level rows in the body.}

\item{italic_labels}{Logical; italicize variable (header) rows in the body.}

\item{italic_levels}{Logical; italicize factor level rows in the body.}

\item{remove_N}{Logical; if \code{TRUE}, drops displayed \code{N} columns
from univariable and stratified package tables. For stratified survival
outputs, event columns are retained unless the original table was created
with \code{show_sample = "none"}.}

\item{remove_N_obs}{Logical; if \code{TRUE}, suppresses multivariable
complete-case footnote.}

\item{remove_abbreviations}{Logical; if \code{TRUE}, removes the
Abbreviations footnote line.}

\item{remove_adjustment_note}{Logical; if \code{TRUE}, removes the automatic
\code{Adjusted for ...} footnote. Use \code{caveat} to add customised
wording.}

\item{caveat}{Optional extra footnote.}
}
\value{
The modified table object (same class as input).
}
\description{
Works with objects created by this package (class \code{"gtregression"}):
\code{uni_reg()}, \code{multi_reg()}, \code{cox_reg()}, \code{surv_reg()},
stratified regression outputs, \code{descriptive_table()}, and
\code{merge_tables()}.
No \pkg{gtsummary} dependency or fallback.
}
\examples{
birthwt_data <- data_birthwt |>
  dplyr::mutate(
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
  )

tbl <- uni_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = c("age", "smoke", "ht"),
  approach = "logit",
  format = "gt"
)

modify_table(
  tbl,
  variable_labels = c(age = "Maternal age", smoke = "Smoking"),
  level_labels = list(smoke = c(Yes = "Smoker")),
  header_labels = c(estimate = "Crude OR", p.value = "P"),
  caption = "Univariable regression for low birth weight"
)$table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stratified_multi_reg.R
\name{stratified_multi_reg}
\alias{stratified_multi_reg}
\title{Stratified multivariable regression}
\usage{
stratified_multi_reg(
  data,
  outcome,
  exposures,
  stratifier,
  adjust_for = NULL,
  interaction = NULL,
  approach = "logit",
  format = c("flextable", "gt"),
  theme = c("minimal"),
  show_ref = TRUE
)
}
\arguments{
\item{data}{A data frame containing the variables.}

\item{outcome}{Character scalar; name of the outcome variable. Quoted and
bare names are accepted.}

\item{exposures}{Character vector of exposure variables to report. Quoted
names are recommended in scripts, and bare names are also accepted.}

\item{stratifier}{Character scalar; name of the stratifying variable. Quoted
and bare names are accepted.}

\item{adjust_for}{Optional character vector of adjustment variables.
Quoted and bare names are accepted. This argument works the same way as in
\code{multi_reg()}.}

\item{interaction}{Optional character scalar specifying one interaction term
using standard formula syntax, e.g. \code{"bmi*sex"}}

\item{approach}{One of \code{"logit"}, \code{"firth"}, \code{"logbinomial"}, \code{"poisson"},
\code{"linear"}, \code{"robpoisson"}, or \code{"negbin"}}

\item{format}{One of \code{"flextable"} (default) or \code{"gt"}.}

\item{theme}{Preset name (e.g. \code{"minimal"}, \code{"striped"}, \code{"clinical"},
\code{"shaded"}, \code{"jama"}) or primitives
\code{c("plain","zebra","lines","labels_bold","compact","header_shaded")}}

\item{show_ref}{Logical; if \code{TRUE} (default), display reference-category
rows as \code{"Ref."}. If \code{FALSE}, hide reference rows; a message
reminds users to use \code{show_ref = TRUE} when reference rows are needed.}
}
\value{
A list of class \code{c("gtregression","stratified_multi_reg", ...)} with:
\describe{
  \item{\code{table}}{A \code{flextable} (format = \code{"flextable"}) or
  \code{gt_tbl} (format = \code{"gt"}).}
  \item{\code{table_display}}{Wide data frame used to build the table.}
  \item{\code{per_stratum}}{Named list of per-stratum regression results.}
  \item{\code{models}}{Named list of fitted models by stratum.}
  \item{\code{model_summaries}}{Named list of model summaries by stratum.}
  \item{\code{variable_labels}}{Named character vector of display labels used
  for exposure variables.}
  \item{\code{reg_check}}{Named list of diagnostics by stratum.}
  \item{\code{by}, \code{levels}, \code{approach}, \code{format}, \code{source}}{Metadata fields.}
}
}
\description{
Fits multivariable regression models within each stratum and returns a
unified wide table with one "Characteristic" column and, under bold
spanners for each stratum, model N, "Adjusted <effect>", and "p-value".
}
\details{
If \code{adjust_for = NULL}, all \code{exposures} are included in one
multivariable model within each stratum. If \code{adjust_for} is supplied,
one adjusted model is fitted per exposure within each stratum.


If exposure variables have a \code{"label"} attribute, for example from
\code{labelled::var_label()}, those labels are used automatically in the
displayed table. Internal matching still uses the original column names.
}
\examples{
birthwt_data <- data_birthwt |>
  transform(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    ht = factor(ht, levels = c(0, 1), labels = c("No", "Yes")),
    ui = factor(ui, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1),
                 labels = c("Normal BW", "Low BW"))
  )

stratified_multi <- stratified_multi_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = c("age", "lwt", "smoke", "ht"),
  stratifier = "race",
  approach = "logit",
  format = "gt"
)

stratified_adjusted <- stratified_multi_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = c("smoke", "ht", "ui"),
  stratifier = "race",
  adjust_for = c("age", "lwt"),
  approach = "logit",
  format = "gt"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{data_SynthDiabetes}
\alias{data_SynthDiabetes}
\title{Synthetic Diabetes Dataset}
\format{
A data frame with 768 observations and 9 variables:
\describe{
  \item{pregnant}{Number of times pregnant}
  \item{glucose}{Plasma glucose concentration (glucose tolerance test)}
  \item{pressure}{Diastolic blood pressure (mm Hg)}
  \item{triceps}{Triceps skin fold thickness (mm)}
  \item{insulin}{2-Hour serum insulin (mu U/ml)}
  \item{mass}{Body mass index (BMI)}
  \item{pedigree}{Diabetes pedigree function}
  \item{age}{Age in years}
  \item{diabetes}{Factor indicating diabetes status (pos/neg)}
}
}
\source{
\code{mlbench::SynthDiabetes2}, available in \pkg{mlbench}
version 2.1-11 or later.
}
\usage{
data_SynthDiabetes
}
\description{
A synthetic replacement for the former diabetes dataset from the
\pkg{mlbench} package. It has the same structure and number of observations
but does not represent real people.
Useful for demonstrating regression approaches for binary outcomes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{data_epilepsy}
\alias{data_epilepsy}
\title{Epilepsy Treatment and Seizure Counts}
\format{
A data frame with 236 observations and 9 variables:
\describe{
  \item{y}{Number of seizures in a 2-week period (count)}
  \item{trt}{Treatment group (factor): \code{placebo} or \code{progabide}}
  \item{base}{Seizure count during baseline period (numeric)}
  \item{age}{Age of patient (numeric)}
  \item{V4}{Indicator for 4th visit (binary)}
  \item{subject}{Patient ID (factor)}
  \item{period}{Follow-up period number (integer)}
  \item{lbase}{Log of baseline seizures (numeric)}
  \item{lage}{Log of age (numeric)}
}
}
\source{
\pkg{MASS} package. Original data from Thall and Vail (1990)
}
\usage{
data_epilepsy
}
\description{
RCT on the effect of a drug on the seizures in patients with epilepsy.
Contains repeated measures data with treatment groups,
baseline seizure counts, and follow-up counts.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers-strata.R
\name{.strata_build_wide_uni}
\alias{.strata_build_wide_uni}
\title{Assemble wide display DF for all strata (univariate)}
\usage{
.strata_build_wide_uni(
  data,
  outcome,
  exposures,
  stratifier,
  per_stratum,
  variable_labels = NULL,
  show_ref = TRUE
)
}
\description{
Assemble wide display DF for all strata (univariate)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.gtregression}
\alias{print.gtregression}
\title{Print gtregression objects (unified)}
\usage{
\method{print}{gtregression}(x, ...)
}
\arguments{
\item{x}{An object with class \code{"gtregression"}.}

\item{...}{Ignored. Present for compatibility with the generic.}
}
\description{
Prints the rendered table for any object produced by this package
(objects that include class \code{"gtregression"}), regardless of subtype
(\code{uni_reg}, \code{multi_reg}, \code{stratified_*}, \code{merged_table},
\code{descriptive_table}, ...). If no rendered table is found, a compact
structure of the object (or its display data) is shown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{data_gt_quin}
\alias{data_gt_quin}
\title{Student Absenteeism in Rural Schools}
\format{
A data frame with 146 observations and 5 variables:
\describe{
  \item{Eth}{Ethnicity (\code{"A"} = Aboriginal,
  \code{"N"} = Non-Aboriginal)}
  \item{Sex}{Sex (\code{"F"} or \code{"M"})}
  \item{Age}{Age group (\code{"F0", "F1", "F2", "F3"})}
  \item{Lrn}{Learner status (\code{"AL"} = average learner,
   \code{"SL"} = slow learner)}
  \item{Days}{Number of days absent from school (count outcome)}
}
}
\source{
\pkg{MASS} package. See also Venables and Ripley (2002),
 *Modern Applied Statistics with S*.
}
\usage{
data_gt_quin
}
\description{
This dataset contains observations on the number of days absent from school
for children in rural Australia,
along with student characteristics. It's commonly used to demonstrate count
models such as Poisson and Negative Binomial regression.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers-strata.R
\name{.strata_pull_cols_uni}
\alias{.strata_pull_cols_uni}
\title{Pull (N, Effect, p) for a univariate stratum, aligned to a skeleton}
\usage{
.strata_pull_cols_uni(dlev, uni_res, skeleton, is_factor, exposures, outcome)
}
\description{
Pull (N, Effect, p) for a univariate stratum, aligned to a skeleton
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reg_check_linear.R
\name{.reg_check_linear}
\alias{.reg_check_linear}
\title{Linear regression diagnostic checks (internal)}
\usage{
.reg_check_linear(model, exposure)
}
\arguments{
\item{model}{A fitted linear model (`lm` object).}

\item{exposure}{Character string giving the name of the exposure variable
for labeling.}
}
\value{
A data frame with one row per diagnostic test, including:
\describe{
  \item{Exposure}{Name of the exposure variable.}
  \item{Test}{Diagnostic test name.}
  \item{Statistic}{Test statistic or summary (e.g., p-values).}
  \item{Interpretation}{Plain-language result interpretation.}
}
}
\description{
Runs diagnostic checks for fitted linear regression models.
}
\details{
The returned rows cover heteroskedasticity, residual normality, model
specification, and influential observations. If a diagnostic cannot be
computed for a given model, the function returns a clear "not available"
result for that row instead of failing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/identify_confounder.R
\name{identify_confounder}
\alias{identify_confounder}
\title{Identify confounders and effect modifiers}
\usage{
identify_confounder(
  data,
  outcome = NULL,
  exposure,
  potential_confounder,
  approach = "logit",
  time = NULL,
  event = NULL,
  distribution = "weibull",
  method = "change",
  threshold = 10,
  emm_threshold = 10,
  emm_test = c("interaction", "both", "estimate"),
  interaction_alpha = 0.05,
  format = c("flextable", "gt"),
  theme = c("minimal")
)
}
\arguments{
\item{data}{A data frame.}

\item{outcome}{Outcome variable name. Quoted and bare names are accepted.
Required for ordinary regression approaches. Leave unset for Cox and
parametric survival approaches and supply \code{time} and \code{event}
instead.}

\item{exposure}{Exposure variable name(s). Can be a character scalar or
vector. Quoted names are recommended in scripts, and bare names are also
accepted.}

\item{potential_confounder}{Candidate confounder/effect-modifier variable
name(s). Can be a character scalar or vector. Quoted names are recommended
in scripts, and bare names are also accepted.}

\item{approach}{Regression approach. One of \code{"logit"},
\code{"logbinomial"}, \code{"poisson"}, \code{"robpoisson"},
\code{"linear"}, \code{"negbin"}, \code{"cox"}, or
\code{"survreg"}.}

\item{time}{Survival time variable name for \code{approach = "cox"} or
\code{approach = "survreg"}. Quoted and bare names are accepted.}

\item{event}{Event indicator variable name for survival approaches. Values
may be coded as 0/1, 1/2, logical, or a two-level factor/character
variable.}

\item{distribution}{Parametric survival distribution for
\code{approach = "survreg"}. One of \code{"weibull"},
\code{"exponential"}, \code{"lognormal"}, or \code{"loglogistic"}.}

\item{method}{Confounding assessment method. One of \code{"change"},
\code{"mh"}, or \code{"both"}. \code{"change"} compares crude and
adjusted model estimates. \code{"mh"} compares crude and
Mantel-Haenszel pooled estimates and is available for binary outcome,
binary exposure, and categorical strata. \code{"both"} uses either method.}

\item{threshold}{Percent change threshold for confounding assessment.}

\item{emm_threshold}{Threshold for relative spread in stratum-specific
estimates when using estimate-based effect-modification screening.}

\item{emm_test}{One of \code{"interaction"}, \code{"both"}, or
\code{"estimate"}.}

\item{interaction_alpha}{Alpha threshold for interaction p-values.}

\item{format}{Output table format. One of \code{"flextable"} (default) or
\code{"gt"}.}

\item{theme}{Table theme preset or primitives.}
}
\value{
If a single exposure-candidate pair is supplied, returns a detailed list.

If multiple combinations are supplied, returns a list with:
\describe{
  \item{summary}{A tibble with one row per exposure-candidate combination.}
  \item{details}{A named list of detailed results for each combination.}
}
}
\description{
Review whether one or more candidate variables may act as confounders or
effect modifiers for one or more exposures.
}
\details{
The function first assesses possible effect modification using
stratum-specific estimates and/or an interaction test. If no important
effect modification is detected, it then assesses confounding using the
selected method.

This is a screening aid for viewing and organising results. Confounding and
effect modification should be interpreted using subject-matter knowledge,
study design, and causal diagrams such as DAGs. Automated change-in-estimate
and interaction checks should not be used as the sole basis for model
adjustment.

For each exposure-candidate pair, the interaction screen fits
\code{outcome ~ exposure + potential_confounder +
exposure:potential_confounder}. It does not adjust for other covariates.
Consequently, the reported interaction p-value is a candidate-only screening
result, not a final adjusted interaction analysis. Use
\code{interaction_models(covariates = ...)} when testing a planned,
covariate-adjusted interaction.

Use this function when you want to screen one or more candidate variables and
organise crude, adjusted, Mantel-Haenszel, and effect-modification signals in
one place. For a focused comparison of models with and without a planned
exposure-by-modifier interaction term, use \code{\link{interaction_models}()}.
}
\examples{
birthwt_data <- data_birthwt |>
  dplyr::mutate(
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other"))
  )

identify_confounder(
  data = birthwt_data,
  outcome = low,
  exposure = smoke,
  potential_confounder = race,
  approach = logit
)

lung_data <- data_lungcancer |>
  dplyr::mutate(
    trt = factor(trt, levels = c(1, 2),
                 labels = c("Standard treatment", "Test treatment")),
    prior = factor(prior, levels = c(0, 10), labels = c("No", "Yes"))
  )

identify_confounder(
  data = lung_data,
  time = time,
  event = status,
  exposure = trt,
  potential_confounder = prior,
  approach = cox
)

}
\seealso{
\code{\link{interaction_models}()} for focused model comparison of
  a planned interaction term.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers-strata.R
\name{.strata_levels}
\alias{.strata_levels}
\title{Get ordered levels for a stratifier}
\usage{
.strata_levels(data, stratifier)
}
\description{
Get ordered levels for a stratifier
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multi_reg.R
\name{multi_reg}
\alias{multi_reg}
\title{Multivariable regression}
\usage{
multi_reg(
  data,
  outcome,
  exposures,
  adjust_for = NULL,
  interaction = NULL,
  approach = "logit",
  format = c("flextable", "gt"),
  theme = c("minimal"),
  model_stats = FALSE,
  show_ref = TRUE
)
}
\arguments{
\item{data}{A \code{data.frame} containing the variables of interest.}

\item{outcome}{Character scalar; name of the outcome variable. Quoted and
bare names are accepted.}

\item{exposures}{Character vector; exposure variable(s) to report.
Quoted names are recommended in scripts, and bare names are also accepted.
If \code{adjust_for = NULL}, all exposures are included in a single
multivariable model. If \code{adjust_for} is supplied, one adjusted
model is fitted per exposure and only exposure-specific adjusted
estimate(s) are displayed.}

\item{adjust_for}{Optional character vector of adjustment variables. Quoted
and bare names are accepted. Must not overlap with \code{exposures}.}

\item{interaction}{Optional character scalar specifying one interaction term
using standard formula syntax, e.g. \code{"bmi*sex"}.
When used with \code{adjust_for}, only a single exposure should be supplied.}

\item{approach}{Character scalar specifying the regression approach.
One of \code{"logit"}, \code{"firth"}, \code{"logbinomial"}, \code{"poisson"},
\code{"linear"}, \code{"robpoisson"}, or \code{"negbin"}.
Use \code{"firth"} for Firth penalized logistic regression, especially
with sparse cells or separation.}

\item{format}{Output table format; one of \code{"flextable"} (default) or
\code{"gt"}.}

\item{theme}{Table styling preset (e.g. \code{"minimal"}, \code{"striped"},
\code{"clinical"}, \code{"shaded"}, \code{"jama"}) or a character vector of
primitives such as \code{c("plain","zebra","lines","labels_bold","compact","header_shaded")}.}

\item{model_stats}{Logical; if \code{TRUE}, extract model-fit statistics
such as AIC, BIC, log-likelihood, deviance, pseudo R-squared for
non-linear models, and R-squared for linear models. Statistics are stored
in the returned object's \code{model_stats} element and are not added to
the publication table.}

\item{show_ref}{Logical; if \code{TRUE} (default), display reference-category
rows as \code{"Ref."}. If \code{FALSE}, hide reference rows; a message
reminds users to use \code{show_ref = TRUE} when reference rows are needed.}
}
\value{
A list of class \code{c("gtregression","multi_reg", ...)} with elements:
\describe{
  \item{table}{A \code{flextable} (if \code{format="flextable"}) or
  \code{gt_tbl} (if \code{format="gt"}).}
  \item{table_body}{A data frame of adjusted estimates and confidence intervals
  for each exposure and level.}
  \item{table_display}{A data frame used for rendering the final table,
  including header and level rows.}
  \item{models}{A list of fitted model(s).}
  \item{model_summaries}{\code{summary()} output for the fitted model(s).}
  \item{model_stats}{Model-fit statistics when \code{model_stats = TRUE};
  otherwise \code{NULL}.}
  \item{variable_labels}{Named character vector of display labels used for
  exposure variables.}
  \item{reg_check}{Regression diagnostics for linear models. Printing this
  element renders a publication-ready table; individual diagnostic rows remain
  available for programmatic use. Other approaches return a message.}
  \item{approach}{The regression approach used.}
  \item{format}{The output format used.}
  \item{source}{Function identifier (\code{"multi_reg"}).}
  \item{adjusted_mode}{Whether one adjusted model per exposure was fitted.}
  \item{adjust_for}{Adjustment variables supplied by the user, if any.}
  \item{exposures}{Exposure variables supplied by the user.}
  \item{interaction}{Interaction term supplied by the user, if any.}
}
}
\description{
Fit adjusted or multivariable regression models and return a manuscript-style
table in \pkg{flextable} or \pkg{gt} format.
}
\details{
Use the default mode when you want all exposures in one model. Use
\code{adjust_for} when you want one adjusted estimate per reported exposure,
with the same adjustment set used repeatedly.

If exposure variables have a \code{"label"} attribute, for example from
\code{labelled::var_label()}, those labels are used automatically in the
displayed table and plots. Internal matching still uses the original column
names.

In default mode (\code{adjust_for = NULL}), all exposures are included in a
single multivariable model. In adjusted mode, one model is fitted per exposure,
adjusting for the variables specified in \code{adjust_for}.

Interaction terms specified via \code{interaction} are included in the model
using standard formula expansion (e.g. \code{bmi*sex}). Interaction effects are
displayed as additional rows beneath the corresponding exposure.
}
\examples{
endometrial_data <- data_endometrial
endometrial_data$HG <- factor(endometrial_data$HG, levels = c(0, 1))
endometrial_data$NV <- factor(endometrial_data$NV, levels = c(0, 1))
multi_reg(endometrial_data, HG, c(NV, PI, EH), approach = firth, format = gt)$table

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stratified_uni_reg.R
\name{stratified_uni_reg}
\alias{stratified_uni_reg}
\title{Stratified univariable regression}
\usage{
stratified_uni_reg(
  data,
  outcome,
  exposures,
  stratifier,
  approach = "logit",
  format = c("flextable", "gt"),
  theme = c("minimal"),
  show_ref = TRUE
)
}
\arguments{
\item{data}{A data frame containing the variables.}

\item{outcome}{Character scalar; name of the outcome variable. Quoted and
bare names are accepted.}

\item{exposures}{Character vector of exposure variables to model. Quoted
names are recommended in scripts, and bare names are also accepted.}

\item{stratifier}{Character scalar; name of the stratifying variable. Quoted
and bare names are accepted.}

\item{approach}{Modeling approach. One of \code{"logit"}, \code{"firth"},
\code{"logbinomial"}, \code{"poisson"}, \code{"robpoisson"},
\code{"linear"}, or \code{"negbin"}.}

\item{format}{Output table format; one of \code{"flextable"} (default) or
\code{"gt"}.}

\item{theme}{Table styling preset or theme primitives.}

\item{show_ref}{Logical; if \code{TRUE} (default), display reference-category
rows as \code{"Ref."}. If \code{FALSE}, hide reference rows; a message
reminds users to use \code{show_ref = TRUE} when reference rows are needed.}
}
\value{
A list of class
  \code{c("gtregression", "stratified_uni_reg", ...)} with elements:
\describe{
  \item{\code{table}}{A rendered \code{gt_tbl} or \code{flextable}.}
  \item{\code{table_display}}{Display-ready wide stratified results.}
  \item{\code{per_stratum}}{List of complete \code{uni_reg()} results by
  stratum.}
  \item{\code{models}}{List of fitted model objects by stratum.}
  \item{\code{model_summaries}}{List of model summaries by stratum.}
  \item{\code{variable_labels}}{Named character vector of display labels used
  for exposure variables.}
  \item{\code{reg_check}}{Regression diagnostics by stratum.}
  \item{\code{by}}{The stratifier variable.}
  \item{\code{levels}}{Strata included in the analysis.}
  \item{\code{approach}}{The regression approach used.}
  \item{\code{format}}{The output format used.}
  \item{\code{source}}{Function identifier
  (\code{"stratified_uni_reg"}).}
}
}
\description{
Fit univariable regression models within each level of a stratifier and
combine the results into a stratified table rendered with
\pkg{gt} or \pkg{flextable}.
}
\details{
If exposure variables have a \code{"label"} attribute, for example from
\code{labelled::var_label()}, those labels are used automatically in the
displayed table. Internal matching still uses the original column names.
}
\section{Accessors}{

\describe{
  \item{\code{$table}}{Rendered stratified regression table.}
  \item{\code{$table_display}}{Wide display data used to build the table.}
  \item{\code{$per_stratum}}{Full \code{uni_reg()} result objects by
  stratum.}
  \item{\code{$models}}{List of fitted model objects for each stratum.}
  \item{\code{$model_summaries}}{List of model summaries.}
  \item{\code{$reg_check}}{Diagnostic check results (when applicable).}
}
}

\examples{
birthwt_data <- data_birthwt |>
  transform(
    race = factor(race, levels = c(1, 2, 3),
                  labels = c("White", "Black", "Other")),
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1),
                 labels = c("Normal BW", "Low BW"))
  )

stratified_uni <- stratified_uni_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = c("age", "lwt", "smoke"),
  stratifier = "race",
  approach = "logit",
  format = "gt"
)

stratified_uni$table

}
\seealso{
[multi_reg()], [plot_reg()], [identify_confounder()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{data_birthwt}
\alias{data_birthwt}
\title{Birth Weight Data}
\format{
A data frame with 189 observations and 10 variables:
\describe{
  \item{low}{Indicator for birth weight < 2500g (binary):
  \code{0 = normal}, \code{1 = low birth weight}}
  \item{age}{Mother's age in years (numeric)}
  \item{lwt}{Mother's weight in pounds at last menstrual period (numeric)}
  \item{race}{Mother's race (factor): \code{1 = White}, \code{2 = Black},
  \code{3 = Other}}
  \item{smoke}{Smoking status during pregnancy (binary): \code{0 = No},
   \code{1 = Yes}}
  \item{ptl}{Number of previous premature labors (integer)}
  \item{ht}{History of hypertension (binary): \code{0 = No}, \code{1 = Yes}}
  \item{ui}{Presence of uterine irritability (binary): \code{0 = No},
  \code{1 = Yes}}
  \item{ftv}{no of physician visits during the 1st trimester (integer, 0–6)}
  \item{bwt}{Birth weight in grams (numeric)}
}
}
\source{
Hosmer, D.W., Lemeshow, S. (1989). *Applied Logistic Regression.*
New York: Wiley.
Also available in \pkg{MASS} and described in detail in its documentation.
}
\usage{
data_birthwt
}
\description{
A dataset from the \pkg{MASS} package containing risk factors
associated with low birth weight (LBW) in newborns.
Originally collected at Baystate Medical Center,
Springfield, Massachusetts, USA.
}
\details{
The outcome variable is binary (`low`): birth weight < 2500g (yes = 1)
or not (no = 0).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save_functions.R
\name{save_table}
\alias{save_table}
\title{Save a single regression or summary table}
\usage{
save_table(
  tbl,
  filename = "table",
  format = c("docx", "rtf", "pdf", "html"),
  orientation = c("auto", "portrait", "landscape"),
  fit_width = TRUE,
  font_size = 9,
  min_font_size = 8
)
}
\arguments{
\item{tbl}{A \code{gtregression} object, \code{merged_table} object,
\code{gt_tbl}, or \code{flextable}.}

\item{filename}{File name for the output. Extension is optional. If no
directory is supplied, the file is saved in \code{tempdir()}.}

\item{format}{Output format. One of \code{"docx"}, \code{"rtf"},
\code{"pdf"}, or \code{"html"}. Flextable objects can be saved as
\code{"docx"}, \code{"rtf"}, or \code{"html"}.}

\item{orientation}{Word page orientation for DOCX output. One of
\code{"auto"}, \code{"portrait"}, or \code{"landscape"}. With
\code{"auto"}, wide tables are saved in landscape orientation before any
font-size reduction is attempted.}

\item{fit_width}{Logical. If \code{TRUE}, try to fit flextable DOCX output
within the selected Word page width. If \code{FALSE}, keep the natural
autofit table width.}

\item{font_size}{Requested font size for flextable DOCX output.}

\item{min_font_size}{Smallest font size allowed when fitting wide flextable
DOCX output. The font size is never reduced below this value.}
}
\value{
Saves the file to disk. Invisibly returns the normalized file path.
}
\description{
Saves a \code{gtregression} table, merged table, \code{gt_tbl}, or
\code{flextable} as a Word, RTF, PDF, or HTML file.
}
\examples{
birthwt_data <- data_birthwt |>
  dplyr::mutate(
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
  )

tbl <- uni_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = c("age", "smoke"),
  approach = "logit"
)

save_table(tbl, filename = tempfile("table"), format = "html")

# Wide Word tables can be saved in landscape orientation.
\donttest{
save_table(
  tbl,
  filename = tempfile("table-wide"),
  format = "docx",
  orientation = "auto",
  fit_width = TRUE,
  font_size = 9,
  min_font_size = 8
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save_functions.R
\name{save_docx}
\alias{save_docx}
\title{Save multiple tables and plots to a Word document}
\usage{
save_docx(
  tables = NULL,
  plots = NULL,
  filename = "report.docx",
  titles = NULL,
  table_width = 6.5,
  plot_width = 6,
  plot_height = 5
)
}
\arguments{
\item{tables}{A list of tables. Each element may be a \code{gtregression}
object, \code{merged_table} object, \code{gt_tbl}, or \code{flextable}.}

\item{plots}{A list of \code{ggplot2} plot objects.}

\item{filename}{File name for the output, with or without \code{.docx}. If no
directory is supplied, the file is saved in \code{tempdir()}.}

\item{titles}{Optional character vector of titles for tables and plots in
the order they are added.}

\item{table_width}{Maximum table width in inches for Word export. The default
\code{6.5} fits a standard portrait Word page with common margins. Use
\code{NULL} to keep the original flextable widths.}

\item{plot_width}{Width of inserted plots in inches.}

\item{plot_height}{Height of inserted plots in inches.}
}
\value{
Saves the Word document to disk. Invisibly returns the normalized
  file path.
}
\description{
Saves a collection of \code{gtregression} tables, merged tables,
\code{gt_tbl}/\code{flextable} objects, and \code{ggplot2} plots into a
single Word document.
}
\examples{
birthwt_data <- data_birthwt |>
  dplyr::mutate(
    smoke = factor(smoke, levels = c(0, 1), labels = c("No", "Yes")),
    low = factor(low, levels = c(0, 1), labels = c("Normal BW", "Low BW"))
  )

tbl <- uni_reg(
  data = birthwt_data,
  outcome = "low",
  exposures = c("age", "smoke"),
  approach = "logit",
  format = "flextable"
)

save_docx(tables = tbl, filename = tempfile("report"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/km_risk_table.R
\name{km_risk_table}
\alias{km_risk_table}
\title{Kaplan-Meier risk table}
\usage{
km_risk_table(
  data,
  time,
  event,
  by = NULL,
  times,
  digits = 1,
  extend = TRUE,
  format = c("flextable", "gt", "tibble"),
  theme = c("minimal")
)
}
\arguments{
\item{data}{A \code{data.frame} containing survival time and event status.}

\item{time}{Survival follow-up time. Quoted and bare names are accepted.}

\item{event}{Event indicator. Quoted and bare names are accepted. Numeric
\code{0/1}, numeric \code{1/2}, logical, character, and factor variables
are accepted. For two-level character or factor variables, the second level
is treated as the event.}

\item{by}{Optional grouping variable for separate Kaplan-Meier risk tables.
Quoted and bare names are accepted.}

\item{times}{Numeric vector of follow-up times for the risk table.}

\item{digits}{Number of digits for displayed follow-up times.}

\item{extend}{Logical. If \code{TRUE}, requested times beyond the observed
follow-up range are retained using the last available Kaplan-Meier risk set.}

\item{format}{Output format. One of \code{"flextable"} (default),
\code{"gt"}, or \code{"tibble"}.}

\item{theme}{Table styling preset.}
}
\value{
A list of class \code{c("gtregression","km_risk_table", ...)}
with elements:
\describe{
  \item{\code{table}}{A \code{flextable}, \code{gt_tbl}, or \code{NULL}
  when \code{format = "tibble"}.}
  \item{\code{table_body}}{Tibble with Kaplan-Meier risk table counts.}
  \item{\code{table_display}}{Display data frame used to render the table.}
  \item{\code{fit}}{Fitted \code{survfit} object.}
  \item{\code{time,event,by,times,format,source}}{Metadata fields.}
}
}
\description{
Tabulate the number at risk at selected follow-up times.
}
\details{
The \code{At risk} column gives the number still under observation at each
requested time. The \code{Events} and \code{Censored} columns are interval
counts up to each requested time point as returned by
\code{summary.survfit()}.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))

km_risk_table(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  times = c(0, 90, 180, 365)
)

km_risk_table(
  data = lung_data,
  time = "time",
  event = "status",
  times = c(0, 90, 180),
  format = tibble
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{data_lungcancer}
\alias{data_lungcancer}
\title{Lung Cancer Trial Data}
\format{
A data frame with 137 observations and 8 variables:
\describe{
  \item{trt}{Treatment group (1 = standard, 2 = test)}
  \item{celltype}{Cell type (squamous, smallcell, adeno, large)}
  \item{time}{Survival time (in days)}
  \item{status}{Censoring status (1 = died, 0 = censored)}
  \item{karno}{Karnofsky performance score (higher = better)}
  \item{diagtime}{Months from diagnosis to randomization}
  \item{age}{Age in years}
  \item{prior}{Prior therapy (0 = no, 10 = yes)}
}
}
\source{
\url{https://CRAN.R-project.org/package=survival}
}
\usage{
data_lungcancer
}
\description{
Survival data from a clinical trial of lung cancer patients conducted
 by the Veteran's Administration.
}
\references{
Kalbfleisch JD and Prentice RL (1980).
 The Statistical Analysis of Failure Time Data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_reg.R
\name{plot_reg}
\alias{plot_reg}
\title{Plot regression estimates}
\usage{
plot_reg(
  tbl,
  title = NULL,
  caption = NULL,
  ref_line = NULL,
  order_y = NULL,
  log_x = FALSE,
  xlim = NULL,
  breaks = NULL,
  point_color = "#6B7280",
  errorbar_color = "#6B7280",
  point_size = 2.8,
  point_stroke = 0.55,
  ci_linewidth = 0.55,
  base_size = 12,
  show_ref = NULL,
  sig_color = "#0072B2",
  sig_errorbar_color = "#0072B2",
  alpha = 0.05,
  show_adjustment_note = TRUE
)
}
\arguments{
\item{tbl}{A fitted \code{gtregression} object.}

\item{title}{Optional plot title.}

\item{caption}{Optional plot caption. If \code{NULL}, notes are added
automatically for adjusted \code{multi_reg()} objects and displayed
reference categories.}

\item{ref_line}{Optional numeric value for the reference line.
Defaults to 0 for linear models and 1 otherwise.}

\item{order_y}{Optional character vector specifying exposure order.}

\item{log_x}{Logical. If \code{TRUE}, use a log-scaled x-axis for ratio
measures. The default \code{FALSE} uses a linear x-axis.}

\item{xlim}{Optional numeric vector of length 2 specifying x-axis limits.}

\item{breaks}{Optional numeric vector of x-axis tick breaks.}

\item{point_color}{Fill color for non-significant estimate points.}

\item{errorbar_color}{Color for non-significant confidence intervals.}

\item{point_size}{Diameter of estimate points in millimetres.}

\item{point_stroke}{Outline width of estimate points.}

\item{ci_linewidth}{Line width of confidence intervals.}

\item{base_size}{Base font size. The default is designed for a clear report
or slide figure; increase it for a single large plot.}

\item{show_ref}{Logical or \code{NULL}. The default \code{NULL} inherits the
reference-row choice used to create \code{tbl}. If \code{TRUE}, reference
rows are shown. If \code{FALSE}, binary exposures are shown as compact rows
for the estimated non-reference category; affirmative levels such as
\code{Yes} or \code{1} are displayed using the variable name.}

\item{sig_color}{Optional fill color for significant points.}

\item{sig_errorbar_color}{Optional color for significant error bars.}

\item{alpha}{Significance level for linear models when \code{p.value} is available.}

\item{show_adjustment_note}{Logical; if \code{TRUE}, add a default caption
describing \code{adjust_for} variables when available.}
}
\value{
A \code{ggplot2} object.
}
\description{
Draw a forest-style plot from a fitted \code{gtregression} object produced by
functions such as \code{uni_reg()}, \code{multi_reg()}, \code{cox_reg()}, or
\code{surv_reg()}. Stratified regression outputs are displayed as one
faceted quick-view plot; use \code{forest_df()} and \code{forest_reg()} for
publication-style stratified forest tables.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/km_plot.R
\name{km_plot}
\alias{km_plot}
\title{Kaplan-Meier survival plot}
\usage{
km_plot(
  data,
  time,
  event,
  by = NULL,
  conf.int = TRUE,
  risk_table = TRUE,
  p_value = TRUE,
  p_value_position = NULL,
  censor = TRUE,
  break_time_by = NULL,
  xlim = NULL,
  ylim = NULL,
  xlab = "Time",
  ylab = "Survival probability",
  title = NULL,
  subtitle = NULL,
  caption = NULL,
  title_size = NULL,
  title_face = "bold",
  legend_title = NULL,
  legend_position = NULL,
  palette = NULL,
  y_percent = TRUE,
  theme = "classic",
  grid = FALSE,
  base_size = 13
)
}
\arguments{
\item{data}{A \code{data.frame} containing survival time and event status.}

\item{time}{Survival follow-up time. Quoted and bare names are accepted.}

\item{event}{Event indicator. Quoted and bare names are accepted. Numeric
\code{0/1}, numeric \code{1/2}, logical, character, and factor variables
are accepted. For two-level character or factor variables, the second level
is treated as the event.}

\item{by}{Optional grouping variable for separate Kaplan-Meier curves. Quoted
and bare names are accepted.}

\item{conf.int}{Logical; if \code{TRUE}, show confidence limits.}

\item{risk_table}{Logical; if \code{TRUE}, add a number-at-risk table below
the curve.}

\item{p_value}{Logical; if \code{TRUE}, show the log-rank p-value when
\code{by} is supplied.}

\item{p_value_position}{Optional numeric vector of length 2 giving the
\code{x} and \code{y} coordinates for the log-rank p-value inside the
plotting panel. If \code{NULL}, a lower-left position is chosen
automatically.}

\item{censor}{Logical; if \code{TRUE}, show censoring marks.}

\item{break_time_by}{Optional numeric interval for x-axis and risk-table time
breaks. If \code{NULL}, breaks are chosen automatically.}

\item{xlim}{Optional numeric vector of length 2 specifying x-axis limits.}

\item{ylim}{Optional numeric vector of length 2 specifying y-axis limits.
Values may be supplied on the survival-probability scale
(for example \code{c(0.5, 1)}) or, when \code{y_percent = TRUE}, on the
percentage scale (for example \code{c(50, 100)}).}

\item{xlab, ylab}{Axis labels.}

\item{title}{Optional plot title.}

\item{subtitle}{Optional plot subtitle.}

\item{caption}{Optional plot caption.}

\item{title_size}{Optional numeric title font size. If \code{NULL}, ggplot2's
theme default is used.}

\item{title_face}{Font face for the title. One of \code{"plain"},
\code{"bold"}, \code{"italic"}, or \code{"bold.italic"}. Quoted and bare
values are accepted.}

\item{legend_title}{Optional legend title. If \code{NULL}, the labelled
\code{by} variable name is used.}

\item{legend_position}{Legend position. One of \code{"bottom"}, \code{"top"},
\code{"right"}, \code{"left"}, or \code{"none"}. If \code{NULL}, grouped
plots use \code{"bottom"} and ungrouped plots hide the legend. Quoted and
bare values are accepted.}

\item{palette}{Optional character vector of colors for grouped curves.}

\item{y_percent}{Logical; if \code{TRUE}, display survival probability as
percentages. If \code{FALSE}, display the raw 0 to 1 probability scale.}

\item{theme}{Plot theme. One of \code{"classic"}, \code{"minimal"},
\code{"bw"}, \code{"light"}, or \code{"none"}. Quoted and bare values are
accepted.}

\item{grid}{Logical; if \code{TRUE}, show major grid lines. The default is
\code{FALSE} for a cleaner publication-style Kaplan-Meier plot.}

\item{base_size}{Base font size.}
}
\value{
A \code{ggplot2} object when \code{risk_table = FALSE}; otherwise a
  \code{patchwork} object combining the survival curve and risk table. The
  returned object has attributes \code{fit}, \code{plot_data},
  \code{risk_table}, and \code{logrank_p}.
}
\description{
Plot observed survival over time, with optional confidence intervals,
censoring marks, a log-rank p-value, and a number-at-risk table.
}
\examples{
lung_data <- data_lungcancer
lung_data$trt <- factor(lung_data$trt, levels = c(1, 2),
                        labels = c("Standard", "Test"))

km_plot(
  data = lung_data,
  time = time,
  event = status
)

km_plot(
  data = lung_data,
  time = time,
  event = status,
  by = trt,
  break_time_by = 200,
  ylim = c(50, 100),
  title = "A. Treatment group",
  title_size = 11,
  legend_position = "none"
)

}
